package com.ibm.gbs.va.grit.activity;

import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.IdRes;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.fragment.login.EnterAppFragment;
import com.ibm.gbs.va.grit.fragment.login.ForgotPasswordFragment;
import com.ibm.gbs.va.grit.fragment.login.LoginFragment;
import com.ibm.gbs.va.grit.fragment.login.PrivacyFragment;
import com.ibm.gbs.va.grit.fragment.login.SignUpFragment;
import com.ibm.gbs.va.grit.repository.NotificationRepository;
import com.ibm.gbs.va.grit.repository.UserRepository;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.rest.NetworkErrorListener;
import com.ibm.gbs.va.grit.util.RemoteLog;

import java.util.LinkedList;

import javax.inject.Inject;

import dagger.android.support.DaggerAppCompatActivity;

public class AuthenticationActivity extends DaggerAppCompatActivity implements NetworkErrorListener {

    @Inject
    UserRepository userRepo;
    @Inject
    NotificationRepository notificationRepo;

    private FragmentManager fragManager;

    private final LinkedList<NetworkErrorDelegate> networkErrorDelegates = new LinkedList<>();

    private final String TAG = "Authentication";

    public final static String AUTHENTICATION_START_FRAG_ID = "AUTHENTICATION_START_FRAG_ID";

    private final static String ENTRY_MSG = "ENTRY_MSG";

    @Override
    public LinkedList<NetworkErrorDelegate> getNetworkErrorListeners() {
        return networkErrorDelegates;
    }

    @Override
    public void addDelegateToList(NetworkErrorDelegate delegate) {
        if (!networkErrorDelegates.contains(delegate)) {
            networkErrorDelegates.add(delegate);
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_authentication);
        fragManager = getSupportFragmentManager();

        int startingFragmentId = getIntent().getIntExtra(AUTHENTICATION_START_FRAG_ID, -1);
        if (startingFragmentId != -1) {
            //Activity started with an id
            switch (startingFragmentId) {
                case R.id.login:
                    goToLogin(getIntent().getStringExtra(ENTRY_MSG));
                    break;
                case R.id.privacyScrollView:
                    goToPrivacy();
                    break;
            }
        } else {
            Fragment fragment = fragManager.findFragmentById(R.id.contentFrame);
            if (fragment == null) {
                fragment = new EnterAppFragment();
                goToFragment(fragment, R.id.enterAppFrag, true);
            }
        }
    }

    public void goToWalkthrough() {
        startActivity(new Intent(this, WalkthroughActivity.class));
    }

    public void goToSignUp() {
        boolean userAcceptedPrivacyPolicy = userRepo.getUserAcceptedPolicy();
        // if has not accepted privacy policy - send to privacy
        // else send to sign-up

        if (userAcceptedPrivacyPolicy) {
            goToFragment(new SignUpFragment(), R.id.login, false);
        } else {
            goToPrivacy();
        }
    }

    public void goToLogin(String snackbarMsg) {
        LoginFragment frag;
        if (snackbarMsg != null && !snackbarMsg.isEmpty()) {
            frag = LoginFragment.newInstance(snackbarMsg);
        } else {
            frag = new LoginFragment();
        }
        goToFragment(frag, R.id.login, false);
    }

    private void goToPrivacy() {
        goToFragment(new PrivacyFragment(), R.id.privacyScrollView, false);
    }

    public void goToForgotPassword() {
        goToFragment(new ForgotPasswordFragment(), R.id.forgotPassword, false);
    }

    public void enterApp() {
        startActivity(MainActivity.injectSkipBadgeFetchingArg(new Intent(this, MainActivity.class)));
        finishAffinity();
    }

    private void goToFragment(Fragment frag, @IdRes int fragId, boolean newFrag) {
        RemoteLog.d(TAG, "Going to fragment: " + fragId);
        if (newFrag) {
            fragManager.beginTransaction()
                    .setCustomAnimations(R.anim.fade_in, R.anim.fade_out, R.anim.fade_in, R.anim.fade_out)
                    .add(R.id.contentFrame, frag, String.valueOf(fragId))
                    .addToBackStack(String.valueOf(fragId))
                    .commit();
        } else {
            fragManager.beginTransaction()
                    .setCustomAnimations(R.anim.fade_in, R.anim.fade_out, R.anim.fade_in, R.anim.fade_out)
                    .replace(R.id.contentFrame, frag, String.valueOf(fragId))
                    .addToBackStack(String.valueOf(fragId))
                    .commit();
        }
    }

    @Override
    public void onBackPressed() {

        if (fragManager.getBackStackEntryCount() < 2) {
            this.moveTaskToBack(true); // move app to background instead of stopping it
        } else {
            getSupportFragmentManager().popBackStack();
        }
    }
}
