package com.ibm.gbs.va.grit.activity;

import android.arch.lifecycle.LiveData;
import android.databinding.DataBindingUtil;
import android.databinding.ObservableBoolean;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.transition.TransitionManager;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentStatePagerAdapter;
import android.support.v4.view.ViewPager;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.ActivityViewPagerBinding;

public abstract class BaseViewPagerActivity extends AnalyticsActivity {
    String TAG = "BaseViewPagerActivity";

    ViewPager viewPager = null;
    final PagerViewModel viewModel = new PagerViewModel();
    private FragmentStatePagerAdapter pagerAdapter;
    private ActivityViewPagerBinding binding;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        TAG = "BaseViewPagerActivity";
        binding = DataBindingUtil.setContentView(this, R.layout.activity_view_pager);
        binding.setLifecycleOwner(this);
        binding.setData(viewModel);

        viewPager = binding.viewPager;
        binding.btnNext.setOnClickListener(v -> {
            if (viewModel.onLastPage.get()) onFinishClicked();
            else viewPager.setCurrentItem(viewPager.getCurrentItem() + 1, true);
        });

        binding.btnPrev.setOnClickListener(v -> viewPager.setCurrentItem(viewPager.getCurrentItem() - 1, true));
        binding.infoBtn.setOnClickListener(v -> onInfoClicked());
        binding.closeBtn.setOnClickListener(v -> onCloseClicked());

        viewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageScrolled(int i, float v, int i1) {
            }

            @Override
            public void onPageSelected(int i) {
                int totalPages = getTotalPages().getValue() != null ? getTotalPages().getValue() : 0;
                viewModel.recalculateData(totalPages, i);
            }

            @Override
            public void onPageScrollStateChanged(int i) {
            }
        });
    }

    @Override
    protected void onPostCreate(@Nullable Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);

        resetAdapter();

        // set onLastPage if less than one page exists from the start
        viewModel.onLastPage.set(pagerAdapter.getCount() <= 1);
    }

    void clearAdapter() {
        // animate changes
        TransitionManager.beginDelayedTransition(viewPager);
        viewPager.setAdapter(null);
    }

    void resetAdapter() {
        // animate changes
        getTotalPages().removeObservers(this);
        TransitionManager.beginDelayedTransition(viewPager);
        pagerAdapter = new FragmentStatePagerAdapter(getSupportFragmentManager()) {
            @Override
            public Fragment getItem(int position) {
                return getFragment(position);
            }

            @Override
            public int getCount() {
                return getTotalPages().getValue() != null ? getTotalPages().getValue() : 0;
            }
        };
        viewPager.setAdapter(pagerAdapter);

        getTotalPages().observe(this, totalPages -> {
            pagerAdapter.notifyDataSetChanged();
            int total = totalPages != null ? totalPages : 0;
            binding.pageIndicator.setCount(totalPages);
            viewModel.recalculateData(total, viewPager.getCurrentItem());
        });
    }

    @Override
    public void onBackPressed() {
        if (!viewModel.onFirstPage.get()) {
            viewPager.setCurrentItem(viewPager.getCurrentItem() - 1, true);
        } else if (!viewModel.disableExiting.get()) {
            onCloseClicked();
        }
    }

    abstract void onFinishClicked();

    abstract void onCloseClicked();

    abstract void onInfoClicked();

    abstract LiveData<Integer> getTotalPages();

    abstract Fragment getFragment(int index);

    public static class PagerViewModel {
        public final ObservableBoolean onFirstPage = new ObservableBoolean(true);
        public final ObservableBoolean onLastPage = new ObservableBoolean(false);
        public final ObservableBoolean showInfoButton = new ObservableBoolean(false);
        public final ObservableBoolean disableExiting = new ObservableBoolean(false);

        public final ObservableBoolean invertColors = new ObservableBoolean(false);

        void recalculateData(int totalPages, int currentPage) {
            onFirstPage.set(currentPage == 0);
            onLastPage.set(currentPage == totalPages - 1);
        }
    }
}


