package com.ibm.gbs.va.grit.activity;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.animation.AnimatorSet;
import android.animation.ObjectAnimator;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.content.Context;
import android.content.Intent;
import android.databinding.DataBindingUtil;
import android.graphics.Rect;
import android.graphics.drawable.LayerDrawable;
import android.os.Bundle;
import android.support.annotation.IdRes;
import android.support.annotation.NonNull;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.view.GravityCompat;
import android.support.v7.app.ActionBar;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.view.ContextThemeWrapper;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.adapter.DashboardMessagePagerAdapter;
import com.ibm.gbs.va.grit.databinding.ActivityDashboardBinding;
import com.ibm.gbs.va.grit.databinding.NavDrawerHeaderBinding;
import com.ibm.gbs.va.grit.drawable.MenuArrowDrawable;
import com.ibm.gbs.va.grit.fragment.chat.ChatFragment;
import com.ibm.gbs.va.grit.fragment.dashboard.DashboardFragment;
import com.ibm.gbs.va.grit.fragment.dashboard.ImmediateHelpFragment;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.fragment.digitalself.DigitalSelfFragment;
import com.ibm.gbs.va.grit.fragment.employment.EmploymentFragment;
import com.ibm.gbs.va.grit.fragment.login.LoginFragment;
import com.ibm.gbs.va.grit.fragment.settings.SettingsFragment;
import com.ibm.gbs.va.grit.fragment.squad.SquadFragment;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.repository.UserRepository;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.rest.NetworkErrorListener;
import com.ibm.gbs.va.grit.util.BadgeDelegate;
import com.ibm.gbs.va.grit.util.Consumer;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.viewmodel.DashboardViewModel;
import com.ibm.gbs.va.grit.viewmodel.EmploymentViewModel;
import com.stfalcon.chatkit.messages.MessageInput;

import java.util.LinkedList;
import java.util.List;

import javax.inject.Inject;

import dagger.android.support.DaggerAppCompatActivity;

public class DashboardActivity extends DaggerAppCompatActivity implements DashboardMessagePagerAdapter.OnMessageClickedDelegate, NetworkErrorListener, ProgressBarDelegate, BadgeDelegate {
    private static final String TAG = "DashboardActivity";

    public static final String SURVEY_SUBMIT_ERROR_OCCURRED_ARG = "survey_submit_error_occurred";

    /**
     * To be implemented by child fragments to notify this activity of any custom back action
     */
    public interface OnBackPressedListener {

        /**
         * @return if the back action was consumed
         */
        boolean customBackPressedAction();
    }

    private ActionBar actionBar;
    private FragmentManager fragManager;
    private ActivityDashboardBinding binding;
    private DashboardViewModel model;
    private EmploymentViewModel employmentViewModel;
    private Notification currentSurveyNotification = null;
    private MenuArrowDrawable drawerArrowDrawable;
    private float toolbarElevationDimen;
    private float zeroDpDimen;

    @Inject
    UserRepository userRepo;
    @Inject
    ViewModelProvider.Factory viewModelFactory;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_dashboard);
        fragManager = getSupportFragmentManager();
        binding = DataBindingUtil.setContentView(this, R.layout.activity_dashboard);
        model = ViewModelProviders.of(this, viewModelFactory).get(DashboardViewModel.class);
        employmentViewModel = ViewModelProviders.of(this, viewModelFactory).get(EmploymentViewModel.class);
        employmentViewModel.getCurrentTab().observe(this, tab -> updateActionBar());

        toolbarElevationDimen = getResources().getDimension(R.dimen.toolbar_elevation);
        zeroDpDimen = getResources().getDimension(R.dimen.zero_dp);

        // add header view to navDrawer
        View headerView = binding.navView.getHeaderView(0);
        NavDrawerHeaderBinding headerBinding = NavDrawerHeaderBinding.bind(headerView);
        headerBinding.setLifecycleOwner(this);
        model.getUser().observe(this, user -> {
            if (user != null) {
                headerBinding.name.setText(user.getName());
                headerBinding.emailAddress.setText(user.getEmail());
            }
        });

        // set up hamburger menu
        setSupportActionBar(binding.toolbar);
        actionBar = getSupportActionBar();
        assert actionBar != null;
        actionBar.setTitle(getString(R.string.dashboard));
        actionBar.setDisplayHomeAsUpEnabled(true);
        this.drawerArrowDrawable = new MenuArrowDrawable(new ContextThemeWrapper(this, R.style.AppTheme), actionBar);
        ActionBarDrawerToggle actionBarDrawerToggle = new ActionBarDrawerToggle(this, binding.drawerLayout, R.string.open_menu, R.string.go_back);
        binding.drawerLayout.addDrawerListener(actionBarDrawerToggle);
        actionBarDrawerToggle.syncState();
        binding.toolbar.setNavigationOnClickListener(view -> { // clicking on the hamburger menu
            if (drawerArrowDrawable.getPosition() == 1) {
                onBackPressed();
            } else {
                binding.drawerLayout.openDrawer(GravityCompat.START);
            }
        });

        // update menu checked statuses on back stack change
        fragManager.addOnBackStackChangedListener(this::updateActionBar);

        updateNavMenuItemUis();
        model.getMenuData().observe(this, menuList -> updateNavMenuItemUis());

        binding.navView.setNavigationItemSelectedListener(menuItem -> { // clicking a menu item
            // set item as selected to persist highlight
            menuItem.setChecked(true);

            // Add code here to update the UI based on the item selected
            // For example, swap UI fragments here
            swapFragments(menuItem.getItemId());

            // close drawer when item is tapped
            binding.drawerLayout.closeDrawers();
            return true;
        });

        model.getCurrentSurveyNotification().observe(this, notification ->
                currentSurveyNotification = notification);

        // set up initial fragment
        Fragment fragment = fragManager.findFragmentById(R.id.contentFrame);
        if (fragment == null) {
            fragment = new DashboardFragment();
            fragManager.beginTransaction()
                    .setCustomAnimations(R.anim.fade_in, R.anim.fade_out, R.anim.fade_in, R.anim.fade_out)
                    .add(R.id.contentFrame, fragment, String.valueOf(R.id.dashboard))
                    .addToBackStack(String.valueOf(R.id.dashboard))
                    .commit();
        }
    }

    // Dismiss keyboard when clicking out of an EditText
    @Override
    public boolean dispatchTouchEvent(MotionEvent event) {
        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            View v = getCurrentFocus();
            if (v instanceof MessageInput || v instanceof EditText) {
                Rect outRect = new Rect();
                v.getGlobalVisibleRect(outRect);
                if (!outRect.contains((int) event.getRawX(), (int) event.getRawY())) {
                    InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                    imm.hideSoftInputFromWindow(v.getWindowToken(), 0);
                }
            }
        }
        return super.dispatchTouchEvent(event);
    }

    @Override
    protected void onResume() {
        super.onResume();
        String surveyError = getIntent().getStringExtra(SURVEY_SUBMIT_ERROR_OCCURRED_ARG);
        if (surveyError != null) {
            MessageHandler.handleError(this, surveyError);
        }
    }

    @Override
    public void onBackPressed() {
        Fragment currentFrag = fragManager.findFragmentById(R.id.contentFrame);
        if (currentFrag instanceof OnBackPressedListener &&
                ((OnBackPressedListener) currentFrag).customBackPressedAction()) {
            return;
        }

        if (fragManager.getBackStackEntryCount() < 2) {
            this.moveTaskToBack(true); // move app to background instead of stopping it
        } else {
            getSupportFragmentManager().popBackStack();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.action_bar_items, menu); // add chat to action bar
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home: // handle clicking the hamburger menu
                binding.drawerLayout.openDrawer(GravityCompat.START);
                return true;
            case R.id.chat:// handle clicking the chat
                Fragment frag = fragManager.findFragmentByTag(String.valueOf(item.getItemId()));
                if (frag != null) { // backStack contains the fragment, pop to it
                    fragManager.popBackStack(String.valueOf(frag.getTag()), 0);
                    return true;
                } else { // pop to the first non-blocking fragment, push the chat fragment
                    for (int index = fragManager.getBackStackEntryCount() - 1; index >= 0; index--) {
                        FragmentManager.BackStackEntry entry = fragManager.getBackStackEntryAt(index);
                        DashboardViewModel.MenuData data = model.getMenuItem(Integer.valueOf(entry.getName()));
                        if (data != null && !data.blocksMenuAccess) {
                            fragManager.popBackStack(entry.getId(), 0);
                            swapFragments(item.getItemId());
                            return true;
                        }
                    }
                }
        }
        return super.onOptionsItemSelected(item);
    }

    public void swapFragments(@IdRes int menuId) {
        MenuItem menuItem = binding.navView.getMenu().findItem(menuId);
        DashboardViewModel.MenuData menuData;
        if (menuItem != null) {
            menuData = model.getMenuItem(menuItem.getItemId());
        } else {    //item to swap came from options menu
            menuData = model.getMenuItem(menuId);
        }
        if (fragManager.findFragmentById(R.id.contentFrame) != null)
            if (fragManager.findFragmentById(R.id.contentFrame).getTag().equals(String.valueOf(menuData.fragmentId))) {
                // The same as the current fragment, ignore this request
                return;
            }

        if (menuId == R.id.signOut) {
            logout(getString(R.string.user_signed_out));
        } else if (menuId == R.id.aboutThisApp) {
            // go to walkthrough activity
            Intent aboutThisAppIntent = new Intent(this, WalkthroughActivity.class);
            startActivity(aboutThisAppIntent);
        } else {
            Fragment frag = null;
            if (menuData.fragmentId == R.id.jobs) { // handle employment frags
                employmentViewModel.setCurrentTab(menuId);
                frag = fragManager.findFragmentByTag(String.valueOf(R.id.jobs));
                if (menuId == R.id.jobFilter) {
                    frag = EmploymentFragment.goToJobFilterInstance(frag);
                } else if (frag == null) {
                    frag = new EmploymentFragment();
                }

            } else if (menuItem == null) {
                frag = fragManager.findFragmentByTag(String.valueOf(menuId));

            } else if (menuItem.getItemId() != binding.navView.getCheckedItem().getItemId()) {
                frag = fragManager.findFragmentByTag(String.valueOf(menuItem.getItemId()));
            }

            if (frag == null) {
                // frag not found, create new one
                switch (menuId) {

                    case R.id.dashboard:
                        // should only occur if the dashboard frag isn't found on the backstack, which means it's being displayed. So we do nothing.
                        return;
                    case R.id.digitalSelf:
                        frag = new DigitalSelfFragment();
                        break;
                    case R.id.squad:
                        frag = new SquadFragment();
                        break;
                    case R.id.chat:
                        frag = new ChatFragment();
                        break;
                    case R.id.immediateHelp:
                        frag = new ImmediateHelpFragment();
                        break;
                    case R.id.settings:
                        frag = new SettingsFragment();
                        break;
                    case R.id.settings_editProfile:
                        frag = new SettingsFragment();
                        ((SettingsFragment) frag).straightToEditProfile();
                        break;
                    default:  // make generic fragment
                        RemoteLog.w(TAG, "fragment not found, serving empty fragment");
                        frag = new Fragment();
                        break;
                }
            }

            if (menuData.blocksMenuAccess) { // do slide animation
                fragManager.beginTransaction()
                        .setCustomAnimations(R.anim.slide_in_right, R.anim.slide_out_left, android.R.anim.slide_in_left, android.R.anim.slide_out_right)
                        .replace(R.id.contentFrame, frag, String.valueOf(menuData.fragmentId))
                        .addToBackStack(String.valueOf(menuData.fragmentId))
                        .commit();
            } else { // do fade animation
                fragManager.beginTransaction()
                        .setCustomAnimations(R.anim.fade_in, R.anim.fade_out, R.anim.fade_in, R.anim.fade_out)
                        .replace(R.id.contentFrame, frag, String.valueOf(menuData.fragmentId))
                        .addToBackStack(String.valueOf(menuData.fragmentId))
                        .commit();
            }

            updateActionBar();
        }
    }

    private void goToSurvey() {
        // go to survey activity
        Intent surveyIntent = new Intent(this, SurveyActivity.class);
        startActivity(surveyIntent);
        finish();
    }

    /**
     * updates menu item icon colors and badge statuses
     */
    private void updateNavMenuItemUis() {
        binding.navView.setItemIconTintList(null);
        for (int i = 0; i < binding.navView.getMenu().size(); i++) {
            MenuItem item = binding.navView.getMenu().getItem(i);
            DashboardViewModel.MenuData menuData = model.getMenuItem(item.getItemId());
            if (item.getGroupId() != R.id.mainItems) {
                item.setTitle(item.getTitle().toString().toUpperCase());
            }
            LayerDrawable layerList = menuData.iconDrawable;
            layerList.findDrawableByLayerId(R.id.badgeIcon).setAlpha(menuData.showBadge ? 255 : 0);
            if (item.getIcon() == null || !item.getIcon().equals(layerList)) {
                item.setIcon(layerList);
            }
        }
    }

    private void updateActionBar() {
        int fragmentId = Integer.valueOf(fragManager.findFragmentById(R.id.contentFrame).getTag());
        DashboardViewModel.MenuData newMenuData;
        if (fragmentId == R.id.jobs)
            newMenuData = model.getMenuItem(employmentViewModel.getCurrentTab().getValue().menuId);
        else
            newMenuData = model.getMenuItem(fragmentId);

        if (newMenuData != null) {
            actionBar.setTitle(newMenuData.blocksMenuAccess ? newMenuData.title : model.getMenuItem(newMenuData.fragmentId).title);
            binding.navView.setCheckedItem(newMenuData.menuId);
            drawerArrowDrawable.animateDrawable(newMenuData.blocksMenuAccess);

            // remove toolbar elevation when in jobs
            ObjectAnimator anim = new ObjectAnimator().setDuration(200);
            anim.setPropertyName("elevation");
            anim.setTarget(binding.appBarLayout);
            anim.setFloatValues(newMenuData.fragmentId == R.id.jobs ? zeroDpDimen : toolbarElevationDimen);
            anim.start();
        } else {
            drawerArrowDrawable.animateDrawable(true);
        }
    }

    public void logout(String msg) {
        Intent intent = new Intent(this, AuthenticationActivity.class);
        Bundle b = new Bundle();
        if (msg != null && !msg.isEmpty()) {
            b.putString(LoginFragment.ENTRY_MSG, msg);
        }
        b.putInt(AuthenticationActivity.AUTHENTICATION_START_FRAG_ID, R.id.login);
        intent.putExtras(b);
        startActivity(intent);
        finish();
        userRepo.signUserOut();
    }

    //region Network Error Listener
    private final LinkedList<NetworkErrorDelegate> networkErrorDelegates = new LinkedList<>();

    @Override
    public LinkedList<NetworkErrorDelegate> getNetworkErrorListeners() {
        return networkErrorDelegates;
    }

    @Override
    public void addDelegateToList(NetworkErrorDelegate delegate) {
        if (!networkErrorDelegates.contains(delegate)) {
            networkErrorDelegates.add(delegate);
        }
    }
    //endregion

    //region On Message Clicked Delegate
    @Override
    public void onMessageClicked(Notification notification) {
        switch (notification.getType().getAction()) {
            case BeginBaseline:
                startSurvey(notification);
                break;
            case BeginMoodFace:
                startSurvey(notification);
                break;
            case BeginMoodText:
                startSurvey(notification);
                break;
            case BeginLifeChangeRe:
                startSurvey(notification);
                break;
            case ContinueSurvey:
            case BeginBaselineInitial:
                goToSurvey();
                break;
            case UpdateProfile:
                swapFragments(R.id.settings_editProfile);
                break;
            case UpdateSquad:
                swapFragments(R.id.squad);
                break;
            case UpdateJobInfo:
                swapFragments(R.id.jobFilter);
                break;
            case DigitalAssist:
                // seed chat with text?
                model.dismissNotification(Notification.Type.DIGITAL_ASSISTANT);
                swapFragments(R.id.chat);
                break;
            case ImmediateHelp:
                model.dismissNotification(Notification.Type.IMMEDIATE_HELP);
                swapFragments(R.id.immediateHelp);
                break;
            case Feedback:
                /*
                TextSection bodyText;
                if (notification.getMetadata() == null)
                    bodyText = TextSection.ofText(R.string.dialog_congrats_body);
                else bodyText = TextSection.ofDynamicText(notification.getMetadata());
                GritDialogFragment.showSingleBtnInfoDialog(getSupportFragmentManager(),
                        R.string.feedback_title, bodyText);*/
                model.dismissNotification(Notification.Type.FEEDBACK);
                break;
            case DEFAULT_NOTIFICATION_TYPE:
                //Default action for notifications
                model.dismissNotification(notification);
                break;
        }
    }
    //endregion

    private void startSurvey(@NonNull Notification notification) {
        if (notification.equals(currentSurveyNotification)) {
            goToSurvey();

        } else if (currentSurveyNotification != null) {
            //redirect to ongoing survey;
            GritDialogFragment.showTwoBtnInfoDialog(getSupportFragmentManager(),
                    R.string.survey_in_progress_title, R.string.survey_in_progress_finish_button, R.string.survey_in_progress_abort_button,
                    okBtn -> goToSurvey(),
                    abortBtn -> {
                        showProgressBar();
                        model.abortCurrentSurvey(notification, booleanBadgePair -> {
                            hideProgressBar();
                            if (booleanBadgePair.first == null) return; // error occurred
                            if (booleanBadgePair.first) { // abort succeeded
                                if (booleanBadgePair.second == null)
                                    goToSurvey();
                                else {
                                    Badge.getDefaultBadgeBehaviour(getSupportFragmentManager(), getResources(), onDismiss -> goToSurvey())
                                            .accept(booleanBadgePair.second);
                                }
                            }
                        });
                    }, TextSection.ofText(R.string.survey_in_progress_body));

        } else {
            model.setSurveyNotification(notification, dismissed -> {
                if (dismissed) goToSurvey();
            });
        }
    }

    //region Progress Bar Delegate
    @Override
    public void showProgressBar() {
        if (binding.progressBar.getVisibility() != View.VISIBLE) {
            AnimatorSet set = new AnimatorSet();
            binding.progressBar.setVisibility(View.VISIBLE);
            set.playTogether(ObjectAnimator.ofFloat(binding.progressBar, "alpha", 0, 1),
                    ObjectAnimator.ofFloat(binding.progressBar, "scaleY", 0, 1));
            set.setDuration(250).start();
        }
    }

    @Override
    public void hideProgressBar() {
        if (binding.progressBar.getVisibility() == View.VISIBLE) {
            AnimatorSet set = new AnimatorSet();
            set.playTogether(ObjectAnimator.ofFloat(binding.progressBar, "alpha", 1, 0),
                    ObjectAnimator.ofFloat(binding.progressBar, "scaleY", 1, 0));
            set.addListener(new AnimatorListenerAdapter() {
                @Override
                public void onAnimationEnd(Animator animation) {
                    super.onAnimationEnd(animation);
                    binding.progressBar.setVisibility(View.GONE);
                }
            });
            set.setDuration(250).start();
        }
    }
    //endregion

    @Override
    public boolean handleBadges(@NonNull List<Badge> badgesToHandle) {
        Consumer<Badge> badgeHandler = Badge.getDefaultBadgeBehaviour(getSupportFragmentManager(), getResources(), null);
        for (Badge curBadge : badgesToHandle) {
            badgeHandler.accept(curBadge);
        }
        return true;
    }

}
