package com.ibm.gbs.va.grit.activity;

import android.content.Intent;
import android.os.Bundle;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.repository.BadgeRepository;
import com.ibm.gbs.va.grit.util.UserPrefs;

import javax.inject.Inject;

import dagger.android.support.DaggerAppCompatActivity;

public class MainActivity extends DaggerAppCompatActivity {
    private static final String SKIP_BADGE_ARG = "skip_badge_arg";

    public static Intent injectSkipBadgeFetchingArg(Intent intent) {
        final Bundle args = new Bundle();
        args.putBoolean(SKIP_BADGE_ARG, true);
        intent.putExtras(args);
        return intent;
    }

    @Inject
    UserPrefs userPrefs;
    @Inject
    BadgeRepository badgeRepository;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        boolean skipBadgeFetching = getIntent().getBooleanExtra(SKIP_BADGE_ARG, false);

        //get a list of valid badges before any calls are made, in case a badge is returned
        if (!skipBadgeFetching)
            badgeRepository.updateBadgeListInitial();

        if (userPrefs.getGlobalFirstTime()) {
            startActivity(new Intent(this, WalkthroughActivity.class));
        } else {
            if (userPrefs.getUserLoggedIn()) {
                if (!userPrefs.getPrefInitialBaselineComplete()) {
                    startActivity(new Intent(this, SurveyActivity.class));
                } else {
                    startActivity(new Intent(this, DashboardActivity.class));
                }
            } else if (userPrefs.getUserHasLoggedIn()) {
                Bundle b = new Bundle();
                b.putInt(AuthenticationActivity.AUTHENTICATION_START_FRAG_ID, R.id.login);
                Intent intent = new Intent(this, AuthenticationActivity.class);
                intent.putExtras(b);
                startActivity(intent);
            } else {
                //User's first time
                startActivity(new Intent(MainActivity.this, AuthenticationActivity.class));
            }
        }
        overridePendingTransition(R.anim.slow_fade_in, R.anim.slow_fade_out);
        finish();
    }
}
