package com.ibm.gbs.va.grit.activity;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.MutableLiveData;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.fragment.NonInteractiveFragment;
import com.ibm.gbs.va.grit.fragment.WalkthroughStepFragment;
import com.ibm.gbs.va.grit.model.type.MultiLangText;
import com.ibm.gbs.va.grit.model.type.WalkthroughStep;
import com.ibm.gbs.va.grit.util.Analytics;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.util.UserPrefs;

import javax.inject.Inject;

public class WalkthroughActivity extends BaseViewPagerActivity {

    @Inject
    UserPrefs userPrefs;
    @Inject
    Analytics analytics;

    private final MutableLiveData<Integer> totalPages = new MutableLiveData<>();

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        TAG = "WalkthroughActivity";
        super.onCreate(savedInstanceState);
        totalPages.setValue(5);

        // We don't need to invert the colors back anymore, as the colors will stay consistently inverted
        viewModel.invertColors.set(viewModel.onFirstPage.get());
    }

    @Override
    protected void onResume() {
        super.onResume();
        analytics.walkthroughBegan(userPrefs.getGlobalFirstTime());
    }

    @Override
    public void onBackPressed() {
        if (!viewModel.onFirstPage.get()) {
            viewPager.setCurrentItem(viewPager.getCurrentItem() - 1, true);
        }
    }

    @Override
    void onFinishClicked() {
        analytics.walkthroughCompleted();
        continueToEnterApp();
    }

    @Override
    void onCloseClicked() {
        analytics.walkthroughCompleted(true, "page " + viewPager.getCurrentItem());
        continueToEnterApp();
    }

    @Override
    void onInfoClicked() {
        throw new UnsupportedOperationException();
    }

    private void continueToEnterApp() {
        if (userPrefs.getGlobalFirstTime()) {
            startActivity(new Intent(this, AuthenticationActivity.class));
        }
        userPrefs.setGlobalFirstTime(false);
        finish();
    }


    @Override
    LiveData<Integer> getTotalPages() {
        return totalPages;
    }

    @Override
    Fragment getFragment(int index) {
        MultiLangText walkthroughTitle = new MultiLangText();
        MultiLangText walkthroughBody = new MultiLangText();
        switch (index) {
            case 0:
                return NonInteractiveFragment.getInstance(R.layout.walkthrough_one);
            case 1:
                walkthroughTitle.setAll(getString(R.string.walk_digital_self_title));
                walkthroughBody.setAll(getString(R.string.walk_digital_self_body));
                WalkthroughStep stepTwo = new WalkthroughStep(walkthroughTitle, walkthroughBody, R.drawable.walk_digital_self);
                return WalkthroughStepFragment.getInstance(stepTwo);
            case 2:
                walkthroughTitle.setAll(getString(R.string.walk_employment_title));
                walkthroughBody.setAll(getString(R.string.walk_employment_body));
                WalkthroughStep stepThree = new WalkthroughStep(walkthroughTitle, walkthroughBody, R.drawable.walk_employment);
                return WalkthroughStepFragment.getInstance(stepThree);
            case 3:
                walkthroughTitle.setAll(getString(R.string.walk_squad_title));
                walkthroughBody.setAll(getString(R.string.walk_squad_body));
                WalkthroughStep stepFour = new WalkthroughStep(walkthroughTitle, walkthroughBody, R.drawable.walk_squad);
                return WalkthroughStepFragment.getInstance(stepFour);
            case 4:
                walkthroughTitle.setAll(getString(R.string.walk_builtin_help_title));
                walkthroughBody.setAll(getString(R.string.walk_builtin_help_body));
                WalkthroughStep stepFive = new WalkthroughStep(walkthroughTitle, walkthroughBody, R.drawable.walk_digital_assistant);
                return WalkthroughStepFragment.getInstance(stepFive);
            default:
                RemoteLog.w(TAG, "could not find a layout to display");
                return new Fragment();
        }
    }
}
