package com.ibm.gbs.va.grit.adapter;

import android.arch.lifecycle.LifecycleOwner;
import android.support.annotation.NonNull;
import android.support.v7.recyclerview.extensions.ListAdapter;
import android.support.v7.util.DiffUtil;
import android.view.LayoutInflater;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.databinding.CardDashboardMessageBinding;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.viewmodel.DashboardViewModel;
import com.ibm.gbs.va.grit.viewmodel.DashboardViewModel.MenuData;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentSkipListSet;

public class DashboardMessagePagerAdapter extends ListAdapter<Notification, NotificationViewHolder> {

    public interface OnMessageClickedDelegate {
        void onMessageClicked(Notification notificationClicked);
    }

    //region Diff Util
    private static final DiffUtil.ItemCallback<Notification> diffCallback = new DiffUtil.ItemCallback<Notification>() {
        @Override
        public boolean areItemsTheSame(@NonNull Notification oldNotification, @NonNull Notification newNotification) {
            return oldNotification.getInternalId().equals(newNotification.getInternalId());
        }

        @Override
        public boolean areContentsTheSame(@NonNull Notification oldNotification, @NonNull Notification newNotification) {
            return oldNotification.equals(newNotification);
        }
    };
    //endregion

    private final OnMessageClickedDelegate delegate;
    private final DashboardViewModel model;

    public DashboardMessagePagerAdapter(DashboardViewModel model, LifecycleOwner owner, OnMessageClickedDelegate delegate) {
        super(diffCallback);
        this.delegate = delegate;
        this.model = model;
        model.getMessages().observe(owner, messages -> {
            submitList(messages != null ? new ArrayList<>(messages) : null);

            processNavMenuItemBadges(messages);
        });
    }

    private void processNavMenuItemBadges(ConcurrentSkipListSet<Notification> set) {
        List<MenuData> menuItems = model.getMenuData().getValue();
        if (menuItems != null) {
            // for all menu items
            for (MenuData menuData : menuItems) {
                boolean notificationExists = false;
                if (set != null)
                    // for each current notification, check if it has a matching menu item
                    for (Notification notification : set) {
                        int menuId = notification.getType().getMenuId();
                        if (menuId != -1 && menuId == menuData.fragmentId) {
                            notificationExists = true;
                            break;
                        }
                    }
                // set the menu item to have a marker if there is a current notification for it
                model.setMenuHasBadge(menuData.fragmentId, notificationExists);
            }
        }
    }

    @NonNull
    @Override
    public NotificationViewHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int type) {
        LayoutInflater inflater = LayoutInflater.from(viewGroup.getContext());
        CardDashboardMessageBinding binding = CardDashboardMessageBinding.inflate(inflater, viewGroup, false);

        return new NotificationViewHolder(binding, delegate);
    }

    @Override
    public void onBindViewHolder(@NonNull NotificationViewHolder notificationViewHolder, int position) {
        notificationViewHolder.bindNotification(getItem(position));
    }
}
