package com.ibm.gbs.va.grit.adapter.viewholder;

import android.content.Context;
import android.support.annotation.NonNull;
import android.support.v7.widget.DividerItemDecoration;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckedTextView;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.ChatButtonIncomingMessageBinding;
import com.ibm.gbs.va.grit.model.chat.ChatMessage;
import com.ibm.gbs.va.grit.model.chat.SelectionOption;
import com.stfalcon.chatkit.messages.MessageHolders;

public class ButtonIncomingMessageViewHolder extends MessageHolders.BaseIncomingMessageViewHolder<ChatMessage> {

    private OnChatItemSelectedListener listener;
    private final ChatButtonIncomingMessageBinding binding;
    private ButtonRecyclerViewAdapter adapter;

    public ButtonIncomingMessageViewHolder(View itemView, Object payload) {
        super(itemView, payload);
        if (payload instanceof OnChatItemSelectedListener)
            listener = (OnChatItemSelectedListener) payload;
        binding = ChatButtonIncomingMessageBinding.bind(itemView);
        DividerItemDecoration itemDecor = new DividerItemDecoration(itemView.getContext(), DividerItemDecoration.VERTICAL);
        binding.recyclerView.addItemDecoration(itemDecor);
        binding.recyclerView.setLayoutManager(new LinearLayoutManager(itemView.getContext()));
    }

    @Override
    public void onBind(ChatMessage message) {
        super.onBind(message);

        binding.messageText.setText(message.getChatResponse().get(0).getText());

        // build adapter
        boolean optionsEnabled = message.getFollowup().getSelection() == null;
        adapter = new ButtonRecyclerViewAdapter(
                message,
                optionsEnabled);
        binding.recyclerView.setAdapter(adapter);
    }

    class ButtonRecyclerViewAdapter extends RecyclerView.Adapter<ButtonViewHolder> {

        private boolean optionsEnabled;
        private final ChatMessage message;

        ButtonRecyclerViewAdapter(ChatMessage message, boolean optionsEnabled) {
            this.message = message;
            this.optionsEnabled = optionsEnabled;
        }

        @NonNull
        @Override
        public ButtonViewHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int type) {
            LayoutInflater li = LayoutInflater.from(itemView.getContext());
            CheckedTextView view = (CheckedTextView) li.inflate(R.layout.list_item_dropdown_spinner, viewGroup, false);
            return new ButtonViewHolder(view);
        }

        @Override
        public void onBindViewHolder(@NonNull ButtonViewHolder buttonViewHolder, int position) {
            buttonViewHolder.bind(message.getFollowup().getSelections().get(position), optionsEnabled);
        }

        @Override
        public int getItemCount() {
            return message.getFollowup().getSelections().size();
        }
    }

    class ButtonViewHolder extends RecyclerView.ViewHolder {

        ButtonViewHolder(@NonNull CheckedTextView itemView) {
            super(itemView);
            itemView.setTextAlignment(View.TEXT_ALIGNMENT_CENTER);
            Context ctx = itemView.getContext();

            itemView.setTextColor(ctx.getColorStateList(R.color.list_item_text_color_list));
            itemView.setBackgroundColor(ctx.getColor(android.R.color.white));
            itemView.setBackgroundTintList(ctx.getColorStateList(R.color.list_item_color_list));
        }

        void bind(SelectionOption option, boolean enabled) {
            CheckedTextView view = (CheckedTextView) itemView;
            view.setText(option.getLabel());
            view.setEnabled(enabled);
            SelectionOption selectionOption = adapter.message.getFollowup().getSelection();
            view.setChecked(TextUtils.equals(selectionOption == null ? null : selectionOption.getValue(), option.getValue()));
            view.setOnClickListener(v -> {
                adapter.optionsEnabled = false;
                adapter.message.getFollowup().setSelection(option);
                adapter.notifyItemRangeChanged(0, adapter.getItemCount());
                listener.onChatItemSelected(option);
            });
        }
    }
}
