package com.ibm.gbs.va.grit.adapter.viewholder;

import android.text.TextUtils;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.ChatDropdownIncomingMessageBinding;
import com.ibm.gbs.va.grit.model.chat.ChatMessage;
import com.ibm.gbs.va.grit.model.chat.SelectionOption;
import com.stfalcon.chatkit.messages.MessageHolders;

import java.util.ArrayList;
import java.util.List;

public class DropdownIncomingMessageViewHolder extends MessageHolders.BaseIncomingMessageViewHolder<ChatMessage> {

    private OnChatItemSelectedListener listener;
    private final ChatDropdownIncomingMessageBinding binding;

    public DropdownIncomingMessageViewHolder(View itemView, Object payload) {
        super(itemView, payload);
        if (payload instanceof OnChatItemSelectedListener)
            listener = (OnChatItemSelectedListener) payload;
        binding = ChatDropdownIncomingMessageBinding.bind(itemView);
    }

    @Override
    public void onBind(ChatMessage message) {
        super.onBind(message);
        // clear any prev listener
        binding.dropdown.setOnItemSelectedListener(null);

        binding.messageText.setText(message.getChatResponse().get(0).getText());

        // build spinner adapter
        List<String> options = new ArrayList<>();
        options.add("");
        for (SelectionOption option : message.getFollowup().getSelections()) {
            options.add(option.getLabel());
        }
        ArrayAdapter<String> adapter = new ArrayAdapter<>(itemView.getContext(),
                R.layout.list_item_dropdown_spinner,
                options.toArray(new String[0]));
        binding.dropdown.setAdapter(adapter);

        SelectionOption optionAlreadySelected = message.getFollowup().getSelection();
        if (optionAlreadySelected != null) { // selection has already been made
            int itemIndex = message.getFollowup().getSelections().indexOf(optionAlreadySelected);
            binding.dropdown.setSelection(itemIndex, false);
            binding.dropdown.setEnabled(false);
        } else {
            binding.dropdown.setEnabled(true);
            binding.dropdown.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                @Override
                public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                    String selection = (String) parent.getItemAtPosition(position);
                    if (!TextUtils.isEmpty(selection)) {
                        binding.dropdown.setEnabled(false);
                        message.getFollowup().setSelection(message.getFollowup().getSelections().get(position - 1));
                        if (listener != null)
                            listener.onChatItemSelected(message.getFollowup().getSelections().get(position - 1));
                    }
                }

                @Override
                public void onNothingSelected(AdapterView<?> parent) {
                }
            });
        }
    }
}
