package com.ibm.gbs.va.grit.adapter.viewholder;

import android.content.Context;
import android.support.v4.text.HtmlCompat;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Toast;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.ChatIncomingMessageBinding;
import com.ibm.gbs.va.grit.databinding.ListItemChatMessageBinding;
import com.ibm.gbs.va.grit.model.chat.ChatMessage;
import com.ibm.gbs.va.grit.model.chat.Paragraph;
import com.stfalcon.chatkit.messages.MessageHolders;

public class GritIncomingMessageViewHolder extends MessageHolders.BaseIncomingMessageViewHolder<ChatMessage> {

    public interface OnHyperLinkClickedListener {
        void onHyperLinkClick(String url);
    }

    private OnHyperLinkClickedListener listener;
    private final ChatIncomingMessageBinding binding;

    public GritIncomingMessageViewHolder(View itemView, Object payload) {
        super(itemView, payload);
        if (payload instanceof OnHyperLinkClickedListener)
            listener = (OnHyperLinkClickedListener) payload;
        binding = ChatIncomingMessageBinding.bind(itemView);
    }

    @Override
    public void onBind(ChatMessage message) {
        super.onBind(message);
        LayoutInflater li = LayoutInflater.from(itemView.getContext());
        binding.list.removeAllViewsInLayout();
        Context ctx = itemView.getContext();
        for (Paragraph line : message.getChatResponse()) {
            if (line.getText().matches("\\s*")) continue; // skip empty lines

            ListItemChatMessageBinding txtBinding = ListItemChatMessageBinding.inflate(li, binding.list, true);
            if (line.getHyperlink() != null) {
                txtBinding.messageText.setText(HtmlCompat.fromHtml(
                        ctx.getString(R.string.href, line.getHyperlink(), line.getText()),
                        HtmlCompat.FROM_HTML_MODE_LEGACY));
                txtBinding.messageText.setOnClickListener(v -> {
                    if (listener != null) listener.onHyperLinkClick(line.getHyperlink());
                });
                txtBinding.messageText.setOnLongClickListener(v -> {
                    Toast.makeText(ctx, line.getHyperlink(), Toast.LENGTH_SHORT).show();
                    return true;
                });

            } else {
                txtBinding.messageText.setText(line.getText());
            }
        }
    }
}
