package com.ibm.gbs.va.grit.adapter.viewholder;

import android.view.View;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapsInitializer;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.model.MarkerOptions;
import com.ibm.gbs.va.grit.databinding.ChatMapMessageBinding;
import com.ibm.gbs.va.grit.model.chat.ChatMap;
import com.ibm.gbs.va.grit.model.chat.ChatMessage;
import com.stfalcon.chatkit.messages.MessageHolders;

public class MapIncomingMessageViewHolder extends MessageHolders.BaseIncomingMessageViewHolder<ChatMessage> implements OnMapReadyCallback {

    public interface OnAddressClickedListener {
        void onAddressClicked(ChatMap chatMap);
    }

    private OnAddressClickedListener listener;
    private final ChatMapMessageBinding binding;
    private ChatMessage message;
    private GoogleMap map;

    public MapIncomingMessageViewHolder(View itemView, Object payload) {
        super(itemView, payload);
        if (payload instanceof OnAddressClickedListener)
            listener = (OnAddressClickedListener) payload;
        binding = ChatMapMessageBinding.bind(itemView);

        // Initialise the MapView
        binding.mapView.onCreate(null);
        // Set the map ready callback to receive the GoogleMap object
        binding.mapView.getMapAsync(this);
    }

    @Override
    public void onBind(ChatMessage message) {
        super.onBind(message);
        ChatMap chatMap = message.getMap();

        binding.title.setText(chatMap.getPlaceName());
        binding.addressLine1.setText(chatMap.getAddressLine1());
        binding.addressLine2.setText(chatMap.getAddressLine2());
        binding.phoneNumber.setText(String.format("Phone: %s", chatMap.getPhoneNumber()));

        this.message = message;
        setMapLocation();
    }

    private void setMapLocation() {
        if (map == null || message == null) return;
        ChatMap data = message.getMap();

        // Add a marker for this item and set the camera
        map.moveCamera(CameraUpdateFactory.newLatLngZoom(data.getLatLng(), 16f));
        map.getUiSettings().setMapToolbarEnabled(false);
        map.addMarker(new MarkerOptions().position(data.getLatLng()));
        map.setOnMapClickListener(m -> {
            if (listener != null) listener.onAddressClicked(data);
        });
        map.setOnMarkerClickListener(m -> true);

        // Set the map type back to normal.
        map.setMapType(GoogleMap.MAP_TYPE_NORMAL);
    }


    @Override
    public void onMapReady(GoogleMap googleMap) {
        MapsInitializer.initialize(binding.getRoot().getContext());
        map = googleMap;
        setMapLocation();

    }
}
