package com.ibm.gbs.va.grit.biometrics;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageManager;
import android.hardware.fingerprint.FingerprintManager;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.FragmentManager;

import com.ibm.gbs.va.grit.util.CommonCallback;
import com.ibm.gbs.va.grit.util.RemoteLog;

import javax.crypto.Cipher;

public class FingerprintHelper {
    private static final String TAG = "FingerprintHelper";

    public void startFingerprintAlert(FragmentManager fragmentManager, Cipher cipher, CommonCallback fingerprintSuccesscallback, CommonCallback fingerprintErrorCallback, FingerprintAuthenticationDialogFragment.Stage fpStage) {
        // Show the fingerprint dialog. The user has the option to use the fingerprint with
        // crypto, or you can fall back to using a server-side verified password.
        FingerprintAuthenticationDialogFragment fragment = new FingerprintAuthenticationDialogFragment(fingerprintSuccesscallback, fingerprintErrorCallback);
        fragment.setCryptoObject(new FingerprintManager.CryptoObject(cipher));
        fragment.setStage(fpStage);
        fragment.show(fragmentManager, "FP_HELPER");
    }

    /*
     *
     * CHECK IF FINGERPRINT IS ENABLED
     *
     */
    public static boolean isFingerprintEnabled(Context context) {
        boolean isFingerprintEnabled = false;
        if (isFingerprintHardwareDetected(context)
                && hasEnrolledFingerprints(context)
                && isPermissionGranted(context)) {
            isFingerprintEnabled = true;
        } else {
            RemoteLog.d(TAG, "fingerprint not enabled.  hasHardware: " + isFingerprintHardwareDetected(context) + ", hasEnrolled: " + hasEnrolledFingerprints(context) + ", isPermission: " + isPermissionGranted(context));
        }
        return isFingerprintEnabled;
    }

    private static boolean isPermissionGranted(Context context) {
        return ActivityCompat.checkSelfPermission(context, Manifest.permission.USE_FINGERPRINT) ==
                PackageManager.PERMISSION_GRANTED;
    }

    private static boolean isFingerprintHardwareDetected(Context context) {
        FingerprintManager fingerprintManager = context.getSystemService(FingerprintManager.class);
        return fingerprintManager.isHardwareDetected();
    }

    private static boolean hasEnrolledFingerprints(Context context) {
        FingerprintManager fingerprintManager = context.getSystemService(FingerprintManager.class);
        return fingerprintManager.hasEnrolledFingerprints();
    }
}
