package com.ibm.gbs.va.grit.dagger;

import android.app.Application;
import android.content.Context;

import com.ibm.gbs.va.grit.GritApplication;
import com.ibm.gbs.va.grit.repository.AuthorizationRepository;
import com.ibm.gbs.va.grit.repository.BadgeRepository;
import com.ibm.gbs.va.grit.repository.ChatRepository;
import com.ibm.gbs.va.grit.repository.DigitalSelfRepository;
import com.ibm.gbs.va.grit.repository.JobRepository;
import com.ibm.gbs.va.grit.repository.LocationRepository;
import com.ibm.gbs.va.grit.repository.NotificationRepository;
import com.ibm.gbs.va.grit.repository.SocialMediaRepository;
import com.ibm.gbs.va.grit.repository.SquadRepository;
import com.ibm.gbs.va.grit.repository.SurveyRepository;
import com.ibm.gbs.va.grit.repository.UserRepository;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.rest.RetrofitApiServices;
import com.ibm.gbs.va.grit.rest.RetrofitInterceptor;
import com.ibm.gbs.va.grit.room.AnswerDao;
import com.ibm.gbs.va.grit.room.BadgeDao;
import com.ibm.gbs.va.grit.room.GritDatabase;
import com.ibm.gbs.va.grit.room.NotificationDao;
import com.ibm.gbs.va.grit.room.QSetDao;
import com.ibm.gbs.va.grit.room.SquadDao;
import com.ibm.gbs.va.grit.room.UserDao;
import com.ibm.gbs.va.grit.util.Analytics;
import com.ibm.gbs.va.grit.util.BadgeDelegate;
import com.ibm.gbs.va.grit.util.Keystore;
import com.ibm.gbs.va.grit.util.UserPrefs;

import javax.inject.Singleton;

import dagger.Binds;
import dagger.Module;
import dagger.Provides;

@Module(subcomponents = UserComponent.class)
public abstract class AppModule {

    @Binds
    public abstract Application bindApplication(GritApplication application);

    @Binds
    public abstract Context bindContext(Application application);

    @Binds
    public abstract NetworkErrorDelegate bindErrorDelegate(GritApplication application);

    @Binds
    public abstract BadgeDelegate bindBadgeDelegate(GritApplication application);


    @Provides
    @Singleton
    static BadgeRepository providesBadgeRepo(RetrofitApiServices restServices, UserPrefs userPrefs, GritApplication gritApplication, BadgeDao badgeDao) {
        return new BadgeRepository(restServices, userPrefs, gritApplication, badgeDao);
    }

    @Provides
    static UserRepository providesUserRepo(GritApplication gritApplication) {
        return gritApplication.getCurrentUserScope().getCurrentUserRepo();
    }

    @Provides
    static SurveyRepository providesSurveyRepo(GritApplication gritApplication) {
        return gritApplication.getCurrentUserScope().getCurrentSurveyRepo();
    }

    @Provides
    static NotificationRepository providesNotificationRepo(GritApplication gritApplication) {
        return gritApplication.getCurrentUserScope().getCurrentNotificationRepo();
    }

    @Provides
    @Singleton
    static LocationRepository providesLocationRepo(Context context) {
        return new LocationRepository(context);
    }

    @Provides
    static JobRepository providesJobRep(GritApplication gritApplication) {
        return gritApplication.getCurrentUserScope().getCurrentJobRepo();
    }

    @Provides
    static ChatRepository providesChatRepo(GritApplication gritApplication) {
        return gritApplication.getCurrentUserScope().getCurrentChatRepo();
    }

    @Provides
    static SocialMediaRepository providesSocialMediaRepo(GritApplication gritApplication) {
        return gritApplication.getCurrentUserScope().getSocialMediaRepository();
    }

    @Provides
    static SquadRepository providesSquadRepo(GritApplication gritApplication) {
        return gritApplication.getCurrentUserScope().getSquadRepository();
    }

    @Provides
    static AuthorizationRepository providesAuthorizationRepository(GritApplication gritApplication) {
        return gritApplication.getCurrentUserScope().getAuthorizationRepository();
    }

    @Provides
    static DigitalSelfRepository providesDigitalJournalRepository(GritApplication gritApplication) {
        return gritApplication.getCurrentUserScope().getDigitalJournalRepository();
    }

    @Provides
    @Singleton
    static UserPrefs providesPrefs(GritApplication gritApplication) {
        return new UserPrefs(gritApplication);
    }

    @Provides
    @Singleton
    static Keystore providesKeystore(UserPrefs userPrefs, Context context) {
        return new Keystore(userPrefs, context);
    }

    @Provides
    @Singleton
    static Analytics providesAnalytics(Context context) {
        return new Analytics(context);
    }

    @Provides
    @Singleton
    static RetrofitApiServices providesRestServices(Context context, RetrofitInterceptor retrofitInterceptor) {
        return new RetrofitApiServices(retrofitInterceptor);
    }

    @Provides
    static RetrofitInterceptor providesRestServicesInterceptor(Context context, UserPrefs userPrefs) {
        return new RetrofitInterceptor(context, userPrefs);
    }


    @Provides
    @Singleton
    static GritDatabase providesGritDatabase(Context context, Keystore keystore, UserPrefs userPrefs) {
        return GritDatabase.getInstance(context, keystore, userPrefs);
    }

    @Provides
    @Singleton
    static UserDao providesUserDao(GritDatabase db) {
        return db.userDao();
    }

    @Provides
    @Singleton
    static QSetDao providesQSetDao(GritDatabase db) {
        return db.qSetDao();
    }

    @Provides
    @Singleton
    static AnswerDao providesAnswerDao(GritDatabase db) {
        return db.answerDao();
    }

    @Provides
    @Singleton
    static BadgeDao providesBadgeDao(GritDatabase db) {
        return db.badgeDao();
    }

    @Provides
    @Singleton
    static SquadDao providesSquadDao(GritDatabase db) {
        return db.squadDao();
    }

    @Provides
    @Singleton
    static NotificationDao providesNotificationDao(GritDatabase db) {
        return db.notificationDao();
    }
}
