package com.ibm.gbs.va.grit.dagger;

import com.ibm.gbs.va.grit.GritApplication;
import com.ibm.gbs.va.grit.repository.AuthorizationRepository;
import com.ibm.gbs.va.grit.repository.BadgeRepository;
import com.ibm.gbs.va.grit.repository.ChatRepository;
import com.ibm.gbs.va.grit.repository.DigitalSelfRepository;
import com.ibm.gbs.va.grit.repository.JobRepository;
import com.ibm.gbs.va.grit.repository.NotificationRepository;
import com.ibm.gbs.va.grit.repository.SocialMediaRepository;
import com.ibm.gbs.va.grit.repository.SquadRepository;
import com.ibm.gbs.va.grit.repository.SurveyRepository;
import com.ibm.gbs.va.grit.repository.UserRepository;
import com.ibm.gbs.va.grit.rest.RetrofitApiServices;
import com.ibm.gbs.va.grit.room.AnswerDao;
import com.ibm.gbs.va.grit.room.NotificationDao;
import com.ibm.gbs.va.grit.room.QSetDao;
import com.ibm.gbs.va.grit.room.SquadDao;
import com.ibm.gbs.va.grit.room.UserDao;
import com.ibm.gbs.va.grit.util.Keystore;
import com.ibm.gbs.va.grit.util.UserPrefs;

import javax.inject.Named;

import dagger.Module;
import dagger.Provides;

@Module
public abstract class UserModule {

    @Provides
    @Named("NewRepo")
    @UserScope
    static UserRepository providesUserRepo(RetrofitApiServices restServices, UserPrefs userPrefs, Keystore keystore, UserDao userDao, NotificationRepository messageRepo, GritApplication gritApplication, AuthorizationRepository authorizationRepository) {
        return new UserRepository(restServices, userPrefs, keystore, userDao, messageRepo, gritApplication, authorizationRepository);
    }

    @Provides
    @Named("NewRepo")
    @UserScope
    static NotificationRepository providesMessageRepo(RetrofitApiServices restServices, UserPrefs userPrefs, GritApplication gritApplication, NotificationDao notificationDao) {
        return new NotificationRepository(restServices, userPrefs, gritApplication, notificationDao);
    }

    @Provides
    @Named("NewRepo")
    @UserScope
    static SurveyRepository providesSurveyRepo(RetrofitApiServices restServices, BadgeRepository badgeRepository, QSetDao surveyDao, AnswerDao answerDao, GritApplication gritApplication, NotificationRepository notificationRepository, UserRepository userRepository) {
        return new SurveyRepository(restServices, badgeRepository, surveyDao, answerDao, gritApplication, notificationRepository, userRepository);
    }

    @Provides
    @Named("NewRepo")
    @UserScope
    static JobRepository providesJobRepo(RetrofitApiServices restServices, UserPrefs userPrefs, NotificationRepository messageRepo, GritApplication gritApplication) {
        return new JobRepository(restServices, userPrefs, messageRepo, gritApplication);
    }

    @Provides
    @Named("NewRepo")
    @UserScope
    static ChatRepository providesChatRepo(RetrofitApiServices restServices, GritApplication gritApplication) {
        return new ChatRepository(restServices, gritApplication);
    }

    @Provides
    @Named("NewRepo")
    @UserScope
    static SocialMediaRepository providesSocialMediaRepo(RetrofitApiServices restServices, GritApplication gritApplication) {
        return new SocialMediaRepository(restServices, gritApplication);
    }

    @Provides
    @Named("NewRepo")
    @UserScope
    static SquadRepository providesSquadRepo(RetrofitApiServices restServices, UserRepository userRepo, GritApplication gritApplication, SquadDao squadDao, NotificationRepository messageRepo) {
        return new SquadRepository(restServices, userRepo, gritApplication, squadDao, messageRepo);
    }

    @Provides
    @Named("NewRepo")
    @UserScope
    static AuthorizationRepository providesAuthorizationRepo(UserPrefs userPrefs, GritApplication gritApplication) {
        return new AuthorizationRepository(userPrefs);
    }

    @Provides
    @Named("NewRepo")
    @UserScope
    static DigitalSelfRepository providesDigitalJournalRepo(GritApplication gritApplication, RetrofitApiServices restServices, BadgeRepository badgeRepository) {
        return new DigitalSelfRepository(gritApplication, restServices, badgeRepository);
    }

}
