package com.ibm.gbs.va.grit.drawable;

import android.animation.ValueAnimator;
import android.content.Context;
import android.support.v7.app.ActionBar;
import android.support.v7.graphics.drawable.DrawerArrowDrawable;

/**
 * A hamburger menu drawable that has a flashy animation to a back arrow.
 */
public class MenuArrowDrawable extends DrawerArrowDrawable {
    private final ValueAnimator mMenuToArrowAnimator;
    private final ValueAnimator mArrowToMenuAnimator;

    public MenuArrowDrawable(Context context, final ActionBar actionBar) {
        super(context);

        setColor(context.getColor(android.R.color.white));

        ValueAnimator.AnimatorUpdateListener animatorUpdateListener = animation -> {
            setPosition((float) animation.getAnimatedValue());
            if (actionBar != null) {
                actionBar.setHomeAsUpIndicator(MenuArrowDrawable.this);
            }
        };

        mMenuToArrowAnimator = ValueAnimator.ofFloat(0f, 1f);
        mMenuToArrowAnimator.setDuration(250);
        mMenuToArrowAnimator.addUpdateListener(animatorUpdateListener);

        mArrowToMenuAnimator = ValueAnimator.ofFloat(1f, 0f);
        mArrowToMenuAnimator.setDuration(250);
        mArrowToMenuAnimator.addUpdateListener(animatorUpdateListener);
    }

    private void setPosition(float position) {
        if (position >= 1f) {
            setVerticalMirror(true);
        } else if (position <= 0f) {
            setVerticalMirror(false);
        }
        setProgress(position);
    }

    public float getPosition() {
        return getProgress();
    }

    public void animateDrawable(boolean menuToArrow) {
        if (menuToArrow && getPosition() >= 1f) return;
        if (!menuToArrow && getPosition() <= 0f) return;

        ValueAnimator animator = menuToArrow ? mMenuToArrowAnimator : mArrowToMenuAnimator;
        if (animator.isRunning()) animator.end();
        animator.start();
    }
}
