package com.ibm.gbs.va.grit.fragment.chat;

import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.content.ActivityNotFoundException;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.speech.RecognizerIntent;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v7.util.DiffUtil;
import android.support.v7.util.ListUpdateCallback;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.EditorInfo;
import android.widget.TextView;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.adapter.viewholder.ButtonIncomingMessageViewHolder;
import com.ibm.gbs.va.grit.adapter.viewholder.DropdownIncomingMessageViewHolder;
import com.ibm.gbs.va.grit.adapter.viewholder.DropdownOutcomingMessageViewHolder;
import com.ibm.gbs.va.grit.adapter.viewholder.GritIncomingMessageViewHolder;
import com.ibm.gbs.va.grit.adapter.viewholder.GritIncomingMessageViewHolder.OnHyperLinkClickedListener;
import com.ibm.gbs.va.grit.adapter.viewholder.GritOutcomingMessageViewHolder;
import com.ibm.gbs.va.grit.adapter.viewholder.MapIncomingMessageViewHolder;
import com.ibm.gbs.va.grit.adapter.viewholder.MapIncomingMessageViewHolder.OnAddressClickedListener;
import com.ibm.gbs.va.grit.adapter.viewholder.OnChatItemSelectedListener;
import com.ibm.gbs.va.grit.databinding.FragmentChatBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.UniqueInfoButtonFragment;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.chat.ChatFollowup;
import com.ibm.gbs.va.grit.model.chat.ChatMessage;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.viewmodel.ChatViewModel;
import com.stfalcon.chatkit.messages.MessageHolders;
import com.stfalcon.chatkit.messages.MessagesListAdapter;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import static android.app.Activity.RESULT_OK;

public class ChatFragment extends AnalyticsFragment implements UniqueInfoButtonFragment {
    private static final int STT_RESULT = 1;

    private static final byte CONTENT_TYPE_MAP = 1;
    private static final byte CONTENT_TYPE_DROP_DOWN = 2;
    private static final byte CONTENT_TYPE_BUTTON = 3;
    private static final byte CONTENT_TYPE_RESPONSE = 4;
    /**
     * determines what ViewHolder to use for a message
     */
    private final static MessageHolders.ContentChecker<ChatMessage> checker = (ChatMessage message, byte type) -> {
        switch (type) {
            case CONTENT_TYPE_MAP:
                return message.getMap() != null;
            case CONTENT_TYPE_RESPONSE:
                return message.getFollowup() != null && !TextUtils.isEmpty(message.getText());
            case CONTENT_TYPE_DROP_DOWN:
                return message.getFollowup() != null &&
                        message.getFollowup().getFollowupType() == ChatFollowup.FollowupType.DROPDOWN;
            case CONTENT_TYPE_BUTTON:
                return message.getFollowup() != null &&
                        message.getFollowup().getFollowupType() == ChatFollowup.FollowupType.BUTTON;
        }
        return false;
    };
    private final ChatDiffUtilCallback chatDiffUtilCallback = new ChatDiffUtilCallback();

    @Inject
    ViewModelProvider.Factory viewModelFactory;
    private MessagesListAdapter<ChatMessage> adapter;
    private FragmentChatBinding binding;
    private ChatViewModel model;
    private List<ChatMessage> localMessageList = new ArrayList<>();

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup parent, @Nullable Bundle savedInstanceState) {
        assert getActivity() != null;
        TAG = "ChatFragment";

        binding = FragmentChatBinding.inflate(inflater, parent, false);
        binding.setLifecycleOwner(this);
        model = ViewModelProviders.of(this, viewModelFactory).get(ChatViewModel.class);

        if (model.isFirstTimeInfo()) {
            showInfoDialog();
            model.setFirstTimeInfo(false);
        }

        setupMessageList();

        // sync viewModel message list with adapter
        model.getMessages().observe(this, this::processMessages);

        // microphone button
        binding.input.setAttachmentsListener(() -> {
            Intent intent = new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH);
            intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, "en-US");
            try {
                startActivityForResult(intent, STT_RESULT);
            } catch (ActivityNotFoundException a) {
                voiceError();
            }
        });

        // on send message
        binding.input.setInputListener(submittedInput -> {
            sendMessage(ChatMessage.userMessage(submittedInput.toString()));
            return true;
        });
        binding.input.getInputEditText().setSingleLine();

        // Set the ime action on the user's keyboard to be send
        binding.input.getInputEditText().setImeOptions(EditorInfo.IME_ACTION_SEND);
        // Set custom action for send ime action, which submits and clears the text to the DA
        binding.input.getInputEditText().setOnEditorActionListener((TextView view, int actionId, KeyEvent event) -> {
            if (actionId == EditorInfo.IME_ACTION_SEND) {
                sendMessage(ChatMessage.userMessage(view.getText().toString()));
                view.setText("");
                return true;
            } else return false;
        });

        setHasOptionsMenu(true); // register for calls to onCreateOptionsMenu()

        return binding.getRoot();
    }

    @Override
    public void onResume() {
        super.onResume();
        if (localMessageList.size() != adapter.getMessagesCount()) {
            adapter.clear();
            localMessageList = new ArrayList<>();
            processMessages(model.getMessages().getValue());
        }
    }

    private void processMessages(List<ChatMessage> chatMessages) {
        DiffUtil.DiffResult diffResult = DiffUtil.calculateDiff(chatDiffUtilCallback.setLists(localMessageList, chatMessages), false);
        diffResult.dispatchUpdatesTo(new ListUpdateCallback() {
            @Override
            public void onInserted(int position, int count) {
                assert chatMessages != null;
                if (count > 1) adapter.addToEnd(new ArrayList<>(chatMessages), true);
                else {
                    adapter.addToStart(chatMessages.get(position), true);
                }
            }

            @Override
            public void onRemoved(int position, int count) {
            }

            @Override
            public void onMoved(int fromPosition, int toPosition) {
            }

            @Override
            public void onChanged(int position, int count, @Nullable Object payload) {
            }
        });
        localMessageList = chatMessages;
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case STT_RESULT: // received speech to text
                if (resultCode == RESULT_OK && null != data) {
                    ArrayList<String> text = data.getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS);
                    if (!text.isEmpty()) {
                        sendMessage(ChatMessage.userMessage(text.get(0)));
                    } else voiceError();
                }
                break;
            default:
                super.onActivityResult(requestCode, resultCode, data);
        }
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        menu.removeItem(R.id.chat);
        inflater.inflate(R.menu.chat_items, menu); // add chat to action bar
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.chatInfo:
                showInfoDialog();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    public void showInfoDialog() {
        GritDialogFragment.showSingleBtnInfoDialog(getFragmentManager(), R.string.dialog_chat_info_title,
                TextSection.ofText(R.string.dialog_chat_info_body, R.style.Caption2, false),
                TextSection.ofBullet(R.string.dialog_chat_info_bullet1, R.style.Caption, false),
                TextSection.ofBullet(R.string.dialog_chat_info_bullet2, R.style.Caption, false),
                TextSection.ofBullet(R.string.dialog_chat_info_bullet3, R.style.Caption, false),
                TextSection.ofBullet(R.string.dialog_chat_info_bullet4, R.style.Caption, false)
        );
    }

    @Override
    public void onStop() {
        super.onStop();
        if (getActivity() != null) getActivity().invalidateOptionsMenu(); // replace chat menu item
    }

    private void sendMessage(ChatMessage message) {
        model.converse(message);
    }

    private void voiceError() {
        MessageHandler.handleError(getActivity(), getString(R.string.network_error));
    }

    private void setupMessageList() {
        // map, hyperlink & button/dropdown OnClickListeners
        final OnAddressClickedListener onAddressClickedListener = chatMap -> {
            String address = String.format("%s %s %s", chatMap.getPlaceName(), chatMap.getAddressLine1(), chatMap.getAddressLine2());
            Uri gmmIntentUri = Uri.parse("https://www.google.com/maps/search/?api=1&query=" + Uri.encode(address));
            Intent mapIntent = new Intent(Intent.ACTION_VIEW, gmmIntentUri);
            startActivity(mapIntent);
        };
        final OnChatItemSelectedListener onChatItemSelectedListener = selectionOption -> sendMessage(ChatMessage.userFollowup(selectionOption));
        final OnHyperLinkClickedListener onHyperLinkClickedListener = url -> {
            Intent i = new Intent(Intent.ACTION_VIEW);
            i.setData(Uri.parse(url));
            startActivity(i);
        };
        // setup message ViewHolders
        MessageHolders holders = new MessageHolders()
                .setIncomingTextHolder(GritIncomingMessageViewHolder.class, onHyperLinkClickedListener)
                .setIncomingTextLayout(R.layout.chat_incoming_message)
                .setOutcomingTextHolder(GritOutcomingMessageViewHolder.class)
                .setOutcomingTextLayout(R.layout.chat_outcoming_message);

        holders.registerContentType(CONTENT_TYPE_MAP,
                // received messages
                MapIncomingMessageViewHolder.class,
                onAddressClickedListener,
                R.layout.chat_map_message,
                // sent messages
                MapIncomingMessageViewHolder.class,
                null,
                0,
                checker);

        holders.registerContentType(CONTENT_TYPE_DROP_DOWN,
                // received messages
                DropdownIncomingMessageViewHolder.class,
                onChatItemSelectedListener,
                R.layout.chat_dropdown_incoming_message,
                // sent messages
                DropdownOutcomingMessageViewHolder.class,
                null,
                R.layout.chat_dropdown_outcoming_message,
                checker);

        holders.registerContentType(CONTENT_TYPE_BUTTON,
                // received messages
                ButtonIncomingMessageViewHolder.class,
                onChatItemSelectedListener,
                R.layout.chat_button_incoming_message,
                // sent messages
                DropdownOutcomingMessageViewHolder.class,
                null,
                R.layout.chat_dropdown_outcoming_message,
                checker);

        holders.registerContentType(CONTENT_TYPE_RESPONSE,
                DropdownOutcomingMessageViewHolder.class,
                0,
                R.layout.chat_dropdown_outcoming_message,
                checker);

        adapter = new MessagesListAdapter<>("user", holders, (imageView, url, payload) -> {
            // set chat profile pictures
            if (TextUtils.equals(url, "user")) {
                imageView.setImageResource(R.drawable.ic_squad);
                imageView.setBackgroundColor(ChatFragment.this.getContext().getColor(R.color.colorContactBackground));
            } else
                imageView.setImageResource(R.drawable.chat_grit);
        });
        binding.list.setAdapter(adapter);
    }

    private class ChatDiffUtilCallback extends DiffUtil.Callback {
        private List<ChatMessage> oldList = new ArrayList<>();
        private List<ChatMessage> newList = new ArrayList<>();

        DiffUtil.Callback setLists(List<ChatMessage> oldList, List<ChatMessage> newList) {
            this.oldList = oldList;
            this.newList = newList;
            return this;
        }

        @Override
        public int getOldListSize() {
            return oldList.size();
        }

        @Override
        public int getNewListSize() {
            return newList.size();
        }

        @Override
        public boolean areItemsTheSame(int oldItemPosition, int newItemPosition) {
            return oldList.get(oldItemPosition).getId().equals(newList.get(newItemPosition).getId());
        }

        @Override
        public boolean areContentsTheSame(int oldItemPosition, int newItemPosition) {
            return oldList.get(oldItemPosition).getId().equals(newList.get(newItemPosition).getId());
        }
    }
}
