package com.ibm.gbs.va.grit.fragment.dashboard;

import android.arch.lifecycle.ViewModelProviders;
import android.databinding.ViewDataBinding;
import android.graphics.Rect;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v7.widget.DividerItemDecoration;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.PagerSnapHelper;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.SnapHelper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.BR;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.activity.DashboardActivity;
import com.ibm.gbs.va.grit.adapter.DashboardMessagePagerAdapter;
import com.ibm.gbs.va.grit.databinding.CardDashboardTileBinding;
import com.ibm.gbs.va.grit.databinding.FragmentDashboardBinding;
import com.ibm.gbs.va.grit.databinding.FragmentRecyclerviewBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.rest.NetworkErrorListener;
import com.ibm.gbs.va.grit.viewmodel.DashboardViewModel;

public class DashboardFragment extends AnalyticsFragment implements NetworkErrorDelegate {

    private DashboardViewModel model;
    private DashboardMessagePagerAdapter dbNotificationAdapter;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "DashboardFragment";
        assert getActivity() != null;

        if (getActivity() instanceof NetworkErrorListener) {
            ((NetworkErrorListener) getActivity()).addDelegateToList(this);
        }

        model = ViewModelProviders.of(getActivity()).get(DashboardViewModel.class);
        FragmentDashboardBinding binding = FragmentDashboardBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        DashboardRecyclerAdapter recyclerAdapter = new DashboardRecyclerAdapter();
        binding.tileRecyclerView.setAdapter(recyclerAdapter);
        model.hasMessages.observe(this, messagesToShow -> {
            if (messagesToShow && recyclerAdapter.getItemViewType(0) == DashboardRecyclerAdapter.MESSAGE_TYPE) {
                recyclerAdapter.notifyItemChanged(0);
            } else if (!messagesToShow) recyclerAdapter.notifyDataSetChanged();
        }); // needed so model can observe itself
        GridLayoutManager layoutManager = new GridLayoutManager(getContext(), 2);
        // expand width of the message widget when displayed
        layoutManager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
            @Override
            public int getSpanSize(int position) {
                return (position == 0) ? 2 : 1;
            }
        });
        binding.tileRecyclerView.setLayoutManager(layoutManager);
        binding.tileRecyclerView.addItemDecoration(leftAndRightMarginDecorator);

        return binding.getRoot();
    }


    @Override
    public boolean handleError(String message) {
        MessageHandler.handleError(getActivity(), message);
        return true;
    }

    @Override
    public boolean handleError(int messageRes) {
        return handleError(getString(messageRes));
    }


    class DashboardRecyclerAdapter extends RecyclerView.Adapter<DashboardViewHolder> {
        private static final int TILE_TYPE = 0;
        private static final int MESSAGE_TYPE = 1;
        private static final int NULL_TYPE = 2;

        @NonNull
        @Override
        public DashboardViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            LayoutInflater inflater = LayoutInflater.from(parent.getContext());

            if (viewType == TILE_TYPE) {
                return new DashboardViewHolder(CardDashboardTileBinding.inflate(inflater, parent, false));
            } else if (viewType == MESSAGE_TYPE) {
                FragmentRecyclerviewBinding binding = FragmentRecyclerviewBinding.inflate(inflater, parent, false);
                LinearLayoutManager layoutManager = new LinearLayoutManager(getContext(), LinearLayoutManager.HORIZONTAL, false);
                SnapHelper snapHelper = new PagerSnapHelper();
                binding.recyclerView.setLayoutManager(layoutManager);
                snapHelper.attachToRecyclerView(binding.recyclerView);

                DividerItemDecoration cardSpacing = new DividerItemDecoration(getContext(), DividerItemDecoration.HORIZONTAL);
                cardSpacing.setDrawable(getContext().getDrawable(R.drawable.notification_card_spacer));
                binding.recyclerView.addItemDecoration(cardSpacing);

                return new DashboardViewHolder(binding);
            } else {
                return new DashboardViewHolder();
            }
        }

        @Override
        public void onBindViewHolder(@NonNull DashboardViewHolder viewHolder, int position) {
            if (viewHolder.getItemViewType() == NULL_TYPE) return;
            if (position == 0 && viewHolder.getItemViewType() == MESSAGE_TYPE) {
                if (dbNotificationAdapter == null)
                    dbNotificationAdapter = new DashboardMessagePagerAdapter(model, DashboardFragment.this, (DashboardActivity) getActivity());
                viewHolder.bindAdapter(dbNotificationAdapter);
            } else {
                viewHolder.bind(model.tiles.get(position - 1));
            }
        }

        @Override
        public int getItemViewType(int position) {
            if (position != 0) return TILE_TYPE;
            Boolean hasMessages = model.hasMessages.getValue();
            return hasMessages != null && hasMessages ? MESSAGE_TYPE : NULL_TYPE;
        }

        @Override
        public int getItemCount() {
            return model.tiles.size() + 1;
        }
    }

    public class DashboardViewHolder extends RecyclerView.ViewHolder {
        final ViewDataBinding binding;

        DashboardViewHolder(@NonNull ViewDataBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
        }

        DashboardViewHolder() {
            super(new View(DashboardFragment.this.getContext()));
            binding = null;
        }

        void bind(DashboardViewModel.TileData model) {
            binding.setVariable(BR.model, model);
            binding.setVariable(BR.onClickListener, (View.OnClickListener) view -> ((DashboardActivity) getActivity()).swapFragments(model.id));
        }

        void bindAdapter(RecyclerView.Adapter adapter) {
            binding.setVariable(BR.adapter, adapter);
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        model.updateMessages();
    }

    /**
     * adds margin to tiles but not the message pager
     */
    private final RecyclerView.ItemDecoration leftAndRightMarginDecorator = new RecyclerView.ItemDecoration() {
        @Override
        public void getItemOffsets(@NonNull Rect outRect, @NonNull View view, @NonNull RecyclerView parent, @NonNull RecyclerView.State state) {
            int position = parent.getChildAdapterPosition(view);
            int sideMargin = (int) getResources().getDimension(R.dimen.grit_side_margin);
            int interCardMargin = (int) getResources().getDimension(R.dimen.dashboard_inter_card_spacing);
            int topMargin = (position == 1 || position == 2) ? sideMargin : 0;
            if (position == -1) super.getItemOffsets(outRect, view, parent, state);

            if (position == 0) return;
            if (position % 2 == 0) outRect.set(interCardMargin, topMargin, sideMargin, 0);
            else outRect.set(sideMargin, topMargin, interCardMargin, 0);

        }
    };
}
