package com.ibm.gbs.va.grit.fragment.dashboard;

import android.Manifest;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.app.AppCompatActivity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.FragmentImmediateHelpBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.squad.SquadFragment;
import com.ibm.gbs.va.grit.util.GritPermissionHelper;
import com.ibm.gbs.va.grit.viewmodel.ListItemLinkViewModel;

public class ImmediateHelpFragment extends AnalyticsFragment {

    public enum ContactActionType {
        Email, Phone, Text, Website
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "ImmediateHelpFragment";
        assert getActivity() != null;
        FragmentImmediateHelpBinding binding = FragmentImmediateHelpBinding.inflate(inflater, container, false);

        // setup action buttons
        binding.executePendingBindings();
        setActionListenerFor(binding.crisisWebsite.getModel());
        setActionListenerFor(binding.crisisChatWebsite.getModel());
        setActionListenerFor(binding.crisisCallLayout.getModel());
        setActionListenerFor(binding.crisisTextLayout.getModel());
        setActionListenerFor(binding.crisisHardOfHearingLayout.getModel());

        setActionListenerFor(binding.crisisTextLine.getModel());
        setActionListenerFor(binding.crisisTextWebsite.getModel());

        setActionListenerFor(binding.objectiveZeroWebsite.getModel());
        binding.objectiveZeroPlaystoreLink.setOnClickListener(v -> openPlaystore(
                getString(R.string.objective_zero_playstore_link), getString(R.string.objective_zero_playstore_web_link)));

        binding.squadLink.setOnClickListener(v -> goToSquad());

        binding.expandedFab.setOnClickListener(v -> callNumber(getString(R.string.veteran_crisis_dialable_number)));

        return binding.getRoot();
    }

    private void goToSquad() {
        Fragment frag = new SquadFragment();
        ((AppCompatActivity) getActivity()).getSupportActionBar().setTitle(R.string.squad);
        getFragmentManager().beginTransaction()
                .setCustomAnimations(R.anim.slide_in_right, R.anim.slide_out_left, android.R.anim.slide_in_left, android.R.anim.slide_out_right)
                .replace(R.id.contentFrame, frag, String.valueOf(R.id.squad))
                .addToBackStack(String.valueOf(R.id.squad))
                .commit();
    }

    private void setActionListenerFor(ListItemLinkViewModel item) {
        String actionText = item.actionAddress != null ? item.actionAddress : item.subtitle;
        View.OnClickListener listener = null;
        switch (item.actionType) {
            case Email:
                listener = v -> sendEmail(actionText);
                break;
            case Phone:
                listener = v -> dialNumber(actionText);
                break;
            case Text:
                listener = v -> textNumber(actionText);
                break;
            case Website:
                listener = v -> openWebPage(actionText);
                break;
        }
        item.onClickListener.set(listener);
    }

    private void dialNumber(String num) {
        startActivity(new Intent(Intent.ACTION_DIAL, Uri.parse("tel:" + num)));
    }

    private void callNumber(String num) {
        GritPermissionHelper gritPermissionHelper = new GritPermissionHelper(getActivity(), getString(R.string.permission_justification_call_phone));
        gritPermissionHelper.checkPermissions(Manifest.permission.CALL_PHONE, () -> {
            startActivity(new Intent(Intent.ACTION_CALL, Uri.parse("tel:" + num)));
            return null;
        });
    }

    private void textNumber(String value) {
        String[] split = value.split("\\|");
        String num = split[0];
        String toText = split.length > 1 ? split[1] : null;

        Intent intent = new Intent(Intent.ACTION_SENDTO, Uri.parse("smsto:" + num));
        intent.putExtra("sms_body", toText);
        startActivity(intent);
    }

    private void sendEmail(String address) {
        startActivity(new Intent(Intent.ACTION_SENDTO, Uri.parse("mailto:" + address)));
    }

    private void openWebPage(String page) {
        startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(page)));
    }

    private void openPlaystore(String page, String webPage) {
        try {
            startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(page)));
        } catch (android.content.ActivityNotFoundException anfe) {
            startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(webPage)));
        }
    }
}
