package com.ibm.gbs.va.grit.fragment.dialog;

import android.app.Dialog;
import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.annotation.StringRes;
import android.support.v4.app.FragmentManager;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.DialogChangePasswordBinding;
import com.ibm.gbs.va.grit.model.ValidationRule;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.rest.NetworkErrorListener;
import com.ibm.gbs.va.grit.util.LocalValidationHelper;
import com.ibm.gbs.va.grit.viewmodel.ChangePasswordViewModel;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import dagger.android.support.DaggerAppCompatDialogFragment;

public class ChangePasswordDialogFragment extends DaggerAppCompatDialogFragment implements NetworkErrorDelegate {
    static final String CHANGE_PASSWORD_DIALOG_TAG = "change_password_dialog";

    public static void showInstance(FragmentManager fragmentManager) {
        ChangePasswordDialogFragment dialog = new ChangePasswordDialogFragment();
        dialog.show(fragmentManager, CHANGE_PASSWORD_DIALOG_TAG);
    }

    private LocalValidationHelper oldPasswordValidator;
    private LocalValidationHelper newPasswordValidator;

    LiveData<Boolean> updatingPassword;
    ChangePasswordViewModel model;
    DialogChangePasswordBinding binding;

    private List<LocalValidationHelper> validationHelpers;

    @Inject
    ViewModelProvider.Factory viewModelFactory;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        assert getActivity() != null;
        if (getActivity() instanceof NetworkErrorListener) {
            ((NetworkErrorListener) getActivity()).addDelegateToList(this);
        }

        binding = DialogChangePasswordBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);
        model = ViewModelProviders.of(this, viewModelFactory).get(ChangePasswordViewModel.class);
        binding.setModel(model);

        validationHelpers = new ArrayList<>();
        newPasswordValidator = new LocalValidationHelper(binding.newPasswordLayout, ValidationRule.PASSWORD);
        oldPasswordValidator = new LocalValidationHelper(binding.oldPasswordLayout, ValidationRule.NO_EMPTY);
        newPasswordValidator.validateAfterChange();
        oldPasswordValidator.validateAfterChange();
        validationHelpers.add(newPasswordValidator);
        validationHelpers.add(oldPasswordValidator);

        binding.cancelBtn.setOnClickListener(v -> dismiss());
        binding.saveBtn.setOnClickListener(v ->
                LocalValidationHelper.validateInputs(validationHelpers, l ->
                        validatePasswordMatch()));

        return binding.getRoot();
    }

    @Override
    public void onResume() {
        super.onResume();
        ViewGroup.LayoutParams params = getDialog().getWindow().getAttributes();
        DisplayMetrics displayMetrics = new DisplayMetrics();
        getActivity().getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);
        int width = displayMetrics.widthPixels;
        params.height = ViewGroup.LayoutParams.MATCH_PARENT;
        params.width = Double.valueOf(width * 0.9).intValue();
        getDialog().getWindow().setAttributes((android.view.WindowManager.LayoutParams) params);
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        // remove title from dialog
        Dialog dialog = new Dialog(getActivity(), R.style.Dialog);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        setCancelable(false);
        return dialog;
    }

    private void validatePasswordMatch() {
        if (TextUtils.equals(binding.newPasswordText.getText(), binding.oldPasswordText.getText())) { // old & new are identical
            newPasswordValidator.setError(getString(R.string.passwords_cannot_match));
            newPasswordValidator.getInputLayout().requestFocus();
        } else {
            onSubmit();
        }
    }

    void onSubmit() {
        updatingPassword = model.updatePassword();
        updatingPassword.observe(this, passwordUpdated -> {
            if (passwordUpdated != null) {
                if (passwordUpdated) {
                    dismiss();
                    GritDialogFragment.showSuccessDialog(getFragmentManager(), R.string.success_dialog_password_update_body);

                } else {
                    passwordUpdateFailed(null);
                }
                updatingPassword.removeObservers(this);
            }
        });
    }

    private void passwordUpdateFailed(@StringRes Integer str) {
        str = str != null ? str : R.string.network_error;
        MessageHandler.handleMessageOnContainer(binding.container, getString(str));
        oldPasswordValidator.clearIcons();
        newPasswordValidator.clearIcons();
    }

    @Override
    public boolean handleError(String message) {
        if (TextUtils.equals(message, getString(R.string.error_incorrect_password))) {
            updatingPassword.removeObservers(this);
            updatingPassword = null;
            passwordUpdateFailed(R.string.error_incorrect_password_display);
            return true;
        }
        if (updatingPassword != null && updatingPassword.hasObservers()) {
            updatingPassword.removeObservers(this);
            updatingPassword = null;
            passwordUpdateFailed(null);
            return true;
        }
        return false;
    }

    @Override
    public boolean handleError(int messageRes) {
        return handleError(getString(messageRes));
    }
}
