package com.ibm.gbs.va.grit.fragment.dialog;

import android.app.Dialog;
import android.content.res.Resources;
import android.databinding.ViewDataBinding;
import android.graphics.Color;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.annotation.StringRes;
import android.support.graphics.drawable.VectorDrawableCompat;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;

import com.ibm.gbs.va.grit.BR;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.DialogBadgeFrameBinding;
import com.ibm.gbs.va.grit.databinding.DialogFrameBinding;
import com.ibm.gbs.va.grit.databinding.DialogImageFrameBinding;
import com.ibm.gbs.va.grit.databinding.DialogScrollableFrameBinding;
import com.ibm.gbs.va.grit.model.dialog.BadgeDialogModel;
import com.ibm.gbs.va.grit.model.dialog.BaseDialogModel;
import com.ibm.gbs.va.grit.model.dialog.ButtonConfig;
import com.ibm.gbs.va.grit.model.dialog.DialogModel;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.util.CompositeOnClickListener;
import com.ibm.gbs.va.grit.util.RemoteLog;

import dagger.android.support.DaggerAppCompatDialogFragment;

public class GritDialogFragment extends DaggerAppCompatDialogFragment {
    private final static String TAG = "GritDialogFragment";

    //region Dialog Constants
    private final static String SUCCESS_DIALOG_TAG = "SUCCESS_DIALOG";
    private final static String INFO_DIALOG_TAG = "INFO_DIALOG";
    private final static String SAVE_AND_EXIT_SURVEY_DIALOG_TAG = "SAVE_AND_EXIT_SURVEY_DIALOG";
    private final static String DYNAMIC_IMAGE_DIALOG_TAG = "DYNAMIC_IMAGE_DIALOG";
    private final static String EARNED_BADGE_DIALOG = "EARNED_BADGE_DIALOG";
    private final static String BLOCKING_INPUT_DIALOG = "BLOCKING_INPUT_DIALOG";

    private final static ButtonConfig INVERSE_BTN_STYLE = new ButtonConfig(0, R.color.colorPrimaryText, R.color.colorBackground);
    private final static ButtonConfig ACTIVE_BTN_STYLE = new ButtonConfig(0, R.color.colorLightText, R.color.colorPrimary);

    private final static DialogModel SUCCESS_MODEL = DialogModel.builder()
            .withTitle(R.string.success)
            .withBannerColor(R.color.colorSuccess)
            .withBannerIcon(DialogModel.BannerIcon.Success)
            .withBottomButton(ACTIVE_BTN_STYLE.copyWithText(R.string.ok))
            .build();

    private final static DialogModel INFO_MODEL = DialogModel.builder()
            .withBannerColor(R.color.colorSecondary)
            .withBannerIcon(DialogModel.BannerIcon.Info)
            .build();

    private final static DialogModel WARNING_MODEL = DialogModel.builder()
            .withBannerColor(R.color.colorWarning)
            .withBannerIcon(DialogModel.BannerIcon.Warning)
            .build();
    //endregion

    private static View.OnClickListener dismissDialog(DialogFragment d) {
        return v -> d.dismiss();
    }

    public static void showSuccessDialog(FragmentManager fragmentManager, @StringRes int bodyText) {
        GritDialogFragment d = new GritDialogFragment();
        d.model = DialogModel.builder().copyFrom(SUCCESS_MODEL)
                .withSections(
                        TextSection.ofText(bodyText))
                .withBottomOnClick(dismissDialog(d)).build();

        d.show(fragmentManager, SUCCESS_DIALOG_TAG);
    }

    //region Info Dialogs
    public static void showSingleBtnInfoDialog(FragmentManager fragmentManager, @StringRes int titleText, TextSection... textSections) {
        GritDialogFragment d = new GritDialogFragment();
        d.model = DialogModel.builder().copyFrom(INFO_MODEL)
                .withTitle(titleText)
                .withScrolling()
                .withSections(textSections)
                .withBottomButton(INVERSE_BTN_STYLE.copyWithText(R.string.close))
                .withBottomOnClick(dismissDialog(d))
                .build();

        d.show(fragmentManager, INFO_DIALOG_TAG);
    }

    public static void showTwoBtnInfoDialog(FragmentManager fragmentManager, @StringRes int titleText, @StringRes int okText,
                                            @StringRes int cancelText, View.OnClickListener okOnClickListener,
                                            View.OnClickListener cancelOnClickListener, TextSection... textSections) {
        GritDialogFragment d = new GritDialogFragment();
        DialogModel.DialogBuilder builder = DialogModel.builder().copyFrom(INFO_MODEL)
                .withTitle(titleText)
                .withSections(textSections)
                .withTopButton(ACTIVE_BTN_STYLE.copyWithText(okText))
                .withBottomButton(INVERSE_BTN_STYLE.copyWithText(cancelText))
                .withTopOnClick(new CompositeOnClickListener(dismissDialog(d), okOnClickListener));

        if (cancelOnClickListener == null) {
            builder.withBottomOnClick(dismissDialog(d));
        } else {
            builder.withBottomOnClick(new CompositeOnClickListener(dismissDialog(d), cancelOnClickListener));
        }

        d.model = builder.build();
        d.show(fragmentManager, INFO_DIALOG_TAG);
    }

    public static void showTwoBtnWarningDialog(FragmentManager fragmentManager, @StringRes int titleText, View.OnClickListener onClickListener, TextSection... textSections) {
        showTwoBtnWarningDialog(fragmentManager, titleText, onClickListener, null, textSections);
    }

    private static void showTwoBtnWarningDialog(FragmentManager fragmentManager, @StringRes int titleText,
                                                View.OnClickListener okOnClickListener,
                                                View.OnClickListener cancelOnClickListener, TextSection... textSections) {
        GritDialogFragment d = new GritDialogFragment();
        DialogModel.DialogBuilder builder = DialogModel.builder().copyFrom(WARNING_MODEL)
                .withTitle(titleText)
                .withSections(textSections)
                .withTopButton(ACTIVE_BTN_STYLE.copyWithText(R.string.ok))
                .withBottomButton(INVERSE_BTN_STYLE.copyWithText(R.string.cancel))
                .withTopOnClick(new CompositeOnClickListener(dismissDialog(d), okOnClickListener));

        if (cancelOnClickListener == null) {
            builder.withBottomOnClick(dismissDialog(d));
        } else {
            builder.withBottomOnClick(new CompositeOnClickListener(dismissDialog(d), cancelOnClickListener));
        }

        d.model = builder.build();
        d.show(fragmentManager, INFO_DIALOG_TAG);
    }

    public static void showSquadInfoDialog(FragmentManager fragmentManager) {
        showSingleBtnInfoDialog(fragmentManager, R.string.dialog_squad_info_title,
                TextSection.ofText(R.string.dialog_squad_info_body1, R.style.Caption2, false),
                TextSection.ofBullet(R.string.dialog_squad_info_body2, R.style.Caption, false),
                TextSection.ofText(R.string.dialog_squad_info_body3, R.style.Caption2, false),
                TextSection.ofBullet(R.string.dialog_squad_info_body4, R.style.Caption, false),
                TextSection.ofText(R.string.dialog_squad_info_body5, R.style.Caption2, false),
                TextSection.ofBullet(R.string.dialog_squad_info_body6, R.style.Caption, false),
                TextSection.ofText(R.string.dialog_squad_info_body7, R.style.Caption2, false),
                TextSection.ofBullet(R.string.dialog_squad_info_body8, R.style.Caption, false),
                TextSection.ofBullet(R.string.dialog_squad_info_body9, R.style.Caption, false)
        );
    }

    public static void showJobsInfoDialog(FragmentManager fragmentManager) {
        GritDialogFragment.showSingleBtnInfoDialog(fragmentManager, R.string.job_dialog_info_title,
                TextSection.ofText(R.string.job_dialog_info_body1, R.style.Caption2, false),
                TextSection.ofBullet(R.string.job_dialog_info_body2, R.style.Caption, false),
                TextSection.ofText(R.string.job_dialog_info_body3, R.style.Caption2, false),
                TextSection.ofBullet(R.string.job_dialog_info_body4, R.style.Caption, false),
                TextSection.ofText(R.string.job_dialog_info_body5, R.style.Caption2, false),
                TextSection.ofBullet(R.string.job_dialog_info_body6, R.style.Caption, false),
                TextSection.ofText(R.string.job_dialog_info_body7, R.style.Caption2, false),
                TextSection.ofBullet(R.string.job_dialog_info_body8, R.style.Caption, false)
        );
    }
    //endregion

    public static void showBadgeDialog(FragmentManager fragmentManager, Resources resources, Badge badge, @Nullable View.OnClickListener onDismissListener) {
        GritDialogFragment d = new GritDialogFragment();
        d.model = BadgeDialogModel.builder()
                .withBannerColor(android.R.color.white)
                .withBackground(badge.getBadgeCategory().getBackground())
                .withBadge(new BitmapDrawable(resources, badge.getEnlargedAwardedImageBitmap()))
                .withTitle(badge.getDefaultTitle())
                .withSections(badge.getBadgeTextSection())
                .withBottomButton(ACTIVE_BTN_STYLE.copyWithText(R.string.ok))
                .withBottomOnClick(new CompositeOnClickListener(dismissDialog(d), onDismissListener))
                .build();

        d.show(fragmentManager, EARNED_BADGE_DIALOG);
    }

    public static void showUnawardedBadgeDialog(FragmentManager fragmentManager, Resources resources, Badge badge) {
        GritDialogFragment d = new GritDialogFragment();
        d.model = BadgeDialogModel.builder()
                .withBannerColor(android.R.color.white)
                .withBackground(badge.getBadgeCategory().getBackground())
                .withBadge(new BitmapDrawable(resources, badge.getEnlargedUnawardedImageBitmap()))
                .withTitle(badge.getDefaultTitle())
                .withSections(badge.getUnawardedBadgeTextSection())
                .withBottomButton(ACTIVE_BTN_STYLE.copyWithText(R.string.ok))
                .withBottomOnClick(dismissDialog(d))
                .build();

        d.show(fragmentManager, DYNAMIC_IMAGE_DIALOG_TAG);
    }

    public static void showSurveyIntroDialog(FragmentManager fragmentManager, Resources resources) {
        GritDialogFragment d = new GritDialogFragment();
        d.model = BadgeDialogModel.builder()
                .withBannerColor(android.R.color.white)
                .withBackground(R.drawable.badge_cat_baseline)
                .withBadge(VectorDrawableCompat.create(resources, R.drawable.ic_journey, null))
                .withTitle(resources.getString(R.string.dialog_survey_start_title))
                .withSections(TextSection.ofCenteredText(R.string.dialog_survey_start_body, R.style.Body_Inverse))
                .withBottomButton(ACTIVE_BTN_STYLE.copyWithText(R.string.ok))
                .withBottomOnClick(dismissDialog(d))
                .build();

        d.show(fragmentManager, DYNAMIC_IMAGE_DIALOG_TAG);
    }

    public static void showResumePreviewDialog(FragmentManager fragmentManager, View.OnClickListener retakeOnClickListener, TextSection... textSections) {
        GritDialogFragment d = new GritDialogFragment();
        DialogModel.DialogBuilder builder = DialogModel.builder()
                .withBannerColor(R.color.colorSuccess)
                .withBannerIcon(DialogModel.BannerIcon.Success)
                .withTitle(R.string.resume_preview)
                .withSections(textSections)
                .withTopButton(ACTIVE_BTN_STYLE.copyWithText(R.string.dialog_continue))
                .withBottomButton(INVERSE_BTN_STYLE.copyWithText(R.string.dialog_retake_photos))
                .withTopOnClick(dismissDialog(d))
                .withBottomOnClick(new CompositeOnClickListener(dismissDialog(d), retakeOnClickListener))
                .withScrolling();

        d.model = builder.build();
        d.show(fragmentManager, BLOCKING_INPUT_DIALOG);
    }

    //region Survey Dialogs

    /**
     * The user is in the middle of a survey and may be trying to navigate away from it
     *
     * @param fragmentManager       manager for the dialog
     * @param onSaveAndExitListener what to do when the user does want to leave the survey for now
     */
    public static void showSaveAndExitSurveyDialog(FragmentManager fragmentManager, View.OnClickListener onSaveAndExitListener) {
        GritDialogFragment d = new GritDialogFragment();
        d.model = DialogModel.builder().copyFrom(INFO_MODEL)
                .withTitle(R.string.dialog_save_exit)
                .withBannerIcon(DialogModel.BannerIcon.Save)
                .withSections(TextSection.ofText(R.string.dialog_save_exit_body))
                .withTopButton(ACTIVE_BTN_STYLE.copyWithText(R.string.dialog_continue))
                .withBottomButton(INVERSE_BTN_STYLE.copyWithText(R.string.dialog_save_exit))
                .withTopOnClick(dismissDialog(d))
                .withBottomOnClick(onSaveAndExitListener)
                .build();

        d.show(fragmentManager, SAVE_AND_EXIT_SURVEY_DIALOG_TAG);
    }

    /**
     * The user earned a badge completing the last survey and has the option to take another or
     * return to the dashboard
     *
     * @param fragmentManager   manager for the dialog
     * @param resources         resources
     * @param badge             the badge earned
     * @param nextSurveyOnClick what to do when the user wants to take the next survey
     * @param exitOnClick       what to do when the user wants to return to the dashboard
     */
    public static void showSurveyEndWithNextAndBadgeDialog(FragmentManager fragmentManager, Resources resources, Badge badge, View.OnClickListener nextSurveyOnClick, View.OnClickListener exitOnClick, TextSection section) {
        GritDialogFragment d = new GritDialogFragment();
        d.model = BadgeDialogModel.builder()
                .withBannerColor(android.R.color.white)
                .withBackground(badge.getBadgeCategory().getBackground())
                .withBadge(new BitmapDrawable(resources, badge.getEnlargedAwardedImageBitmap()))
                .withTitle(badge.getDefaultTitle())
                .withSections(section)
                .withTopButton(ACTIVE_BTN_STYLE.copyWithText(R.string.next_survey))
                .withBottomButton(INVERSE_BTN_STYLE.copyWithText(R.string.dialog_save_exit))
                .withTopOnClick(new CompositeOnClickListener(
                        dismissDialog(d),
                        nextSurveyOnClick))
                .withBottomOnClick(new CompositeOnClickListener(
                        dismissDialog(d),
                        exitOnClick))
                .build();

        d.show(fragmentManager, EARNED_BADGE_DIALOG);
    }

    /**
     * The user completed a survey and has the option to take another or return to the dashboard
     *
     * @param fragmentManager   manager for the dialog
     * @param nextSurveyOnClick what to do when the user wants to take the next survey
     * @param exitOnClick       what to do when the user wants to return to the dashboard
     */
    public static void showSurveyEndWithNextDialog(FragmentManager fragmentManager, String surveyName, View.OnClickListener nextSurveyOnClick, View.OnClickListener exitOnClick, TextSection section) {
        GritDialogFragment d = new GritDialogFragment();
        d.model = DialogModel.builder().copyFrom(SUCCESS_MODEL)
                .withTitle(surveyName)
                .withSections(section)
                .withTopButton(ACTIVE_BTN_STYLE.copyWithText(R.string.next_survey))
                .withBottomButton(INVERSE_BTN_STYLE.copyWithText(R.string.dialog_save_exit))
                .withTopOnClick(new CompositeOnClickListener(
                        dismissDialog(d),
                        nextSurveyOnClick))
                .withBottomOnClick(new CompositeOnClickListener(
                        dismissDialog(d),
                        exitOnClick))
                .build();

        d.show(fragmentManager, SUCCESS_DIALOG_TAG);
    }

    /**
     * The user earned a badge completing all available surveys and can only return to the
     * dashboard
     *
     * @param fragmentManager manager for the dialog
     * @param resources       resources
     * @param badge           the badge earned
     * @param exitOnClick     what to do when the user clicks the dismiss button
     */
    public static void showSurveyEndWithBadgeDialog(FragmentManager fragmentManager, Resources resources, Badge badge, View.OnClickListener exitOnClick, TextSection textSection) {
        GritDialogFragment d = new GritDialogFragment();
        d.model = BadgeDialogModel.builder()
                .withBannerColor(android.R.color.white)
                .withBackground(badge.getBadgeCategory().getBackground())
                .withBadge(new BitmapDrawable(resources, badge.getEnlargedAwardedImageBitmap()))
                .withTitle(badge.getDefaultTitle())
                .withSections(textSection)
                .withBottomButton(ACTIVE_BTN_STYLE.copyWithText(R.string.ok))
                .withBottomOnClick(new CompositeOnClickListener(dismissDialog(d), exitOnClick))
                .build();

        d.show(fragmentManager, EARNED_BADGE_DIALOG);
    }

    /**
     * The user is out of surveys and can only return to the dashboard
     *
     * @param fragmentManager manager for the dialog
     * @param exitOnClick     what to do when the user clicks the dismiss button
     */
    public static void showSurveyEndDialog(FragmentManager fragmentManager, String surveyName, View.OnClickListener exitOnClick, TextSection textSection) {
        GritDialogFragment d = new GritDialogFragment();
        d.model = DialogModel.builder().copyFrom(SUCCESS_MODEL)
                .withTitle(surveyName)
                .withSections(textSection)
                .withBottomOnClick(new CompositeOnClickListener(
                        dismissDialog(d),
                        exitOnClick))
                .build();

        d.show(fragmentManager, SUCCESS_DIALOG_TAG);
    }

    /**
     * The user chose to deactivate their account
     *
     * @param fragmentManager   manager for the dialog
     * @param deactivateOnClick what to do when the user wants to deactivate their account
     */
    public static void showDeactivateAccountDialog(FragmentManager fragmentManager, View.OnClickListener deactivateOnClick) {
        GritDialogFragment d = new GritDialogFragment();
        d.model = DialogModel.builder().copyFrom(SUCCESS_MODEL)
                .withTitle(R.string.deactivate_dialog_title)
                .withSections(
                        TextSection.ofText(R.string.deactivate_dialog_body))
                .withTopButton(ACTIVE_BTN_STYLE.copyWithText(R.string.deactivate_button))
                .withBottomButton(INVERSE_BTN_STYLE.copyWithText(R.string.cancel))
                .withTopOnClick(new CompositeOnClickListener(
                        dismissDialog(d),
                        deactivateOnClick))
                .withBottomOnClick(dismissDialog(d))
                .build();
        d.show(fragmentManager, BLOCKING_INPUT_DIALOG);
    }
    //endregion

    private BaseDialogModel model;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        ViewDataBinding binding;
        if (model instanceof DialogModel) {
            if (((DialogModel) model).getBannerImage() != null) {
                binding = DialogImageFrameBinding.inflate(inflater, container, false);
            } else if (model.isScrollable()) {
                binding = DialogScrollableFrameBinding.inflate(inflater, container, false);
            } else {
                binding = DialogFrameBinding.inflate(inflater, container, false);
            }
        } else /* if (model instanceof BadgeDialogModel) */ {
            binding = DialogBadgeFrameBinding.inflate(inflater, container, false);
        }

        binding.setVariable(BR.model, model);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        dismissOpenDialogs(getFragmentManager());
    }

    @Override
    public void onResume() {
        super.onResume();
        if (model instanceof BadgeDialogModel) {
            ViewGroup.LayoutParams params = getDialog().getWindow().getAttributes();
            DisplayMetrics displayMetrics = new DisplayMetrics();
            getActivity().getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);
            int height = displayMetrics.heightPixels;
            params.height = Double.valueOf(height * 0.9).intValue();
            getDialog().getWindow().setAttributes((android.view.WindowManager.LayoutParams) params);
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        // remove title from dialog
        Dialog dialog = new Dialog(getActivity(), R.style.Dialog);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        setCancelable(false);
        return dialog;
    }

    private void dismissOpenDialogs(FragmentManager fragManager) {
        for (Fragment frag : fragManager.getFragments()) {
            if (frag instanceof DialogFragment && frag != this
                    && !TextUtils.equals(frag.getTag(), EARNED_BADGE_DIALOG)
                    && !TextUtils.equals(frag.getTag(), BLOCKING_INPUT_DIALOG)) {
                RemoteLog.d(TAG, "dismissed dialog with tag \'" + frag.getTag() + "\' early.");
                ((DialogFragment) frag).dismiss();
            }
        }
    }
}
