package com.ibm.gbs.va.grit.fragment.digitalself;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.Transformations;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.content.Intent;
import android.graphics.Typeface;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.button.MaterialButton;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.style.ForegroundColorSpan;
import android.text.style.StyleSpan;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.activity.SurveyActivity;
import com.ibm.gbs.va.grit.databinding.TabBadgeListBinding;
import com.ibm.gbs.va.grit.fragment.UniqueInfoButtonFragment;
import com.ibm.gbs.va.grit.fragment.ViewPagerItemFragment;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.model.type.BadgeListBadge;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.viewmodel.BadgeListViewModel;

import java.util.HashMap;
import java.util.List;

import javax.inject.Inject;

public class BadgeListFragment extends ViewPagerItemFragment implements NetworkErrorDelegate, UniqueInfoButtonFragment {

    @Inject
    ViewModelProvider.Factory viewModelFactory;

    private BadgeListViewModel model;
    private TabBadgeListBinding binding;


    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "BadgeListFragment";
        assert getActivity() != null;

        binding = TabBadgeListBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        model = ViewModelProviders.of(this, viewModelFactory).get(BadgeListViewModel.class);

        if (isInitialPage) onSelected();

        model.updateAllBadges();
        // update the list of badgeListBadges to have the current fragment manager to create dialogs
        LiveData<HashMap<Badge.BadgeCategory, List<BadgeListBadge>>> badgeListWithContext = Transformations.map(model.getAllBadges(), (badgeCategoryListHashMap) -> {
            for (List<BadgeListBadge> curBadgeList : badgeCategoryListHashMap.values())
                for (BadgeListBadge curBadge : curBadgeList) {
                    curBadge.setFragmentManager(getFragmentManager());
                }
            return badgeCategoryListHashMap;
        });

        model.getCurrentSurveyNotificationName().observe(this,
                (newName) -> initializeContinueSurveyButton(binding.ContinueSurveyButtonBadgeList));

        model.setAllBadgesWithContext(badgeListWithContext);

        binding.setModel(model);

        initializeContinueSurveyButton(binding.ContinueSurveyButtonBadgeList);

        return binding.getRoot();
    }

    private void initializeContinueSurveyButton(MaterialButton buttonToInitialize) {
        boolean ongoingSurvey = model.isOngoingSurvey();
        Notification surveyQueued = model.getNextQueuedSurveyNotification();
        if (ongoingSurvey) {
            // Direct to current survey
            String text = getString(R.string.ds_survey_continue);
            buttonToInitialize.setText(formatText(text));
            buttonToInitialize.setOnClickListener((View v) -> {
                Intent surveyIntent = new Intent(getActivity(), SurveyActivity.class);
                startActivity(surveyIntent);
                getActivity().finish();
            });
        } else if (surveyQueued != null) {
            // Direct to new survey
            buttonToInitialize.setText(formatText(getString(R.string.ds_survey_new)));
            buttonToInitialize.setOnClickListener(v ->
                    model.setSurveyNotificationNextSurvey(dismissed -> {
                        if (dismissed) {
                            Intent surveyIntent = new Intent(getActivity(), SurveyActivity.class);
                            startActivity(surveyIntent);
                            getActivity().finish();
                        }
                    }));
        } else {
            // No survey queued or ongoing,
            // disable button
            buttonToInitialize.setText(formatText(getString(R.string.ds_survey_complete)));
            buttonToInitialize.setClickable(false);
            buttonToInitialize.setIcon(getContext().getDrawable(R.drawable.ic_circled_checkmark));
            buttonToInitialize.setLayoutDirection(View.LAYOUT_DIRECTION_LTR);
            final int iconPaddingDP = 10;
            final float scale = getResources().getDisplayMetrics().density;
            final int iconPadding = (int) (iconPaddingDP * scale + 0.5f);
            buttonToInitialize.setIconPadding(iconPadding);
        }

    }

    private SpannableString formatText(String text) {
        // _ next after is primary color
        // * next word after is red
        // %csname is replaced by current survey name
        // %fsname is replaced by next survey name

        int underscoreFirstIndex;
        int underscoreLastIndex = 0;
        int starFirstIndex;
        int starLastIndex = 0;
        if (model.getCurrentSurveyNotificationName().getValue() != null)
            text = text.replaceAll("%csname", model.getCurrentSurveyNotificationName().getValue());
        else text = text.replaceAll("%csname", "");
        if (model.getNextQueuedSurveyNotification() != null)
            text = text.replaceAll("%fsname", model.getNextQueuedSurveyNotification().getTitle().getEn());
        else text = text.replaceAll("%fsname", "");

        StringBuilder stringBuilder = new StringBuilder(text);
        SpannableString spannableString;

        starFirstIndex = text.indexOf("*");
        if (starFirstIndex != -1) {
            text = stringBuilder.replace(starFirstIndex, starFirstIndex + 1, "").toString();
            starLastIndex = text.substring(starFirstIndex).indexOf(" ");
            if (starLastIndex == -1) starLastIndex = text.length();
            stringBuilder = new StringBuilder(text);
        }
        underscoreFirstIndex = text.indexOf("_");
        if (underscoreFirstIndex != -1) {
            text = stringBuilder.replace(underscoreFirstIndex, underscoreFirstIndex + 1, "").toString();
            underscoreLastIndex = text.substring(underscoreFirstIndex).indexOf(" ");
            if (underscoreLastIndex == -1) underscoreLastIndex = text.length();
        }
        spannableString = new SpannableString(text);
        if (underscoreFirstIndex != -1) {
            spannableString.setSpan(new ForegroundColorSpan(getContext().getColor(R.color.colorPrimary)), underscoreFirstIndex, underscoreLastIndex, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
            spannableString.setSpan(new StyleSpan(Typeface.BOLD), underscoreFirstIndex, underscoreLastIndex, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
        }
        if (starFirstIndex != -1) {
            spannableString.setSpan(new ForegroundColorSpan(getContext().getColor(R.color.colorError)), starFirstIndex, starLastIndex, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
        }
        return spannableString;
    }

    @Override
    public void showInfoDialog() {
        GritDialogFragment.showSingleBtnInfoDialog(getFragmentManager(), R.string.ds_journal_info_title,
                TextSection.ofText(R.string.ds_journal_info_body, R.style.Caption2, false),
                TextSection.ofBullet(R.string.ds_journal_info_body2, R.style.Caption, false),
                TextSection.ofBullet(R.string.ds_journal_info_body3, R.style.Caption, false),
                TextSection.ofText(R.string.ds_journal_info_body4, R.style.Caption2, false),
                TextSection.ofBullet(R.string.ds_journal_info_body5, R.style.Caption, false),
                TextSection.ofText(R.string.ds_journal_info_body6, R.style.Caption2, false),
                TextSection.ofBullet(R.string.ds_journal_info_body7, R.style.Caption, false)
        );
    }

    @Override
    public void onSelected() {
        // Do nothing on select
    }

    @Override
    public boolean handleError(String message) {
        return false;
    }

    @Override
    public boolean handleError(int messageRes) {
        return false;
    }
}
