package com.ibm.gbs.va.grit.fragment.digitalself;

import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.content.Intent;
import android.graphics.Typeface;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.button.MaterialButton;
import android.support.v4.content.ContextCompat;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.style.ForegroundColorSpan;
import android.text.style.StyleSpan;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;

import com.github.mikephil.charting.charts.LineChart;
import com.github.mikephil.charting.components.Description;
import com.github.mikephil.charting.components.XAxis;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineData;
import com.github.mikephil.charting.data.LineDataSet;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.activity.SurveyActivity;
import com.ibm.gbs.va.grit.adapter.SelectedSpinnerAdapter;
import com.ibm.gbs.va.grit.databinding.TabDigitalJournalBinding;
import com.ibm.gbs.va.grit.fragment.UniqueInfoButtonFragment;
import com.ibm.gbs.va.grit.fragment.ViewPagerItemFragment;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.type.MoodScore;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.util.ComparatorUtil;
import com.ibm.gbs.va.grit.viewmodel.DigitalJournalViewModel;

import org.joda.time.DateTime;
import org.joda.time.LocalDate;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.inject.Inject;

public class DigitalJournalFragment extends ViewPagerItemFragment implements NetworkErrorDelegate, UniqueInfoButtonFragment {

    //truncate to nearest: seconds
    private static final double ENTRY_PRECISION = 1000;
    private static final double DAY_IN_PRECISION = 8.64e+7 / ENTRY_PRECISION;

    @Inject
    ViewModelProvider.Factory viewModelFactory;

    private DigitalJournalViewModel model;
    private TabDigitalJournalBinding binding;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "DigitalJournalFragment";
        assert getActivity() != null;
        binding = TabDigitalJournalBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        model = ViewModelProviders.of(this, viewModelFactory).get(DigitalJournalViewModel.class);

        if (isInitialPage) onSelected();

        binding.setModel(model);

        initializeJournalEvents();
        initializeDigitalJournalGraph(binding.digitalJournalChart);
        initializeContinueSurveyButton(binding.ContinueSurveyButtonDigitalJournal);

        model.getCurrentMoodScores().observe(this, moodScores -> {
            if (moodScores != null) {
                updateMoodScoreGraphEntries(calculateEntries(moodScores));
                calculateLimits(moodScores);
            }
        });

        model.getCurrentSurveyNotificationName().observe(this,
                (newName) -> initializeContinueSurveyButton(binding.ContinueSurveyButtonDigitalJournal));

        return binding.getRoot();
    }

    private void initializeDigitalJournalGraph(LineChart digitalJournalGraph) {
        // It looks like everything has to be set programmatically with this package
        digitalJournalGraph.setDoubleTapToZoomEnabled(false);
        digitalJournalGraph.setDragEnabled(false);
        digitalJournalGraph.setHighlightPerTapEnabled(false);
        digitalJournalGraph.setHighlightPerDragEnabled(false);
        digitalJournalGraph.getLegend().setEnabled(false);
        digitalJournalGraph.getXAxis().setDrawAxisLine(true);
        digitalJournalGraph.getXAxis().setDrawGridLines(false);
        digitalJournalGraph.getXAxis().setDrawLabels(false);
        digitalJournalGraph.getXAxis().setPosition(XAxis.XAxisPosition.BOTTOM);
        digitalJournalGraph.getAxisLeft().setEnabled(true);
        digitalJournalGraph.getAxisRight().setEnabled(false);
        digitalJournalGraph.getAxisLeft().setAxisMinimum(-0.05f);
        digitalJournalGraph.getAxisLeft().setAxisMaximum(1.05f);
        digitalJournalGraph.getAxisLeft().setDrawGridLines(false);
        digitalJournalGraph.getAxisLeft().setDrawLabels(false);
        Description emptyDescription = new Description();
        emptyDescription.setText("");
        digitalJournalGraph.setDescription(emptyDescription);
        digitalJournalGraph.setNoDataText("");
    }

    private void initializeJournalEvents() {
        ArrayList<String> rangeNames = new ArrayList<>();
        for (LocalDate validDate : model.getValidEventRanges().values()) {
            rangeNames.add(validDate.monthOfYear().getAsText());
        }
        rangeNames.add(getString(R.string.ds_activity_tracker_range_overall));
        Collections.sort(rangeNames, ComparatorUtil.getCalendarMonthComparator());
        SelectedSpinnerAdapter<String> validRangesAdapter = new SelectedSpinnerAdapter<>(this.getContext(), R.layout.spinner_ds_range_selected_item, rangeNames);
        validRangesAdapter.setDropDownViewResource(R.layout.spinner_ds_range_dropdown_item);
        validRangesAdapter.setSelectedDropdownLayoutID(R.layout.spinner_ds_range_selected_dropdown_item);
        binding.EventRangeSpinner.setAdapter(validRangesAdapter);

        binding.EventRangeSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parentView, View selectedItemView, int position, long id) {
                validRangesAdapter.setCurrentSelectedIndex(position);
                String selectedRange = (String) parentView.getItemAtPosition(position);
                if (selectedRange.equals(getString(R.string.ds_activity_tracker_range_overall)))
                    model.updateEventData(null);
                else {
                    model.updateEventData(model.getValidEventRanges().get(selectedRange));
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parentView) {
                validRangesAdapter.notifyDataSetChanged();
            }
        });
        binding.EventRangeSpinner.setSelection(3);
    }

    private void initializeContinueSurveyButton(MaterialButton buttonToInitialize) {
        boolean ongoingSurvey = model.isOngoingSurvey();
        Notification surveyQueued = model.getNextQueuedSurveyNotification();
        if (ongoingSurvey) {
            // Direct to current survey
            String text = getString(R.string.ds_survey_continue);
            buttonToInitialize.setText(formatText(text));
            buttonToInitialize.setOnClickListener((View v) -> {
                Intent surveyIntent = new Intent(getActivity(), SurveyActivity.class);
                startActivity(surveyIntent);
                getActivity().finish();
            });
        } else if (surveyQueued != null) {
            // Direct to new survey
            buttonToInitialize.setText(formatText(getString(R.string.ds_survey_new)));
            buttonToInitialize.setOnClickListener(v ->
                    model.setSurveyNotificationNextSurvey(dismissed -> {
                        if (dismissed) {
                            Intent surveyIntent = new Intent(getActivity(), SurveyActivity.class);
                            startActivity(surveyIntent);
                            getActivity().finish();
                        }
                    }));
        } else {
            // No survey queued or ongoing,
            // disable button
            buttonToInitialize.setText(formatText(getString(R.string.ds_survey_complete)));
            buttonToInitialize.setClickable(false);
            buttonToInitialize.setIcon(getContext().getDrawable(R.drawable.ic_circled_checkmark));
            buttonToInitialize.setLayoutDirection(View.LAYOUT_DIRECTION_LTR);
            final int iconPaddingDP = 10;
            final float scale = getResources().getDisplayMetrics().density;
            final int iconPadding = (int) (iconPaddingDP * scale + 0.5f);
            buttonToInitialize.setIconPadding(iconPadding);
        }

    }

    private SpannableString formatText(String text) {
        // _ next after is primary color
        // * next word after is red
        // %csname is replaced by current survey name
        // %fsname is replaced by next survey name

        int underscoreFirstIndex;
        int underscoreLastIndex = 0;
        int starFirstIndex;
        int starLastIndex = 0;
        if (model.getCurrentSurveyNotificationName().getValue() != null)
            text = text.replaceAll("%csname", model.getCurrentSurveyNotificationName().getValue());
        if (model.getNextQueuedSurveyNotification() != null)
            text = text.replaceAll("%fsname", model.getNextQueuedSurveyNotification().getTitle().getEn());

        StringBuilder stringBuilder = new StringBuilder(text);
        SpannableString spannableString;

        starFirstIndex = text.indexOf("*");
        if (starFirstIndex != -1) {
            text = stringBuilder.replace(starFirstIndex, starFirstIndex + 1, "").toString();
            starLastIndex = text.substring(starFirstIndex).indexOf(" ");
            if (starLastIndex == -1) starLastIndex = text.length();
            stringBuilder = new StringBuilder(text);
        }
        underscoreFirstIndex = text.indexOf("_");
        if (underscoreFirstIndex != -1) {
            text = stringBuilder.replace(underscoreFirstIndex, underscoreFirstIndex + 1, "").toString();
            underscoreLastIndex = text.substring(underscoreFirstIndex).indexOf(" ");
            if (underscoreLastIndex == -1) underscoreLastIndex = text.length();
        }
        spannableString = new SpannableString(text);
        if (underscoreFirstIndex != -1) {
            spannableString.setSpan(new ForegroundColorSpan(getContext().getColor(R.color.colorPrimary)), underscoreFirstIndex, underscoreLastIndex, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
            spannableString.setSpan(new StyleSpan(Typeface.BOLD), underscoreFirstIndex, underscoreLastIndex, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
        }
        if (starFirstIndex != -1) {
            spannableString.setSpan(new ForegroundColorSpan(getContext().getColor(R.color.colorError)), starFirstIndex, starLastIndex, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
        }
        return spannableString;
    }

    private List<Entry> calculateEntries(List<MoodScore> moodScores) {
        List<Entry> journalGraphEntries = new ArrayList<>();
        for (MoodScore currentMoodScore : moodScores) {
            Entry entryToAdd = new Entry(
                    (float) ((new DateTime(currentMoodScore.getDate())).getMillis() / ENTRY_PRECISION),
                    (float) currentMoodScore.getScore());
            journalGraphEntries.add(entryToAdd);
        }

        Collections.sort(journalGraphEntries, (o1, o2) -> Float.compare(o1.getX(), o2.getX()));

        return journalGraphEntries;
    }

    private void calculateLimits(List<MoodScore> moodScores) {
        DateTime oldestDateTime = null;
        // find the oldest date for mood score, this will be our start date limit
        for (MoodScore currentMoodScore : moodScores) {
            DateTime dateToCheck = new DateTime(currentMoodScore.getDate());
            if (oldestDateTime == null || dateToCheck.getMillis() < oldestDateTime.getMillis()) {
                oldestDateTime = dateToCheck;
            }
        }
        // return an empty list if there are no mood scores
        if (oldestDateTime == null) return;
        else {
            model.setLowerBoundXAxis(oldestDateTime.monthOfYear().getAsText().substring(0, 3) + " " + oldestDateTime.dayOfMonth().getAsText() + ", " + oldestDateTime.year().getAsText());
            binding.digitalJournalChart.getXAxis().setAxisMinimum((float) ((oldestDateTime.getMillis() / ENTRY_PRECISION) - DAY_IN_PRECISION));
        }

        // add an end date limit
        DateTime currentTime = new DateTime();
        model.setUpperBoundXAxis(currentTime.monthOfYear().getAsText().substring(0, 3) + " " + currentTime.dayOfMonth().getAsText() + ", " + currentTime.year().getAsText());
        binding.digitalJournalChart.getXAxis().setAxisMaximum((float) ((currentTime.getMillis() / ENTRY_PRECISION) + DAY_IN_PRECISION));
    }

    private void updateMoodScoreGraphEntries(List<Entry> dataPoints) {
        if (dataPoints.size() > 1) {
            showGraph(true);
            LineDataSet moodScoreData = new LineDataSet(dataPoints, getString(R.string.ds_journal_graph_mood_legend));
            moodScoreData.setDrawCircles(false);
            moodScoreData.setCircleColor(R.color.colorPrimaryDark);
            moodScoreData.setCircleHoleColor(R.color.colorPrimaryDark);
            moodScoreData.setColor(R.color.colorPrimaryDark);
            moodScoreData.setDrawFilled(true);
            moodScoreData.setFillDrawable(ContextCompat.getDrawable(getContext(), R.drawable.dj_graph_fill_gradient));
            if (dataPoints.size() > 2) moodScoreData.setMode(LineDataSet.Mode.CUBIC_BEZIER);
            LineData lineData = new LineData(moodScoreData);
            lineData.setDrawValues(false);
            binding.digitalJournalChart.setData(lineData);
        } else {
            showGraph(false);
        }
        binding.digitalJournalChart.invalidate();
    }


    private void showGraph(boolean isShown) {
        if (isShown) {
            binding.digitalJournalChart.setVisibility(View.VISIBLE);
            binding.ChartLowerBoundText.setVisibility(View.VISIBLE);
            binding.ChartUpperBoundText.setVisibility(View.VISIBLE);
            binding.digitalJournalHappyScale.setVisibility(View.VISIBLE);
            binding.digitalJournalUnHappyScale.setVisibility(View.VISIBLE);
            binding.ChartGoneText.setVisibility(View.INVISIBLE);
        } else {
            binding.digitalJournalChart.setVisibility(View.INVISIBLE);
            binding.ChartLowerBoundText.setVisibility(View.INVISIBLE);
            binding.ChartUpperBoundText.setVisibility(View.INVISIBLE);
            binding.digitalJournalHappyScale.setVisibility(View.INVISIBLE);
            binding.digitalJournalUnHappyScale.setVisibility(View.INVISIBLE);
            binding.ChartGoneText.setVisibility(View.VISIBLE);
        }
    }


    @Override
    public void showInfoDialog() {
        GritDialogFragment.showSingleBtnInfoDialog(getFragmentManager(), R.string.ds_journal_info_title,
                TextSection.ofText(R.string.ds_journal_info_body, R.style.Caption2, false),
                TextSection.ofBullet(R.string.ds_journal_info_body2, R.style.Caption, false),
                TextSection.ofBullet(R.string.ds_journal_info_body3, R.style.Caption, false),
                TextSection.ofText(R.string.ds_journal_info_body4, R.style.Caption2, false),
                TextSection.ofBullet(R.string.ds_journal_info_body5, R.style.Caption, false),
                TextSection.ofText(R.string.ds_journal_info_body6, R.style.Caption2, false),
                TextSection.ofBullet(R.string.ds_journal_info_body7, R.style.Caption, false)
        );
    }

    @Override
    public void onSelected() {
        if (model.isFirstTimeInfo()) {
            showInfoDialog();
            model.setFirstTimeInfo(false);
        }
    }

    @Override
    public boolean handleError(String message) {
        //MessageHandler.handleError(getActivity(), message, updatingJobs);
        return true;
    }

    @Override
    public boolean handleError(int messageRes) {
        return handleError(getString(messageRes));
    }
}
