package com.ibm.gbs.va.grit.fragment.digitalself;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentStatePagerAdapter;
import android.support.v4.view.ViewPager;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.activity.DashboardActivity;
import com.ibm.gbs.va.grit.databinding.FragmentDigitalselfBinding;
import com.ibm.gbs.va.grit.fragment.UniqueInfoButtonFragment;
import com.ibm.gbs.va.grit.fragment.ViewPagerItemFragment;
import com.ibm.gbs.va.grit.util.RemoteLog;

import java.lang.ref.WeakReference;
import java.util.Arrays;
import java.util.List;

import dagger.android.support.DaggerFragment;

public class DigitalSelfFragment extends DaggerFragment implements DashboardActivity.OnBackPressedListener {

    private final static String TAG = "DigitalSelfFragment";


    private FragmentDigitalselfBinding binding;
    private final List<WeakReference<Fragment>> tabFragments = Arrays.asList(null, null, null);
    private int startingFragmentIndex;
    private int currentFragmentIndex;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        binding = FragmentDigitalselfBinding.inflate(inflater, container, false);
        FragmentStatePagerAdapter adapter = new FragmentStatePagerAdapter(getChildFragmentManager()) {
            @Override
            public Fragment getItem(int index) {
                switch (index) {
                    case 0:
                        return new DigitalJournalFragment();
                    case 1:
                        return new BadgeListFragment();
                    default:
                        RemoteLog.w(TAG, "could not find a layout to display");
                        return new Fragment();
                }
            }

            @NonNull
            @Override
            public Object instantiateItem(@NonNull ViewGroup container, int position) {
                Fragment createdFragment = (Fragment) super.instantiateItem(container, position);
                tabFragments.set(position, new WeakReference<>(createdFragment));
                if (position == startingFragmentIndex) {
                    if (tabFragments.get(binding.viewPager.getCurrentItem()) != null && tabFragments.get(binding.viewPager.getCurrentItem()).get() != null) {
                        ((ViewPagerItemFragment) tabFragments.get(binding.viewPager.getCurrentItem()).get()).setIsInitialPage();
                    }
                    startingFragmentIndex = -1;
                }
                return createdFragment;
            }

            @Nullable
            @Override
            public CharSequence getPageTitle(int index) {
                return getResources().getStringArray(R.array.digital_self_tabs)[index];
            }

            @Override
            public int getCount() {
                return 2;
            }
        };
        binding.viewPager.setAdapter(adapter);

        binding.viewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageScrolled(int i, float v, int i1) {

            }

            @Override
            public void onPageSelected(int i) {
                currentFragmentIndex = i;
                if (tabFragments.get(i) != null && tabFragments.get(i).get() != null) {
                    ((ViewPagerItemFragment) tabFragments.get(i).get()).onSelected();
                }
            }

            @Override
            public void onPageScrollStateChanged(int i) {

            }
        });

        setHasOptionsMenu(true); // register for calls to onCreateOptionsMenu()
        return binding.getRoot();
    }


    @Override
    public boolean customBackPressedAction() {
        Fragment currentFrag = tabFragments.get(binding.viewPager.getCurrentItem()).get();
        return currentFrag instanceof DashboardActivity.OnBackPressedListener && ((DashboardActivity.OnBackPressedListener) currentFrag).customBackPressedAction();
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        menu.removeItem(R.id.chat);
        inflater.inflate(R.menu.digital_self_items, menu); // add chat to action bar
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.digital_self_info:
                showInfoButton();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private void showInfoButton() {
        if (tabFragments.get(currentFragmentIndex) != null && tabFragments.get(currentFragmentIndex).get() != null) {
            ((UniqueInfoButtonFragment) tabFragments.get(currentFragmentIndex).get()).showInfoDialog();
        }
    }
}
