package com.ibm.gbs.va.grit.fragment.employment;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.animation.ObjectAnimator;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.ColorInt;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.CoordinatorLayout;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.graphics.drawable.DrawableCompat;
import android.support.v4.view.ViewCompat;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.view.inputmethod.InputMethodManager;
import android.widget.ArrayAdapter;
import android.widget.TextView;

import com.adityaarora.liveedgedetection.constants.ScanConstants;
import com.adityaarora.liveedgedetection.util.ScanUtils;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.activity.CameraActivity;
import com.ibm.gbs.va.grit.activity.DashboardActivity;
import com.ibm.gbs.va.grit.activity.ProgressBarDelegate;
import com.ibm.gbs.va.grit.databinding.FragmentEmploymentFilterBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.ValidationRule;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.model.request.UploadResumeRequest.UploadType;
import com.ibm.gbs.va.grit.model.response.UploadResumeInfoResponse;
import com.ibm.gbs.va.grit.model.type.EmploymentInfo;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.rest.NetworkErrorListener;
import com.ibm.gbs.va.grit.util.Consumer;
import com.ibm.gbs.va.grit.util.FileUtil;
import com.ibm.gbs.va.grit.util.GritPermissionHelper;
import com.ibm.gbs.va.grit.util.ImageUtil;
import com.ibm.gbs.va.grit.util.LocalValidationHelper;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.viewmodel.EmploymentViewModel;

import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.List;

import javax.inject.Inject;

public class EmploymentFilterFragment extends AnalyticsFragment implements DashboardActivity.OnBackPressedListener, NetworkErrorDelegate {
    private static final int CAMERA_REQUEST_CODE = 101;
    private static final int PICKFILE_REQUEST_CODE = 102;
    private static final String[] MIME_TYPES = new String[]{
            "application/msword",
            "application/vnd.openxmlformats-officedocument.wordprocessingml.document", // .doc & .docx
            "application/pdf"
    };

    @Inject
    ViewModelProvider.Factory viewModelFactory;

    private EmploymentViewModel model;
    private FragmentEmploymentFilterBinding binding;
    private boolean haveEditsBeenMade = false;
    private boolean mosOrResumeNonNull = false;
    private LiveData<UploadResumeInfoResponse> updatingInfo;
    private LiveData<Boolean> uploadedResume;
    private LiveData<String> uploadingComplete;
    private GritPermissionHelper gritPermissionHelper;
    private List<LocalValidationHelper> validationList;
    private LocalValidationHelper locationValidationHelper;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "EmploymentFilterFragment";
        assert getActivity() != null;
        if (getActivity() instanceof NetworkErrorListener) {
            ((NetworkErrorListener) getActivity()).addDelegateToList(this);
        }
        binding = FragmentEmploymentFilterBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        model = ViewModelProviders.of(getActivity(), viewModelFactory).get(EmploymentViewModel.class);
        binding.setModel(model);

        setupFab();

        // spinners
        binding.militarySpinner.setOptions(EmploymentInfo.MilitaryBranch.displayValues());
        binding.rankSpinner.setOptions(EmploymentInfo.Rank.displayValues());
        model.getTargetIndustries().observe(this, targetIndustries -> {
            if (targetIndustries != null)
                binding.industrySpinner.setOptions(targetIndustries.toArray());
        });

        // location Autocomplete TextField
        ArrayAdapter<String> locationDropdownAdapter = new ArrayAdapter<>(getActivity(), R.layout.list_item_dropdown, model.citiesList);
        binding.locationText.setAdapter(locationDropdownAdapter);
        binding.locationText.setOnItemClickListener((adapterView, view, position, id) -> {
            // hide soft keyboard
            InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(binding.locationText.getWindowToken(), 0);
        });

        // Validation
        locationValidationHelper = new LocalValidationHelper(binding.locationLayout, ValidationRule.NO_EMPTY);
        validationList = Arrays.asList(
                locationValidationHelper,
                new LocalValidationHelper(binding.industryLayout, ValidationRule.NO_EMPTY));
        for (LocalValidationHelper item : validationList) {
            item.validateAfterChange();
        }

        binding.mosInput.addTextChangedListener(new TextWatcher() {
            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                setMosOrResumeError(false);
            }

            //region TextWatcher Unused
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void afterTextChanged(Editable s) {
            }
            //endregion
        });

        // Submission
        model.haveEditsBeenMade.observe(this, haveEditsBeenMade -> this.haveEditsBeenMade = haveEditsBeenMade != null && haveEditsBeenMade);
        model.getEmploymentInfo().observe(this, info -> this.mosOrResumeNonNull = info != null && (info.hasResume() || !TextUtils.isEmpty(info.getMos())));
        binding.submitBtn.setOnClickListener(v -> {
            if (!haveEditsBeenMade) {
                MessageHandler.handleWarning(getActivity(), getString(R.string.please_edit_a_field));
            } else if (mosOrResumeNonNull) {
                LocalValidationHelper.validateInputs(validationList,
                        ifValid -> submitJobInfoUpdate());
            } else {
                // set error for MOS/Resume
                setMosOrResumeError(true);
            }
        });

        gritPermissionHelper = new GritPermissionHelper(getActivity(), getString(R.string.permission_justification_resume_upload));

        return binding.getRoot();
    }

    @Override
    public void onResume() {
        super.onResume();
        model.onResume();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        model.discardUnsavedResumeInfo();
    }

    //region MOS or Resume Error
    @ColorInt
    private Integer colorError,
            colorSecondaryText,
            colorActive;

    @SuppressWarnings("ConstantConditions")
    private void setMosOrResumeError(boolean hasError) {
        if (colorError == null) {
            colorError = getContext().getColor(R.color.colorError);
        }
        if (colorSecondaryText == null) {
            colorSecondaryText = getContext().getColor(R.color.colorSecondaryText);
        }
        if (colorActive == null) {
            colorActive = getContext().getColor(R.color.colorActive);
        }

        binding.resumeInfoText.setTextColor(hasError ? colorError : colorSecondaryText);
        if (hasError) {
            setMosError();
        } else {
            binding.mosLayout.setErrorEnabled(false);
        }
        DrawableCompat.setTint(DrawableCompat.wrap(binding.baseFab.getBackground()), hasError ? colorError : colorActive);
        if (hasError) {
            binding.baseFab.show();
        }
    }
    //endregion

    private void submitJobInfoUpdate() {
        updatingInfo = model.updateInfo();
        updatingInfo.observe(this, uploadResumeInfoResponse -> {
            if (uploadResumeInfoResponse != null && !uploadResumeInfoResponse.hasValidationErrors()) {
                handleSubmitJobInfoUpdateSuccess();
            } else if (uploadResumeInfoResponse != null) {
                String err = "";
                if (uploadResumeInfoResponse.hasValidationErrorOfType(UploadResumeInfoResponse.ResumeValidationError.INVALID_LOCATION)) {
                    locationValidationHelper.setError(getString(R.string.city_comma_state));
                    err += getString(R.string.invalid_location);
                }
                if (uploadResumeInfoResponse.hasValidationErrorOfType(UploadResumeInfoResponse.ResumeValidationError.INVALID_MOS)) {
                    setMosError();
                    if (!err.isEmpty()) {
                        err += ".  ";
                    }
                    err += getString(R.string.invalid_mos);
                }
                if (!err.isEmpty()) {
                    MessageHandler.handleError(getActivity(), err, updatingInfo);
                } else {
                    handleSubmitJobInfoUpdateSuccess();
                }
            }
        });
    }

    private void handleSubmitJobInfoUpdateSuccess() {
        getFragmentManager().popBackStack(); // return to Employment Fragment
        MessageHandler.handleSuccess(getActivity(), updatingInfo);
    }

    private void setMosError() {
        binding.mosLayout.setError(getString(R.string.mos_code_helper_text));
    }

    private void beginPickingResumeFile() {
        // upload action
        Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
        intent.addCategory(Intent.CATEGORY_OPENABLE);
        intent.setType("application/pdf");   //Need to set something here for picker to work
        intent.putExtra(Intent.EXTRA_MIME_TYPES, MIME_TYPES);
        Intent i = Intent.createChooser(intent, "Open File:");
        startActivityForResult(i, PICKFILE_REQUEST_CODE);
    }

    private void beginCapturingResumeImage() {
        if (getActivity() == null) {
            //Solves an issue with starting the image capture activity before fragment attaches to activity
            RemoteLog.w(TAG, "Activity is null while trying to capture image.  Closing FAB");
            closeFab();
        } else {
            startActivityForResult(new Intent(getActivity(), CameraActivity.class), CAMERA_REQUEST_CODE);
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        switch (requestCode) {
            case CAMERA_REQUEST_CODE:
                if (resultCode == Activity.RESULT_OK) {
                    if (data != null && data.getExtras() != null) {
                        String filePath = data.getExtras().getString(ScanConstants.SCANNED_RESULT);
                        Bitmap baseBitmap = ScanUtils.decodeBitmapFromFile(filePath, ScanConstants.IMAGE_NAME);
                        String encodedCleanedImg = ImageUtil.cleanForOcr(baseBitmap, true);
                        //Only write image for debugging purposes
                        //FileUtil.writeStringToDownloads(this.gritPermissionHelper, "ResumeImage.dat", encodedCleanedImg);
                        showProgress(true);
                        uploadedResume = model.uploadedResume(UploadType.PHOTO, encodedCleanedImg, null);
                        uploadedResume.observe(this, resumeImgUploaded
                                -> resumeUploadedObserver(uploadedResume, resumeImgUploaded, this::promptMorePages));
                    }
                }
                break;
            case PICKFILE_REQUEST_CODE:
                if (resultCode == Activity.RESULT_OK && data != null) {
                    Uri selectedFile = data.getData();
                    String filename = FileUtil.getURIPath(getActivity(), selectedFile);
                    String extension = filename.substring(filename.lastIndexOf(".") + 1);
                    RemoteLog.d(TAG, "filename: " + filename);
                    RemoteLog.d(TAG, "extension: " + extension);
                    UploadType uploadType = extension.toUpperCase().equals(UploadType.PDF.value()) ?
                            UploadType.PDF :
                            UploadType.WORD;
                    try {
                        InputStream in = getActivity().getContentResolver().openInputStream(selectedFile);
                        String encodedFile = FileUtil.encodeFile(in);
                        showProgress(true);
                        uploadedResume = model.uploadedResume(uploadType, encodedFile, filename);
                        uploadedResume.observe(this, resumeImgUploaded ->
                                resumeUploadedObserver(uploadedResume, resumeImgUploaded, onUpload -> postResumeUploadJobInfoUpdate()));

                        in.close();

                    } catch (IOException ioe) {
                        RemoteLog.e(TAG, "ERROR reading resume file: " + ioe.getMessage(), ioe);
                    }
                }
                break;
        }
    }

    private void resumeUploadedObserver(LiveData<Boolean> observable, Boolean uploaded, @Nullable Consumer<Boolean> onUpload) {
        onUpload = onUpload != null ? onUpload : b -> {
        };
        if (uploaded != null) {
            showProgress(false);
            if (uploaded) {
                setMosOrResumeError(false);
                MessageHandler.handleSuccess(getActivity(), observable);
                onUpload.accept(true);
            } else {
                MessageHandler.handleError(getActivity(), getString(R.string.network_error));
                onUpload.accept(false);
            }
        }
    }

    private void promptMorePages(Boolean promptMore) {
        if (promptMore == null || !promptMore) return;
        GritDialogFragment.showTwoBtnInfoDialog(getFragmentManager(),
                R.string.dialog_add_more_pages_title,
                R.string.add_more_pages,
                R.string.submit_resume,
                onOK -> beginCapturingResumeImage(),
                onCancel -> submitResumeImg(),
                TextSection.ofText(R.string.dialog_add_more_pages_body));
    }

    private void submitResumeImg() {
        uploadingComplete = model.uploadComplete();
        uploadingComplete.observe(this, uploadComplete -> {
            if (uploadComplete != null) {
                //show text of resume img
                FileUtil.writeStringToDownloads(this.gritPermissionHelper, "ResumeOCR.txt", uploadComplete);
                GritDialogFragment.showResumePreviewDialog(getFragmentManager(),
                        onRetake -> beginCapturingResumeImage(),
                        TextSection.ofText(R.string.dialog_resume_preview_disclaimer_title, R.style.TextAppearance_AppCompat_Headline, false),
                        TextSection.ofText(R.string.dialog_resume_preview_disclaimer_text, R.style.Body, false),
                        TextSection.ofDynamicText(uploadComplete));
                uploadingComplete.removeObservers(this);
                uploadingComplete = null;
                postResumeUploadJobInfoUpdate();
            }
        });
    }

    private void postResumeUploadJobInfoUpdate() {
        if (mosOrResumeNonNull) {
            // We now need to also update the job info, regardless of changes to other job info
            // As all calls to uploadResume MUST be followed by uploadResumeInfo
            LocalValidationHelper.validateInputsSilently(validationList, ifValid -> submitJobInfoUpdate());
        }
    }

    //region Floating Action Button
    private Animation animMainExpand;
    private Animation animMainCollapse;

    private void setupFab() {
        animMainExpand = AnimationUtils.loadAnimation(getContext(), R.anim.fab_anim_main_expand);
        animMainCollapse = AnimationUtils.loadAnimation(getContext(), R.anim.fab_anim_main_collapse);

        binding.baseFab.setOnClickListener(v -> {
            if (binding.baseFab.isExpanded()) closeFab();
            else expandFab();
        });
        binding.uploadResumeFab.setOnClickListener(resumeFabOnClickListener(then -> beginPickingResumeFile()));
        binding.takePictureFab.setOnClickListener(resumeFabOnClickListener(then -> beginCapturingResumeImage()));

        //region hide fab when scrolling downward
        FloatingActionButton.Behavior fabBehavior = new FloatingActionButton.Behavior() {
            @Override
            public boolean onStartNestedScroll(@NonNull CoordinatorLayout coordinatorLayout, @NonNull FloatingActionButton child, @NonNull View directTargetChild, @NonNull View target, int nestedScrollAxes, int type) {
                if (child.isExpanded()) {
                    closeFab();
                }
                return nestedScrollAxes == ViewCompat.SCROLL_AXIS_VERTICAL || super.onStartNestedScroll(coordinatorLayout, child, directTargetChild, target, nestedScrollAxes, type);
            }

            @Override
            public void onNestedScroll(@NonNull CoordinatorLayout coordinatorLayout, @NonNull FloatingActionButton child, @NonNull View target, int dxConsumed, int dyConsumed, int dxUnconsumed, int dyUnconsumed, int type) {
                super.onNestedScroll(coordinatorLayout, child, target, dxConsumed, dyConsumed, dxUnconsumed, dyUnconsumed, type);
                if (dyConsumed > 0 && child.isOrWillBeShown()) {
                    child.hide(new FloatingActionButton.OnVisibilityChangedListener() {
                        @SuppressLint("RestrictedApi")
                        @Override
                        public void onHidden(FloatingActionButton fab) {
                            super.onHidden(fab);
                            fab.setVisibility(View.INVISIBLE);
                        }
                    });
                } else if (dyConsumed < 0 && child.isOrWillBeHidden()) {
                    child.show();
                }
            }
        };
        CoordinatorLayout.LayoutParams params = (CoordinatorLayout.LayoutParams) binding.baseFab.getLayoutParams();
        params.setBehavior(fabBehavior);
        //endregion
    }

    private View.OnClickListener resumeFabOnClickListener(Consumer<Void> action) {
        return onClick -> {
            closeFab();
            EmploymentInfo info = model.getEmploymentInfo().getValue();
            if (info != null && info.hasResume()) {
                GritDialogFragment.showTwoBtnWarningDialog(getFragmentManager(),
                        R.string.dialog_replace_resume_title,
                        onOk -> action.accept(null),
                        TextSection.ofText(R.string.dialog_replace_resume_body));

            } else action.accept(null);
        };
    }

    private void expandFab() {
        binding.baseFab.setExpanded(true);

        binding.baseFab.startAnimation(animMainExpand);
        binding.uploadResumeFab.show();
        binding.takePictureFab.show();

        revealFabLabel(binding.uploadResumeLabel);
        revealFabLabel(binding.takePictureLabel);
    }

    private void closeFab() {
        binding.baseFab.setExpanded(false);

        binding.baseFab.startAnimation(animMainCollapse);
        binding.uploadResumeFab.hide();
        binding.takePictureFab.hide();

        hideFabLabel(binding.uploadResumeLabel);
        hideFabLabel(binding.takePictureLabel);
    }

    private void hideFabLabel(TextView view) {
        ObjectAnimator anim = ObjectAnimator.ofFloat(view, "alpha", 1, 0).setDuration(200);
        anim.setAutoCancel(true);
        anim.addListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationCancel(Animator animation) {
                super.onAnimationCancel(animation);
                view.setVisibility(View.GONE);
            }

            @Override
            public void onAnimationEnd(Animator animation) {
                super.onAnimationEnd(animation);
                view.setVisibility(View.GONE);
            }
        });
        anim.start();
    }

    private void revealFabLabel(TextView view) {
        ObjectAnimator anim = ObjectAnimator.ofFloat(view, "alpha", 0, 1).setDuration(200);
        anim.setAutoCancel(true);
        anim.setStartDelay(500);
        anim.addListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationStart(Animator animation) {
                view.setVisibility(View.VISIBLE);
            }
        });
        anim.start();
    }
    //endregion

    //region Interfaces
    @Override
    public boolean customBackPressedAction() {
        if (binding.baseFab.isExpanded()) {
            closeFab();
            return true;
        }
        return false;
    }

    @Override
    public boolean handleError(String message) {
        MessageHandler.removeObservable(getActivity(), updatingInfo, uploadedResume, uploadingComplete);
        showProgress(false);
        return false;
    }

    @Override
    public boolean handleError(int messageRes) {
        return handleError(getString(messageRes));
    }

    private synchronized void showProgress(final boolean show) {
        if (getActivity() instanceof ProgressBarDelegate) {
            ProgressBarDelegate delegate = (ProgressBarDelegate) getActivity();
            if (show) delegate.showProgressBar();
            else delegate.hideProgressBar();
        }
    }
    //endregion
}
