package com.ibm.gbs.va.grit.fragment.employment;

import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.view.ViewPager;
import android.support.v7.app.AppCompatActivity;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.activity.DashboardActivity;
import com.ibm.gbs.va.grit.activity.ProgressBarDelegate;
import com.ibm.gbs.va.grit.databinding.FragmentEmploymentBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.UniqueInfoButtonFragment;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.viewmodel.EmploymentViewModel;
import com.ibm.gbs.va.grit.viewmodel.EmploymentViewModel.Tab;

import java.lang.ref.WeakReference;
import java.util.Arrays;
import java.util.List;

import javax.inject.Inject;

import dagger.android.support.DaggerFragment;

public class EmploymentFragment extends DaggerFragment implements DashboardActivity.OnBackPressedListener {
    private final static String TAG = "EmploymentFragment";
    private final static String GO_TO_JOB_FILTER_ARG = "go_to_job_filter_tab";
    private EmploymentViewModel model;
    private ViewPager.OnPageChangeListener onPageChangeListener = new ViewPager.OnPageChangeListener() {
        @Override
        public void onPageSelected(int i) {
            RemoteLog.d(TAG, "We are in onPageSelected for: " + i);
            model.setCurrentTab(Tab.fromValue(i));
            model.checkDataAvailable();
        }

        //region OnPageChangeListener Unused
        @Override
        public void onPageScrolled(int i, float v, int i1) {
        }

        @Override
        public void onPageScrollStateChanged(int i) {
        }
        //endregion
    };

    public static EmploymentFragment goToJobFilterInstance(@Nullable Fragment frag) {
        if (!(frag instanceof EmploymentFragment)) frag = new EmploymentFragment();

        Bundle args = new Bundle();
        args.putBoolean(GO_TO_JOB_FILTER_ARG, true);
        frag.setArguments(args);
        return (EmploymentFragment) frag;
    }


    private FragmentEmploymentBinding binding;
    private final List<WeakReference<Fragment>> tabFragments = Arrays.asList(null, null, null);
    private FragmentPagerAdapter adapter;

    @Inject
    ViewModelProvider.Factory viewModelFactory;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        assert getActivity() != null;
        binding = FragmentEmploymentBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);
        model = ViewModelProviders.of(getActivity(), viewModelFactory).get(EmploymentViewModel.class);
        model.getEmploymentInfo().observe(this, employmentInfo -> RemoteLog.d(TAG, "observing employmentInfo"));


        initViewPagerAdapter();
        binding.employmentTabViewPager.setAdapter(adapter);

        model.getCurrentTab().observe(this, currentTab -> binding.employmentTabViewPager.setCurrentItem(currentTab.value));

        model.isJobsOrSkillsLoading().observe(this, this::showProgress);

        setHasOptionsMenu(true); // register for calls to onCreateOptionsMenu()
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        Bundle args = getArguments();
        if (args != null && args.getBoolean(GO_TO_JOB_FILTER_ARG, false)) {
            setArguments(null);
            goToJobFilter();
        }
    }

    @Override
    public void onStart() {
        super.onStart();
        binding.employmentTabViewPager.addOnPageChangeListener(onPageChangeListener);
    }

    @Override
    public void onStop() {
        super.onStop();
        binding.employmentTabViewPager.removeOnPageChangeListener(onPageChangeListener);
        showProgress(false);
    }

    @Override
    public boolean customBackPressedAction() {
        Fragment currentFrag = tabFragments.get(binding.employmentTabViewPager.getCurrentItem()).get();
        return currentFrag instanceof DashboardActivity.OnBackPressedListener && ((DashboardActivity.OnBackPressedListener) currentFrag).customBackPressedAction();
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        menu.removeItem(R.id.chat);
        inflater.inflate(R.menu.employment_items, menu); // add chat to action bar
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.employmentInfo:
                showInfoButton();
                return true;
            case R.id.jobFilter: // handle clicking the job filter
                goToJobFilter();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private void goToJobFilter() {
        ((AppCompatActivity) getActivity()).getSupportActionBar().setTitle(getString(R.string.filter));
        getFragmentManager().beginTransaction()
                .setCustomAnimations(R.anim.slide_in_right, R.anim.slide_out_left, android.R.anim.slide_in_left, android.R.anim.slide_out_right)
                .replace(R.id.contentFrame, new EmploymentFilterFragment(), String.valueOf(R.id.jobFilter))
                .addToBackStack(String.valueOf(R.id.jobFilter))
                .commit();
    }

    private void initViewPagerAdapter() {
        adapter = new FragmentPagerAdapter(getChildFragmentManager()) {
            @Override
            public Fragment getItem(int index) {
                switch (Tab.fromValue(index)) {
                    case Jobs:
                        return new JobsTabFragment();
                    case SavedJobs:
                        return new SavedJobsTabFragment();
                    case Skills:
                        return new SkillsTabFragment();
                }
                return new Fragment();
            }

            @NonNull
            @Override
            public Object instantiateItem(@NonNull ViewGroup container, int position) {
                RemoteLog.d(TAG, "creating fragment at position: " + position);
                Fragment createdFragment = (Fragment) super.instantiateItem(container, position);
                tabFragments.set(position, new WeakReference<>(createdFragment));
                return createdFragment;
            }

            @Nullable
            @Override
            public CharSequence getPageTitle(int index) {
                return getResources().getStringArray(R.array.jobs_tabs)[index];
            }

            @Override
            public int getCount() {
                return 3;
            }

            //region Primary Item
            /* workaround to know when a tab is displayed
                ref: https://stackoverflow.com/questions/10024739/how-to-determine-when-fragment-becomes-visible-in-viewpager */
            private int currentPos = -1;

            @Override
            public void setPrimaryItem(@NonNull ViewGroup container, int position, @NonNull Object object) {
                super.setPrimaryItem(container, position, object);
                if (position == currentPos) {
                    return;
                }

                if (object instanceof Fragment) {
                    Fragment frag = (Fragment) object;
                    if (frag.isResumed()) {
                        currentPos = position;
                    }
                }
            }
            //endregion
        };
    }

    private void showInfoButton() {
        int currentIndex = binding.employmentTabViewPager.getCurrentItem();
        Fragment current = tabFragments.get(currentIndex).get();
        if (current instanceof UniqueInfoButtonFragment) {
            ((UniqueInfoButtonFragment) current).showInfoDialog();
        }
    }

    private synchronized void showProgress(final Boolean show) {
        if (getActivity() instanceof ProgressBarDelegate && show != null) {
            ProgressBarDelegate delegate = (ProgressBarDelegate) getActivity();
            if (show) delegate.showProgressBar();
            else delegate.hideProgressBar();
        }
    }
}
