package com.ibm.gbs.va.grit.fragment.employment;

import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.widget.TextViewCompat;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.databinding.FragmentJobBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.model.type.Job;
import com.ibm.gbs.va.grit.viewmodel.EmploymentViewModel;

import javax.inject.Inject;

public class JobFragment extends AnalyticsFragment {
    private static final String ARG_LIST_INDEX = "arg_list_index";

    public interface SaveJobDelegate {
        void saveJob(boolean save, Job job);
    }

    public static JobFragment newInstance(int index) {
        JobFragment frag = new JobFragment();
        Bundle args = new Bundle();
        args.putInt(ARG_LIST_INDEX, index);

        frag.setArguments(args);
        return frag;
    }

    private FragmentJobBinding binding;
    private SaveJobDelegate delegate;
    private Job job;
    private int index;

    @Inject
    ViewModelProvider.Factory viewModelFactory;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "JobFragment";
        assert (getActivity() != null);
        if (getParentFragment() instanceof SaveJobDelegate) {
            delegate = (SaveJobDelegate) getParentFragment();
        }

        Bundle args = getArguments();
        index = args.getInt(ARG_LIST_INDEX);
        EmploymentViewModel model = ViewModelProviders.of(getActivity(), viewModelFactory).get(EmploymentViewModel.class);

        model.currentJobs.observe(this, list -> {
            if (list == null || index > list.size() - 1) return;
            job = list.get(index);
            binding.saveJobBtn.setChecked(!TextUtils.isEmpty(job.getSavedJobId()));
            binding.setData(job);
        });

        binding = FragmentJobBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        TextViewCompat.setCompoundDrawablesRelative(binding.applyButton, null, null, binding.applyButton.getIcon(), null);

        binding.applyButton.setOnClickListener(v -> startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(job.getApplicationURL()))));
        binding.saveJobBtn.setOnClickListener(v -> {
            if (delegate != null)
                delegate.saveJob(binding.saveJobBtn.isChecked(), job);
        });

        return binding.getRoot();
    }

    public int getIndex() {
        return index;
    }
}
