package com.ibm.gbs.va.grit.fragment.employment;

import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentStatePagerAdapter;
import android.support.v4.view.ViewPager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.TabJobsBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.UniqueInfoButtonFragment;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.type.Job;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.viewmodel.EmploymentViewModel;

import javax.inject.Inject;

public class JobsTabFragment extends AnalyticsFragment implements UniqueInfoButtonFragment, JobFragment.SaveJobDelegate {
    @Inject
    ViewModelProvider.Factory viewModelFactory;

    private EmploymentViewModel model;
    private TabJobsBinding binding;

    private FragmentStatePagerAdapter pagerAdapter;
    private int jobsCount = 0;

    private final static String TAG = "JobsTabFragment";

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        assert getActivity() != null;
        binding = TabJobsBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        model = ViewModelProviders.of(getActivity(), viewModelFactory).get(EmploymentViewModel.class);
        if (this.getUserVisibleHint() && model.isJobsFirstTimeInfo()) {
            showInfoDialog();
            model.setJobsFirstTimeInfo(false);
        }

        model.currentJobs.observe(this, jobs -> pagerAdapter.notifyDataSetChanged());

        model.jobSize.observe(this, totalPages -> {
            jobsCount = totalPages != null ? totalPages : 0;
            RemoteLog.d(TAG, "size changed! New size: " + jobsCount);
            pagerAdapter.notifyDataSetChanged();
            if (jobsCount > 0) {
                binding.contentLayout.setVisibility(View.VISIBLE);
                binding.emptyListLayout.setVisibility(View.GONE);
            } else {
                binding.contentLayout.setVisibility(View.GONE);
                binding.emptyListLayout.setVisibility(View.VISIBLE);
            }
        });
        ViewPager viewPager = binding.viewPager;
        viewPager.setPageMargin(20);

        pagerAdapter = new FragmentStatePagerAdapter(getChildFragmentManager()) {
            @Override
            public Fragment getItem(int position) {
                return JobFragment.newInstance(position);
            }

            @Override
            public int getCount() {
                return jobsCount;
            }

            @Override
            public int getItemPosition(@NonNull Object object) {
                JobFragment frag = (JobFragment) object;
                int position = frag.getIndex();
                return position < jobsCount ? position : POSITION_NONE;
            }
        };
        viewPager.setAdapter(pagerAdapter);
        RemoteLog.d(TAG, "viewPager item1: " + viewPager.getCurrentItem());

        return binding.getRoot();
    }

    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        super.setUserVisibleHint(isVisibleToUser);
        if (isVisibleToUser && model != null && model.isJobsFirstTimeInfo()) {
            showInfoDialog();
            model.setJobsFirstTimeInfo(false);
        }
    }

    @Override
    public void showInfoDialog() {
        GritDialogFragment.showJobsInfoDialog(getFragmentManager());
    }

    @Override
    public void saveJob(boolean save, @Nullable Job job) {
        if (job == null) return;
        RemoteLog.d(TAG, (save ? "saving" : "removing") + " job: " + job.getJobId());
        if (save) {
            model.addSavedJob(job, onSuccess ->
                    MessageHandler.handleSuccess(getActivity(), getString(R.string.saved_job_added)));
        } else {
            model.deleteSavedJob(job, onSuccess ->
                    MessageHandler.handleSuccess(getActivity(), getString(R.string.saved_job_removed)));
        }
    }
}
