package com.ibm.gbs.va.grit.fragment.employment;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.recyclerview.extensions.ListAdapter;
import android.support.v7.util.DiffUtil;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.databinding.FragmentRoleBinding;
import com.ibm.gbs.va.grit.databinding.ListItemRoleBinding;
import com.ibm.gbs.va.grit.model.type.Role;
import com.ibm.gbs.va.grit.model.type.SkillDetail;

public class RoleFragment extends Fragment {

    private FragmentRoleBinding binding;
    private SkillDetailAdapter adapter;
    private Role roleModel;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        binding = FragmentRoleBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        adapter = new SkillDetailAdapter();

        binding.skillDetails.setLayoutManager(new LinearLayoutManager(getContext()));
        binding.skillDetails.setAdapter(adapter);

        if (roleModel != null) {
            binding.setData(roleModel);
            adapter.submitList(roleModel.getSkills());
        }

        return binding.getRoot();
    }

    public void setRoleModel(@NonNull Role roleModel) {
        this.roleModel = roleModel;
        if (binding != null) {
            binding.setData(roleModel);
            adapter.submitList(roleModel.getSkills());
        }
    }

    public Role getRoleModel() {
        return roleModel;
    }

    private static final DiffUtil.ItemCallback<SkillDetail> diffutil = new DiffUtil.ItemCallback<SkillDetail>() {
        @Override
        public boolean areItemsTheSame(@NonNull SkillDetail oldSkillDetail, @NonNull SkillDetail newSkillDetail) {
            return TextUtils.equals(oldSkillDetail.toString(), newSkillDetail.toString());
        }

        @Override
        public boolean areContentsTheSame(@NonNull SkillDetail oldSkillDetail, @NonNull SkillDetail newSkillDetail) {
            return TextUtils.equals(oldSkillDetail.getTitle(), newSkillDetail.getTitle());
        }
    };

    class SkillDetailAdapter extends ListAdapter<SkillDetail, SkillDetailViewHolder> {
        SkillDetailAdapter() {
            super(diffutil);
        }

        @NonNull
        @Override
        public SkillDetailViewHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int type) {
            LayoutInflater inflater = LayoutInflater.from(getContext());
            ListItemRoleBinding binding = ListItemRoleBinding.inflate(inflater, viewGroup, false);
            return new SkillDetailViewHolder(binding);
        }

        @Override
        public void onBindViewHolder(@NonNull SkillDetailViewHolder skillDetailViewHolder, int position) {
            skillDetailViewHolder.bind(getItem(position));
        }
    }

    class SkillDetailViewHolder extends RecyclerView.ViewHolder {
        private final ListItemRoleBinding binding;

        SkillDetailViewHolder(@NonNull ListItemRoleBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
        }

        public void bind(SkillDetail item) {
            binding.setModel(item);
            binding.hasSkillIcon.setImageDrawable(getContext().getDrawable(item.getSkillIcon()));
        }
    }
}
