package com.ibm.gbs.va.grit.fragment.employment;

import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.Snackbar;
import android.support.v4.widget.TextViewCompat;
import android.support.v7.recyclerview.extensions.ListAdapter;
import android.support.v7.util.DiffUtil;
import android.support.v7.widget.DividerItemDecoration;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.ListItemSavedJobBinding;
import com.ibm.gbs.va.grit.databinding.TabJobsSavedBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.model.type.SavedJob;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.viewmodel.EmploymentViewModel;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

public class SavedJobsTabFragment extends AnalyticsFragment {
    @Inject
    ViewModelProvider.Factory viewModelFactory;

    private EmploymentViewModel model;
    private TabJobsSavedBinding binding;

    private Snackbar snackbar;
    private SavedJobsAdapter adapter;

    private final static String TAG = "SavedJobsTabFragment";

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        assert getActivity() != null;
        binding = TabJobsSavedBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        model = ViewModelProviders.of(getActivity(), viewModelFactory).get(EmploymentViewModel.class);

        adapter = new SavedJobsAdapter();
        binding.savedJobs.setLayoutManager(new LinearLayoutManager(getContext()));
        binding.savedJobs.addItemDecoration(new DividerItemDecoration(getContext(), DividerItemDecoration.VERTICAL));
        binding.savedJobs.setAdapter(adapter);

        model.savedJobs.observe(this, jobs -> {
            List<SavedJob> list = jobs != null ? new ArrayList<>(jobs) : new ArrayList<>();
            adapter.submitList(list);
            if (list.isEmpty()) {
                binding.savedJobs.setVisibility(View.GONE);
                binding.emptyListLayout.setVisibility(View.VISIBLE);
            } else {
                binding.savedJobs.setVisibility(View.VISIBLE);
                binding.emptyListLayout.setVisibility(View.GONE);
            }
        });

        setHasOptionsMenu(true);
        return binding.getRoot();
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        menu.removeItem(R.id.employmentInfo);
    }

    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        super.setUserVisibleHint(isVisibleToUser);
        if (!isVisibleToUser && snackbar != null && snackbar.isShown()) {
            snackbar.dismiss();
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        if (snackbar != null && snackbar.isShown()) {
            snackbar.dismiss();
        }
    }

    private void deleteSavedJob(SavedJob job) {
        RemoteLog.d(TAG, "removing job: " + job.getId());
        model.deleteSavedJob(job, onSuccess ->
                snackbar = MessageHandler.handleSuccessWithUndo(getActivity(),
                        getString(R.string.saved_job_removed),
                        undoAction ->
                                addSavedJob(job)));
    }

    private void addSavedJob(SavedJob job) {
        RemoteLog.d(TAG, "saving job: " + job.getId());
        model.addSavedJob(job, onSuccess ->
                MessageHandler.handleSuccess(getActivity(), getString(R.string.saved_job_added)));
    }

    private static final DiffUtil.ItemCallback<SavedJob> diffUtil = new DiffUtil.ItemCallback<SavedJob>() {
        @Override
        public boolean areItemsTheSame(@NonNull SavedJob oldSavedJob, @NonNull SavedJob newSavedJob) {
            return TextUtils.equals(oldSavedJob.toString(), newSavedJob.toString());
        }

        @Override
        public boolean areContentsTheSame(@NonNull SavedJob oldSavedJob, @NonNull SavedJob newSavedJob) {
            return TextUtils.equals(oldSavedJob.getId(), newSavedJob.getId());
        }
    };

    class SavedJobsAdapter extends ListAdapter<SavedJob, SavedJobViewHolder> {
        SavedJobsAdapter() {
            super(diffUtil);
        }

        @NonNull
        @Override
        public SavedJobViewHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int type) {
            LayoutInflater inflater = LayoutInflater.from(getContext());
            ListItemSavedJobBinding binding = ListItemSavedJobBinding.inflate(inflater, viewGroup, false);
            return new SavedJobViewHolder(binding);
        }

        @Override
        public void onBindViewHolder(@NonNull SavedJobViewHolder savedJobViewHolder, int position) {
            savedJobViewHolder.bind(getItem(position));
        }
    }

    class SavedJobViewHolder extends RecyclerView.ViewHolder {
        private final ListItemSavedJobBinding binding;

        SavedJobViewHolder(ListItemSavedJobBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
            TextViewCompat.setCompoundDrawablesRelative(binding.applyButton, null, null, binding.applyButton.getIcon(), null);
            binding.applyButton.setOnClickListener(v -> startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(binding.getData().getApplicationURL()))));
            binding.saveJobBtn.setOnClickListener(v -> deleteSavedJob(binding.getData()));
        }

        public void bind(SavedJob job) {
            binding.setData(job);
        }
    }
}
