package com.ibm.gbs.va.grit.fragment.employment;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentStatePagerAdapter;
import android.support.v4.view.ViewPager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.activity.ProgressBarDelegate;
import com.ibm.gbs.va.grit.databinding.TabSkillsBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.UniqueInfoButtonFragment;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.model.response.SkillsResponse;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.rest.NetworkErrorListener;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.viewmodel.EmploymentViewModel;

import javax.inject.Inject;

public class SkillsTabFragment extends AnalyticsFragment implements NetworkErrorDelegate, UniqueInfoButtonFragment {
    @Inject
    ViewModelProvider.Factory viewModelFactory;

    private EmploymentViewModel model;
    private TabSkillsBinding binding;

    private FragmentStatePagerAdapter pagerAdapter;
    private int skillsCount = 0;

    private LiveData<Boolean> updatingSkills;

    private final static String TAG = "SkillsTabFragment";

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        assert getActivity() != null;
        if (getActivity() instanceof NetworkErrorListener) {
            RemoteLog.d(TAG, "Adding error delegate from skillstabFragment");
            ((NetworkErrorListener) getActivity()).addDelegateToList(this);
        }
        binding = TabSkillsBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        model = ViewModelProviders.of(getActivity(), viewModelFactory).get(EmploymentViewModel.class);
        if (this.getUserVisibleHint() && model.isSkillsFirstTimeInfo()) {
            showInfoDialog();
            model.setSkillsFirstTimeInfo(false);
        }

        model.skillsSize.observe(this, totalPages -> {
            skillsCount = totalPages != null ? totalPages : 0;
            RemoteLog.d(TAG, "size changed! New size: " + skillsCount);
            pagerAdapter.notifyDataSetChanged();
            if (skillsCount > 0) {
                binding.viewPager.setVisibility(View.VISIBLE);
                binding.emptyListLayout.setVisibility(View.GONE);
            } else {
                binding.viewPager.setVisibility(View.GONE);
                binding.emptyListLayout.setVisibility(View.VISIBLE);
            }
        });

        binding.setModel(model);
        ViewPager viewPager = binding.viewPager;
        viewPager.setPageMargin(20);

        pagerAdapter = new FragmentStatePagerAdapter(getChildFragmentManager()) {
            @Override
            public Fragment getItem(int position) {
                return new RoleFragment();
            }

            @NonNull
            @Override
            public Object instantiateItem(@NonNull ViewGroup container, int position) {
                RoleFragment frag = (RoleFragment) super.instantiateItem(container, position);
                frag.setRoleModel(model.getRoleAt(position));
                return frag;
            }

            @Override
            public int getCount() {
                return skillsCount;
            }

            @Override
            public int getItemPosition(@NonNull Object object) {
                RoleFragment frag = (RoleFragment) object;
                int index = model.getRoles().indexOf(frag.getRoleModel());
                return index >= 0 ? index : POSITION_NONE;
            }
        };
        viewPager.setAdapter(pagerAdapter);

        // spinners
        model.getTargetIndustries().observe(this, targetIndustries -> {
            if (targetIndustries != null)
                binding.industrySpinner.setOptions(targetIndustries.toArray());
        });

        binding.industrySpinner.setOnItemClickListener((av, v, i, l) -> updateSkills());
        binding.experienceSpinner.setOptions(SkillsResponse.getYearsOfExperienceOptions());
        binding.experienceSpinner.setOnItemClickListener((av, v, i, l) -> updateSkills());

        return binding.getRoot();
    }

    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        super.setUserVisibleHint(isVisibleToUser);
        if (isVisibleToUser && binding != null) {
            binding.toolbarLayout.setExpanded(true);
        }
        if (isVisibleToUser && model != null && model.isSkillsFirstTimeInfo()) {
            showInfoDialog();
            model.setSkillsFirstTimeInfo(false);
        }
    }

    private void updateSkills() {
        showProgress(true);
        updatingSkills = model.updateSkills();
        updatingSkills.observe(this, skillsUpdated -> {
            showProgress(false);
            if (skillsUpdated != null) {
                if (skillsUpdated) {
                    RemoteLog.d(TAG, "Skills updated");
                }
            }
        });
    }

    @Override
    public void showInfoDialog() {
        GritDialogFragment.showSingleBtnInfoDialog(getFragmentManager(), R.string.skills_dialog_info_title,
                TextSection.ofText(R.string.skills_dialog_info_body1, R.style.Caption2, false),
                TextSection.ofBullet(R.string.skills_dialog_info_body2, R.style.Caption, false),
                TextSection.ofBullet(R.string.skills_dialog_info_body3, R.style.Caption, false),
                TextSection.ofText(R.string.skills_dialog_info_body4, R.style.Caption2, false),
                TextSection.ofBullet(R.string.skills_dialog_info_body5, R.style.Caption, false),
                TextSection.ofBullet(R.string.skills_dialog_info_body6, R.style.Caption, false)
        );
    }

    @Override
    public boolean handleError(String message) {
        MessageHandler.removeObservable(getActivity(), updatingSkills);
        showProgress(false);
        return false;
    }

    @Override
    public boolean handleError(int messageRes) {
        return handleError(getString(messageRes));
    }

    private synchronized void showProgress(final boolean show) {
        if (getActivity() instanceof ProgressBarDelegate) {
            ProgressBarDelegate delegate = (ProgressBarDelegate) getActivity();
            if (show) delegate.showProgressBar();
            else delegate.hideProgressBar();
        }
    }
}
