package com.ibm.gbs.va.grit.fragment.login;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.activity.AuthenticationActivity;
import com.ibm.gbs.va.grit.databinding.FragmentForgotPasswordBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.dialog.CheckEmailDialogFragment;
import com.ibm.gbs.va.grit.fragment.dialog.ResetPasswordDialogFragment;
import com.ibm.gbs.va.grit.model.ValidationRule;
import com.ibm.gbs.va.grit.model.type.User;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.util.LocalValidationHelper;
import com.ibm.gbs.va.grit.viewmodel.ForgotPasswordViewModel;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

public class ForgotPasswordFragment extends AnalyticsFragment
        implements IAuthenticationFragment {

    private ForgotPasswordViewModel model;

    private LiveData<User> forgotPasswordCall;
    private CheckEmailDialogFragment checkEmailDialog;

    @Inject
    ViewModelProvider.Factory viewModelFactory;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "ForgotPasswordFragment";
        FragmentForgotPasswordBinding binding = FragmentForgotPasswordBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        model = ViewModelProviders.of(this, viewModelFactory).get(ForgotPasswordViewModel.class);
        binding.setModel(model);

        List<LocalValidationHelper> validationHelpers = new ArrayList<>();
        LocalValidationHelper emailValidation = new LocalValidationHelper(binding.emailInputLayout, ValidationRule.EMAIL);
        emailValidation.validateAfterChange();
        validationHelpers.add(emailValidation);

        binding.sendBtn.setOnClickListener(v -> LocalValidationHelper.validateInputs(validationHelpers, l -> onSubmit()));
        binding.cancelBtn.setOnClickListener(v -> getAuthenticationActivity().goToLogin(null));

        return binding.getRoot();
    }

    private void onSubmit() {
        forgotPasswordCall = model.callForgotPassword();
        forgotPasswordCall.observe(this, user -> {
            if (user != null) {
                forgotPasswordCall.removeObservers(this);
                // show check email dialog
                checkEmailDialog = CheckEmailDialogFragment.showDialog(getFragmentManager(),
                        model.email.get(),
                        onNextClick -> {
                            ResetPasswordDialogFragment.showInstance(getFragmentManager());
                            checkEmailDialog = null;
                        },
                        onResendClicked);
            }
        });
    }

    private final View.OnClickListener onResendClicked = v -> {
        forgotPasswordCall = model.callForgotPassword();
        forgotPasswordCall.observe(this, user -> {
            if (user == null) return;
            forgotPasswordCall.removeObservers(this);
            forgotPasswordCall = null;
            MessageHandler.handleMessageOnContainer(checkEmailDialog.getContainer(), getString(R.string.forgot_password_resend_success, model.email.get()));
        });
    };

    @Override
    public AuthenticationActivity getAuthenticationActivity() {
        return (AuthenticationActivity) getActivity();
    }
}
