package com.ibm.gbs.va.grit.fragment.login;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.android.databinding.library.baseAdapters.BR;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.biometrics.BiometricsManager;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.ValidationRule;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.util.LocalValidationHelper;
import com.ibm.gbs.va.grit.util.StringUtil;
import com.ibm.gbs.va.grit.util.UserPrefs;

import javax.inject.Inject;

public class SignUpFragment extends LoginFragment {
    private LocalValidationHelper nameValidationHelper;

    @Inject
    UserPrefs userPrefs;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        super.onCreateView(inflater, container, savedInstanceState);

        //Add Name input to form
        binding.nameInputText.setVisibility(View.VISIBLE);
        binding.nameInputLayout.setVisibility(View.VISIBLE);

        //Disable fingerprint checkbox
        binding.fingerprintCheckbox.setVisibility(View.GONE);

        //Hide login page links
        binding.learnMoreLink.setVisibility(View.GONE);
        binding.learnMoreBtn.setVisibility(View.GONE);
        binding.forgotPasswordLink.setVisibility(View.GONE);

        //Start local validation on name input
        nameValidationHelper = new LocalValidationHelper(binding.nameInputLayout, ValidationRule.NO_EMPTY);
        nameValidationHelper.validateAfterChange();
        validationHelpers.add(0, nameValidationHelper);

        //Update local validation to validate more than isEmpty
        emailValidationHelper.setValidationRule(ValidationRule.EMAIL);
        passwordValidationHelper.setValidationRule(ValidationRule.PASSWORD);

        //Modify login title to say signup
        binding.titleBar.setVariable(BR.titleText, getString(R.string.signup));

        //Modify Login link to point back to login page instead of signup page
        binding.loginLink.setText(R.string.login_link);
        binding.loginLinkBtn.setText(R.string.login);
        binding.loginLinkBtn.setOnClickListener(v -> getAuthenticationActivity().goToLogin(null));

        binding.loginSubmitBtn.setText(R.string.signup);

        return binding.getRoot();
    }

    @Override
    protected void onSubmit(String email, String password) {
        showProgress(true);
        email = StringUtil.convertEmailCase(email);
        loggingIn = userRepo.signUp(binding.nameInputText.getText().toString(), email, password);
        loggingIn.observe(this, userLoggedIn -> {
            if (userLoggedIn != null && userLoggedIn) {
                if (BiometricsManager.isBiometricsEnabled(getContext())) {
                    if (getActivity().hasWindowFocus()) {
                        GritDialogFragment.showTwoBtnInfoDialog(getFragmentManager(), R.string.fingerprint_login,
                                R.string.ok, R.string.cancel, v -> biometricEnrollment(), w -> getAuthenticationActivity().enterApp(),
                                TextSection.ofText(R.string.enroll_in_biomtrics_login));
                    } else {
                        getAuthenticationActivity().enterApp();
                    }
                } else {
                    getAuthenticationActivity().enterApp();
                }
            }
            loggingIn.removeObservers(this);
            showProgress(false);
        });
    }

    @Override
    protected void showBiometricsLoginPrompt() {
        //do not show biometrics login prompt
    }

    @Override
    protected void prePopulateEmail() {
        //do not pre populate email for signup
    }

    @Override
    protected void addNetworkDelegate() {
        //do not add network delegate
    }

    @Override
    public boolean handleError(String message) {
        super.handleError(message);
        nameValidationHelper.setErrorIcon(true);
        return false;
    }

    @Override
    public boolean handleError(int strResource) {
        super.handleError(strResource);
        nameValidationHelper.setErrorIcon(true);
        return false;
    }

}
