package com.ibm.gbs.va.grit.fragment.question;

import android.arch.lifecycle.LifecycleOwner;
import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.Transformations;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.databinding.ViewDataBinding;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.transition.TransitionManager;
import android.support.v7.util.DiffUtil;
import android.support.v7.widget.DividerItemDecoration;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.BR;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.ListItemChoiceImageBinding;
import com.ibm.gbs.va.grit.databinding.ListItemChoiceMultiBinding;
import com.ibm.gbs.va.grit.databinding.ListItemChoiceSingleBinding;
import com.ibm.gbs.va.grit.databinding.SubFragmentSelectBinding;
import com.ibm.gbs.va.grit.databinding.SubFragmentSelectWithOtherBinding;
import com.ibm.gbs.va.grit.databinding.SubFragmentWongBakerBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.model.survey.Answer;
import com.ibm.gbs.va.grit.model.survey.QuestionContent;
import com.ibm.gbs.va.grit.model.survey.SelectOption;
import com.ibm.gbs.va.grit.room.AnswerDao.QuestionAnswerJoin;
import com.ibm.gbs.va.grit.viewmodel.SurveyViewModel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;

public class ListChoiceSubFragment extends AnalyticsFragment {
    private static final String ARG_IS_SINGLE_SELECT = "arg_is_single_select";
    private static final String ARG_WONG_BAKER = "arg_is_wong_baker";
    private static final String ARG_HAS_OTHER_OPTION = "arg_has_other_option";

    //region Constructors
    public static ListChoiceSubFragment newImageInstance(int index) {
        return newInstance(index, true, true, false);
    }

    public static ListChoiceSubFragment newMultiInstance(int index) {
        return newInstance(index, false, false, false);
    }

    public static ListChoiceSubFragment newSingleInstance(int index) {
        return newInstance(index, true, false, false);
    }

    public static ListChoiceSubFragment newMultiOtherInstance(int index) {
        return newInstance(index, false, false, true);
    }

    private static ListChoiceSubFragment newInstance(int index, Boolean isSingleSelect, Boolean usesImages, Boolean hasOtherOption) {
        ListChoiceSubFragment frag = new ListChoiceSubFragment();
        Bundle args = new Bundle();
        args.putBoolean(ARG_WONG_BAKER, usesImages);
        args.putBoolean(ARG_IS_SINGLE_SELECT, isSingleSelect);
        args.putBoolean(ARG_HAS_OTHER_OPTION, hasOtherOption);
        args.putInt(QuestionFragment.ARG_LIST_INDEX, index);

        frag.setArguments(args);
        return frag;
    }
    //endregion

    private SurveyViewModel model;

    @Inject
    ViewModelProvider.Factory viewModelFactory;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "ListChoiceSubFragment";
        assert getActivity() != null;

        Bundle args = getArguments();
        int itemIndex = args.getInt(QuestionFragment.ARG_LIST_INDEX);
        model = ViewModelProviders.of(getActivity(), viewModelFactory).get(SurveyViewModel.class);
        Boolean isSingleSelect = args.getBoolean(ARG_IS_SINGLE_SELECT);
        Boolean hasOtherOption = args.getBoolean(ARG_HAS_OTHER_OPTION);
        Boolean isWongBaker = args.getBoolean(ARG_WONG_BAKER);

        RecyclerView recyclerView;
        ViewDataBinding binding;
        if (isWongBaker) {
            binding = SubFragmentWongBakerBinding.inflate(inflater, container, false);
            recyclerView = ((SubFragmentWongBakerBinding) binding).recyclerView;

        } else if (hasOtherOption) {
            binding = SubFragmentSelectWithOtherBinding.inflate(inflater, container, false);
            recyclerView = ((SubFragmentSelectWithOtherBinding) binding).recyclerView;
            OtherTextEntryViewModel textEntryModel = new OtherTextEntryViewModel(model.getQuestionAnswerJoinAt(itemIndex));
            textEntryModel.showTextField.observe(this, visibility -> {
                if (visibility == null) return;
                TransitionManager.beginDelayedTransition(((SubFragmentSelectWithOtherBinding) binding).rootLayout);
                ((SubFragmentSelectWithOtherBinding) binding).otherEntryGroup.setVisibility(visibility);
                if (visibility == View.VISIBLE)
                    ((SubFragmentSelectWithOtherBinding) binding).textEntry.requestFocus();
            });
            binding.setVariable(BR.otherModel, textEntryModel);
        } else {
            binding = SubFragmentSelectBinding.inflate(inflater, container, false);
            recyclerView = ((SubFragmentSelectBinding) binding).recyclerView;
        }
        binding.setLifecycleOwner(this);

        DiscreteChoiceAdapter adapter = new DiscreteChoiceAdapter(model.getQuestionAnswerJoinAt(itemIndex), isSingleSelect, isWongBaker, this);
        recyclerView.setAdapter(adapter);

        // setup recyclerView
        if (!isWongBaker) {
            DividerItemDecoration itemDecor = new DividerItemDecoration(getActivity(), DividerItemDecoration.VERTICAL);
            recyclerView.addItemDecoration(itemDecor);
        }
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));
        recyclerView.setHasFixedSize(true);

        return binding.getRoot();
    }

    private class DiffCallback extends DiffUtil.Callback {
        private final List<DiscreteChoiceViewModel> oldList;
        private final List<DiscreteChoiceViewModel> newList;

        private DiffCallback(List<DiscreteChoiceViewModel> oldList, List<DiscreteChoiceViewModel> newList) {
            this.oldList = oldList == null ? new ArrayList<>() : oldList;
            this.newList = newList;
        }

        @Override
        public int getOldListSize() {
            return oldList.size();
        }

        @Override
        public int getNewListSize() {
            return newList.size();
        }

        @Override
        public boolean areItemsTheSame(int oldIndex, int newIndex) {
            return oldList.get(oldIndex).isSelected != newList.get(newIndex).isSelected;
        }

        @Override
        public boolean areContentsTheSame(int oldIndex, int newIndex) {
            return oldList.get(oldIndex).option.getAnswerId().equals(newList.get(newIndex).option.getAnswerId());
        }
    }

    public class DiscreteChoiceAdapter extends RecyclerView.Adapter<DiscreteChoiceAdapter.ChoiceHolder> {

        private final boolean isSingleSelect;
        private final boolean isWongBaker;
        private final LifecycleOwner owner;

        private List<DiscreteChoiceViewModel> list;
        private final LiveData<QuestionAnswerJoin> join;

        private static final int MULTI_TYPE = 0;
        private static final int SINGLE_TYPE = 1;
        private static final int WONG_BAKER_TYPE = 2;

        DiscreteChoiceAdapter(LiveData<QuestionAnswerJoin> questionAnswerJoin, Boolean isSingleSelect, Boolean isWongBaker, LifecycleOwner owner) {
            this.isSingleSelect = isSingleSelect;
            this.isWongBaker = isWongBaker;
            this.owner = owner;
            this.join = questionAnswerJoin;

            questionAnswerJoin.observe(owner, join -> {
                // wait until data exists, but only init list once
                if (join != null && list == null) {
                    List<DiscreteChoiceViewModel> newList = new ArrayList<>();
                    for (SelectOption i : join.getQuestion().getContent().getOptions()) {
                        newList.add(new DiscreteChoiceViewModel(i, this.join, isSingleSelect, isWongBaker));
                    }
                    // add 'none of the above' option if applicable
                    if (join.getQuestion().getContent().getNoneOfTheAbove())
                        newList.add(new DiscreteChoiceViewModel(null, this.join, isSingleSelect, isWongBaker));
                    DiffUtil.calculateDiff(new DiffCallback(list, newList), false).dispatchUpdatesTo(DiscreteChoiceAdapter.this);
                    list = newList;
                }
            });
        }

        @NonNull
        @Override
        public ChoiceHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            LayoutInflater inflater = LayoutInflater.from(parent.getContext());

            if (viewType == MULTI_TYPE) {
                return new MultiChoiceHolder(ListItemChoiceMultiBinding.inflate(inflater, parent, false));
            } else if (viewType == SINGLE_TYPE) {
                return new SingleChoiceHolder(ListItemChoiceSingleBinding.inflate(inflater, parent, false));
            } else /* if (viewType == WONG_BAKER_TYPE) */ {
                return new SingleChoiceHolder(ListItemChoiceImageBinding.inflate(inflater, parent, false));
            }
        }

        @Override
        public int getItemViewType(int position) {
            return isWongBaker ? WONG_BAKER_TYPE :
                    isSingleSelect ? SINGLE_TYPE : MULTI_TYPE;
        }

        @Override
        public int getItemCount() {
            return list.size();
        }

        @Override
        public void onBindViewHolder(@NonNull ChoiceHolder choiceHolder, int position) {
            choiceHolder.bind(list.get(position));
        }

        // ////////////////////////////////////////////////////
        // View Holders
        public abstract class ChoiceHolder extends RecyclerView.ViewHolder {
            final ViewDataBinding binding;
            DiscreteChoiceViewModel vhModel;

            ChoiceHolder(@NonNull ViewDataBinding binding) {
                super(binding.getRoot());
                binding.setLifecycleOwner(owner);
                this.binding = binding;
            }

            void bind(DiscreteChoiceViewModel vhModel) {
                this.vhModel = vhModel;
            }

            abstract public class Handlers {
                abstract public void onItemClick(View view);
            }
        }

        public class SingleChoiceHolder extends ChoiceHolder {
            private final Handlers handlers = new Handlers();

            SingleChoiceHolder(@NonNull ViewDataBinding binding) {
                super(binding);
            }

            @Override
            void bind(DiscreteChoiceViewModel vhModel) {
                super.bind(vhModel);
                vhModel.setHandler(handlers);
                this.binding.setVariable(BR.model, vhModel);
            }

            public class Handlers extends ChoiceHolder.Handlers {
                @Override
                public void onItemClick(View view) {
                    Answer answer = join.getValue().getAnswer();
                    answer = (answer != null) ? answer : new Answer(join.getValue().getQuestion());
                    if (answer.setSingularAnswer(vhModel.option.getAnswerId()))
                        model.saveAnswer(answer);
                }
            }
        }

        public class MultiChoiceHolder extends ChoiceHolder {
            private final Handlers handlers = new Handlers();

            MultiChoiceHolder(@NonNull ViewDataBinding binding) {
                super(binding);
            }

            @Override
            void bind(DiscreteChoiceViewModel vhModel) {
                super.bind(vhModel);
                vhModel.setHandler(handlers);
                this.binding.setVariable(BR.model, vhModel);
            }

            public class Handlers extends ChoiceHolder.Handlers {
                @Override
                public void onItemClick(View view) {
                    Answer answer = join.getValue().getAnswer();
                    answer = (answer != null) ? answer : new Answer(join.getValue().getQuestion());
                    // then this option is a 'none of the above'
                    if (vhModel.option == null)
                        answer.setNoneOfTheAbove(!answer.isNoneOfTheAbove());
                    else
                        answer.setMultiAnswer(vhModel.option.getAnswerId());
                    model.saveAnswer(answer);
                }
            }
        }
    }

    // ////////////////////////////////////////////////////
    // View Model
    public class DiscreteChoiceViewModel {
        private DiscreteChoiceAdapter.ChoiceHolder.Handlers handler;
        private final SelectOption option;
        public final LiveData<Boolean> isSelected;
        public final LiveData<Boolean> isEnabled;
        public final Drawable image;

        DiscreteChoiceViewModel(SelectOption option, LiveData<QuestionAnswerJoin> join, boolean isSingleSelect, boolean isWongBaker) {
            this.option = option;
            this.image = isWongBaker ? getContext().getDrawable(emojiMapping.get(option.getAnswerId())) : null;
            this.isSelected = isSingleSelect ? evalSingleSelected(join) : evalMultiSelected(join);
            this.isEnabled = Transformations.map(join, j -> j == null || j.getAnswer() == null || option == null || !j.getAnswer().isNoneOfTheAbove());
        }

        private LiveData<Boolean> evalSingleSelected(LiveData<QuestionAnswerJoin> join) {
            return Transformations.map(join, j -> {
                if (j == null) return false;
                Answer answer = j.getAnswer();
                String selected = answer != null && answer.getAnswerIds() != null ? answer.getAnswerIds().peek() : null;
                return (option.getAnswerId().equals(selected));
            });
        }

        private LiveData<Boolean> evalMultiSelected(LiveData<QuestionAnswerJoin> join) {
            return Transformations.map(join, j -> {
                if (j == null) return false;
                Answer answer = j.getAnswer();
                return option == null ? answer != null && answer.isNoneOfTheAbove() : // then this option is a 'none of the above'
                        answer != null && answer.getAnswerIds() != null && answer.getAnswerIds().contains(option.getAnswerId());
            });
        }

        public SelectOption getOption() {
            return option;
        }

        void setHandler(DiscreteChoiceAdapter.ChoiceHolder.Handlers handler) {
            this.handler = handler;
        }

        public DiscreteChoiceAdapter.ChoiceHolder.Handlers getHandler() {
            return handler;
        }
    }

    private static final Map<String, Integer> emojiMapping = new HashMap<String, Integer>() {{
        put("1", R.drawable.emoji_very_happy);
        put("2", R.drawable.emoji_happy);
        put("3", R.drawable.emoji_neutral);
        put("4", R.drawable.emoji_sad);
        put("5", R.drawable.emoji_very_sad);
        put("6", R.drawable.emoji_angry);
    }};

    public class OtherTextEntryViewModel {
        private final LiveData<Integer> showTextField;
        private final LiveData<String> promptText;
        private final LiveData<Answer> answer;

        OtherTextEntryViewModel(LiveData<QuestionAnswerJoin> join) {
            this.showTextField = Transformations.map(join, j -> {
                if (j == null || j.getAnswer() == null) return View.GONE;
                QuestionContent content = j.getQuestion().getContent();
                for (String answerId : j.getAnswer().getAnswerIds()) {
                    Integer optionIndex = content.getIndexForAnswerId(answerId);
                    if (optionIndex != null && content.getOptions().get(optionIndex).getAskOther())
                        return View.VISIBLE;
                }
                return View.GONE;
            });
            this.promptText = Transformations.map(join, j -> {
                if (j == null || j.getQuestion() == null) return null;
                return j.getQuestion().getContent().getOtherSelectOption().getLabel().getEn();
            });
            this.answer = Transformations.map(join, j -> {
                if (j == null || j.getAnswer() == null) return null;
                return j.getAnswer();
            });
        }

        public LiveData<Integer> getShowTextField() {
            return showTextField;
        }

        public LiveData<String> getPromptText() {
            return promptText;
        }

        public LiveData<Answer> getAnswer() {
            return answer;
        }
    }
}