package com.ibm.gbs.va.grit.fragment.question;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentTransaction;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.FragmentQuestionBinding;
import com.ibm.gbs.va.grit.model.survey.Question;
import com.ibm.gbs.va.grit.room.AnswerDao;
import com.ibm.gbs.va.grit.viewmodel.SurveyViewModel;

import javax.inject.Inject;

public class QuestionFragment extends Fragment {
    static final String ARG_LIST_INDEX = "arg_list_index";

    public static QuestionFragment newInstance(int index) {
        QuestionFragment frag = new QuestionFragment();
        Bundle args = new Bundle();
        args.putInt(ARG_LIST_INDEX, index);

        frag.setArguments(args);
        return frag;
    }

    @Inject
    ViewModelProvider.Factory viewModelFactory;

    private FragmentQuestionBinding binding;
    private LiveData<AnswerDao.QuestionAnswerJoin> data;
    private int itemIndex;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        binding = FragmentQuestionBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);
        assert (getActivity() != null);
        Bundle args = getArguments();
        itemIndex = args.getInt(ARG_LIST_INDEX);

        SurveyViewModel model = ViewModelProviders.of(getActivity(), viewModelFactory).get(SurveyViewModel.class);
        data = model.getQuestionAnswerJoinAt(itemIndex);

        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        // insert answer choice fragment
        FragmentTransaction transaction = getChildFragmentManager().beginTransaction();
        data.observe(this, questionAnswerJoin -> {
            if (questionAnswerJoin != null && binding.getData() == null) {
                binding.setData(questionAnswerJoin.getQuestion());
                transaction.replace(R.id.answerFrame, determineAnswerFrame(questionAnswerJoin.getQuestion())).commit();
            }
        });
    }

    private Fragment determineAnswerFrame(Question q) {
        switch (q.getQuestionType()) {
            case LIKERT:
                return SliderSubFragment.newInstance(itemIndex);
            case SINGLE_SELECT:
                return ListChoiceSubFragment.newSingleInstance(itemIndex);
            case MULTI_SELECT:
                return ListChoiceSubFragment.newMultiInstance(itemIndex);
            case MULTI_SELECT_OTHER:
                return ListChoiceSubFragment.newMultiOtherInstance(itemIndex);
            case WONG_BAKER:
                return ListChoiceSubFragment.newImageInstance(itemIndex);
            case TEXT:
                return TextEntrySubFragment.newInstance(itemIndex);
            case NUMERICAL:
                return TextEntrySubFragment.newInstance(itemIndex, true);
        }
        return new Fragment();
    }
}
