package com.ibm.gbs.va.grit.fragment.question;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.Transformations;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.SeekBar;

import com.ibm.gbs.va.grit.databinding.SubFragmentSliderBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.model.survey.Answer;
import com.ibm.gbs.va.grit.model.survey.Question;
import com.ibm.gbs.va.grit.model.survey.SelectOption;
import com.ibm.gbs.va.grit.room.AnswerDao.QuestionAnswerJoin;
import com.ibm.gbs.va.grit.viewmodel.SurveyViewModel;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.inject.Inject;

public class SliderSubFragment extends AnalyticsFragment {

    private LiveData<QuestionAnswerJoin> join;
    private SurveyViewModel model;

    public static SliderSubFragment newInstance(int index) {
        SliderSubFragment frag = new SliderSubFragment();
        Bundle args = new Bundle();
        args.putInt(QuestionFragment.ARG_LIST_INDEX, index);

        frag.setArguments(args);
        return frag;
    }

    @Inject
    ViewModelProvider.Factory viewModelFactory;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "SliderSubFragment";
        assert getActivity() != null;
        SubFragmentSliderBinding binding = SubFragmentSliderBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        Bundle args = getArguments();
        int itemIndex = args.getInt(QuestionFragment.ARG_LIST_INDEX);
        model = ViewModelProviders.of(getActivity(), viewModelFactory).get(SurveyViewModel.class);

        join = model.getQuestionAnswerJoinAt(itemIndex);
        binding.setSliderModel(new SliderModel(join));

        return binding.getRoot();
    }

    public class SliderModel {
        private final LiveData<Integer> progress;
        private final LiveData<List<String>> optionStrings;
        private final LiveData<List<Boolean>> isBoldList;

        SliderModel(LiveData<QuestionAnswerJoin> join) {
            this.progress = Transformations.map(join, j -> {
                if (j == null) return null;
                Question question = j.getQuestion();
                Answer answer = j.getAnswer();
                if (question != null && answer != null && !answer.getAnswerIds().isEmpty()) {
                    int index = question.getContent().getIndexForAnswerId(answer.getAnswerIds().getFirst());
                    return modIndex(index);
                } else return 2;
            });
            this.optionStrings = Transformations.map(join, j -> {
                if (j == null || j.getQuestion() == null) return null;
                List<String> list = new ArrayList<>();
                for (SelectOption option : j.getQuestion().getContent().getOptions()) {
                    list.add(option.getLabel().getEn());
                }
                return list;
            });
            this.isBoldList = Transformations.map(join, j -> {
                if (j == null || j.getQuestion() == null || j.getAnswer() == null) return null;
                List<Boolean> list = new ArrayList<>();
                List<String> selectedAnswerIds = j.getAnswer().getAnswerIds();
                if (selectedAnswerIds == null || selectedAnswerIds.isEmpty())
                    return Collections.nCopies(5, false);

                String selectedAnswer = selectedAnswerIds.get(0);
                for (SelectOption option : j.getQuestion().getContent().getOptions()) {
                    list.add(TextUtils.equals(selectedAnswer, option.getAnswerId()));
                }
                return list;
            });
        }

        public LiveData<Integer> getProgress() {
            return progress;
        }

        public LiveData<List<String>> getOptionStrings() {
            return optionStrings;
        }

        public LiveData<List<Boolean>> getIsBoldList() {
            return isBoldList;
        }

        public void onProgressChanged(SeekBar s, int index, boolean fromUser) {
            if (fromUser) {
                Answer answer = join.getValue().getAnswer();
                answer = (answer != null) ? answer : new Answer(join.getValue().getQuestion());
                answer.setSingularAnswer(join.getValue().getQuestion().getContent().getOptions().get(modIndex(index)).getAnswerId());
                model.saveAnswer(answer);
            }
        }

        /**
         * Reverse the mapping of the 5 indexes (I.E. 0 -> 4, 1 -> 3)
         */
        private int modIndex(int i) {
            return -i + 4;
        }
    }
}
