package com.ibm.gbs.va.grit.fragment.question;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.Transformations;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.databinding.ViewDataBinding;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.BR;
import com.ibm.gbs.va.grit.databinding.SubFragmentNumericBinding;
import com.ibm.gbs.va.grit.databinding.SubFragmentTextEntryBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.model.survey.Answer;
import com.ibm.gbs.va.grit.room.AnswerDao.QuestionAnswerJoin;
import com.ibm.gbs.va.grit.viewmodel.SurveyViewModel;

import javax.inject.Inject;

public class TextEntrySubFragment extends AnalyticsFragment {
    private static final String ARG_IS_NUMERICAL = "arg_is_numerical";

    public static TextEntrySubFragment newInstance(int index) {
        return newInstance(index, false);
    }

    public static TextEntrySubFragment newInstance(int index, boolean isNumerical) {
        TextEntrySubFragment frag = new TextEntrySubFragment();
        Bundle args = new Bundle();
        args.putInt(QuestionFragment.ARG_LIST_INDEX, index);
        args.putBoolean(ARG_IS_NUMERICAL, isNumerical);

        frag.setArguments(args);
        return frag;
    }

    @Inject
    ViewModelProvider.Factory viewModelFactory;

    private SurveyViewModel model;
    private LiveData<QuestionAnswerJoin> questionModel;
    private boolean isNumerical;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "TextEntrySubFragment";

        Bundle args = getArguments();
        int itemIndex = args.getInt(QuestionFragment.ARG_LIST_INDEX);
        isNumerical = args.getBoolean(ARG_IS_NUMERICAL);
        model = ViewModelProviders.of(getActivity(), viewModelFactory).get(SurveyViewModel.class);
        questionModel = model.getQuestionAnswerJoinAt(itemIndex);

        ViewDataBinding binding = isNumerical ? SubFragmentNumericBinding.inflate(inflater, container, false) :
                SubFragmentTextEntryBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);

        binding.setVariable(BR.model, new TextEntryModel(questionModel));

        return binding.getRoot();
    }

    public class TextEntryModel {
        private final LiveData<Answer> answer;

        TextEntryModel(LiveData<QuestionAnswerJoin> join) {
            this.answer = Transformations.map(join, j -> {
                if (j == null || j.getAnswer() == null) return null;
                return j.getAnswer();
            });
        }

        public LiveData<Answer> getAnswer() {
            return answer;
        }

        public void onTextChanged(CharSequence s, int start, int before, int count) {
            Answer a = answer.getValue();
            if (a == null) a = new Answer(questionModel.getValue().getQuestion());

            if (isNumerical)
                a.setNumericalAnswer(TextUtils.isEmpty(s.toString()) ? null : Integer.valueOf(s.toString()));
            else a.setText(s.toString());
            model.saveAnswer(a);
        }
    }
}
