package com.ibm.gbs.va.grit.fragment.settings;

import android.arch.lifecycle.LiveData;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.app.AppCompatActivity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.activity.DashboardActivity;
import com.ibm.gbs.va.grit.databinding.FragmentSettingsBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.repository.UserRepository;
import com.ibm.gbs.va.grit.util.BadgeDelegate;
import com.ibm.gbs.va.grit.util.Consumer;
import com.ibm.gbs.va.grit.util.RemoteLog;

import java.util.List;

import javax.inject.Inject;

public class SettingsFragment extends AnalyticsFragment implements BadgeDelegate {

    @Inject
    UserRepository userRepo;

    private Handler curHandler;

    private FragmentSettingsBinding binding;
    private boolean straightToProfile = false;
    private LiveData<Boolean> deactivatingAccount;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "SettingsFragment";
        assert getActivity() != null;

        binding = FragmentSettingsBinding.inflate(inflater, container, false);
        Handler settingHandler = new Handler();
        binding.setHandler(settingHandler);
        curHandler = settingHandler;

        return binding.getRoot();
    }

    @Override
    public void onStart() {
        super.onStart();
        if (straightToProfile) {
            //simulate going to settings, then immediately going to profile
            straightToProfile = false;
            curHandler.onTextViewClicked(binding.settingsEditProfile);
        }
    }

    public void straightToEditProfile() {
        straightToProfile = true;
    }

    @Override
    public boolean handleBadges(@NonNull List<Badge> badgesToHandle) {
        Consumer<Badge> badgeHandler = Badge.getDefaultBadgeBehaviour(getFragmentManager(), getResources(), null);
        for (Badge curBadge : badgesToHandle) {
            badgeHandler.accept(curBadge);
        }
        return true;
    }

    public class Handler {
        public void onTextViewClicked(View view) {
            TextView textView = (TextView) view;
            Fragment frag = new Fragment();
            switch (textView.getId()) {
                case R.id.settings_editProfile:
                    frag = new EditProfileFragment();
                    break;
                case R.id.settings_socialMedia:
                    frag = new SocialMediaFragment();
                    break;
                case R.id.settings_changePassword:
                    frag = EditProfileFragment.createInstanceForChangePassword();
                    textView = binding.settingsEditProfile; // change textView to Edit Profile
                    break;
                case R.id.settings_privacy:
                    frag = new PrivacySettingsFragment();
                    break;
                case R.id.settings_deactivateAccount:
                    GritDialogFragment.showDeactivateAccountDialog(getFragmentManager(), deactivateAccountBtn -> deactivateAccount());
                    return;
                default:
                    break;
            }
            ((AppCompatActivity) getActivity()).getSupportActionBar().setTitle(textView.getText());
            getFragmentManager().beginTransaction()
                    .setCustomAnimations(R.anim.slide_in_right, R.anim.slide_out_left, android.R.anim.slide_in_left, android.R.anim.slide_out_right)
                    .replace(R.id.contentFrame, frag, String.valueOf(textView.getId()))
                    .addToBackStack(String.valueOf(textView.getId()))
                    .commit();
        }
    }

    private void deactivateAccount() {
        RemoteLog.d(TAG, "Deactivating account...");
        deactivatingAccount = userRepo.deactivateAccount();
        deactivatingAccount.observe(this, accountDeactivated -> {
            if (accountDeactivated != null && accountDeactivated) {
                if (deactivatingAccount != null && deactivatingAccount.hasObservers()) {
                    deactivatingAccount.removeObservers(getActivity());
                }
                ((DashboardActivity) getActivity()).logout(getString(R.string.account_deactivated));
            }
        });
    }
}
