package com.ibm.gbs.va.grit.fragment.settings;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.FragmentSocialMediaBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.repository.SocialMediaRepository;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.rest.NetworkErrorDelegate;
import com.ibm.gbs.va.grit.rest.NetworkErrorListener;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.viewmodel.SocialMediaViewModel;

import javax.inject.Inject;

public class SocialMediaFragment extends AnalyticsFragment implements NetworkErrorDelegate {
    @Inject
    ViewModelProvider.Factory viewModelFactory;

    private SocialMediaViewModel model;
    private FragmentSocialMediaBinding binding;
    private boolean haveEditsBeenMade = false;
    private LiveData<Boolean> updatingTwitterHandle;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "SocialMediaFragment";
        assert getActivity() != null;
        if (getActivity() instanceof NetworkErrorListener) {
            ((NetworkErrorListener) getActivity()).addDelegateToList(this);
        }
        binding = FragmentSocialMediaBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);
        model = ViewModelProviders.of(this, viewModelFactory).get(SocialMediaViewModel.class);
        binding.setModel(model);
        model.haveEditsBeenMade.observe(this, haveEditsBeenMade -> this.haveEditsBeenMade = haveEditsBeenMade != null && haveEditsBeenMade);
        model.setBulletedTwitterCanBody(TextSection.ofBullet(R.string.twitter_can_body, R.style.Caption_Secondary));
        model.setBulletedTwitterCannotBody(
                TextSection.ofBullet(R.string.twitter_cannot_body_0, R.style.Caption_Secondary),
                TextSection.ofBullet(R.string.twitter_cannot_body_1, R.style.Caption_Secondary),
                TextSection.ofBullet(R.string.twitter_cannot_body_2, R.style.Caption_Secondary),
                TextSection.ofBullet(R.string.twitter_cannot_body_3, R.style.Caption_Secondary));
        setInputHandler();
        binding.saveBtn.setOnClickListener(v -> {
            if (!haveEditsBeenMade) {
                MessageHandler.handleWarning(getActivity(), getString(R.string.please_edit_a_field));
            } else {
                submitUpdate();
            }
        });
        binding.removeBtn.setOnClickListener(v -> removeHandle());
        return binding.getRoot();
    }

    private void setInputHandler() {
        binding.twitterHandleInput.setOnFocusChangeListener((v, hasfocus) -> autoFormatTwitterHandle(binding.twitterHandleInput.getText(), true));
        binding.twitterHandleInput.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                binding.twitterHandleLayout.setError(null);
                binding.twitterValidationIcon.setVisibility(View.GONE);
                autoFormatTwitterHandle(s, false);
            }
        });
    }

    private void autoFormatTwitterHandle(Editable text, boolean fromFocus) {
        RemoteLog.d(TAG, "text: " + text);
        if (text.length() > 0 && text.charAt(0) != '@') {
            binding.twitterHandleInput.getText().insert(0, "@");
        } else if (text.length() > 1 && text.charAt(0) == '@' && text.charAt(1) == '@') {
            binding.twitterHandleInput.getText().replace(1, 2, "");
        } else if (text.length() == 0 && fromFocus) {
            binding.twitterHandleInput.getText().insert(0, "@");
        }
    }

    private void submitUpdate() {
        LiveData<SocialMediaRepository.UPDATE_STATE> TriUpdatingTwitterHandle = model.updateTwitterHandle();
        TriUpdatingTwitterHandle.observe(this, twitterHandleUpdated -> {
            if (twitterHandleUpdated != null) {
                switch (TriUpdatingTwitterHandle.getValue()) {
                    case UPDATED_WITH_BADGE:
                        //we are showing the badge dialog, so we don't show the confirmation dialog here
                        binding.twitterHandleInput.clearFocus();
                        TriUpdatingTwitterHandle.removeObservers(this);
                        break;
                    case UPDATED_NO_BADGE:
                        //no badge is shown, so we want to show the confirmation dialog
                        binding.twitterHandleInput.clearFocus();
                        GritDialogFragment.showSuccessDialog(getFragmentManager(), R.string.twitter_save_success_message);
                        TriUpdatingTwitterHandle.removeObservers(this);
                        break;
                    case UPDATED_FAILED:
                        break;
                }
            }
        });
    }

    private void removeHandle() {
        updatingTwitterHandle = model.removeTwitterHandle();
        updatingTwitterHandle.observe(this, twitterHandleUpdated -> {
            if (twitterHandleUpdated != null && twitterHandleUpdated) {
                binding.twitterHandleInput.clearFocus();
                GritDialogFragment.showSuccessDialog(getFragmentManager(), R.string.twitter_remove_success_message);
                updatingTwitterHandle.removeObservers(this);
            }
        });
    }

    @Override
    public boolean handleError(String message) {
        if (message.equals(getString(R.string.twitter_handle_not_valid))) {
            binding.twitterHandleLayout.setError(message);
            binding.twitterValidationIcon.setVisibility(View.VISIBLE);
        } else {
            MessageHandler.handleError(getActivity(), message, updatingTwitterHandle);
        }
        return true;
    }

    @Override
    public boolean handleError(int messageRes) {
        return handleError(getString(messageRes));
    }
}