package com.ibm.gbs.va.grit.fragment.squad;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.telephony.PhoneNumberFormattingTextWatcher;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.databinding.FragmentAddSquadMateBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.ValidationRule;
import com.ibm.gbs.va.grit.model.type.SquadMember;
import com.ibm.gbs.va.grit.util.LocalValidationHelper;

import java.util.ArrayList;
import java.util.List;

public class AddSquadFragment extends AnalyticsFragment {

    public interface AddSquadDelegate {
        void showConfirmContactDialog(SquadMember member);
    }

    public static AddSquadFragment instance(AddSquadDelegate delegate) {
        AddSquadFragment frag = new AddSquadFragment();
        frag.addSquadDelegate = delegate;

        return frag;
    }

    private AddSquadDelegate addSquadDelegate;
    private FragmentAddSquadMateBinding binding;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "AddSquadFragment";
        assert getActivity() != null;
        assert addSquadDelegate != null;

        binding = FragmentAddSquadMateBinding.inflate(inflater, container, false);
        binding.setLifecycleOwner(this);
        LocalValidationHelper nameValidator = new LocalValidationHelper(binding.nameLayout, ValidationRule.NO_EMPTY),
                phoneValidator = new LocalValidationHelper(binding.phoneLayout, ValidationRule.PHONE);
        nameValidator.validateAfterChange();
        phoneValidator.validateAfterChange();

        binding.phoneInputText.addTextChangedListener(new PhoneNumberFormattingTextWatcher("US"));

        List<LocalValidationHelper> validationHelpers = new ArrayList<>();
        validationHelpers.add(nameValidator);
        validationHelpers.add(phoneValidator);
        binding.addBtn.setOnClickListener(v -> LocalValidationHelper.validateInputs(validationHelpers, ifValid -> {
            getFragmentManager().popBackStack();
            String phone = binding.phoneInputText.getText().toString().replaceAll("[^\\d]", ""); // remove non-numerals
            if (phone.length() == 11)
                phone = phone.substring(1);

            addSquadDelegate.showConfirmContactDialog(new SquadMember(
                    binding.nameInputText.getText().toString(),
                    phone,
                    null));
        }));
        binding.cancelBtn.setOnClickListener(v -> getFragmentManager().popBackStack());

        setHasOptionsMenu(true); // register for calls to onCreateOptionsMenu()
        return binding.getRoot();
    }

    //region Info Dialog
    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        menu.removeItem(R.id.chat);
        inflater.inflate(R.menu.squad_items, menu); // add chat to action bar
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.squadInfo:
                showInfoDialog();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private void showInfoDialog() {
        GritDialogFragment.showSquadInfoDialog(getFragmentManager());
    }
    //endregion
}
