package com.ibm.gbs.va.grit.fragment.squad;

import android.Manifest;
import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.animation.ObjectAnimator;
import android.annotation.SuppressLint;
import android.arch.lifecycle.Transformations;
import android.arch.lifecycle.ViewModelProvider;
import android.arch.lifecycle.ViewModelProviders;
import android.content.Intent;
import android.database.Cursor;
import android.databinding.ViewDataBinding;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.provider.ContactsContract;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.CoordinatorLayout;
import android.support.design.widget.FloatingActionButton;
import android.support.transition.TransitionManager;
import android.support.v4.graphics.drawable.RoundedBitmapDrawable;
import android.support.v4.graphics.drawable.RoundedBitmapDrawableFactory;
import android.support.v4.view.ViewCompat;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.recyclerview.extensions.ListAdapter;
import android.support.v7.util.DiffUtil;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.PopupMenu;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.TextView;

import com.ibm.gbs.va.grit.BR;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.activity.DashboardActivity;
import com.ibm.gbs.va.grit.databinding.FragmentSquadBinding;
import com.ibm.gbs.va.grit.databinding.ListItemSquadInfoBinding;
import com.ibm.gbs.va.grit.databinding.ListItemSquadMemberBinding;
import com.ibm.gbs.va.grit.fragment.AnalyticsFragment;
import com.ibm.gbs.va.grit.fragment.UniqueInfoButtonFragment;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.model.type.SquadMember;
import com.ibm.gbs.va.grit.rest.MessageHandler;
import com.ibm.gbs.va.grit.util.Converter;
import com.ibm.gbs.va.grit.util.GritPermissionHelper;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.viewmodel.SquadViewModel;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import static android.app.Activity.RESULT_OK;

public class SquadFragment extends AnalyticsFragment implements DashboardActivity.OnBackPressedListener, UniqueInfoButtonFragment {
    @Inject
    ViewModelProvider.Factory viewModelFactory;

    private FragmentSquadBinding binding;
    private SquadViewModel model;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        TAG = "SquadFragment";
        assert getActivity() != null;

        binding = FragmentSquadBinding.inflate(inflater, container, false);
        model = ViewModelProviders.of(this, viewModelFactory).get(SquadViewModel.class);
        binding.setLifecycleOwner(this);

        if (model.isFirstTimeInfo()) {
            showInfoDialog();
            model.setFirstTimeInfo(false);
        }

        binding.recyclerView.setLayoutManager(new LinearLayoutManager(getActivity()));
        SquadRecyclerAdapter adapter = new SquadRecyclerAdapter();
        Transformations.map(model.getSquad(), list -> {
            if (list == null || list.isEmpty()) {
                showEmptyListLayout(true);
                return null;
            }
            List<SquadCardModel> uiList = new ArrayList<>();
            for (SquadMember member : list) {
                uiList.add(new SquadCardModel(member));
            }
            // hide fab if full
            hideBaseFab(uiList.size() == 5);

            if (uiList.size() < 3 || uiList.size() == 5)
                uiList.add(0, new SquadCardModel(uiList.size()));

            showEmptyListLayout(false);
            return uiList;
        }).observe(this, adapter::submitList);
        binding.recyclerView.setAdapter(adapter);

        setupFab();

        setHasOptionsMenu(true); // register for calls to onCreateOptionsMenu()
        return binding.getRoot();
    }

    private void showConfirmContactDialog(SquadMember member) {
        GritDialogFragment.showTwoBtnInfoDialog(getFragmentManager(),
                R.string.dialog_confirm_contact_title,
                R.string.confirm,
                R.string.cancel,
                onConfirmClicked -> model.addSquad(member),
                null,
                TextSection.ofDynamicText(member.getName(), R.style.Body, true),
                TextSection.ofDynamicText(Converter.phoneFormat("1" + member.getCellPhone()), R.style.Body, true));
    }

    //region Info Dialog
    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        menu.removeItem(R.id.chat);
        inflater.inflate(R.menu.squad_items, menu); // add chat to action bar
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.squadInfo:
                showInfoDialog();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    public void showInfoDialog() {
        GritDialogFragment.showSquadInfoDialog(getFragmentManager());
    }
    //endregion

    //region Manual Add Contact
    private void showManualAdd() {
        if (getActivity() == null || getFragmentManager() == null) return;
        String title = getString(R.string.add_squad_mate_title);
        AddSquadFragment frag = AddSquadFragment.instance(this::showConfirmContactDialog);

        AppCompatActivity activity = ((AppCompatActivity) getActivity());
        if (activity.getSupportActionBar() != null)
            activity.getSupportActionBar().setTitle(title);
        getFragmentManager().beginTransaction()
                .setCustomAnimations(R.anim.slide_in_right, R.anim.slide_out_left, android.R.anim.slide_in_left, android.R.anim.slide_out_right)
                .replace(R.id.contentFrame, frag, String.valueOf(binding.addManuallyFab.getId()))
                .addToBackStack(String.valueOf(binding.addManuallyFab.getId()))
                .commit();
    }
    //endregion

    //region Add Contact Intent
    private final static int RESULT_PICK_CONTACT = 1;

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == RESULT_PICK_CONTACT && resultCode == RESULT_OK) {
            assert getContext() != null;
            Uri contactUri = data.getData();
            if (contactUri == null) return;
            String[] projection = new String[]{
                    ContactsContract.CommonDataKinds.Phone.NUMBER,
                    ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME,
                    ContactsContract.CommonDataKinds.Phone.PHOTO_THUMBNAIL_URI};
            Cursor cursor = getContext().getContentResolver().query(contactUri, projection,
                    null, null, null);

            if (cursor != null && cursor.moveToFirst()) {
                int numberIndex = cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER);
                int nameIndex = cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME);
                int photoIndex = cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.PHOTO_THUMBNAIL_URI);
                String number = cursor.getString(numberIndex).replaceAll("[^\\d]", ""); // remove non-numerals
                String name = cursor.getString(nameIndex);
                String photo = cursor.getString(photoIndex);
                Bitmap bitmap = null;
                if (photo != null) {
                    Uri photoUri = Uri.parse(photo);
                    try {
                        bitmap = MediaStore.Images.Media.getBitmap(getContext().getContentResolver(), photoUri);
                    } catch (IOException e) {
                        RemoteLog.e(TAG, "ERROR getting contact photo: " + e.getMessage(), e);
                    }
                }

                if (number.length() == 11) // remove country code
                    number = number.substring(1);

                if (number.length() == 10) // check formatting
                    showConfirmContactDialog(new SquadMember(name, number, bitmap));
                else
                    MessageHandler.handleMessageOnContainer(binding.coordinatorLayout, getString(R.string.invalid_phone_number));
                cursor.close();
            }
        } else
            super.onActivityResult(requestCode, resultCode, data);
    }
    //endregion

    //region FAB
    private Animation animMainExpand;
    private Animation animMainCollapse;

    private void setupFab() {
        animMainExpand = AnimationUtils.loadAnimation(getContext(), R.anim.fab_anim_main_expand);
        animMainCollapse = AnimationUtils.loadAnimation(getContext(), R.anim.fab_anim_main_collapse);

        binding.baseFab.setOnClickListener(v -> {
            if (binding.baseFab.isExpanded()) closeFab();
            else expandFab();
        });
        binding.addFromContactsFab.setOnClickListener(v -> {
            closeFab();
            GritPermissionHelper gritPermissionHelper = new GritPermissionHelper(getActivity(), getString(R.string.permission_justification_get_contacts));
            gritPermissionHelper.checkPermissions(Manifest.permission.READ_CONTACTS, () -> {
                Intent contactPickerIntent = new Intent(Intent.ACTION_PICK);
                contactPickerIntent.setType(ContactsContract.CommonDataKinds.Phone.CONTENT_TYPE);
                startActivityForResult(contactPickerIntent, RESULT_PICK_CONTACT);
                return null;
            });
        });
        binding.addManuallyFab.setOnClickListener(v -> {
            closeFab();
            showManualAdd();
        });

        // hide fab when scrolling downward
        FloatingActionButton.Behavior fabBehavior = new FloatingActionButton.Behavior() {
            @Override
            public boolean onStartNestedScroll(@NonNull CoordinatorLayout coordinatorLayout, @NonNull FloatingActionButton child, @NonNull View directTargetChild, @NonNull View target, int nestedScrollAxes, int type) {
                if (child.isExpanded()) {
                    closeFab();
                }
                return nestedScrollAxes == ViewCompat.SCROLL_AXIS_VERTICAL || super.onStartNestedScroll(coordinatorLayout, child, directTargetChild, target, nestedScrollAxes, type);
            }

            @Override
            public void onNestedScroll(@NonNull CoordinatorLayout coordinatorLayout, @NonNull FloatingActionButton child, @NonNull View target, int dxConsumed, int dyConsumed, int dxUnconsumed, int dyUnconsumed, int type) {
                super.onNestedScroll(coordinatorLayout, child, target, dxConsumed, dyConsumed, dxUnconsumed, dyUnconsumed, type);
                if (dyConsumed > 0 && child.isOrWillBeShown()) {
                    child.hide(new FloatingActionButton.OnVisibilityChangedListener() {
                        @SuppressLint("RestrictedApi")
                        @Override
                        public void onHidden(FloatingActionButton fab) {
                            super.onHidden(fab);
                            fab.setVisibility(View.INVISIBLE);
                        }
                    });
                } else if (dyConsumed < 0 && child.isOrWillBeHidden()) {
                    child.show();
                }
            }
        };
        CoordinatorLayout.LayoutParams params = (CoordinatorLayout.LayoutParams) binding.baseFab.getLayoutParams();
        params.setBehavior(fabBehavior);
    }

    private void hideBaseFab(boolean hide) {
        if (hide && binding.baseFab.getVisibility() == View.VISIBLE) {
            TransitionManager.beginDelayedTransition(binding.coordinatorLayout);
            binding.baseFab.hide();
        } else if (!hide && binding.baseFab.getVisibility() == View.GONE) {
            TransitionManager.beginDelayedTransition(binding.coordinatorLayout);
            binding.baseFab.show();
        }
    }

    private void expandFab() {
        binding.baseFab.setExpanded(true);

        binding.baseFab.startAnimation(animMainExpand);
        binding.addFromContactsFab.show();
        binding.addManuallyFab.show();

        revealFabLabel(binding.addFromContactsLabel);
        revealFabLabel(binding.addManuallyLabel);
    }

    private void closeFab() {
        binding.baseFab.setExpanded(false);

        binding.baseFab.startAnimation(animMainCollapse);
        binding.addFromContactsFab.hide();
        binding.addManuallyFab.hide();

        hideFabLabel(binding.addFromContactsLabel);
        hideFabLabel(binding.addManuallyLabel);
    }

    private void hideFabLabel(TextView view) {
        ObjectAnimator anim = ObjectAnimator.ofFloat(view, "alpha", 1, 0).setDuration(200);
        anim.setAutoCancel(true);
        anim.addListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationCancel(Animator animation) {
                super.onAnimationCancel(animation);
                view.setVisibility(View.GONE);
            }

            @Override
            public void onAnimationEnd(Animator animation) {
                super.onAnimationEnd(animation);
                view.setVisibility(View.GONE);
            }
        });
        anim.start();
    }

    private void revealFabLabel(TextView view) {
        ObjectAnimator anim = ObjectAnimator.ofFloat(view, "alpha", 0, 1).setDuration(200);
        anim.setAutoCancel(true);
        anim.setStartDelay(500);
        anim.addListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationStart(Animator animation) {
                view.setVisibility(View.VISIBLE);
            }
        });
        anim.start();
    }

    @Override
    public boolean customBackPressedAction() {
        if (binding.baseFab.isExpanded()) {
            closeFab();
            return true;
        }
        return false;
    }
    //endregion

    //region RecyclerView
    private void showEmptyListLayout(boolean show) {
        if (show &&
                (binding.emptyListLayout.getVisibility() == View.GONE || binding.nestedScrollView.getVisibility() == View.VISIBLE)) {
            binding.emptyListLayout.setVisibility(View.VISIBLE);
            binding.nestedScrollView.setVisibility(View.GONE);
        } else if (!show &&
                (binding.emptyListLayout.getVisibility() == View.VISIBLE || binding.nestedScrollView.getVisibility() == View.GONE)) {
            binding.emptyListLayout.setVisibility(View.GONE);
            binding.nestedScrollView.setVisibility(View.VISIBLE);
        }
    }


    class SquadRecyclerAdapter extends ListAdapter<SquadCardModel, SquadViewHolder> {
        private static final int MEMBER_TYPE = 0;
        private static final int INFO_CARD_TYPE = 1;

        SquadRecyclerAdapter() {
            super(new DiffUtil.ItemCallback<SquadCardModel>() {
                @Override
                public boolean areItemsTheSame(@NonNull SquadCardModel oldModel, @NonNull SquadCardModel newModel) {
                    return oldModel.equals(newModel);
                }

                @Override
                public boolean areContentsTheSame(@NonNull SquadCardModel oldModel, @NonNull SquadCardModel newModel) {
                    return TextUtils.equals(oldModel.getKey(), newModel.getKey());
                }
            });
        }

        @NonNull
        @Override
        public SquadViewHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int viewType) {
            SquadViewHolder holder;
            if (viewType == MEMBER_TYPE)
                holder = new SquadViewHolder(ListItemSquadMemberBinding.inflate(getLayoutInflater(), viewGroup, false));
            else
                holder = new SquadViewHolder(ListItemSquadInfoBinding.inflate(getLayoutInflater(), viewGroup, false));
            return holder;
        }

        @Override
        public void onBindViewHolder(@NonNull SquadViewHolder squadViewHolder, int position) {
            squadViewHolder.bindModel(getItem(position));
        }

        @Override
        public int getItemViewType(int position) {
            return getItem(position).getMember() != null ? MEMBER_TYPE : INFO_CARD_TYPE;
        }
    }

    class SquadViewHolder extends RecyclerView.ViewHolder {

        private final ViewDataBinding binding;

        SquadViewHolder(@NonNull ViewDataBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
        }

        void bindModel(SquadCardModel cardModel) {
            if (cardModel.getListCount() != null) {
                binding.setVariable(BR.currentCount, cardModel.getListCount());

            } else {
                SquadMember member = cardModel.getMember();
                ListItemSquadMemberBinding bind = (ListItemSquadMemberBinding) binding;
                bind.setMember(member);

                if (member.getProfileImage() != null) {
                    bind.profileText.setVisibility(View.GONE);
                    RoundedBitmapDrawable roundDrawable = RoundedBitmapDrawableFactory.create(getResources(), member.getProfileImage());
                    roundDrawable.setCircular(true);
                    bind.profileImage.setImageDrawable(roundDrawable);
                } else {
                    bind.profileText.setVisibility(View.VISIBLE);
                    bind.profileText.setText(member.getInitials());
                }

                bind.moreBtn.setOnClickListener(moreBtn -> {
                    if (getActivity() == null) return;
                    PopupMenu popup = new PopupMenu(getActivity(), moreBtn);

                    popup.setOnMenuItemClickListener(menuItem -> {
                        GritDialogFragment.showTwoBtnInfoDialog(getFragmentManager(),
                                R.string.dialog_remove_squad_member_title,
                                R.string.confirm,
                                R.string.cancel,
                                onOkayClicked -> model.removeSquadMember(member),
                                null,
                                TextSection.ofDynamicText(member.getName(), R.style.Body, true),
                                TextSection.ofDynamicText(Converter.phoneFormat(member.getCellPhone()), R.style.Body, true));
                        return true;
                    });
                    popup.inflate(R.menu.squad_member_actions);
                    popup.show();
                });
            }
        }
    }

    /**
     * Joint model for the info card that displays above the squad list and the squad list cards
     */
    class SquadCardModel {
        private final String key;
        private SquadMember member;
        private Integer listCount;

        SquadCardModel(SquadMember member) {
            this.member = member;
            this.key = member.getId();
        }

        SquadCardModel(Integer listCount) {
            this.listCount = listCount;
            this.key = "infoCard";
        }

        String getKey() {
            return key;
        }

        SquadMember getMember() {
            return member;
        }

        Integer getListCount() {
            return listCount;
        }

        @Override
        public boolean equals(@Nullable Object obj) {
            if (obj instanceof SquadCardModel) {
                SquadCardModel other = (SquadCardModel) obj;
                if (member != null && other.member != null) return member.compareTo(other.member);
                if (listCount != null && other.listCount != null)
                    return listCount.equals(other.listCount);
                return false;
            }
            return super.equals(obj);
        }
    }
    //endregion
}
