package com.ibm.gbs.va.grit.model;

import android.content.Context;
import android.content.res.Resources;

import com.ibm.gbs.va.grit.R;

public enum ValidationRule {
    NO_EMPTY(false, R.string.empty_input_err, 0, 0, "", 0),
    PHONE(false, R.string.empty_input_err, 0, 0, "^(\\+?1\\D?)?\\(?\\d{3}\\)?\\D\\d{3}\\D\\d{4}$", R.string.invalid_phone_number),
    EMAIL(false, R.string.empty_input_err, 6, R.string.email_invalid_err, "^[A-Za-z0-9!#$%&'*+/=?^_`{|}~;-]+(\\.[A-Za-z0-9!#$%&'*+/=?^_`{|}~;-]+)*@[A-Za-z0-9.]+(-[A-Za-z0-9.]+)*(\\.[A-Za-z]{2,})+$", R.string.email_invalid_err),
    PASSWORD(false, R.string.empty_input_err, 8, R.string.password_invalid_err, "^(?=.*[A-Za-z].*)(?=.*\\d.*).*$", R.string.password_invalid_err),
    DAY(true, 0, 0, 0, "^(0[1-9]|[12]\\d|3[01])$", R.string.invalid_number),
    MONTH(true, 0, 0, 0, "^(0[1-9]|1[012])$", R.string.invalid_number),
    YEAR(true, 0, 0, 0, "^[1-9]\\d{3}$", R.string.invalid_number);

    private final boolean canBeEmpty;
    private final int emptyErr;
    private final int maxLength;
    private final int maxLengthErr;
    private final int minLength;
    private final int minLengthErr;
    private final String requiredRegex;
    private final int requiredRegexErr;

    ValidationRule(boolean canBeEmpty, int emptyErr, int minLength, int minLengthErr, String requiredRegex, int requireRegexErr) {
        this.canBeEmpty = canBeEmpty;
        this.emptyErr = emptyErr;
        this.maxLength = 0;
        this.maxLengthErr = 0;
        this.minLength = minLength;
        this.minLengthErr = minLengthErr;
        this.requiredRegex = requiredRegex;
        this.requiredRegexErr = requireRegexErr;
    }

    public Boolean canBeEmpty() {
        return this.canBeEmpty;
    }

    public String getEmptyErr(Context context) {
        return getLocalizedString(context, this.emptyErr, "");
    }

    public Boolean hasMaxLength() {
        return this.maxLength > 0;
    }

    public int getMaxLength() {
        return this.maxLength;
    }

    public String getMaxLengthErr(Context context) {
        return getLocalizedString(context, this.maxLengthErr, Integer.toString(this.maxLength));
    }

    public Boolean hasMinLength() {
        return this.minLength > 0;
    }

    public int getMinLength() {
        return this.minLength;
    }

    public String getMinLengthErr(Context context) {
        return getLocalizedString(context, this.minLengthErr, "");
    }

    public Boolean hasRequiredRegex() {
        return !this.requiredRegex.isEmpty();
    }

    public String getRequiredRegex() {
        return this.requiredRegex;
    }

    public String getRequiredRegexErr(Context context) {
        return getLocalizedString(context, this.requiredRegexErr, "");
    }

    private String getLocalizedString(Context context, int strRes, String suffix) {
        Resources res = context.getResources();
        if (0 != strRes) {
            return (res.getString(strRes) + suffix);
        } else {
            return ("");
        }
    }

    @Override
    public String toString() {
        return "ValidationRule{" +
                "maxLength=" + maxLength +
                ", maxLengthErr=" + maxLengthErr +
                ", minLength=" + minLength +
                ", minLengthErr=" + minLengthErr +
                ", requiredRegex='" + requiredRegex + '\'' +
                ", requiredRegexErr=" + requiredRegexErr +
                "} " + super.toString();
    }
}
