package com.ibm.gbs.va.grit.model.chat;

import android.support.annotation.NonNull;

import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ChatFollowup {
    @SerializedName("followupType")
    private FollowupType followupType;
    @SerializedName("selections")
    private List<SelectionOption> selections;

    // GETTERS/SETTERS
    public FollowupType getFollowupType() {
        return followupType;
    }

    public void setFollowupType(FollowupType followupType) {
        this.followupType = followupType;
    }

    public List<SelectionOption> getSelections() {
        return selections;
    }

    public void setSelections(List<SelectionOption> selections) {
        this.selections = selections;
    }

    @NonNull
    @Override
    public String toString() {
        return new Gson().toJson(this);
    }

    // TRANSIENT FIELDS
    private transient SelectionOption selection;

    public SelectionOption getSelection() {
        return selection;
    }

    public void setSelection(SelectionOption selection) {
        this.selection = selection;
    }

    // Initialization

    /**
     * returns a ChatFollowup object intended to be included in a user's response to a followup.
     * Indicates the user's selection.
     *
     * @param option The option the user selected
     * @return The object
     */
    static ChatFollowup withSelection(SelectionOption option) {
        ChatFollowup ret = new ChatFollowup();
        ret.setSelection(option);
        return ret;
    }

    public enum FollowupType {
        @SerializedName("BUTTON")
        BUTTON("BUTTON"),
        @SerializedName("DROPDOWN")
        DROPDOWN("DROPDOWN");

        private final String value;
        private final static Map<String, FollowupType> CONSTANTS = new HashMap<>();

        static {
            for (FollowupType c : values()) {
                CONSTANTS.put(c.value, c);
            }
        }

        FollowupType(String value) {
            this.value = value;
        }

        @NonNull
        @Override
        public String toString() {
            return value();
        }

        String value() {
            return this.value;
        }

        @SuppressWarnings("unused")
        public static FollowupType fromValue(String value) {
            FollowupType constant = CONSTANTS.get(value);
            if (constant == null) {
                throw new IllegalArgumentException(value);
            } else {
                return constant;
            }
        }
    }
}
