package com.ibm.gbs.va.grit.model.chat;

import android.support.annotation.NonNull;

import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;
import com.stfalcon.chatkit.commons.models.IUser;
import com.stfalcon.chatkit.commons.models.MessageContentType;

import java.util.Date;
import java.util.List;

public class ChatMessage implements MessageContentType {
    @SerializedName("conversationToken")
    private String conversationToken;
    @SerializedName("chatResponse")
    private List<Paragraph> chatResponse;
    @SerializedName("map")
    private ChatMap map;
    @SerializedName("followup")
    private ChatFollowup followup;

    // GETTERS/SETTERS
    public String getConversationToken() {
        return conversationToken;
    }

    public void setConversationToken(String conversationToken) {
        this.conversationToken = conversationToken;
    }

    public List<Paragraph> getChatResponse() {
        return chatResponse;
    }

    public void setChatResponse(List<Paragraph> chatResponse) {
        this.chatResponse = chatResponse;
    }

    public ChatMap getMap() {
        return map;
    }

    public void setMap(ChatMap map) {
        this.map = map;
    }

    public ChatFollowup getFollowup() {
        return followup;
    }

    public void setFollowup(ChatFollowup followup) {
        this.followup = followup;
    }

    @NonNull
    @Override
    public String toString() {
        return new Gson().toJson(this);
    }

    // TRANSIENT FIELDS
    transient private Date createdAt = new Date();
    transient private String text;

    @Override
    public String getId() {
        return String.valueOf(this.hashCode());
    }

    @Override
    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    @Override
    public Date getCreatedAt() {
        return createdAt;
    }

    public void setCreatedAt(Date date) {
        createdAt = date;
    }

    private transient static final IUser USER_OBJ = new IUser() {
        @Override
        public String getId() {
            return "user";
        }

        @Override
        public String getName() {
            return "user";
        }

        @Override
        public String getAvatar() {
            return "user";
        }
    };
    private transient static final IUser BOT_OBJ = new IUser() {
        @Override
        public String getId() {
            return "grit";
        }

        @Override
        public String getName() {
            return "grit";
        }

        @Override
        public String getAvatar() {
            return "grit";
        }
    };

    @Override
    public IUser getUser() {
        return text != null ? USER_OBJ : BOT_OBJ;
    }

    // Initialization

    /**
     * Creates a ChatMessage that is a response to a follow up (dropdown / button list)
     *
     * @param option The option the user selected
     * @return The generated message
     */
    public static ChatMessage userFollowup(SelectionOption option) {
        ChatMessage ret = new ChatMessage();
        ret.text = option.getLabel();
        ret.followup = ChatFollowup.withSelection(option);
        return ret;
    }

    /**
     * Creates a general text message response with the text the user inputted
     *
     * @param text The text to send
     * @return The generated message
     */
    public static ChatMessage userMessage(String text) {
        ChatMessage ret = new ChatMessage();
        ret.text = text;
        return ret;
    }

    /**
     * Creates a new Chat message using only the param message's:
     *  <li>chatResponse</li>
     *  <li>createdAt date</li>
     *  <li>conversationToken</li>
     * @param message The message to duplicate from
     * @return The new message
     */
    public static ChatMessage duplicateBasicPropertiesOf(ChatMessage message) {
        ChatMessage ret = new ChatMessage();
        ret.chatResponse = message.chatResponse;
        ret.createdAt = message.createdAt;
        ret.conversationToken = message.conversationToken;
        return ret;
    }
}
