package com.ibm.gbs.va.grit.model.dialog;

import android.support.annotation.ColorRes;
import android.view.View;

import java.util.Arrays;
import java.util.List;

public class BaseDialogModel {

    ///////////////////////////////////
    // Fields
    @ColorRes
    private final int bannerColor;
    private final ButtonConfig topButton;
    private final ButtonConfig bottomButton;
    private final List<TextSection> sections;
    private final boolean scrollable;

    BaseDialogModel(int bannerColor, ButtonConfig topButton, ButtonConfig bottomButton, List<TextSection> sections, boolean scrollable) {
        this.bannerColor = bannerColor;
        this.topButton = topButton;
        this.bottomButton = bottomButton;
        this.sections = sections;
        this.scrollable = scrollable;
    }

    ///////////////////////////////////
    // Getters
    public int getBannerColor() {
        return bannerColor;
    }


    public ButtonConfig getTopButton() {
        return topButton;
    }

    public ButtonConfig getBottomButton() {
        return bottomButton;
    }

    public List<TextSection> getSections() {
        return sections;
    }

    public boolean isScrollable() {
        return scrollable;
    }

    ///////////////////////////////////
    // Builder
    protected static class Builder<T> {

        final Class<T> builderClass;
        @ColorRes
        int bannerColor;
        ButtonConfig topButton;
        ButtonConfig bottomButton;
        List<TextSection> sections;
        boolean scrollable;

        Builder(Class<T> builderClass) {
            this.builderClass = builderClass;
        }

        public void copyFrom(BaseDialogModel model) {
            this.bannerColor = model.bannerColor;
            this.topButton = model.topButton;
            this.bottomButton = model.bottomButton;
            this.sections = model.sections;
            this.scrollable = model.scrollable;
            builderClass.cast(this);
        }

        public T withBannerColor(@ColorRes int bannerColor) {
            this.bannerColor = bannerColor;
            return builderClass.cast(this);
        }

        public T withTopButton(ButtonConfig topButton) {
            this.topButton = topButton;
            return builderClass.cast(this);
        }

        public T withTopOnClick(View.OnClickListener onClickListener) {
            this.topButton = topButton.withOnClick(onClickListener);
            return builderClass.cast(this);
        }

        public T withBottomButton(ButtonConfig bottomButton) {
            this.bottomButton = bottomButton;
            return builderClass.cast(this);
        }

        public T withBottomOnClick(View.OnClickListener onClickListener) {
            this.bottomButton = bottomButton.withOnClick(onClickListener);
            return builderClass.cast(this);
        }

        public T withSections(TextSection... sections) {
            this.sections = Arrays.asList(sections);
            return builderClass.cast(this);
        }

        public T withScrolling() {
            this.scrollable = true;
            return builderClass.cast(this);
        }

    }

}

