package com.ibm.gbs.va.grit.model.dialog;

import android.support.annotation.DrawableRes;
import android.support.annotation.StringRes;

import com.ibm.gbs.va.grit.R;

import java.util.List;

public class DialogModel extends BaseDialogModel {
    public static DialogBuilder builder() {
        return new DialogBuilder();
    }

    ///////////////////////////////////
    // Fields
    private @DrawableRes
    final
    Integer bannerImage;
    private final BannerIcon bannerIcon;
    private @StringRes
    Integer titleRes;
    private String title;

    private DialogModel(int bannerColor, ButtonConfig topButton, ButtonConfig bottomButton, List<TextSection> sections, boolean scrollable, Integer bannerImage, BannerIcon bannerIcon, Integer titleRes) {
        super(bannerColor, topButton, bottomButton, sections, scrollable);
        this.bannerImage = bannerImage;
        this.bannerIcon = bannerIcon;
        this.titleRes = titleRes;
    }

    private DialogModel(int bannerColor, ButtonConfig topButton, ButtonConfig bottomButton, List<TextSection> sections, boolean scrollable, Integer bannerImage, BannerIcon bannerIcon, String title) {
        super(bannerColor, topButton, bottomButton, sections, scrollable);
        this.bannerImage = bannerImage;
        this.bannerIcon = bannerIcon;
        this.title = title;
    }

    ///////////////////////////////////
    // Getters
    public Integer getBannerImage() {
        return bannerImage;
    }

    public BannerIcon getBannerIcon() {
        return bannerIcon;
    }

    public Integer getTitleRes() {
        return titleRes;
    }

    public String getTitle() {
        return title;
    }

    public enum BannerIcon {
        Success(R.drawable.ic_success_filled),
        Info(R.drawable.ic_info_white),
        Save(R.drawable.ic_save),
        Warning(R.drawable.ic_warning);

        public @DrawableRes
        final
        int icon;

        BannerIcon(@DrawableRes int icon) {
            this.icon = icon;
        }
    }

    ///////////////////////////////////
    // Builder
    public static class DialogBuilder extends Builder<DialogBuilder> {

        private @DrawableRes
        Integer bannerImage;
        private BannerIcon bannerIcon;
        private @StringRes
        Integer titleRes;
        private String title;

        DialogBuilder() {
            super(DialogBuilder.class);
        }

        public DialogBuilder copyFrom(DialogModel model) {
            super.copyFrom(model);
            this.bannerImage = model.bannerImage;
            this.bannerIcon = model.bannerIcon;
            this.titleRes = model.titleRes;
            this.title = model.title;
            return this;
        }

        public DialogBuilder withBannerIcon(BannerIcon bannerIcon) {
            this.bannerIcon = bannerIcon;
            return this;
        }

        public DialogBuilder withTitle(@StringRes Integer titleRes) {
            this.titleRes = titleRes;
            this.title = null;
            return this;
        }


        public DialogBuilder withTitle(String title) {
            this.title = title;
            this.titleRes = null;
            return this;
        }

        public DialogModel build() {
            if (title == null)
                return new DialogModel(
                        this.bannerColor,
                        this.topButton,
                        this.bottomButton,
                        this.sections,
                        this.scrollable,
                        this.bannerImage,
                        this.bannerIcon,
                        this.titleRes);
            else
                return new DialogModel(
                        this.bannerColor,
                        this.topButton,
                        this.bottomButton,
                        this.sections,
                        this.scrollable,
                        this.bannerImage,
                        this.bannerIcon,
                        this.title);
        }
    }
}
