package com.ibm.gbs.va.grit.model.survey;

import android.arch.persistence.room.ColumnInfo;
import android.arch.persistence.room.Embedded;
import android.arch.persistence.room.Entity;
import android.arch.persistence.room.ForeignKey;
import android.arch.persistence.room.Index;
import android.arch.persistence.room.PrimaryKey;
import android.support.annotation.NonNull;

import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;
import com.ibm.gbs.va.grit.model.type.MultiLangVariableText;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Entity(indices = @Index("question_set_id"))
public class Question {

    // ////////////////////////////////
    // FIELDS
    @SerializedName("questionId")
    @PrimaryKey
    @NonNull
    @ColumnInfo(name = "id")
    private String questionId;

    @SerializedName("questionText")
    @ColumnInfo(name = "question_text")
    private List<MultiLangVariableText> questionText;

    @SerializedName("variantIndex")
    @ColumnInfo(name = "variant_index")
    private Integer variantIndex;

    @SerializedName("questionType")
    @ColumnInfo(name = "question_type")
    private QuestionType questionType;

    @SerializedName("content")
    @Embedded
    private QuestionContent content;

    // ////////////////////////////////
    // TRANSIENT FIELDS
    @ColumnInfo(name = "question_set_id")
    @ForeignKey(entity = QuestionSet.class, parentColumns = "id", childColumns = "question_set_id", onUpdate = ForeignKey.CASCADE, onDelete = ForeignKey.RESTRICT)
    transient
    private String questionSetId;

    // ////////////////////////////////
    // GETTERS & SETTERS
    @NonNull
    public String getQuestionId() {
        return questionId;
    }

    public void setQuestionId(@NonNull String questionId) {
        this.questionId = questionId;
    }

    public List<MultiLangVariableText> getQuestionText() {
        return questionText;
    }

    public void setQuestionText(List<MultiLangVariableText> questionText) {
        this.questionText = questionText;
    }

    public QuestionType getQuestionType() {
        return questionType;
    }

    public void setQuestionType(QuestionType questionType) {
        this.questionType = questionType;
    }

    public QuestionContent getContent() {
        return content;
    }

    public void setContent(QuestionContent content) {
        this.content = content;
    }

    public String getQuestionSetId() {
        return questionSetId;
    }

    public void setQuestionSetId(String questionSetId) {
        this.questionSetId = questionSetId;
    }

    public Integer getVariantIndex() {
        return variantIndex == null ? 0: variantIndex;
    }

    public void setVariantIndex(Integer variantIndex) {
        this.variantIndex = variantIndex;
    }

    public List<Integer> getVariantIdList() {
        List<Integer> variantIdList = new ArrayList<>();
        for (MultiLangVariableText item : questionText) {
            variantIdList.add(item.getVariantId());
        }
        return variantIdList;
    }

    @NonNull
    @Override
    public String toString() {
        return new Gson().toJson(this);
    }

    public enum QuestionType {
        @SerializedName("LIKERT")
        LIKERT("LIKERT"),
        @SerializedName("SINGLE_SELECT")
        SINGLE_SELECT("SINGLE_SELECT"),
        @SerializedName("MULTI_SELECT")
        MULTI_SELECT("MULTI_SELECT"),
        @SerializedName("MULTI_SELECT_OTHER")
        MULTI_SELECT_OTHER("MULTI_SELECT_OTHER"),
        @SerializedName("NUMERICAL")
        NUMERICAL("NUMERICAL"),
        @SerializedName("TEXT")
        TEXT("TEXT"),
        @SerializedName("WONG_BAKER")
        WONG_BAKER("WONG_BAKER");
        private final String value;

        private final static Map<String, Question.QuestionType> CONSTANTS = new HashMap<>();

        static {
            for (Question.QuestionType c : values()) {
                CONSTANTS.put(c.value, c);
            }
        }

        QuestionType(String value) {
            this.value = value;
        }

        @NonNull
        @Override
        public String toString() {
            return value();
        }

        public String value() {
            return this.value;
        }

        @SuppressWarnings("unused")
        public static Question.QuestionType fromValue(String value) {
            Question.QuestionType constant = CONSTANTS.get(value);
            if (constant == null) {
                throw new IllegalArgumentException(value);
            } else {
                return constant;
            }
        }

    }
}
