package com.ibm.gbs.va.grit.model.survey;

import android.arch.persistence.room.ColumnInfo;
import android.arch.persistence.room.Entity;
import android.arch.persistence.room.Ignore;
import android.arch.persistence.room.PrimaryKey;
import android.support.annotation.NonNull;

import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;
import com.ibm.gbs.va.grit.model.notifications.Notification;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Entity(tableName = "question_set")
public class QuestionSet {

    // ////////////////////////////////
    // FIELDS
    @SerializedName("questionSetId")
    @PrimaryKey
    @NonNull
    @ColumnInfo(name = "id")
    private String questionSetId;

    @SerializedName("questionSetType")
    @ColumnInfo(name = "type")
    private Type questionSetType;

    @SerializedName("version")
    @ColumnInfo(name = "version")
    private int version;

    @SerializedName("questions")
    @Ignore
    private List<Question> questions;

    // ////////////////////////////////
    // GETTERS & SETTERS
    @NonNull
    public String getQuestionSetId() {
        return questionSetId;
    }

    public void setQuestionSetId(@NonNull String questionSetId) {
        this.questionSetId = questionSetId;
    }

    public Type getQuestionSetType() {
        return questionSetType;
    }

    public void setQuestionSetType(Type questionSetType) {
        this.questionSetType = questionSetType;
    }

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    public List<Question> getQuestions() {
        return questions;
    }

    public void setQuestions(List<Question> questions) {
        this.questions = questions;
    }

    @NonNull
    @Override
    public String toString() {
        return new Gson().toJson(this);
    }

    public enum Type {
        @SerializedName("BASELINE")
        BASELINE("BASELINE"),
        @SerializedName("MOOD_FACES")
        MOOD_FACES("MOOD_FACES"),
        @SerializedName("MOOD_TEXT")
        MOOD_TEXT("MOOD_TEXT"),
        @SerializedName("LIFE_CHANGES")
        LIFE_CHANGES("LIFE_CHANGES");
        private final String value;

        private final static Map<String, Type> CONSTANTS = new HashMap<>();

        static {
            for (Type c : values()) {
                CONSTANTS.put(c.value, c);
            }
            CONSTANTS.put(Notification.Type.BASELINE_INITIAL.value(), Type.BASELINE);
        }

        Type(String value) {
            this.value = value;
        }

        @NonNull
        @Override
        public String toString() {
            return value();
        }

        public String value() {
            return this.value;
        }

        public static Type fromValue(String value) {
            Type constant = CONSTANTS.get(value);
            if (constant == null) {
                throw new IllegalArgumentException(value);
            } else {
                return constant;
            }
        }

        public static Type fromValue(Notification notification) {
            return fromValue(notification.getType().value());
        }

    }
}
