package com.ibm.gbs.va.grit.model.type;

import android.arch.persistence.room.ColumnInfo;
import android.arch.persistence.room.Entity;
import android.arch.persistence.room.PrimaryKey;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.support.annotation.DrawableRes;
import android.support.annotation.IdRes;
import android.support.annotation.NonNull;
import android.support.v4.app.FragmentManager;
import android.util.Base64;
import android.view.View;

import com.google.gson.annotations.SerializedName;
import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.fragment.dialog.GritDialogFragment;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.util.Consumer;
import com.ibm.gbs.va.grit.util.RemoteLog;

import java.util.HashMap;
import java.util.Map;

@Entity(tableName = "known_badges")
public class Badge {

    @PrimaryKey
    @NonNull
    @ColumnInfo(name = "badge_name")
    @SerializedName("name")
    private String name;

    @ColumnInfo(name = "add_order")
    @SerializedName("addOrder")
    private int addOrder;

    @ColumnInfo(name = "badge_category")
    @SerializedName("category")
    private String category;

    @ColumnInfo(name = "badge_title")
    @SerializedName("title")
    private MultiLangText title;

    @ColumnInfo(name = "badge_description")
    @SerializedName("description")
    private MultiLangText description;

    @ColumnInfo(name = "badge_congrats")
    @SerializedName("congratulatoryMessage")
    private MultiLangText congratulatoryMessage;

    @ColumnInfo(name = "unawarded_image")
    @SerializedName("unawardedImage")
    private String unawardedImage;

    @ColumnInfo(name = "awarded_image")
    @SerializedName("awardedImage")
    private String awardedImage;

    @ColumnInfo(name = "enlarged_awarded_image")
    @SerializedName("enlargedAwardedImage")
    private String enlargedAwardedImage;

    @ColumnInfo(name = "enlarged_unawarded_image")
    @SerializedName("enlargedUnawardedImage")
    private String enlargedUnawardedImage;

    @NonNull
    public String getName() {
        return name;
    }

    public void setName(@NonNull String name) {
        this.name = name;
    }

    public int getAddOrder() {
        return addOrder;
    }

    public void setAddOrder(int addOrder) {
        this.addOrder = addOrder;
    }

    public String getCategory() {
        return category;
    }

    public void setCategory(String category) {
        this.category = category;
    }

    public MultiLangText getTitle() {
        return title;
    }

    public void setTitle(MultiLangText title) {
        this.title = title;
    }

    public MultiLangText getDescription() {
        return description;
    }

    public void setDescription(MultiLangText description) {
        this.description = description;
    }

    public MultiLangText getCongratulatoryMessage() {
        return congratulatoryMessage;
    }

    public void setCongratulatoryMessage(MultiLangText congratulatoryMessage) {
        this.congratulatoryMessage = congratulatoryMessage;
    }

    public String getUnawardedImage() {
        return unawardedImage;
    }

    public void setUnawardedImage(String unawardedImage) {
        this.unawardedImage = unawardedImage;
    }

    public String getAwardedImage() {
        return awardedImage;
    }

    public void setAwardedImage(String awardedImage) {
        this.awardedImage = awardedImage;
    }

    public String getEnlargedAwardedImage() {
        return enlargedAwardedImage;
    }

    public void setEnlargedAwardedImage(String enlargedAwardedImage) {
        this.enlargedAwardedImage = enlargedAwardedImage;
    }

    public String getEnlargedUnawardedImage() {
        return enlargedUnawardedImage;
    }

    public void setEnlargedUnawardedImage(String enlargedUnawardedImage) {
        this.enlargedUnawardedImage = enlargedUnawardedImage;
    }

    public String getDefaultTitle() {
        return title.getEn();
    }

    public String getDefaultDescription() {
        return description.getEn();
    }


    public Bitmap getUnawardedImageBitmap() {
        int index = unawardedImage.indexOf(",");
        String imageToDecode = index != -1 ? unawardedImage.substring(index) : unawardedImage;
        byte[] decodedString = Base64.decode(imageToDecode, Base64.DEFAULT);

        return BitmapFactory.decodeByteArray(decodedString, 0, decodedString.length);
    }

    public Bitmap getAwardedImageBitmap() {
        int index = awardedImage.indexOf(",");
        String imageToDecode = index != -1 ? awardedImage.substring(index) : awardedImage;
        byte[] decodedString = Base64.decode(imageToDecode, Base64.DEFAULT);

        return BitmapFactory.decodeByteArray(decodedString, 0, decodedString.length);
    }

    public Bitmap getEnlargedUnawardedImageBitmap() {
        int index = enlargedUnawardedImage.indexOf(",");
        String imageToDecode = index != -1 ? enlargedUnawardedImage.substring(index) : enlargedUnawardedImage;
        byte[] decodedString = Base64.decode(imageToDecode, Base64.DEFAULT);

        return BitmapFactory.decodeByteArray(decodedString, 0, decodedString.length);
    }

    public Bitmap getEnlargedAwardedImageBitmap() {
        int index = enlargedAwardedImage.indexOf(",");
        String imageToDecode = index != -1 ? enlargedAwardedImage.substring(index) : enlargedAwardedImage;
        byte[] decodedString = Base64.decode(imageToDecode, Base64.DEFAULT);

        return BitmapFactory.decodeByteArray(decodedString, 0, decodedString.length);
    }

    public BadgeCategory getBadgeCategory() {
        return BadgeCategory.fromValue(category);
    }

    @NonNull
    @Override
    public String toString() {
        return "{" +
                "\"name\":\"" + name + '\"' +
                ", \"addOrder\":\"" + addOrder + '\"' +
                ", \"category\":\"" + category + '\"' +
                ", \"title\":\"" + title + '\"' +
                ", \"description\":\"" + description + '\"' +
                ", \"congratulatoryMessage\":\"" + congratulatoryMessage + '\"' +
                ", \"awardedImage\":\"" + awardedImage.substring(0, 10) + "..." + '\"' +
                ", \"unawardedImage\":\"" + unawardedImage.substring(0, 10) + "..." + '\"' +
                ", \"enlargedAwardedImage\":\"" + enlargedAwardedImage.substring(0, 10) + "..." + '\"' +
                ", \"enlargedUnawardedImage\":\"" + enlargedUnawardedImage.substring(0, 10) + "..." + '\"' +
                '}';
    }

    public TextSection getBadgeTextSection() {
        return TextSection.ofDynamicText(congratulatoryMessage.getEn(), R.style.Body_Inverse, true);
    }

    public TextSection getUnawardedBadgeTextSection() {
        return TextSection.ofDynamicText(description.getEn(), R.style.Body_Inverse, true);
    }

    public static Consumer<Badge> getDefaultBadgeBehaviour(FragmentManager fragmentManager, Resources resources, View.OnClickListener onDismissListener) {
        return badge -> {
            if (badge != null) {
                GritDialogFragment.showBadgeDialog(fragmentManager, resources, badge, onDismissListener);
            } else {
                RemoteLog.d("DefaultBadgeBehaviour", "Badge attempting to process could not be found");
            }
        };
    }

    public static Consumer<Badge> getUnawardedBadgeBehaviour(FragmentManager fragmentManager, Resources resources) {
        return badge -> {
            if (badge != null) {
                GritDialogFragment.showUnawardedBadgeDialog(fragmentManager, resources, badge);
            } else {
                RemoteLog.d("DefaultBadgeBehaviour", "Badge attempting to process could not be found");
            }
        };
    }

    public enum BadgeCategory {
        @SerializedName("DEFAULT_BADGE_CATEGORY")
        DEFAULT_BADGE_CATEGORY("DEFAULT_BADGE_CATEGORY", R.drawable.badge_cat_generic, false, R.string.ds_badge_human_cat_baseline),
        @SerializedName("BASELINE")
        BASELINE("BASELINE", R.drawable.badge_cat_baseline, true, R.string.ds_badge_human_cat_baseline),
        @SerializedName("REASSESSMENT")
        REASSESSMENT("REASSESSMENT", R.drawable.badge_cat_reassessment_mood, true, R.string.ds_badge_human_cat_reassement),
        @SerializedName("PROFILE")
        PROFILE("PROFILE", R.drawable.badge_cat_profile, false, R.string.ds_badge_human_cat_profile),
        @SerializedName("SQUAD")
        SQUAD("SQUAD", R.drawable.badge_cat_squad, false, R.string.ds_badge_human_cat_squad),
        @SerializedName("EMPLOYMENT")
        EMPLOYMENT("EMPLOYMENT", R.drawable.badge_cat_filter_resume_saved, false, R.string.ds_badge_human_cat_employment),
        @SerializedName("SKILLS")
        SKILLS("SKILLS", R.drawable.card_survey_end_badge, false, R.string.ds_badge_human_cat_skills);

        @DrawableRes
        private final int badgeBackground;
        private final String categoryName;
        @IdRes
        private final int humanCategoryNameResId;
        private final boolean isSurveyType;
        private final static Map<String, BadgeCategory> CONSTANTS = new HashMap<>();

        static {
            for (BadgeCategory c : values()) {
                CONSTANTS.put(c.categoryName, c);
            }
        }

        BadgeCategory(String categoryName, int badgeBackground, boolean isSurveyType, int humanCategoryName) {
            this.badgeBackground = badgeBackground;
            this.categoryName = categoryName;
            this.isSurveyType = isSurveyType;
            this.humanCategoryNameResId = humanCategoryName;
        }

        @NonNull
        @Override
        public String toString() {
            return value();
        }

        String value() {
            return this.categoryName;
        }

        public int getHumanCategoryNameResId() {
            return this.humanCategoryNameResId;
        }

        public boolean isSurveyType() {
            return isSurveyType;
        }

        static BadgeCategory fromValue(String categoryName) {
            BadgeCategory constant = CONSTANTS.get(categoryName);
            // if we won't find a valid category, we use the default category
            return constant != null ?
                    constant :
                    BadgeCategory.DEFAULT_BADGE_CATEGORY;
        }

        public int getBackground() {
            return badgeBackground;
        }
    }
}
