package com.ibm.gbs.va.grit.model.type;

import android.support.annotation.NonNull;

import com.google.gson.annotations.SerializedName;
import com.ibm.gbs.va.grit.R;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DigitalJournalEvent implements Comparable<DigitalJournalEvent> {

    @SerializedName("eventType")
    private String eventType;

    @SerializedName("instances")
    private List<EventInstance> instances;

    public DigitalJournalEvent(String eventType, List<EventInstance> instances) {
        this.eventType = eventType;
        this.instances = instances;
    }

    public EventType getEventType() {
        return EventType.fromValue(eventType);
    }

    public void setEventType(String eventType) {
        this.eventType = eventType;
    }

    public List<EventInstance> getInstances() {
        return instances;
    }

    public void setInstances(List<EventInstance> instances) {
        this.instances = instances;
    }

    public int getEventCount() {
        return instances.size();
    }

    @Override
    public int compareTo(@NonNull DigitalJournalEvent o) {
        if (getEventType() != null && o.getEventType() != null)
            return getEventType().compareTo(o.getEventType());
        else return 0;
    }

    public class EventInstance {
        @SerializedName("date")
        private String date;

        private String detail;

        public String getDate() {
            return date;
        }

        public void setDate(String date) {
            this.date = date;
        }

        public String getDetail() {
            return detail;
        }

        public void setDetail(String detail) {
            this.detail = detail;
        }
    }

    public enum EventType {
        @SerializedName("DEFAULT_EVENT_TYPE")
        DEFAULT_EVENT_TYPE("DEFAULT_EVENT_TYPE", R.string.ds_activity_tracker_default, R.drawable.ic_shield_filled),
        @SerializedName("SQUAD_MEMBER_ADDED")
        SQUAD_MEMBER_ADDED("SQUAD_MEMBER_ADDED", R.string.ds_activity_tracker_squad, R.drawable.ic_squad_filled),
        @SerializedName("DIGITAL_ASSISTANT")
        DIGITAL_ASSISTANT("DIGITAL_ASSISTANT", R.string.ds_activity_tracker_da, R.drawable.ic_chat_filled),
        @SerializedName("SURVEY_COMPLETED")
        SURVEY_COMPLETED("SURVEY_COMPLETED", R.string.ds_activity_tracker_survey, R.drawable.ic_checked_filled),
        @SerializedName("BADGES_AWARDED")
        BADGES_AWARDED("BADGES_AWARDED", R.string.ds_activity_tracker_badges, R.drawable.ic_shield_filled),
        @SerializedName("SAVED_JOB")
        SAVED_JOB("SAVED_JOB", R.string.ds_activity_tracker_jobs, R.drawable.ic_bookmark_filled);

        private String typeName;
        private int stringID;
        private int iconID;
        private final static Map<String, EventType> CONSTANTS = new HashMap<>();

        EventType(String typeName, int stringID, int iconID) {
            this.typeName = typeName;
            this.stringID = stringID;
            this.iconID = iconID;
        }

        public int getStringID() {
            return stringID;
        }

        public int getIconID() {
            return iconID;
        }

        public String getTypeName() {
            return typeName;
        }

        static {
            for (EventType c : values()) {
                CONSTANTS.put(c.typeName, c);
            }
        }

        public static List<EventType> normalizeEvents(List<EventType> currentListOfEvents) {
            // input list of EventTypes,
            // will return a list of EventTypes which contains any non-default eventTypes that are missing
            List<EventType> missingEvents = new ArrayList<>();
            for (EventType curEventTypeConstants : CONSTANTS.values()) {
                if (!curEventTypeConstants.typeName.equals(DEFAULT_EVENT_TYPE.typeName)) {
                    boolean found = false;
                    for (EventType curEventTypeChecking : currentListOfEvents) {
                        if (curEventTypeConstants.typeName.equals(DEFAULT_EVENT_TYPE.typeName) || curEventTypeChecking.typeName.equals(curEventTypeConstants.typeName)) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) missingEvents.add(curEventTypeConstants);
                }
            }
            return missingEvents;
        }

        @SuppressWarnings("WeakerAccess")
        public static EventType fromValue(String typeName) {
            EventType constant = CONSTANTS.get(typeName);
            // if we won't find a valid category, we use the default category
            return constant != null ?
                    constant :
                    EventType.DEFAULT_EVENT_TYPE;
        }
    }
}
