package com.ibm.gbs.va.grit.model.type;

import android.arch.persistence.room.ColumnInfo;
import android.arch.persistence.room.Entity;
import android.arch.persistence.room.ForeignKey;
import android.arch.persistence.room.Index;
import android.arch.persistence.room.PrimaryKey;
import android.databinding.BaseObservable;
import android.databinding.Bindable;
import android.support.annotation.NonNull;

import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;
import com.ibm.gbs.va.grit.BR;
import com.ibm.gbs.va.grit.util.StringUtil;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Entity(
        tableName = "job_info",
        foreignKeys = @ForeignKey(
                entity = User.class,
                parentColumns = "email",
                childColumns = "user_email",
                onDelete = ForeignKey.CASCADE,
                onUpdate = ForeignKey.CASCADE),
        indices = @Index("user_email"))
public class EmploymentInfo extends BaseObservable {

    // ////////////////////////////////
    // FIELDS
    @SerializedName("desiredLocation")
    @ColumnInfo(name = "desired_location")
    private String desiredLocation;

    @SerializedName("militaryBranch")
    @ColumnInfo(name = "military_branch")
    private MilitaryBranch militaryBranch;

    @SerializedName("mos")
    @ColumnInfo(name = "mos")
    private String mos;

    @SerializedName("rank")
    @ColumnInfo(name = "rank")
    private Rank rank;

    @SerializedName("targetIndustry")
    @ColumnInfo(name = "target_industry")
    private String targetIndustry;

    @SerializedName("minimumSalary")
    @ColumnInfo(name = "minimum_salary")
    private String minimumSalary;

    @SerializedName("jobTypes")
    @ColumnInfo(name = "job_types")
    private List<JobType> jobTypes;

    @SerializedName("resumeFilename")
    @ColumnInfo(name = "resume_filename")
    private String resumeFilename;

    @SerializedName("uploadTimestamp")
    @ColumnInfo(name = "upload_timestamp")
    private String uploadTimestamp;

    @SerializedName("wemProfileExists")
    @ColumnInfo(name = "wem_profile_exists")
    private Boolean wemProfileExists;

    // ////////////////////////////////
    // GETTERS & SETTERS
    @Bindable
    public String getDesiredLocation() {
        return desiredLocation;
    }

    public void setDesiredLocation(String desiredLocation) {
        this.desiredLocation = desiredLocation;
        notifyPropertyChanged(BR.desiredLocation);
    }

    @Bindable
    public MilitaryBranch getMilitaryBranch() {
        return militaryBranch;
    }

    public void setMilitaryBranch(MilitaryBranch militaryBranch) {
        this.militaryBranch = militaryBranch;
        notifyPropertyChanged(BR.militaryBranch);
    }

    @Bindable
    public String getMos() {
        return mos;
    }

    public void setMos(String mos) {
        this.mos = mos;
        notifyPropertyChanged(BR.mos);
    }

    @Bindable
    public Rank getRank() {
        return rank;
    }

    public void setRank(Rank rank) {
        this.rank = rank;
        notifyPropertyChanged(BR.rank);
    }

    @Bindable
    public String getTargetIndustry() {
        return targetIndustry;
    }

    public void setTargetIndustry(String targetIndustry) {
        this.targetIndustry = targetIndustry;
        notifyPropertyChanged(BR.targetIndustry);
    }

    @Bindable
    public String getMinimumSalary() {
        return minimumSalary;
    }

    @SuppressWarnings("WeakerAccess")
    public void setMinimumSalary(Integer minimumSalary) {
        this.minimumSalary = minimumSalary.toString();
        notifyPropertyChanged(BR.minimumSalary);
    }

    public void setMinimumSalary(String minimumSalary) {
        this.minimumSalary = minimumSalary;
        notifyPropertyChanged(BR.minimumSalary);
    }


    @Bindable
    public List<JobType> getJobTypes() {
        return jobTypes;
    }

    public void setJobTypes(List<JobType> jobTypes) {
        this.jobTypes = jobTypes;
        notifyPropertyChanged(BR.jobTypes);
    }

    @Bindable
    public String getResumeFilename() {
        return resumeFilename;
    }

    public void setResumeFilename(String resumeFilename) {
        this.resumeFilename = resumeFilename;
        notifyPropertyChanged(BR.resumeFilename);
    }

    @Bindable
    public String getUploadTimestamp() {
        return uploadTimestamp;
    }

    public void setUploadTimestamp(String uploadTimestamp) {
        this.uploadTimestamp = uploadTimestamp;
        notifyPropertyChanged(BR.uploadTimestamp);
    }

    @Bindable
    public Boolean getWemProfileExists() {
        return wemProfileExists;
    }

    public void setWemProfileExists(Boolean wemProfileExists) {
        this.wemProfileExists = wemProfileExists;
        notifyPropertyChanged(BR.wemProfileExists);
    }

    // ////////////////////////////////
    // TRANSIENT FIELDS
    @PrimaryKey(autoGenerate = true)
    @ColumnInfo(name = "primary_id")
    transient
    private int primaryId;

    @ColumnInfo(name = "user_email")
    transient
    private String userEmail;

    public int getPrimaryId() {
        return primaryId;
    }

    public void setPrimaryId(int primaryId) {
        this.primaryId = primaryId;
    }

    public String getUserEmail() {
        return userEmail;
    }

    public void setUserEmail(String userEmail) {
        this.userEmail = userEmail;
    }

    public EmploymentInfo duplicate() {
        EmploymentInfo ret = new EmploymentInfo();
        ret.desiredLocation = desiredLocation;
        ret.militaryBranch = militaryBranch;
        ret.mos = mos;
        ret.rank = rank;
        ret.targetIndustry = targetIndustry;
        ret.minimumSalary = minimumSalary;
        ret.jobTypes = jobTypes;
        ret.resumeFilename = resumeFilename;
        ret.uploadTimestamp = uploadTimestamp;
        ret.wemProfileExists = wemProfileExists;
        return ret;
    }

    public boolean hasResume() {
        return this.resumeFilename != null && !this.resumeFilename.isEmpty();
    }

    public boolean isComplete() {
        return getWemProfileExists();
    }

    @NonNull
    @Override
    public String toString() {
        return new Gson().toJson(this);
    }

    public enum MilitaryBranch {
        @SerializedName("ARMY")
        ARMY("ARMY"),
        @SerializedName("NAVY")
        NAVY("NAVY"),
        @SerializedName("AIR_FORCE")
        AIR_FORCE("AIR_FORCE"),
        @SerializedName("MARINES")
        MARINES("MARINES"),
        @SerializedName("COAST_GUARD")
        COAST_GUARD("COAST_GUARD");

        private final String value;
        private final static Map<String, MilitaryBranch> CONSTANTS = new HashMap<>();

        static {
            for (MilitaryBranch c : values()) {
                CONSTANTS.put(c.value, c);
                CONSTANTS.put(StringUtil.toTitleCase(c.value), c);
            }
        }

        MilitaryBranch(String value) {
            this.value = value;
        }

        @NonNull
        @Override
        public String toString() {
            return value();
        }

        public String value() {
            return this.value;
        }

        public String displayValue() {
            return StringUtil.toTitleCase(this.value);
        }

        public static String[] displayValues() {
            String[] ret = new String[values().length];
            for (int i = 0; i < values().length; i++) {
                ret[i] = StringUtil.toTitleCase(values()[i].value());
            }
            return ret;
        }

        public static MilitaryBranch fromValue(String value) {
            MilitaryBranch constant = CONSTANTS.get(value);
            if (constant == null) {
                throw new IllegalArgumentException(value);
            } else {
                return constant;
            }
        }
    }

    public enum Rank {
        @SerializedName("E-1")
        E1("E-1"),
        @SerializedName("E-2")
        E2("E-2"),
        @SerializedName("E-3")
        E3("E-3"),
        @SerializedName("E-4")
        E4("E-4"),
        @SerializedName("E-5")
        E5("E-5"),
        @SerializedName("E-6")
        E6("E-6"),
        @SerializedName("E-7")
        E7("E-7"),
        @SerializedName("E-8")
        E8("E-8"),
        @SerializedName("E-9")
        E9("E-9"),
        @SerializedName("O-1")
        O1("O-1"),
        @SerializedName("O-2")
        O2("O-2"),
        @SerializedName("O-3")
        O3("O-3"),
        @SerializedName("O-4")
        O4("O-4"),
        @SerializedName("O-5")
        O5("O-5"),
        @SerializedName("O-6")
        O6("O-6"),
        @SerializedName("O-7")
        O7("O-7"),
        @SerializedName("O-8")
        O8("O-8"),
        @SerializedName("O-9")
        O9("O-9"),
        @SerializedName("WO-1")
        WO1("WO-1"),
        @SerializedName("WO-2")
        WO2("WO-2"),
        @SerializedName("WO-3")
        WO3("WO-3"),
        @SerializedName("WO-4")
        WO4("WO-4"),
        @SerializedName("WO-5")
        WO5("WO-5");

        private final String value;
        private final static Map<String, Rank> CONSTANTS = new HashMap<>();

        static {
            for (Rank c : values()) {
                CONSTANTS.put(c.value, c);
                CONSTANTS.put(c.value.replace('-', '–'), c);
            }
        }

        Rank(String value) {
            this.value = value;
        }

        @NonNull
        @Override
        public String toString() {
            return value();
        }

        public String value() {
            return this.value;
        }

        public String displayValue() {
            return this.value.replace('-', '–');
        }

        public static String[] displayValues() {
            String[] ret = new String[values().length];
            for (int i = 0; i < values().length; i++) {
                ret[i] = values()[i].value().replace('-', '–');
            }
            return ret;
        }

        public static Rank fromValue(String value) {
            Rank constant = CONSTANTS.get(value);
            if (constant == null) {
                throw new IllegalArgumentException(value);
            } else {
                return constant;
            }
        }
    }

    public enum JobType {
        @SerializedName("FULLTIME")
        FULLTIME("FULLTIME"),
        @SerializedName("PARTTIME")
        PARTTIME("PARTTIME");

        private final String value;
        private final static Map<String, JobType> CONSTANTS = new HashMap<>();

        static {
            for (JobType c : values()) {
                CONSTANTS.put(c.value, c);
            }
        }

        JobType(String value) {
            this.value = value;
        }

        @NonNull
        @Override
        public String toString() {
            return value();
        }

        @SuppressWarnings("WeakerAccess")
        public String value() {
            return this.value;
        }

        @SuppressWarnings("unused")
        public static JobType fromValue(String value) {
            JobType constant = CONSTANTS.get(value);
            if (constant == null) {
                throw new IllegalArgumentException(value);
            } else {
                return constant;
            }
        }
    }
}
