package com.ibm.gbs.va.grit.model.type;

import android.arch.persistence.room.ColumnInfo;
import android.arch.persistence.room.Entity;
import android.arch.persistence.room.ForeignKey;
import android.arch.persistence.room.Ignore;
import android.arch.persistence.room.Index;
import android.arch.persistence.room.PrimaryKey;
import android.graphics.Bitmap;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;

import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Entity(tableName = "squad_member",
        foreignKeys = {
                @ForeignKey(entity = User.class,
                        parentColumns = "email",
                        childColumns = "user_email",
                        onDelete = ForeignKey.RESTRICT,
                        onUpdate = ForeignKey.CASCADE)},
        indices = {
                @Index("user_email")}
)
public class SquadMember {
    // ////////////////////////////////
    // FIELDS
    @SerializedName("name")
    @ColumnInfo(name = "name")
    private String name;

    @SerializedName("cellPhone")
    @ColumnInfo(name = "cell_phone")
    private String cellPhone;

    @SerializedName("state")
    @ColumnInfo(name = "state")
    private State state;

    @SerializedName("id")
    @ColumnInfo(name = "id")
    @PrimaryKey
    @NonNull
    private String id;

    public SquadMember() {
    }

    @Ignore
    public SquadMember(String name, String cellPhone, Bitmap profileImage) {
        this.name = name;
        this.cellPhone = cellPhone;
        this.state = State.PENDING;
        this.profileImage = profileImage;
    }

    // ////////////////////////////////
    // GETTERS & SETTERS
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getCellPhone() {
        return cellPhone;
    }

    public void setCellPhone(String cellPhone) {
        this.cellPhone = cellPhone;
    }

    public State getState() {
        return state;
    }

    public void setState(State state) {
        this.state = state;
    }

    @NonNull
    public String getId() {
        return id;
    }

    public void setId(@NonNull String id) {
        this.id = id;
    }

    // ////////////////////////////////
    // TRANSIENT FIELDS
    @ColumnInfo(name = "user_email")
    transient
    private String userEmail;

    @ColumnInfo(name = "profile_image")
    transient
    private Bitmap profileImage;

    // ////////////////////////////////
    // TRANSIENT GETTERS & SETTERS
    public String getUserEmail() {
        return userEmail;
    }

    public void setUserEmail(String userEmail) {
        this.userEmail = userEmail;
    }

    public Bitmap getProfileImage() {
        return profileImage;
    }

    public void setProfileImage(Bitmap profileImage) {
        this.profileImage = profileImage;
    }
    ///////////////////////////////////

    public String getInitials() {
        Pattern p = Pattern.compile("((^| )[A-Za-z])");
        Matcher matcher = p.matcher(name);
        StringBuilder initials = new StringBuilder();
        while (matcher.find()) {
            initials.append(matcher.group().trim());
        }
        if (initials.length() > 3) initials.setLength(2);
        return initials.toString().toUpperCase();
    }

    @NonNull
    @Override
    public String toString() {
        return new Gson().toJson(this);
    }

    public boolean compareTo(SquadMember other) {
        return TextUtils.equals(name, other.name) &&
                TextUtils.equals(cellPhone, other.cellPhone) &&
                state == other.state &&
                TextUtils.equals(id, other.id) &&
                ((profileImage == null && other.profileImage == null) ||
                        profileImage != null && profileImage.sameAs(other.profileImage));
    }

    @Override
    public boolean equals(@Nullable Object obj) {
        if (obj instanceof SquadMember) {
            SquadMember other = (SquadMember) obj;
            return TextUtils.equals(id, other.id);
        } else
            return super.equals(obj);
    }

    public enum State {
        @SerializedName("ACCEPTED")
        ACCEPTED("ACCEPTED"),
        @SerializedName("PENDING")
        PENDING("PENDING");

        private final String value;
        private final static Map<String, State> CONSTANTS = new HashMap<>();

        static {
            for (State c : values()) {
                CONSTANTS.put(c.value, c);
            }
        }

        State(String value) {
            this.value = value;
        }

        @NonNull
        @Override
        public String toString() {
            return value();
        }

        public String value() {
            return this.value;
        }

        public static State fromValue(String value) {
            State constant = CONSTANTS.get(value);
            if (constant == null) {
                throw new IllegalArgumentException(value);
            } else {
                return constant;
            }
        }
    }
}
