package com.ibm.gbs.va.grit.repository;

import android.database.sqlite.SQLiteConstraintException;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.ibm.gbs.va.grit.GritApplication;
import com.ibm.gbs.va.grit.model.request.GetBadgeTypesRequest;
import com.ibm.gbs.va.grit.model.response.GetBadgeTypesResponse;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.rest.GritCallback;
import com.ibm.gbs.va.grit.rest.RetrofitApiServices;
import com.ibm.gbs.va.grit.room.BadgeDao;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.util.UserPrefs;

import java.util.ArrayList;
import java.util.List;

public class BadgeRepository {
    private static final String TAG = "BadgeRepository";

    private final int curAddOrder;

    private final RetrofitApiServices restServices;
    private final GritApplication gritApplication;
    private final UserPrefs userPrefs;
    private final BadgeDao badgeDao;

    public BadgeRepository(RetrofitApiServices restServices, UserPrefs userPrefs, GritApplication gritApplication, BadgeDao badgeDao) {
        this.restServices = restServices;
        this.gritApplication = gritApplication;
        this.userPrefs = userPrefs;
        this.badgeDao = badgeDao;
        curAddOrder = userPrefs.getGlobalPrefHighestAddorder();
    }

    public void updateBadgeListInitial() {
        if (curAddOrder > 0) // initial badges are provided via assets
            updateBadgeList(curAddOrder);
    }

    private void updateBadgeList(int currentAddOrder) {
        RemoteLog.d(TAG, "Updating badges with addOrder: " + currentAddOrder);
        restServices.getBadgeTypes(new GetBadgeTypesRequest(currentAddOrder))
                .enqueue(new GritCallback<GetBadgeTypesResponse>(gritApplication) {
                    @Override
                    public void onSuccess(GetBadgeTypesResponse payload) {
                        RemoteLog.d(TAG, "payload: " + payload);
                        RemoteLog.d(TAG, "payload: badges.size: " + payload.getBadges().size());
                        List<Badge> newBadges = payload.getBadges();
                        if (newBadges.size() != 0) {

                            int highestAddOrderFound = 0;
                            for (Badge curBadge : newBadges) {
                                if (curBadge.getAddOrder() > highestAddOrderFound)
                                    highestAddOrderFound = curBadge.getAddOrder();
                            }

                            RemoteLog.d(TAG, "New highest addOrder: " + highestAddOrderFound);
                            userPrefs.setGlobalPrefHighestAddorder(highestAddOrderFound);

                            try {
                                badgeDao.saveMultipleBadges(newBadges);
                            } catch (SQLiteConstraintException e) {
                                // Shouldn't ever be thrown, conflicts should be ignored
                                RemoteLog.d(TAG, "Duplicate badge names in badges: " + newBadges.toString() + " addOrder: " + currentAddOrder);
                                throw new RuntimeException("Duplicate badge name downloaded");
                            }
                        }
                    }

                });
    }

    @Nullable
    private Badge getBadgeFromString(String badgeName) {
        return badgeDao.getBadgeFromString(badgeName);
    }

    @NonNull
    public List<Badge> getBadgeListFromStrings(@Nullable List<String> badgeNames) {
        if (badgeNames == null) return new ArrayList<>();

        ArrayList<Badge> returnedBadges = new ArrayList<>();
        for (String curBadgeName : badgeNames) {
            Badge badge = getBadgeFromString(curBadgeName);
            // don't add nulls to list
            if (badge != null) returnedBadges.add(badge);
        }
        return returnedBadges;
    }

    List<Badge> getAllBadges() {
        return badgeDao.getAllBadges();
    }
}
