package com.ibm.gbs.va.grit.repository;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.MutableLiveData;

import com.ibm.gbs.va.grit.GritApplication;
import com.ibm.gbs.va.grit.model.request.AddHandleRequest;
import com.ibm.gbs.va.grit.model.request.BaseRequest;
import com.ibm.gbs.va.grit.model.request.RemoveHandleRequest;
import com.ibm.gbs.va.grit.model.request.UpdateHandleRequest;
import com.ibm.gbs.va.grit.model.response.AddHandleResponse;
import com.ibm.gbs.va.grit.model.response.GetHandlesResponse;
import com.ibm.gbs.va.grit.model.type.Handle;
import com.ibm.gbs.va.grit.rest.GritCallback;
import com.ibm.gbs.va.grit.rest.RetrofitApiServices;
import com.ibm.gbs.va.grit.util.RemoteLog;

public class SocialMediaRepository {
    private static final String TAG = "SocialMediaRepository";
    private final MutableLiveData<Handle> twitterHandle = new MutableLiveData<>();
    private final RetrofitApiServices restServices;
    private final GritApplication gritApplication;

    public SocialMediaRepository(RetrofitApiServices restServices, GritApplication gritApplication) {
        this.restServices = restServices;
        this.gritApplication = gritApplication;
    }

    public LiveData<Handle> getTwitterHandle() {
        if (twitterHandle.getValue() == null) {
            restServices.getHandles(new BaseRequest())
                    .enqueue(new GritCallback<GetHandlesResponse>(gritApplication) {
                        @Override
                        public void onSuccess(GetHandlesResponse payload) {
                            twitterHandle.setValue(payload.getHandleOfType(AddHandleRequest.SocialMediaType.TWITTER));
                        }
                    });
        }
        return twitterHandle;
    }

    public enum UPDATE_STATE {
        UPDATED_NO_BADGE, UPDATED_WITH_BADGE, UPDATED_FAILED
    }

    public LiveData<UPDATE_STATE> addTwitterHandle(String handleStr) {
        MutableLiveData<UPDATE_STATE> result = new MutableLiveData<>();
        RemoteLog.d(TAG, "Adding twitter handle: " + handleStr);
        restServices.addHandle(new AddHandleRequest(AddHandleRequest.SocialMediaType.TWITTER, handleStr))
                .enqueue(new GritCallback<AddHandleResponse>(gritApplication) {
                    @Override
                    public void onSuccess(AddHandleResponse payload, boolean badgeAwarded) {
                        RemoteLog.d(TAG, "twitter handle has been added");
                        twitterHandle.setValue(new Handle(AddHandleRequest.SocialMediaType.TWITTER,
                                handleStr, payload.getId()));

                        result.setValue(badgeAwarded ? UPDATE_STATE.UPDATED_WITH_BADGE : UPDATE_STATE.UPDATED_NO_BADGE);
                    }

                    @Override
                    public void onSuccess(AddHandleResponse payload) {
                        // empty because handled above
                    }
                });
        return result;
    }

    public LiveData<Boolean> removeTwitterHandle() {
        MutableLiveData<Boolean> result = new MutableLiveData<>();
        RemoteLog.d(TAG, "Removing twitter handle");
        AddHandleRequest.SocialMediaType type = twitterHandle.getValue().getSocialMediaType();
        restServices.removeHandle(new RemoveHandleRequest(twitterHandle.getValue().getId()))
                .enqueue(new GritCallback<Object>(gritApplication) {
                    @Override
                    public void onSuccess(Object payload) {
                        RemoteLog.d(TAG, "twitter handle successfully removed");
                        twitterHandle.setValue(new Handle(type));
                        result.setValue(true);
                    }
                });
        return result;
    }

    public LiveData<UPDATE_STATE> updateTwitterHandle(String handleStr) {
        MutableLiveData<UPDATE_STATE> result = new MutableLiveData<>();
        RemoteLog.d(TAG, "Twitter handle being updated to: " + handleStr);
        AddHandleRequest.SocialMediaType type = twitterHandle.getValue().getSocialMediaType();
        restServices.updateHandle(new UpdateHandleRequest(twitterHandle.getValue().getId(), handleStr))
                .enqueue(new GritCallback<AddHandleResponse>(gritApplication) {
                    @Override
                    public void onSuccess(AddHandleResponse payload, boolean badgeAwarded) {
                        RemoteLog.d(TAG, "twitter handle successfully updated");
                        twitterHandle.setValue(new Handle(type, handleStr, payload.getId()));
                        result.setValue(badgeAwarded ? UPDATE_STATE.UPDATED_WITH_BADGE : UPDATE_STATE.UPDATED_NO_BADGE);
                    }

                    @Override
                    public void onSuccess(AddHandleResponse payload) {
                        // empty because handled above
                    }
                });
        return result;
    }
}
