package com.ibm.gbs.va.grit.repository;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.MutableLiveData;
import android.arch.lifecycle.Transformations;

import com.ibm.gbs.va.grit.GritApplication;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.request.AddSquadRequest;
import com.ibm.gbs.va.grit.model.request.BaseRequest;
import com.ibm.gbs.va.grit.model.request.RemoveSquadRequest;
import com.ibm.gbs.va.grit.model.response.AddSquadResponse;
import com.ibm.gbs.va.grit.model.response.GetSquadResponse;
import com.ibm.gbs.va.grit.model.type.SquadMember;
import com.ibm.gbs.va.grit.rest.GritCallback;
import com.ibm.gbs.va.grit.rest.RetrofitApiServices;
import com.ibm.gbs.va.grit.room.SquadDao;
import com.ibm.gbs.va.grit.util.RemoteLog;

import java.util.List;

public class SquadRepository {
    private static final String TAG = "SquadRepository";

    private final SquadDao squadDao;
    private final RetrofitApiServices restServices;
    private final GritApplication gritApplication;
    private final NotificationRepository messageRepo;
    private final UserRepository userRepo;

    private final LiveData<List<SquadMember>> squad;

    private final String userEmail;

    public SquadRepository(RetrofitApiServices restServices, UserRepository userRepo, GritApplication gritApplication, SquadDao squadDao, NotificationRepository messageRepo) {
        this.restServices = restServices;
        this.gritApplication = gritApplication;
        this.squadDao = squadDao;
        this.messageRepo = messageRepo;
        this.userRepo = userRepo;

        RemoteLog.d("DaggerScope", TAG);
        userEmail = userRepo.getCurrentUserEmail();
        MutableLiveData<String> signedInUserEmail = new MutableLiveData<>();
        signedInUserEmail.setValue(userEmail);

        squad = Transformations.switchMap(signedInUserEmail, squadDao::getSquadForUser);
    }

    public LiveData<List<SquadMember>> getSquad() {
        restServices.getSquad(new BaseRequest()).enqueue(new GritCallback<GetSquadResponse>(gritApplication) {
            @Override
            public void onSuccess(GetSquadResponse payload) {
                RemoteLog.d(TAG, "payload: " + payload);
                squadDao.saveSquadForUser(payload.getSquad(), userEmail);
                if (!userRepo.getOnboardingSquadComplete()) {
                    if (payload.getSquad() != null && payload.getSquad().size() > 0)
                        messageRepo.dismissNotification(Notification.Type.UPDATE_SQUAD);
                }
            }
        });

        return squad;
    }

    public void addSquad(SquadMember member) {
        String formattedCell = "1" + member.getCellPhone().replaceAll("[^0-9]", "");
        member.setCellPhone(formattedCell);
        restServices.addSquad(new AddSquadRequest(member)).enqueue(new GritCallback<AddSquadResponse>(gritApplication) {
            @Override
            public void onSuccess(AddSquadResponse payload) {
                RemoteLog.d(TAG, "payload: " + payload);
                member.setId(payload.getId());
                member.setUserEmail(userEmail);
                squadDao.insertSquadMember(member);
                if (!userRepo.getOnboardingSquadComplete()) {
                    messageRepo.dismissNotification(Notification.Type.UPDATE_SQUAD);
                }
            }
        });
    }

    public void removeSquadMember(SquadMember member) {
        restServices.removeSquad(new RemoveSquadRequest(member.getId())).enqueue(new GritCallback<Object>(gritApplication) {
            @Override
            public void onSuccess(Object payload) {
                squadDao.deleteSquadMember(member);
            }
        });
    }
}
