package com.ibm.gbs.va.grit.rest;

import android.app.Activity;
import android.arch.lifecycle.LiveData;
import android.support.design.widget.Snackbar;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.view.ViewGroup;

import com.ibm.gbs.va.grit.R;


public class MessageHandler {
    public static void handleError(FragmentActivity activity, String message, LiveData<?>... observables) {
        removeObservable(activity, observables);
        showErrorSnackbar(activity, message);
    }

    public static void handleMessageOnContainer(ViewGroup container, String message) {
        Snackbar.make(container, message, Snackbar.LENGTH_LONG)
                .show();
    }

    public static void handleSuccess(FragmentActivity activity, LiveData<?>... observables) {
        removeObservable(activity, observables);
        showSuccessSnackbar(activity, activity.getString(R.string.success));
    }

    public static void handleSuccess(FragmentActivity activity, String message, LiveData<?>... observables) {
        removeObservable(activity, observables);
        showSuccessSnackbar(activity, message);
    }

    public static Snackbar handleSuccessWithUndo(FragmentActivity activity, String message, View.OnClickListener undoAction, LiveData<?>... observables) {
        removeObservable(activity, observables);
        return showSuccessSnackbarWithUndoAction(activity, message, undoAction);
    }

    public static void handleWarning(FragmentActivity activity, String message) {
        showSuccessSnackbar(activity, message);
    }

    private static void showSuccessSnackbar(Activity activity, String message) {
        Snackbar.make(((ViewGroup) activity.findViewById(android.R.id.content)).getChildAt(0), message, Snackbar.LENGTH_SHORT)
                .show();
    }

    private static Snackbar showSuccessSnackbarWithUndoAction(Activity activity, String message, View.OnClickListener undoAction) {
        Snackbar snackbar = Snackbar.make(((ViewGroup) activity.findViewById(android.R.id.content)).getChildAt(0), message, Snackbar.LENGTH_INDEFINITE)
                .setAction(R.string.undo, undoAction)
                .setActionTextColor(activity.getColor(R.color.colorWarning));
        snackbar.show();
        return snackbar;
    }

    private static void showErrorSnackbar(Activity activity, String message) {
        Snackbar.make(((ViewGroup) activity.findViewById(android.R.id.content)).getChildAt(0), message, Snackbar.LENGTH_LONG)
                .show();
    }

    public static void removeObservable(FragmentActivity activity, LiveData<?>... observables) {
        for (LiveData<?> observable : observables) {
            if (observable != null && observable.hasObservers()) {
                observable.removeObservers(activity);
            }
        }

    }
}
