package com.ibm.gbs.va.grit.rest;

import com.ibm.gbs.va.grit.BuildConfig;
import com.ibm.gbs.va.grit.model.chat.ChatMessage;
import com.ibm.gbs.va.grit.model.request.AddHandleRequest;
import com.ibm.gbs.va.grit.model.request.AddSavedJobRequest;
import com.ibm.gbs.va.grit.model.request.AddSquadRequest;
import com.ibm.gbs.va.grit.model.request.BaseRequest;
import com.ibm.gbs.va.grit.model.request.ConverseRequest;
import com.ibm.gbs.va.grit.model.request.ExistsRequest;
import com.ibm.gbs.va.grit.model.request.ForgotPasswordRequest;
import com.ibm.gbs.va.grit.model.request.GetBadgeTypesRequest;
import com.ibm.gbs.va.grit.model.request.GetJournalEventsRequest;
import com.ibm.gbs.va.grit.model.request.GetQuestionSetRequest;
import com.ibm.gbs.va.grit.model.request.RemoveHandleRequest;
import com.ibm.gbs.va.grit.model.request.RemoveSquadRequest;
import com.ibm.gbs.va.grit.model.request.ResetPasswordRequest;
import com.ibm.gbs.va.grit.model.request.ResolveNotificationRequest;
import com.ibm.gbs.va.grit.model.request.SearchSkillsRequest;
import com.ibm.gbs.va.grit.model.request.SignInRequest;
import com.ibm.gbs.va.grit.model.request.SignUpRequest;
import com.ibm.gbs.va.grit.model.request.SubmitSurveyRequest;
import com.ibm.gbs.va.grit.model.request.UpdateHandleRequest;
import com.ibm.gbs.va.grit.model.request.UpdatePasswordRequest;
import com.ibm.gbs.va.grit.model.request.UploadResumeInfoRequest;
import com.ibm.gbs.va.grit.model.request.UploadResumeRequest;
import com.ibm.gbs.va.grit.model.response.AddHandleResponse;
import com.ibm.gbs.va.grit.model.response.AddSavedJobResponse;
import com.ibm.gbs.va.grit.model.response.AddSquadResponse;
import com.ibm.gbs.va.grit.model.response.BaseResponse;
import com.ibm.gbs.va.grit.model.response.ExistsResponse;
import com.ibm.gbs.va.grit.model.response.ForgotPasswordResponse;
import com.ibm.gbs.va.grit.model.response.GetAwardedBadgesResponse;
import com.ibm.gbs.va.grit.model.response.GetBadgeTypesResponse;
import com.ibm.gbs.va.grit.model.response.GetHandlesResponse;
import com.ibm.gbs.va.grit.model.response.GetJournalEventsResponse;
import com.ibm.gbs.va.grit.model.response.GetMoodScoresResponse;
import com.ibm.gbs.va.grit.model.response.GetNotificationsResponse;
import com.ibm.gbs.va.grit.model.response.GetSquadResponse;
import com.ibm.gbs.va.grit.model.response.GetUserResponse;
import com.ibm.gbs.va.grit.model.response.SignInResponse;
import com.ibm.gbs.va.grit.model.response.SubmitSurveyResponse;
import com.ibm.gbs.va.grit.model.response.TokenResponse;
import com.ibm.gbs.va.grit.model.response.UploadCompleteResponse;
import com.ibm.gbs.va.grit.model.response.UploadResumeInfoResponse;
import com.ibm.gbs.va.grit.model.response.UploadResumeResponse;
import com.ibm.gbs.va.grit.model.survey.QuestionSet;
import com.ibm.gbs.va.grit.model.type.EmploymentInfo;
import com.ibm.gbs.va.grit.model.type.Job;
import com.ibm.gbs.va.grit.model.type.Role;
import com.ibm.gbs.va.grit.model.type.SavedJob;
import com.ibm.gbs.va.grit.model.type.User;
import com.ibm.gbs.va.grit.util.RemoteLog;

import org.joda.time.DateTime;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Date;
import java.util.List;

import javax.inject.Inject;
import javax.inject.Singleton;

import okhttp3.CertificatePinner;
import okhttp3.OkHttpClient;
import retrofit2.Call;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;
import retrofit2.http.Body;
import retrofit2.http.Headers;
import retrofit2.http.POST;

@Singleton
public class RetrofitApiServices {

    private final IRetrofitApi IRetrofitApi;

    //If true all calls will use mock data, if false individual setting will be applied
    private static final boolean useMockDataGlobal = false;
    //Individual Call MockData Settings
    private static final boolean useMockDataSignUp = useMockDataGlobal;
    private static final boolean useMockDataSignIn = useMockDataGlobal;
    private static final boolean useMockDataExists = useMockDataGlobal;
    private static final boolean useMockDataGetNotifications = useMockDataGlobal;
    private static final boolean useMockGetUser = useMockDataGlobal;
    private static final boolean useMockUpdateProfile = useMockDataGlobal;
    private static final boolean useMockUpdatePassword = useMockDataGlobal;
    private static final boolean useMockResetPassword = useMockDataGlobal;
    private static final boolean useMockForgotPassword = useMockDataGlobal;
    private static final boolean useMockGetQuestionSet = useMockDataGlobal;
    private static final boolean useMockSubmitSurvey = useMockDataGlobal;
    private static final boolean useMockGetTargetIndustries = useMockDataGlobal;
    private static final boolean useMockGetEmploymentInfo = useMockDataGlobal;
    private static final boolean useMockUploadEmploymentInfo = useMockDataGlobal;
    private static final boolean useMockUploadResume = useMockDataGlobal;
    private static final boolean useMockUploadComplete = useMockDataGlobal;
    private static final boolean useMockStartConversation = useMockDataGlobal;
    private static final boolean useMockConverse = useMockDataGlobal;
    private static final boolean useMockResolveNotifications = useMockDataGlobal;
    // Be careful when using mock getBadgeTypes, as it will always return the same values regardless
    // of addOrder. This could lead to an endless loop of requests (and a complaint from the database)
    private static final boolean useMockGetBadgeTypes = useMockDataGlobal;
    private static final boolean useMockGetAwardedBadges = useMockDataGlobal;
    private static final boolean useMockGetSquad = useMockDataGlobal;
    private static final boolean useMockAddSquad = useMockDataGlobal;
    private static final boolean useMockRemoveSquad = useMockDataGlobal;
    private static final boolean useMockGetHandles = useMockDataGlobal;
    private static final boolean useMockAddHandle = useMockDataGlobal;
    private static final boolean useMockRemoveHandle = useMockDataGlobal;
    private static final boolean useMockUpdateHandle = useMockDataGlobal;
    private static final boolean useMockDataDeleteAccount = useMockDataGlobal;
    private static final boolean useMockDigitalJournalMoodScores = useMockDataGlobal;
    private static final boolean useMockDigitalJournalEvents = useMockDataGlobal;

    private static final boolean useMockSearchJobs = useMockDataGlobal;
    private static final boolean useMockSearchSkills = useMockDataGlobal;

    private static final boolean useMockGetSavedJobs = useMockDataGlobal;
    private static final boolean useMockAddSavedJob = useMockDataGlobal;
    private static final boolean useMockDeleteSavedJob = useMockDataGlobal;

    private static final String TAG = "RetrofitApiServices";
    private static final boolean pinCert = true;
    private static final DateTime certCutoff = new DateTime(2019, 11, 25, 0, 0);
    private static final String cert = "sha256/8Rw90Ej3Ttt8RRkrg+WYDS9n7IS03bk5bjP/UXPtaY8=";

    @Inject
    public RetrofitApiServices(RetrofitInterceptor retrofitInterceptor) {
        String baseUrl = BuildConfig.base_url;
        OkHttpClient.Builder httpClient = new OkHttpClient.Builder();
        httpClient.addInterceptor(retrofitInterceptor);

        if (RetrofitApiServices.pinCert) {
            pinCert(baseUrl, httpClient);
        }

        //Create Retrofit
        Retrofit retrofit = new Retrofit.Builder()
                .baseUrl(baseUrl)
                .addConverterFactory(GsonConverterFactory.create())
                .client(httpClient.build())
                .build();

        IRetrofitApi = retrofit.create(IRetrofitApi.class);
    }

    private String getDomainName(String url) {
        try {
            URI uri = new URI(url);
            return uri.getHost();
        } catch (URISyntaxException use) {
            RemoteLog.e(TAG, "ERROR parsing domain from url: " + use.getMessage(), use);
            return url;
        }
    }

    private boolean isTodayAfterA(DateTime a) {
        return new DateTime(new Date()).toLocalDate().isAfter(a.toLocalDate());
    }

    private void pinCert(String baseUrl, OkHttpClient.Builder httpClient) {
        if (RetrofitApiServices.pinCert) {
            RemoteLog.d(TAG, "Pinning Certificate...");
            if (!isTodayAfterA(RetrofitApiServices.certCutoff)) {
                RemoteLog.d(TAG, "Cert has not expired yet, adding pin to client");
                String domain = getDomainName(baseUrl);
                RemoteLog.d(TAG, "Domain: " + domain);
                CertificatePinner certPinner = new CertificatePinner.Builder()
                        .add(domain,
                                RetrofitApiServices.cert)
                        .build();
                httpClient.certificatePinner(certPinner);
            } else {
                RemoteLog.d(TAG, "Cert has expired, not pinning");
            }
        }
    }

    public Call<BaseResponse<SignInResponse>> signIn(SignInRequest req) {
        return IRetrofitApi.signin(req);
    }

    public Call<BaseResponse<TokenResponse>> signUp(SignUpRequest req) {
        return IRetrofitApi.signup(req);
    }

    public Call<BaseResponse<ExistsResponse>> exists(ExistsRequest req) {
        return IRetrofitApi.exists(req);
    }

    public Call<BaseResponse<GetNotificationsResponse>> getNotifications(BaseRequest req) {
        return IRetrofitApi.getNotifications(req);
    }

    public Call<BaseResponse<GetUserResponse>> getUser(BaseRequest req) {
        return IRetrofitApi.getUser(req);
    }

    public Call<BaseResponse<TokenResponse>> updateProfile(User req) {
        return IRetrofitApi.updateProfile(req);
    }

    public Call<BaseResponse<TokenResponse>> updatePassword(UpdatePasswordRequest req) {
        return IRetrofitApi.updatePassword(req);
    }

    public Call<BaseResponse<SignInResponse>> resetPassword(ResetPasswordRequest req) {
        return IRetrofitApi.resetPassword(req);
    }

    public Call<BaseResponse<ForgotPasswordResponse>> forgotPassword(ForgotPasswordRequest req) {
        return IRetrofitApi.forgotPassword(req);
    }

    public Call<BaseResponse<QuestionSet>> getQuestionSet(GetQuestionSetRequest req) {
        return IRetrofitApi.getQuestionSet(req);
    }

    public Call<BaseResponse<SubmitSurveyResponse>> submitSurvey(SubmitSurveyRequest req) {
        return IRetrofitApi.submitSurvey(req);
    }

    public Call<BaseResponse<UploadResumeResponse>> uploadResume(UploadResumeRequest req) {
        return IRetrofitApi.uploadResume(req);
    }

    public Call<BaseResponse<UploadCompleteResponse>> uploadComplete(BaseRequest req) {
        return IRetrofitApi.uploadComplete(req);
    }

    public Call<BaseResponse<List<String>>> getTargetIndustries(BaseRequest req) {
        return IRetrofitApi.getTargetIndustries(req);
    }

    public Call<BaseResponse<EmploymentInfo>> getEmploymentInfo(BaseRequest req) {
        return IRetrofitApi.getEmploymentInfo(req);
    }

    public Call<BaseResponse<UploadResumeInfoResponse>> uploadEmploymentInfo(UploadResumeInfoRequest req) {
        return IRetrofitApi.uploadEmploymentInfo(req);
    }

    public Call<BaseResponse<ChatMessage>> startConversation(BaseRequest req) {
        return IRetrofitApi.startConversation(req);
    }

    public Call<BaseResponse<ChatMessage>> converse(ConverseRequest req) {
        return IRetrofitApi.converse(req);
    }

    public Call<BaseResponse<Object>> resolveNotification(ResolveNotificationRequest req) {
        return IRetrofitApi.resolveNotifications(req);
    }

    public Call<BaseResponse<GetBadgeTypesResponse>> getBadgeTypes(GetBadgeTypesRequest req) {
        return IRetrofitApi.getBadgeTypes(req);
    }

    public Call<BaseResponse<GetAwardedBadgesResponse>> getAwardedBadges(BaseRequest req) {
        return IRetrofitApi.getAwardedBadges(req);
    }

    public Call<BaseResponse<GetMoodScoresResponse>> getMoodScores(BaseRequest req) {
        return IRetrofitApi.getMoodScores(req);
    }

    public Call<BaseResponse<GetJournalEventsResponse>> getJournalEvents(GetJournalEventsRequest req) {
        return IRetrofitApi.getJournalEvents(req);
    }

    public Call<BaseResponse<GetSquadResponse>> getSquad(BaseRequest req) {
        return IRetrofitApi.getSquad(req);
    }

    public Call<BaseResponse<AddSquadResponse>> addSquad(AddSquadRequest req) {
        return IRetrofitApi.addSquad(req);
    }

    public Call<BaseResponse<Object>> removeSquad(RemoveSquadRequest req) {
        return IRetrofitApi.removeSquad(req);
    }

    public Call<BaseResponse<GetHandlesResponse>> getHandles(BaseRequest req) {
        return IRetrofitApi.getHandles(req);
    }

    public Call<BaseResponse<AddHandleResponse>> addHandle(AddHandleRequest req) {
        return IRetrofitApi.addHandle(req);
    }

    public Call<BaseResponse<Object>> removeHandle(RemoveHandleRequest req) {
        return IRetrofitApi.removeHandle(req);
    }

    public Call<BaseResponse<AddHandleResponse>> updateHandle(UpdateHandleRequest req) {
        return IRetrofitApi.updateHandle(req);
    }

    public Call<BaseResponse<Object>> deleteAccount(BaseRequest req) {
        return IRetrofitApi.deleteAccount(req);
    }

    public Call<BaseResponse<List<Job>>> searchJobs(BaseRequest req) {
        return IRetrofitApi.searchJobs(req);
    }

    public Call<BaseResponse<List<SavedJob>>> getSavedJobs(BaseRequest req) {
        return IRetrofitApi.getSavedJobs(req);
    }

    public Call<BaseResponse<AddSavedJobResponse>> addSavedJob(AddSavedJobRequest req) {
        return IRetrofitApi.addSavedJob(req);
    }

    public Call<BaseResponse<Object>> deleteSavedJob(AddSavedJobResponse req) {
        return IRetrofitApi.deleteSavedJob(req);
    }

    public Call<BaseResponse<List<Role>>> searchSkills(SearchSkillsRequest req) {
        return IRetrofitApi.searchSkills(req);
    }

    //API Interface
    private interface IRetrofitApi {
        //region LOGIN
        @Headers({"UseMockdata: " + useMockDataSignUp, "MockdataJsonFile: signup-response"})
        @POST("user/vet/signup")
        Call<BaseResponse<TokenResponse>> signup(@Body SignUpRequest signUpRequest);

        @Headers({"UseMockdata: " + useMockDataSignIn, "MockdataJsonFile: signin-response"})
        @POST("user/vet/signin")
        Call<BaseResponse<SignInResponse>> signin(@Body SignInRequest signInRequest);

        @Headers({"UseMockdata: " + useMockDataExists, "MockdataJsonFile: exists-response"})
        @POST("user/vet/exist")
        Call<BaseResponse<ExistsResponse>> exists(@Body ExistsRequest existsRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockDataDeleteAccount, "MockdataJsonFile: empty-response"})
        @POST("user/vet/deleteAccount")
        Call<BaseResponse<Object>> deleteAccount(@Body BaseRequest deleteAccountRequest);
        //endregion

        //region DASHBOARD
        @Headers({"Authorization: true", "UseMockdata: " + useMockDataGetNotifications, "MockdataJsonFile: getNotifications-response"})
        @POST("user/vet/getNotifications")
        Call<BaseResponse<GetNotificationsResponse>> getNotifications(@Body BaseRequest baseRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockResolveNotifications, "MockdataJsonFile: empty-response"})
        @POST("user/vet/resolveNotification")
        Call<BaseResponse<Object>> resolveNotifications(@Body ResolveNotificationRequest resolveNotificationRequest);
        //endregion

        //region USER-PROFILE
        @Headers({"Authorization: true", "UseMockdata: " + useMockGetUser, "MockdataJsonFile: getUser-response"})
        @POST("user/vet/getUser")
        Call<BaseResponse<GetUserResponse>> getUser(@Body BaseRequest baseRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockUpdateProfile, "MockdataJsonFile: updateProfile-response"})
        @POST("user/vet/updateProfile")
        Call<BaseResponse<TokenResponse>> updateProfile(@Body User user);

        @Headers({"Authorization: true", "UseMockdata: " + useMockUpdatePassword, "MockdataJsonFile: updatePassword-response"})
        @POST("user/vet/updatePassword")
        Call<BaseResponse<TokenResponse>> updatePassword(@Body UpdatePasswordRequest updatePasswordRequest);

        @Headers({"UseMockdata: " + useMockResetPassword, "MockdataJsonFile: signin-response"})
        @POST("user/vet/resetPassword")
        Call<BaseResponse<SignInResponse>> resetPassword(@Body ResetPasswordRequest resetPasswordRequest);

        @Headers({"UseMockdata: " + useMockForgotPassword, "MockdataJsonFile: forgotPassword-response"})
        @POST("user/vet/forgotPassword")
        Call<BaseResponse<ForgotPasswordResponse>> forgotPassword(@Body ForgotPasswordRequest forgotPasswordRequest);
        //endregion

        //region SOCIAL-MEDIA
        @Headers({"Authorization: true", "UseMockdata: " + useMockGetHandles, "MockdataJsonFile: getHandles-response"})
        @POST("social/handles/getHandles")
        Call<BaseResponse<GetHandlesResponse>> getHandles(@Body BaseRequest baseRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockAddHandle, "MockdataJsonFile: addHandle-response"})
        @POST("social/handles/addHandle")
        Call<BaseResponse<AddHandleResponse>> addHandle(@Body AddHandleRequest request);

        @Headers({"Authorization: true", "UseMockdata: " + useMockRemoveHandle, "MockdataJsonFile: empty-response"})
        @POST("social/handles/removeHandle")
        Call<BaseResponse<Object>> removeHandle(@Body RemoveHandleRequest request);

        @Headers({"Authorization: true", "UseMockdata: " + useMockUpdateHandle, "MockdataJsonFile: addHandle-response"})
        @POST("social/handles/updateHandle")
        Call<BaseResponse<AddHandleResponse>> updateHandle(@Body UpdateHandleRequest request);
        //endregion

        //region SURVEY
        @Headers({"Authorization: true", "UseMockdata: " + useMockGetQuestionSet, "MockdataJsonFile: getQuestionSet-response"})
        @POST("survey/question/getQuestionSet")
        Call<BaseResponse<QuestionSet>> getQuestionSet(@Body GetQuestionSetRequest getQuestionSet);

        @Headers({"Authorization: true", "UseMockdata: " + useMockSubmitSurvey, "MockdataJsonFile: submitSurvey-response"})
        @POST("survey/answer/submitSurvey")
        Call<BaseResponse<SubmitSurveyResponse>> submitSurvey(@Body SubmitSurveyRequest submitSurveyRequest);
        //endregion

        //region EMPLOYMENT
        @Headers({"Authorization: true", "UseMockdata: " + useMockUploadResume, "MockdataJsonFile: uploadResume-response"})
        @POST("job/resume/uploadResume")
        Call<BaseResponse<UploadResumeResponse>> uploadResume(@Body UploadResumeRequest uploadResumeRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockUploadComplete, "MockdataJsonFile: uploadComplete-response"})
        @POST("job/resume/uploadComplete")
        Call<BaseResponse<UploadCompleteResponse>> uploadComplete(@Body BaseRequest uploadCompleteRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockGetTargetIndustries, "MockdataJsonFile: getTargetIndustries-response"})
        @POST("job/resume/getTargetIndustries")
        Call<BaseResponse<List<String>>> getTargetIndustries(@Body BaseRequest baseRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockGetEmploymentInfo, "MockdataJsonFile: getEmploymentInfo-response"})
        @POST("job/resume/getResumeInfo")
        Call<BaseResponse<EmploymentInfo>> getEmploymentInfo(@Body BaseRequest baseRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockUploadEmploymentInfo, "MockdataJsonFile: uploadResumeInfo-response"})
        @POST("job/resume/uploadResumeInfo")
        Call<BaseResponse<UploadResumeInfoResponse>> uploadEmploymentInfo(@Body UploadResumeInfoRequest uploadResumeInfoRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockSearchJobs, "MockdataJsonFile: searchJobs-response"})
        @POST("job/jobs/searchJobs")
        Call<BaseResponse<List<Job>>> searchJobs(@Body BaseRequest baseRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockGetSavedJobs, "MockdataJsonFile: getSavedJobs-response"})
        @POST("job/jobs/getSavedJobs")
        Call<BaseResponse<List<SavedJob>>> getSavedJobs(@Body BaseRequest baseRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockAddSavedJob, "MockdataJsonFile: addSavedJob-response"})
        @POST("job/jobs/addSavedJob")
        Call<BaseResponse<AddSavedJobResponse>> addSavedJob(@Body AddSavedJobRequest baseRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockDeleteSavedJob, "MockdataJsonFile: deleteSavedJob-response"})
        @POST("job/jobs/deleteSavedJob")
        Call<BaseResponse<Object>> deleteSavedJob(@Body AddSavedJobResponse baseRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockSearchSkills, "MockdataJsonFile: searchSkills-response"})
        @POST("job/skills/searchSkills")
        Call<BaseResponse<List<Role>>> searchSkills(@Body SearchSkillsRequest searchSkillsRequest);
        //endregion

        //region DIGITAL-ASSISTANT
        @Headers({"Authorization: true", "UseMockdata: " + useMockStartConversation, "MockdataJsonFile: startConversation-response"})
        @POST("chat/da/startConversation")
        Call<BaseResponse<ChatMessage>> startConversation(@Body BaseRequest baseRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockConverse, "MockdataJsonFile: converse-response"})
        @POST("chat/da/converse")
        Call<BaseResponse<ChatMessage>> converse(@Body ConverseRequest converseRequest);
        //endregion

        //region DIGITAL-SELF
        @Headers({"UseMockdata: " + useMockGetBadgeTypes, "MockdataJsonFile: getBadgeTypes-response"})
        @POST("user/badges/getBadgeTypes")
        Call<BaseResponse<GetBadgeTypesResponse>> getBadgeTypes(@Body GetBadgeTypesRequest resolveNotificationRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockGetAwardedBadges, "MockdataJsonFile: getAwardedBadges-response"})
        @POST("user/vet/getAwardedBadges")
        Call<BaseResponse<GetAwardedBadgesResponse>> getAwardedBadges(@Body BaseRequest getAwardedBadgesRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockDigitalJournalMoodScores, "MockdataJsonFile: getMoodScores-response"})
        @POST("user/journal/getJournalMoodScores")
        Call<BaseResponse<GetMoodScoresResponse>> getMoodScores(@Body BaseRequest getMoodScoresRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockDigitalJournalEvents, "MockdataJsonFile: getJournalEvents-response"})
        @POST("user/journal/getJournalEvents")
        Call<BaseResponse<GetJournalEventsResponse>> getJournalEvents(@Body GetJournalEventsRequest getJournalEventsRequest);
        //endregion

        //region SQUAD
        @Headers({"Authorization: true", "UseMockdata: " + useMockGetSquad, "MockdataJsonFile: getSquad-response"})
        @POST("squad/mates/getAllMates")
        Call<BaseResponse<GetSquadResponse>> getSquad(@Body BaseRequest baseRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockAddSquad, "MockdataJsonFile: addSquad-response"})
        @POST("squad/mates/addMate")
        Call<BaseResponse<AddSquadResponse>> addSquad(@Body AddSquadRequest addSquadRequest);

        @Headers({"Authorization: true", "UseMockdata: " + useMockRemoveSquad, "MockdataJsonFile: empty-response"})
        @POST("squad/mates/removeMate")
        Call<BaseResponse<Object>> removeSquad(@Body RemoveSquadRequest removeSquadRequest);
        //endregion
    }
}

