package com.ibm.gbs.va.grit.room;

import android.arch.lifecycle.LiveData;
import android.arch.persistence.room.Dao;
import android.arch.persistence.room.Delete;
import android.arch.persistence.room.Embedded;
import android.arch.persistence.room.Insert;
import android.arch.persistence.room.Query;
import android.arch.persistence.room.Transaction;
import android.arch.persistence.room.Update;
import android.support.annotation.NonNull;

import com.google.gson.Gson;
import com.ibm.gbs.va.grit.model.survey.Answer;
import com.ibm.gbs.va.grit.model.survey.Question;

import java.util.List;

@Dao
public abstract class AnswerDao {

    @Query("SELECT * FROM answer WHERE user_email LIKE :userEmail AND question_id LIKE :questionId")
    abstract Answer getAnswerNow(String userEmail, String questionId);

    @Insert
    abstract void insertAnswer(Answer answer);

    @Update
    abstract void updateAnswer(Answer answer);


    @Transaction
    public void saveAnswer(Answer answer) {
        if (getAnswerNow(answer.getUserEmail(), answer.getQuestionId()) != null) {
            updateAnswer(answer);
        } else {
            insertAnswer(answer);
        }
    }

    @Query("SELECT * FROM answer WHERE user_email LIKE :userEmail")
    public abstract List<Answer> getAnswersForUserNow(String userEmail);

    @Query("SELECT Question.*, Answer.* From user " +
            "INNER JOIN question_set ON user.question_set_id = question_set.id " +
            "INNER JOIN question ON question.question_set_id = question_set.id " +
            "LEFT JOIN answer ON answer.question_id = question.id AND answer.user_email = user.email " +
            "WHERE user.email LIKE :userEmail")
    public abstract LiveData<List<QuestionAnswerJoin>> getQuestionsAndAnswersForUser(String userEmail);

    @Transaction
    public void setQuestionsAndAnswersForUser(List<QuestionAnswerJoin> set, String userEmail) {
        if (set == null) return;
        for (QuestionAnswerJoin item : set) {
            if (item.getAnswer() == null) {
                item.setAnswer(new Answer(item.getQuestion(), userEmail));
            } else {
                item.getAnswer().setUserEmail(userEmail);
            }
            saveAnswer(item.answer);
        }
    }

    @Delete
    public abstract void deleteAnswers(List<Answer> answers);

    @Transaction
    public void purgeUserAnswers(String userEmail) {
        deleteAnswers(getAnswersForUserNow(userEmail));
    }

    public class QuestionAnswerJoin {
        @Embedded
        private Question question;

        @Embedded
        private Answer answer;

        QuestionAnswerJoin(Question question, Answer answer) {
            this.question = question;
            this.answer = answer;
        }

        public Question getQuestion() {
            return question;
        }

        public void setQuestion(Question question) {
            this.question = question;
        }

        public Answer getAnswer() {
            return answer;
        }

        void setAnswer(Answer answer) {
            this.answer = answer;
        }

        @NonNull
        @Override
        public String toString() {
            return new Gson().toJson(this);
        }
    }
}
