package com.ibm.gbs.va.grit.room;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.Transformations;
import android.arch.persistence.room.Dao;
import android.arch.persistence.room.Delete;
import android.arch.persistence.room.Insert;
import android.arch.persistence.room.Query;
import android.arch.persistence.room.Transaction;

import com.ibm.gbs.va.grit.model.survey.Question;
import com.ibm.gbs.va.grit.model.survey.QuestionSet;
import com.ibm.gbs.va.grit.model.type.User;

import java.util.List;

@Dao
public abstract class QSetDao {
    private final UserDao userDao;

    protected QSetDao(GritDatabase db) {
        this.userDao = db.userDao();
    }

    @Query("SELECT * FROM question_set WHERE id LIKE :questionSetId")
    abstract LiveData<QuestionSet> _getQuestionSet(String questionSetId);

    @Query("SELECT * FROM question_set WHERE id LIKE :questionSetId")
    abstract QuestionSet _getQuestionSetNow(String questionSetId);

    @Query("SELECT * FROM question WHERE id LIKE :questionId")
    abstract Question getQuestionNow(String questionId);

    @Query("SELECT * FROM question_set")
    public abstract LiveData<List<QuestionSet>> getQuestionSets();

    @Query("SELECT * FROM question WHERE question_set_id LIKE :questionSetId")
    abstract LiveData<List<Question>> getQuestionsForSet(String questionSetId);

    @Query("SELECT * FROM question WHERE question_set_id LIKE :questionSetId")
    abstract List<Question> getQuestionsForSetNow(String questionSetId);

    private LiveData<QuestionSet> getQuestionSet(String qSetId) {
        LiveData<QuestionSet> qSetLiveData = _getQuestionSet(qSetId);

        qSetLiveData = Transformations.switchMap(qSetLiveData, inputQSet -> {
            LiveData<List<Question>> questionListLiveData = getQuestionsForSet(qSetId);

            return Transformations.map(questionListLiveData, inputQuestions -> {
                if (inputQSet != null)
                    inputQSet.setQuestions(inputQuestions);
                return inputQSet;
            });
        });

        return qSetLiveData;
    }

    @Transaction
    public QuestionSet getQuestionSetNow(String questionSetId) {
        List<Question> qList = getQuestionsForSetNow(questionSetId);
        QuestionSet set = _getQuestionSetNow(questionSetId);
        if (set == null) return null;
        set.setQuestions(qList);
        return set;
    }

    @Transaction
    public void removeQuestionSet(QuestionSet set) {
        if (getUserCountForQSetNow(set.getQuestionSetId()) == 0) {
            deleteQuestions(set.getQuestions());
            deleteQuestionSet(set);
        }
    }

    @Query("SELECT COUNT(*) FROM user WHERE question_set_id LIKE :qSetId")
    abstract int getUserCountForQSetNow(String qSetId);

    @Transaction
    public LiveData<QuestionSet> getQSetForUser(String userEmail) {
        return Transformations.switchMap(userDao.getUser(userEmail), user -> getQuestionSet(user.getQuestionSetId()));
    }

    @Transaction
    public QuestionSet getQSetForUserNow(String userEmail) {
        User user = userDao.getUserNow(userEmail);
        if (user == null) return null;
        return getQuestionSetNow(user.getQuestionSetId());
    }

    @Insert
    abstract void _saveQuestionSet(QuestionSet questionSet);

    @Insert
    abstract void _saveQuestion(Question question);

    @Transaction
    public void saveQuestionSet(QuestionSet questionSet) {
        if (getQuestionSetNow(questionSet.getQuestionSetId()) != null) return;

        _saveQuestionSet(questionSet);
        String setId = questionSet.getQuestionSetId();
        for (Question q : questionSet.getQuestions()) {
            if (getQuestionNow(q.getQuestionId()) == null) {
                q.setQuestionSetId(setId);
                _saveQuestion(q);
            }
        }
    }

    @Delete
    abstract void deleteQuestionSet(QuestionSet questionSet);

    @Delete
    abstract void deleteQuestions(List<Question> question);
}
