package com.ibm.gbs.va.grit.room;

import android.arch.lifecycle.LiveData;
import android.arch.persistence.room.Dao;
import android.arch.persistence.room.Delete;
import android.arch.persistence.room.Insert;
import android.arch.persistence.room.Query;
import android.arch.persistence.room.Transaction;
import android.arch.persistence.room.Update;

import com.ibm.gbs.va.grit.model.type.SquadMember;
import com.ibm.gbs.va.grit.util.RemoteLog;

import java.util.List;

@Dao
public abstract class SquadDao {
    private static final String TAG = "SquadDao";

    @Insert
    public abstract void insertSquadMember(SquadMember member);

    @Update
    public abstract void updateSquadMember(SquadMember member);

    @Delete
    public abstract void deleteSquadMember(SquadMember member);

    @Delete
    abstract void deleteSquadMembers(List<SquadMember> members);

    @Transaction
    public void purgeUserSquadMembers(String userEmail) {
        deleteSquadMembers(getSquadForUserNow(userEmail));
    }

    /**
     * Deletes squad members that are not included in the new list from the DB
     *
     * @param squad     the squad to save
     * @param userEmail user to save the squad to
     */
    @Transaction
    public void saveSquadForUser(List<SquadMember> squad, String userEmail) {
        List<SquadMember> dbSquad = getSquadForUserNow(userEmail);

        for (SquadMember member : squad) {
            member.setUserEmail(userEmail);
            int dbListIndex = dbSquad.indexOf(member);
            if (dbListIndex == -1) insertSquadMember(member);
            else {
                // retain transient field
                member.setProfileImage(dbSquad.get(dbListIndex).getProfileImage());
                updateSquadMember(member);
            }
        }

        // remove members that were not in the new list
        dbSquad.removeAll(squad);
        RemoteLog.d(TAG, "Removing un-backed squadMembers: " + dbSquad);
        deleteSquadMembers(dbSquad);
    }

    @Query("SELECT squad_member.* FROM squad_member INNER JOIN user ON squad_member.user_email == user.email WHERE user.email LIKE :userEmail")
    abstract List<SquadMember> getSquadForUserNow(String userEmail);

    @Query("SELECT squad_member.* FROM squad_member INNER JOIN user ON squad_member.user_email == user.email WHERE user.email LIKE :userEmail")
    public abstract LiveData<List<SquadMember>> getSquadForUser(String userEmail);
}
