package com.ibm.gbs.va.grit.room;

import android.arch.persistence.room.TypeConverter;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Base64;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.survey.NumericRange;
import com.ibm.gbs.va.grit.model.survey.OtherSelectOption;
import com.ibm.gbs.va.grit.model.survey.Question;
import com.ibm.gbs.va.grit.model.survey.QuestionSet;
import com.ibm.gbs.va.grit.model.survey.SelectOption;
import com.ibm.gbs.va.grit.model.type.EmploymentInfo;
import com.ibm.gbs.va.grit.model.type.MultiLangText;
import com.ibm.gbs.va.grit.model.type.MultiLangVariableText;
import com.ibm.gbs.va.grit.model.type.SquadMember;

import org.joda.time.DateTime;

import java.io.ByteArrayOutputStream;
import java.lang.reflect.Type;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

public class TypeConverters {

    private static final Gson gson = new Gson();

    @TypeConverter
    public static String questionTypeToString(Question.QuestionType data) {
        return data.value();
    }

    @TypeConverter
    public static Question.QuestionType questionTypeToString(String data) {
        return Question.QuestionType.valueOf(data);
    }

    // EmploymentInfo - start
    @TypeConverter
    @SuppressWarnings("unused")
    public static EmploymentInfo.MilitaryBranch strToMilitaryBranch(String data) {
        return EmploymentInfo.MilitaryBranch.fromValue(data);
    }

    @TypeConverter
    @SuppressWarnings("unused")
    public static String militaryBranchToStr(EmploymentInfo.MilitaryBranch branch) {
        return branch.value();
    }

    @TypeConverter
    @SuppressWarnings("unused")
    public static EmploymentInfo.Rank strToRank(String data) {
        return EmploymentInfo.Rank.fromValue(data);
    }

    @TypeConverter
    @SuppressWarnings("unused")
    public static String rankToStr(EmploymentInfo.Rank rank) {
        return rank.value();
    }

    @TypeConverter
    @SuppressWarnings("unused")
    public static List<EmploymentInfo.JobType> strToJobTypeList(String data) {
        if (data == null) return Collections.emptyList();

        Type listType = new TypeToken<List<EmploymentInfo.JobType>>() {
        }.getType();

        return gson.fromJson(data, listType);
    }

    @TypeConverter
    @SuppressWarnings("unused")
    public static String jobTypeListToStr(List<EmploymentInfo.JobType> data) {
        return gson.toJson(data);
    }
    // EmploymentInfo - end

    // Question - start
    @TypeConverter
    public static List<MultiLangVariableText> strToMultiLangVarTextList(String data) {
        if (data == null) return Collections.emptyList();

        Type listType = new TypeToken<List<MultiLangVariableText>>() {
        }.getType();

        return gson.fromJson(data, listType);
    }

    @TypeConverter
    public static String multiLangVarTextListToStr(List<MultiLangVariableText> data) {
        return gson.toJson(data);
    }

    @TypeConverter
    public static OtherSelectOption strToOtherSelectOption(String data) {

        Type listType = new TypeToken<OtherSelectOption>() {
        }.getType();

        return gson.fromJson(data, listType);
    }

    @TypeConverter
    public static String otherSelectOptionToStr(OtherSelectOption data) {
        return gson.toJson(data);
    }

    @TypeConverter
    public static List<SelectOption> strToSelectOptionList(String data) {
        if (data == null) return Collections.emptyList();

        Type listType = new TypeToken<List<SelectOption>>() {
        }.getType();

        return gson.fromJson(data, listType);
    }

    @TypeConverter
    public static String selectOptionListToStr(List<SelectOption> data) {
        return gson.toJson(data);
    }

    @TypeConverter
    public static NumericRange strToNumericRange(String data) {
        Type listType = new TypeToken<NumericRange>() {
        }.getType();

        return gson.fromJson(data, listType);
    }

    @TypeConverter
    public static String NumericRangeToStr(NumericRange data) {
        return gson.toJson(data);
    }

    @TypeConverter
    public static QuestionSet.Type strToQuestionSetType(String data) {
        return QuestionSet.Type.fromValue(data);
    }

    @TypeConverter
    public static String questionSetTypeToStr(QuestionSet.Type type) {
        return type.value();
    }
    // Question - end

    // Notification - start
    @TypeConverter
    public static Notification.Type strToNotificationType(String data) {
        return Notification.Type.fromValue(data);
    }

    @TypeConverter
    public static String notificationTypeToStr(Notification.Type type) {
        return type.value();
    }

    @TypeConverter
    public static Date strToDate(String data) {
        return new DateTime(data).toDate();
    }

    @TypeConverter
    public static String dateToStr(Date date) {
        return new DateTime(date).toString();
    }
    // Notifications - end

    // Answer - start
    @TypeConverter
    public static LinkedList<String> strToLinkedStrList(String data) {
        if (data == null) return new LinkedList<>();

        Type listType = new TypeToken<LinkedList<String>>() {
        }.getType();

        return gson.fromJson(data, listType);
    }

    @TypeConverter
    public static String LinkedStrListToStr(LinkedList<String> data) {
        return gson.toJson(data);
    }
    // Answer - end

    //Badge - start
    @TypeConverter
    public static String multiLangTextToStr(MultiLangText data) {
        return gson.toJson(data);
    }

    @TypeConverter
    public static MultiLangText strToMultiLangText(String data) {
        Type listType = new TypeToken<MultiLangText>() {
        }.getType();

        return gson.fromJson(data, listType);
    }
    //Badge - end

    //Squad - start
    @TypeConverter
    public static SquadMember.State strToSquadMemberState(String data) {
        return SquadMember.State.fromValue(data);
    }

    @TypeConverter
    public static String squadMemberStateToStr(SquadMember.State state) {
        return state.value();
    }

    @TypeConverter
    public static Bitmap strToBitmap(String data) {
        if (data == null) return null;
        byte[] decodedString = Base64.decode(data, Base64.DEFAULT);
        return BitmapFactory.decodeByteArray(decodedString, 0, decodedString.length);
    }

    @TypeConverter
    public static String bitmapToString(Bitmap data) {
        if (data == null) return null;
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        data.compress(Bitmap.CompressFormat.PNG, 100, outputStream);
        byte[] b = outputStream.toByteArray();
        return Base64.encodeToString(b, Base64.DEFAULT);
    }
    //Squad - end
}