package com.ibm.gbs.va.grit.room;

import android.arch.lifecycle.LiveData;
import android.arch.persistence.room.Dao;
import android.arch.persistence.room.Delete;
import android.arch.persistence.room.Insert;
import android.arch.persistence.room.Query;
import android.arch.persistence.room.Transaction;
import android.arch.persistence.room.Update;

import com.ibm.gbs.va.grit.model.type.User;

@Dao
public abstract class UserDao {
    @Query("SELECT * FROM user WHERE email LIKE :userEmail")
    public abstract LiveData<User> getUser(String userEmail);

    @Query("SELECT * FROM user WHERE email LIKE :userEmail")
    public abstract User getUserNow(String userEmail);

    @Query("SELECT * FROM user WHERE primary_key LIKE :key")
    public abstract User getUserNow(int key);

    @Update
    abstract void updateUser(User user);

    @Insert
    public abstract void insertUser(User user);

    @Transaction
    public void stubUser(String email) {
        User user = getUserNow(email);
        if (user == null) {
            user = new User();
            user.setEmail(email);
            insertUser(user);
        }
    }

    @Transaction
    public void setQSetForUser(String email, String qSetId, Long notificationId) {
        User user = getUserNow(email);
        if (user == null) {
            user = new User();
            user.setEmail(email);
            insertUser(user);
        }
        user.setQuestionSetId(qSetId);
        user.setSurveySpawningNotificationId(notificationId);
        updateUser(user);
    }

    @Transaction
    public void setSurveySpawningNotificationForUser(String email, Long notificationId) {
        User user = getUserNow(email);
        if (user == null) {
            user = new User();
            user.setEmail(email);
            insertUser(user);
        }
        user.setSurveySpawningNotificationId(notificationId);
        updateUser(user);
    }

    @Transaction
    public void saveUserPersonalInfo(User user) {
        User dbUser = getUserNow(user.getEmail());
        if (dbUser != null) {
            // copy over transient fields
            user.setPrimaryKey(dbUser.getPrimaryKey());
            user.setToken(dbUser.getToken());
            user.setQuestionSetId(dbUser.getQuestionSetId());
            user.setSurveySpawningNotificationId(dbUser.getSurveySpawningNotificationId());
            updateUser(user);
            return;
        }
        dbUser = getUserNow(user.getPrimaryKey());
        if (dbUser != null) {
            // copy over primary key
            user.setPrimaryKey(dbUser.getPrimaryKey());
            updateUser(user);
        } else {
            insertUser(user);
        }
    }

    @Transaction
    public void saveUserToken(String email, String token) {
        User user = getUserNow(email);
        if (user == null) {
            insertUser(new User(email, token));
        } else {
            user.setToken(token);
            updateUser(user);
        }
    }

    @Delete
    public abstract void deleteUser(User user);

    @Transaction
    public void purgeUser(String userEmail) {
        deleteUser(getUserNow(userEmail));
    }
}
