package com.ibm.gbs.va.grit.util;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.google.firebase.analytics.FirebaseAnalytics;

import javax.inject.Inject;
import javax.inject.Singleton;

@Singleton
public final class Analytics {

    private final FirebaseAnalytics instance;

    @Inject
    public Analytics(Context context) {
        instance = FirebaseAnalytics.getInstance(context.getApplicationContext());
    }

    // ////////////////////////////////
    // METHODS ------------------------

    /**
     * Sets the current screen name, which specifies the current visual context in your app. This
     * helps identify the areas in your app where users spend their time and how they interact with
     * your app.
     * <p>
     * Note: that screen reporting is enabled automatically and records the class name of the
     * current Activity for you without requiring you to call this function. The class name can
     * optionally be overridden by calling this function in the onResume callback of your Activity
     * and specifying the screenClassOverride parameter.
     * <p>
     * If your app does not use a distinct Activity for each screen, you should call this function
     * and specify a distinct screenName each time a new screen is presented to the user.
     * <p>
     * The name and classOverride remain in effect until the current Activity changes or a new call
     * to setCurrentScreen is made.
     * <p>
     * This method must be called from the app's main thread.
     *
     * @param activity   The activity to which the screen name and class name apply.
     * @param screenName The name of the current screen. Set to null to clear the current screen
     *                   name.
     */
    public void setCurrentScreen(@NonNull Activity activity, @Nullable String screenName) {
        RemoteLog.setCurrentScreen((screenName != null) ? screenName : activity.getClass().getSimpleName());
        instance.setCurrentScreen(activity, screenName, null);
    }

    /**
     * @param activity The activity to which the screen name and class name apply.
     * @see com.ibm.gbs.va.grit.util.Analytics#setCurrentScreen
     */
    public void setCurrentScreen(@NonNull Activity activity) {
        setCurrentScreen(activity, null);
    }

    /**
     * @param isFirstTime true when viewing the walkthrough for the very first time
     */
    public void walkthroughBegan(@NonNull Boolean isFirstTime) {
        Bundle bundle = new Bundle();
        bundle.putBoolean("is_first_time", isFirstTime);
        instance.logEvent(FirebaseAnalytics.Event.TUTORIAL_BEGIN, bundle);
    }

    /**
     * @param skipped         if the user skipped part of the walkthrough
     * @param exitedPageTitle the title of the page they skipped on
     */
    public void walkthroughCompleted(@NonNull Boolean skipped, @Nullable String exitedPageTitle) {
        Bundle bundle = new Bundle();
        bundle.putBoolean(Param.exitedEarly, skipped);
        if (skipped && exitedPageTitle != null)
            bundle.putString(Param.exitedPageTitle, exitedPageTitle);
        instance.logEvent(FirebaseAnalytics.Event.TUTORIAL_COMPLETE, bundle);
    }

    /**
     * Use this override when the user exited the walkthrough normally
     *
     * @see com.ibm.gbs.va.grit.util.Analytics#walkthroughCompleted
     */
    public void walkthroughCompleted() {
        walkthroughCompleted(false, null);
    }

    // ////////////////////////////////
    // CONSTANTS ----------------------
    interface Param {
        String exitedEarly = "exited_early";
        String exitedPageTitle = "exited_page";
    }
}
