package com.ibm.gbs.va.grit.util;

import android.content.Context;
import android.content.res.Resources;
import android.databinding.BindingAdapter;
import android.databinding.InverseBindingAdapter;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.support.annotation.ColorRes;
import android.support.annotation.DrawableRes;
import android.support.constraint.ConstraintLayout;
import android.support.design.button.MaterialButton;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.style.BulletSpan;
import android.util.DisplayMetrics;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.TextView;

import com.ibm.gbs.va.grit.R;
import com.ibm.gbs.va.grit.model.dialog.BadgeDialogModel;
import com.ibm.gbs.va.grit.model.dialog.ButtonConfig;
import com.ibm.gbs.va.grit.model.dialog.DialogModel;
import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.model.type.BadgeListBadge;
import com.ibm.gbs.va.grit.model.type.DigitalJournalEvent;
import com.ibm.gbs.va.grit.model.type.SquadMember;
import com.ibm.gbs.va.grit.view.GritSpinner;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

public class BindingAdapters {

    @BindingAdapter("piv_selectedColor")
    public static void setSelectedColor(com.rd.PageIndicatorView indicator, @ColorRes Integer color) {
        indicator.setSelectedColor(color);
    }

    @BindingAdapter("piv_unselectedColor")
    public static void setUnselectedColor(com.rd.PageIndicatorView indicator, @ColorRes Integer color) {
        indicator.setUnselectedColor(color);
    }

    @BindingAdapter("srcCompat")
    public static void setImageResource(ImageView view, @DrawableRes int resource) {
        view.setImageResource(resource);
    }

    @BindingAdapter("boldText")
    public static void setBold(TextView textView, boolean isBold) {
        textView.setTextAppearance(isBold ? R.style.Title2 : R.style.Body_Secondary);
    }

    @BindingAdapter("android:checked")
    public static void setChecked(CompoundButton checkableView, Boolean isChecked) {
        checkableView.setChecked(isChecked != null ? isChecked : false);
    }

    @BindingAdapter("android:progress")
    public static void setProgress(SeekBar bar, Integer val) {
        bar.setProgress(val != null ? val : 0);
    }

    @BindingAdapter(value = {"android:progress", "android:max"})
    public static void setProgress(SeekBar bar, int progressVal, int progressMax) {
        bar.setMax(progressMax);
        bar.setProgress(progressVal);
    }

    @BindingAdapter("dialogTitleSetup")
    public static void setupTitle(TextView view, DialogModel model) {
        if (model.getTitleRes() != null) {
            view.setText(model.getTitleRes());
        } else {
            view.setText(model.getTitle());
        }
    }

    @BindingAdapter("dialogTitleSetup")
    public static void setupTitle(TextView view, BadgeDialogModel model) {
        view.setText(model.getTitle());
    }

    @BindingAdapter("dialogBtnSetup")
    public static void setupBtn(MaterialButton btn, ButtonConfig setup) {
        Context ctx = btn.getContext();
        if (setup == null) {
            btn.setVisibility(View.GONE);
        } else {
            btn.setVisibility(View.VISIBLE);
            btn.setText(setup.text);
            btn.setTextColor(ctx.getColor(setup.textColor));
            Drawable background = btn.getBackground();
            background.setTint(ctx.getColor(setup.buttonColor));
            if (setup.strokeColor != null) {
                btn.setStrokeColorResource(setup.strokeColor);
            }
            if (setup.strokeWidth != null) {
                btn.setStrokeWidthResource(setup.strokeWidth);
            }
            btn.setOnClickListener(setup.getOnClickListener());
        }
    }

    @BindingAdapter("textSectionSetup")
    public static void setupBody(LinearLayout layout, List<TextSection> sections) {
        Context ctx = layout.getContext();
        Resources res = layout.getResources();
        DisplayMetrics metrics = res.getDisplayMetrics();

        LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
        for (TextSection section : sections) {
            TextViewWithImages text = new TextViewWithImages(ctx);
            text.setLayoutParams(params);
            text.setTextAppearance(section.textAppearance);
            if (section.centered) text.setGravity(Gravity.CENTER); // center text if needed

            if (section.bulletedText != null) { // create bullet
                params.bottomMargin = ctx.getResources().getDimensionPixelSize(R.dimen.bulleted_list_bottom_margin);
                Spannable spannable = new SpannableString(ctx.getString(section.bulletedText));
                spannable.setSpan(new BulletSpan(ctx.getResources().getDimensionPixelSize(R.dimen.bulleted_list_leading_margin)), 0, spannable.length(), Spanned.SPAN_INCLUSIVE_EXCLUSIVE);
                text.setText(spannable);
            } else if (section.dynamicText != null) {
                text.setText(section.dynamicText);
            } else if (section.simpleText != null)
                text.setText(section.simpleText);

            /*
             * TextViewWithImages extends AppCompatTextView which for whatever reason locks off
             * setting the style (not textAppearance) of the TextView, so line height has to be
             * handled like this here
             */
            float textSize = text.getTextSize();
            if (textSize == //12sp
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, 12, metrics)) {
                text.setLineHeight(res.getDimensionPixelSize(R.dimen.small_text_line_height));
            } else if (textSize == //14sp
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, 14, metrics)) {
                text.setLineHeight(res.getDimensionPixelSize(R.dimen.caption_text_line_height));
            } else if (textSize == //16sp
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, 16, metrics)) {
                text.setLineHeight(res.getDimensionPixelSize(R.dimen.body_text_line_height));
            }
            layout.addView(text);
        }
    }

    @BindingAdapter("android:text")
    public static void setText(GritSpinner view, Integer value) {
        view.setText(String.format(Locale.US, "%d", value));
    }

    @InverseBindingAdapter(attribute = "android:text")
    public static Integer getText(GritSpinner view) {
        return Integer.parseInt(view.getText().toString());
    }

    @BindingAdapter("setSquadStatus")
    public static void setSquadStatus(TextView view, SquadMember member) {
        if (member.getState() == SquadMember.State.ACCEPTED) {
            view.setText(R.string.squad_status_response_active);
            view.setTextAppearance(R.style.Caption_Success);
        } else {
            view.setText(R.string.squad_status_response_pending);
            view.setTextAppearance(R.style.Caption_Secondary);
        }
    }

    @BindingAdapter("setSquadStatus")
    public static void setSquadStatusImage(ImageView view, SquadMember member) {
        if (member.getState() == SquadMember.State.ACCEPTED) {
            view.setImageResource(R.drawable.ic_circled_checkmark);
        } else {
            view.setImageResource(R.drawable.ic_pending);
        }
    }

    @BindingAdapter("srcCompat")
    public static void setImageDrawable(ImageView view, Drawable drawable) {
        view.setImageDrawable(drawable);
    }

    @BindingAdapter("journalEventsSetup")
    public static void setupEvents(ConstraintLayout layout, List<DigitalJournalEvent> digitalJournalEvents) {
        Context ctx = layout.getContext();
        layout.removeAllViewsInLayout();


        final int iconSize = (int) ctx.getResources().getDimension(R.dimen.ds_journal_icon_dimensions);
        final int topMargin = (int) ctx.getResources().getDimension(R.dimen.ds_journal_events_top_margin);
        final int textStartMargin = (int) ctx.getResources().getDimension(R.dimen.ds_journal_events_text_start_margin);

        int previousSectionId = -1;
        if (digitalJournalEvents != null) {
            Collections.sort(digitalJournalEvents);
            //if order matters, assign a weight to eventType and sort the collection by it before processing
            for (DigitalJournalEvent event : digitalJournalEvents) {
                ConstraintLayout.LayoutParams eventTextParams = new ConstraintLayout.LayoutParams(ConstraintLayout.LayoutParams.WRAP_CONTENT, ConstraintLayout.LayoutParams.WRAP_CONTENT);
                ConstraintLayout.LayoutParams countTextParams = new ConstraintLayout.LayoutParams(ConstraintLayout.LayoutParams.WRAP_CONTENT, ConstraintLayout.LayoutParams.WRAP_CONTENT);
                ConstraintLayout.LayoutParams iconImageParams = new ConstraintLayout.LayoutParams(ConstraintLayout.LayoutParams.WRAP_CONTENT, ConstraintLayout.LayoutParams.WRAP_CONTENT);


                if (previousSectionId == -1) {
                    eventTextParams.topToTop = ConstraintLayout.LayoutParams.PARENT_ID;
                    countTextParams.topToTop = ConstraintLayout.LayoutParams.PARENT_ID;
                    iconImageParams.topToTop = ConstraintLayout.LayoutParams.PARENT_ID;
                } else {
                    eventTextParams.topToBottom = previousSectionId;
                    countTextParams.topToBottom = previousSectionId;
                    iconImageParams.topToBottom = previousSectionId;
                }

                ImageView currentIcon = new ImageView(ctx);
                currentIcon.setId(View.generateViewId());
                currentIcon.setImageResource(event.getEventType().getIconID());
                currentIcon.setScaleType(ImageView.ScaleType.FIT_XY);
                iconImageParams.width = iconSize;
                iconImageParams.height = iconSize;
                iconImageParams.startToStart = ConstraintLayout.LayoutParams.PARENT_ID;
                iconImageParams.topMargin = topMargin;
                currentIcon.setLayoutParams(iconImageParams);
                previousSectionId = currentIcon.getId();


                TextView currentEventText = new TextView(ctx, null, 0, R.style.BodyTextViewStyle);
                currentEventText.setId(View.generateViewId());
                currentEventText.setText(event.getEventType().getStringID());
                currentEventText.setTextAppearance(R.style.Body);
                eventTextParams.topMargin = topMargin;
                eventTextParams.setMarginStart(textStartMargin);
                eventTextParams.startToEnd = currentIcon.getId();
                currentEventText.setLayoutParams(eventTextParams);

                TextView currentCountText = new TextView(ctx, null, 0, R.style.SubheaderTextViewStyle);
                currentCountText.setId(View.generateViewId());
                currentCountText.setTextAppearance(R.style.Subheader);
                countTextParams.endToEnd = ConstraintLayout.LayoutParams.PARENT_ID;
                countTextParams.topMargin = topMargin;
                currentCountText.setText(String.valueOf(event.getEventCount()));
                currentCountText.setLayoutParams(countTextParams);

                layout.addView(currentIcon);
                layout.addView(currentEventText);
                layout.addView(currentCountText);
            }
        }
    }

    @BindingAdapter("listBadges")
    public static void listBadges(LinearLayout layout, HashMap<Badge.BadgeCategory, List<BadgeListBadge>> badgeCategoryListHashMap) {
        Context ctx = layout.getContext();
        layout.removeAllViewsInLayout();

        final int rowWidth = 4; // The number of badges per row;

        final int badgeSizeWidth = (int) ctx.getResources().getDimension(R.dimen.ds_badge_list_badge_width);
        final int badgeSizeHeight = (int) ctx.getResources().getDimension(R.dimen.ds_badge_list_badge_height);
        final int badgeMargin = (int) ctx.getResources().getDimension(R.dimen.ds_badge_list_badge_margin);


        if (badgeCategoryListHashMap != null) {
            ArrayList<Badge.BadgeCategory> keyList = new ArrayList<>(badgeCategoryListHashMap.keySet());
            Collections.sort(keyList);
            //if ordering matters, add weighting to badgeCategories, then sort the keySet according to the weight
            for (Badge.BadgeCategory currentBadgeCat : keyList) {
                List<BadgeListBadge> badgeListBadges = badgeCategoryListHashMap.get(currentBadgeCat);
                if (badgeListBadges.size() > 0) {

                    ConstraintLayout curLayout = new ConstraintLayout(ctx);
                    LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, ConstraintLayout.LayoutParams.WRAP_CONTENT);
                    // This makes the view width always the width of the row, giving the effect of the entire matrix being populated with empty values
                    layoutParams.width = (badgeSizeWidth * rowWidth) + (badgeMargin * (rowWidth + 1));
                    layoutParams.gravity = Gravity.CENTER_HORIZONTAL;
                    curLayout.setLayoutParams(layoutParams);

                    //define layout info here

                    int currentRowWidth = 0;
                    int previousRowSectionID = -1;
                    int currentRowSectionID = -1;
                    int previousColumnSectionID = -1;
                    if (badgeListBadges != null)
                        for (BadgeListBadge curBadgeListBadge : badgeListBadges) {

                            ConstraintLayout.LayoutParams badgeParams = new ConstraintLayout.LayoutParams(ConstraintLayout.LayoutParams.WRAP_CONTENT, ConstraintLayout.LayoutParams.WRAP_CONTENT);

                            ImageView currentBadgeView = new ImageView(ctx);
                            currentBadgeView.setId(View.generateViewId());
                            currentBadgeView.setImageDrawable(new BitmapDrawable(ctx.getResources(), curBadgeListBadge.getBadgeImage()));
                            currentBadgeView.setScaleType(ImageView.ScaleType.CENTER_INSIDE);
                            badgeParams.width = badgeSizeWidth;
                            badgeParams.height = badgeSizeHeight;
                            badgeParams.setMarginStart(badgeMargin);
                            badgeParams.topMargin = badgeMargin;
                            // set what happens when the badge is clicked;
                            currentBadgeView.setOnClickListener(v -> {
                                if (curBadgeListBadge.getFragmentManager() != null) {
                                    if (curBadgeListBadge.isAwarded())
                                        Badge.getDefaultBadgeBehaviour(curBadgeListBadge.getFragmentManager(), ctx.getResources(), null).accept(curBadgeListBadge.getCurrentBadge());
                                    else
                                        Badge.getUnawardedBadgeBehaviour(curBadgeListBadge.getFragmentManager(), ctx.getResources()).accept(curBadgeListBadge.getCurrentBadge());
                                }
                            });


                            if (previousColumnSectionID == -1) {
                                // initial column setup
                                currentRowWidth = 1;
                                previousColumnSectionID = currentBadgeView.getId();
                                badgeParams.startToStart = ConstraintLayout.LayoutParams.PARENT_ID;
                                previousRowSectionID = currentRowSectionID;
                                currentRowSectionID = currentBadgeView.getId();
                            } else {
                                //not initial column setup
                                if (currentRowWidth >= rowWidth) {
                                    // go to next row
                                    currentRowWidth = 1;
                                    previousRowSectionID = currentRowSectionID;
                                    currentRowSectionID = currentBadgeView.getId();
                                    badgeParams.startToStart = ConstraintLayout.LayoutParams.PARENT_ID;
                                    previousColumnSectionID = currentBadgeView.getId();
                                } else {
                                    // stay on current row
                                    currentRowWidth++;
                                    badgeParams.startToEnd = previousColumnSectionID;
                                    previousColumnSectionID = currentBadgeView.getId();
                                }
                            }

                            if (previousRowSectionID == -1) {
                                //initial row setup
                                badgeParams.topToTop = ConstraintLayout.LayoutParams.PARENT_ID;
                            } else {
                                //not initial row setup
                                badgeParams.topToBottom = previousRowSectionID;
                            }

                            currentBadgeView.setLayoutParams(badgeParams);
                            curLayout.addView(currentBadgeView);
                        }
                    TextView curCategoryText = new TextView(ctx, null, 0, R.style.SubheaderTextViewStyle);
                    curCategoryText.setText(ctx.getString(currentBadgeCat.getHumanCategoryNameResId()));
                    curCategoryText.setGravity(Gravity.CENTER_HORIZONTAL);
                    curCategoryText.setTextAppearance(R.style.Subheader);
                    LinearLayout.LayoutParams catTextLayoutParams = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, ConstraintLayout.LayoutParams.WRAP_CONTENT);
                    curCategoryText.setLayoutParams(catTextLayoutParams);

                    layout.addView(curCategoryText);
                    layout.addView(curLayout);
                }
            }
        }
    }

}
