package com.ibm.gbs.va.grit.util;

import android.content.res.Resources;
import android.databinding.InverseMethod;
import android.support.annotation.DrawableRes;
import android.support.graphics.drawable.VectorDrawableCompat;
import android.text.TextUtils;

import com.ibm.gbs.va.grit.model.type.EmploymentInfo;

import org.joda.time.DateTime;
import org.joda.time.Period;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.PeriodFormatter;
import org.joda.time.format.PeriodFormatterBuilder;

import java.math.BigDecimal;
import java.text.NumberFormat;
import java.util.Locale;

public class Converter {
    private final static String TAG = "Converter";

    public static String intToDollarAmount(String value) {
        return strToCurrency(value, true);
    }

    public static String strToCurrency(String value, boolean defaultEmptyToZero) {
        if (!defaultEmptyToZero && (value == null || value.isEmpty())) {
            return value;
        }
        NumberFormat n = NumberFormat.getCurrencyInstance(Locale.US);
        n.setMaximumFractionDigits(0);
        if (value == null || value.isEmpty()) {
            return n.format(0);
        } else {
            //strip commas, dollar signs
            String newValue = value.replaceAll(",", "");
            newValue = newValue.replaceAll("\\$", "");
            try {
                return n.format(new BigDecimal(newValue));
            } catch (Exception e) {
                //can't be formatted, return original value
                RemoteLog.w(TAG, "Cannot format string as currency: " + e.getMessage(), e);
                return value;
            }
        }
    }

    public static String intToDollarAmount(Integer value) {
        NumberFormat n = NumberFormat.getCurrencyInstance(Locale.US);
        n.setMaximumFractionDigits(0);
        return n.format(value);
    }

    public static int storeRounded(String num) {
        return num != null && !num.isEmpty() ? Integer.parseInt(num) : 0;
    }

    @InverseMethod("storeRounded")
    public static String roundTo1000s(int num) {
        return String.valueOf((int) Math.round(num / 5000.0) * 5000);
    }

    public static String milEnumToStr(EmploymentInfo.MilitaryBranch branch) {
        return branch != null ? branch.displayValue() : null;
    }

    @InverseMethod("milEnumToStr")
    public static EmploymentInfo.MilitaryBranch strToMilitaryEnum(String str) {
        if (str.isEmpty()) return null;
        return EmploymentInfo.MilitaryBranch.fromValue(str);
    }

    public static String rankEnumToStr(EmploymentInfo.Rank rank) {
        return rank != null ? rank.displayValue() : null;
    }

    @InverseMethod("rankEnumToStr")
    public static EmploymentInfo.Rank strToRankEnum(String str) {
        if (str.isEmpty()) return null;
        return EmploymentInfo.Rank.fromValue(str);
    }

    public static String strToRelativeDateStr(String str) {
        if (TextUtils.isEmpty(str)) return str;
        DateTime date = new DateTime(str);
        Period period = new Period(date, new DateTime());

        PeriodFormatter formatter;

        if (period.getYears() != 0) {
            formatter = new PeriodFormatterBuilder().appendYears().appendSuffix(" year ago", " years ago").printZeroNever().toFormatter();
        } else if (period.getMonths() != 0) {
            formatter = new PeriodFormatterBuilder().appendMonths().appendSuffix(" month ago", " months ago").printZeroNever().toFormatter();
        } else if (period.getWeeks() != 0) {
            formatter = new PeriodFormatterBuilder().appendWeeks().appendSuffix(" week ago", " weeks ago").printZeroNever().toFormatter();
        } else if (period.getDays() != 0) {
            formatter = new PeriodFormatterBuilder().appendDays().appendSuffix(" day ago", " days ago").printZeroNever().toFormatter();
        } else if (period.getHours() != 0) {
            formatter = new PeriodFormatterBuilder().appendHours().appendSuffix(" hour ago", " hours ago").printZeroNever().toFormatter();
        } else if (period.getMinutes() != 0) {
            formatter = new PeriodFormatterBuilder().appendMinutes().appendSuffix(" minute ago", " minutes ago").printZeroNever().toFormatter();
        } else {
            formatter = new PeriodFormatterBuilder().appendLiteral("seconds ago").printZeroNever().toFormatter();
        }


        return "Uploaded " + DateTimeFormat.mediumDate().print(date) + " (" + formatter.print(period) + ")";
    }

    public static String numToString(Integer value) {
        return value != null ? value.toString() : null;
    }

    public static String phoneFormat(String data) {
        if (data == null || data.length() != 11) return null;

        return "(" +
                data.substring(1, 4) +
                ") " +
                data.substring(4, 7) +
                '-' +
                data.substring(7);
    }

    public static VectorDrawableCompat toCompatDrawable(Resources resources, @DrawableRes int id) {
        return VectorDrawableCompat.create(resources, id, null);
    }
}
