package com.ibm.gbs.va.grit.util;

import android.Manifest;
import android.app.Activity;
import android.database.Cursor;
import android.net.Uri;
import android.os.Environment;
import android.provider.OpenableColumns;
import android.util.Base64;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;

public class FileUtil {
    private static final String TAG = FileUtil.class.getSimpleName();

    public static String encodeFile(InputStream inputStream) throws IOException {
        byte[] bytes = readBytes(inputStream);
        return Base64.encodeToString(bytes, Base64.DEFAULT);
    }

    public static void writeStringToDownloads(GritPermissionHelper gritPermissionHelper, String fileName, String body) {
        writeStringToStorage(gritPermissionHelper, fileName, body, Environment.DIRECTORY_DOWNLOADS);
    }

    private static void writeStringToStorage(GritPermissionHelper gritPermissionHelper, String fileName, String body, String dir) {
        File file = makeFile(fileName, dir);
        RemoteLog.d(TAG, "writing file to: " + file.getAbsolutePath());

        gritPermissionHelper.checkPermissions(Manifest.permission.WRITE_EXTERNAL_STORAGE, () -> {
            /*Write to file*/
            try {
                FileWriter fw = new FileWriter(file);
                fw.write(body);
                fw.flush();
                fw.close();
            } catch (IOException e) {
                RemoteLog.e(TAG, "Error writing to file: " + e.getMessage(), e);
            }
            return null;
        });
    }

    private static byte[] readBytes(InputStream inputStream) throws IOException {
        byte[] b = new byte[1024];
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        int c;
        while ((c = inputStream.read(b)) != -1) {
            os.write(b, 0, c);
        }
        return os.toByteArray();
    }

    public static String getURIPath(Activity activity, Uri uri) {
        String uriString = uri.toString();
        File myFile = new File(uriString);
        String displayName = null;

        if (uriString.startsWith("content://")) {
            try (Cursor cursor = activity.getContentResolver().query(uri, null, null, null, null)) {
                if (cursor != null && cursor.moveToFirst()) {
                    displayName = cursor.getString(cursor.getColumnIndex(OpenableColumns.DISPLAY_NAME));
                }
            }
        } else if (uriString.startsWith("file://")) {
            displayName = myFile.getName();
        }
        return displayName;
    }

    private static File makeFile(String fileName, String dir) {
        File fileDir = new File(Environment.getExternalStorageDirectory(), dir);

        /*if directory doesn't exist, create it*/
        if (!fileDir.exists()) {
            fileDir.mkdirs();
        }

        return new File(fileDir, fileName);
    }
}
