package com.ibm.gbs.va.grit.util;

import android.content.Context;
import android.graphics.Typeface;

import java.lang.reflect.Field;

/**
 * Sourced from
 * <a href="https://stackoverflow.com/questions/34517055/is-there-a-way-to-block-an-android-oem-from-changing-the-font-in-my-app">this
 * stack overflow answer</a>
 * &
 * <a href="https://gist.github.com/artem-zinnatullin/7749076">this gist</a>
 */
public class TypefaceUtil {

    /**
     * Using reflection to override default typeface NOTICE: DO NOT FORGET TO SET TYPEFACE FOR APP
     * THEME AS DEFAULT TYPEFACE WHICH WILL BE OVERRIDDEN
     *
     * @param context                    to work with assets
     * @param defaultFontNameToOverride  for example "monospace"
     * @param customFontFileNameInAssets file name of the font from assets
     */
    public static void overrideFont(Context context, String defaultFontNameToOverride, String customFontFileNameInAssets) {
        try {
            final Typeface customFontTypeface = Typeface.createFromAsset(context.getAssets(), customFontFileNameInAssets);

            final Field defaultFontTypefaceField = Typeface.class.getDeclaredField(defaultFontNameToOverride);
            defaultFontTypefaceField.setAccessible(true);
            defaultFontTypefaceField.set(null, customFontTypeface);
        } catch (Exception e) {
            RemoteLog.e("TypefaceUtil", "Can not set custom font " + customFontFileNameInAssets + " instead of " + defaultFontNameToOverride, e);
        }
    }
}
