package com.ibm.gbs.va.grit.view;

import android.content.Context;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.AppCompatAutoCompleteTextView;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;

import com.ibm.gbs.va.grit.R;

import java.util.Calendar;

/**
 * reference: https://github.com/Lesilva/BetterSpinner
 */
public class GritSpinner extends AppCompatAutoCompleteTextView implements AdapterView.OnItemClickListener {

    private static final int MAX_CLICK_DURATION = 200;
    private long startClickTime;
    private boolean isPopup;
    private int mPosition = ListView.INVALID_POSITION;
    private final Drawable dropdownIcon;

    public GritSpinner(Context context) {
        super(context);
        setOnItemClickListener(this);
        dropdownIcon = ContextCompat.getDrawable(getContext(), R.drawable.ic_arrow_drop_down);
        setCompoundDrawablesWithIntrinsicBounds();
    }

    public GritSpinner(Context context, AttributeSet attrs) {
        super(context, attrs);
        setOnItemClickListener(this);
        dropdownIcon = ContextCompat.getDrawable(getContext(), R.drawable.ic_arrow_drop_down);
        setCompoundDrawablesWithIntrinsicBounds();
    }

    public GritSpinner(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        setOnItemClickListener(this);
        dropdownIcon = ContextCompat.getDrawable(getContext(), R.drawable.ic_arrow_drop_down);
        setCompoundDrawablesWithIntrinsicBounds();
    }


    /**
     * set the elements adapter
     *
     * @param list list of options
     */
    public void setOptions(Object[] list) {
        this.setAdapter(new ArrayAdapter<>(getContext(), R.layout.list_item_dropdown, list));
    }

    @Override
    public boolean enoughToFilter() {
        return true;
    }

    @Override
    protected void onFocusChanged(boolean focused, int direction,
                                  Rect previouslyFocusedRect) {
        super.onFocusChanged(focused, direction, previouslyFocusedRect);
        if (focused) {
            performFiltering("", 0);
            InputMethodManager imm = (InputMethodManager) getContext().getSystemService(Context.INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(getWindowToken(), 0);
            setKeyListener(null);
            dismissDropDown();
        } else {
            isPopup = false;
        }
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (!isEnabled())
            return false;

        if (getAdapter() == null) return false;

        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN: {
                startClickTime = Calendar.getInstance().getTimeInMillis();
                break;
            }
            case MotionEvent.ACTION_UP: {
                long clickDuration = Calendar.getInstance().getTimeInMillis() - startClickTime;
                if (clickDuration < MAX_CLICK_DURATION) {
                    if (isPopup) {
                        dismissDropDown();
                        isPopup = false;
                    } else {
                        requestFocus();
                        showDropDown();
                        isPopup = true;
                    }
                }
            }
        }

        return super.onTouchEvent(event);
    }

    @Override
    public void onItemClick(AdapterView<?> adapterView, View view, int position, long id) {
        setCompoundDrawablesWithIntrinsicBounds();
        mPosition = position;
        isPopup = false;
    }

    /**
     * shorthand to call setCompoundDrawablesWithIntrinsicBounds with null params
     */
    private void setCompoundDrawablesWithIntrinsicBounds() {
        setCompoundDrawablesWithIntrinsicBounds(null, null, null, null);
    }

    @Override
    public void setCompoundDrawablesWithIntrinsicBounds(Drawable left, Drawable top, Drawable right, Drawable bottom) {
        if (right == null) {
            right = dropdownIcon;
        }
        super.setCompoundDrawablesWithIntrinsicBounds(left, top, right, bottom);
    }

    public int getPosition() {
        return mPosition;
    }
}
