package com.ibm.gbs.va.grit.view;

import android.content.Context;
import android.support.annotation.Nullable;
import android.support.design.widget.TextInputLayout;
import android.support.v4.widget.NestedScrollView;
import android.util.AttributeSet;
import android.view.ViewParent;
import android.widget.FrameLayout;
import android.widget.ScrollView;

public class SmartTextInputLayout extends TextInputLayout {
    private FrameLayout scrollView;

    public SmartTextInputLayout(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        scrollView = findParentScrollView();
    }

    public SmartTextInputLayout(Context context, AttributeSet attrs) {
        super(context, attrs);
    }

    @Override
    public void setError(@Nullable CharSequence errorText) {
        CharSequence oldError = this.getError();
        super.setError(errorText);
        boolean changed = false;
        if (oldError == null && errorText != null) {
            changed = true;
        } else if (oldError != null && errorText == null) {
            changed = true;
        }
        if (errorText == null) {
            setErrorEnabled(false);
        }

        if (changed) {
            manualScroll(errorText);
        }
    }

    private void manualScroll(CharSequence errorText) {
        if (scrollView == null) {
            scrollView = findParentScrollView();
        }
        if (errorText != null) {
            scrollView.scrollBy(0, 120);
        } else {
            scrollView.scrollBy(0, -120);
        }
    }

    private FrameLayout findParentScrollView() {
        ViewParent p = this.getParent();
        while (p != null && p.getClass() != ScrollView.class && p.getClass() != NestedScrollView.class) {
            p = p.getParent();
        }
        return (FrameLayout) p;
    }
}
