package com.ibm.gbs.va.grit.viewmodel;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.ViewModel;

import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.type.Badge;
import com.ibm.gbs.va.grit.model.type.BadgeListBadge;
import com.ibm.gbs.va.grit.repository.DigitalSelfRepository;
import com.ibm.gbs.va.grit.repository.NotificationRepository;
import com.ibm.gbs.va.grit.util.Consumer;

import java.util.HashMap;
import java.util.List;

import javax.inject.Inject;

public class BadgeListViewModel extends ViewModel {

    private final DigitalSelfRepository digitalSelfRepo;
    private final NotificationRepository notificationRepo;


    private LiveData<HashMap<Badge.BadgeCategory, List<BadgeListBadge>>> allBadgesWithContext;
    private LiveData<HashMap<Badge.BadgeCategory, List<BadgeListBadge>>> allBadges;

    @Inject
    public BadgeListViewModel(DigitalSelfRepository digitalSelfRepo, NotificationRepository notificationRepo) {
        this.digitalSelfRepo = digitalSelfRepo;
        this.notificationRepo = notificationRepo;
    }

    public LiveData<HashMap<Badge.BadgeCategory, List<BadgeListBadge>>> getAllBadgesWithContext() {
        return allBadgesWithContext;
    }

    public void setAllBadgesWithContext(LiveData<HashMap<Badge.BadgeCategory, List<BadgeListBadge>>> allBadgesWithContext) {
        this.allBadgesWithContext = allBadgesWithContext;
    }

    public LiveData<HashMap<Badge.BadgeCategory, List<BadgeListBadge>>> getAllBadges() {
        return allBadges;
    }

    public void setAllBadges(LiveData<HashMap<Badge.BadgeCategory, List<BadgeListBadge>>> allBadges) {
        this.allBadges = allBadges;
    }

    public void updateAllBadges() {
        allBadges = digitalSelfRepo.getAwardedBadges();
    }

    public Notification getNextQueuedSurveyNotification() {
        return notificationRepo.getNextAvailableSurveyNotification();
    }

    public boolean isOngoingSurvey() {
        return notificationRepo.isOngoingSurvey();
    }

    public void setSurveyNotificationNextSurvey(Consumer<Boolean> callback) {
        notificationRepo.setSurveyNotification(notificationRepo.getNextAvailableSurveyNotification(), callback);
    }

    public LiveData<String> getCurrentSurveyNotificationName() {
        return notificationRepo.getCurrentSurveyName();
    }

}
