package com.ibm.gbs.va.grit.viewmodel;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.MutableLiveData;
import android.arch.lifecycle.ViewModel;

import com.ibm.gbs.va.grit.model.notifications.Notification;
import com.ibm.gbs.va.grit.model.type.DigitalJournalEvent;
import com.ibm.gbs.va.grit.model.type.MoodScore;
import com.ibm.gbs.va.grit.repository.DigitalSelfRepository;
import com.ibm.gbs.va.grit.repository.NotificationRepository;
import com.ibm.gbs.va.grit.repository.UserRepository;
import com.ibm.gbs.va.grit.util.Consumer;

import org.joda.time.LocalDate;

import java.util.HashMap;
import java.util.List;

import javax.inject.Inject;

public class DigitalJournalViewModel extends ViewModel {

    private final DigitalSelfRepository digitalJournalRepo;
    private final NotificationRepository notificationRepo;
    private final UserRepository userRepo;

    private LiveData<List<DigitalJournalEvent>> currentEvents;

    private final MutableLiveData<String> lowerBoundXAxis = new MutableLiveData<>();
    private final MutableLiveData<String> upperBoundXAxis = new MutableLiveData<>();

    @Inject
    public DigitalJournalViewModel(DigitalSelfRepository digitalJournalRepo, NotificationRepository notificationRepo, UserRepository userRepo) {
        this.digitalJournalRepo = digitalJournalRepo;
        this.notificationRepo = notificationRepo;
        this.userRepo = userRepo;
        this.currentEvents = digitalJournalRepo.getActivityTrackerEvents(null);
    }

    public LiveData<List<MoodScore>> getCurrentMoodScores() {
        return digitalJournalRepo.getMoodGraphEntries();
    }

    public MutableLiveData<String> getLowerBoundXAxis() {
        return lowerBoundXAxis;
    }

    public void setLowerBoundXAxis(String lowerBoundXAxis) {
        this.lowerBoundXAxis.setValue(lowerBoundXAxis);
    }

    public MutableLiveData<String> getUpperBoundXAxis() {
        return upperBoundXAxis;
    }

    public void setUpperBoundXAxis(String upperBoundXAxis) {
        this.upperBoundXAxis.setValue(upperBoundXAxis);
    }

    public LiveData<List<DigitalJournalEvent>> getCurrentEvents() {
        return currentEvents;
    }

    public void setCurrentEvents(LiveData<List<DigitalJournalEvent>> currentEvents) {
        this.currentEvents = currentEvents;
    }

    public HashMap<String, LocalDate> getValidEventRanges() {
        return digitalJournalRepo.getValidEventRanges();
    }

    public void updateEventData(LocalDate dateToUpdate) {
        digitalJournalRepo.getActivityTrackerEvents(dateToUpdate);
    }

    public Notification getNextQueuedSurveyNotification() {
        return notificationRepo.getNextAvailableSurveyNotification();
    }

    public boolean isOngoingSurvey() {
        return notificationRepo.isOngoingSurvey();
    }

    public void setSurveyNotificationNextSurvey(Consumer<Boolean> callback) {
        notificationRepo.setSurveyNotification(notificationRepo.getNextAvailableSurveyNotification(), callback);
    }

    public LiveData<String> getCurrentSurveyNotificationName() {
        return notificationRepo.getCurrentSurveyName();
    }

    public boolean isFirstTimeInfo() {
        return userRepo.getOnboardingDigitalJournalInfo();
    }

    public void setFirstTimeInfo(boolean isFirstTime) {
        userRepo.setOnboardingDigitalJournalInfo(isFirstTime);
    }
}
