package com.ibm.gbs.va.grit.viewmodel;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.Transformations;
import android.arch.lifecycle.ViewModel;

import com.ibm.gbs.va.grit.model.type.User;
import com.ibm.gbs.va.grit.repository.LocationRepository;
import com.ibm.gbs.va.grit.repository.UserRepository;
import com.ibm.gbs.va.grit.util.RemoteLog;
import com.ibm.gbs.va.grit.util.StringUtil;

import java.util.List;

import javax.inject.Inject;

public class EditProfileViewModel extends ViewModel {
    private final static String TAG = "EditProfileViewModel";

    private final LiveData<User> currentUser;

    public final LiveData<Boolean> isEmailBeingEdited;
    public final LiveData<Boolean> haveEditsBeenMade;
    public final GritMutableLiveData<User> user = new GritMutableLiveData<>();

    public final List<String> citiesList;

    private final UserRepository userRepo;

    @Inject
    public EditProfileViewModel(LocationRepository locationRepository, UserRepository userRepo) {
        this.userRepo = userRepo;

        currentUser = userRepo.getUserInfo();
        citiesList = locationRepository.getCities();

        // update UI when the backing data updates
        user.addSource(currentUser, currentUser -> {
            if (currentUser != null)
                user.setValue(currentUser.duplicate());
        });


        // set up observations
        isEmailBeingEdited = Transformations.map(user, user -> {
            String prevVal = currentUser.getValue() != null ? currentUser.getValue().getEmail() : null;
            String newVal = user.getEmail();
            boolean emailEdited = newVal != null && !newVal.equals(prevVal);
            RemoteLog.d(TAG, "email is being changed: " + emailEdited);
            return emailEdited;
        });
        haveEditsBeenMade = Transformations.map(user, user -> {
            boolean editsMade = !user.equals(currentUser.getValue());
            RemoteLog.d(TAG, "edits have been made: " + editsMade);
            return editsMade;
        });
    }

    public LiveData<UserRepository.UPDATE_STATE> updateUser() {
        User userToSet = user.getValue();
        userToSet.setEmail(StringUtil.convertEmailCase(userToSet.getEmail()));
        return userRepo.updateUser(userToSet);
    }

}
