package com.ibm.gbs.va.grit.viewmodel;

import android.arch.lifecycle.LiveData;
import android.arch.lifecycle.Transformations;
import android.arch.lifecycle.ViewModel;

import com.ibm.gbs.va.grit.model.dialog.TextSection;
import com.ibm.gbs.va.grit.model.type.Handle;
import com.ibm.gbs.va.grit.repository.SocialMediaRepository;
import com.ibm.gbs.va.grit.util.RemoteLog;

import java.util.Arrays;
import java.util.List;

import javax.inject.Inject;

public class SocialMediaViewModel extends ViewModel {
    public final GritMutableLiveData<Handle> twitterHandle = new GritMutableLiveData<>();
    public final LiveData<Boolean> haveEditsBeenMade;
    private final LiveData<Handle> currentTwitterHandle;
    private final SocialMediaRepository socialMediaRepo;
    private List<TextSection> twitterCanBody;
    private List<TextSection> twitterCannotBody;
    private final String TAG;

    @Inject
    public SocialMediaViewModel(SocialMediaRepository socialMediaRepo) {
        this.TAG = this.getClass().getSimpleName();
        this.socialMediaRepo = socialMediaRepo;
        currentTwitterHandle = socialMediaRepo.getTwitterHandle();
        twitterHandle.addSource(currentTwitterHandle, currentTwitterHandle -> twitterHandle.setValue(currentTwitterHandle.duplicate()));
        haveEditsBeenMade = Transformations.map(twitterHandle, twitterHandle -> {
            String prevVal = currentTwitterHandle.getValue() != null ? currentTwitterHandle.getValue().toString() : "";
            String newVal = twitterHandle.toString();
            RemoteLog.d(TAG, "checking if edits have been made.  prevVal: " + prevVal + "; newVal: " + newVal);
            return !newVal.equals(prevVal);
        });
    }

    public List<TextSection> getTwitterCanBody() {
        return twitterCanBody;
    }

    public void setBulletedTwitterCanBody(TextSection... sections) {
        this.twitterCanBody = Arrays.asList(sections);
    }

    public List<TextSection> getTwitterCannotBody() {
        return twitterCannotBody;
    }

    public void setBulletedTwitterCannotBody(TextSection... sections) {
        this.twitterCannotBody = Arrays.asList(sections);
    }

    public LiveData<SocialMediaRepository.UPDATE_STATE> updateTwitterHandle() {
        RemoteLog.d(TAG, "currentTwitterHandle.getValue() at update: " + currentTwitterHandle.getValue());
        Handle handle = currentTwitterHandle.getValue();
        if (handle.getHandle() == null || handle.getHandle().isEmpty()) {
            return socialMediaRepo.addTwitterHandle(twitterHandle.getValue().getHandle());
        } else {
            return socialMediaRepo.updateTwitterHandle(twitterHandle.getValue().getHandle());
        }
    }

    public LiveData<Boolean> removeTwitterHandle() {
        return socialMediaRepo.removeTwitterHandle();
    }
}
