# va-grit-cognitive
Veteran's Affairs GRIT Cognitive Backend

Notes for using WEM API

#### wem-user

Endpoints:

POST /WEMUsers/createUserProfile/
Implementation Notes
Upload the User profile in below form-data format

Content-Type: multipart/form-data
-F 'candidateName=Marc-Jimerson'
-F 'veteranID=unique ID generated from midtier when user creates profile'
-F 'careerHistory=[] (optional)'
-F 'previousCompany=XYZ (optional)'
-F 'jobRole=some job role (optional)'
-F 'moc=25N (military MOS)'
-F 'qualifications=some qualifications (optional)'
-F 'location=[] (desired location as an array)'
-F 'availableIn=30 (optional)'
-F 'experience=12 (optional)'
-F 'targetedIndustries=[]'
-F 'minSalary'
-F 'jobType=fulltime'
-F 'militaryBranch=Army'
-F 'militaryRank=private'
-F resumeFile=@'Marc-Jimerson.pdf'

- To create a profile, users must submit a minimum of a resume OR an MOS code. Other fields may produce errors if not submitted. The more complete the profile, the more accurate job search results will be. Location must be in City, ST format.

GET /WEMUsers/deleteUserProfile/{veteranId}
- Deletes a users profile from MongoDB based on veteranID


GET /WEMUsers/getCandidateSkills/{veteranID}
- based on veteranID returns the extracted skills from the candidate's resume


GET /WEMUsers/getUserProfile/{veteranId}
- returns the details of the user's profile by veteranID

POST /WEMUsers/performJobSearch
query:[result of getCandidateSkills]
isMOS: [true if MOS provided for lite search]
  - skills will be used from MOS (pulled from Onet) as the query input
isLite:[False if resume true if no resume]
noOfJobs:number of jobs returned
veteranId:{veteranID}

- The job search query is performed based on the skills from the resume or the skills from the MOS so as not to be too specific in the job search.

POST /WEMUsers/performSkillGapAnalysis

veteranId:{veteranID}
desiredIndustry:[if not provided will be used from employment profile]
yearsOfExp:[if not provided will be used from employment profile]
minSalaryExpectation:[if not provided will be used from employment profile]
preferredLocation:[if not provided will be used from employment profile]

#### wem-admin

Endpoints:

POST /WEMAdmins/{jobIndustry}/uploadJobs
- Daily added jobs file is a .zip from BG server, use runFTPclient to FTP and unzip. Run parseAddJobs.js to convert to JSON. POST JSON {jobIndustry} should be the same as the industry for removeJobs (I recommend BurningGlass for both). This process uploads the daily jobs to MongoDB and writes a directory of JSON files (1 JSON per Job)

POST /WEMAdmins/{jobIndustry}/removeJobs
- remove jobs file is a .zip from BG server, use runFTPclient to FTP and unzip. Run parseDeleteJobs.js to convert to JSON. POST JSON {jobIndustry} should be the same as the industry for uploadJobs (I recommend BurningGlass for both)

POST /WEMAdmins/createDocuments/{directory}
- point WEM to the directory of individual job JSON files created in uploadJobs. This creates a document for each job in WDS as well as populates the lookup table linking JobID with WDS document id to job removal

POST /WEMAdmins/createCollection
- creates a collection in WDS that createDocuments posts documents to

### xml-to-json

script #1
runFTPclient.js
run with command:
```
node runFTPclient.js
```
This will pull the latest Add jobs file and Delete jobs file. Add files are placed in directory ./BGDailyFeed/addJobs and ./BGDailyFeed/deleteJobs as .xml

script #2
parseAddJobs.js
run with command:
```
node parseAddJobs.js -f ./BGDailyFeed/addJobs/filename.xml
```
This will parse the above created .xml and write a .json with the date that the file is parsed

script #3
parseDeleteJobs.js
run with command:
```
node parseAddJobs.js -f ./BGDailyFeed/deleteJobs/filename.xml
```
This will parse the above created .xml and write a .json with the date that the file is parsed
