'use strict';

var request = require('request');

var util = require('util');
var db = require('../config/db');
var async = require('async');
var services  = require('../config/config').services;
var FormData = require('form-data');

//Search Skills & MOC Context variable names
var SKILLS_CTX = "skills";
var MOC_CTX = "moc";

//Search Skills & MOC Intent names
var SEARCH_SKILLS_INT="results_skills";
var SEARCH_MOC_INT = "results_MOC";

//Channel names
var CHANNEL_TELEGRAM = "Telegram";

//Error Codes
var ERROR_MSG_TO_USER = "Sorry! I am not understanding your message. Please reword it. Thank you.";

//Chat IDs
var CHAT_ID_MOBILE = "mobile";
var CHAT_ID_WEB = "web";

module.exports = function(Chatbot) {
	

	Chatbot.remoteMethod(
		    'chatWithBot', {
		      http: {
		        path: '/chatWithBot',
		        verb: 'post'
		      },
		      accepts: [
		    	  {arg: 'usermessage',type: 'string',required: true},
		    	  {arg: 'chatId',type: 'string', required: true},
		    	  {arg: 'botName',type: 'string', required: true},
				  {arg: 'channel',type: 'string', required: true}
		    	  
		      ],
		      returns: {
		        arg: 'response',
		        type: 'object',
		        description: "response format"
		      }
		    }
		  );
	
	
	Chatbot.chatWithBot = function(usermessage, chatTXId, botName, channel, cb) {
		
		var baseURL = Chatbot.app.get('url').replace(/\/$/, '');
		
		//Prepare the chat response with job search process
		prepareChatResponse(usermessage, chatTXId, botName, baseURL, channel, cb);
		
	};
};


function prepareChatResponse(usermessage, chatTXId, botName, baseURL, channel, cb) {
	
	var carlaURL = services.carlaURL;
	
	var isChatFound = false;
	
	var replyMgs = {
			
			"chatId" : chatTXId,
			"botName" : botName,
			"outputMsg" : ""
	};
	
	async.waterfall([
		
		function(callback) {
				
				//#1 Check whether a chat record available in data base with the give Chat ID
				db.transactionModel.findOne({ 'transaction.chatId': chatTXId }, function (err, doc){
					
					var prevChat = {};
					
					if(!err) {
						
						if (doc) {
							
							// Found chat record for a given chatId
							
							prevChat = doc;
							
							//set the current user chat into the previous chat record.
							prevChat.transaction.carlaoutput.input['text'] = usermessage;
							
							isChatFound =  true;
							
							callback(null, prevChat.transaction.carlaoutput);
							
						} else {
							
							//It's the first message from a user, as no previous record found
							// Initial chat input to conversation service, with user's first message
							prevChat = {"input":
											{ "text": usermessage
												}
										};
											
							
							callback(null, prevChat);
						}
						
					} else {
						
						//Error in finding chat record in DB
						callback(err, null);
					}
					
				});
		}, function (prevChat, callback) {
			
			//#2 Send the user message to CARLA
			
			console.log("Req to server ", prevChat);
			
			request({
			    url: carlaURL,
			    method: "POST",
			    json: true,
			    headers: {
			        "content-type": "application/json",
			    },
			    body: prevChat
			}, function(error, response, chatResponse){
				
				if(!error) {
					
					//callback(null, chatResponse);
					//parseChatResponse(chatResponse, callback);
					
					var searchReq = isJobSearchRequest(chatResponse);
					
					if (searchReq.isJobSearch) {
						
						var searchType = searchReq.type;
						
						performJobSearch(searchType, chatResponse,channel, baseURL, callback);
						
					} else {
						callback(null, chatResponse, null);
					}
				}
			});
		}, function (chatResp, jobSearchResult, callback) {
			
			//#3 Process chat response from CARLA
			///console.log("Outpput:: ", chatResp);
			
			console.log("jobSearchResult", jobSearchResult);
			if (isChatFound) {
				
				//Update the chat record with the chat response, for future use.
				db.transactionModel.findOneAndUpdate({ 'transaction.chatId': chatTXId },{'transaction.carlaoutput' : chatResp} ,function (err, doc){	
					
					console.log("Doc updated!");
					
					try {
						
						if(jobSearchResult) {
		                	
		                	console.log("sending job search result");
		                	replyMgs.outputMsg = jobSearchResult;
		                	
		                } else {
		                	
		                	replyMgs.outputMsg = chatResp.output.text[0];
		                }
						
					} catch (error) {
						
						replyMgs.outputMsg = ERROR_MSG_TO_USER;
					}
					
					
					
					callback(null, replyMgs);
				});
				
				
			} else {
				
				//This the first message from User. Create a record and save to DB
				var doc = new db.transactionModel();
				
				// Check if the channel is other than Telegram and then set CARLA conversation id as chat id. 
				// As other channels like web,mobile won't generate chat id and pass in the first conversation, in subsequent they can pass 
				
				if(channel !== CHANNEL_TELEGRAM && (chatTXId === CHAT_ID_MOBILE || chatTXId === CHAT_ID_WEB)){
					
					chatTXId = (chatResp.context.conversation_id).toString();
					
					//Set Carla conversation id in the replyMsg as chatId
					replyMgs.chatId = chatTXId;
				}
				var saveInDB = {
						chatId: chatTXId,
						botName: botName,
						carlaoutput: chatResp
				};
				
				doc.transaction = saveInDB;
				
				doc.save(function (err) {
					
					if (err) {
		                 console.log('Error saving TX: ' + err);
		                 callback(err, null);
		              } else {
		                console.log('Success saving TX');
		                
		                replyMgs.outputMsg = chatResp.output.text[0];
		                
		                callback(null, replyMgs);
		                	
		              }
				});
				
			}
			
		}
	
		
	],function (err, chatResp) {
			
			//#4 Send chat response to the client
			if(err) {
				
				cb(err, null);
			} else {
				cb(null, chatResp);
			}
		
		}
		
	);
	
}


function isJobSearchRequest(chatResponse) {
	
	//#1 Rule for Job search 
	
	//Condition 1 - Search by skills - check for results: true
	
	console.log("isJobSearchRequest:: ", chatResponse);
	
	var isResultSet = false;
	var isSearchByMOC = false;
	
	var jobSearchAttrs = {
			
			"isJobSearch" : false,
			"type" : "none"
	};
	
	//Condition 1 - Search by Skills - check for Search Skills Intent and existence of skills Context Variable
	
	try {
		
		if(chatResponse.context[SKILLS_CTX] && chatResponse.intents[0].intent === SEARCH_SKILLS_INT){
			
			jobSearchAttrs.isJobSearch = true;
			jobSearchAttrs.type = SKILLS_CTX
		}

		//Condition 2 - Search by MOC - check for Search MOC Intent and existence of MOC Context Variable

		if(chatResponse.context[MOC_CTX] && chatResponse.intents[0].intent === SEARCH_MOC_INT){
			
			jobSearchAttrs.isJobSearch = true;
			jobSearchAttrs.type = MOC_CTX
			
		}
		
	} catch (err) {
		
		console.log("[isJobSearchRequest] Error!:: ", err);
		jobSearchAttrs.isJobSearch = false;
		jobSearchAttrs.type = "none";
		
	}

	return jobSearchAttrs;
	
}


function performJobSearch(searchType, chatResponse, channel, baseURL, parentCallback) {
	
	
	console.log(":: performJobSearch ::");
	var searchByMOC = false;
	var searchQuery = "";
	
	if(searchType === MOC_CTX) {
		searchByMOC = true;
		searchQuery = chatResponse.context.moc;
	} else {
	
		searchQuery = chatResponse.context.skills
	}
	
	
	var formData = {
			
			"query": searchQuery, //"Project Management",
			"isMOS": searchByMOC,
			"isLite": true, //for chatbot use lite version of results
			"noOfJobs": 10 //
	};
	
	request.post({url: baseURL +'/api/WEMUsers/performJobSearch', form: formData}, function (err, httpResponse, body) {
		
		  if (err) {
			  parentCallback(err, "Unable to searach jobs now. Please try after sometime.");
		  } else {
			  
			  var searchRes = JSON.parse(body);
			  var jobsOutput = ""
			  
			  //Check if output has to be prepare results for Telegram
			  
			  if(channel === CHANNEL_TELEGRAM){
				
				var jobResults = searchRes.searchresult.results;
				for(var i = 0; i < jobResults.length; i++) {
				  
					jobsOutput += '<a href="' + jobResults[i].jobURL + '">' + jobResults[i].title + '</a>'+'"\n"';
				}
			  } else {
				  
				   // Pass the JSON output from performJobSearch for other channles
				   jobsOutput = searchRes;
			  }
			  
			  parentCallback(null, chatResponse, jobsOutput);
		  }
		  
		});
	
}