package com.ibm.grit.rest;

import static com.ibm.grit.utils.NotificationsUtils.addDays;

import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.TimeZone;
import java.util.concurrent.TimeUnit;

import com.google.gson.Gson;
import com.ibm.grit.scoring.Tweet;

import okhttp3.HttpUrl;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

public class BlueTas {
    
    //static OkHttpClient client = new OkHttpClient();
    static OkHttpClient client = new OkHttpClient.Builder()
                .connectTimeout(10, TimeUnit.SECONDS)
                .writeTimeout(10, TimeUnit.SECONDS)
                .readTimeout(10, TimeUnit.SECONDS)
                .build();
    
    static String run(String url) throws Exception{
        Request request = new Request.Builder()
            .url(url)
            .build();
        //System.out.println("DEBUG: Trying to make request call...");
        Response response = null;
        try {
            response = client.newCall(request).execute();
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        //System.out.println("DEBUG: Request completed!");
        try {
            return response.body().string();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return "Response failed";
    }
    
    public static List<Tweet> getTweets(String author, String personId) throws Exception {
        // Example call:
        // "http://169.62.10.213:9080/BluetasAPI/api/v1/search/?author=morpheo&source=mongodb&channel=twitter&start_date=5/15/2000&end_date=12/01/2018";
        Date now = new Date();
        Date endDate = addDays(now, 1);
        Date startDate = addDays(now, -30);
        
        //HttpUrl.Builder urlBuilder = HttpUrl.parse("http://169.62.10.213:9080/BluetasAPI/api/v1/search/").newBuilder();
        HttpUrl.Builder urlBuilder = HttpUrl.parse("http://bluetasv2/BluetasAPI/api/v1/search/").newBuilder();
        //urlBuilder.port(80);
        urlBuilder.addQueryParameter("author", author);
        urlBuilder.addQueryParameter("source", "mongodb");
        urlBuilder.addQueryParameter("channel", "twitter");
        urlBuilder.addQueryParameter("start_date", tweetDateToString(startDate));
        urlBuilder.addQueryParameter("end_date", tweetDateToString(endDate));
        
        String url = urlBuilder.build().toString();
        
        //url = "http://bluetasv2:80/BluetasAPI/api/v1/search/?author=jbeers8&source=mongodb&channel=twitter&start_date=12%2F11%2F2018&end_date=01%2F11%2F2019";
        System.out.println(url);
        
        String response = run(url);
        //System.out.println("DEBUG: " + response);
        //System.out.println("DEBUG: BlueTas.getTweets()");
        List<Tweet> tweets = parseBlueTasResponse(response, personId);
        //System.out.println("DEBUG: BlueTas response parsed!!");
        
        return tweets;
        
    }
    
    static List<Tweet> parseBlueTasResponse(String blueTasResponse, String personId){
        Gson gson = new Gson();
        Object object = gson.fromJson(blueTasResponse, Object.class);
        //System.out.println(object);
        
        Map<String, Object> searchParms = (Map<String, Object>) ((Map<String, Object>) object).get("search_parms");
        
        String author = (String) searchParms.get("author");
        System.out.println(author);
        
        List<Object> results = (List<Object>)((Map<String, Object>) object).get("results");

        List<Tweet> tweets = new ArrayList<>();
        for (Object result: results) {
            Tweet tweet = new Tweet();
            tweet.author = author;
            tweet.personId = personId;
            
            tweet.tweetId = ((Map<String, Double>) result).get("id");
            
            Map<String, Object> analytics = (Map<String, Object>) ((Map<String, Object>) result).get("analytics");
            tweet.sentimentScore = (double)((Map<String, Object>) analytics).get("sentimentScore");
            
            //Map<String, Object> sentimentScoreMap = (Map<String, Object>) ((Map<String, Object>) analytics).get("sentimentScore");
            //String sentimentScoreString = (String)((Map<String, Object>) sentimentScoreMap).get("$numberDouble");
            //tweet.sentimentScore = Double.parseDouble(sentimentScoreString);
            
            tweet.text = ((Map<String, String>) result).get("text");
            
            String createdAt = ((Map<String, String>) result).get("created_at");
            tweet.createdAt = tweetStringToDate(createdAt);
            
            tweets.add(tweet);
        }
        return tweets;
    }
    
    static Date tweetStringToDate(String dateString) {
        TimeZone tz = TimeZone.getTimeZone("UTC");
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        df.setTimeZone(tz);
        
        try {
            return df.parse(dateString);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        
        return null;
    }
    
    static String tweetDateToString(Date date) {
        TimeZone tz = TimeZone.getTimeZone("UTC");
        DateFormat df = new SimpleDateFormat("MM/dd/yyyy");
        df.setTimeZone(tz);
        return df.format(date);
    }
}
