package com.ibm.grit.rest;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;

import org.bson.codecs.configuration.CodecRegistry;
import org.bson.codecs.pojo.PojoCodecProvider;

import static org.bson.codecs.configuration.CodecRegistries.fromProviders;
import static org.bson.codecs.configuration.CodecRegistries.fromRegistries;

import com.mongodb.Block;
import com.mongodb.ConnectionString;
import com.mongodb.MongoClientSettings;
import com.mongodb.MongoCredential;
import com.mongodb.client.FindIterable;
import com.mongodb.client.MongoClients;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import com.mongodb.client.model.FindOneAndReplaceOptions;
import com.mongodb.client.model.Sorts;
import com.mongodb.client.model.UpdateOptions;

import static com.mongodb.client.model.Filters.*;
import static com.mongodb.client.model.Updates.*;

import org.kie.api.KieServices;
import org.kie.api.event.rule.DebugAgendaEventListener;
import org.kie.api.event.rule.DebugRuleRuntimeEventListener;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;

import com.ibm.grit.scoring.BasicNeeds;
import com.ibm.grit.scoring.Employment;
import com.ibm.grit.scoring.Health;
import com.ibm.grit.scoring.Mental;
import com.ibm.grit.scoring.Military;
import com.ibm.grit.scoring.Mood;
import com.ibm.grit.scoring.RiskAssessment;
import com.ibm.grit.scoring.Social;
import com.ibm.grit.scoring.Tweet;

import static com.ibm.grit.rest.BlueTas.getTweets;
import static com.ibm.grit.utils.ScoringUtils.computeMoodScoreFromAnswer;
import static com.ibm.grit.utils.ScoringUtils.parseSocialMedia;

@Path("risk-service")
public class RiskServiceResource {
    
    static Properties properties = new Properties();
    static {
        InputStream inputStream = null;
        try {
            inputStream = new FileInputStream("src/main/resources/config.properties");
            properties.load(inputStream);
            
            inputStream.close();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    //public static final boolean PRODUCTION = false; // Set to true before deploying to server. Set to false if testing in development environment.
    //public static final boolean MLAB = false; // Set to true to use mLab hosted MongoDB
    public static final boolean PRODUCTION = Boolean.parseBoolean(properties.getProperty("production"));
    public static final boolean MLAB = Boolean.parseBoolean(properties.getProperty("mlab"));
    
    static CodecRegistry pojoCodecRegistry = fromRegistries(com.mongodb.MongoClient.getDefaultCodecRegistry(),
            fromProviders(PojoCodecProvider.builder().automatic(true).build()));
    
    // Use testSettings for internal testing with local MongoDB instance.
    static MongoClientSettings testSettings = MongoClientSettings.builder()
        .codecRegistry(pojoCodecRegistry)
        .build();
    
    //static ConnectionString connectionString = new ConnectionString("mongodb://vagritmongoadmin:iA2Lb5rI8Zlw@169.62.10.213:27017/?authSource=admin");
    //static ConnectionString connectionString = new ConnectionString("mongodb://vagritmongoadmin:DMDKPWXErwJbC8vt@ds223274-a0.cwl63.fleet.mlab.com:23274,ds223274-a1.cwl63.fleet.mlab.com:23274/?authSource=admin&replicaSet=rs-ds223274&ssl=true");
    static ConnectionString connectionString = new ConnectionString(properties.getProperty("mongoConnectionString"));
    
    // Use productionSettings for deployment to server.
    static MongoClientSettings productionSettings = MongoClientSettings.builder()
            .codecRegistry(pojoCodecRegistry)
            .applyConnectionString(connectionString)
            .build();
    
    // Change the settings parameter based on testing or production environment!
    static com.mongodb.client.MongoClient mongoClient = (MLAB ? MongoClients.create(productionSettings) : MongoClients.create(testSettings));
    
    static MongoDatabase database = mongoClient.getDatabase("risk");
    static MongoCollection<BasicNeeds> bnCollection = database.getCollection("basicneeds", BasicNeeds.class);
    static MongoCollection<Employment> empCollection = database.getCollection("employment", Employment.class);
    static MongoCollection<Health> heaCollection = database.getCollection("health", Health.class);
    static MongoCollection<Mental> mentCollection = database.getCollection("mental", Mental.class);
    static MongoCollection<Military> milCollection = database.getCollection("military", Military.class);
    static MongoCollection<Social> socCollection = database.getCollection("social", Social.class);
    static MongoCollection<ScoreRequestData> dataCollection = database.getCollection("data", ScoreRequestData.class);
    static MongoCollection<RiskAssessment> raCollection = database.getCollection("assessment", RiskAssessment.class);
    static MongoCollection<Notification> notifCollection = database.getCollection("notification", Notification.class);
    static MongoCollection<Mood> moodCollection = database.getCollection("mood", Mood.class);
    static MongoCollection<Sms> smsCollection = database.getCollection("sms", Sms.class);
    static MongoCollection<Tweet> tweetCollection = database.getCollection("tweet", Tweet.class);
    
    @Path("score")
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    public ScoreResponse score(ScoreRequestData data) {
        /***
         * This is the score API as defined in Swagger and the README.
         */
        dataCollection.insertOne(data); // Store all incoming data
        
        List<Flag> nextSessionFlags = new ArrayList<Flag>(); // Stays empty unless BASELINE survey type.
        
        if (data.surveyType.equals("BASELINE") || data.surveyType.equals("MOOD_FACES")) {
            // Flags modified only for BASELINE type (ICD draft 4 p.17)
            runFlagsRules(data, nextSessionFlags);
            runScoringRules(data);
        }
        
        // Schedule reassessment and create or update notifications
        runNotificationsRules(data, nextSessionFlags);
        
        ScoreResponse response = new ScoreResponse(true, "Success", data.currentSessionFlags);
        System.out.println("Session completed for " + data.vetId);
        System.out.println("-------------------------------------------------------------");
        return response;
    }
    
    private void runFlagsRules(ScoreRequestData data, List<Flag> nextSessionFlags) {
        KieServices ks = KieServices.Factory.get();
        KieContainer kc = ks.getKieClasspathContainer();
        KieSession ksessionFlags = kc.newKieSession("FlagsKS");
        
        // The application can also setup listeners
        //ksessionFlags.addEventListener( new DebugAgendaEventListener() );
        //ksessionFlags.addEventListener( new DebugRuleRuntimeEventListener() );
        
        // Insert global variables
        ksessionFlags.setGlobal("bnCollection", bnCollection);
        ksessionFlags.setGlobal("empCollection", empCollection);
        ksessionFlags.setGlobal("heaCollection", heaCollection);
        ksessionFlags.setGlobal("mentCollection", mentCollection);
        ksessionFlags.setGlobal("milCollection", milCollection);
        ksessionFlags.setGlobal("socCollection", socCollection);
        ksessionFlags.setGlobal("nextSessionFlags", nextSessionFlags);
        ksessionFlags.setGlobal("moodCollection", moodCollection);
        ksessionFlags.setGlobal("tweetCollection", tweetCollection);
        
        
        ksessionFlags.insert(data);
        ksessionFlags.fireAllRules();
        ksessionFlags.dispose();
        
        System.out.println("currentSessionFlags that are set:");
        for (Flag flag: data.currentSessionFlags) {
            if (flag.value) {
                System.out.println("  "+ flag.flagName + ": " + flag.value);
            }
        }
        System.out.println("nextSessionFlags that are set:");
        for (Flag flag: nextSessionFlags) {
            System.out.println("  "+ flag.flagName + ": " + flag.value);
        }
    }
    
    private RiskAssessment runScoringRules(ScoreRequestData data) {
        System.out.println("Performing Risk Assessment!");
        String vetId = data.vetId;
        
        KieServices ks = KieServices.Factory.get();
        KieContainer kc = ks.getKieClasspathContainer();
        KieSession ksession = kc.newKieSession("AllKS");
        
        RiskAssessment ra = new RiskAssessment();
        ra.personId = vetId;
        
        BasicNeeds bn = bnCollection.find(eq("personId", vetId)).first();
        Employment emp = empCollection.find(eq("personId", vetId)).first();
        Health hea = heaCollection.find(eq("personId", vetId)).first();
        Military mil = milCollection.find(eq("personId", vetId)).first();
        Mental ment = mentCollection.find(eq("personId", vetId)).first();
        Social soc = socCollection.find(eq("personId", vetId)).first();
        Iterable<Mood> moodIterable = moodCollection.find(eq("personId", vetId)).sort(Sorts.descending("timestamp")).limit(30);
        
        // Insert facts
        ksession.insert(ra);
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        for (Mood mood: moodIterable) {
            System.out.println("Inserting mood for " + mood.personId);
            System.out.println("  timestamp = " + mood.timestamp);
            System.out.println("  answer = " + mood.answer);
            ksession.insert(mood);
        }
        
        List<Tweet> tweets = parseSocialMedia(data);
        tweets.stream().forEach(tweet -> {
            tweetCollection.findOneAndReplace(eq("tweetId", tweet.tweetId), tweet, new FindOneAndReplaceOptions().upsert(true));
            System.out.println("Tweet");
            System.out.println("   author: " + tweet.author);
            System.out.println("   created at: " + tweet.createdAt);
            System.out.println("   text: " + tweet.text);
            System.out.println("   sentiment score: " + tweet.sentimentScore);
        });
        
        Iterable<Tweet> tweetIterable = tweetCollection.find(eq("personId", vetId)).sort(Sorts.descending("createdAt")).limit(60);
        for (Tweet tweet: tweetIterable) {
            System.out.println("Inserting tweet for " + vetId);
            System.out.println("Tweet text: " + tweet.text);
            ksession.insert(tweet);  // Insert into working memory for scoring rules
        }
        
        ksession.fireAllRules();
        ksession.dispose();
        
        // Store new risk assessment in MongoDB
        raCollection.insertOne(ra);
        
        return ra;
    }
    
    private void runNotificationsRules(ScoreRequestData data, List<Flag> nextSessionFlags) {
        KieServices ks = KieServices.Factory.get();
        KieContainer kc = ks.getKieClasspathContainer();
        
        KieSession ksessionNotif = kc.newKieSession("NotificationsKS");
        System.out.println("NotificationsKS: " + ksessionNotif);
        
        //ksessionNotif.addEventListener( new DebugAgendaEventListener() );
        //ksessionNotif.addEventListener( new DebugRuleRuntimeEventListener() );
        
        ksessionNotif.setGlobal("notifCollection", notifCollection);
        ksessionNotif.setGlobal("raCollection", raCollection);
        ksessionNotif.setGlobal("dataCollection", dataCollection);
        ksessionNotif.setGlobal("smsCollection", smsCollection);
        
        // Get all previously stored notifications for veteran and insert them into ksession
        Iterable<Notification> notificationList = notifCollection.find(eq("vetId", data.vetId));
        for (Notification notification: notificationList) {
            System.out.println("Inserting notification for " + notification.vetId);
            System.out.println("  notificationId = " + notification.notificationId);
            System.out.println("  notificationType = " + notification.notificationType);
            System.out.println("  effectiveDate = " + notification.effectiveDate);
            ksessionNotif.insert(notification);
        }
        
        ksessionNotif.insert(data);
        
        List<RiskAssessment> raList = getRiskAssessments(data.vetId, 180); // Setting arbitrary limit of 180
        raList.stream().forEach(riskAssessment -> ksessionNotif.insert(riskAssessment));
        
        for (Flag flag: nextSessionFlags) {
            ksessionNotif.insert(flag);
        }
        
        Sms sms = smsCollection.find(eq("personId", data.vetId)).first();
        ksessionNotif.insert(sms);
        if (sms != null) {
            System.out.println("Inserted SMS for " + sms.personId);
        }
        
        
        ksessionNotif.fireAllRules();
        ksessionNotif.dispose();
    }
    
    private static List<RiskAssessment> getRiskAssessments(String personId, int limit) {
        Iterable<RiskAssessment> raIterable = raCollection.find(eq("personId", personId)).sort(Sorts.descending("timestamp")).limit(limit);
        List<RiskAssessment> raList = new ArrayList<>();
        for (RiskAssessment riskAssessment: raIterable) {
            raList.add(riskAssessment);
        }
        return raList;
    }
    
    @Path("getMoodScores")
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    public GetMoodScoresResponse getMoodScores(VetId vetId) {
        // TODO: Implement try/catch for handling this.
        
        System.out.println("Getting all mood scores for " + vetId.vetId);
        
        // Return all MOOD_FACES answer values for given vetId parameter
        Iterable<Mood> moodIterable = moodCollection.find(eq("personId", vetId.vetId)).sort(Sorts.descending("timestamp")).limit(30);
        List<MoodScore> moodList= new ArrayList<>();
        int counter = 0;
        for (Mood mood: moodIterable) {
            moodList.add(new MoodScore(mood.timestamp, computeMoodScoreFromAnswer(mood)));
            counter++;
        }
        System.out.println("Total number of mood scores: " + counter);
        
        GetMoodScoresResponse response = new GetMoodScoresResponse();
        response.success = true;
        response.message = "Success";
        response.scores = moodList;
        
        return response;
    }
    
    @Path("purgeData")
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    public PurgeDataResponse purgeData(VetId vetId) {
        System.out.println("Purging data for " + vetId.vetId);
        
        //Purge all scoring facts
        bnCollection.deleteMany(eq("personId", vetId.vetId));
        System.out.println("Deleted basic needs: " + vetId.vetId);
        empCollection.deleteMany(eq("personId", vetId.vetId));
        System.out.println("Deleted employment: " + vetId.vetId);
        heaCollection.deleteMany(eq("personId", vetId.vetId));
        System.out.println("Deleted health: " + vetId.vetId);
        mentCollection.deleteMany(eq("personId", vetId.vetId));
        System.out.println("Deleted mental: " + vetId.vetId);
        milCollection.deleteMany(eq("personId", vetId.vetId));
        System.out.println("Deleted military: " + vetId.vetId);
        socCollection.deleteMany(eq("personId", vetId.vetId));
        System.out.println("Deleted social: " + vetId.vetId);
        moodCollection.deleteMany(eq("personId", vetId.vetId));
        System.out.println("Deleted mood: " + vetId.vetId);
        
        //Purge all notifications
        notifCollection.deleteMany(eq("vetId", vetId.vetId));
        System.out.println("Deleted notifications: " + vetId.vetId);
        
        //Purge all risk assessments
        raCollection.deleteMany(eq("personId", vetId.vetId));
        System.out.println("Deleted risk assessments: " + vetId.vetId);
        
        //Purge all score request data (metrics)
        dataCollection.deleteMany(eq("vetId", vetId.vetId));
        System.out.println("Deleted score request data: " + vetId.vetId);
        
        //Purge all SMS data
        smsCollection.deleteMany(eq("personId", vetId.vetId));
        System.out.println("Deleted SMS data: " + vetId.vetId);
        
        //Purge all Tweets
        tweetCollection.deleteMany(eq("personId", vetId.vetId));
        System.out.println("Deleted tweets: " + vetId.vetId);
        
        PurgeDataResponse response = new PurgeDataResponse();
        response.success = true;
        response.message = "Success";
        
        return response;
    }
    
    @Path("test")
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    public ScoreResponse testPost(ScoreRequestData data) {
        ScoreResponse response = new ScoreResponse();
        response.success = true;
        response.message = "Success";
        response.currentSessionFlags.add(new Flag("testPost", true));
        return response;
    }
    
    @Path("getrisk")
    @GET
    @Produces(MediaType.APPLICATION_JSON)
    public List<RiskAssessment> getAllRiskAssessments(@QueryParam("id") String id) {
        System.out.println("Getting all risk assessments for " + id);
        // Return all risk assessments for given vetId parameter
        Iterable<RiskAssessment> riskAssessments = raCollection.find(eq("personId", id));
        List<RiskAssessment> raList= new ArrayList<>();
        int counter = 0;
        for (RiskAssessment ra: riskAssessments) {
            raList.add(ra);
            counter++;
        }
        System.out.println("Total number or risk assessments: " + counter);
        return raList;
    }
}

