package com.ibm.grit.rest;

import java.util.HashMap;
import java.util.Map;

import javax.inject.Inject;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import org.kie.api.KieServices;
import org.kie.api.cdi.KReleaseId;
import org.kie.api.cdi.KSession;
import org.kie.api.event.rule.DebugAgendaEventListener;
import org.kie.api.event.rule.DebugRuleRuntimeEventListener;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;

import com.ibm.grit.scoring.BasicNeeds;
import com.ibm.grit.scoring.RiskAssessment;
import com.ibm.grit.scoring.Social;

@Path("test")
public class TestResource {
	
	@GET
	@Path("testget")
	@Produces(MediaType.APPLICATION_JSON)
	public String scoreString() {
		String r = "{\n" + 
				"  \"currentSessionFlags\": [\n" + 
				"    {\n" + 
				"      \"flagName\": \"string\",\n" + 
				"      \"value\": true\n" + 
				"    }\n" + 
				"  ],\n" + 
				"  \"nextSessionFlags\": [\n" + 
				"    {\n" + 
				"      \"flagName\": \"string\",\n" + 
				"      \"value\": true\n" + 
				"    }\n" + 
				"  ]\n" + 
				"}";
		return r;
	}
	
	
	@POST
	@Path("testsimplepost")
	@Produces(MediaType.APPLICATION_JSON)
	public String testSimplePost(String data) {
		return data;
	}
	
	@POST
	@Path("testpost")
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	public ScoreRequestData testPost(ScoreRequestData data) {
		
		return data;
	}
	
	
	@GET
	@Path("testrunrules")
	@Produces(MediaType.TEXT_PLAIN)
	public String testRunRules() {
		System.out.println("DEBUG: Try to get KieServices object");
		// KieServices is the factory for all KIE services 
        KieServices ks = KieServices.Factory.get();
        System.out.println("ks = " + ks);
        
        System.out.println("DEBUG: Try to get KieContainer object");
        // From the kie services, a container is created from the classpath
        KieContainer kc = ks.getKieClasspathContainer();
        System.out.println("kc = " + kc);
        
        System.out.println("DEBUG: Try to get KieSession object");
        // From the container, a session is created based on
        // its definition and configuration in the META-INF/kmodule.xml file
        //KieSession ksession = kc.newKieSession("BasicNeedsKS");
        KieSession ksession = kc.newKieSession("BasicNeedsKS");
		
        BasicNeeds bn = new BasicNeeds();
        Social soc = new Social();
        RiskAssessment riskAssessment = new RiskAssessment();
        
        // Basic Needs
        bn.housing = BasicNeeds.HOMELESS;
        bn.transp = BasicNeeds.UNRELIABLE;
        bn.food = BasicNeeds.OFTEN_HUNGRY;
        bn.finan = BasicNeeds.ONE_TO_TWO;
        
        soc.marital = Social.DIVORCED;
        soc.moMaritalChg = 6;        
        soc.interact = Social.NONE_OR_RARELY;        
        soc.conflict = Social.FREQUENT_OR_REGULAR;        
        soc.relig = Social.YES_UP_TO_TWICE_PER_YEAR;
        soc.edu = Social.PARTIAL_HIGH_SCHOOL;
        
        // Insert facts
        ksession.insert(bn);
        ksession.insert(soc);
        ksession.insert(riskAssessment);
        
        // and fire the rules
        ksession.fireAllRules();

        // Remove comment if using logging
        // logger.close();

        // and then dispose the session
        ksession.dispose();
        
        String msg = "Basic needs risk score: " + riskAssessment.basicNeedsRiskScore + "\n";
		
		return msg;
	}

}
