package com.ibm.grit.utils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ibm.grit.rest.Flag;
import com.ibm.grit.rest.Metric;
import com.ibm.grit.rest.ScoreRequestData;
import com.ibm.grit.scoring.BasicNeeds;
import com.ibm.grit.scoring.Employment;
import com.ibm.grit.scoring.Health;
import com.ibm.grit.scoring.Mental;
import com.ibm.grit.scoring.Military;
import com.ibm.grit.scoring.Social;

public class FlagsUtils {
    
    public static Map<String, Integer> mapMetrics(List<Metric> metricList) {
        Map<String, Integer> metricMap = new HashMap<>();
        metricList.stream().forEach(metric -> metricMap.put(metric.metricName, metric.value));
        return metricMap;
    }
    
    public static void updateBasicNeeds(ScoreRequestData data, BasicNeeds bn) {
        List<Metric> metricList = data.metrics;
        Map<String, Integer> metricMap = mapMetrics(metricList);
        
        bn.personId = data.vetId;
        
        if (metricMap.containsKey("BN_Sex_Ans")) {
            bn.sex = metricMap.get("BN_Sex_Ans");
        }
        if (metricMap.containsKey("BN_Sex_Refuse")) {
            bn.sexRefuse = metricMap.get("BN_Sex_Refuse");
        }
        
        if (metricMap.containsKey("BN_Housing_Ans")) {
            bn.housing = metricMap.get("BN_Housing_Ans");
        }
        if (metricMap.containsKey("BN_Housing_Refuse")) {
            bn.housingRefuse = metricMap.get("BN_Housing_Refuse");
        }
        
        if (metricMap.containsKey("BN_Transp_Ans")) {
            bn.transp = metricMap.get("BN_Transp_Ans");
        }
        if (metricMap.containsKey("BN_Transp_Refuse")) {
            bn.transpRefuse = metricMap.get("BN_Transp_Refuse");
        }
        
        if (metricMap.containsKey("BN_Food_Ans")) {
            bn.food = metricMap.get("BN_Food_Ans");
        }
        if (metricMap.containsKey("BN_Food_Refuse")) {
            bn.foodRefuse = metricMap.get("BN_Food_Refuse");
        }
        
        if (metricMap.containsKey("BN_Finan_Ans")) {
            bn.finan = metricMap.get("BN_Finan_Ans");
        }
        if (metricMap.containsKey("BN_Finan_Refuse")) {
            bn.finanRefuse = metricMap.get("BN_Finan_Refuse");
        }
    }
    
    public static void updateEmployment(ScoreRequestData data, Employment emp) {
        List<Metric> metricList = data.metrics;
        Map<String, Integer> map = mapMetrics(metricList);
        
        emp.personId = data.vetId;
        
        if (map.containsKey("Emp_Employed_Ans")) {
            emp.employed = map.get("Emp_Employed_Ans");
        }
        if (map.containsKey("Emp_Employed_Refuse")) {
            emp.employedRefuse = map.get("Emp_Employed_Refuse");
        }
        
        if (map.containsKey("Emp_MoUnemp_Ans")) {
            emp.moUnemp = map.get("Emp_MoUnemp_Ans");
        }
        if (map.containsKey("Emp_MoUnemp_Refuse")) {
            emp.moUnempRefuse = map.get("Emp_MoUnemp_Refuse");
        }
    }
    
    public static void updateHealth(ScoreRequestData data, Health hea) {
        List<Metric> metricList = data.metrics;
        Map<String, Integer> map = mapMetrics(metricList);
        
        hea.personId = data.vetId;
        
        if (map.containsKey("Hea_Activity_Ans")) {
            hea.activity = map.get("Hea_Activity_Ans");
        }
        if (map.containsKey("Hea_Activity_Refuse")) {
            hea.activityRefuse = map.get("Hea_Activity_Refuse");
        }
        
        if (map.containsKey("Hea_Access_None")) {
            hea.noAccessToHealthServices = map.get("Hea_Access_None");
        }
        if (map.containsKey("Hea_Access_Ans1")) {
            hea.insurance = map.get("Hea_Access_Ans1");
        }
        if (map.containsKey("Hea_Access_Ans2")) {
            hea.doctor = map.get("Hea_Access_Ans2");
        }
        if (map.containsKey("Hea_Access_Ans3")) {
            hea.healthServices = map.get("Hea_Access_Ans3");
        }
        if (map.containsKey("Hea_Access_Ans4")) {
            hea.vaHealthServices = map.get("Hea_Access_Ans4");
        }
        if (map.containsKey("Hea_Access_Refuse")) {
            hea.accessRefuse = map.get("Hea_Access_Refuse");
        }
        
        if (map.containsKey("Hea_ChrCond_None")) {
            hea.noChronicConditions = map.get("Hea_ChrCond_None");
        }
        if (map.containsKey("Hea_ChrCond_Ans1")) {
            hea.cardiovascularDisease = map.get("Hea_ChrCond_Ans1");
        }
        if (map.containsKey("Hea_ChrCond_Ans2")) {
            hea.diabetes = map.get("Hea_ChrCond_Ans2");
        }
        if (map.containsKey("Hea_ChrCond_Ans3")) {
            hea.cancer = map.get("Hea_ChrCond_Ans3");
        }
        if (map.containsKey("Hea_ChrCond_Ans4")) {
            hea.obesity = map.get("Hea_ChrCond_Ans4");
        }
        if (map.containsKey("Hea_ChrCond_Ans5")) {
            hea.arthritis = map.get("Hea_ChrCond_Ans5");
        }
        if (map.containsKey("Hea_ChrCond_Ans6")) {
            hea.asthma = map.get("Hea_ChrCond_Ans6");
        }
        if (map.containsKey("Hea_ChrCond_Ans7")) {
            hea.stroke = map.get("Hea_ChrCond_Ans7");
        }
        if (map.containsKey("Hea_ChrCond_Ans8")) {
            hea.hypertension = map.get("Hea_ChrCond_Ans8");
        }
        if (map.containsKey("Hea_ChrCond_Ans9")) {
            hea.smokingAlcohol = map.get("Hea_ChrCond_Ans9");
        }
        if (map.containsKey("Hea_ChrCond_Ans10")) {
            hea.disability = map.get("Hea_ChrCond_Ans10");
        }
        if (map.containsKey("Hea_ChrCond_Refuse")) {
            hea.chronicConditionRefuse = map.get("Hea_ChrCond_Refuse");
        }
        
        if (map.containsKey("Hea_Compli_Ans")) {
            hea.compliance = map.get("Hea_Compli_Ans");
        }
        if (map.containsKey("Hea_Compli_Refuse")) {
            hea.complianceRefuse = map.get("Hea_Compli_Refuse");
        }
    }
    
    public static void updateMental(ScoreRequestData data, Mental ment) {
        List<Metric> metricList = data.metrics;
        Map<String, Integer> map = mapMetrics(metricList);
        
        ment.personId = data.vetId;
        
        if (map.containsKey("Ment_Diagnos_None")) {
            ment.noMentDiagnosis = map.get("Ment_Diagnos_None");
        }
        if (map.containsKey("Ment_Diagnos_Ans1")) {
            ment.depression = map.get("Ment_Diagnos_Ans1");
        }
        if (map.containsKey("Ment_Diagnos_Ans2")) {
            ment.bipolar = map.get("Ment_Diagnos_Ans2");
        }
        if (map.containsKey("Ment_Diagnos_Ans3")) {
            ment.ptsd = map.get("Ment_Diagnos_Ans3");
        }
        if (map.containsKey("Ment_Diagnos_Ans4")) {
            ment.sleepDisorder = map.get("Ment_Diagnos_Ans4");
        }
        if (map.containsKey("Ment_Diagnos_Ans5")) {
            ment.otherMentDiagnosis = map.get("Ment_Diagnos_Ans5");
        }
        if (map.containsKey("Ment_Diagnos_Ans6")) {
            ment.otherMentDiagnosis = map.get("Ment_Diagnos_Ans6");
        }
        if (map.containsKey("Ment_Diagnos_Refuse")) {
            ment.mentDiagnosisRefuse = map.get("Ment_Diagnos_Refuse");
        }
        
        if (map.containsKey("Ment_Alcohol_Ans")) {
            ment.weeklyAlcoholDrinks = map.get("Ment_Alcohol_Ans");
        }
        if (map.containsKey("Ment_Alcohol_Refuse")) {
            ment.alcoholRefuse = map.get("Ment_Alcohol_Refuse");
        }
        
        if (map.containsKey("Ment_SubstAbuse_None")) {
            ment.noSubstAbuse = map.get("Ment_SubstAbuse_None");
        }
        if (map.containsKey("Ment_SubstAbuse_Ans1")) {
            ment.tobaccoVape = map.get("Ment_SubstAbuse_Ans1");
        }
        if (map.containsKey("Ment_SubstAbuse_Ans2")) {
            ment.opioids = map.get("Ment_SubstAbuse_Ans2");
        }
        if (map.containsKey("Ment_SubstAbuse_Ans3")) {
            ment.cannabis = map.get("Ment_SubstAbuse_Ans3");
        }
        if (map.containsKey("Ment_SubstAbuse_Ans4")) {
            ment.amphetamine = map.get("Ment_SubstAbuse_Ans4");
        }
        if (map.containsKey("Ment_SubstAbuse_Ans5")) {
            ment.cocaine = map.get("Ment_SubstAbuse_Ans5");
        }
        if (map.containsKey("Ment_SubstAbuse_Ans6")) {
            ment.heroin = map.get("Ment_SubstAbuse_Ans6");
        }
        if (map.containsKey("Ment_SubstAbuse_Ans7")) {
            ment.otherSubstAbuse = map.get("Ment_SubstAbuse_Ans7");
        }
        if (map.containsKey("Ment_SubstAbuse_Refuse")) {
            ment.substAbuseRefuse = map.get("Ment_SubstAbuse_Refuse");
        }
        
        if (map.containsKey("Ment_Meds_None")) {
            ment.noMentRx = map.get("Ment_Meds_None");
        }
        if (map.containsKey("Ment_Meds_Ans1")) {
            ment.antidepressants = map.get("Ment_Meds_Ans1");
        }
        if (map.containsKey("Ment_Meds_Ans2")) {
            ment.sleepMeds = map.get("Ment_Meds_Ans2");
        }
        if (map.containsKey("Ment_Meds_Ans3")) {
            ment.otherMentMeds = map.get("Ment_Meds_Ans3");
        }
        if (map.containsKey("Ment_Meds_Refuse")) {
            ment.medsRefuse = map.get("Ment_Meds_Refuse");
        }
        
        if (map.containsKey("Ment_Visits_Ans")) {
            ment.annualMentVisits = map.get("Ment_Visits_Ans");
        }
        if (map.containsKey("Ment_Visits_Refuse")) {
            ment.visitsRefuse = map.get("Ment_Visits_Refuse");
        }
        
        if (map.containsKey("Ment_Visits2_Ans")) {
            ment.annualMentVisits2 = map.get("Ment_Visits2_Ans");
        }
        if (map.containsKey("Ment_Visits2_Refuse")) {
            ment.visitsRefuse2 = map.get("Ment_Visits2_Refuse");
        }
    }
    
    public static void updateMilitary(ScoreRequestData data, Military mil) {
        List<Metric> metricList = data.metrics;
        Map<String, Integer> map = mapMetrics(metricList);
        
        mil.personId = data.vetId;
        
        if (map.containsKey("Mil_Officer_Ans")) {
            mil.officer = map.get("Mil_Officer_Ans");
        }
        if (map.containsKey("Mil_Officer_Refuse")) {
            mil.officerRefuse = map.get("Mil_Officer_Refuse");
        }
        
        if (map.containsKey("Mil_SeriousInj_Ans")) {
            mil.seriousInj = map.get("Mil_SeriousInj_Ans");
        }
        if (map.containsKey("Mil_SeriousInj_Refuse")) {
            mil.seriousInjRefuse = map.get("Mil_SeriousInj_Refuse");
        }
        
        if (map.containsKey("Mil_DischType_Ans")) {
            mil.dischType = map.get("Mil_DischType_Ans");
        }
        if (map.containsKey("Mil_DischType_Refuse")) {
            mil.dischTypeRefuse = map.get("Mil_DischType_Refuse");
        }
        
        if (map.containsKey("Mil_ReductRank_Ans")) {
            mil.reductRank = map.get("Mil_ReductRank_Ans");
        }
        if (map.containsKey("Mil_ReductRank_Refuse")) {
            mil.reductRankRefuse = map.get("Mil_ReductRank_Refuse");
        }
        
        if (map.containsKey("Mil_Combat_Ans")) {
            mil.combat = map.get("Mil_Combat_Ans");
        }
        if (map.containsKey("Mil_Combat_Refuse")) {
            mil.combatRefuse = map.get("Mil_Combat_Refuse");
        }
        
        if (map.containsKey("Mil_Firearms_Ans")) {
            mil.firearms = map.get("Mil_Firearms_Ans");
        }
        if (map.containsKey("Mil_Firearms_Refuse")) {
            mil.firearmsRefuse = map.get("Mil_Firearms_Refuse");
        }
        
        if (map.containsKey("Mil_CloseSuicide_Ans")) {
            mil.closeSuicide = map.get("Mil_CloseSuicide_Ans");
        }
        if (map.containsKey("Mil_CloseSuicide_Refuse")) {
            mil.closeSuicideRefuse = map.get("Mil_CloseSuicide_Refuse");
        }
    }
    
    public static void updateSocial(ScoreRequestData data, Social soc) {
        List<Metric> metricList = data.metrics;
        Map<String, Integer> map = mapMetrics(metricList);
        
        soc.personId = data.vetId;
        
        if (map.containsKey("Soc_Marital_Ans")) {
            soc.marital = map.get("Soc_Marital_Ans");
        }
        if (map.containsKey("Soc_Marital_Refuse")) {
            soc.maritalRefuse = map.get("Soc_Marital_Refuse");
        }
        
        if (map.containsKey("Soc_MoMaritalChg_Ans")) {
            soc.moMaritalChg = map.get("Soc_MoMaritalChg_Ans");
        }
        if (map.containsKey("Soc_MoMaritalChg_Refuse")) {
            soc.moMaritalChgRefuse = map.get("Soc_MoMaritalChg_Refuse");
        }
        
        if (map.containsKey("Soc_Interact_Ans")) {
            soc.interact = map.get("Soc_Interact_Ans");
        }
        if (map.containsKey("Soc_Interact_Refuse")) {
            soc.interactRefuse = map.get("Soc_Interact_Refuse");
        }
        
        if (map.containsKey("Soc_Conflict_Ans")) {
            soc.conflict = map.get("Soc_Conflict_Ans");
        }
        if (map.containsKey("Soc_Conflict_Refuse")) {
            soc.conflictRefuse = map.get("Soc_Conflict_Refuse");
        }
        
        if (map.containsKey("Soc_Relig_Ans")) {
            soc.relig = map.get("Soc_Relig_Ans");
        }
        if (map.containsKey("Soc_Relig_Refuse")) {
            soc.religRefuse = map.get("Soc_Relig_Refuse");
        }
        
        if (map.containsKey("Soc_Edu_Ans")) {
            soc.edu = map.get("Soc_Edu_Ans");
        }
        if (map.containsKey("Soc_Edu_Refuse")) {
            soc.eduRefuse = map.get("Soc_Edu_Refuse");
        }
    }
    
    public static Map<String, Integer> getFlagIndexMap(List<Flag> flags){
        Map<String, Integer> map = new HashMap<>();
        flags.stream()
            .forEach(flag -> map.put(flag.flagName, flags.indexOf(flag)));
        return map;
    }
    
    public static void setFlag(List<Flag> flags, String flagName) {
        Map<String, Integer> map = getFlagIndexMap(flags);
        if (map.containsKey(flagName)) { // Set the existing flag
            flags.get(map.get(flagName)).value = true;
        }
        else {
            flags.add(new Flag(flagName, true));
        }
    }
    
    public static void unsetFlag(List<Flag> flags, String flagName) {
        Map<String, Integer> map = getFlagIndexMap(flags);
        if (map.containsKey(flagName)) { // Unset the existing flag
            flags.get(map.get(flagName)).value = false;
        }
        else {
            flags.add(new Flag(flagName, false));
        }
    }
    
    private static boolean intToBoolean(int b) {
        if (b == 0) {
            return false;
        }
        else {
            return true;
        }
    }
}













