package com.ibm.grit.utils;

import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.apache.commons.math3.stat.regression.SimpleRegression;

import com.google.gson.Gson;

import com.ibm.grit.rest.Notification;
import com.ibm.grit.rest.NotificationUpdate;
import com.ibm.grit.rest.SendSmsRequest;
import com.ibm.grit.scoring.Mood;
import com.ibm.grit.scoring.RiskAssessment;

import static com.ibm.grit.rest.RiskServiceResource.PRODUCTION;
import static com.ibm.grit.utils.NotificationsUtils.addDays;

import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

public class NotificationsUtils {
    public static final MediaType JSON = MediaType.get("application/json; charset=utf-8");
    public static final String EARLIEST_NOTIFICATION_DATE = "2000-01-01T00:00Z";
    
    public static void sendNotification(Notification notification) throws IOException {
        Gson gson = new Gson();
        String notificationJson = gson.toJson(notification);
        
        String uri;
        
        if (PRODUCTION) {
            uri = "http://va-grit-midtier-microservice-internal:5080/internal/notifications/createNotification";
            //uri = "http://169.62.10.213:5080/internal/notifications/createNotification";
        }
        else {
            uri = "http://0.0.0.0/post";
        }
        
        String response = post(uri, notificationJson);
        
        System.out.println(response);
    }
    
    public static void updateNotification(NotificationUpdate nu) throws IOException {
        Gson gson = new Gson();
        String notificationJson = gson.toJson(nu);
        String uri;
        if (PRODUCTION) {
            uri = "http://va-grit-midtier-microservice-internal:5080/internal/notifications/updateNotification";
            //uri = "http://169.62.10.213:5080/internal/notifications/updateNotification";
        }
        else {
            uri = "http://0.0.0.0/post";
        }
        
        String response = post(uri, notificationJson);
        
        System.out.println(response);
    }
    
    public static void sendSmsToSquad(SendSmsRequest sms) throws IOException {
        Gson gson = new Gson();
        String sendSmsRequest = gson.toJson(sms);
        String uri;
        if (PRODUCTION) {
            //uri = "http://169.62.10.213:5080/internal/squad/sendSmsToSquad";
            uri = "http://va-grit-midtier-microservice-internal:5080/internal/squad/sendSmsToSquad";
        }
        else {
            uri = "http://0.0.0.0/post";
        }
        
        String response = post(uri, sendSmsRequest);
        
        System.out.println(response);
    }
    
    private static String post(String url, String json) throws IOException {
        OkHttpClient client = new OkHttpClient();
        
        RequestBody body = RequestBody.create(JSON, json);
        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .build();
        try (Response response = client.newCall(request).execute()) {
            return response.body().string();
        }
    }
    
    // Reference: https://gist.github.com/nickrussler/7527851
    public static String dateToString(Date date) {
        TimeZone tz = TimeZone.getTimeZone("UTC");
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm'Z'");
        df.setTimeZone(tz);
        return df.format(date);
    }
    
    public static Date stringToDate(String dateString) {
        TimeZone tz = TimeZone.getTimeZone("UTC");
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm'Z'");
        df.setTimeZone(tz);
        
        try {
            return df.parse(dateString);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        
        return null;
    }
    
    public static Date addDays(Date date, int days) {
        Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
        cal.setTime(date);
        cal.add(Calendar.DAY_OF_MONTH, days);
        return cal.getTime();
    }
    
    public static Date addSeconds(Date date, int seconds) {
        Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
        cal.setTime(date);
        cal.add(Calendar.SECOND, seconds);
        return cal.getTime();
    }
    
    public static boolean checkForAllLowRisk(List<RiskAssessment> raList) {
        Date now = new Date();
        Date ninetyDaysAgo = addDays(now, -90);
        
        for (RiskAssessment ra: raList) {
            if (ra.basicNeedsRiskLevel.equals("low") && 
                    ra.employabilityRiskLevel.equals("low") &&
                    ra.healthRiskLevel.equals("low") &&
                    ra.socialRiskLevel.equals("low") &&
                    ra.suicideRiskLevel.equals("low") &&
                    ra.timestamp.after(ninetyDaysAgo)) {
                continue;
            }
            else {
                return false;
            }
        }
        return true;
    }
    
    public static Notification getDigitalAssistantNotification(String vetId, Date date) {
        Notification notification = new Notification();
        notification.vetId = vetId;
        notification.notificationId = "DIGITAL_ASSISTANT";
        notification.effectiveDate = date;
        notification.title.en = "Digital Assistant";
        notification.title.es = "Asistente digital";
        notification.description.en = "If things are getting more challenging for you, some resources might help.  Would you like any personalized help from the digital assistant?";
        notification.description.es = "Si las cosas se ponen más difíciles para usted, algunos recursos podrían ayudar. ¿Te gustaría alguna ayuda personalizada del asistente digital?";
        notification.metadata = "METADATA";
        notification.systemNotification = false;
        notification.notificationType = "DIGITAL_ASSISTANT";
        
        return notification;
    }
    
    public static Notification getMoodFacesNotification(String vetId, Date date) {
        Notification notification = new Notification();
        notification.vetId = vetId;
        notification.notificationId = "MOOD_FACES";
        notification.effectiveDate = date;
        notification.title.en = "Time for a check in";
        notification.title.es = "Tiempo para un check in";
        notification.description.en = "How is your mood today?";
        notification.description.es = "¿Como esta tu humor hoy?";
        notification.metadata = "METADATA";
        notification.systemNotification = true;
        notification.notificationType = "MOOD_FACES";
        
        return notification;
    }
    
    public static Notification getLifeChangesNotification(String vetId, Date date) {
        Notification notification = new Notification();
        notification.vetId = vetId;
        notification.notificationId = "LIFE_CHANGES";
        notification.effectiveDate = date;
        notification.title.en = "Life changes";
        notification.title.es = "La vida cambia";
        notification.description.en = "Time to check if you've had any recent life changes.";
        notification.description.es = "Hora de comprobar si has tenido algún cambio reciente en tu vida.";
        notification.metadata = "METADATA";
        notification.systemNotification = true;
        notification.notificationType = "LIFE_CHANGES";
        
        return notification;
    }
    
    public static List<RiskAssessment> getLastTwoRiskAssessments(List<RiskAssessment> raList){
        List<RiskAssessment> lastTwo = new ArrayList<>();
        raList.sort((ra1, ra2) -> ra2.timestamp.compareTo(ra1.timestamp));
        
        lastTwo.add(raList.get(0));
        lastTwo.add(raList.get(1));
        
        //System.out.println(lastTwo.get(0).timestamp);
        //System.out.println(lastTwo.get(1).timestamp);
        
        return lastTwo;
    }
}




