package com.ibm.grit.utils;

import static com.ibm.grit.rest.BlueTas.getTweets;
import static com.ibm.grit.utils.NotificationsUtils.addDays;
import static com.mongodb.client.model.Filters.eq;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.math3.stat.regression.SimpleRegression;

import com.ibm.grit.rest.ScoreRequestData;
import com.ibm.grit.scoring.BasicNeeds;
import com.ibm.grit.scoring.Employment;
import com.ibm.grit.scoring.Health;
import com.ibm.grit.scoring.Mental;
import com.ibm.grit.scoring.Military;
import com.ibm.grit.scoring.Mood;
import com.ibm.grit.scoring.RiskAssessment;
import com.ibm.grit.scoring.Social;
import com.ibm.grit.scoring.Tweet;
import com.mongodb.client.model.FindOneAndReplaceOptions;

public class ScoringUtils {
    
    public static final int FALSE = 0;
    public static final int TRUE = 1;
    
    public static final long SECOND = 1000;
    public static final long MINUTE = 60 * SECOND;
    public static final long HOUR = 60 * MINUTE;
    public static final long DAY = 24 * HOUR;
    public static final long WEEK = 7 * DAY;
    
    public static List<Tweet> parseSocialMedia(ScoreRequestData data){
        /***
         * This function will parse the socialMedia field of the incoming ScoreRequestData.
         * If the field is not present, then it will return an empty List.
         */
        List<Tweet> tweets= new ArrayList<>();
        if (data.socialMedia != null) { // Social media section present
            if (!data.socialMedia.twitter.isEmpty()) {  // There is at least one Twitter handle
                String handle = data.socialMedia.twitter.get(0); // Only use the first Twitter handle
                System.out.println("Retrieving tweets for " + handle);
                
                try {
                    tweets = getTweets(handle.substring(1, handle.length()), data.vetId);
                    
                } catch (Exception e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
        return tweets;
    }
    
    public static double compute30DaySentimentRegressionSlope(List<Tweet> tweets) {
        Date now = new Date();
        Date thirtyDaysAgo = addDays(now, -30);
        SimpleRegression regression = new SimpleRegression();
        for (Tweet tweet: tweets) {
            if (tweet.createdAt.after(thirtyDaysAgo)) {
                // Transform times to 0-based index starting from an offset of -30 days
                Long timeDiff = tweet.createdAt.getTime() - thirtyDaysAgo.getTime();
                double days = (double) timeDiff / (double) DAY;
                
                // Add data to regression model
                regression.addData((double) days,  tweet.sentimentScore) ;
            }
        }
        return regression.getSlope();
    }
    
    public static double compute30DaySentimentMean(List<Tweet> tweets) {
        Date now = new Date();
        Date thirtyDaysAgo = addDays(now, -30);
        
        int count = 0;
        double sum = 0;
        for (Tweet tweet: tweets) {
            if (tweet.createdAt.after(thirtyDaysAgo)) {
                count++;
                sum += tweet.sentimentScore;
            }
        }
        
        return sum/(double) count;
    }
    
    static double computeSentimentScoreForMean(Tweet tweet) {
        // Normalize sentiment score to [0,1] where 0 is positive sentiment and 1 is negative sentiment.
        double score = (-1 * tweet.sentimentScore + 1)/2;
        return score;
    }
    
    public static double compute30DayMoodRegressionSlope(List<Mood> moodList) {
        Date now = new Date();
        Date thirtyDaysAgo = addDays(now, -30);
        SimpleRegression regression = new SimpleRegression();
        for (Mood mood: moodList) {
            if (mood.timestamp.after(thirtyDaysAgo)) {
                System.out.println(mood.timestamp);
                // Transform times to 0-based index starting from an offset of -30 days
                Long timeDiff = mood.timestamp.getTime() - thirtyDaysAgo.getTime();
                //Long days = timeDiff / DAY;
                
                double days = (double) timeDiff / (double) DAY;
                
                //System.out.println("Offset from 30 days ago: " + days);
                
                // Compute scoring value based on answer
                double score = computeMoodScoreFromAnswer(mood);
                
                // Add data to regression model
                regression.addData((double) days,  score) ;
            }
        }
        return regression.getSlope();
    }
    
    public static double compute30DayMoodMean(List<Mood> moodList) {
        Date now = new Date();
        Date thirtyDaysAgo = addDays(now, -30);
        
        int count = 0;
        double sum = 0;
        for (Mood mood: moodList) {
            if (mood.timestamp.after(thirtyDaysAgo)) {
                count++;
                sum += computeMoodScoreFromAnswer(mood);
            }
        }
        
        return sum/(double) count;
    }
    
    public static double computeMoodScoreFromAnswer(Mood mood) {
        return (double) (mood.answer - 1) / 5;
    }
	
    public void minRiskFemale(BasicNeeds bn, Employment emp, Health hea, Mental ment, Military mil, Social soc, RiskAssessment riskAssessment) {
        bn.personId = "MIN_RISK_FEMALE";
        bn.sex = BasicNeeds.FEMALE;
        bn.housing = BasicNeeds.OWN;
        bn.transp = BasicNeeds.VERY_RELIABLE;
        bn.food = BasicNeeds.ABOVE_AVERAGE;
        bn.finan = BasicNeeds.TWO_OR_MORE;
        
        emp.personId = "MIN_RISK_FEMALE";
        emp.employed = Employment.YES_HAVE_JOB_NOW;
        emp.moUnemp = 0;
        
        hea.personId = "MIN_RISK_FEMALE";
        hea.disability = FALSE;
        
        mil.personId = "MIN_RISK_FEMALE";
        mil.seriousInj = Military.NO;
        mil.officer = Military.COMMISSIONED_OFFICER;
        
        ment.personId = "MIN_RISK_FEMALE";
        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.weeklyAlcoholDrinks = 0;
        
        soc.personId = "MIN_RISK_FEMALE";
        soc.edu = Social.ADVANCED;
        soc.conflict = Social.NONE_OR_RARELY;
        
        riskAssessment.personId = "MIN_RISK_FEMALE";
    }
	
    public void maxRiskMale(BasicNeeds bn, Employment emp, Health hea, Mental ment, Military mil, Social soc, RiskAssessment riskAssessment) {
        bn.personId = "MAX_RISK_MALE";
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.HOMELESS;
        bn.transp = BasicNeeds.UNRELIABLE;
        bn.food = BasicNeeds.OFTEN_HUNGRY;
        bn.finan = BasicNeeds.IN_DEBT;
        
        emp.personId = "MAX_RISK_MALE";
        emp.employed = Employment.NO_HAVE_JOB;
        emp.moUnemp = 24;
        
        hea.personId = "MAX_RISK_MALE";
        hea.disability = TRUE;
        hea.activity = Health.SEDENTARY;
        hea.noAccessToHealthServices = TRUE;
        hea.cardiovascularDisease = TRUE;
        hea.diabetes = TRUE;
        hea.cancer = TRUE;
        hea.obesity = TRUE;
        hea.arthritis = TRUE;
        hea.asthma = TRUE;
        hea.stroke = TRUE;
        hea.hypertension = TRUE;
        hea.smokingAlcohol = TRUE;
        hea.disability = TRUE;
        hea.compliance = Health.NEVER_TAKE;
        
        ment.personId = "MAX_RISK_MALE";
        ment.depression = TRUE;
        ment.bipolar = TRUE;
        ment.ptsd = TRUE;
        ment.sleepDisorder = TRUE;
        ment.otherMentDiagnosis = TRUE;
        ment.weeklyAlcoholDrinks = 20;
        ment.tobaccoVape = TRUE;
        ment.opioids = TRUE;
        ment.cannabis = TRUE;
        ment.amphetamine = TRUE;
        ment.cocaine = TRUE;
        ment.heroin = TRUE;
        ment.antidepressants = TRUE;
        ment.sleepMeds = TRUE;
        ment.otherMentMeds = TRUE;
        ment.annualMentVisits = 0;
        
        mil.personId = "MAX_RISK_MALE";
        mil.officer = Military.ENLISTED;
        mil.seriousInj = Military.YES_SERIOUS_INJ;
        mil.dischType = Military.DISHONORABLE;
        mil.reductRank = Military.YES;
        mil.combat = Military.YES;
        
        soc.personId = "MAX_RISK_MALE";
        soc.marital = Social.SEPARATED;
        soc.moMaritalChg = 0;
        soc.interact = Social.NONE_OR_RARELY;
        soc.conflict = Social.FREQUENT_OR_REGULAR;
        soc.relig = Social.YES_UP_TO_TWICE_PER_YEAR;
        soc.edu = Social.PARTIAL_HIGH_SCHOOL;
        
        riskAssessment.personId = "MAX_RISK_MALE";
    }
	
    public void highRiskMale(BasicNeeds bn, Employment emp, Health hea, Mental ment, Military mil, Social soc, RiskAssessment riskAssessment) {
        bn.personId = "HIGH_RISK_MALE";
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.HOMELESS;
        bn.transp = BasicNeeds.OFTEN_UNAVAILABLE;
        bn.food = BasicNeeds.SOMETIMES_MISS;
        bn.finan = BasicNeeds.MINIMAL;
        
        emp.personId = "HIGH_RISK_MALE";
        emp.employed = Employment.NO_HAVE_JOB;
        emp.moUnemp = 12;
        
        hea.personId = "HIGH_RISK_MALE";
        hea.activity = Health.LIGHTLY_ACTIVE;
        hea.noAccessToHealthServices = TRUE;
        hea.asthma = TRUE;
        hea.hypertension = TRUE;
        hea.smokingAlcohol = TRUE;
        hea.disability = TRUE;
        hea.compliance = Health.NO_MEDS;
        
        ment.personId = "HIGH_RISK_MALE";
        ment.depression = TRUE;
        ment.ptsd = TRUE;
        ment.weeklyAlcoholDrinks = 15;
        ment.tobaccoVape = TRUE;
        ment.cannabis = TRUE;
        ment.noMentRx = TRUE;
        ment.annualMentVisits = 0;
        
        mil.personId = "HIGH_RISK_MALE";
        mil.officer = Military.ENLISTED;
        mil.seriousInj = Military.PRIOR_TO_9_11;
        mil.dischType = Military.OTH;
        mil.reductRank = Military.NO;
        mil.combat = Military.YES;
        
        soc.personId = "HIGH_RISK_MALE";
        soc.marital = Social.SEPARATED;
        soc.moMaritalChg = 12;
        soc.interact = Social.NONE_OR_RARELY;
        soc.conflict = Social.OCCASIONAL;
        soc.relig = Social.NO;
        soc.edu = Social.HIGH_SCHOOL;
        
        riskAssessment.personId = "HIGH_RISK_MALE";
    }
	
    public void mediumRiskFemale(BasicNeeds bn, Employment emp, Health hea, Mental ment, Military mil, Social soc, RiskAssessment riskAssessment) {
        bn.personId = "MEDIUM_RISK_FEMALE";
        bn.sex = BasicNeeds.FEMALE;
        bn.housing = BasicNeeds.LIVE_FRIENDS;
        bn.transp = BasicNeeds.LESS_RELIABLE;
        bn.food = BasicNeeds.AVERAGE;
        bn.finan = BasicNeeds.MINIMAL;
        
        emp.personId = "MEDIUM_RISK_FEMALE";
        emp.employed = Employment.YES_DONT_HAVE_JOB;
        emp.moUnemp = 0;
        
        hea.personId = "MEDIUM_RISK_FEMALE";
        hea.activity = Health.ACTIVE;
        hea.healthServices = TRUE;
        hea.obesity = TRUE;
        hea.hypertension = TRUE;
        hea.compliance = Health.USUALLY_TAKE;
        
        ment.personId = "MEDIUM_RISK_FEMALE";
        ment.depression = TRUE;
        ment.weeklyAlcoholDrinks = 6;
        ment.cannabis = TRUE;
        ment.antidepressants = TRUE;
        ment.annualMentVisits = 6;
        
        mil.personId = "MEDIUM_RISK_FEMALE";
        mil.officer = Military.ENLISTED;
        mil.seriousInj = Military.NO_SERIOUS_INJ;
        mil.dischType = Military.GENERAL_UNDER_HONORABLE;
        mil.reductRank = Military.NO;
        mil.combat = Military.YES;
        
        soc.personId = "MEDIUM_RISK_FEMALE";
        soc.marital = Social.DIVORCED;
        soc.moMaritalChg = 18;
        soc.interact = Social.AVERAGE;
        soc.conflict = Social.OCCASIONAL;
        soc.relig = Social.YES_ATTEND_REGULARLY;
        soc.edu = Social.SOME_COLLEGE_OR_ASSOCIATES;
        
        riskAssessment.personId = "MEDIUM_RISK_FEMALE";
    }
	
    public void lowRiskMale(BasicNeeds bn, Employment emp, Health hea, Mental ment, Military mil, Social soc, RiskAssessment riskAssessment) {
        bn.personId = "LOW_RISK_MALE";
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.RENT;
        bn.transp = BasicNeeds.USUALLY_AVAILABLE;
        bn.food = BasicNeeds.AVERAGE;
        bn.finan = BasicNeeds.ONE_TO_TWO;
        
        emp.personId = "LOW_RISK_MALE";
        emp.employed = Employment.YES_HAVE_JOB_NOW;
        emp.moUnemp = 0;
        
        hea.personId = "LOW_RISK_MALE";
        hea.activity = Health.MEDIUM_ACTIVE;
        hea.noAccessToHealthServices = FALSE;
        hea.insurance = TRUE;
        hea.doctor = TRUE;
        hea.healthServices = TRUE;
        hea.hypertension = TRUE;
        hea.compliance = Health.USUALLY_TAKE;
        
        ment.personId = "LOW_RISK_MALE";
        ment.sleepDisorder = TRUE;
        ment.weeklyAlcoholDrinks = 7;
        ment.sleepMeds = TRUE;        
        ment.annualMentVisits = 2;
        
        mil.personId = "LOW_RISK_MALE";
        mil.officer = Military.COMMISSIONED_OFFICER;
        mil.seriousInj = Military.NO_SERIOUS_INJ;
        mil.dischType = Military.HONORABLE;
        mil.reductRank = Military.NO;
        mil.combat = Military.YES;
        
        soc.personId = "LOW_RISK_MALE";
        soc.marital = Social.SINGLE;
        soc.interact = Social.OFTEN;
        soc.conflict = Social.NONE_OR_RARELY;
        soc.relig = Social.NO;
        soc.edu = Social.TRADE_SCHOOL;
        
        riskAssessment.personId = "LOW_RISK_MALE";
    }

}
