package com.ibm.grit.rest;

import static com.ibm.grit.rest.RiskServiceResource.PRODUCTION;
import static com.ibm.grit.utils.ScoringUtils.computeMoodScoreFromAnswer;
import static com.mongodb.client.model.Filters.eq;
import static org.bson.codecs.configuration.CodecRegistries.fromProviders;
import static org.bson.codecs.configuration.CodecRegistries.fromRegistries;
import static org.junit.jupiter.api.Assertions.*;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.bson.codecs.configuration.CodecRegistry;
import org.bson.codecs.pojo.PojoCodecProvider;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.TestMethodOrder;

import com.google.gson.Gson;
import com.ibm.grit.scoring.BasicNeeds;
import com.ibm.grit.scoring.Employment;
import com.ibm.grit.scoring.Health;
import com.ibm.grit.scoring.Mental;
import com.ibm.grit.scoring.Military;
import com.ibm.grit.scoring.Mood;
import com.ibm.grit.scoring.RiskAssessment;
import com.ibm.grit.scoring.Social;
import com.ibm.grit.scoring.Tweet;
import com.mongodb.MongoClientSettings;
import com.mongodb.client.MongoClients;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import com.mongodb.client.model.Sorts;

import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;


@TestMethodOrder(OrderAnnotation.class)
class RestTest {
    public static final MediaType JSON = MediaType.get("application/json; charset=utf-8");
    
    static CodecRegistry pojoCodecRegistry = fromRegistries(com.mongodb.MongoClient.getDefaultCodecRegistry(),
            fromProviders(PojoCodecProvider.builder().automatic(true).build()));

    static MongoClientSettings settings = MongoClientSettings.builder()
            .codecRegistry(pojoCodecRegistry)
            .build();
    static com.mongodb.client.MongoClient mongoClient = MongoClients.create(settings);
    
    static MongoDatabase database = mongoClient.getDatabase("risk");
    static MongoCollection<BasicNeeds> bnCollection = database.getCollection("basicneeds", BasicNeeds.class);
    static MongoCollection<Employment> empCollection = database.getCollection("employment", Employment.class);
    static MongoCollection<Health> heaCollection = database.getCollection("health", Health.class);
    static MongoCollection<Mental> mentCollection = database.getCollection("mental", Mental.class);
    static MongoCollection<Military> milCollection = database.getCollection("military", Military.class);
    static MongoCollection<Social> socCollection = database.getCollection("social", Social.class);
    static MongoCollection<ScoreRequestData> dataCollection = database.getCollection("data", ScoreRequestData.class);
    static MongoCollection<RiskAssessment> raCollection = database.getCollection("assessment", RiskAssessment.class);
    static MongoCollection<Notification> notifCollection = database.getCollection("notification", Notification.class);
    static MongoCollection<Mood> moodCollection = database.getCollection("mood", Mood.class);
    static MongoCollection<Sms> smsCollection = database.getCollection("sms", Sms.class);
    static MongoCollection<Tweet> tweetCollection = database.getCollection("tweet", Tweet.class);

    @BeforeAll
        static void setUpBeforeClass() throws Exception {
    }
    
    @AfterAll
        static void tearDownAfterClass() throws Exception {
    }
    
    @BeforeEach
        void setUp() throws Exception {
        System.out.println("-----------------------------------------------------------------");
    }
    
    @AfterEach
        void tearDown() throws Exception {
    }
    
    @Test
    @Order(1)
    void blCustom1() throws Exception {
        VetId vetId = getVetIdJson("VetId_HRM.json");
        purgeData(vetId);
        
        ScoreRequestData data = createRequestFromFile("BL_CUSTOM_1_HRM.json");
        String response = score(data);
        System.out.println(response);
        
        Iterable<Notification> notificationIter = notifCollection.find(eq("vetId", data.vetId));
        List<String> notificationIds = new ArrayList<>();
        for (Notification notification: notificationIter) {
            notificationIds.add(notification.notificationId);
            System.out.println("notificationId = " + notification.notificationId);
            System.out.println("  notificationType = " + notification.notificationType);
            System.out.println("  effectiveDate = " + notification.effectiveDate);
        }
        
        assert(notificationIds.contains("BL_CUSTOM_2"));
        assert(notificationIds.contains("RA_MENT"));
        assert(notificationIds.contains("LIFE_CHANGES"));
        assert(notificationIds.contains("MOOD_FACES"));
    }
    
    @Test
    @Order(2)
    void blCustom2() throws Exception {
        ScoreRequestData data = createRequestFromFile("BL_CUSTOM_2_HRM.json");
        String response = score(data);
        System.out.println(response);
        
        Iterable<Notification> notificationIter = notifCollection.find(eq("vetId", data.vetId));
        List<String> notificationIds = new ArrayList<>();
        for (Notification notification: notificationIter) {
            notificationIds.add(notification.notificationId);
            System.out.println("notificationId = " + notification.notificationId);
            System.out.println("  notificationType = " + notification.notificationType);
            System.out.println("  effectiveDate = " + notification.effectiveDate);
        }
        
        assert(notificationIds.contains("BL_CUSTOM_3"));
        assert(notificationIds.contains("DIGITAL_ASSISTANT"));
    }
    
    @Test
    @Order(3)
    void blCustom3() throws Exception {
        ScoreRequestData data = createRequestFromFile("BL_CUSTOM_3_HRM.json");
        String response = score(data);
        System.out.println(response);
        
        Iterable<Notification> notificationIter = notifCollection.find(eq("vetId", data.vetId));
        List<String> notificationIds = new ArrayList<>();
        for (Notification notification: notificationIter) {
            notificationIds.add(notification.notificationId);
            System.out.println("notificationId = " + notification.notificationId);
            System.out.println("  notificationType = " + notification.notificationType);
            System.out.println("  effectiveDate = " + notification.effectiveDate);
        }
        
        assert(notificationIds.contains("BL_CUSTOM_4"));
    }
    
    @Test
    @Order(4)
    void blCustom4() throws Exception {
        ScoreRequestData data = createRequestFromFile("BL_CUSTOM_4_HRM.json");
        String response = score(data);
        System.out.println(response);
        
        Iterable<Notification> notificationIter = notifCollection.find(eq("vetId", data.vetId));
        List<String> notificationIds = new ArrayList<>();
        for (Notification notification: notificationIter) {
            notificationIds.add(notification.notificationId);
            System.out.println("notificationId = " + notification.notificationId);
            System.out.println("  notificationType = " + notification.notificationType);
            System.out.println("  effectiveDate = " + notification.effectiveDate);
        }
        
        assert(notificationIds.contains("BL_CUSTOM_4"));
    }
    
    @Test
    @Order(5)
    void moodFaces1() throws Exception {
        ScoreRequestData data = createRequestFromFile("MOOD_FACES_1_HRM.json");
        String response = score(data);
        System.out.println(response);
        
        Iterable<Notification> notificationIter = notifCollection.find(eq("vetId", data.vetId));
        List<String> notificationIds = new ArrayList<>();
        for (Notification notification: notificationIter) {
            notificationIds.add(notification.notificationId);
            System.out.println("notificationId = " + notification.notificationId);
            System.out.println("  notificationType = " + notification.notificationType);
            System.out.println("  effectiveDate = " + notification.effectiveDate);
        }
        
        System.out.println("Getting all mood scores for " + data.vetId);
        // Return all MOOD_FACES answer values for given vetId parameter
        Iterable<Mood> moodIterable = moodCollection.find(eq("personId", data.vetId)).sort(Sorts.descending("timestamp")).limit(30);
        List<MoodScore> moodList= new ArrayList<>();
        int counter = 0;
        for (Mood mood: moodIterable) {
            moodList.add(new MoodScore(mood.timestamp, computeMoodScoreFromAnswer(mood)));
            counter++;
        }
        System.out.println("Total number of mood scores: " + counter);
        
        assert(counter == 1);
    }
    
    @Test
    @Order(6)
    void lifeChanges1() throws Exception {
        ScoreRequestData data = createRequestFromFile("LIFE_CHANGES_1_HRM.json");
        String response = score(data);
        System.out.println(response);
        
        Iterable<Notification> notificationIter = notifCollection.find(eq("vetId", data.vetId));
        List<String> notificationIds = new ArrayList<>();
        for (Notification notification: notificationIter) {
            notificationIds.add(notification.notificationId);
            System.out.println("notificationId = " + notification.notificationId);
            System.out.println("  notificationType = " + notification.notificationType);
            System.out.println("  effectiveDate = " + notification.effectiveDate);
        }
        
        assert(notificationIds.contains("LIFE_CHANGES_FOLLOWUP"));
    }
    
    public static String readFileToString(String file) throws Exception {
        String filePath = new File("").getAbsolutePath();
        BufferedReader br = new BufferedReader(new FileReader(filePath.concat("/src/test/resources/" + file)));
        
        String str;
        while ((str = br.readLine()) != null)
            System.out.println(str);
       return str;
    }
    
    private static VetId getVetIdJson(String jsonFile) throws FileNotFoundException {
        VetId data = new VetId();
        
        String filePath = new File("").getAbsolutePath();
        BufferedReader br;
        Gson gson = new Gson();
        br = new BufferedReader(new FileReader(filePath.concat("/src/test/resources/" + jsonFile)));
        data = gson.fromJson(br, VetId.class);
        return data;
    }

    private static ScoreRequestData createRequestFromFile(String jsonFile) throws FileNotFoundException {
        ScoreRequestData data = new ScoreRequestData();
        
        String filePath = new File("").getAbsolutePath();
        BufferedReader br;
        Gson gson = new Gson();
        br = new BufferedReader(new FileReader(filePath.concat("/src/test/resources/" + jsonFile)));
        data = gson.fromJson(br, ScoreRequestData.class);
        
        System.out.println("vetId: " + data.vetId);
        System.out.println("questionSetId: " + data.questionSetId);
        System.out.println("Metrics:");
        for (Metric metric: data.metrics) {
            System.out.println("  " + metric.metricName + ":" + metric.value);
        }
        return data;
    }
    
    public static String score(ScoreRequestData data) throws IOException {
        Gson gson = new Gson();
        String json = gson.toJson(data);
        
        String uri;
        uri = "http://0.0.0.0:5500/risk-service/score";
        
        String response = post(uri, json);
        
        return response;
    }
    
    public static String purgeData(VetId vetId) throws IOException {
        Gson gson = new Gson();
        String json = gson.toJson(vetId);
        String uri = "http://0.0.0.0:5500/risk-service/purgeData";
        
        String response = post(uri, json);
        
        return response;
    }
    
    public static String getMoodScores(VetId vetId) throws IOException {
        Gson gson = new Gson();
        String json = gson.toJson(vetId);
        String uri = "http://0.0.0.0:5500/risk-service/getMoodScores";
        
        String response = post(uri, json);
        
        return response;
    }

    private static String post(String url, String json) throws IOException {
        OkHttpClient client = new OkHttpClient();
        
        RequestBody body = RequestBody.create(JSON, json);
        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .build();
        try (Response response = client.newCall(request).execute()) {
            return response.body().string();
        }
    }

}
