package com.ibm.grit.scoring;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;
import org.kie.api.KieServices;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.StatelessKieSession;

//@RunWith(JUnitPlatform.class)
class BasicNeedsTest {
    
    static KieServices ks;
    static KieContainer kc;
    KieSession ksession;
    RiskAssessment riskAssessment;
    BasicNeeds bn;

    @BeforeAll
    static void setUpBeforeClass() throws Exception {
        ks = KieServices.Factory.get();
        kc = ks.getKieClasspathContainer();
    }

    @AfterAll
    static void tearDownAfterClass() throws Exception {
    }

    @BeforeEach
    void setUp() throws Exception {
        ksession = kc.newKieSession("BasicNeedsKS");

        riskAssessment = new RiskAssessment();
        riskAssessment.personId = "TEST_ID";

        bn = new BasicNeeds();
        bn.personId = "TEST_ID";
    }

    @AfterEach
        void tearDown() throws Exception {
    }

    @Test
    void minRisk() {
        bn.housing = BasicNeeds.OWN;
        bn.transp = BasicNeeds.VERY_RELIABLE;
        bn.food = BasicNeeds.ABOVE_AVERAGE;
        bn.finan = BasicNeeds.TWO_OR_MORE;

        ksession.insert(bn);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.0, riskAssessment.basicNeedsRiskScore);
        assertEquals("low", riskAssessment.basicNeedsRiskLevel);
    }

    @Test
    void lowRiskMale() {
        riskAssessment.personId = "LOW_RISK_MALE";
        
        bn.personId = "LOW_RISK_MALE";
        bn.housing = BasicNeeds.RENT;
        bn.transp = BasicNeeds.USUALLY_AVAILABLE;
        bn.food = BasicNeeds.AVERAGE;
        bn.finan = BasicNeeds.ONE_TO_TWO;

        ksession.insert(bn);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.22, riskAssessment.basicNeedsRiskScore, 0.001);
        assertEquals("low", riskAssessment.basicNeedsRiskLevel);
    }

    @Test
    void medRiskFemale() {
        riskAssessment.personId = "MEDIUM_RISK_FEMALE";
        
        bn.personId = "MEDIUM_RISK_FEMALE";
        bn.housing = BasicNeeds.LIVE_FRIENDS;
        bn.transp = BasicNeeds.LESS_RELIABLE;
        bn.food = BasicNeeds.AVERAGE;
        bn.finan = BasicNeeds.MINIMAL;

        ksession.insert(bn);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.402, riskAssessment.basicNeedsRiskScore, 0.001);
        assertEquals("medium", riskAssessment.basicNeedsRiskLevel);
    }

    @Test
    void highRiskMale() {
        riskAssessment.personId = "HIGH_RISK_MALE";
        
        bn.personId = "HIGH_RISK_MALE";
        bn.housing = BasicNeeds.HOMELESS;
        bn.transp = BasicNeeds.OFTEN_UNAVAILABLE;
        bn.food = BasicNeeds.SOMETIMES_MISS;
        bn.finan = BasicNeeds.MINIMAL;

        ksession.insert(bn);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.7275, riskAssessment.basicNeedsRiskScore, 0.001);
        assertEquals("high", riskAssessment.basicNeedsRiskLevel);
    }

    @Test
    void maxRisk() {
        //	BasicNeeds bn = new BasicNeeds();
        bn.housing = BasicNeeds.HOMELESS;
        bn.transp = BasicNeeds.UNRELIABLE;
        bn.food = BasicNeeds.OFTEN_HUNGRY;
        bn.finan = BasicNeeds.IN_DEBT;

        ksession.insert(bn);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(1.0, riskAssessment.basicNeedsRiskScore);
        assertEquals("high", riskAssessment.basicNeedsRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.0, 1, low", 
                "0.025, 2, low", 
                "0.025, 3, low",
                "0.0625, 4, low", 
                "0.25, 5, medium"})
    void housing(double score, int housingVal, String cat) {
        bn.housing = housingVal;
        bn.transp = BasicNeeds.VERY_RELIABLE;
        bn.food = BasicNeeds.ABOVE_AVERAGE;
        bn.finan = BasicNeeds.TWO_OR_MORE;

        ksession.insert(bn);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.basicNeedsRiskScore);
        assertEquals(cat, riskAssessment.basicNeedsRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.0, 1, low", 
                "0.0625, 2, low", 
                "0.125, 3, low", 
                "0.1875, 4, low",
                "0.25, 5, medium"})
    void transportation(double score, int transpVal, String cat) {
        bn.housing = BasicNeeds.OWN;
        bn.transp = transpVal;
        bn.food = BasicNeeds.ABOVE_AVERAGE;
        bn.finan = BasicNeeds.TWO_OR_MORE;

        ksession.insert(bn);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.basicNeedsRiskScore);
        assertEquals(cat, riskAssessment.basicNeedsRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.05, 1, low", 
                "0.0, 2, low", 
                "0.05, 3, low", 
                "0.125, 4, low",
                "0.25, 5, medium"})
    void food(double score, int foodVal, String cat) {
        bn.housing = BasicNeeds.OWN;
        bn.transp = BasicNeeds.VERY_RELIABLE;
        bn.food = foodVal;
        bn.finan = BasicNeeds.TWO_OR_MORE;

        ksession.insert(bn);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.basicNeedsRiskScore);
        assertEquals(cat, riskAssessment.basicNeedsRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.0, 1, low", 
                "0.0825, 2, low", 
                "0.165, 3, low", 
                "0.25, 4, medium",})
    void finance(double score, int finanVal, String cat) {
        bn.housing = BasicNeeds.OWN;
        bn.transp = BasicNeeds.VERY_RELIABLE;
        bn.food = BasicNeeds.ABOVE_AVERAGE;
        bn.finan = finanVal;

        ksession.insert(bn);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.basicNeedsRiskScore);
        assertEquals(cat, riskAssessment.basicNeedsRiskLevel);
    }
}
