package com.ibm.grit.scoring;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;
import org.kie.api.KieServices;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;

import com.ibm.grit.utils.ScoringUtils;

class EmployabilityTest {
    public static final int FALSE = 0;
    public static final int TRUE = 1;
	
    static KieServices ks;
    static KieContainer kc;
    KieSession ksession;
    RiskAssessment riskAssessment;
    BasicNeeds bn;
    Employment emp;
    Health hea;
    Military mil;
    Mental ment;
    Social soc;    

    @BeforeAll
    static void setUpBeforeClass() throws Exception {
        ks = KieServices.Factory.get();
        kc = ks.getKieClasspathContainer();
    }

    @AfterAll
        static void tearDownAfterClass() throws Exception {
    }

    @BeforeEach
    void setUp() throws Exception {
        ksession = kc.newKieSession("EmployabilityKS");
        
        riskAssessment = new RiskAssessment();
        riskAssessment.personId = "TEST_ID";
        
        bn = new BasicNeeds();
        bn.personId = "TEST_ID";
        
        emp = new Employment();
        emp.personId = "TEST_ID";
        
        hea = new Health();
        hea.personId = "TEST_ID";
        
        mil = new Military();
        mil.personId = "TEST_ID";
        
        ment = new Mental();
        ment.personId = "TEST_ID";
        
        soc = new Social();
        soc.personId = "TEST_ID";
    }

    @AfterEach
        void tearDown() throws Exception {
    }

    @Test
    void minRisk() {
        ScoringUtils su = new ScoringUtils();
        su.minRiskFemale(bn, emp, hea, ment, mil, soc, riskAssessment);

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.0, riskAssessment.employabilityRiskScore);
        assertEquals("low", riskAssessment.employabilityRiskLevel);
    }

    @Test
    void lowRiskMale() {
        ScoringUtils su = new ScoringUtils();
        su.lowRiskMale(bn, emp, hea, ment, mil, soc, riskAssessment);
        
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.04, riskAssessment.employabilityRiskScore, .001);
        assertEquals("low", riskAssessment.employabilityRiskLevel);
    }
    
    @Test
    void mediumRiskFemale() {
        ScoringUtils su = new ScoringUtils();
        su.mediumRiskFemale(bn, emp, hea, ment, mil, soc, riskAssessment);
        
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.192, riskAssessment.employabilityRiskScore, .001);
        assertEquals("medium", riskAssessment.employabilityRiskLevel);
    }

    
    @Test
    void highRiskMale() {
        ScoringUtils su = new ScoringUtils();
        su.highRiskMale(bn, emp, hea, ment, mil, soc, riskAssessment);
        
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.67, riskAssessment.employabilityRiskScore, .01);
        assertEquals("high", riskAssessment.employabilityRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.00833, 1, low", 
                "0.01667, 2, low", 
                "0.025, 3, low", 
                "0.03333, 4, low",
                "0.04167, 5, low",
                "0.05, 6, low",
                "0.05833, 7, low",
                "0.0667, 8, low",
                "0.075, 9, low",
                "0.0833, 10, low",
                "0.09167, 11, low",
                "0.1, 12, low",
                "0.10833, 13, low",
                "0.11667, 14, low",
                "0.125, 15, low",
                "0.13333, 16, low",
                "0.14167, 17, low",
                "0.15, 18, medium",
                "0.15833, 19, medium",
                "0.16667, 20, medium",
                "0.175, 21, medium",
                "0.18333, 22, medium",
                "0.19167, 23, medium",
                "0.2, 24, medium",
                "0.2, 25, medium",
                })
    void unemployed(double score, int monthsUnemployed, String cat) {
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.OWN;

        emp.employed = Employment.NO_HAVE_JOB;
        emp.moUnemp = monthsUnemployed;

        hea.disability = Health.FALSE;

        mil.seriousInj = Military.NO;
        mil.officer = Military.COMMISSIONED_OFFICER;

        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.weeklyAlcoholDrinks = 0;

        soc.edu = Social.ADVANCED;
        soc.conflict = Social.NONE_OR_RARELY;

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.employabilityRiskScore, 0.001);
        assertEquals(cat, riskAssessment.employabilityRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.0, 0, low", 
                "0.1, 1, low", 
                })
    void disability(double score, int disabilityValue, String cat) {
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.OWN;

        emp.employed = Employment.YES_HAVE_JOB_NOW;
        emp.moUnemp = 0;

        hea.disability = disabilityValue;

        mil.seriousInj = Military.NO;
        mil.officer = Military.COMMISSIONED_OFFICER;

        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.weeklyAlcoholDrinks = 0;

        soc.edu = Social.ADVANCED;
        soc.conflict = Social.NONE_OR_RARELY;

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.employabilityRiskScore, 0.001);
        assertEquals(cat, riskAssessment.employabilityRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.0, 1, low", 
                "0.1, 2, low",
                "0.05, 3, low",
                })
    void milSeriousInjury(double score, int seriousInjVal, String cat) {
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.OWN;

        emp.employed = Employment.YES_HAVE_JOB_NOW;
        emp.moUnemp = 0;

        hea.disability = Health.FALSE;

        mil.seriousInj = seriousInjVal;
        mil.officer = Military.COMMISSIONED_OFFICER;

        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.weeklyAlcoholDrinks = 0;

        soc.edu = Social.ADVANCED;
        soc.conflict = Social.NONE_OR_RARELY;

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.employabilityRiskScore, 0.001);
        assertEquals(cat, riskAssessment.employabilityRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.0, 0, low", 
                "0.06667, 1, low", 
                })
    void manicDepressive(double score, int manicDepressiveValue, String cat) {
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.OWN;

        emp.employed = Employment.YES_HAVE_JOB_NOW;
        emp.moUnemp = 0;

        hea.disability = Health.FALSE;

        mil.seriousInj = Military.NO;
        mil.officer = Military.COMMISSIONED_OFFICER;

        ment.bipolar = manicDepressiveValue;
        ment.ptsd = FALSE;
        ment.weeklyAlcoholDrinks = 0;

        soc.edu = Social.ADVANCED;
        soc.conflict = Social.NONE_OR_RARELY;

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.employabilityRiskScore, 0.001);
        assertEquals(cat, riskAssessment.employabilityRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.0, 0, low", 
                "0.13333, 1, low", 
                })
    void ptsd(double score, int ptsdValue, String cat) {
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.OWN;

        emp.employed = Employment.YES_HAVE_JOB_NOW;
        emp.moUnemp = 0;

        hea.disability = Health.FALSE;

        mil.seriousInj = Military.NO;
        mil.officer = Military.COMMISSIONED_OFFICER;

        ment.bipolar = FALSE;
        ment.ptsd = ptsdValue;
        ment.weeklyAlcoholDrinks = 0;

        soc.edu = Social.ADVANCED;
        soc.conflict = Social.NONE_OR_RARELY;

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.employabilityRiskScore, 0.001);
        assertEquals(cat, riskAssessment.employabilityRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.0, 1, low", 
                "0.03, 2, low",
                "0.1, 3, low",
                })
    void milOfficer(double score, int officerVal, String cat) {
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.OWN;

        emp.employed = Employment.YES_HAVE_JOB_NOW;
        emp.moUnemp = 0;

        hea.disability = Health.FALSE;

        mil.seriousInj = Military.NO_SERIOUS_INJ;
        mil.officer = officerVal;

        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.weeklyAlcoholDrinks = 0;

        soc.edu = Social.ADVANCED;
        soc.conflict = Social.NONE_OR_RARELY;

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.employabilityRiskScore, 0.001);
        assertEquals(cat, riskAssessment.employabilityRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.0, 1, low", 
                "0.0, 2, low",
                "0.01, 3, low",
                "0.025, 4, low",
                "0.05, 5, low",
                "0.1, 6, low"
                })
    void education(double score, int eduVal, String cat) {
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.OWN;

        emp.employed = Employment.YES_HAVE_JOB_NOW;
        emp.moUnemp = 0;

        hea.disability = Health.FALSE;

        mil.seriousInj = Military.NO_SERIOUS_INJ;
        mil.officer = Military.COMMISSIONED_OFFICER;

        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.weeklyAlcoholDrinks = 0;

        soc.edu = eduVal;
        soc.conflict = Social.NONE_OR_RARELY;

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.employabilityRiskScore, 0.001);
        assertEquals(cat, riskAssessment.employabilityRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.0, 1, low", 
                "0.01667, 2, low",
                "0.03333, 3, low",
                "0.05, 4, low",
                "0.06667, 5, low"
                })
    void conflict(double score, int conflictVal, String cat) {
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.OWN;

        emp.employed = Employment.YES_HAVE_JOB_NOW;
        emp.moUnemp = 0;

        hea.disability = Health.FALSE;

        mil.seriousInj = Military.NO_SERIOUS_INJ;
        mil.officer = Military.COMMISSIONED_OFFICER;

        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.weeklyAlcoholDrinks = 0;

        soc.edu = Social.ADVANCED;
        soc.conflict = conflictVal;

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.employabilityRiskScore, 0.001);
        assertEquals(cat, riskAssessment.employabilityRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.0, 1, low", 
                "0.00667, 2, low",
                "0.00667, 3, low",
                "0.01667, 4, low",
                "0.06667, 5, low"
                })
    void housing(double score, int housingVal, String cat) {
        bn.sex = BasicNeeds.MALE;
        bn.housing = housingVal;

        emp.employed = Employment.YES_HAVE_JOB_NOW;
        emp.moUnemp = 0;

        hea.disability = Health.FALSE;

        mil.seriousInj = Military.NO_SERIOUS_INJ;
        mil.officer = Military.COMMISSIONED_OFFICER;

        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.weeklyAlcoholDrinks = 0;

        soc.edu = Social.ADVANCED;
        soc.conflict = Social.NONE_OR_RARELY;

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.employabilityRiskScore, 0.001);
        assertEquals(cat, riskAssessment.employabilityRiskLevel);
    }

    @ParameterizedTest
    @CsvSource({"0.00333, 1, low", 
                "0.00667, 2, low", 
                "0.01, 3, low", 
                "0.01333, 4, low",
                "0.01667, 5, low",
                "0.02, 6, low",
                "0.02333, 7, low",
                "0.02667, 8, low",
                "0.03, 9, low",
                "0.03333, 10, low",
                "0.03667, 11, low",
                "0.04, 12, low",
                "0.04333, 13, low",
                "0.04667, 14, low",
                "0.05, 15, low",
                "0.05333, 16, low",
                "0.05667, 17, low",
                "0.06, 18, low",
                "0.06333, 19, low",
                "0.06667, 20, low",
                "0.06667, 21, low"
                })
    void alcoholMale(double score, int drinksPerWeek, String cat) {
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.OWN;

        emp.employed = Employment.YES_HAVE_JOB_NOW;
        emp.moUnemp = 0;

        hea.disability = Health.FALSE;

        mil.seriousInj = Military.NO;
        mil.officer = Military.COMMISSIONED_OFFICER;

        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.weeklyAlcoholDrinks = drinksPerWeek;

        soc.edu = Social.ADVANCED;
        soc.conflict = Social.NONE_OR_RARELY;

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.employabilityRiskScore, 0.001);
        assertEquals(cat, riskAssessment.employabilityRiskLevel);
    }
}
