package com.ibm.grit.scoring;

import static org.junit.jupiter.api.Assertions.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DynamicTest;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.RepeatedTest;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestFactory;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;
import org.kie.api.KieServices;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;

import com.ibm.grit.utils.ScoringUtils;

class HealthTest {
	
    static KieServices ks;
    static KieContainer kc;
    KieSession ksession;
    RiskAssessment riskAssessment;
    BasicNeeds bn;
    Employment emp;
    Health hea;
    Military mil;
    Mental ment;
    Social soc;  

    @BeforeAll
    static void setUpBeforeClass() throws Exception {
        ks = KieServices.Factory.get();
        kc = ks.getKieClasspathContainer();
    }

    @AfterAll
        static void tearDownAfterClass() throws Exception {
    }

    @BeforeEach
    void setUp() throws Exception {
        ksession = kc.newKieSession("HealthKS");

        riskAssessment = new RiskAssessment();
        riskAssessment.personId = "TEST_ID";

        bn = new BasicNeeds();
        bn.personId = "TEST_ID";

        emp = new Employment();
        emp.personId = "TEST_ID";

        hea = new Health();
        hea.personId = "TEST_ID";

        mil = new Military();
        mil.personId = "TEST_ID";

        ment = new Mental();
        ment.personId = "TEST_ID";

        soc = new Social();
        soc.personId = "TEST_ID";
    }

    @AfterEach
        void tearDown() throws Exception {
    }

    @Test
    void minRiskFemale() {
        ScoringUtils su = new ScoringUtils();
        su.minRiskFemale(bn, emp, hea, ment, mil, soc, riskAssessment);

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();
        
        assertEquals(0.0, riskAssessment.healthRiskScore, .0001);
        assertEquals("low", riskAssessment.healthRiskLevel);
    }
	
    @Test
    void lowRiskMale() {
        ScoringUtils su = new ScoringUtils();
        su.lowRiskMale(bn, emp, hea, ment, mil, soc, riskAssessment);
        
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.196, riskAssessment.healthRiskScore, .0001);
        assertEquals("low", riskAssessment.healthRiskLevel);
    }
	
    @Test
    void mediumRiskMale() {
        ScoringUtils su = new ScoringUtils();
        su.mediumRiskFemale(bn, emp, hea, ment, mil, soc, riskAssessment);
                
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.33, riskAssessment.healthRiskScore, .0001);
        assertEquals("medium", riskAssessment.healthRiskLevel);
    }

    @Test
    void highRiskMale() {
        ScoringUtils su = new ScoringUtils();
        su.highRiskMale(bn, emp, hea, ment, mil, soc, riskAssessment);
        
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.718, riskAssessment.healthRiskScore, .0001);
        assertEquals("high", riskAssessment.healthRiskLevel);
    }

    @Test
    void maxRiskMale() {
        ScoringUtils su = new ScoringUtils();
        su.maxRiskMale(bn, emp, hea, ment, mil, soc, riskAssessment);
        
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(1.0, riskAssessment.healthRiskScore, .0001);
        assertEquals("high", riskAssessment.healthRiskLevel);
    }
	
    @RepeatedTest(16)
    void chronicConditions() {
        System.out.println("TEST <==> START repeated test with random chronic Conditions <==> TEST");
        double[] scores = {0.08, 0.16, 0.24, 0.32, 0.4 , 0.4 , 0.4 , 0.4 , 0.4 , 0.4};
        String[] categories = {"low", "low", "medium", "medium", "high","high","high","high","high","high"};
        
        //boolean[] booleans = new boolean[10];
        int[] booleans = new int[10];
        Random random = new Random();
        int trueCount = 0;

        while (trueCount < 1) {
                trueCount = 0;
                for (int i = 0; i < 10; i++) {
                //booleans[i] = random.nextBoolean();
                booleans[i] = random.nextInt(2);
                if (booleans[i] == 1) {
                        trueCount++;
                }
            }
        }
        System.out.println(" >>> trueCount = " + trueCount);
        System.out.println(" >>> booleans: " + Arrays.toString(booleans));

        hea.noChronicConditions = Health.FALSE;
        hea.cardiovascularDisease = booleans[0];
        hea.diabetes = booleans[1];
        hea.cancer = booleans[2];
        hea.obesity = booleans[3];
        hea.arthritis = booleans[4];
        hea.asthma = booleans[5];
        hea.stroke = booleans[6];
        hea.hypertension = booleans[7];
        hea.smokingAlcohol = booleans[8];
        hea.disability = booleans[9];

        hea.activity = Health.ACTIVE;
        hea.noAccessToHealthServices = Health.FALSE;
        hea.doctor = Health.TRUE;
        hea.insurance = Health.TRUE;
        hea.healthServices = Health.TRUE;
        hea.compliance = Health.ALWAYS_TAKE;

        ksession.insert(hea);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        // Subtract 1 from trueCount to index correctly
        assertEquals(scores[trueCount - 1], riskAssessment.healthRiskScore, 0.001);
        assertEquals(categories[trueCount - 1], riskAssessment.healthRiskLevel);
        System.out.println("TEST <==> END repeated test with random chronic Conditions <==> TEST");
    }
	
	
    @TestFactory
    Stream<DynamicTest> dynamicActivity() {

        List<Integer> activityLevels = Arrays.asList(1, 2, 3, 4, 5);
        List<Double> adjustments = Arrays.asList(1.0, 0.66, 0.33, 0.0, 0.33);

        for (int i = 0; i < adjustments.size(); i++) {
                adjustments.set(i, adjustments.get(i) * 0.2);
                //System.out.println(adjustments.get(i));
        }

        return activityLevels.stream()
                .map(level -> DynamicTest.dynamicTest("Activity level: " + level, () -> 
                                                {
                                                    int i = activityLevels.indexOf(level);
                                                    assertEquals(adjustments.get(i), getActivityScore(level), 0.001);
                                                }
                                            ));
    }

    private double getActivityScore(int activityLevel){
        ksession = kc.newKieSession("HealthKS");

        hea.activity = activityLevel;
        hea.noAccessToHealthServices = Health.FALSE;
        hea.doctor = Health.TRUE;
        hea.insurance = Health.TRUE;
        hea.healthServices = Health.TRUE;
        hea.compliance = Health.ALWAYS_TAKE;

        ksession.insert(hea);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        return riskAssessment.getHealthRiskScore();
    }
	
	
    @TestFactory
    Stream<DynamicTest> dynamicActivityCompliance() {

        double[][] scores = new double[5][5];

        List<Integer[]> tuples = new ArrayList<Integer[]>();

        List<Integer> activityLevels = Arrays.asList(1, 2, 3, 4, 5);
        List<Integer> complianceLevels = Arrays.asList(1, 2, 3, 4, 5);
        List<Double> activityAdjustments = Arrays.asList(1.0, 0.66, 0.33, 0.0, 0.33);
        List<Double> complianceAdjustments = Arrays.asList(0.33, 0.0, 0.25, 0.75, 1.0);
        
        // Initialize list of arrays
        for (int i = 0; i < activityLevels.size(); i++) {
            for (int j = 0; j < complianceLevels.size(); j++) {
                Integer[] t = {activityLevels.get(i), complianceLevels.get(j)};
                tuples.add(t);
            }
        }

        // weight activity adjustments
        for (int i = 0; i < activityAdjustments.size(); i++) {
            activityAdjustments.set(i, activityAdjustments.get(i) * 0.2);
        }

        // weight compliance adjustments
        for (int i = 0; i < complianceAdjustments.size(); i++) {
            complianceAdjustments.set(i, complianceAdjustments.get(i) * 0.2);
        }

        // Initialize array of scores
        for (int i = 0; i < activityLevels.size(); i++) {
            for (int j = 0; j < complianceLevels.size(); j++) {
                    scores[i][j] = activityAdjustments.get(i) + complianceAdjustments.get(j);
            }
        }

        return tuples.stream()
                .map(combo -> DynamicTest.dynamicTest("Levels: [activity, compliance] = " + Arrays.toString(combo), () -> 
                                        {
                                                int i = activityLevels.indexOf(combo[0]);
                                                int j = complianceLevels.indexOf(combo[1]);
                                                assertEquals(scores[i][j], getActivityComplianceScore(activityLevels.get(i), complianceLevels.get(j)), 0.001);
                                        }
                                        ));
    }
	
    private double getActivityComplianceScore(int activityLevel, int complianceLevel){
        ksession = kc.newKieSession("HealthKS");
        hea.activity = activityLevel;

        hea.noAccessToHealthServices = Health.FALSE;
        hea.doctor = Health.TRUE;
        hea.insurance = Health.TRUE;
        hea.healthServices = Health.TRUE;

        hea.compliance = complianceLevel;

        ksession.insert(hea);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        return riskAssessment.getHealthRiskScore();
    }
	
    @Test
    void noAccess() {
        hea.noAccessToHealthServices = Health.TRUE;
        hea.insurance = Health.FALSE;
        hea.doctor = Health.FALSE;
        hea.healthServices = Health.FALSE;

        ksession.insert(hea);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.2, riskAssessment.healthRiskScore, .0001);
        assertEquals("medium", riskAssessment.healthRiskLevel);
    }

    @RepeatedTest(8)
    void access() {
        System.out.println("TEST <==> START repeated test with random access levels <==> TEST");
        double[] scores = {0.12, 0.06, 0.0};
        String[] categories = {"low", "low", "low"};

        int boolSize = 3;
        //boolean[] booleans = new boolean[boolSize];
        int[] booleans = new int[boolSize];
        Random random = new Random();
        int trueCount = 0;

        while (trueCount < 1) {
            trueCount = 0;
            for (int i = 0; i < boolSize; i++) {
                booleans[i] = random.nextInt(2);
                if (booleans[i] == 1) {
                        trueCount++;
                }
            }
        }
        System.out.println(" >>> trueCount = " + trueCount);
        System.out.println(" >>> booleans: " + Arrays.toString(booleans));

        hea.noAccessToHealthServices = Health.FALSE;
        hea.insurance = booleans[0];
        hea.doctor = booleans[1];
        hea.healthServices = booleans[2];

        ksession.insert(hea);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        // Subtract 1 from trueCount to index correctly
        assertEquals(scores[trueCount - 1], riskAssessment.healthRiskScore, 0.001);
        assertEquals(categories[trueCount - 1], riskAssessment.healthRiskLevel);
        System.out.println("TEST <==> END repeated test with random access levels <==> TEST");
    }
}







