package com.ibm.grit.scoring;

import org.kie.api.KieServices;
import org.kie.api.event.rule.DebugAgendaEventListener;
import org.kie.api.event.rule.DebugRuleRuntimeEventListener;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.StatelessKieSession;

import  com.ibm.grit.utils.ScoringUtils;

public class ManualEmployabilityTest {
    public static final int FALSE = 0;
    public static final int TRUE = 1;
	public static final void main(final String[] args) {
        // KieServices is the factory for all KIE services 
        KieServices ks = KieServices.Factory.get();
        
        // From the kie services, a container is created from the classpath
        KieContainer kc = ks.getKieClasspathContainer();

        // From the container, a session is created based on
        // its definition and configuration in the META-INF/kmodule.xml file
        KieSession ksession = kc.newKieSession("EmployabilityKS");

        // The application can also setup listeners
        //ksession.addEventListener( new DebugAgendaEventListener() );
        //ksession.addEventListener( new DebugRuleRuntimeEventListener() );

        // To setup a file based audit logger, uncomment the next line
        // KieRuntimeLogger logger = ks.getLoggers().newFileLogger( ksession, "manualtest" );

        // To setup a ThreadedFileLogger, so that the audit view reflects events whilst debugging,
        // uncomment the next line
        // KieRuntimeLogger logger = ks.getLoggers().newThreadedFileLogger( ksession, "./helloworld", 1000 );

        // The application can insert facts into the session
        BasicNeeds bn = new BasicNeeds();
        Employment emp = new Employment();
        Health hea = new Health();
        Military mil = new Military();
        Mental ment = new Mental();
        Social soc = new Social();
        RiskAssessment riskAssessment = new RiskAssessment();
        
        riskAssessment.personId = "TEST_ID";
        
        bn.personId = "TEST_ID";
        bn.sex = BasicNeeds.MALE;
        bn.housing = BasicNeeds.OWN;
        
        emp.personId = "TEST_ID";
        emp.employed = Employment.YES_HAVE_JOB_NOW;
        emp.moUnemp = 0;
        
        hea.personId = "TEST_ID";
        hea.disability = Health.FALSE;
        
        mil.personId = "TEST_ID";
        mil.seriousInj = Military.NO;
        mil.officer = Military.COMMISSIONED_OFFICER;
        
        ment.personId = "TEST_ID";
        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.weeklyAlcoholDrinks = 0;
        
        soc.personId = "TEST_ID";
        soc.edu = Social.ADVANCED;
        soc.conflict = Social.NONE_OR_RARELY;
        
        
        ScoringUtils su = new ScoringUtils();
        //su.highRiskMale(bn, emp, hea, ment, mil, soc);
        //su.maxRiskMale(bn, emp, hea, ment, mil, soc);
        su.mediumRiskFemale(bn, emp, hea, ment, mil, soc, riskAssessment);
        //su.lowRiskMale(bn, emp, hea, ment, mil, soc);
                
        // Stateful execution
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);
        
        // and fire the rules
        ksession.fireAllRules();

        // Remove comment if using logging
        // logger.close();

        // and then dispose the session
        ksession.dispose();
    }

}
