package com.ibm.grit.scoring;

import static com.ibm.grit.utils.NotificationsUtils.stringToDate;
import static com.ibm.grit.utils.ScoringUtils.compute30DayMoodRegressionSlope;

import org.apache.commons.math3.stat.regression.SimpleRegression;
import org.kie.api.KieServices;
import org.kie.api.event.rule.DebugAgendaEventListener;
import org.kie.api.event.rule.DebugRuleRuntimeEventListener;
import org.kie.api.logger.KieRuntimeLogger;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.StatelessKieSession;

import com.ibm.grit.utils.ScoringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.ibm.grit.utils.NotificationsUtils.addDays;;

public class ManualMoodTest {
    public static final long SECOND = 1000;
    public static final long MINUTE = 60 * SECOND;
    public static final long HOUR = 60 * MINUTE;
    public static final long DAY = 24 * HOUR;
    
    public static final void main(final String[] args) {
        // KieServices is the factory for all KIE services 
        KieServices ks = KieServices.Factory.get();
        
        // From the kie services, a container is created from the classpath
        KieContainer kc = ks.getKieClasspathContainer();

        // From the container, a session is created based on
        // its definition and configuration in the META-INF/kmodule.xml file
        KieSession ksession = kc.newKieSession("MoodKS");

        // The application can also setup listeners
        ksession.addEventListener( new DebugAgendaEventListener() );
        ksession.addEventListener( new DebugRuleRuntimeEventListener() );

        // To setup a file based audit logger, uncomment the next line
        // KieRuntimeLogger logger = ks.getLoggers().newFileLogger( ksession, "manualtest" );

        // To setup a ThreadedFileLogger, so that the audit view reflects events whilst debugging,
        // uncomment the next line
        // KieRuntimeLogger logger = ks.getLoggers().newThreadedFileLogger( ksession, "./helloworld", 1000 );
        
        
        String id = "TEST_ID";
        
        RiskAssessment riskAssessment = new RiskAssessment();
        riskAssessment.personId = id;
        
        // Mood
        Date now = new Date();
        List<Mood> moodList = new ArrayList<>();
        //moodList.add(new Mood(id, addDays(now, -49), 1));
        moodList.add(new Mood(id, addDays(now, -21), 1));
        moodList.add(new Mood(id, addDays(now, -14), 2));
        moodList.add(new Mood(id, addDays(now, -13), 3));
        moodList.add(new Mood(id, addDays(now, -7), 2));
        moodList.add(new Mood(id, addDays(now, -2), 2));
        
        
        // Insert facts
        ksession.insert(riskAssessment);
        for (Mood mood: moodList) {
            ksession.insert(mood);
        }
        
        // and fire the rules
        ksession.fireAllRules();

        // Remove comment if using logging
        // logger.close();

        // and then dispose the session
        ksession.dispose();
    }

}