package com.ibm.grit.scoring;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;
import org.kie.api.KieServices;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;

import com.ibm.grit.utils.ScoringUtils;

class SocialTest {
    static KieServices ks;
    static KieContainer kc;
    KieSession ksession;
    RiskAssessment riskAssessment;
    BasicNeeds bn;
    Employment emp;
    Health hea;
    Military mil;
    Mental ment;
    Social soc;
/*
    @BeforeAll
    static void setUpBeforeClass() throws Exception {
            ks = KieServices.Factory.get();
    kc = ks.getKieClasspathContainer();
    }

    @AfterAll
    static void tearDownAfterClass() throws Exception {
    }

    @BeforeEach
    void setUp() throws Exception {
        ksession = kc.newKieSession("SocialKS");
        riskAssessment = new RiskAssessment();
        riskAssessment.personId = "TEST_ID";
        
        bn = new BasicNeeds();
        bn.personId = "TEST_ID";
        
        emp = new Employment();
        emp.personId = "TEST_ID";
        
        hea = new Health();
        hea.personId = "TEST_ID";
        
        mil = new Military();
        mil.personId = "TEST_ID";
        
        ment = new Mental();
        ment.personId = "TEST_ID";
        
        soc = new Social();
        soc.personId = "TEST_ID";
    }

    @AfterEach
    void tearDown() throws Exception {
    }

    @Test
    void minRisk() {

        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;        
        soc.interact = Social.FREQUENT_OR_REGULAR;        
        soc.conflict = Social.NONE_OR_RARELY;
        soc.relig = Social.YES_BUT_NOT_AFFILIATED;
        soc.edu = Social.ADVANCED;

        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.0, riskAssessment.socialRiskScore);
        assertEquals("low", riskAssessment.socialRiskLevel);        
    }
	
    @Test
    void lowRiskMale() {
        ScoringUtils su = new ScoringUtils();
        su.lowRiskMale(bn, emp, hea, ment, mil, soc, riskAssessment);

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.094, riskAssessment.socialRiskScore, .001);
        assertEquals("low", riskAssessment.socialRiskLevel);
    }
	
    @Test
    void mediumRiskFemale() {
        ScoringUtils su = new ScoringUtils();
        su.mediumRiskFemale(bn, emp, hea, ment, mil, soc, riskAssessment);

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.361, riskAssessment.socialRiskScore, .001);
        assertEquals("medium", riskAssessment.socialRiskLevel);
    }
	
    @Test
    void highRiskMale() {
        ScoringUtils su = new ScoringUtils();
        su.highRiskMale(bn, emp, hea, ment, mil, soc, riskAssessment);

        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.611, riskAssessment.socialRiskScore, .001);
        assertEquals("high", riskAssessment.socialRiskLevel);
    }
	
    @Test
    void maxRisk() {
        soc.marital = Social.DIVORCED;
        soc.moMaritalChg = 6;        
        soc.interact = Social.NONE_OR_RARELY;        
        soc.conflict = Social.FREQUENT_OR_REGULAR;        
        soc.relig = Social.YES_UP_TO_TWICE_PER_YEAR;
        soc.edu = Social.PARTIAL_HIGH_SCHOOL;

        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(1.0, riskAssessment.socialRiskScore);
        assertEquals("high", riskAssessment.socialRiskLevel);        
    }
	
    @ParameterizedTest
    @CsvSource({"0.0, 1, low", 
                "0.0, 2, low",
                "0.11111, 3, low",
                "0.11111, 4, low",
                "0.11111, 5, low",})
    void marital24Months(double score, int maritalVal, String cat) {
        soc.marital = maritalVal;
        soc.moMaritalChg = 24;        
        soc.interact = Social.FREQUENT_OR_REGULAR;        
        soc.conflict = Social.NONE_OR_RARELY;
        soc.relig = Social.YES_BUT_NOT_AFFILIATED;
        soc.edu = Social.ADVANCED;

        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.socialRiskScore, 0.001);
        assertEquals(cat, riskAssessment.socialRiskLevel);
    }
	
    @ParameterizedTest
    @CsvSource({"0.0, 1, low", 
                "0.0, 2, low",
                "0.22222, 3, medium",
                "0.22222, 4, medium",
                "0.22222, 5, medium",})
    void maritalTwelveMonths(double score, int maritalVal, String cat) {
        soc.marital = maritalVal;
        soc.moMaritalChg = 12;        
        soc.interact = Social.FREQUENT_OR_REGULAR;        
        soc.conflict = Social.NONE_OR_RARELY;
        soc.relig = Social.YES_BUT_NOT_AFFILIATED;
        soc.edu = Social.ADVANCED;

        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.socialRiskScore, 0.001);
        assertEquals(cat, riskAssessment.socialRiskLevel);
    }
	
    @ParameterizedTest
    @CsvSource({"0.22222, 1, medium", 
                "0.16667, 2, medium",
                "0.11111, 3, low",
                "0.05555, 4, low",
                "0.0, 5, low",})
    void interaction(double score, int interactVal, String cat) {
        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;        
        soc.interact = interactVal;        
        soc.conflict = Social.NONE_OR_RARELY;
        soc.relig = Social.YES_BUT_NOT_AFFILIATED;
        soc.edu = Social.ADVANCED;

        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.socialRiskScore, 0.001);
        assertEquals(cat, riskAssessment.socialRiskLevel);
    }
	
    @ParameterizedTest
    @CsvSource({"0.0, 1, low",				 
                "0.08333, 2, low",
                "0.16667, 3, medium",				
                "0.25, 4, medium",
                "0.33333, 5, medium",
                })
    void conflict(double score, int conflictVal, String cat) {
        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;        
        soc.interact = Social.FREQUENT_OR_REGULAR;        
        soc.conflict = conflictVal;
        soc.relig = Social.YES_BUT_NOT_AFFILIATED;
        soc.edu = Social.ADVANCED;

        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.socialRiskScore, 0.001);
        assertEquals(cat, riskAssessment.socialRiskLevel);
    }
	
    @ParameterizedTest
    @CsvSource({"0.02778, 1, low",				 
                "0.0, 2, low",
                "0.02778, 3, low",				
                "0.05556, 4, low",
                "0.08333, 5, low",
                "0.11111, 6, low",
                })
    void religion(double score, int religionVal, String cat) {
        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;        
        soc.interact = Social.FREQUENT_OR_REGULAR;        
        soc.conflict = Social.NONE_OR_RARELY;
        soc.relig = religionVal;
        soc.edu = Social.ADVANCED;

        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.socialRiskScore, 0.001);
        assertEquals(cat, riskAssessment.socialRiskLevel);
    }
	
    @ParameterizedTest
    @CsvSource({"0.0, 1, low",				 
                "0.0, 2, low",
                "0.01111, 3, low",				
                "0.02778, 4, low",
                "0.05556, 5, low",
                "0.11111, 6, low",
                })
    void education(double score, int eduVal, String cat) {
        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;        
        soc.interact = Social.FREQUENT_OR_REGULAR;        
        soc.conflict = Social.NONE_OR_RARELY;
        soc.relig = Social.YES_BUT_NOT_AFFILIATED;
        soc.edu = eduVal;

        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(score, riskAssessment.socialRiskScore, 0.001);
        assertEquals(cat, riskAssessment.socialRiskLevel);
    }
*/

}
