package com.ibm.grit.scoring;

import static org.junit.jupiter.api.Assertions.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.RepeatedTest;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;
import org.kie.api.KieServices;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;

import com.ibm.grit.utils.ScoringUtils;

class SuicideTest {
    public static final int FALSE = 0;
    public static final int TRUE = 1;
    
    static KieServices ks;
    static KieContainer kc;
    KieSession ksession;
    RiskAssessment riskAssessment;
    BasicNeeds bn;
    Employment emp;
    Health hea;
    Military mil;
    Mental ment;
    Social soc;  
/* Commenting out 
    @BeforeAll
    static void setUpBeforeClass() throws Exception {
        ks = KieServices.Factory.get();
        kc = ks.getKieClasspathContainer();
    }

    @AfterAll
        static void tearDownAfterClass() throws Exception {
    }

    @BeforeEach
    void setUp() throws Exception {
        ksession = kc.newKieSession("SuicideKS");
        riskAssessment = new RiskAssessment();
        riskAssessment.personId = "TEST_ID";
        
        bn = new BasicNeeds();
        bn.personId = "TEST_ID";
        
        emp = new Employment();
        emp.personId = "TEST_ID";
        
        hea = new Health();
        hea.personId = "TEST_ID";
        
        mil = new Military();
        mil.personId = "TEST_ID";
        
        ment = new Mental();
        ment.personId = "TEST_ID";
        
        soc = new Social();
        soc.personId = "TEST_ID";
    }

    @AfterEach
        void tearDown() throws Exception {
    }

    @Test
    void minRiskFemale() {
        ScoringUtils su = new ScoringUtils();
        su.minRiskFemale(bn, emp, hea, ment, mil, soc, riskAssessment);
        
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.0, riskAssessment.suicideRiskScore, .0001);
        assertEquals("low", riskAssessment.suicideRiskLevel);
    }
    
    @Test
    void lowRiskMale() {
        ScoringUtils su = new ScoringUtils();
        su.lowRiskMale(bn, emp, hea, ment, mil, soc, riskAssessment);
        
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.17741, riskAssessment.suicideRiskScore, .0001);
        assertEquals("low", riskAssessment.suicideRiskLevel);
    }
    
    @Test
    void mediumRiskFemale() {
        ScoringUtils su = new ScoringUtils();
        su.mediumRiskFemale(bn, emp, hea, ment, mil, soc, riskAssessment);
        
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.30167, riskAssessment.suicideRiskScore, .0001);
        assertEquals("medium", riskAssessment.suicideRiskLevel);
    }

    @Test
    void highRiskMale() {
        ScoringUtils su = new ScoringUtils();
        su.highRiskMale(bn, emp, hea, ment, mil, soc, riskAssessment);
        
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(0.527, riskAssessment.suicideRiskScore, .001);
        assertEquals("high", riskAssessment.suicideRiskLevel);
    }
	
    @Test
    void maxRiskMale() {
        ScoringUtils su = new ScoringUtils();
        su.maxRiskMale(bn, emp, hea, ment, mil, soc, riskAssessment);
        
        ksession.insert(bn);
        ksession.insert(emp);
        ksession.insert(hea);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);

        ksession.fireAllRules();
        ksession.dispose();

        assertEquals(1.0, riskAssessment.suicideRiskScore, .0001);
        assertEquals("high", riskAssessment.suicideRiskLevel);
    }
	
    @ParameterizedTest
    @CsvSource({"0, 0, 0, 0", 
                "0, 0, 0, 1",
                "0, 0, 1, 0",
                "0, 0, 1, 1",
                "0, 1, 0, 0",
                "0, 1, 0, 1",
                "0, 1, 1, 0",
                "0, 1, 1, 1",
                "1, 0, 0, 0",
                "1, 0, 0, 1",
                "1, 0, 1, 0",
                "1, 0, 1, 1",
                "1, 1, 0, 0",
                "1, 1, 0, 1",
                "1, 1, 1, 0",				
                "1, 1, 1, 1",
                })
    void mentDiagnosisMale(int depression, 
            int manicDepression,
            int PTSD,
            int other) {
        System.out.println("TEST <==> START mentDiagnosisMale Test <==> TEST");
        System.out.println("depression = " + depression);
        System.out.println("manicDepression = " + manicDepression);
        System.out.println("PTSD = " + PTSD);
        System.out.println("otherMentDiagnosis = " + other);

        bn.sex = BasicNeeds.MALE;

        ment.depression = depression;
        ment.bipolar = manicDepression;
        ment.ptsd = PTSD;
        ment.otherMentDiagnosis = other;

        ment.weeklyAlcoholDrinks = 0;

        ment.tobaccoVape = FALSE;
        ment.opioids = FALSE;
        ment.cannabis = FALSE;
        ment.amphetamine = FALSE;
        ment.cocaine = FALSE;
        ment.heroin = FALSE;
        ment.otherSubstAbuse = FALSE;

        ment.antidepressants = FALSE;
        ment.sleepMeds = FALSE;
        ment.otherMentMeds = FALSE;

        ment.annualMentVisits = 1;

        mil.reductRank = Military.NO;
        mil.dischType = Military.HONORABLE;

        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;

        ksession.insert(bn);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);


        ksession.fireAllRules();
        ksession.dispose();

        double score = mentDiagnosisScore(ment, bn);
        score += visitScore(bn, ment, mil, soc);

        String cat = category(score);

        assertEquals(score, riskAssessment.suicideRiskScore, 0.001);
        assertEquals(cat, riskAssessment.suicideRiskLevel);
        System.out.println("TEST <==> END mentDiagnosisMale Test <==> TEST");
    }
	
    boolean hasMentalIllness(Mental ment) {
        boolean illness = false;
        if (ment.depression == TRUE) {
            illness = true;
        }
        if (ment.bipolar == TRUE) {
            illness = true;
        }
        if (ment.ptsd == TRUE) {
            illness = true;
        }
        if (ment.sleepDisorder == TRUE) {
            illness = true;
        }
        if (ment.otherMentDiagnosis == TRUE) {
            illness = true;
        }
        if (ment.anxiety == TRUE) {
            illness = true;
        }
        return illness;		
    }
		
    double mentDiagnosisScore(Mental ment, BasicNeeds bn) {
        double d = (ment.depression == TRUE) ? 1: 0;
        double md = (ment.bipolar == TRUE) ? 1: 0;
        double ptsd = (ment.ptsd == TRUE) ? 1: 0;
        double o = (ment.otherMentDiagnosis == TRUE) ? 1: 0;

        double score = d * 1.0 + md * 1.0 + ptsd * 0.5 + o * 0.25;
        score /= 2.0;
        score = Math.min(1.0, score);
        score *= 0.2;

        if (bn.sex == BasicNeeds.FEMALE) {
            score *= 0.75;
        }
        return score;		
    }
	
    double visitScore(BasicNeeds bn, Mental ment, Military mil, Social soc) {
        double score = 0;
        int visits = ment.annualMentVisits;

        if (mil.reductRank == Military.YES) {
            visits += 1;
        }
        if (mil.reductRankRefuse == TRUE && mil.dischType == Military.DISHONORABLE) {
            visits += 1;
        }
        if (soc.marital == Social.SEPARATED || 
                soc.marital == Social.DIVORCED || 
                soc.marital == Social.WIDOWED) 
        {
            if (soc.moMaritalChg <= 12) {
                visits += 1;
            }
        }

        // Compute visits
        if (visits > 0) {
            score = (double)visits / 18.0;
        } 
        else if (visits == 0 && hasMentalIllness(ment)) {
            score = 1;
        }

        score = Math.min(1, score);
        score /= 15.0;

        if (bn.sex == BasicNeeds.FEMALE) {
            score *= 0.75;
        }

        return score;
    }
	
    @RepeatedTest(10)
    void alcohol() {
    Random random = new Random();
        //bn.sex = BasicNeeds.MALE;
        //bn.sex = BasicNeeds.UNSPECIFIED;
        bn.sex = random.nextInt(3) + 1; // randomly choose sex

        ment.depression = FALSE;
        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.otherMentDiagnosis = FALSE;
        
        // Randomly select number of drinks
        ment.weeklyAlcoholDrinks = random.nextInt(22);

        ment.tobaccoVape = FALSE;
        ment.opioids = FALSE;
        ment.cannabis = FALSE;
        ment.amphetamine = FALSE;
        ment.cocaine = FALSE;
        ment.heroin = FALSE;
        ment.otherSubstAbuse = FALSE;

        ment.antidepressants = FALSE;
        ment.sleepMeds = FALSE;
        ment.otherMentMeds = FALSE;

        ment.annualMentVisits = 0;

        mil.reductRank = Military.NO;
        mil.dischType = Military.HONORABLE;

        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;

        ksession.insert(bn);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);


        ksession.fireAllRules();
        ksession.dispose();

        double score = alcoholScore(ment.weeklyAlcoholDrinks, bn.sex);
        String cat = category(score);

        assertEquals(score, riskAssessment.suicideRiskScore, 0.001);
        assertEquals(cat, riskAssessment.suicideRiskLevel);
    }

    double alcoholScore(int drinks, int sex) {
        double score;
        if (sex == BasicNeeds.FEMALE) {
            score = (double)drinks / 12.0;
            score = Math.min(1, score);
            score *= 0.15;
            return score;
        }
        else {
            score = (double)drinks / 20.0;
            score = Math.min(1, score);
            score *= 0.2;
            return score;
        }
    }
	
	
    @RepeatedTest(16)
    void substanceAbuse() {
        //boolean[] booleans = new boolean[7];
        int[] booleans = new int[7];
        Random random = new Random();
        int trueCount = 0;

        while (trueCount < 1) {
            trueCount = 0;
            for (int i = 0; i < 7; i++) {
                booleans[i] = random.nextInt(2);
                if (booleans[i] == TRUE) {
                    trueCount++;
                }
            }
        }
        System.out.println(" >>> trueCount = " + trueCount);
        System.out.println(" >>> booleans: " + Arrays.toString(booleans));

        //bn.sex = BasicNeeds.MALE;
        //bn.sex = BasicNeeds.UNSPECIFIED;
        bn.sex = random.nextInt(3) + 1; // randomly choose sex

        ment.depression = FALSE;
        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.otherMentDiagnosis = FALSE;

        ment.weeklyAlcoholDrinks = 0;

        ment.tobaccoVape = booleans[0];
        ment.opioids = booleans[1];
        ment.cannabis = booleans[2];
        ment.amphetamine = booleans[3];
        ment.cocaine = booleans[4];
        ment.heroin = booleans[5];
        ment.otherSubstAbuse = booleans[6];

        ment.antidepressants = FALSE;
        ment.sleepMeds = FALSE;
        ment.otherMentMeds = FALSE;

        ment.annualMentVisits = 0;

        mil.reductRank = Military.NO;
        mil.dischType = Military.HONORABLE;

        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;

        ksession.insert(bn);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);


        ksession.fireAllRules();
        ksession.dispose();

        double score = substanceAbuseScore(ment.tobaccoVape,
                                            ment.opioids,
                                            ment.cannabis,
                                            ment.amphetamine,
                                            ment.cocaine,
                                            ment.heroin,
                                            ment.otherSubstAbuse,
                                            bn.sex);
        String cat = category(score);

        assertEquals(score, riskAssessment.suicideRiskScore, 0.001);
        assertEquals(cat, riskAssessment.suicideRiskLevel);
    }


    private double substanceAbuseScore(int tobaccoVape, 
            int opioids, 
            int cannabis, 
            int amphetamine,
            int cocaine, 
            int heroin, 
            int otherSubstAbuse, 
            int sex) {
        double score = 0.0;
        Map<String, Double> v = new HashMap<String, Double>();
        v.put("tobaccoVape", 0.35);
        v.put("opioids", 0.3);
        v.put("cannabis", 0.35);
        v.put("amphetamine", 0.5);
        v.put("cocaine", 0.7);
        v.put("heroin", 1.0);
        v.put("other", 0.3);

        if (tobaccoVape == TRUE) {
            score += v.get("tobaccoVape");
        }
        if (opioids == TRUE) {
            score += v.get("opioids");
        }
        if (cannabis == TRUE) {
            score += v.get("cannabis");
        }
        if (amphetamine == TRUE) {
            score += v.get("amphetamine");
        }
        if (cocaine == TRUE) {
            score += v.get("cocaine");
        }
        if (heroin == TRUE) {
            score += v.get("heroin");
        }
        if (otherSubstAbuse == TRUE) {
            score += v.get("other");
        }

        score /= 1.5;
        score = Math.min(1.0, score);
        score *= 0.2;
        if (sex == BasicNeeds.FEMALE) {
            score *= 0.75;
        }

        return score;
    }
	
    @ParameterizedTest
    @CsvSource({"0, 0, 0", 
                "0, 0, 1",
                "0, 1, 0",
                "0, 1, 1",
                "1, 0, 0",
                "1, 0, 1",
                "1, 1, 0",
                "1, 1, 1",
                })
    void medsMale(int ad, int sm, int omm) {
        bn.sex = BasicNeeds.MALE;
        //bn.sex = BasicNeeds.UNSPECIFIED;
        //bn.sex = random.nextInt(3) + 1; // randomly choose sex

        ment.depression = FALSE;
        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.otherMentDiagnosis = FALSE;

        ment.weeklyAlcoholDrinks = 0;

        ment.tobaccoVape = FALSE;
        ment.opioids = FALSE;
        ment.cannabis = FALSE;
        ment.amphetamine = FALSE;
        ment.cocaine = FALSE;
        ment.heroin = FALSE;
        ment.otherSubstAbuse = FALSE;

        ment.antidepressants = ad;
        ment.sleepMeds = sm;
        ment.otherMentMeds = omm;

        ment.annualMentVisits = 0;

        mil.reductRank = Military.NO;
        mil.dischType = Military.HONORABLE;

        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;

        ksession.insert(bn);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);


        ksession.fireAllRules();
        ksession.dispose();

        double score = medScore(ment.antidepressants,
                                ment.sleepMeds,
                                ment.otherMentMeds,
                                bn.sex);
        String cat = category(score);

        assertEquals(score, riskAssessment.suicideRiskScore, 0.001);
        assertEquals(cat, riskAssessment.suicideRiskLevel);
    }
	

    @ParameterizedTest
    @CsvSource({"0, 0, 0", 
                "0, 0, 1",
                "0, 1, 0",
                "0, 1, 1",
                "1, 0, 0",
                "1, 0, 1",
                "1, 1, 0",
                "1, 1, 1",
                })
    void medsFemale(int ad, int sm, int omm) {
        bn.sex = BasicNeeds.FEMALE;

        ment.depression = FALSE;
        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.otherMentDiagnosis = FALSE;

        ment.weeklyAlcoholDrinks = 0;

        ment.tobaccoVape = FALSE;
        ment.opioids = FALSE;
        ment.cannabis = FALSE;
        ment.amphetamine = FALSE;
        ment.cocaine = FALSE;
        ment.heroin = FALSE;
        ment.otherSubstAbuse = FALSE;

        ment.antidepressants = ad;
        ment.sleepMeds = sm;
        ment.otherMentMeds = omm;

        ment.annualMentVisits = 0;

        mil.reductRank = Military.NO;
        mil.dischType = Military.HONORABLE;

        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;

        ksession.insert(bn);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);


        ksession.fireAllRules();
        ksession.dispose();

        double score = medScore(ment.antidepressants,
                                ment.sleepMeds,
                                ment.otherMentMeds,
                                bn.sex);
        String cat = category(score);

        assertEquals(score, riskAssessment.suicideRiskScore, 0.001);
        assertEquals(cat, riskAssessment.suicideRiskLevel);
    }
	
	
    @ParameterizedTest
    @CsvSource({"0, 0, 0", 
                "0, 0, 1",
                "0, 1, 0",
                "0, 1, 1",
                "1, 0, 0",
                "1, 0, 1",
                "1, 1, 0",
                "1, 1, 1",
                })
    void medsUnspecifiedSex(int ad, int sm, int omm) {
        bn.sex = BasicNeeds.UNSPECIFIED;

        ment.depression = FALSE;
        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.otherMentDiagnosis = FALSE;

        ment.weeklyAlcoholDrinks = 0;

        ment.tobaccoVape = FALSE;
        ment.opioids = FALSE;
        ment.cannabis = FALSE;
        ment.amphetamine = FALSE;
        ment.cocaine = FALSE;
        ment.heroin = FALSE;
        ment.otherSubstAbuse = FALSE;

        ment.antidepressants = ad;
        ment.sleepMeds = sm;
        ment.otherMentMeds = omm;

        ment.annualMentVisits = 0;

        mil.reductRank = Military.NO;
        mil.dischType = Military.HONORABLE;

        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;

        ksession.insert(bn);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);


        ksession.fireAllRules();
        ksession.dispose();

        double score = medScore(ment.antidepressants,
                                ment.sleepMeds,
                                ment.otherMentMeds,
                                bn.sex);
        String cat = category(score);

        assertEquals(score, riskAssessment.suicideRiskScore, 0.001);
        assertEquals(cat, riskAssessment.suicideRiskLevel);
    }
	

    private double medScore(int antidepressants, 
            int sleepMeds, 
            int otherMentMeds, 
            int sex) {
        
        double score = 0.0;
        Map<String, Double> v = new HashMap<String, Double>();
        v.put("antidepressants", 1.0);
        v.put("sleepMeds", 1.0);
        v.put("otherMentMeds", 0.3);

        if (antidepressants == TRUE) {
            score += v.get("antidepressants");
        }
        if (sleepMeds == TRUE) {
            score += v.get("sleepMeds");
        }
        if (otherMentMeds == TRUE) {
            score += v.get("otherMentMeds");
        }

        score /= 2.0;
        score = Math.min(1.0, score); // clip to 1
        score *= 0.2;
        if (sex == BasicNeeds.FEMALE) {
            score *= 0.75;
        }

        return score;
    }
	
	
    @ParameterizedTest
    @CsvSource({"0, 0, 0, 0", 
                "0, 0, 0, 1",
                "0, 0, 1, 0",
                "0, 0, 1, 1",
                "0, 1, 0, 0",
                "0, 1, 0, 1",
                "0, 1, 1, 0",
                "0, 1, 1, 1",
                "1, 0, 0, 0",
                "1, 0, 0, 1",
                "1, 0, 1, 0",
                "1, 0, 1, 1",
                "1, 1, 0, 0",
                "1, 1, 0, 1",
                "1, 1, 1, 0",               
                "1, 1, 1, 1",
                })
    void visits(int d, int md, int ptsd, int other) {
        Random random = new Random();

        //bn.sex = BasicNeeds.MALE;
        //bn.sex = BasicNeeds.UNSPECIFIED;
        bn.sex = random.nextInt(3) + 1; // randomly choose sex

        ment.depression = d;
        ment.bipolar = md;
        ment.ptsd = ptsd;
        ment.otherMentDiagnosis = other;

        ment.weeklyAlcoholDrinks = 0;

        ment.tobaccoVape = FALSE;
        ment.opioids = FALSE;
        ment.cannabis = FALSE;
        ment.amphetamine = FALSE;
        ment.cocaine = FALSE;
        ment.heroin = FALSE;
        ment.otherSubstAbuse = FALSE;

        ment.antidepressants = FALSE;
        ment.sleepMeds = FALSE;
        ment.otherMentMeds = FALSE;

        ment.annualMentVisits = random.nextInt(20);
        //ment.annualMentVisits = 19;

        mil.reductRank = Military.NO;
        mil.dischType = Military.HONORABLE;

        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;

        ksession.insert(bn);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);


        ksession.fireAllRules();
        ksession.dispose();

        double score = mentDiagnosisScore(ment, bn);
        score += visitScore(bn, ment, mil, soc);

        String cat = category(score);

        assertEquals(score, riskAssessment.suicideRiskScore, 0.001);
        assertEquals(cat, riskAssessment.suicideRiskLevel);
    }
	
	
    @ParameterizedTest
    @CsvSource({"1", 
                "2",
                })
    void rank(int rankReduction) {
        bn.sex = BasicNeeds.MALE;

        ment.depression = FALSE;
        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.otherMentDiagnosis = FALSE;

        ment.weeklyAlcoholDrinks = 0;

        ment.tobaccoVape = FALSE;
        ment.opioids = FALSE;
        ment.cannabis = FALSE;
        ment.amphetamine = FALSE;
        ment.cocaine = FALSE;
        ment.heroin = FALSE;
        ment.otherSubstAbuse = FALSE;

        ment.antidepressants = FALSE;
        ment.sleepMeds = FALSE;
        ment.otherMentMeds = FALSE;

        ment.annualMentVisits = 0;

        mil.reductRank = rankReduction;
        mil.dischType = Military.HONORABLE;

        soc.marital = Social.SINGLE;
        soc.moMaritalChg = 0;

        ksession.insert(bn);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);


        ksession.fireAllRules();
        ksession.dispose();

        double score = rankScore(mil, bn);
        String cat = category(score);

        assertEquals(score, riskAssessment.suicideRiskScore, 0.001);
        assertEquals(cat, riskAssessment.suicideRiskLevel);
    }

    private double rankScore(Military mil, BasicNeeds bn) {
        double score = 0;

        if (mil.reductRank == Military.YES) {
            score = 1;
        }

        score /= 15;

        if (bn.sex == BasicNeeds.FEMALE) {
            score *= 0.75;
        }

        return score;
    }
    
	
    @ParameterizedTest
    @CsvSource({"1, 0", 
                "2, 0",
                "3, 12",
                "3, 24",
                "4, 12",
                "4, 24",
                "5, 12",
                "5, 24",
                })
    void marital(int marital, int moMaritalChg) {
        bn.sex = BasicNeeds.MALE;

        ment.depression = FALSE;
        ment.bipolar = FALSE;
        ment.ptsd = FALSE;
        ment.otherMentDiagnosis = FALSE;

        ment.weeklyAlcoholDrinks = 0;

        ment.tobaccoVape = FALSE;
        ment.opioids = FALSE;
        ment.cannabis = FALSE;
        ment.amphetamine = FALSE;
        ment.cocaine = FALSE;
        ment.heroin = FALSE;
        ment.otherSubstAbuse = FALSE;

        ment.antidepressants = FALSE;
        ment.sleepMeds = FALSE;
        ment.otherMentMeds = FALSE;

        ment.annualMentVisits = 0;

        mil.reductRank = Military.NO;
        mil.dischType = Military.HONORABLE;

        soc.marital = marital;
        soc.moMaritalChg = moMaritalChg;

        ksession.insert(bn);
        ksession.insert(mil);
        ksession.insert(ment);
        ksession.insert(soc);
        ksession.insert(riskAssessment);


        ksession.fireAllRules();
        ksession.dispose();

        double score = maritalScore(bn, soc);
        String cat = category(score);

        assertEquals(score, riskAssessment.suicideRiskScore, 0.001);
        assertEquals(cat, riskAssessment.suicideRiskLevel);
    }

    private double maritalScore(BasicNeeds bn, Social soc) {
        double score = 0;

        if (soc.marital == Social.SEPARATED || 
                soc.marital == Social.DIVORCED ||
                soc.marital == Social.WIDOWED) 
        {
            if (soc.moMaritalChg <= 12) {
                score += 1;
            }
            else {
                score += 0.5;
            }
        }

        score /= 15;

        if (bn.sex == BasicNeeds.FEMALE) {
            score *= 0.75;
        }

        return score;
    }

    String category(double score) {
        if (score < 0.2) {
            return "low";
        }
        else if (score >= 0.2 && score < 0.4) {
            return "medium";
        }
        else {
            return "high";
        }
    }
*/
}
