from risk import *
from veteran import Veteran


def set_none(d):
    set_false(d)
    d['none'] = True


def set_false(d):
    for k in d:
        d[k] = False


def set_true(d):
    for k in d:
        d[k] = True


def gen_zero_risk_vet():
    vet = Veteran()
    vet.G2_sex = 'male'
    vet.G6_housing = 'own'
    vet.G7_transportation = 'very reliable'
    vet.G8_food = 4
    vet.G9_savings = '2+'

    set_none(vet.P1_psych_dx)
    vet.P2_weekly_drinks = 0
    set_none(vet.P3_drugs)
    set_none(vet.P4_psych_drugs)
    vet.P5_annual_psych_visits = 0

    vet.E1_employed = True

    vet.H1_activity = 'strenuous'
    set_true(vet.H2_coverage)
    set_false(vet.H3_chronic_cond)
    vet.H4_meds = 0
    vet.H5_compliance = 'consistently'

    vet.M1_officer = 'officer'
    vet.M2_mil_inj = False
    vet.M3_discharge = 'honorable'
    vet.M4_combat = False
    vet.M5_rank_reduction = False
    vet.M6_understand_missions = 'well'

    vet.S1_marriage = 'married'
    vet.S2_interactions = 20
    vet.S3_conflict = 'rarely'
    vet.S4_religion = 'regularly'
    vet.S5_education = 'advanced'

    return vet


def gen_low_risk_vet():
    vet = Veteran()
    vet.G2_sex = 'male'
    vet.G6_housing = 'own'
    vet.G7_transportation = 'very reliable'
    vet.G8_food = 4
    vet.G9_savings = '2+'

    set_none(vet.P1_psych_dx)
    vet.P2_weekly_drinks = 7
    set_none(vet.P3_drugs)
    set_none(vet.P4_psych_drugs)
    vet.P5_annual_psych_visits = 0

    vet.E1_employed = True

    vet.H1_activity = 'strenuous'
    set_true(vet.H2_coverage)
    set_false(vet.H3_chronic_cond)
    vet.H3_chronic_cond['high blood pressure'] = True
    vet.H4_meds = 1
    vet.H5_compliance = 'consistently'

    vet.M1_officer = 'enlisted'
    vet.M2_mil_inj = False
    vet.M3_discharge = 'honorable'
    vet.M4_combat = False
    vet.M5_rank_reduction = False
    vet.M6_understand_missions = 'N/A'

    vet.S1_marriage = 'single'
    vet.S2_interactions = 8
    vet.S3_conflict = 'rarely'
    vet.S4_religion = 'often'
    vet.S5_education = 'some college'

    return vet


def gen_med_risk_vet():
    vet = Veteran()
    vet.G2_sex = 'male'
    vet.G6_housing = 'rent'
    vet.G7_transportation = 'very reliable'
    vet.G8_food = 4
    vet.G9_savings = '1-2'

    set_false(vet.P1_psych_dx)
    vet.P1_psych_dx['PTSD'] = True
    vet.P2_weekly_drinks = 4
    set_none(vet.P3_drugs)
    set_none(vet.P4_psych_drugs)
    vet.P5_annual_psych_visits = 0

    vet.E1_employed = True

    vet.H1_activity = 'none'
    set_true(vet.H2_coverage)
    set_false(vet.H3_chronic_cond)
    vet.H3_chronic_cond['physical disability'] = True
    vet.H4_meds = 1
    vet.H5_compliance = 'consistently'

    vet.M1_officer = 'enlisted'
    vet.M2_mil_inj = False
    vet.M3_discharge = 'honorable'
    vet.M4_combat = False
    vet.M5_rank_reduction = False
    vet.M6_understand_missions = 'N/A'

    vet.S1_marriage = 'single'
    vet.S2_interactions = 8
    vet.S3_conflict = 'occasional'
    vet.S4_religion = 'often'
    vet.S5_education = 'high school'

    return vet


def gen_hi_risk_vet():
    vet = Veteran()
    vet.G2_sex = 'male'
    vet.G6_housing = 'rent'
    vet.G7_transportation = 'less reliable'
    vet.G8_food = 3
    vet.G9_savings = '0-1'

    set_false(vet.P1_psych_dx)
    vet.P1_psych_dx['PTSD'] = True
    vet.P1_psych_dx['manic-depression'] = True
    vet.P2_weekly_drinks = 14
    set_none(vet.P3_drugs)
    set_none(vet.P4_psych_drugs)
    vet.P5_annual_psych_visits = 0

    vet.E1_employed = False
    vet.E2_months_unemp = 24

    vet.H1_activity = 'none'
    set_true(vet.H2_coverage)
    set_false(vet.H3_chronic_cond)
    vet.H3_chronic_cond['high blood pressure'] = True
    vet.H3_chronic_cond['physical disability'] = True
    vet.H4_meds = 1
    vet.H5_compliance = 'often forget'

    vet.M1_officer = 'enlisted'
    vet.M2_mil_inj = True
    vet.M3_discharge = 'dishonorable'
    vet.M4_combat = True
    vet.M5_rank_reduction = True
    vet.M6_understand_missions = 'N/A'

    vet.S1_marriage = 'in last year'
    vet.S2_interactions = 8
    vet.S3_conflict = 'regularly'
    vet.S4_religion = 'rarely'
    vet.S5_education = 'high school'

    return vet


def gen_default_vet():
    vet = Veteran()
    return vet


def print_results(vet: Veteran):
    print('-' * 40)
    print(vet)
    suicide_risk = calc_suicide_risk(vet)
    basic_needs_risk = calc_basic_needs_risk(vet)
    employability_risk = calc_employability_risk(vet)
    health_risk = calc_health_risk(vet)
    social_risk = calc_social_risk(vet)
    print('Suicide risk: ' + str(suicide_risk[:2]))
    pprint(suicide_risk[2])
    print()
    print('Basic needs risk: ' + str(basic_needs_risk[:2]))
    pprint(basic_needs_risk[2])
    print()
    print('Employability risk: ' + str(employability_risk[:2]))
    pprint(employability_risk[2])
    print()
    print('Health risk: ' + str(health_risk[:2]))
    pprint(health_risk[2])
    print()
    print('Social risk: ' + str(social_risk[:2]))
    pprint(social_risk[2])
    print()
    print('-' * 40)


if __name__ == '__main__':
    v0 = gen_zero_risk_vet()
    print_results(v0)

    v1 = gen_low_risk_vet()
    print_results(v1)

    v2 = gen_med_risk_vet()
    print_results(v2)

    v3 = gen_hi_risk_vet()
    print_results(v3)

    vd = gen_default_vet()
    print_results(vd)


