
"""
This file contains the dictionaries containing the scoring values for
various fields using in risk scoring.
"""

default = 0.75

G = {  # General
    2: {  # Sex
        'male': 1.0,
        'other': 1.0,
        'female': 0.75,
        'default': 0.88
    },
    6: {  # housing
        'own': 0.0,
        'rent': 0.1,
        'live friend': 0.25,
        'no house': 1.0,
        'default': 0.75
    },
    7: {  # transportation
        'very reliable': 0.0,
        'usually available': 0.25,
        'less reliable': 0.5,
        'often unavailable': 0.25,  # TODO: value appears to mistyped, should be 0.75
        'unreliable': 1.0,
        'default': 0.75
    },
    8: {  # Food availability
        5: 0.2,
        4: 0.0,
        3: 0.2,
        2: 0.5,
        1: 1.0,
        'default': 0.75
    },
    9: {  # Months of savings
        '2+': 0.0,
        '1-2': 0.33,
        '0-1': 0.66,
        'none': 1.0,
        'default': 0.75
    }
}


P = {  # Mental Health
    1: {  # Pyschiatric Dx
        'none': 0.0,
        'depression': 1.0,
        'manic-depression': 1.0,
        'PTSD': 0.5,
        'sleep disorder': 0.25,
        'other': 0.25,
        'default': 0.75
    },
    2: {  # divisors for drinks per week
        'male': 20,
        'female': 12,
        'other': 16,
        'default': 0.75
    },
    3: {  # Substance abuse: tobacco & non-alcoholic drugs
        'none': 0.0,
        'heroine': 1.0,
        'cocaine': 0.7,
        'amphetamine': 0.5,
        'cannabis': 0.35,
        'tobacco': 0.35,
        'opioids': 0.3,
        'default': 0.75
    },
    4: {  # Pyschiatric medications
        'antidepressants': 1.0,
        'sleep meds': 1.0,
        'other psych meds': 0.3,
        'none': 0.0,
        'default': 0.75
    },
    5: {  # Other suicide factors summary
        'visits divisor': 18.0,  # divisor for mental health visits per year
        'reduction in rank or dishonorable': 1.0,
        'divorced/widowed/sep in last year': 1.0,
        'default': 0.75,
        'none': 0.0  # TODO: P5 none value missing from questionnaire
    },
    6: {  # Depression survey
    }
}


E = {  # Employment
    1: {  # Are you employed?
        'no': 1.0,
        'yes': 0.0,
        'default': 0.75
    }
}


H = {  # Health value dict
    1: {  # Activity
        'strenuous': 0.0,
        'none': 1.0,
        'default': 0.75
    },
    2: {  # Health coverage
        'none': 1.0,
        'one': 0.6,
        'two': 0.3
    },
    3: {  # Chronic health overview
        # Count of chronic conditions
    },
    4: {  # Number of medications in each category
        # Count of medications in each category
    },
    5: {  # Lack of compliance with meds or appointment
        'consistently': 0.0,
        'usually': 0.5,  # but sometimes misdose
        'often forget': 1.0,
        'default': 0.75
    }
}


M = {  # Military
    1: {  # officer or enlisted
        'officer': 1.0,
        'enlisted': 0.0,
        'default': 0.75  # TODO: M1 missing warrant officer and default
    },
    2: {  # serious service connected injury
        True: 1.0,
        False: 0.0,
        'default': 0.75
    },
    3: {  # Discharge type
        'honorable': 0.0,
        'dishonorable': 1.0,
        'default': 0.75
    },
    4: {  # Combat experience
        True: 1.0,
        False: 0.0,
        'default': 0.75
    },
    5: {  # Reduction in military rank
        True: 1.0,
        False: 0.0,
        'default': 0.75
    },
    6: {  # Understand military missions?
        'well': 1.0,
        'N/A': 0.5,
        'poorly': 0.0
    }
}


S = {  # Social value dict
    1: {  # Interpersonal - marriage
        'in last year': 1,  # divorced, separated, or widowed
        'more than one year': 0.5,
        'single': 0,
        'married': 0,
        'default': 0.75
    },
    2:  # Interpersonal friend network, interactions per week divisor
        10,  # clip at 1
    3: {  # Conflict
        'rarely': 0.0,
        'occasional': 0.25,
        'on & off': 0.5,
        'often': 0.75,
        'regularly': 1.0
    },
    4: {  # Religious service attendance.
        'rarely': 1.0,
        'occasional': 0.75,
        'on & off': 0.5,
        'often': 0.25,
        'regularly': 0.0
    },
    5: {  # Highest level of completed education
        'advanced': 0.0,
        'college': 0.0,
        'trade school': 0.1,
        'some college': 0.25,
        'high school': 0.5,
        '< high school': 1.0,
        'default': 0.75}
}

thresholds = {  # cutoffs for risk levels: low, medium, high
    'suicide': {
        'l': 0.1,
        'm': 0.4,
    },
    'basic': {
        'l': 0.0,
        'm': 0.25,
        'h': 0.5
    },
    'employability': {
        'l': 0.15,
        'm': 0.4
    },
    'health': {
        'l': 0.15,
        'm': 0.4
    },
    'social': {
        'l': 0.15,
        'm': 0.4
    }
}

