"""
This python file contains the Veteran class which holds the values
for a specific veteran

This class depends on Python version >=3.7
"""

from dataclasses import dataclass, field, asdict
from typing import Dict, List


def get_P1_dict():
    d = {
        'none': None,
        'depression': None,
        'manic-depression': None,
        'PTSD': None,
        'sleep disorder': None,
        'other': None
    }
    return d


def get_P3_dict():
    d = {
        'none': None,
        'heroine': None,
        'cocaine': None,
        'amphetamine': None,
        'cannabis': None,
        'tobacco': None,
        'opioids': None,
        'other': None
    }
    return d


def get_P4_dict():
    d = {
        'none': None,
        'antidepressants': None,
        'sleep meds': None,
        'other psych meds': None
    }
    return d


def get_H2_dict():
    d = {
        'insurance': None,
        'doctor': None,
        'access': None
    }
    return d

def get_H3_dict():
    d = {
        'physical disability': None,
        'diabetes': None,
        'cancer': None,
        'obesity': None,
        'migraines': None,
        'high blood pressure': None
    }
    return d


@dataclass
class Veteran:
    G2_sex: str = None  # sex
    G6_housing: str = None  # housing
    G7_transportation: str = None  # transportation
    G8_food: str = None  # food availability
    G9_savings: str = None  # months of savings

    E1_employed: bool = None  # employed flag
    E2_months_unemp: int = None  # months unemployed

    H1_activity: str = None  # activity levels
    H2_coverage: Dict[str, bool] = field(default_factory=get_H2_dict)  # health coverage
    H3_chronic_cond: Dict[str, bool] = field(default_factory=get_H3_dict)  # chronic health overview
    H4_meds: int = None  # meds in each category
    H5_compliance:str = None  # compliance with meds or appointments

    P1_psych_dx: Dict[str, bool] = field(default_factory=get_P1_dict)  # psychiatric Dx
    P2_weekly_drinks: int = None  # drinks per week
    P3_drugs: Dict[str, bool] = field(default_factory=get_P3_dict)  # substance abuse: tobacco & non-alcoholic
    P4_psych_drugs: Dict[str, bool] = field(default_factory=get_P4_dict)  # psychiatric meds
    P5_annual_psych_visits: int = None  # pychiatric visits per year
    P6_depression: List[str] = field(default_factory=list)  # depression survey

    M1_officer: str = None  # officer or enlisted
    M2_mil_inj: bool = None  # serious service connected injury
    M3_discharge: str = None  # discharge type
    M4_combat: bool = None  # combat experience
    M5_rank_reduction: bool = None  # reduction in military rank
    M6_understand_missions: str = None  # understand military missions

    S1_marriage: str = None  # marriage
    S2_interactions: int = None  # social network interactions per week
    S3_conflict: str = None  # conflict
    S4_religion: str = None  # religious service attendance
    S5_education: str = None  # highest level of education

