'use strict';

const fs = require('fs');
const myXml = require('./myXML.js');
const yargs = require('yargs');

const argv = yargs
	.options({
		f: {
			demand: true,
			alias: 'file',
			describe: 'file to be unzipped',
			string: true
		}
	})
	.help()
	.alias('help', 'h')
    .argv;
    


let today = new Date().toISOString().split('T')[0].replace(/\-/g, "");
let outFile = `./BGDailyFeed/addJobs/AddBurningGlassJobs_${today}.json`;

let writer = fs.createWriteStream(outFile);


var reader = myXml.createReader(argv.file, /^(Job)$/, {
    gzip: false,
    encoding: 'iso-8859-1'
})

let counter = 0;

reader.on('error', (err) => {
    console.log("ERROR: " + err);
});

reader.on('data', data => {
    console.log(`Data ${JSON.stringify(data)}`);
});


reader.on('record', record => {
    if (counter++ === 0) {
        writer.write('[')
    }
    let merged = {};
    record.children.map(child => {
        if (child.children) {
            let attrList = [];
            child.children.map(attr => {
                const name = attr.attrs.name;
                attrList.push(name);
            });
            // console.log(attrList);
            const tagStr = `{ "${child.tag}": "${attrList}" }`;
            const tagJson = JSON.parse(tagStr);
            merged = {...merged, ...tagJson};
        } else {
            let str = '';
            try {
                str = `{ "${child.tag}": "${escapeSpecialChars(child.text)}" }`;
                // console.log(`Str to parse: ${str}`);
                const sink = JSON.parse(str);
                merged = {...merged, ...sink};
            } catch (e) {
                console.error(`Error: ${e} parsing: ${str}`);
            }
        }
    });
    // console.log(`Counter: ${counter}`);
    // console.log(JSON.stringify(merged));


    writer.write(JSON.stringify(merged), (err) => {
        if (err) throw err;
        console.log('Data written to file');
    });
    writer.write(',', (err) => {
        if (err) throw err;
        console.log('Data written to file');
    });
});

reader.on('end', () => {
    writer.write(`{"counter" : ${counter}} ]`, (err) => {
        if (err) throw err;
        console.log('Data written to file');
    });
    console.log('JSON written')
    writer.end();
});

function escapeSpecialChars(jsonString) {
    if (jsonString) {
        return jsonString.replace(/\n/g, '\\n')
            .replace(/\r/g, '\\r')
            .replace(/\t/g, '\\t')
            .replace(/\f/g, '\\f')
            .replace(/\"/g, '')
            .replace(/\'/g, '');
    } else {
        return '';
    }
}
