'use strict';

const fs = require("fs");
const parse = require("csv-parse/lib/sync");
const QuestionSet = require("./questionSet");
const ss = require("./questionSetSpreadsheetConstants");

function print_usage() {
    const nodeName = process.argv[0];
    const scriptName = process.argv[1];
    const message = "USAGE:  " + nodeName + " " + scriptName + " <input CSV file> <output JSON file>";
    console.log(message);
}

if (process.argv.length != 4) {
    print_usage();
    process.exit();
}

const inputFile = process.argv[2];
const outputFile = process.argv[3];

if (fs.existsSync(inputFile) === false) {
    console.log("ERROR:  Unable to find file " + inputFile);
    print_usage();
    process.exit();
}

console.log(">> Reading file...");
const inputSource = fs.readFileSync(inputFile, 'utf-8');
const lines = parse(inputSource);

console.log(">> " + lines.length + " lines read.");

var questionSets = [];
var currentLine = 2; // We'll start on the third line to skip past the header

// Skip down until we find the first line with something in the third column, which indicates the start of the actual data
while (lines.length > currentLine) {
    if (lines[currentLine][ss.QUESTION_SET_ID].trim().length > 0) {
        break;
    }
    currentLine++;
}

console.log(">> Found first question set at line " + currentLine);

// The main loop.
while (lines.length > currentLine) {
    let line = lines[currentLine];

    // Assume we're on the first line of the question set
    let questionSetId = line[ss.QUESTION_SET_ID];
    let version = line[ss.VERSION];
    let orderNum = line[ss.ORDER_NUMBER];
    let predicateString = line[ss.PREDICATE];

    console.log(">> Line " + currentLine + ": Processing question set " + questionSetId + "...");

    let questionSet = new QuestionSet(questionSetId, version, orderNum);
    questionSets.push(questionSet);
    
    // Add predicate
    console.log(">> Line " + currentLine + ":  Processing predicate string: " + predicateString);
    questionSet.addPredicate(predicateString);

    let firstLine = true;

    // Deal with each of the question iods, starting with the current line
    while (lines.length > currentLine) {
        line = lines[currentLine];

        if (line[ss.QUESTION_SET_ID] && line[ss.QUESTION_SET_ID].trim().length > 0) {
            if (firstLine === false) {
                break;
            }
        }

        questionSet.addQuestion(line[ss.QUESTION_ID]);

        firstLine = false;
        currentLine++;
    }
}

var outputObj = { questionSets : questionSets };
var outputString = JSON.stringify(outputObj);

console.log(">> Writing out to " + outputFile);
fs.writeFileSync(outputFile, outputString, 'utf-8');
