
var TokenType = {
    NOT: 'not',
    AND: 'and',
    OR: 'or',
    COMMA: 'comma',
    OPEN: 'open',
    CLOSE: 'close',
    VALUE: 'value'
}

class Token {
    constructor(tokenType, name) {
        this.tokenType = tokenType;
        this.name = name;
    }

    addToName(c) {
        this.name = this.name + c;
    }
}

var Predicate = module.exports = class Predicate {
    constructor(tokens) {
        // All token sets here should be either:
        // <symbol> ( <token-subset> ) 
        // or
        // <value token>

        let firstToken = tokens[0];

        if (firstToken.tokenType === TokenType.VALUE) {
            this.value = firstToken.name;
        } else {
            let subTokens = [];
            let parenLevel = 0;
            for (let t of tokens) {
                if (t.tokenType === TokenType.CLOSE) {
                    parenLevel--;
                }
                if (parenLevel > 0) {
                    subTokens.push(t);
                }
                if (t.tokenType === TokenType.OPEN) {
                    parenLevel++;
                }
            }

            if (firstToken.tokenType === TokenType.NOT) {
                this.not = new Predicate(subTokens);
            } else {
                let predicateElements = [];
                let elementTokens = [];
                let elementParenLevel = 0;
                for (let st of subTokens) {
                    if (st.tokenType === TokenType.OPEN) {
                        elementParenLevel++;
                    } else if (st.tokenType === TokenType.CLOSE) {
                        elementParenLevel--;
                    }

                    if (st.tokenType === TokenType.COMMA && elementParenLevel === 0) {
                        predicateElements.push(new Predicate(elementTokens));
                        elementTokens = [];
                    } else {
                        elementTokens.push(st);
                    }
                }
                predicateElements.push(new Predicate(elementTokens));

                switch (firstToken.tokenType) {
                    case TokenType.AND:
                        this.and = predicateElements;
                        break;
                    case TokenType.OR:
                        this.or = predicateElements;
                        break;
                }
            }
        }

    }

    static tokenize(predicateString) {
        // first remove all spaces
        predicateString = predicateString.replace(/[\t\n\r\s]/gm, '');
    
        let tokens = [];
    
        for (let c of predicateString) {
            switch (c) {
                case '!':
                    tokens.push(new Token(TokenType.NOT));
                    break;
                case '&':
                    tokens.push(new Token(TokenType.AND));
                    break;
                case '|':
                    tokens.push(new Token(TokenType.OR));
                    break;
                case '(':
                    tokens.push(new Token(TokenType.OPEN));
                    break;
                case ')':
                    tokens.push(new Token(TokenType.CLOSE));
                    break;
                case ',':
                    tokens.push(new Token(TokenType.COMMA));
                    break;
                default:
                    let lastToken = tokens.pop();
                    if (lastToken === undefined) {
                        tokens.push(new Token(TokenType.VALUE, c));
                    } else if (lastToken.tokenType === TokenType.VALUE) {
                        lastToken.addToName(c);
                        tokens.push(lastToken);
                    } else {
                        tokens.push(lastToken);
                        tokens.push(new Token(TokenType.VALUE, c));
                    }
            }
        }
    
        return tokens;
    }
}