'use strict';

module.exports = class Question {

    constructor(questionId, scope, questionType, question1, question2, question3) {
        this.questionId = questionId.trim();
        this.scope = scope.trim();
        this.questionType = questionType.trim();

        this.questionText = [];
        if (question1 && question1.trim().length > 0) {
            this.questionText.push({ variantId: 1, en: question1.replace(/[\t\n\r]/gm, ' ').replace(/\s+/g, ' ').trim() } );
        }
        if (question2 && question2.trim().length > 0) {
            this.questionText.push({ variantId: 2, en: question2.replace(/[\t\n\r]/gm, ' ').replace(/\s+/g, ' ').trim() } );
        }
        if (question3 && question3.trim().length > 0) {
            this.questionText.push({ variantId: 3, en: question3.replace(/[\t\n\r]/gm, ' ').replace(/\s+/g, ' ').trim() } );
        }

        this.content = {};
    }

    addOption(metricName, value, label, askOther) {
        if (this.content.options === undefined) {
            this.content.options = [];
        }

        var optionObj = {};
        optionObj.answerId = (this.content.options.length + 1) + "";
        optionObj.answerMetricName = metricName.trim();

        if (value && value.length > 0) {
            optionObj.metricValue = parseInt(value);
        }

        if (label && label.length > 0) {
            optionObj.label = { en: label.trim() };
        }

        if (this.questionType === 'MULTI_SELECT_OTHER' && askOther && askOther.trim().length > 0) {
            optionObj.askOther = true;
        }

        this.content.options.push(optionObj);
    }

    addDefault(defaultMetricName, defaultValue) {
        this.content.defaultMetricName = defaultMetricName.trim();
        if (defaultValue && defaultValue.length > 0) {
            this.content.defaultValue = parseInt(defaultValue);
        } else {
            this.content.defaultValue = 0;
        }
    }

    addRange(metricName, minimum, maximum, stepInterval) {
        var rangeObj = {};
        rangeObj.answerMetricName = metricName.trim();

        if (minimum && minimum.trim().length > 0) {
            rangeObj.minimum = parseInt(minimum);
        }

        if (maximum && maximum.trim().length > 0) {
            rangeObj.maximum = parseInt(maximum);
        }

        if (stepInterval && stepInterval.trim().length > 0) {
            rangeObj.stepInterval = parseInt(stepInterval);
        }

        this.content.range = rangeObj;
    }

    addNoneOfTheAbove(metricName, value) {
        var notaObj = {};
        notaObj.answerMetricName = metricName.trim();

        if (value && value.trim().length > 0) {
            notaObj.value = parseInt(value);
        }

        this.content.noneOfTheAbove = notaObj;
    }

    addFollowup(followupText, metricName) {
        var fuObj = {};

        fuObj.answerMetricName = metricName.trim();

        if (followupText && followupText.trim().length > 0) {
            fuObj.followupText = { en: followupText.replace(/[\t\n\r]/gm, ' ').replace(/\s+/g, ' ').trim() };
        }

        this.content.followup = fuObj;
    }

}


