'use strict';

var app = require('../../server/server');
var constants = require('../constants').constants(app);
const carlaServer = constants.cognitive.carlaUrl;
var mongoUrl = constants.mongo.url;
var mongoDbName = constants.mongo.dbName;
const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');
var request = require('request-json');//request = require('request');
var Base64 = require('js-base64').Base64;

var parser = require("../utils/daHtmlParser");
var MongoClient = require('mongodb').MongoClient;

var journalEventConstants = require('va-grit-microservices/constants/journalEventConstants');



module.exports = function(Da) {

  //Loopback wraps the response in a "response" object and we don't want to
  // return that, this removes the response we created from the "response" object
  // and only returns the response we created.
  // TODO: May need to adjust this so only works on custom remote methods we created.
  var remotes = app.remotes();
  remotes.after('**', function (ctx, next) {
      if (ctx) {
        if (ctx.result.response !== null) {
          ctx.result = ctx.result.response;
        }
      } else {
          //console.log("ERROR. vet.js. remotes.after. ctx is not set");
          next({payload: { },
                status: {
                 statusType: responseBuilder.StatusTypes.UNEXPECTED_EXCEPTION,
                 errorMessage: "There was an error on the server. ctx is not set"
               }
          });
      }
      next();
  });

  Da.startConversation = function(header, body, cb) {
     console.log("****Start of Da.startConversation.  header: ", JSON.stringify(header));
     console.log("****Start of Da.startConversation.  body: ", body);

     //console.log("****Start of Da.startConversation.  body.notificationId: ", body.notificationId);
     var response = responseBuilder.createResponseObject();
     var errorMessage;

          VaGrit.getUserFromToken(app, header, function (err, userFromToken) {
             if (userFromToken === undefined || userFromToken === null){
                responseBuilder.setInvalidTokenStatus(response,header);
                cb(null,response);
             }
             else { // userFromToken is defined
                console.log(' VaGrit.getUserFromToken userFromToken: ', userFromToken);
                //var notification = app.models.notification;

                // 1) Get vetInfo (vet name)
                // 2) Send initial conversation to Carla
                // 3) Get response from Carla and send to app

               
                MongoClient.connect(mongoUrl, function(err, db) {
                  console.log('Da.startConversation mongodbConnection: OK');
                  if (err) {
                    //console.log('Da.startConversation mongodbConnection: err: ', err);
                     errorMessage = "Error Connecting to db";
                     //console.log("Da.startConversation Error invoking vetServerClient.getUser for userFromToken " + userFromToken +" err: ", err);
                     responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                     cb(null,response);
                     return;
                  }
                  
                  var dbo = db.db(mongoDbName);
                  dbo.collection("vet").findOne({_id:userFromToken}, function(err, vetInstance) {
                    //console.log('Da.startConversation mongodbConnection: vetInstance: ', vetInstance);
                    console.log('Da.startConversation mongodbConnection: err: ', err);
                    if (err) {
                       errorMessage = "Da.startConversation Da.startConversation Error invoking vetServerClient.getUser - Error [" + err.message + "]";
                       responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                       cb(null,response);
                    } else {
                      //console.log('Da.startConversation mongodbConnection: vetInstance.name: ', vetInstance.name);
                       // Send initial conversation to Carla
                       var carlaData = {
                          "input": {
                            "text": "conversation_start"
                          },
                          "context": {
                            "name": vetInstance.name
                          }
                        };
                      

                       Da.callCarla(carlaData, function(err, carlaResponse){
                      
                         if (carlaResponse.errorMessage !== undefined && carlaResponse.errorMessage !== null) {
                            errorMessage = "Error invoking Da.callCarla A Error [" + carlaResponse.errorMessage + "]";
                            console.log("Error invoking Da.callCarla A for userFromToken " + userFromToken +" err: ", carlaResponse.errorMessage);
                            responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                            cb(null,response);
                         } else {
                            console.log('======================== Da.callCarla carlaResponse OK');
                           responseBuilder.setSuccessStatus(response);
                           response.payload = carlaResponse;
                           cb(null,response);
                         }
                         // 3) Get response from Carla and send to app
                         //console.log("Da.startConversation. Response", response);

                       });//callCarla - END
                    }
                    db.close();
                  });
                });

            } // else userFromToken is defined
         });  // VaGrit.getUserFromToken - END
  };  // Da.startConversation - END


  Da.converse = function(header, body, cb) {
     console.log("****Start of Da.startConversation.  header: ", JSON.stringify(header));
     //console.log("****Start of Da.converse.  body: ", JSON.stringify(body));
     var response = responseBuilder.createResponseObject();
     var errorMessage;


        VaGrit.getUserFromToken(app, header, function (err, userFromToken) {
           if (userFromToken === undefined || userFromToken === null){
              responseBuilder.setInvalidTokenStatus(response,header);
              cb(null,response);
           }
           else { // userFromToken is defined

              //Check all fields are included
              if (body === undefined || body === null 
                    || body.conversationToken === undefined || body.conversationToken === null || body.conversationToken === ""
                   || body.query === undefined || body.query === null || body.query === "") {
                     
                    errorMessage = "Missing Required fields or values";
                    responseBuilder.setInvalidRequestStatus(response, errorMessage);
                    cb(null,response);
                    return;
               }
               else {

                // Prepare Request for Carla (Decode Context from App Request)

                 try {
                   var carlaData = {
                    "input": {
                      "text": body.query
                    },
                    "context": JSON.parse(Base64.decode(body.conversationToken))
                  };

                  Da.callCarla(carlaData, function(err, carlaResponse){
                    if (carlaResponse.errorMessage !== undefined && carlaResponse.errorMessage !== null) {
                       errorMessage = "Error invoking Da.callCarla B - Error [" + carlaResponse.errorMessage + "]";
                       console.log("Error invoking Da.callCarla B for userFromToken " + userFromToken +" err: ", carlaResponse.errorMessage);
                       responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                       cb(null,response);
                    } else {

                       Da.createJournalEvent(userFromToken, function(err,eventResponse) {
                          if (err){
                             errorMessage = "Error with createJournalEvent";
                             console.log(errorMessage,err);
                             responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                             cb(null,response);
                          }
                          else {
                            responseBuilder.setSuccessStatus(response);
                            response.payload = carlaResponse;
                            cb(null,response);
                          }
                     });

                     }
                     //console.log("Da.startConversation. Response", response);
                  });//callCarla - END

               }
               catch(err) {
                  errorMessage = "Da.converse Error creating carlaData - Error [" + err + "]";
                  console.log("Da.converse Error creating carlaData for userFromToken " + userFromToken +" err: ", err);
                  responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                  cb(null,response);
               }

            }//required fields check

         } // else userFromToken is defined

      });  // VaGrit.getUserFromToken


  };  // Da.converse



  Da.callCarla = function(carlaCallBody, cb) {
    var carlaServerClient = request.createClient(carlaServer);
    var errorMessage = "";
    var response = responseBuilder.createResponseObject();

    carlaServerClient.post('', carlaCallBody, function(err, res, body) {

       if (body === undefined || body.context === undefined || body.context === null) {
          response.errorMessage = "Da.callCarla - Unexpected Carla Error";
          if (body && body.error) {
            response.errorMessage = "Da.callCarla - Error Received from CARLA - Error [" + body.error + "]";
          }
          responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
          //response.payload.carlaError = body; //For Testing
          cb(null,response);
       } else {
        var carlaResponseForAppPayload = {};

        // Convert Carla response to Base64 to send to app
        var token = Base64.encode(JSON.stringify(body.context));

       //console.log("callCarla!!! carlaServerClient RESULT: body.output.text: ", body.output.text);
       carlaResponseForAppPayload = parser.parseDigitialAssistantHTML(body.output.text, token);

        cb(null, carlaResponseForAppPayload);
      }

    });//carlaServerClient - END

  };//Da.callCarla - END



  Da.createJournalEvent = function(vetId,cb){

    app.models.journalEvent.upsert({vetId: vetId, eventType: journalEventConstants.JournalEventTypes.DIGITAL_ASSISTANT ,
                                    date:new Date(),detail:""}, function(err, eventInstance) {
       if (err){
          console.log("An error with journalEvent.upsert", err);
          cb(err,null);

       }
       else {
          console.log("Successfully added a DIGITAL_ASSISTANT journal event for vet id : " + vetId);
          cb(null,eventInstance)
       }

      });

  }

  Da.remoteMethod('startConversation', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/startConversation', verb: 'post'}
  });


  Da.remoteMethod('converse', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/converse', verb: 'post'}
  });




};
