'use strict';

const cheerio = require('cheerio');

exports.parseDigitialAssistantHTML = function(rawHtml, conversationToken) {
  
  console.log("parseDigitialAssistantHTML rawHtml: ", rawHtml);
  
    var rawHtmlString = "";
    for (var i = 0; i < rawHtml.length; i++) { 
      rawHtmlString += rawHtml[i];
    }
  
    var parser = new DigitalAssistParser(rawHtmlString, conversationToken);
    parser.walkTheDom();
    return parser.conversationResponse;
}

function ConversationResponse(conversationToken) {
 
    this.conversationToken = conversationToken;

    this.addResponseSegment = function(text, hyperlink, isParagraph) {
      
      if (text && text.trim() !== "") {
        if (!(this.chatResponse)) {
            this.chatResponse = [ ];
        }

        // Normalize the string.  First convert new lines to spaces, then normalize spaces.
        text = text.replace(/[\t\n\r]/gm, ' ').replace(/\s+/g, ' ');

        var segment = { }
        segment.text = text;

        if (hyperlink) {
            segment.hyperlinkURL = hyperlink;
        }
        if (isParagraph) {
            segment.isParagraph = true;
        }

        this.chatResponse.push(segment);
      }
    }

    this.addNewLineSegment = function() {
        if (!(this.chatResponse)) {
            this.chatResponse = [ ];
        }

        // Interpret a new line as two returns
        var segment = { };
        segment.text = "";

        this.chatResponse.push(segment);
        
    }

    this.addMap = function(placeName, address1, address2, city, state, zip, phoneNumber, hyperlink) {
        this.map = { 
            placeName : placeName,
            address1 : address1,
            address2 : address2,
            city : city,
            state : state,
            zip : zip,
            phoneNumber : phoneNumber,
            hyperlinkURL : hyperlink
        };
    }

    this.addFollowup = function(isButton, optionName, optionValue) {
        if (!(this.followup)) {
            this.followup = { 
                selections : [ ]
            };
        }

        this.followup.followupType = isButton ? 'BUTTON' : 'DROPDOWN';
        
        var newSelection = {
            label : optionName,
            value : optionValue
        };

        this.followup.selections.push(newSelection);
    }

    this.coalescse = function() {
        if (!(this.chatResponse)) {
            return;
        }

        var coalescedResponse = [ ];
        var currentText = "";
        for (let orig of this.chatResponse) {
            if (orig.hyperlinkURL) {
                if (currentText.length > 0) {
                    coalescedResponse.push({
                        text : currentText
                    });
                    currentText = "";
                }
                coalescedResponse.push(orig);
            } else if (orig.isParagraph) {
              if (currentText.length > 0) {
                  coalescedResponse.push({
                      text : currentText
                  });
              }
              currentText = "";
              coalescedResponse.push({
                  text : orig.text
              });
            }
            
            else {
              currentText += orig.text;
            }
        }

        if (currentText.length > 0) {
            coalescedResponse.push({
                text : currentText
            });
        }
        this.chatResponse = coalescedResponse;
    }
}

function DigitalAssistParser(rawHtml, conversationToken) {
    this._rawHtml = rawHtml;
    this.$ = cheerio.load(rawHtml);
    this.conversationResponse = new ConversationResponse(conversationToken);

    this.walkTheDom = function() {
        _walk(this, this.$('html').get(0), "");
        this.conversationResponse.coalescse();
    }

    var _handleHyperlink = function(parserObject, node) { 
        var text = "";
        var firstChild = node.firstChild;
        if (firstChild) {
            text = firstChild.nodeValue;
        }

        var hyperlink = "";
        if (node.attribs && node.attribs.href) {
            hyperlink = node.attribs.href;
        }

        parserObject.conversationResponse.addResponseSegment(text, hyperlink, false);
    }

    var _handleBreak = function(parserObject) { 
        parserObject.conversationResponse.addNewLineSegment();
    }

    var _handleText = function(parserObject, textValue, isParagraph) { 
        //console.log("HANDLE TEXT textValue: ", textValue);
        parserObject.conversationResponse.addResponseSegment(textValue, null, isParagraph);
    }

    var _handleMap = function(parserObject, node) { 
        var placeName = node.attribs.name;
        var address1 = node.attribs.address1;
        var address2 = node.attribs.address2;
        var city = node.attribs.city;
        var state = node.attribs.state;
        var zip = node.attribs.zip;
        var phone = node.attribs.phone;
        var hyperlink = node.attribs.href;

        parserObject.conversationResponse.addMap(placeName, address1, address2, city, state, zip, phone, hyperlink);
    }

    var _handleFollowup = function(parserObject, node, isButton) { 
        var value = node.attribs.value;

        var text = "";
        var firstChild = node.firstChild;
        if (firstChild) {
            text = firstChild.nodeValue;
        }

        parserObject.conversationResponse.addFollowup(isButton, text, value);
    }

    var _walk = function(parserObject, node) {
        if (node.type === "text" && node.parent.name !== 'p') {
            _handleText(parserObject, node.nodeValue, false);
        } else if (node.type === "tag") {
            switch (node.name) {
                case 'a':
                    _handleHyperlink(parserObject, node);
                    return;
                case 'br':
                    _handleBreak(parserObject);
                    return;
                case 'map':
                    _handleMap(parserObject, node);
                    return;
                case 'button':
                    _handleFollowup(parserObject, node, true);
                    return;
                case 'option':
                    _handleFollowup(parserObject, node, false);
                    return;
            }

            for (let child of node.children) {
                _walk(parserObject, child);
            } 

            if (node.name === 'p') {
                _handleText(parserObject, node.children[0].data, true);
            }
        }
    } 
}
