'use strict';

var app = require('../../server/server');
const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');


module.exports = function(Notifications) {
  
  Notifications.createNotification = function(body, cb) {
   var response = responseBuilder.createResponseObject();
   var errorMessage;
   
   //Required fields check 
   if (body === undefined || body === null 
         || body.vetId === undefined || body.vetId === null || body.vetId === ""
         || body.notificationId === undefined || body.notificationId === null || body.notificationId === ""
         || body.effectiveDate === undefined || body.effectiveDate === null || body.effectiveDate === ""
         || body.metadata === undefined || body.metadata === null || body.metadata === ""
         || body.systemNotification === undefined || body.systemNotification === null || body.systemNotification === ""
         || body.notificationType === undefined || body.notificationType === null || body.notificationType === ""
         || body.title === undefined || body.title === null || body.title === ""
         || body.description === undefined || body.description === null || body.description === ""
         || body.title.en === undefined || body.title.en === null || body.title.en === ""
         || body.description.en === undefined || body.description.en === null || body.description.en === "") {
     errorMessage = "Missing Required fields or values";
     responseBuilder.setInvalidRequestStatus(response, errorMessage);
     cb(null,response);
     return;
   }
   
   
   //console.log("****Start of Notifications.createNotification.  body: ", JSON.stringify(body));
   
   var notification = app.models.notification;


   
   //CHECK IF NOTIFICATION FOR USER WITH ID ALREADY EXISTS
   notification.find({where : {and: [{notificationId: body.notificationId}, {vetId: body.vetId}]}}, function(err, foundNotifications) {
      if (err){
        errorMessage = "notification.create error: " + err;
        console.log(errorMessage, err);
        responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
        cb(null,response);
      }
      else{
        console.log("notification.find foundNotifications:", foundNotifications[0]);
        if (foundNotifications.length > 0) {
          
            console.log("Notifications.createNotification Notification Exists Update foundNotifications[0]:", foundNotifications[0]);
          
             foundNotifications[0].effectiveDate = body.effectiveDate; 
             foundNotifications[0].title = body.title;  
             foundNotifications[0].description = body.description;  
             foundNotifications[0].metadata = body.metadata;  
             foundNotifications[0].systemNotification = body.systemNotification; 
             foundNotifications[0].notificationType = body.notificationType; 
             
             foundNotifications[0].save(function(err, savedNotification) {
                if (err){
                  errorMessage = "notification.create error: " + err;
                  console.log(errorMessage, err);
                  responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                  cb(null,response);
                }
                else{
                  console.log("notification.find savedNotification:", savedNotification);
                  
                  app.models.auditEvent.addEvent({
                    vetId: body.vetId.toString(),
                    eventType: app.models.auditEvent.eventTypes.UPDATE_NOTIFICATION,
                    detail: {
                      id: savedNotification.id.toString(),
                      notificationId: savedNotification.notificationId,
                      notificationType: savedNotification.notificationType,
                      effectiveDate: savedNotification.effectiveDate,
                      title: foundNotifications[0].title.en,
                      metadata: foundNotifications[0].metadata
                    }
                  });
                  
                  responseBuilder.setSuccessStatus(response);
                  //console.log("Notifications.updateNotification. Response", response);
                  cb(null,response);
                }
             }); 
            
            
            
        } else {
          //Creare Notification
           notification.create(body, function(err, createdNotification) {
              if (err){
                errorMessage = "notification.create error: " + err;
                console.log(errorMessage, err);
                responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                cb(null,response);
              }
              else{
                console.log("notification.find createdNotification:", createdNotification);
                
                
                app.models.auditEvent.addEvent({
                  vetId: body.vetId.toString(),
                  eventType: app.models.auditEvent.eventTypes.CREATE_NOTIFICATION,
                  detail: {
                    id: createdNotification.id.toString(),
                    notificationId: createdNotification.notificationId,
                    notificationType: createdNotification.notificationType,
                    effectiveDate: createdNotification.effectiveDate,
                    title: createdNotification.title.en,
                    metadata: createdNotification.metadata
                  }
                });
                
                responseBuilder.setSuccessStatus(response);
                console.log("Notifications.createNotification. Response", response);
                cb(null,response);
              }
           }); 
        }
      }
        
    });

  };  // Notifications.createNotification


  
  Notifications.updateNotification = function(body, cb) {
   var response = responseBuilder.createResponseObject();
   var errorMessage;
   
   
   //Required fields check 
   if (body === undefined || body === null 
         || body.vetId === undefined || body.vetId === null || body.vetId === ""
         || body.notificationId === undefined || body.notificationId === null || body.notificationId === ""
         || body.effectiveDate === undefined || body.effectiveDate === null || body.effectiveDate === ""
         || body.metadata === undefined || body.metadata === null || body.metadata === "") {
     errorMessage = "Missing Required fields or values";
     responseBuilder.setInvalidRequestStatus(response, errorMessage);
     cb(null,response);
     return;
   }
   
    //console.log("****Start of Notifications.updateNotification.  body: ", JSON.stringify(body));
    //console.log("****Start of Notifications.updateNotification.  body: ", JSON.stringify(body.vetId));
   
     var notification = app.models.notification;

     notification.find({where : {and: [{notificationId: body.notificationId}, {vetId: body.vetId}]}}, function(err, foundNotifications) {
        if (err){
          errorMessage = "notification.create error: " + err;
          console.log(errorMessage, err);
          responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
          cb(null,response);
        }
        else{
          console.log("notification.find foundNotifications:", foundNotifications[0]);
          
            if (foundNotifications.length > 0) {
              foundNotifications[0].effectiveDate = body.effectiveDate;
              foundNotifications[0].metadata = body.metadata;
                
               foundNotifications[0].save(function(err, savedNotification) {
                  if (err){
                    errorMessage = "notification.create error: " + err;
                    console.log(errorMessage, err);
                    responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                    cb(null,response);
                  }
                  else{
                    console.log("notification.find savedNotification:", savedNotification);
                    
                    app.models.auditEvent.addEvent({
                      vetId: body.vetId.toString(),
                      eventType: app.models.auditEvent.eventTypes.UPDATE_NOTIFICATION,
                      detail: {
                        id: savedNotification.id.toString(),
                        notificationId: savedNotification.notificationId,
                        notificationType: savedNotification.notificationType,
                        effectiveDate: savedNotification.effectiveDate,
                        title: foundNotifications[0].title.en,
                        metadata: foundNotifications[0].metadata
                      }
                    });
                    
                    responseBuilder.setSuccessStatus(response);
                    console.log("Notifications.updateNotification. Response", response);
                    cb(null,response);
                  }
               }); 
             } else {
               errorMessage = "notification.update error. cannot update notificationId: " + body.notificationId;
               console.log(errorMessage, err);
               responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
               cb(null,response);
             }
        }
     }); 
     

  };  // Notifications.updateNotification





  Notifications.remoteMethod('createNotification', {
      accepts: [{ arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/createNotification', verb: 'post'}
  });
  

  Notifications.remoteMethod('updateNotification', {
      accepts: [{ arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/updateNotification', verb: 'post'}
  });
  


  

};
