'use strict';

var app = require('../../server/server');
var constants = require('../constants').constants(app);
var mongoUrl = constants.mongo.url;
var mongoDbName = constants.mongo.dbName;
var ACCEPTED = constants.accepted;
var accountSid = constants.twilio.accountSid;
var authToken = constants.twilio.authToken;
var fromPhoneNumber = constants.twilio.fromPhoneNumber;


const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');

var mongo = require('mongodb');
var MongoClient = require('mongodb').MongoClient;
const twilio = require('twilio');

module.exports = function(Squad) {
  
  //Get vet info, vet name
  //Get vet squad mates that are ACCEPTED
  //Loop through squad mates.
     //Replace VET_NAME, SQUAD_MATE_NAME in message
     //Send SMS via twilio
  //Do callback after loop is complete
  
  Squad.sendSmsToSquad = function(body, cb) {
   var response = responseBuilder.createResponseObject();
   var errorMessage;
   
   //Required fields check 
   if (body === undefined || body === null 
         || body.vetId === undefined || body.vetId === null || body.vetId === ""
         || body.message === undefined || body.message === null || body.message === "") {
     errorMessage = "Missing Required fields or values";
     responseBuilder.setInvalidRequestStatus(response, errorMessage);
     cb(null,response);
     return;
   }
   //console.log("****Start of Squad.sendSmsToSquad.  body: ", JSON.stringify(body));
   
   
   console.log("****Start of Squad.sendSmsToSquad.  body.vetId: ", body.vetId);
   //console.log("****Start of Squad.sendSmsToSquad.  body.message: ", body.message);
   
   
   MongoClient.connect(mongoUrl, function(err, db) {
      console.log('Squad.sendSmsToSquad: START body.vetId: ', body.vetId);
     if (err) throw err;
     var dbo = db.db(mongoDbName);
     
     
     var objectId = new mongo.ObjectID(body.vetId);
     var bodyVetId = body.vetId;
     
     dbo.collection("vet").find({_id:objectId}).toArray(function(err, vetInstance) {
       //console.log('********************** Squad.sendSmsToSquad: vetInstance: ', vetInstance);
       //console.log('********************** Squad.sendSmsToSquad: err: ', err);
       if (err !== null) {
          db.close();
          console.log('Squad.sendSmsToSquad: err: ', err);
          var errorMessage = "Squad.sendSmsToSquad Error invoking vetServerClient.getUser - Error [" + err + "]";
          responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
          cb(null,response);
          
       } else {
          console.log('********************** Squad.sendSmsToSquad send reponse to vetInstance: ', vetInstance);
          if (vetInstance === undefined || vetInstance === null || vetInstance.length === 0) {
             db.close();
             errorMessage = "Squad.sendSmsToSquad No matching record found";
             responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
             cb(null,response);
          } 
          else {
            console.log('********************** Squad.sendSmsToSquad send reponse to vetInstance: ', vetInstance);
            
            dbo.collection("mate").find({$and: [{vetId:body.vetId}, {state:ACCEPTED}]}).toArray(function(err, mateInstances) {
              //console.log('********************** Squad.sendSmsToSquad: mateInstances: ', mateInstances);
              //console.log('********************** Squad.sendSmsToSquad: err: ', err);
              if (err !== null) {
                
                 db.close();
                 console.log('Squad.sendSmsToSquad: err: ', err);
                 var errorMessage = "Squad.sendSmsToSquad Error invoking vetServerClient.getUser - Error [" + err + "]";
                 responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                 cb(null,response);
                 
              } else {
                   console.log('********************** Squad.sendSmsToSquad send reponse to mateInstances: ', mateInstances);
                 
                 if (mateInstances === undefined || mateInstances === null || mateInstances.length === 0) {
                       
                   
                   //Add to auditEvents - Do not wait
                   app.models.auditEvent.addEvent({
                     vetId: bodyVetId.toString(),
                     eventType: app.models.auditEvent.eventTypes.SENT_SMS_TO_SQUAD,
                     detail: {squadMatesSentTo:[], message: "This user has no Accepted SquadMates"}
                   });
                   
                    db.close();
                    errorMessage = "Squad.sendSmsToSquad No matching record found";
                    responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                    cb(null,response);
                 } 
                 else {
                   console.log('********************** Squad.sendSmsToSquad send reponse to mateInstances: ', mateInstances);
                   db.close();
                   
                   response.payload.smsMessagesSent = 0;
                   var squadMatesSentTo = [];
                   
                   for (var i = 0; i < mateInstances.length; i++) { 
                     
                     
                     //EXTRA CHECK FOR ACCEPTED
                     if (mateInstances[i].state = ACCEPTED) {
                       squadMatesSentTo.push(mateInstances[i]._id.toString());
                       //SEND SMS TO SQUAD MATE 
                       var client = new twilio(accountSid, authToken);
                       var toPhone = mateInstances[i].cellPhone;
                       
                       //FIELDS FOR TEMPLATE LITERALS
                       var vet_name = vetInstance[0].name;//${vet_name}
                       var squad_mate_name = mateInstances[i].name;//${squad_mate_name}
                       var smsBody = eval('`'+body.message+'`');
                       
                       
                       
                       //For Testing
                       //console.log('********************** Squad.sendSmsToSquad send reponse to vetInstance: ', vetInstance);
                       //console.log('~~~~~~~~~~Squad.sendSmsToSquad send reponse to mateInstances[i]: ', mateInstances[i]);
                       //console.log('********************** Squad.sendSmsToSquad send reponse to smsBody: ', smsBody);
                       //response.payload.smsBody = smsBody;
                       //response.payload.toPhone = toPhone;
                       response.payload.smsMessagesSent++;
                       
                       
                       client.messages.create({
                         body: smsBody,
                         from: fromPhoneNumber,
                         to: toPhone
                       })
                       .then(function(message) {
                         
                         console.log("TWILIO Response message: ", message);
                         console.log("TWILIO Response message.sid: ", message.sid);
                           
                       });
                       
                       
                     }//EXTRA CHECK FOR ACCEPTED - END
                   }// Loop through Squad Mates - END
                   
                   
                   //Add to auditEvents - Do not wait
                   app.models.auditEvent.addEvent({
                     vetId: bodyVetId.toString(),
                     eventType: app.models.auditEvent.eventTypes.SENT_SMS_TO_SQUAD,
                     detail: {squadMatesSentTo: squadMatesSentTo}
                   });
                   
                   
                  
                   responseBuilder.setSuccessStatus(response);
                   //response.payload.vetInstance = vetInstance;
                   //response.payload.mateInstances = mateInstances;
                   
                   cb(null, response);
                 }// ELSE - Has Squadmates - END
              }// ELSE mateInstances is undefined - END
            });//dbo.collection("mate").find - END
            
          }
       }
     });
   });
   
   
  };  // Squad.sendSmsToSquad


  Squad.remoteMethod('sendSmsToSquad', {
      accepts: [{ arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/sendSmsToSquad', verb: 'post'}
  });

};
