'use strict';
var badgeConstants = require('va-grit-microservices/constants/badgeConstants');
var journalEventConstants = require('va-grit-microservices/constants/journalEventConstants');

var app = require('../../server/server');
var constants = require('../constants').constants(app);
var wemServerBaseUrl = constants.cognitive.wemServerBaseUrl;
var numberOfJobSearchResultsToRetrieveFromWEM = constants.cognitive.jobSearchNumberOfResultToRetrieve;

const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');

var request = require('request-json');

module.exports = function(Jobs) {


  Jobs.searchJobs = function(header, body, cb) {
     console.log("****Start of Jobs.searchJobs  header: ", JSON.stringify(header));
     var response = responseBuilder.createResponseObject();
     var errorMessage;

     VaGrit.getUserFromToken(app, header, function (err, userId) {
        if (userId === undefined || userId === null){
           responseBuilder.setInvalidTokenStatus(response, header);
           response.payload = [];
           cb(null,response);
        }
        else { // userId is defined
            console.log(' VaGrit.getUserFromToken START Jobs.find userId: ', userId);

            var jobSearchUrl = "WEMUsers/performJobSearch";
            jobSearchUrl += "?isMOS="+false.toString();
            jobSearchUrl += "&isLite="+false.toString();
            jobSearchUrl += "&veteranId="+userId;
            jobSearchUrl += "&noOfJobs="+numberOfJobSearchResultsToRetrieveFromWEM;
            //jobSearchUrl += "&query="+' ';//Leave out
            //console.log(' VaGrit.getUserFromToken START Jobs.find jobSearchUrl: ', jobSearchUrl);

            var wemServerClient = request.createClient(wemServerBaseUrl);
            wemServerClient.post(jobSearchUrl, {}, function(err, res, body) {
              console.log(">>>>>>>>>>>>>>>>>> call WEM!!! wemServerClient RESULT: error: ", err);
              console.log(">>>>>>>>>>>>>>>>>> call WEM!!! wemServerClient RESULT: body: ", body);
              
               if (body !== undefined && body !== null && body.error !== undefined && body.error !== null && body.error.message !== undefined && body.error.message === 'NO_WEM_PROFILE') {
                  response.errorMessage = "This user does not have a profile in WEM.";
                  responseBuilder.setNoWemProfileStatus(response, errorMessage);
                  response.payload = [];
                  cb(null,response);
                  return;
               }
               
               if (body === undefined || body.searchresult === undefined || body.searchresult === null) {
                  response.errorMessage = "Call WEM - Unexpected WEM Error";
                  if (body && body.error) {
                    response.errorMessage = "Call WEM - Error Received from WEM - Error [" + body.error + "]";
                  }
                  responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                  response.payload = [];
                  cb(null,response);
                  return;
               }
                 

               // 1) Get Saved Jobs
               Jobs.getSavedJobs(header, {}, function(err, savedJobsResults){
                 //console.log('Jobs.searchJobs Jobs.getSavedJobs savedJobsResults: ', savedJobsResults);
                  if (err !== undefined && err !== null) {
                    response.errorMessage = "Call Jobs.getSavedJobs - Error [" + err + "]";
                    responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                    response.payload = [];
                    cb(null,response);
                    return;
                  }

                  var searchResultsToReturn = [];

                  // 2) Loop through WEMJobSearchResults
                  //console.log("Jobs.searchJobs Jobs.getSavedJobs body.searchresult.results: ", body.searchresult.results);
                  for (var i = 0; i < body.searchresult.results.length; i++) {
                    var jobSearchResultRaw = body.searchresult.results[i];
                    //console.log("+++++++++++++++ Jobs.getSavedJobs jobSearchResultRaw: ", jobSearchResultRaw);

                      // 2a) Create Job Object for result to app
                      // 2b) And Check if this jobId is saved - if yes set saved to true
                      var savedJobId = "";
                      var jobSearchResultFinal = {
                        "jobId": jobSearchResultRaw.JobID,
                        "jobTitle": jobSearchResultRaw.CanonJobTitle,
                        "employer": jobSearchResultRaw.CanonEmployer,
                        "location": jobSearchResultRaw.CanonCity+", "+ jobSearchResultRaw.CanonState,
                        "minSalary": jobSearchResultRaw.MinAnnualSalary,
                        "maxSalary": jobSearchResultRaw.MaxAnnualSalary,
                        "applicationURL": jobSearchResultRaw.JobURL,
                        "employmentType": jobSearchResultRaw.CanonJobType,
                        "hoursType": jobSearchResultRaw.CanonJobHours,
                        "overview": jobSearchResultRaw.JobText,
                        "experience": jobSearchResultRaw.CanonYearsOfExperienceCanonLevel,
                        "savedJobId": savedJobsResults.payload.some(function(job) {
                          if (job.jobId === jobSearchResultRaw.JobID) {
                            savedJobId = job.id;
                          }
                          return job.jobId === jobSearchResultRaw.JobID;
                        })
                      }
                      
                      jobSearchResultFinal.savedJobId = savedJobId;
                      searchResultsToReturn.push(jobSearchResultFinal);
                  }

                  
                  //Add to auditEvents - Do not wait
                  app.models.auditEvent.addEvent({
                    vetId: userId.toString(),
                    eventType: app.models.auditEvent.eventTypes.SEARCH_JOBS,
                    detail: {}
                  });

                  responseBuilder.setSuccessStatus(response);
                  response.payload = searchResultsToReturn;
                  //console.log(' VaGrit.getUserFromToken START Jobs.find response: ', response);
                  cb(null, response);
               });// Get Saved Jobs - END
             });//wemServerClient.post(jobSearchUrl) -- END
        } // else userId is defined - END
    });  // VaGrit.getUserFromToken - END
  }// END - searchJobs



  Jobs.addSavedJob = function(header, body, cb) {
     var response = responseBuilder.createResponseObject();
     var errorMessage;

     if (body === undefined || body === null 
           || body.jobId === undefined || body.jobId === null || body.jobId === ""
           || body.jobTitle === undefined || body.jobTitle === null
           || body.employer === undefined || body.employer === null
           || body.location === undefined || body.location === null
           || body.minSalary === undefined || body.minSalary === null
           || body.maxSalary === undefined || body.maxSalary === null
           || body.applicationURL === undefined || body.applicationURL === null) {
        errorMessage = "Required Fields are missing";
        responseBuilder.setInvalidRequestStatus(response, errorMessage);
        cb(null,response);
        return;
     }
     //console.log("****Start of Jobs.addSavedJob  header: ", JSON.stringify(header));
     //console.log("****Start of Jobs.addSavedJob.  body: ", JSON.stringify(body));


     VaGrit.getUserFromToken(app, header, function (err, userId) {
        if (userId === undefined || userId === null){
           responseBuilder.setInvalidTokenStatus(response, header);
           cb(null,response);
           return;
        }
        else { // userId is defined
            var jobToSave = body;
            jobToSave.vetId = userId;

            console.log(' VaGrit.getUserFromToken START Jobs.find userId: ', userId);
            //console.log(' VaGrit.getUserFromToken START Jobs.find body.jobId: ', body.jobId);
            //console.log(' VaGrit.getUserFromToken START Jobs.find jobToSave: ', jobToSave);

            app.models.job.findOrCreate({where: {and: [{vetId:userId}, {jobId:body.jobId}]}}, jobToSave, function(err, jobToSaveResults){
              if (err !== null || jobToSaveResults === null) {
                 errorMessage = "Error Saving Job [" + err + "]";
                 console.log("Error Saving Job for userId " + userId +" err: ", err);
                 responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                 cb(null,response);
                 return;
              }
              //console.log(' VaGrit.getUserFromToken START Jobs.find jobToSaveResults: ', jobToSaveResults);

              Jobs.handleEventJournalAndBadge(userId, jobToSave.jobTitle,function(err,badge) {
                 if (err){
                   errorMessage = "Error with handleEventJournalAndBadge";
                   console.log(errorMessage,err);
                   responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                   cb(null,response);
                   return;
                 }
                 else {
                   
                   
                   //Add to auditEvents - Do not wait
                   app.models.auditEvent.addEvent({
                     vetId: userId.toString(),
                     eventType: app.models.auditEvent.eventTypes.SAVE_JOB,
                     detail: {}
                   });
                   
                   responseBuilder.setSuccessStatus(response);
                   response.payload.id = jobToSaveResults.id;
                   if (badge !== null){
                      response.badgeAward.push(badge);
                   }

                   //console.log(' VaGrit.getUserFromToken START Jobs.find response: ', response);
                   cb(null, response);

                 }

              });


            });
        } // else userId is defined
    });  // VaGrit.getUserFromToken
  }// END - addSavedJob


  Jobs.getSavedJobs = function(header, body, cb) {
     //console.log("****Start of Jobs.getSavedJobs  header: ", JSON.stringify(header));
     //console.log("****Start of Jobs.getSavedJobs.  body: ", JSON.stringify(body));
     var response = responseBuilder.createResponseObject();
     var errorMessage;

     VaGrit.getUserFromToken(app, header, function (err, userId) {
        if (userId === undefined || userId === null){
           responseBuilder.setInvalidTokenStatus(response, header);
           cb(null,response);
           return;
        }
        else { // userId is defined

            app.models.job.find({where: {vetId:userId}, fields: {vetId: false}}, function(err, getSavedJobsResults){
              if (err !== null || getSavedJobsResults === null) {
                 errorMessage = "Error Saving Job [" + err + "]";
                 console.log("Error Saving Job for userId " + userId +" err: ", err);
                 responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                 cb(null,response);
                 return;
              }
              //console.log(' VaGrit.getUserFromToken START Jobs.find getSavedJobsResults: ', getSavedJobsResults);
              responseBuilder.setSuccessStatus(response);
              response.payload = getSavedJobsResults;
              //console.log(' VaGrit.getUserFromToken START Jobs.find response: ', response);
              cb(null, response);
            });
        } // else userId is defined
    });  // VaGrit.getUserFromToken
  }// END - getSavedJobs




  Jobs.deleteSavedJob = function(header, body, cb) {
     //console.log("****Start of Jobs.deleteSavedJob  header: ", JSON.stringify(header));
     var response = responseBuilder.createResponseObject();
     var errorMessage;

     if (body === undefined || body === null 
           || body.id === undefined || body.id === null || body.id === "") {
        errorMessage = "Required Fields are missing";
        responseBuilder.setInvalidRequestStatus(response, errorMessage);
        cb(null,response);
        return;
     }
     //console.log("****Start of Jobs.deleteSavedJob.  body: ", JSON.stringify(body));
     

     VaGrit.getUserFromToken(app, header, function (err, userId) {
        if (userId === undefined || userId === null){
           responseBuilder.setInvalidTokenStatus(response, header);
           cb(null,response);
           return;
        }
        else { // userId is defined

            console.log(' VaGrit.getUserFromToken START Jobs.find userId: ', userId);
            //console.log(' VaGrit.getUserFromToken START Jobs.find body.id: ', body.id);

            app.models.job.destroyAll({and: [{vetId:userId}, {id:body.id}]}, function(err, deleteSavedJobResults){
              if (err !== null || deleteSavedJobResults === null) {
                 errorMessage = "Error Saving Job [" + err + "]";
                 console.log("Error Saving Job for userId " + userId +" err: ", err);
                 responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                 cb(null,response);
                 return;
              }
              console.log(' VaGrit.getUserFromToken START Jobs.find deleteSavedJobResults: ', deleteSavedJobResults);
              
              //Add to auditEvents - Do not wait
              app.models.auditEvent.addEvent({
                vetId: userId.toString(),
                eventType: app.models.auditEvent.eventTypes.DELETE_JOB,
                detail: {}
              });
              
              responseBuilder.setSuccessStatus(response);
              response.payload = deleteSavedJobResults;
              console.log(' VaGrit.getUserFromToken START Jobs.find response: ', response);
              cb(null, response);
            });
        } // else userId is defined
    });  // VaGrit.getUserFromToken
  }// END - deleteSavedJob


  Jobs.handleEventJournalAndBadge = function(vetId,jobTitle,cb){
    // Add the Journal Event for adding a job
    app.models.journalEvent.upsert({vetId: vetId, eventType: journalEventConstants.JournalEventTypes.SAVED_JOB  ,
                                    date:new Date(),detail:jobTitle}, function(err, eventInstance) {
      if (err){
        console.log("An error with journalEvent.upsert", err);
        cb(err,null);

      }
      else {   // else not an error with journalEvent.upsert

          var badgeToAdd = badgeConstants.BadgeTypes.JOB_SAVED;
          app.models.userBadge.addBadge({vetId: vetId, badge: badgeToAdd },function(err, badge) {
            if (err){
              console.log("Error with userBadge.addBadge for badge [" + badgeToAdd + "] for vetId[" + vetId + "]", err);
              cb(err,null);
            }
            else {
              //console.log("Successful with userBadge.addBadge for badge [" + badgeToAdd + "] for vetId[" + vetId + "]", badge);
              cb(null,badge);
            }
          });   // userBadge.addBadge



    } // end. / else not an error with journalEvent.upsert
   });  // end journalEvent.upsert
  };


  Jobs.remoteMethod('getSavedJobs', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/getSavedJobs', verb: 'post'}
  });



  Jobs.remoteMethod('addSavedJob', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/addSavedJob', verb: 'post'}
  });



  Jobs.remoteMethod('deleteSavedJob', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/deleteSavedJob', verb: 'post'}
  });


  Jobs.remoteMethod('searchJobs', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/searchJobs', verb: 'post'}
  });













};
