'use strict';

var app = require('../../server/server');
var constants = require('../constants').constants(app);
var mongoUrl = constants.mongo.url;
var mongoDbName = constants.mongo.dbName;
var wemServerBaseUrl = constants.cognitive.wemServerBaseUrl;
const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');
var MongoClient = require('mongodb').MongoClient;

var base64 = require('file-base64');
const tesseract = require('tesseractocr');
const recognize = tesseract.withOptions({
    psm: 4
});

var resumeStoragePath = "./common/resume-temp-storage";

var request = require('request');
var FormData = require('form-data');

var resumeTypeFile = "FILE";
var resumeTypePhoto = "PHOTO";
var resumeTypePhotoTemp = "PHOTO-TEMP";
var resumePhotoTempFileName = "-photo.txt";
var resumeTypeFilePageNumber = 1;

var writeFile = require('write');
var fs = require('file-system');
var gm = require('gm').subClass({imageMagick: true});
var badgeConstants = require('va-grit-microservices/constants/badgeConstants');

module.exports = function(Resume) {

 var resumeText = app.models.resumeText;

  //Loopback wraps the response in a "response" object and we don't want to
  // return that, this removes the response we created from the "response" object
  // and only returns the response we created.
  // TODO: May need to adjust this so only works on custom remote methods we created.
  var remotes = app.remotes();
  remotes.after('**', function (ctx, next) {
      if (ctx) {
        if (ctx.result.response !== null) {
          ctx.result = ctx.result.response;
        }
      } else {
          //console.log("ERROR. vet.js. remotes.after. ctx is not set");
          next({payload: { },
                status: {
                 statusType: responseBuilder.StatusTypes.UNEXPECTED_EXCEPTION,
                 errorMessage: "There was an error on the server. ctx is not set"
               }
          });
      }
      next();
  });

  // THIS FUNCTION WILL ACCEPT A BASE-64 FILE OF EITHER A PHOTO OR A FILE (accepted types .doc .docx .pdf)
  // IT WILL SAVE THE FILE AS TEXT FOR PHOTOS CONVERTED TO TEXT AND AS BASE-64 FOR NON-PHOTO FILES
  // IT SAVES THE UPLOADED RESUME TO THE DATABASE FOR FUTURE USE WITH uploadComplete
  Resume.uploadResume = function(header, body, cb) {
     //console.log("****Start of Resume.uploadResume. header: ", JSON.stringify(header));
     //console.log("****Start of Resume.uploadResume. body.uploadType: ", JSON.stringify(body.uploadType));
     //console.log("****Start of Resume.uploadResume. body.filename: ", JSON.stringify(body.filename));
     //console.log("****Start of Resume.uploadResume. body.page: ", body.page);
     var response = responseBuilder.createResponseObject();
     var errorMessage;
     VaGrit.getUserFromToken(app, header, function (err, userId) {
        if (userId === undefined || userId === null){
           responseBuilder.setInvalidTokenStatus(response,header);
           cb(null,response);
        }
        else { // userId is defined
           //console.log('Resume.uploadResume VaGrit.getUserFromToken userId: ', userId);
            /*
            body.uploadType
            body.filename
            body.page
            body.data

            //1) If PDF or WORD, it will be base-64
            //1a) Convert it back to PDF or WORD
            //1b) Send to WEM
            //1c) Save WEM response

            //2) If IMAGE, it will be base-64
            //2a) Convert to an image
            //2b) Run the image through Tessaract to get the text
            //2c) Temporarily save the text to the database
            //2d) Repeat above steps for other pages if needed
            //2e) Upon resumeUploadComplete...
            //2f) Combine the saved pages to a text file and send to WEM
            //2g) Save WEM Response
            // 2h) SEND TEXT OF IMAGE BACK TO APP
            */
             
            //Required fields check 
            if (body === undefined || body === null 
                  || body.uploadType === undefined || body.uploadType === null || body.uploadType === "" || typeof body.uploadType !== "string"
                  || body.data === undefined || body.data === null || body.data === "" || typeof body.data !== "string") {
              errorMessage = "Missing Required fields or values";
              responseBuilder.setInvalidRequestStatus(response, errorMessage);
              cb(null,response);
              return;
            }


            if (body.uploadType !== resumeTypePhoto) {
              
  // NEW SAVE RESUME FILE TO DB UNTIL uploadComplete
  // 1) DONE - Save the base64 with resumeType=FILE
  // 1a) DONE - Save photos with resumeType=IMAGE/PHOTO
  // 2) On Upload Complete - 
  // 2a) If resumeType===FILE convert from Base64 - send file

              //console.log("Resume.uploadResume - NOT PHOTO");
              //console.log("Resume.uploadResume - body.data: ", body.data);
              
              
              //Required fields check 
              if (body.filename === undefined || body.filename === null || body.filename === "" || typeof body.filename !== "string") {
                errorMessage = "Missing Required fields or values";
                responseBuilder.setInvalidRequestStatus(response, errorMessage);
                cb(null,response);
                return;
              }
              
              
              var userFileNameOriginal = body.filename.replace(/\s/g, '_');//replace any spaces from file name with underscore
              var userFileName = userId + userFileNameOriginal;
              //console.log("Resume.uploadResume - userFileName: ", userFileName);
              
              //Delete any existing old PHOTO files (cannot mix PHOTO and NON-PHOTO files)
              app.models.resumeText.destroyAll({and: [{vetId: userId}, {resumeType:resumeTypePhoto}]}, function(err, deleteResponse) {
                  //console.log('Resume.uploadComplete - resumeText.destroyAll - err: ', err);
                  //console.log('Resume.uploadComplete - resumeText.destroyAll - deleteResponse: ', deleteResponse);
                
                if (err) {
                  errorMessage = "Error Deleting Old User Resume Files";
                  responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                  cb(null,response);
                  return;
                }
                  
                // 1) Save the base64 with resumeType=FILE
                app.models.resumeText.find({where: {and: [{vetId: userId}, {page: resumeTypeFilePageNumber}, {resumeType: resumeTypeFile}]}}, function(err, resumeTextSaveFileResponse) {
                  if (err) {
                     errorMessage = "Error invoking resumeTextSaveFile.find. Error [" + err.message + "]";
                     //console.log("Error invoking resumeTextSaveFile.find for userId " + userId +" err: ", err);
                     responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                     cb(null,response);
                  }
                  else {
                    //console.log('Resume.uploadResume - resumeText.find - resumeTextSaveFileResponse: ', resumeTextSaveFileResponse);
                    var resumeTextData = { 
                      vetId: userId, 
                      page: resumeTypeFilePageNumber, 
                      text: body.data, 
                      resumeType: resumeTypeFile,
                      resumeFileName: userFileName
                    };
                    if (resumeTextSaveFileResponse === null || resumeTextSaveFileResponse.length === 0) {
                      //CREATE NEW
                      app.models.resumeText.create(resumeTextData, function(err, resumeTextSaveFileResponse) {
                        if (err) {
                           errorMessage = "Error invoking resumeTextSaveFile.create. Error [" + err.message + "]";
                           //console.log("Error invoking resumeTextSaveFile.create for userId " + userId +" err: ", err);
                           responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                           cb(null,response);
                        }
                        else {
                          //console.log('Resume.uploadResume - resumeTextSaveFile.create - resumeTextSaveFileResponse: ', resumeTextSaveFileResponse);
                          response.payload.resumeFilename = userFileNameOriginal;
                          response.payload.uploadTimestamp = new Date().toISOString();
                          responseBuilder.setSuccessStatus(response);
                          cb(null,response);
                        }
                      });

                    } else {
                      //UPDATE
                      resumeTextSaveFileResponse[0].vetId = userId;
                      resumeTextSaveFileResponse[0].text = body.data;
                      resumeTextSaveFileResponse[0].resumeFileName = userFileName;

                      resumeTextSaveFileResponse[0].save(function(err, resumeTextSaveFileResponse) {
                        if (err) {
                         errorMessage = "Error invoking resumeTextSaveFileResponse[0].save. Error [" + err.message + "]";
                         //console.log("Error invoking resumeTextSaveFileResponse[0].save for userId " + userId +" err: ", err);
                         responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                         cb(null,response);
                        }
                        else {
                          //console.log('Resume.uploadResume - resumeTextSaveFileResponse[0].save - resumeTextSaveFileResponse[0]: ', resumeTextSaveFileResponse[0]);
                          response.payload.resumeFilename = userFileNameOriginal;
                          response.payload.uploadTimestamp = new Date().toISOString();
                          responseBuilder.setSuccessStatus(response);
                          cb(null,response);
                        }
                      });
                    }
                  }
                });
                
              });// END - Delete and existing old PHOTO files
              
              
              
              
            } /// NOT PHOTO - END
            
            
            else {
              //console.log("Resume.uploadResume - IS PHOTO");
              
              
              //Required fields check 
              if (body.page === undefined || body.page === null || typeof body.page !== "number") {
                errorMessage = "Missing Required fields or values";
                responseBuilder.setInvalidRequestStatus(response, errorMessage);
                cb(null,response);
                return;
              }
              
              //Delete any existing old NON-PHOTO files (cannot mix PHOTO and NON-PHOTO files)
              app.models.resumeText.destroyAll({and: [{vetId: userId}, {resumeType:resumeTypeFile}]}, function(err, deleteResponse) {
                  //console.log('Resume.uploadComplete - resumeText.destroyAll - err: ', err);
                  //console.log('Resume.uploadComplete - resumeText.destroyAll - deleteResponse: ', deleteResponse);
                
                if (err) {
                  errorMessage = "Error Deleting Old User Resume Files";
                  responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                  cb(null,response);
                  return;
                }
                  
                var userFileName = resumeTypePhoto+"-"+body.page;

                //2) If IMAGE, it will be base-64
                //2a) Convert to an image
                base64.decode(body.data, resumeStoragePath+'/'+userFileName, function(err, output) {
                  //console.log('Resume.uploadResume - base64.decode - success!!!');
                  //console.log('Resume.uploadResume - base64.decode - output: ', output);
                  //console.log('Resume.uploadResume - base64.decode - err: ', err);

                  //FOR TESTING ONLY TO SEE IF BASE-64 MATCHES
                  /*base64.encode(output, function(err, base64String) {
                    //console.log('Resume.uploadResume - base64.ENCODE - base64String: ', base64String);
                    console.log('Resume.uploadResume - base64.ENCODE - err: ', err);
                  });*/

                  // START #2
                  //2b-1) Run the image through ImageMagick


  //ImageMagick

                  // Adjust Image with ImageMagick auto-orient an image
                  gm(output)
                  .modulate(120)
                  .contrast(+3)
                  .sharpen(20)
                  .autoOrient()
                  .write(output, function (err) {
                    if (err) {
                      //console.log("************ Error invoking imagemagick.autoOrient for userId " + userId +" err: ", err);
                       
                       errorMessage = "Error invoking imagemagick.";
                       if (err.message){
                         errorMessage = "Error invoking imagemagick. Error [" + err.message + "]";
                       }
                       //console.log("Error invoking recognize for userId " + userId +" err: ", err);
                       responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                       cb(null,response);
                       return;
                    } else {
                      //console.log("************ imagemagick for userId SUCCESS!!!! " + userId);

                      //2b) Run the image through Tessaract to get the text
                      recognize(output, (err, text) => {

                          // DELETE IMAGE
                          fs.unlink(resumeStoragePath+'/'+userFileName, (err) => {
                            if (err === null) {
                              //console.log('ResumeUpload - successfully deleted file: ', resumeStoragePath+'/'+userFileName);
                            }
                            else {
                            //  console.log('ResumeUpload - error deleting file: ', resumeStoragePath+'/'+userFileName);
                            }
                          });


                          if (err) {
                             errorMessage = "Error invoking recognize. Error [" + err.message + "]";
                             //console.log("Error invoking recognize for userId " + userId +" err: ", err);
                             responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                             cb(null,response);
                          }
                          else {
                            //console.log('Resume.uploadResume - recognize - text: ', text);

                            //2c) Temporarily save the text to the database
                            app.models.resumeText.find({where: {and: [{vetId: userId}, {page: body.page}, {resumeType: resumeTypePhoto}]}}, function(err, resumeTextResponse) {
                              if (err) {
                                 errorMessage = "Error invoking resumeText.find. Error [" + err.message + "]";
                                 //console.log("Error invoking resumeText.find for userId " + userId +" err: ", err);
                                 responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                 cb(null,response);
                              }
                              else {
                                //console.log('Resume.uploadResume - resumeText.find - resumeTextResponse: ', resumeTextResponse);
                                var resumeTextData = { 
                                      vetId: userId, 
                                      page: body.page, 
                                      text: text, 
                                      resumeType: resumeTypePhoto,
                                      resumeFileName: resumeTypePhoto+"-"+body.page
                                };
                                if (resumeTextResponse === null || resumeTextResponse.length === 0) {
                                  //CREATE NEW
                                  app.models.resumeText.create(resumeTextData, function(err, resumeTextResponse) {
                                    if (err) {
                                       errorMessage = "Error invoking resumeText.create. Error [" + err.message + "]";
                                       //console.log("Error invoking resumeText.create for userId " + userId +" err: ", err);
                                       responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                       cb(null,response);
                                    }
                                    else {
                                      //console.log('Resume.uploadResume - resumeText.create - resumeTextResponse: ', resumeTextResponse);
                                      response.payload.text = text;
                                      responseBuilder.setSuccessStatus(response);
                                      cb(null,response);
                                    }
                                  });

                                } else {
                                  //UPDATE
                                  resumeTextResponse[0].vetId = userId;
                                  resumeTextResponse[0].page = body.page;
                                  resumeTextResponse[0].text = text;
                                  resumeTextResponse[0].resumeFileName = resumeTypePhoto+"-"+body.page;

                                  resumeTextResponse[0].save(function(err, resumeTextResponse) {
                                    if (err) {
                                     errorMessage = "Error invoking resumeTextResponse[0].save. Error [" + err.message + "]";
                                     //console.log("Error invoking resumeTextResponse[0].save for userId " + userId +" err: ", err);
                                     responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                     cb(null,response);
                                    }
                                    else {
                                      //console.log('Resume.uploadResume - resumeTextResponse[0].save - resumeTextResponse[0]: ', resumeTextResponse[0]);
                                      response.payload.text = text;
                                      responseBuilder.setSuccessStatus(response);
                                      cb(null,response);
                                    }
                                  });
                                }
                              }
                            });
                            
                            
                            
                            
                          }
                       });

  //ImageMagick

                     }
                   })



                });
                
              });//END - Delete and existing old PHOTO files
            
            }// END - IS PHOTO 
          } // else userId is defined
        });  // VaGrit.getUserFromToken
      }

    // THIS WILL GET THE RESUME FILE/PHOTO INFO, SAVE IT TO USER RESUME INFO, SEND THE RESUME INFO AND THE RESUME TO FILE TO WEM
    // FOR PHOTOS CONVERTED TO TEXT RESUMES - IT WILL COMBINE ALL THE TEXT INTO A .TXT FILE, SEND IT TO WEM, AND THEN DELETE THE DATA AND FILE
    // FOR NON-PHOTO FILE RESUMES - IT WILL CONVERT THE SAVED BASE-64 TO A FILE, SEND IT TO WEM, AND THEN DELETE THE DATA AND FILE
    Resume.uploadComplete = function(header, body, cb) {
       //console.log("****Start of Resume.uploadComplete  header: ", JSON.stringify(header));
       //console.log("****Start of Resume.uploadComplete.  body: ", JSON.stringify(body));
       var response = responseBuilder.createResponseObject();
       var errorMessage;

       VaGrit.getUserFromToken(app, header, function (err, userId) {
          if (userId === undefined || userId === null){
             responseBuilder.setInvalidTokenStatus(response, header);
             cb(null,response);
          }
          else { // userId is defined
            //  console.log(' Resume.uploadComplete START Resume.find userId: ', userId);
              responseBuilder.setSuccessStatus(response);
              
              // 1) resumeText.find where vetId 
              // 2) - DONE - If photo, process photo 
              // 3) If file, process file.
                // 3a) Convert file from base-64 to correct file type (need to add filename to resumeText)
                // 3b) Send to WEM (Resume.wemCreateUserProfile)
                // 3c) Delete data from resumeText and the file 
              
              
              
                
              // 1. Combine images to a single record in the database, return final text to app
              
              //COMBINE TEXT FROM ALL IMAGES FROM OCR
              app.models.resumeText.find({order: 'page ASC', where: {and: [{vetId: userId}, {resumeType: resumeTypePhoto}]}}, function(err, resumeTextResponseForCombining) {
                if (err) {
                   errorMessage = "Error invoking Resume.uploadComplete - resumeText.find. Error [" + err.message + "]";
                   //console.log("Error invoking Resume.uploadComplete - resumeText.find for userId " + userId +" err: ", err);
                   responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                   cb(null,response);
                }
                else {
                  //console.log('Resume.uploadComplete - resumeText.find - resumeTextResponseForCombining: ', resumeTextResponseForCombining);
                  //console.log('Resume.uploadComplete - resumeText.find - resumeTextResponseForCombining.resumeType: ', resumeTextResponseForCombining.resumeType);

                  if (resumeTextResponseForCombining === undefined || resumeTextResponseForCombining === null || resumeTextResponseForCombining.length < 1) {
                       errorMessage = "No resume on file for this user";
                       //console.log("Error invoking Resume.uploadComplete - resumeText.find for userId " + userId +" err: ", err);
                       responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                       cb(null,response);
                       return;
                  }

                    console.log("Resume.uploadComplete - DO IMAGE ");

                    var combinedText = "";
                    // 2f) Combine the saved pages to a text file
                    for (var i = 0; i < resumeTextResponseForCombining.length; i++) {
                        combinedText += resumeTextResponseForCombining[i].text;
                    }

                    /// DELETE OLD text FILES FROM DB
                    app.models.resumeText.destroyAll({and: [{vetId: userId}, {resumeType:resumeTypePhoto}]}, function(err, deleteResponse) {
                        //console.log('Resume.uploadComplete - resumeText.destroyAll - err: ', err);
                        //console.log('Resume.uploadComplete - resumeText.destroyAll - deleteResponse: ', deleteResponse);
                        if (err) {
                           errorMessage = "Error deleting old resume files. Error [" + err.message + "]";
                           responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                           cb(null,response);
                           return;
                        }
                        
                        
                        /// Save combinedText to db
                        var resumeTextData = { 
                          vetId: userId, 
                          page: resumeTypeFilePageNumber, 
                          text: combinedText, 
                          resumeType: resumeTypePhoto,
                          resumeFileName: resumeTypePhoto
                        };
                        app.models.resumeText.create(resumeTextData, function(err, resumeTextSaveFileResponse) {
                          if (err) {
                             errorMessage = "Error invoking resumeTextSaveFile.create. Error [" + err.message + "]";
                             //console.log("Error invoking resumeTextSaveFile.create for userId " + userId +" err: ", err);
                             responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                             cb(null,response);
                             return;
                          }
                          //console.log('Resume.uploadResume - resumeTextSaveFile.create - resumeTextSaveFileResponse: ', resumeTextSaveFileResponse);
                          response.payload.resumeFilename = resumeTypePhoto;
                          response.payload.uploadTimestamp = new Date().toISOString();
                          response.payload.readText = combinedText;
                          responseBuilder.setSuccessStatus(response);
                          cb(null,response);
                          return;
                        
                        });// resumeText.create - END
                        
                    });/// DELETE OLD text FILES FROM DB - END
                  
                }// FOUND RESUME OK - ELSE - END
              });
          } // else userId is defined

      });  // VaGrit.getUserFromToken

    }




    // THIS FUNCTION WILL GET THE RESUME DATA IN THE DATABASE AND CONVERT IT TO IT'S CORRECT FILE FORMAT FOR SENDING TO WEM
    Resume.getResumeFileToSendToWem = function(userId, cb) {
      console.log("getResumeFileToSendToWem. START");
      console.log("getResumeFileToSendToWem. userId: ", userId);
      
      var response = {
          resumeId: "",
          resumeType: "",
          filePath: null,
          status: "SUCCESS"
      };
      
      var errorMessage;
      
      app.models.resumeText.find({where: {vetId: userId}}, function(err, findResumeFilesResults) {
           console.log('Resume.getResumeFileToSendToWem - resumeText.find - err: ', err);
           //console.log('Resume.getResumeFileToSendToWem - resumeText.find - findResumeFilesResults: ', findResumeFilesResults);
           if (err !== null || findResumeFilesResults === null) {
              errorMessage = "Error resumeText.find";
              if (err !== null && err.message) {
                errorMessage = "Error resumeText.find [" + err.message + "]";
              }
              console.log("Error resumeText.find for id " + userId +" err: ", errorMessage);
              cb(errorMessage, null);
              return;
           }
          
           if (findResumeFilesResults.length < 1) {
               cb(null, response);
               return;
           }
            
           response.resumeId = findResumeFilesResults[0].id;
           response.resumeType = findResumeFilesResults[0].resumeType;
          
           if (findResumeFilesResults[0].resumeType === resumeTypeFile) {
             response.filePath = findResumeFilesResults[0].resumeFileName;//resumeStoragePath+'/'+findResumeFilesResults[0].resumeFileName;
             base64.decode(findResumeFilesResults[0].text, resumeStoragePath+'/'+findResumeFilesResults[0].resumeFileName, function(err, output) {
               //console.log('Resume.uploadResume - base64.decode - success!!!');
               //console.log('Resume.uploadResume - base64.decode - output: ', output);
               //console.log('Resume.uploadResume - base64.decode - err: ', err);

               if (err) {
                  errorMessage = "Error invoking getResumeFileToSendToWem base64.decode. Error [" + err.message + "]";
                  //console.log("Error invoking base64.decode for userId " + userId +" err: ", err);
                  cb(errorMessage, null);
                  return;
               }
               
               response.status = "SUCCESS";
               cb(null,response);
               return;
               
             });
           }// END - PROCESS NON-PHOTO FILE
           
           if (findResumeFilesResults[0].resumeType === resumeTypePhoto) {
             var userFileName = userId + resumePhotoTempFileName;
             response.filePath = userFileName;//resumeStoragePath+'/'+userFileName;
             writeFile(resumeStoragePath+'/'+userFileName, findResumeFilesResults[0].text, function(err) {
             if (err) {
                errorMessage = "Error invoking getResumeFileToSendToWem writeFile. Error [" + err.message + "]";
                //console.log("Error invoking base64.decode for userId " + userId +" err: ", err);
                cb(errorMessage, null);
                return;
              }
              else {
                response.status = "SUCCESS";
                cb(null,response);
                return;
              }
            });///writeFile - END 
           } // END - PROCESS PHOTO FILE
      });// END - app.models.resumeText.find
    };// Resume.getResumeFileToSendToWem - END 


    // THIS FUNCTION WILL UPLOAD RESUME INFO TO WEM, A RESUME FILE WILL BE INCLUDED IF IT EXISTS
    Resume.uploadResumeInfo = function(header, body, cb) {
         //console.log("**** Resume.uploadResumeInfo  header: ", JSON.stringify(header));
         console.log("**** Resume.uploadResumeInfo.  body: ", JSON.stringify(body));
         var response = responseBuilder.createResponseObject();
         var errorMessage;
         
         VaGrit.getUserFromToken(app, header, function (err, userId) {
            if (userId === undefined || userId === null){
              responseBuilder.setInvalidTokenStatus(response, header);
              cb(null,response);
            }
            else { // userId is defined
              //console.log('**** Resume.uploadResumeInfo. VaGrit.getUserFromToken userId: ', userId);
             
             //Required fields check 
             if (body === undefined || body === null 
                   || body.desiredLocation === undefined || body.desiredLocation === null || body.desiredLocation === "" || typeof body.desiredLocation !== "string"
                   || body.militaryBranch === undefined || body.militaryBranch === null
                   || body.rank === undefined || body.rank === null
                   || body.targetIndustry === undefined || body.targetIndustry === null || body.targetIndustry === "" || typeof body.targetIndustry !== "string"
                   || body.minimumSalary === undefined || body.minimumSalary === null
                   || body.jobTypes === undefined || body.jobTypes === null) {
               errorMessage = "Missing Required fields or values";
               responseBuilder.setInvalidRequestStatus(response, errorMessage);
               cb(null,response);
               return;
             }
             
             

        app.models.resume.find({where: {vetId: userId}}, function(err, resumeFileCheckResponse) {
          
          if (err !== null) {
              errorMessage = "Error resumeText.find [" + err.message + "]";
              responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
              cb(null,response);
              return;
          }
           
            
      app.models.resumeText.find({where: {vetId: userId}}, function(err, resumeFileTextCheckResponse) {
        
        if (err !== null) {
            errorMessage = "Error resumeText.find [" + err.message + "]";
            responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
            cb(null,response);
            return;
        }
           
           
             //Required fields check - If no resume - mos is required
           if (body.mos === undefined || body.mos === null || body.mos === "" || typeof body.mos !== "string") {           
             if (
                  (
                    (resumeFileCheckResponse === null || resumeFileCheckResponse.length === 0) 
                    || (resumeFileCheckResponse[0].resumeFilename === undefined || resumeFileCheckResponse[0].resumeFilename === null || resumeFileCheckResponse[0].resumeFilename === "")
                  )
                    && 
                  (
                    (resumeFileTextCheckResponse === null || resumeFileTextCheckResponse.length === 0) 
                    || (resumeFileTextCheckResponse[0].text === undefined || resumeFileTextCheckResponse[0].text === null || resumeFileTextCheckResponse[0].text === "")
                  )
                ) {
                
                errorMessage = "Missing Required fields or values - No Mos - Create";
                responseBuilder.setInvalidRequestStatus(response, errorMessage);
                cb(null,response);
                return;
                
             }
           }

             
             MongoClient.connect(mongoUrl, function(err, db) {
               if (err) throw err;
               var dbo = db.db(mongoDbName);
               dbo.collection("vet").findOne({_id:userId}, function(err, vetInstance) {
                 if (err) {
                   //console.log('~~~~~~~~~~~~~~~~~~~~~~~~~~~ Resume.wemCreateUserProfile mongodbConnection: err: ', err);
                   cb(err, null);
                 } else {
                   console.log('~~~~~~~~~~~~~~~~~~~~~~~~~~~ Resume.wemCreateUserProfile mongodbConnection: userId: ', userId);



               // TODO The request body will have the minimum form fields
               // Invoke WEM with the data.
               // For now just return a success status


                /*

                1) DONE - Get the resumeFile to send 
                2) DONE - Create the profile on Wem
                3) DONE - Create/Update the profile on the midtier
                  3a) DONE - Give badge on create profile on midtier
                4) DONE - Delete the files created that were sent to WEM - even if not successfull
                4a) DONE - If wemCreateUserProfile successfull, Delete the files from the database

                */



              // 1) Get the resumeFile to send  AND 1a) Prepare file to send
              Resume.getResumeFileToSendToWem(userId, function(err, getResumeFileToSendToWemResults) {
                  console.log('Resume.uploadResumeInfo - resumeText.find - err: ', err);
                  console.log('Resume.uploadResumeInfo - resumeText.find - getResumeFileToSendToWemResults: ', getResumeFileToSendToWemResults);
                  if (err !== null) {
                      errorMessage = "Error resumeText.find [" + err.message + "]";
                      responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                      console.log("Error resumeText.find for id " + userId +" err: ", err);
                      console.log("resumeText.find. setUnexpectedExceptionStatus: ", errorMessage);
                      cb(null,response);
                      return;
                  }
                  
                  
                  //2) Create the profile on Wem
                  
                  var wemFormData = {
                    veteranID: userId.toString(),
                    editprofile: "false",
                    targetedIndustries: body.targetIndustry,
                    location: body.desiredLocation,
                    moc: (body.mos !== undefined && body.mos !== null) ? body.mos : "",
                    candidateName: (vetInstance.name !== undefined && vetInstance.name !== null) ? vetInstance.name : "",
                    minSalary: body.minimumSalary,
                    jobType: body.jobTypes,
                    militaryBranch: body.militaryBranch,
                    militaryRank: body.rank,
                    jobRole: "",
                    qualifications: "",
                    availableIn: "",
                    experience: ""
                 };
                 if (getResumeFileToSendToWemResults.filePath !== null) {
                      wemFormData.resumeFile = getResumeFileToSendToWemResults.filePath;
                 } 
                 
                  //console.log('Resume.uploadResumeInfo - wemFormData: ', wemFormData);
                   

                  Resume.wemCreateUserProfile(userId, wemFormData, function(err, wemCreateUserProfileResponse) {

                      console.log('Resume.uploadResumeInfo ......... - wemCreateUserProfile err: ', err);
                    
                      if (err) {
                        
                        //4) Delete the files created that were sent to WEM - even if not successfull
                        Resume.deleteFileFromSystem(getResumeFileToSendToWemResults.filePath);
                        
                         errorMessage = err.toString();
                        // console.log("Error creating Wem User Profile for userId " + userId +" err: ", err);
                         responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                         cb(null,response);
                         return;
                      }
                      else {
                        
                        if (Array.isArray(wemCreateUserProfileResponse)) {
                          responseBuilder.setSuccessStatus(response);
                          response.payload.validationErrors = wemCreateUserProfileResponse;
                          cb(null,response);
                          return;
                        }
                        
                        
                        //3) Create/Update the profile on the midtier
                        
                        var userResumeData = {
                            "resumeFile": getResumeFileToSendToWemResults.filePath,
                            "vetId" : userId,
                            "desiredLocation": body.desiredLocation,
                            "militaryBranch": body.militaryBranch,
                            "mos": body.mos,
                            "rank": body.rank,
                            "targetIndustry": body.targetIndustry,
                            "minimumSalary": body.minimumSalary,
                            "jobTypes": body.jobTypes
                        };
                        
                        //3) Create/Update the profile on the midtier
                        Resume.updateResumeInfo(userId, userResumeData, function(err, updateResumeInfoResponse) {
                          
                            //4) Delete the files created that were sent to WEM - even if not successfull
                            Resume.deleteFileFromSystem(getResumeFileToSendToWemResults.filePath);
                            
                            if (err) {
                               errorMessage = "Error creating Wem User Profile [" + err + "]";
                              // console.log("Error creating Wem User Profile for userId " + userId +" err: ", err);
                               responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                               cb(null,response);
                               return;
                            }
                            else {
                              
                              responseBuilder.setSuccessStatus(response);
                              
                              //Add to auditEvents - Do not wait
                              app.models.auditEvent.addEvent({
                                vetId: userId.toString(),
                                eventType: app.models.auditEvent.eventTypes.UPLOAD_RESUME_INFO,
                                detail: {}
                              });
                              
                              //  3a) Give badge on create profile on midtier (done in wemCreateUserProfile)
                              if (updateResumeInfoResponse.badgeAward && updateResumeInfoResponse.badgeAward !== null) {
                                if (Array.isArray(updateResumeInfoResponse.badgeAward)) {
                                  response.badgeAward = updateResumeInfoResponse.badgeAward;
                                }
                                else {
                                    response.badgeAward.push(updateResumeInfoResponse.badgeAward);
                                }
                              }
                              
                      
                              cb(null,response);
                              return;
                              
                            }
                        });// Resume.updateResumeInfo -- END 
                        
                        //4a) If wemCreateUserProfile successfull, Delete the files from the database 
                        //Delete temporary resumeText - Don't wait for it
                        app.models.resumeText.destroyAll({vetId: userId}, function(err, deleteResponse) {
                            //console.log('Resume.uploadComplete - resumeText.destroyAll - err: ', err);
                            //console.log('Resume.uploadComplete - resumeText.destroyAll - deleteResponse: ', deleteResponse);
                        });
                        
                      }// Else - wemCreateUserProfile no error
                  
                });// Resume.wemCreateUserProfile - END
                  
               });// Resume.getResumeFileToSendToWem - END

              }// dbo.collection("vet") No Err - END
             });// dbo.collection("vet") END
          });// MongoClient.connect - END


              
              });
              //// END - RESUME FILE TEXT CHECK HERE
        });
        //// END - RESUME FILE CHECK HERE


        } // else userId is defined

      });  // VaGrit.getUserFromToken

    }
    
    
    
    Resume.deleteFileFromSystem = function(fileToDelete) {
      if (fileToDelete !== null && fileToDelete !== "") {
        fs.unlink(resumeStoragePath+'/'+fileToDelete, (err) => {
          if (err === null) {
            //console.log('Resume.deleteFileFromSystem - successfully deleted file: ', resumeStoragePath+'/'+fileToDelete);
          }
          else {
            //console.log('Resume.deleteFileFromSystem - error deleting file: ', resumeStoragePath+'/'+fileToDelete);
          }
        });
      }
    }
    

    // THIS GETS THE RESUME INFO - ALSO CALLS WEM TO CHECK TO SEE IF THE USER EXISTS ON WEM
    Resume.getResumeInfo = function(header, body, cb) {
       //console.log("****Start of Resume.getResumeInfo  header: ", JSON.stringify(header));
       //console.log("****Start of Resume.getResumeInfo.  body: ", JSON.stringify(body));
       var response = responseBuilder.createResponseObject();
       var errorMessage;

       VaGrit.getUserFromToken(app, header, function (err, userId) {
          if (userId === undefined || userId === null){
             responseBuilder.setInvalidTokenStatus(response, header);
             cb(null,response);
          }
          else { // userId is defined
              console.log(' VaGrit.getUserFromToken START Resume.find userId: ', userId);
              Resume.find({where: {vetId: userId}}, function(err, resumeInstance) {

                 if (err) {
                   console.log(' Resume.find START err : ', err);
                   var errorText = err;
                   if (err.message && err.message !== "") {
                      errorText = err.message;
                   }
                    errorMessage = "Error invoking Resume.find. Error [" + errorText + "]";
                    console.log("Error invoking Resume.find for userId " + userId +" err: ", err);
                    responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                    cb(null, response);
                    return;
                 }
                 else {
                    response.payload = {
                       "desiredLocation": "",
                       "militaryBranch": "",
                       "mos": "",
                       "rank": "",
                       "targetIndustry": "",
                       "minimumSalary": "",
                       "jobTypes": [],
                       "resumeFilename": "",
                       "uploadTimestamp": "",
                       "wemProfileExists": false
                   };

                    if (resumeInstance !== undefined && resumeInstance.length > 0) {
                       //console.log("Success Resume.find. userId " + userId + " exist - resumeInstance: ", resumeInstance);
                       //console.log("Success Resume.find resumeInstance[0] " + resumeInstance[0]);
                       response.payload = {
                           "desiredLocation": (resumeInstance[0].desiredLocation !== undefined && resumeInstance[0].desiredLocation !== null) ? resumeInstance[0].desiredLocation : "",
                           "militaryBranch": (resumeInstance[0].militaryBranch !== undefined && resumeInstance[0].militaryBranch !== null) ? resumeInstance[0].militaryBranch : "",
                           "mos": (resumeInstance[0].mos !== undefined && resumeInstance[0].mos !== null) ? resumeInstance[0].mos : "",
                           "rank": (resumeInstance[0].rank !== undefined && resumeInstance[0].rank !== null) ? resumeInstance[0].rank : "",
                           "targetIndustry": (resumeInstance[0].targetIndustry !== undefined && resumeInstance[0].targetIndustry !== null) ? resumeInstance[0].targetIndustry : "",
                           "minimumSalary": (resumeInstance[0].minimumSalary !== undefined && resumeInstance[0].minimumSalary !== null) ? resumeInstance[0].minimumSalary : "",
                           "jobTypes": (resumeInstance[0].jobTypes !== undefined && resumeInstance[0].jobTypes !== null) ? resumeInstance[0].jobTypes : [],
                           "resumeFilename": (resumeInstance[0].resumeFilename !== undefined && resumeInstance[0].resumeFilename !== null) ? resumeInstance[0].resumeFilename : "",
                           "uploadTimestamp": (resumeInstance[0].uploadTimestamp !== undefined && resumeInstance[0].uploadTimestamp !== null) ? resumeInstance[0].uploadTimestamp : "",
                           "wemProfileExists": false
                       };
                       
                      var wemGetUserProfileUrl = wemServerBaseUrl + "WEMUsers/getUserProfile/"+userId;
                        //console.log('>>>>>>>>>>>>>> Resume.wemCreateUserProfile getUserProfile.submit wemGetUserProfileUrl: ', wemGetUserProfileUrl);
                       request.get(wemGetUserProfileUrl, function optionalCallback(err, httpResponse, wemGetUserProfileResponse) {
                         console.log('>>>>>>>>>>>>>> Resume.wemCreateUserProfile getUserProfile.submit err: ', err);
                         //console.log('>>>>>>>>>>>>>> Resume.wemCreateUserProfile getUserProfile.submit wemGetUserProfileResponse: ', wemGetUserProfileResponse);
                         if (err !== null) {
                           var errorText = err;
                           if (err.message && err.message !== "") {
                              errorText = err.message;
                           }
                           errorMessage = "Resume.getResumeInfo Error checking if WEM Profile Exists [" + errorText + "]";
                           responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                         }
                         else {
                           if (JSON.parse(wemGetUserProfileResponse).userProfile !== undefined && JSON.parse(wemGetUserProfileResponse).userProfile !== null) {
                              response.payload.wemProfileExists = true;
                           }
                           responseBuilder.setSuccessStatus(response);
                         }
                         cb(null, response);
                       });
                    } // if resumeInstance == undefined - END
                     else {
                       responseBuilder.setSuccessStatus(response);
                       cb(null, response);
                     }
                    
                  }  //else
              });   // end of Resume.find
          } // else userId is defined

      });  // VaGrit.getUserFromToken

    }



    Resume.getTargetIndustries = function(header, body, cb) {
       //console.log("****Start of Resume.getTargetIndustries  header: ", JSON.stringify(header));
       //console.log("****Start of Resume.getTargetIndustries.  body: ", JSON.stringify(body));
       var response = responseBuilder.createResponseObject();
       var errorMessage;

       VaGrit.getUserFromToken(app, header, function (err, userId) {
          if (userId === undefined || userId === null){
             responseBuilder.setInvalidTokenStatus(response, header);
             cb(null,response);
          }
          else { // userId is defined
              //console.log(' VaGrit.getUserFromToken START Resume.find userId: ', userId);
              responseBuilder.setSuccessStatus(response);
              response.payload = ["IT", "Logistics", "Engineering", "Management", "Construction"];
              cb(null, response);
          } // else userId is defined

      });  // VaGrit.getUserFromToken

    }

    // THIS FUNCTION JUST CREATES OR UPDATES THE USERS RESUME INFO IN MODELS.RESUME - IT DOES NOT UPLOAD THE FILE NOR DOES IT SAVE TO WEM
    Resume.updateResumeInfo = function(userId, userResumeUploadData, cb) {
       console.log("****Start of updateResumeInfo  userResumeUploadData: ", JSON.stringify(userResumeUploadData));
       //console.log("****Start of updateResumeInfo  resumeFilename: ", resumeFilename);
       console.log("****Start of updateResumeInfo  userId: ", userId);
       var response = responseBuilder.createResponseObject();
       var errorMessage;
       
       var resumeFilename = userResumeUploadData.resumeFile;
       //console.log("****Start of updateResumeInfo  resumeFilename: ", resumeFilename);

        // 1) Save the data
        app.models.resume.find({where: {vetId: userId}}, function(err, resumeInstance) {

           //console.log('*********************** Resume.updateResumeInfo userResumeData: ', userResumeData);
           //console.log('Resume.updateResumeInfo - Resume.find resumeInstance: ', resumeInstance);
           //console.log(' Resume.find err: ', err);           
           if (err === null) {

              if (resumeInstance === null || resumeInstance.length === 0) {
                
                 var userResumeData =  {
                     "vetId" : userId,
                     "desiredLocation": userResumeUploadData.desiredLocation,
                     "militaryBranch": userResumeUploadData.militaryBranch,
                     "mos": userResumeUploadData.mos,
                     "rank": userResumeUploadData.rank,
                     "targetIndustry": userResumeUploadData.targetIndustry,
                     "minimumSalary": userResumeUploadData.minimumSalary,
                     "jobTypes": userResumeUploadData.jobTypes
                 };
                 
                 if (resumeFilename !== null) {
                   var resumeFilenameForSavingResumeInfo = resumeFilename.replace(userId, "");
                   if (resumeFilenameForSavingResumeInfo === resumePhotoTempFileName) {
                     resumeFilenameForSavingResumeInfo = resumeTypePhoto; 
                   }
                   userResumeData.resumeFilename = resumeFilenameForSavingResumeInfo;
                   userResumeData.uploadTimestamp = new Date().toISOString();
                 }

                
                //console.log(' Resume.find ADD RESUME userResumeData: ', userResumeData);
                Resume.create(userResumeData, function(err, createResponse) {
                  // console.log("Resume.create. createResponse: ", createResponse);
                   console.log("Resume.create. err: ", err);

                   if (err === null) {
                     
                     createResponse.badgeAward = [];
                     // Add the badge if the user has not been awarded that badge
                     Resume.handleBadge(userId,badgeConstants.BadgeTypes.EMPLOYMENT_PROFILE_FILLED,function(err,badge){
                       if (err){
                         cb(err, null);
                         return;
                       } 
                       
                       responseBuilder.setSuccessStatus(response);
                       if (badge !== null){
                          createResponse.badgeAward.push(badge);
                       }
                     
                      if (resumeFilename !== null) {
                        // Add the badge if the user has not been awarded that badge
                        Resume.handleBadge(userId,badgeConstants.BadgeTypes.RESUME_UPLOADED,function(err,badge){
                          if (err){
                            cb(err, null);
                            return;
                          } 
                          if (badge !== null){
                             createResponse.badgeAward.push(badge);
                          }
                          cb(null, createResponse);
                          return;
                        });
                      }
                      else {
                        cb(null, createResponse);
                        return;
                      }
                  
                  });// END BADGES
                      
                    
                   } else {
                     console.log("Resume.create setUnexpectedExceptionStatus: ", errorMessage);
                     cb(err, null);
                   }
                });
              } else {
                
                  //resumeInstance[0].vetId = userId;
                  resumeInstance[0].desiredLocation = userResumeUploadData.desiredLocation;
                  resumeInstance[0].militaryBranch = userResumeUploadData.militaryBranch;
                  resumeInstance[0].mos = userResumeUploadData.mos;
                  resumeInstance[0].rank = userResumeUploadData.rank;
                  resumeInstance[0].targetIndustry = userResumeUploadData.targetIndustry;
                  resumeInstance[0].minimumSalary = userResumeUploadData.minimumSalary;
                  resumeInstance[0].jobTypes = userResumeUploadData.jobTypes;
                  
                  if (resumeFilename !== null) {
                    var resumeFilenameForSavingResumeInfo = resumeFilename.replace(userId, "");
                    if (resumeFilenameForSavingResumeInfo === resumePhotoTempFileName) {
                      resumeFilenameForSavingResumeInfo = resumeTypePhoto; 
                    }
                    resumeInstance[0].resumeFilename = resumeFilenameForSavingResumeInfo;
                    resumeInstance[0].uploadTimestamp = new Date().toISOString();
                  }

                  resumeInstance[0].save(function (err, savedResume){
                   if (err === null) {
                     // 5a) If update success - send back success

                     // TODO: WEM
                     // 2) Send to WEM
                     // 3) Process WEM Response
                     
                     if (resumeFilename !== null) {
                       // Add the badge if the user has not been awarded that badge
                       Resume.handleBadge(userId,badgeConstants.BadgeTypes.RESUME_UPLOADED,function(err,badge){
                         if (err){
                           cb(err, null);
                           return;
                         } 
                         if (badge !== null){
                            savedResume.badgeAward = badge;
                         }
                         cb(null,savedResume);
                         return;
                       });
                     } else {
                       
                       cb(null,savedResume);
                       return;
                     }

                   } else {
                      // 5b) If update fail - send error
                      console.log("Error userResumeData for id " + userId +" err: ", err);
                      cb(err, null);
                   }
                });
              }

            } else {
              console.log("Error finding vet resume info for id " + userId +" err: ", err);
              cb(err, null);
            }
       });


       ///cb(null, "updateResumeInfo OK!!!");

    };
    
    
    
    
    //THIS CREATES OR UPDATES THE USER PROFILE ON WEM BASED ON THE DATA IN THE DATABASE BASED ON THE userId
    //A FILE CAN BE PASSED TO THIS FUNCTION TO BE INCLUDED IN SENDING AND CREATING THE WEM PROFILE
    Resume.wemCreateUserProfile = function(userId, formData, cb) {

      console.log("wemCreateUserProfile. START");
      console.log("wemCreateUserProfile. userId: ", userId);
      //console.log("wemCreateUserProfile. fileToSend: ", fileToSend);
      var errorMessage = "";
      // Get info to send to WEM
      // Get getResumeInfo
      // Get VET info for candidateName
      
      
      var wemCreateUserProfileUrl = wemServerBaseUrl + "WEMUsers/createUserProfile/";
      
      if (formData.resumeFile !== undefined && formData.resumeFile !== null && formData.resumeFile !== "") {
        
        try {
          formData.resumeFile = fs.createReadStream(resumeStoragePath+'/'+formData.resumeFile);
        
          formData.resumeFile.on('open', function () {
            // This just pipes the read stream to the response object (which goes to the client)
            console.log('########### Resume.uploadForm Resume.find fileToSend: READ STREAM IS VALID');
            request.post({url:wemCreateUserProfileUrl, formData: formData}, function optionalCallback(err, httpResponse, body) {
              console.log('>>>>>>>>>>>>>> Resume.wemCreateUserProfile wemServerClient.submit err: ', err);
              
                if (err !== null || body === undefined || body === null 
                  || JSON.parse(body).status === undefined || JSON.parse(body).status === null 
                  && (JSON.parse(body).status !== "success" && JSON.parse(body).status !== "ValidationError")
                  //|| Array.isArray(JSON.parse(body).status)
                ) {
                  if (err !== null) {
                    cb(err, null);
                    return;
                  }
                  if (body === undefined || body === null || JSON.parse(body).status === undefined 
                  || JSON.parse(body).status === null) {
                    cb("WEM No Response", null);
                    return;
                  }
                  if (JSON.parse(body).status !== "success" && JSON.parse(body).status !== "ValidationError") {
                    cb(JSON.parse(body).status, null);
                    return;
                  }
                    
                }
                else {
                  
                  var wemSuccessResponse = "SUCCESS";
                  if (JSON.parse(body).status === "ValidationError" && JSON.parse(body).error !== undefined) {
                    wemSuccessResponse = JSON.parse(body).error;
                  }
                  
                  
                  cb(null, wemSuccessResponse);
                  return;
                }
            
              
              
              
            });
          });
          // This catches any errors that happen while creating the readable stream (usually invalid names)
          formData.resumeFile.on('error', function(err) {
            errorMessage = "Error Reading Resume Stream - err: ", err;
            console.log("Resume.wemCreateUserProfile. setUnexpectedExceptionStatus: ", errorMessage);
            cb(errorMessage, null);
          });

        }
        
        catch(e) {
          cb("UNEXPECTED WEM ERROR", null);
          return;
        }
      } else {
        
        try {
          //formData.resumeFile = "NO_FILE";
          request.post({url:wemCreateUserProfileUrl, formData: formData}, function optionalCallback(err, httpResponse, body) {
            console.log('Resume.wemCreateUserProfile wemServerClient.submit err: ', err);
            
              if (err !== null || body === undefined || body === null 
                || JSON.parse(body).status === undefined || JSON.parse(body).status === null 
                && (JSON.parse(body).status !== "success" && JSON.parse(body).status !== "ValidationError")
                //|| Array.isArray(JSON.parse(body).status)
              ) {
                if (err !== null) {
                  cb(err, null);
                  return;
                }
                if (body === undefined || body === null || JSON.parse(body).status === undefined 
                || JSON.parse(body).status === null) {
                  cb("WEM No Response", null);
                  return;
                }
                if (JSON.parse(body).status !== "success" && JSON.parse(body).status !== "ValidationError") {
                  cb(JSON.parse(body).status, null);
                  return;
                }
                  
              }
              else {
                var wemSuccessResponse = "SUCCESS";
                if (JSON.parse(body).status === "ValidationError" && JSON.parse(body).error !== undefined) {
                  wemSuccessResponse = JSON.parse(body).error;
                }
                
                
                cb(null, wemSuccessResponse);
                return;
              }
          
            
            
          });
        }
        
        catch(e) {
          cb("UNEXPECTED WEM ERROR", null);
          return;
        }
      }
      
    }// Resume.wemCreateUserProfile - END    

    Resume.handleBadge = function(vetId,badgeType,cb){
      var badgeToAdd = badgeType
      app.models.userBadge.addBadge({vetId: vetId, badge: badgeToAdd },function(err, badge) {
        if (err){
          console.log("Error with userBadge.addBadge for badge [" + badgeToAdd + "] for vetId[" + vetId + "]", err);
          cb(err,null);
        }
        else {
          //console.log("Successful with userBadge.addBadge for badge [" + badgeToAdd + "] for vetId[" + vetId + "]", badge);
          cb(null,badge);
        }
      });   // userBadge.addBadge

    }



    Resume.remoteMethod('uploadResume', {
        accepts: [
          {arg: 'Authorization', type: 'string', http: { source: 'header' }},
          { arg: 'data', type: 'object', http: { source: 'body' } }],
        returns: {arg: 'response', type: 'string'},
        http: {path: '/uploadResume', verb: 'post'}
    });


    Resume.remoteMethod('uploadResumeInfo', {
        accepts: [
          {arg: 'Authorization', type: 'string', http: { source: 'header' }},
          { arg: 'data', type: 'object', http: { source: 'body' } }],
        returns: {arg: 'response', type: 'string'},
        http: {path: '/uploadResumeInfo', verb: 'post'}
    });


    Resume.remoteMethod('getResumeInfo', {
        accepts: [
          {arg: 'Authorization', type: 'string', http: { source: 'header' }},
          { arg: 'data', type: 'object', http: { source: 'body' } }],
        returns: {arg: 'response', type: 'string'},
        http: {path: '/getResumeInfo', verb: 'post'}
    });

    Resume.remoteMethod('getTargetIndustries', {
        accepts: [
          {arg: 'Authorization', type: 'string', http: { source: 'header' }},
          { arg: 'data', type: 'object', http: { source: 'body' } }],
        returns: {arg: 'response', type: 'string'},
        http: {path: '/getTargetIndustries', verb: 'post'}
    });

    Resume.remoteMethod('uploadComplete', {
        accepts: [
          {arg: 'Authorization', type: 'string', http: { source: 'header' }},
          { arg: 'data', type: 'object', http: { source: 'body' } }],
        returns: {arg: 'response', type: 'string'},
        http: {path: '/uploadComplete', verb: 'post'}
    });

};
