'use strict';

var app = require('../../server/server');
var constants = require('../constants').constants(app);
var wemServerBaseUrl = constants.cognitive.wemServerBaseUrl;
const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');
var badgeConstants = require('va-grit-microservices/constants/badgeConstants');
var request = require('request-json');

module.exports = function(Skills) {


  
  Skills.searchSkills = function(header, body, cb) {
     //console.log("****Start of Skills.searchSkills  header: ", JSON.stringify(header));
     
     var response = responseBuilder.createResponseObject();
     var errorMessage;
     
     if (body === undefined || body === null 
           || body.targetIndustry === undefined || body.targetIndustry === null || body.targetIndustry === ""
        || body.yearsOfExperience === undefined || body.yearsOfExperience === null || body.yearsOfExperience === "") {
          
           errorMessage = "Required Fields are missing";
           responseBuilder.setInvalidRequestStatus(response, errorMessage);
           response.payload = [];
           cb(null,response);
           return;      
     }
     
     var searchSkillsBody = body;
     //console.log("****Start of Skills.searchSkills  body: ", JSON.stringify(body));
     //console.log("****Start of Skills.searchSkills  searchSkillsBody: ", JSON.stringify(searchSkillsBody));

     VaGrit.getUserFromToken(app, header, function (err, userId) {
        if (userId === undefined || userId === null){
           responseBuilder.setInvalidTokenStatus(response, header);
           response.payload = [];
           cb(null,response);
        }
        else { // userId is defined
          //console.log('Skills.searchSkills VaGrit.getUserFromToken START Skills.find userId: ', userId);
          
          var skillsSearchUrl = "WEMUsers/performSkillGapAnalysis";
          skillsSearchUrl += "?veteranId="+userId;
          skillsSearchUrl += "&desiredIndustry="+searchSkillsBody.targetIndustry;
          skillsSearchUrl += "&yearsOfExp="+searchSkillsBody.yearsOfExperience;
          //skillsSearchUrl += "&minSalaryExpectation="+((resumeInfoResponse.payload.minimumSalary !== null && resumeInfoResponse.payload.minimumSalary !== "") ? resumeInfoResponse.payload.minimumSalary : 0);
          //skillsSearchUrl += "&preferredLocation="+((resumeInfoResponse.payload.desiredLocation !== null && resumeInfoResponse.payload.desiredLocation !== "") ? resumeInfoResponse.payload.desiredLocation : "");
          //skillsSearchUrl = encodeURIComponent(skillsSearchUrl);
          //console.log(' VaGrit.getUserFromToken START Skills.find skillsSearchUrl: ', skillsSearchUrl);
          
          var wemServerClient = request.createClient(wemServerBaseUrl);
          wemServerClient.post(skillsSearchUrl, {}, function(err, res, wemSearchSkillsResultBody) {
            
             if (wemSearchSkillsResultBody !== undefined && wemSearchSkillsResultBody !== null && wemSearchSkillsResultBody.error !== undefined && wemSearchSkillsResultBody.error !== null 
                 && wemSearchSkillsResultBody.error.message !== undefined && wemSearchSkillsResultBody.error.message === 'NO_WEM_PROFILE') {
               response.errorMessage = "This user does not have a profile in WEM.";
               responseBuilder.setNoWemProfileStatus(response, errorMessage);
               response.payload = [];
               cb(null,response);
               return;
             }
              
             if (err !== null || wemSearchSkillsResultBody === undefined || wemSearchSkillsResultBody.skillGapResult === undefined || wemSearchSkillsResultBody.skillGapResult === null) {
                response.errorMessage = "Call WEM - Unexpected WEM Error";
                if (wemSearchSkillsResultBody && wemSearchSkillsResultBody.error) {
                  response.errorMessage = "Call WEM - Error Received from WEM - Error [" + wemSearchSkillsResultBody.error + "]";
                  
                  if (wemSearchSkillsResultBody && wemSearchSkillsResultBody.error.message) {
                    response.errorMessage = "Call WEM - Error Received from WEM - Error [" + wemSearchSkillsResultBody.error.message + "]";
                  }
                  console.log(">>>>>>>>>>>>>>>>>> call WEM!!! wemServerClient RESULT: wemSearchSkillsResultBody.error: ", wemSearchSkillsResultBody.error);
                }
                if (err) {
                  console.log(">>>>>>>>>>>>>>>>>> call WEM!!! wemServerClient RESULT: error: ", err);
                }
                responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                response.payload = [];
                cb(null,response);
                return;
             }
             
             
             var skillsResultsToReturn = [];
             
             if (wemSearchSkillsResultBody.skillGapResult !== undefined && wemSearchSkillsResultBody.skillGapResult !== null) {
               // 2) Loop through WEMJobSearchResults
               for (var i = 0; i < wemSearchSkillsResultBody.skillGapResult.length; i++) { 
                 var skillSearchResultRaw = wemSearchSkillsResultBody.skillGapResult[i];
                    
                   // 2a) Create Job Object for result to app
                   // 2b) And Check if this jobId is saved - if yes set saved to true
                   var skillSearchResultFinal = {
                     "title": skillSearchResultRaw.jobRoleName,
                     "industry": skillSearchResultRaw.industry,
                     "meanExperience": (skillSearchResultRaw.meanExperience !== undefined && skillSearchResultRaw.meanExperience !== null && skillSearchResultRaw.meanExperience !== "") ? skillSearchResultRaw.meanExperience : null,
                     "demandLevel": skillSearchResultRaw.demandLevel,
                     "employmentGrowth": (skillSearchResultRaw.employmentGrowth !== undefined && skillSearchResultRaw.employmentGrowth !== null && skillSearchResultRaw.employmentGrowth !== "") ? skillSearchResultRaw.employmentGrowth : null,
                     "skills": []
                   }
                   var skillsArray = [];

                  if (skillSearchResultRaw.skills !== undefined && skillSearchResultRaw.skills !== null) {
                    
                   if (skillSearchResultRaw.skills.baselineSkills !== undefined && skillSearchResultRaw.skills.baselineSkills !== null) 
                   {
                       for (var b = 0; b < skillSearchResultRaw.skills.baselineSkills.length; b++) { 
                       var skillResult = skillSearchResultRaw.skills.baselineSkills[b];
                       var skillSet = {
                         "title": skillResult.name,
                         "hasSkill": skillResult.isVeteranSkill
                       };
                       skillsArray.push(skillSet);
                     } 
                   }
                   
                   if (skillSearchResultRaw.skills.specializedSkills !== undefined && skillSearchResultRaw.skills.specializedSkills !== null) 
                   {
                     for (var c = 0; c < skillSearchResultRaw.skills.specializedSkills.length; c++) { 
                       var skillResult = skillSearchResultRaw.skills.specializedSkills[c];
                       var skillSet = {
                         "title": skillResult.name,
                         "hasSkill": skillResult.isVeteranSkill
                       };
                       skillsArray.push(skillSet);
                     } 
                   }
                   
                   skillSearchResultFinal.skills = skillsArray;
                   skillsResultsToReturn.push(skillSearchResultFinal);
                   
                 }// skillSearchResultRaw.skills !== undefined - END
               }// Loop through wemSearchSkillsResultBody.skillGapResult - END
             }// wemSearchSkillsResultBody.skillGapResult !== undefined - END
             
             
             
             
             
             // Add the badge if the user has not been awarded that badge
             Skills.handleBadge(userId,badgeConstants.BadgeTypes.SKILLS_SEARCHED,function(err,badge){
               if (err){
                 errorMessage = "Error with Skills.handleBadge";
                 console.log(errorMessage,err);
                 responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                 response.payload = [];
                 cb(null,response);
               } else {
                 responseBuilder.setSuccessStatus(response);
                 if (badge !== null){
                    response.badgeAward.push(badge);
                 }
                 
                 //Add to auditEvents - Do not wait
                 app.models.auditEvent.addEvent({
                   vetId: userId.toString(),
                   eventType: app.models.auditEvent.eventTypes.SEARCH_SKILLS,
                   detail: {}
                 });
                 
                 response.payload = skillsResultsToReturn;
                 cb(null,response);
               }

             });
             
             
           });//wemServerClient.post(skillsSearchUrl) -- END
        } // else userId is defined - END
    });  // VaGrit.getUserFromToken - END
  }// END - searchSkills


  
  Skills.handleBadge = function(vetId,badgeType,cb){
    var badgeToAdd = badgeType
    app.models.userBadge.addBadge({vetId: vetId, badge: badgeToAdd },function(err, badge) {
      if (err){
        console.log("Error with userBadge.addBadge for badge [" + badgeToAdd + "] for vetId[" + vetId + "]", err);
        cb(err,null);
      }
      else {
        console.log("Successful with userBadge.addBadge for badge [" + badgeToAdd + "] for vetId[" + vetId + "]", badge);
        cb(null,badge);
      }
    });   // userBadge.addBadge

  }
  
  Skills.remoteMethod('searchSkills', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/searchSkills', verb: 'post'}
  });


};
