'use strict';


var app = require('../../server/server');
var constants = require('../constants').constants(app);
var blueTasServer = constants.cognitive.blueTasServer;
var blueTasCollectionId = constants.cognitive.blueTasCollectionId;

const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');

var request = require('request-json');//request = require('request');
var blueTasServerClient = request.createClient(blueTasServer);
var badgeConstants = require('va-grit-microservices/constants/badgeConstants');

module.exports = function(Handles) {

    //Loopback wraps the response in a "response" object and we don't want to
    // return that, this removes the response we created from the "response" object
    // and only returns the response we created.
    // TODO: May need to adjust this so only works on custom remote methods we created.
    var remotes = app.remotes();
    remotes.after('**', function (ctx, next) {
        if (ctx) {
          if (ctx.result.response !== null) {
            ctx.result = ctx.result.response;
          }
        } else {
            //console.log("ERROR. vet.js. remotes.after. ctx is not set");
            next({payload: { },
                  status: {
                   statusType: responseBuilder.StatusTypes.UNEXPECTED_EXCEPTION,
                   errorMessage: "There was an error on the server. ctx is not set"
                 }
            });
        }
        next();
    });



    Handles.getHandles = function(header, body, cb) {
       console.log("****Start of Handles.getHandles.  header: ", JSON.stringify(header));
       //console.log("****Start of Handles.getHandles.  body: ", body);

       //console.log("****Start of Handles.getHandles.  body.notificationId: ", body.notificationId);
       var response = responseBuilder.createResponseObject();
       var errorMessage;

            VaGrit.getUserFromToken(app, header, function (err, userFromToken) {
               if (userFromToken === undefined || userFromToken === null){
                  responseBuilder.setInvalidTokenStatus(response,header);
                  cb(null,response);
               }
               else { // userFromToken is defined
                  console.log(' VaGrit.getUserFromToken userFromToken: ', userFromToken);

                  // handle.find - START
                  app.models.handle.find({where:{vetId:userFromToken}, fields: {vetId:false}, order: 'socialMediaType ASC'}, function(err, findHandlesResponse){

                    if (err !== undefined && err !== null) {
                       errorMessage = "Error invoking handle.find - Error [" + err.message + "]";
                       console.log("Error invoking handle.find for userFromToken " + userFromToken +" err: ", err.message);
                       responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                    } else {
                      // console.log('======================== handle.find findHandlesResponse: ', findHandlesResponse);
                      responseBuilder.setSuccessStatus(response);
                      response.payload.handles = findHandlesResponse;
                    }

                    //console.log("Handles.getHandles. Response", response);
                     cb(null,response);

                  });// handle.find - END
              } // else userFromToken is defined
           });  // VaGrit.getUserFromToken - END
    };  // Handles.getHandles - END


    Handles.addHandle = function(header, body, cb) {
       //console.log("****Start of Handles.addHandle.  header: ", JSON.stringify(header));

       var response = responseBuilder.createResponseObject();
       var errorMessage;


            VaGrit.getUserFromToken(app, header, function (err, userFromToken) {
               if (userFromToken === undefined || userFromToken === null){
                  responseBuilder.setInvalidTokenStatus(response,header);
                  cb(null,response);
               }
               else { // userFromToken is defined
                  console.log(' VaGrit.getUserFromToken userFromToken: ', userFromToken);

          
                 if (header === undefined || header === null 
                    || body === undefined || body === null
                    || body.socialMediaType === undefined || body.socialMediaType === "" || typeof body.socialMediaType !== "string"
                    || body.handle === undefined || body.handle === "" || typeof body.handle !== "string") {
                      
                    errorMessage = "Missing Required fields or values";
                    responseBuilder.setInvalidRequestStatus(response, errorMessage);
                    cb(null,response);
                    return;
                  }
                  //console.log("****Start of Handles.addHandle.  body: ", body);



                  var newHandle = {
                    vetId: userFromToken,
                    socialMediaType: body.socialMediaType,
                    handle: body.handle
                  };

                  //Validate That Twitter Handle Exists
                  var handleExistsUrl = 'username_check/?user='+newHandle.handle+'&source='+body.socialMediaType.toLowerCase();
                  blueTasServerClient.post(handleExistsUrl, {}, function(err, res, blueTasHandleExistsResponse) {
                    
                    if (err || blueTasHandleExistsResponse === undefined || blueTasHandleExistsResponse === null || blueTasHandleExistsResponse.username_exists === undefined) {
                       errorMessage = "Error invoking blueTasServerClient.post.";
                       if (err && err.message) {
                          errorMessage = errorMessage + " Error [" + err.message + "]";
                          console.log("Error invoking blueTasServerClient.post for userFromToken " + userFromToken +" err: ", err.message);
                       }
                       console.log("Error invoking blueTasServerClient.post for userFromToken " + userFromToken);
                       responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                       cb(null,response);

                    }
                    else {
                      
                      if (!blueTasHandleExistsResponse.username_exists) {
                        errorMessage = "The social media handle you are trying to add is not valid.";
                        console.log("The social media handle you are trying to add is not valid for user: " + userFromToken);
                        responseBuilder.setSocialHandleNotValidStatus(response, errorMessage);
                        cb(null,response);
                      }
                      else {

                        //Register Handle On BlueTas
                        var addHandleUrl = 'collections/search_terms/add/?terms='+newHandle.handle+'&collection_id='+blueTasCollectionId;
                        blueTasServerClient.post(addHandleUrl, {}, function(err, res, blueTasAddHandleResponse) {
                          
                          if (err || blueTasAddHandleResponse === undefined || blueTasAddHandleResponse === null || blueTasAddHandleResponse.response === undefined || blueTasAddHandleResponse.response.status === undefined) {
                             errorMessage = "Error invoking blueTasServerClient.post.";
                             if (blueTasAddHandleResponse && blueTasAddHandleResponse.response && blueTasAddHandleResponse.response.message) {
                                errorMessage = errorMessage + " Error [" + blueTasAddHandleResponse.response.message + "]";
                                console.log("Error invoking blueTasServerClient.post for userFromToken " + userFromToken +" blueTasAddHandleResponse.response.message: ", blueTasAddHandleResponse.response.message);
                             }
                             console.log("Error invoking blueTasServerClient.post for userFromToken " + userFromToken);
                             responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                             cb(null,response);

                          }
                          else {
                            
                            if (blueTasAddHandleResponse.response.status === "Error") {
                              if (blueTasAddHandleResponse.response.message) {
                                 errorMessage = errorMessage + " Error adding handle to BlueTas [" + blueTasAddHandleResponse.response.message + "]";
                                 console.log("Error invoking blueTasServerClient.post for userFromToken " + userFromToken +" blueTasAddHandleResponse.response.message: ", blueTasAddHandleResponse.response.message);
                               }
                               responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                               cb(null,response);
                            } else {
                              //Successfully Registered Handle w BlueTas Add To Mongo




                               // handle.findOrCreate - START
                              app.models.handle.findOrCreate(newHandle, function(err, addHandleResponse){
                                if (err !== undefined && err !== null) {
                                   errorMessage = "Error invoking handle.findOrCreate - Error [" + err.message + "]";
                                   console.log("Error invoking handle.findOrCreate for userFromToken " + userFromToken +" err: ", err.message);
                                   responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                } else {
                                   console.log('======================== handle.findOrCreate addHandleResponse: ', addHandleResponse);
                                   Handles.handleBadge(userFromToken, function (err,badge){
                                     if (err){
                                       errorMessage = "Error Handles.handleBadge - Error [" + err.message + "]";
                                       console.log("Error invoking Handles.handleBadge for userFromToken " + userFromToken +" err: ", err);
                                       responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                       cb(null,response);
                                     }
                                     else {
                                       
                                       
                                       //Add to auditEvents - Do not wait
                                       app.models.auditEvent.addEvent({
                                         vetId: userFromToken.toString(),
                                         eventType: app.models.auditEvent.eventTypes.ADD_SOCIAL_MEDIA_HANDLE,
                                         detail: {}
                                       });

                                       
                                       responseBuilder.setSuccessStatus(response);

                                       if (badge !== null){
                                          response.badgeAward.push(badge);
                                       }
                                       response.payload.id = addHandleResponse.id;
                                       //console.log("Handles.addHandle. Response", response);
                                       cb(null,response);

                                     }
                                   });

                                }

                              });// handle.findOrCreate - END

                            }// ELSE - Successfully Registered Handle w BlueTas Add To Mongo - END
                            
                          }
                          
                          
                          
                          
                          
                        });//Register Handle On BlueTas = END

                      }// else body.username_exists === true - END

                    }// ELSE blueTasServerClient.post handleExistsUrl is success - END

                  });//Validate That Twitter Handle Exists  - END

              } // else userFromToken is defined
           });  // VaGrit.getUserFromToken - END

        

    };  // Handles.addHandle - END




    Handles.removeHandle = function(header, body, cb) {
       console.log("****Start of Handles.removeHandle.  header: ", JSON.stringify(header));
       var response = responseBuilder.createResponseObject();
       var errorMessage;

        VaGrit.getUserFromToken(app, header, function (err, userFromToken) {
           if (userFromToken === undefined || userFromToken === null){
              responseBuilder.setInvalidTokenStatus(response,header);
              cb(null,response);
           }
           else { // userFromToken is defined
              console.log(' VaGrit.getUserFromToken userFromToken: ', userFromToken);

            
            //Required fields check 
            if (body === undefined || body === null 
                  || body.id === undefined || body.id === null || body.id === "" || typeof body.id !== "string") {
              errorMessage = "Missing Required fields or values";
              responseBuilder.setInvalidRequestStatus(response, errorMessage);
              cb(null,response);
              return;
            }
            
            const handleId = body.id;
            //console.log("****Start of Handles.removeHandle.  body: ", body);
            //console.log("****Start of Handles.removeHandle.  body.notificationId: ", body.notificationId);


              // handle.find - START
              app.models.handle.find({where:{and: [{vetId:userFromToken}, {id:handleId}]}}, function(err, findHandlesResponse){

                if (err !== undefined && err !== null) {
                   errorMessage = "Error invoking handle.find - Error [" + err.message + "]";
                   console.log("Error invoking handle.find for userFromToken " + userFromToken +" err: ", err.message);
                   responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                   cb(null,response);
                } else {
                   
                   if (findHandlesResponse === null || (findHandlesResponse !== null && findHandlesResponse.length === 0)) {
                     responseBuilder.setSuccessStatus(response);
                     cb(null,response);
                   }
                   else {

                     //Remove Twitter Handle

                     var removeHandleUrl = "collections/search_terms/delete/?terms="+findHandlesResponse[0].handle+"&collection_id="+blueTasCollectionId;
                     blueTasServerClient.post(removeHandleUrl, {}, function(err, res, body) {
                      
                       if ((err && err !== null) || body === undefined || body === null || body.response === undefined || body.response.status === undefined) {
                          errorMessage = "Error invoking blueTasServerClient.post.";
                          if (err && err.message) {
                             errorMessage = errorMessage + " Error [" + err.message + "]";
                             console.log("Error invoking blueTasServerClient.post for userFromToken " + userFromToken +" err: ", err.message);
                          }
                          console.log("Error invoking blueTasServerClient.post for userFromToken " + userFromToken);
                          responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                          cb(null,response);

                       }
                       else {
                            
                        if (body.response.status === "Error") {
                           errorMessage = "Error invoking handle.destroyById from BlueTas - Error [" + body.response.message + "]";
                           console.log("Error invoking handle.destroyById for userFromToken " + userFromToken +" body.response.message: ", body.response.message);
                           responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                         }
                         else
                         {
                           // handle.destroyById - START
                           app.models.handle.destroyById(handleId, function(err, destroyByIdHandlesResponse){

                             if (err) {
                                errorMessage = "Error invoking handle.destroyById - Error [" + err.message + "]";
                                console.log("Error invoking handle.destroyById for userFromToken " + userFromToken +" body.response.message: ", err.message);
                                responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                             } else {
                               
                               //Add to auditEvents - Do not wait
                               app.models.auditEvent.addEvent({
                                 vetId: userFromToken.toString(),
                                 eventType: app.models.auditEvent.eventTypes.REMOVE_SOCIAL_MEDIA_HANDLE,
                                 detail: {}
                               });
                               
                              //console.log('======================== handle.destroyById destroyByIdHandlesResponse: ', destroyByIdHandlesResponse);
                               responseBuilder.setSuccessStatus(response);
                               //response.payload.recordsRemoved = destroyByIdHandlesResponse.count;
                             }
                             cb(null,response);
                           });// handle.destroyById - END
                         }
                       }
                     });// removeHandleUrl - END

                   }// ELSE - FIND HANDLE NOT NULL - END
                }
              });// handle.find - END
          } // else userFromToken is defined
       });  // VaGrit.getUserFromToken - END
    };  // Handles.removeHandle - END



    Handles.updateHandle = function(header, body, cb) {
       //console.log("****Start of Handles.updateHandle.  header: ", JSON.stringify(header));
       var response = responseBuilder.createResponseObject();
       var errorMessage;

       // WITH UPDATE
       // 1) Make sure handle belongs to user - THIS WILL ALSO GET ID FOR UPDATE
       // 2) Validate new handle with BlueTas
       // 3) If valid,
       // 3a) Delete old handle from BlueTas
       // 3b) Add new handle to bluetas
       // 3c) update handle in midtier
       // 4) ELSE - Don't update - send error


            VaGrit.getUserFromToken(app, header, function (err, userFromToken) {
               if (userFromToken === undefined || userFromToken === null){
                  responseBuilder.setInvalidTokenStatus(response,header);
                  cb(null,response);
               }
               else { // userFromToken is defined
                  console.log(' VaGrit.getUserFromToken userFromToken: ', userFromToken);
                  
                  
                 if (header === undefined || header === null 
                    || body === undefined || body === null
                    || body.id === undefined || body.id === null || body.id === "" || typeof body.id !== "string"
                    || body.handle === undefined || body.handle === null || body.handle === "" || typeof body.handle !== "string") {
                      
                    errorMessage = "Missing Required fields or values";
                    responseBuilder.setInvalidRequestStatus(response, errorMessage);
                     cb(null,response);
                     return;
                  }
                  
                  const handleId = body.id;
                  const newHandle = body.handle;
                  
                  app.models.handle.find({where:{vetId:userFromToken}}, function(err, findHandlesResponse){

                    if (err !== undefined && err !== null) {
                       errorMessage = "Error invoking handle.find - Error [" + err.message + "]";
                       console.log("Error invoking handle.find for userFromToken " + userFromToken +" err: ", err.message);
                       responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                       cb(null,response);
                    } else {
                       console.log('======================== handle.find findHandlesResponse: ', findHandlesResponse);

                       // 1) Make sure handle belongs to user - THIS WILL ALSO GET ID FOR UPDATE
                       if (findHandlesResponse === null || (findHandlesResponse !== null && findHandlesResponse.length === 0)) {
                         // 1a) DOES NOT BELONG TO USER - SEND ERROR
                         errorMessage = "Cannot update handle - this handle does not belong to this user";
                         console.log("Error invoking handle.updateHandle for userFromToken " + userFromToken +" errorMessage: ", errorMessage);
                         responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                         cb(null,response);
                       }
                       else {

                        /// CHECK USER OWNS THE RECORD and
                        /// CHECK IF HANDLE DUPLICATE FOR USER
                        var userOwnsHandle = false;
                        var handleIsDuplicate = false;
                        var handleRecordToUpdate;
                        for (var i = 0; i < findHandlesResponse.length; i++) {
                          if (findHandlesResponse[i].id.toString() === handleId.toString()) {
                            userOwnsHandle = true;
                            handleRecordToUpdate = findHandlesResponse[i];
                          }
                          if (findHandlesResponse[i].id.toString() !== handleId.toString() && findHandlesResponse[i].handle.toString() === newHandle.toString()) {
                            handleIsDuplicate = true;
                          }

                        }

                        if (!userOwnsHandle) {
                          // 1a) DOES NOT BELONG TO USER - SEND ERROR
                          errorMessage = "Cannot update handle - this handle does not belong to this user";
                          console.log("Error invoking handle.updateHandle for userFromToken " + userFromToken +" errorMessage: ", errorMessage);
                          responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                          cb(null,response);
                        }

                        else if (handleIsDuplicate) {
                          // 1b) handle already exists for this user - SEND ERROR
                          errorMessage = "Cannot update handle - this handle already exists for this user";
                          console.log("Error invoking handle.updateHandle for userFromToken " + userFromToken +" errorMessage: ", errorMessage);
                          responseBuilder.setSocialHandleNotUniqueStatus(response, errorMessage);
                          cb(null,response);
                        }

                        else {


                          var handleToUpdateId = handleRecordToUpdate.id;
                          var oldHandle = handleRecordToUpdate.handle;
                          var updateHandleType = handleRecordToUpdate.socialMediaType;

                          var updateHandleBody = {
                             "socialMediaType": updateHandleType,
                             "handle": newHandle
                           };

                           
                           // 2) Validate new handle with BlueTas

                           Handles.blueTasValidateHandle(updateHandleBody, function(err, validationResult) {

                             if (err !== null) {
                               errorMessage = "Error Validating Handle On BlueTas";
                               console.log("Error Validating Handle On BlueTas for userFromToken " + userFromToken +" errorMessage: ", errorMessage);
                               responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                               cb(null,response);
                             }
                             else {
                               if (!validationResult) {
                                 errorMessage = "The social media handle you are trying to add is not valid.";
                                 console.log("The social media handle you are trying to add is not valid: " + newHandle);
                                 responseBuilder.setSocialHandleNotValidStatus(response, errorMessage);
                                 cb(null,response);
                               }
                               else {
                                 //HANDLE IS VALID

                                 // 3) If valid,
                                 // 3a) Delete old handle from BlueTas
                                 Handles.blueTasRemoveHandle(oldHandle, function(err, deleteResult) {

                                   if (err !== null) {
                                     errorMessage = "Error Deleting Handle On BlueTas";
                                     console.log("Error Deleting Handle On BlueTas for userFromToken " + userFromToken +" errorMessage: ", errorMessage);
                                     responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                     cb(null,response);
                                   }
                                   else {
                                     if (!deleteResult) {
                                       errorMessage = "Error Deleting Handle On BlueTas.";
                                       responseBuilder.setSocialHandleNotValidStatus(response, errorMessage);
                                       cb(null,response);
                                     }
                                     else {
                                       //HANDLE IS DELETED FROM BLUETAS

                                       // 3b) Add new handle to bluetas

                                       Handles.blueTasAddHandle(newHandle, function(err, addHandleResponse) {
                                         console.log('+++++ Handles.updateHandle call.blueTasAddHandle err: ', err);
                                         
                                         if (err !== null) {
                                           errorMessage = "Error Adding Handle On BlueTas";
                                           //console.log("Error Adding Handle On BlueTas for userFromToken " + userFromToken +" errorMessage: ", errorMessage);
                                           responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                           cb(null,response);
                                         }
                                         else {
                                           if (!addHandleResponse) {
                                             errorMessage = "Error Adding Handle On BlueTas: " + newHandle;
                                             //console.log("Error Adding Handle On BlueTas newHandle: " + newHandle);
                                             responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                             cb(null,response);
                                           }
                                           else {
                                             //HANDLE IS ADDED TO BLUETAS

                                             // 3c) update handle in midtier

                                             handleRecordToUpdate.handle = newHandle;
                                             handleRecordToUpdate.socialMediaType = updateHandleType;
                                             handleRecordToUpdate.save(function(err, savedHandle){
                                                if (err !== undefined && err !== null || (savedHandle === undefined || savedHandle === null)) {
                                                   errorMessage = "Error Saving Handle";
                                                   responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                                   if (err.details && err.details.messages && err.details.messages.handle[0] && err.details.messages.handle[0] !== null) {
                                                     errorMessage = "Error Updating Handle: " + newHandle;
                                                     responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                                   }
                                                   console.log("Error Saving Handle for userFromToken " + userFromToken +" errorMessage: ", errorMessage);
                                                   cb(null,response);
                                               }
                                               else {
                                                 
                                                 //Add to auditEvents - Do not wait
                                                 app.models.auditEvent.addEvent({
                                                   vetId: userFromToken.toString(),
                                                   eventType: app.models.auditEvent.eventTypes.UPDATE_SOCIAL_MEDIA_HANDLE,
                                                   detail: {}
                                                 });
                                                 
                                                 responseBuilder.setSuccessStatus(response);
                                                 response.payload.id = savedHandle.id;
                                                 cb(null,response);
                                               }
                                             });// handleRecordToUpdate.save - END



                                           }//HANDLE IS ADDED TO BLUETAS - ELSE END
                                         }//HANDLE IS ADDED TO BLUETAS - NO error
                                       });//HANDLE IS ADDED TO BLUETAS - END



                                     }//HANDLE IS DELETED FROM BLUETAS - ELSE END
                                   }//HANDLE IS DELETED FROM BLUETAS - NO error
                                 });//HANDLE IS DELETED FROM BLUETAS - END

                               }//HANDLE IS VALID - ELSE END
                             }//HANDLE IS VALID - NO error
                           });//HANDLE IS VALID - END

                         }// Check if user owns record and is not duplicate

                       }// ELSE - FIND HANDLE NOT NULL - END
                    }
                  });// handle.find - END
              } // else userFromToken is defined
           });  // VaGrit.getUserFromToken - END
    };  // Handles.updateHandle - END


    Handles.blueTasValidateHandle = function(body, cb) {
       //console.log("****Start of Handles.blueTasValidateHandle.  body: ", body);
       //var response = responseBuilder.createResponseObject();
       var errorMessage;

       //Validate That Twitter Handle Exists
       var handleExistsUrl = 'username_check/?user='+body.handle+'&source='+body.socialMediaType.toLowerCase();
       blueTasServerClient.post(handleExistsUrl, {}, function(err, res, body) {
         if (err || body === undefined || body === null || body.username_exists === undefined) {
            errorMessage = "Error invoking blueTasServerClient.post.";
            if (err && err.message) {
               errorMessage = errorMessage + " Error [" + err + "]";
               console.log("Error invoking blueTasServerClient.username_check err: ", err);
            }
            cb(errorMessage, false);
         }
         else {
           if (!body.username_exists) {
             cb(null, false);
           }
           else {
             cb(null, true);
           }
         }
       });
    };  // Handles.blueTasValidateHandle - END


  Handles.blueTasRemoveHandle = function(handleToRemove, cb) {
     //console.log("****Start of Handles.blueTasRemoveHandle.  handleToRemove: ", handleToRemove);
     //var response = responseBuilder.createResponseObject();
     var errorMessage;

     var removeHandleUrl = "collections/search_terms/delete/?terms="+handleToRemove+"&collection_id="+blueTasCollectionId;
     blueTasServerClient.post(removeHandleUrl, {}, function(err, res, body) {
       console.log("SOCIAL ++++++ callblueTasServerClient!!! blueTasServerClient2 removeHandle error: ", err);

       if ((err && err !== null) || body === undefined || body === null || body.response === undefined || body.response.status === undefined) {
          errorMessage = "Error invoking blueTasServerClient.delete Handle.";
          if (err && err.message) {
             errorMessage = errorMessage + " Error [" + err.message + "]";
             //console.log("Error invokingblueTasServerClient.delete Handle for handleToRemove " + handleToRemove +" err: ", err.message);
          }
          //console.log("Error invoking blueTasServerClient.delete Handle for handleToRemove " + handleToRemove);
          cb(errorMessage, null);
       }
       else {
        if (body.response.status === "Error") {
           errorMessage = "Error invoking blueTasServerClient.delete Handle.";
           if (body.response.message) {
             errorMessage = errorMessage + " Error: [" + body.response.message + "]";
           }
           cb(errorMessage, false);
         }
         else
         {
            ///HANDLE REMOVED SUCCESS
            cb(null, true);
         }
       }
    });

  };  // Handles.blueTasRemoveHandle - END




  Handles.blueTasAddHandle = function(handleToAdd, cb) {
     //console.log("****Start of Handles.blueTasAddHandle.  handleToAdd: ", handleToAdd);
     //var response = responseBuilder.createResponseObject();
     var errorMessage;

     //Register Handle On BlueTas
     var addHandleUrl = 'collections/search_terms/add/?terms='+handleToAdd+'&collection_id='+blueTasCollectionId;
     blueTasServerClient.post(addHandleUrl, {}, function(err, res, body) {
       console.log("SOCIAL callblueTasServerClient blueTasServerClient2 addHandle err: ", err);
       
       if ((err && err !== null) || body === undefined || body === null || body.response === undefined || body.response.status === undefined) {
          errorMessage = "Error invoking blueTasServerClient.add Handle.";
          if (err && err.message) {
             errorMessage = errorMessage + " Error [" + err.message + "]";
             console.log("Error invokingblueTasServerClient.add Handle err: ", err.message);
          }
          cb(errorMessage, null);
       }
       else {
         if (body.response.status === "Error")
         {
           errorMessage = "Error invoking blueTasServerClient.add Handle.";
           if (body.response.message) {
             errorMessage = errorMessage + " Error: [" + body.response.message + "]";
           }
           cb(errorMessage, false);
         }
         else
         {
            ///HANDLE ADD SUCCESS
            cb(null, true);
         }
       }
     });
  };  // Handles.blueTasAddHandle - END

  Handles.handleBadge = function(vetId,cb){
    var badgeToAdd = badgeConstants.BadgeTypes.TWITTER_HANDLE_ADDED;
    app.models.userBadge.addBadge({vetId: vetId, badge:badgeToAdd},function(err, badge) {
      if (err){
        console.log("Error with userBadge.addBadge for badge [" + badgeToAdd + "] for vetId[" + vetId + "]", err);
        cb(err,null);
      }
      else {
        //console.log("Successful with userBadge.addBadge for badge [" + badgeToAdd + "] for vetId[" + vetId + "]", badge);
        cb(null,badge);
      }
    });   // userBadge.addBadge

  }


    Handles.remoteMethod('removeHandle', {
        accepts: [
          {arg: 'Authorization', type: 'string', http: { source: 'header' }},
          { arg: 'data', type: 'object', http: { source: 'body' } }],
        returns: {arg: 'response', type: 'string'},
        http: {path: '/removeHandle', verb: 'post'}
    });


    Handles.remoteMethod('addHandle', {
        accepts: [
          {arg: 'Authorization', type: 'string', http: { source: 'header' }},
          { arg: 'data', type: 'object', http: { source: 'body' } }],
        returns: {arg: 'response', type: 'string'},
        http: {path: '/addHandle', verb: 'post'}
    });

    Handles.remoteMethod('updateHandle', {
        accepts: [
          {arg: 'Authorization', type: 'string', http: { source: 'header' }},
          { arg: 'data', type: 'object', http: { source: 'body' } }],
        returns: {arg: 'response', type: 'string'},
        http: {path: '/updateHandle', verb: 'post'}
    });


    Handles.remoteMethod('getHandles', {
        accepts: [
          {arg: 'Authorization', type: 'string', http: { source: 'header' }},
          { arg: 'data', type: 'object', http: { source: 'body' } }],
        returns: {arg: 'response', type: 'string'},
        http: {path: '/getHandles', verb: 'post'}
    });



};
