// Version Dec 6, 2018
var app = require('../../server/server');
var journalEventConstants = require('va-grit-microservices/constants/journalEventConstants');

// user-badge is a model that is shared by different microservices.
// It is in a shared folder that is copied from the va-grit-MIDTIER
// common shared folder to the microservice shared folder
module.exports = function(userBadge) {

// Internal function and not an external remote api. Invoked by other models to add a badge.
userBadge.addBadge = function(request, cb) {
   //console.log("****Starting userBadge.addBadge.  request: ", JSON.stringify(request));

   // The vetId and badge is in the request
   var vetId = request.vetId;
   var badge = request.badge;
   var badges = [];

   if (vetId === undefined || vetId === null || badge === undefined || badge === null || badge.length < 1) {
      var errorMessage = "The request does not have all the required field.  vetId and badge is required for the userBadge.addBadge";
      console.log("Detected error with userBadge.addBadge. " + errorMessage);
      var err = new Object();
      err.message = errorMessage;
      cb(err,null);
   }
   else { // else valid request
      console.log("Internal userBadge.addBadge. Performing userBadgeFind for vetId " + vetId);
      userBadge.find({where: {_id: vetId}}, function(err, userBadges) {
         if (err) {
            console.log("userBadge.find for vetId [" + vetId + "] resulted in an err", err);
            cb(err,null);
         }
         else {  // no error with userBadge.find
            if (userBadges !== undefined && userBadges.length > 0 ){
               console.log("found badges for user " + vetId, userBadges);
               badges = userBadges[0].badges;
            }

            // Check and make sure the badge isn't already in the list
            if ( (badges.length > 0)  && (badges.includes(badge)) ){
               console.log("Vet " + vetId + " already has badge " + badge + ". No need to add");

               // A null badge is returned as the 2nd parameter to indicate the badge was not added.
               // The caller to this function should NOT push the badge into the response.badgeAward array
               cb(null,null);
            }
            else {  // else update the badge
               badges.push(badge);
               //console.log("userBadge.upsert. For vetId " + vetId + " badge [" + badge + "]. List:", badges);
               userBadge.upsert({vetId: vetId, badges: badges}, function(err, userBadgeInstance) {
               if (err) {
                  console.log("Error with userBadge.upsert. err:", err);
                  cb(err,null);
               }
               else { // else not an error with userBadge.upsert
                  console.log("Successfully added badge " + badge + " for user " + vetId + ". user badges:", userBadgeInstance);

                  app.models.journalEvent.upsert({vetId: vetId, eventType: journalEventConstants.JournalEventTypes.BADGES_AWARDED,date:new Date(),detail:badge}, function(err, eventInstance) {
                    if (err){
                      console.log("An error with journalEvent.upsert", err);
                      cb(err,null);
                    }
                    else {
                      // The badge is returned in the second parameter to indicate the badge was added.
                      // The caller to this function should push this badge into the response.badgeAward array
                      cb(null,badge);
                   }
                 });
               }  // else not an error with userBadge.upsert

             } );  // userBadge.upsert
            }  // else update the badge

          }  // else no error with userBadge.find
        });  // userBadge.find

     }  // else valid request
   }
  };
