'use strict';
var badgeConstants = require('va-grit-microservices/constants/badgeConstants');
var journalEventConstants = require('va-grit-microservices/constants/journalEventConstants');

var app = require('../../server/server');
var constants = require('../constants').constants(app);
var mongoUrl = constants.mongo.url;
var mongoDbName = constants.mongo.dbName;
var accountSid = constants.twilio.accountSid;
var authToken = constants.twilio.authToken;
var fromPhoneNumber = constants.twilio.fromPhoneNumber;
var appStoreLink = constants.appStores.appStoreLink;// If empty string this will hide the link the response SMS to Squad Mate

const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');

const twilio = require('twilio');
var MongoClient = require('mongodb').MongoClient;


module.exports = function(Mates) {

  //Loopback wraps the response in a "response" object and we don't want to
  // return that, this removes the response we created from the "response" object
  // and only returns the response we created.
  // TODO: May need to adjust this so only works on custom remote methods we created.
  var remotes = app.remotes();
  remotes.after('**', function (ctx, next) {
      if (ctx) {
        if (ctx.result.response !== null) {
          ctx.result = ctx.result.response;
        }
      } else {
          //console.log("ERROR. vet.js. remotes.after. ctx is not set");
          next({payload: { },
                status: {
                 statusType: responseBuilder.StatusTypes.UNEXPECTED_EXCEPTION,
                 errorMessage: "There was an error on the server. ctx is not set"
               }
          });
      }
      next();
  });

  Mates.getAllMates = function(header, body, cb) {
   //console.log("****Start of Da.startConversation.  header: ", JSON.stringify(header));
   //console.log("****Start of Mates.getAllMates.  body: ", JSON.stringify(body));
   var response = responseBuilder.createResponseObject();
   var errorMessage;

      VaGrit.getUserFromToken(app, header, function (err, userIdFromToken) {
         if (userIdFromToken === undefined || userIdFromToken === null){
            responseBuilder.setInvalidTokenStatus(response,header);
            cb(null,response);
         }
         else { // userIdFromToken is defined
            console.log(' VaGrit.getUserFromToken userIdFromToken: ', userIdFromToken);

            app.models.mate.find({where: {vetId: userIdFromToken}}, function(err, squadMatesResponse) {
              if (err) {
                 errorMessage = "Error invoking mate.find. Error [" + err.message + "]";
                 console.log("Error invoking mate.find for userId " + userIdFromToken +" err: ", err);
                 responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                 cb(null,response);
              }
              else {
                
                var allSquaMates = []
                for (var i = 0; i < squadMatesResponse.length; i++) {
                    allSquaMates.push({
                      "name": squadMatesResponse[i].name,
                      "cellPhone": squadMatesResponse[i].cellPhone,
                      "state": squadMatesResponse[i].state,
                      "id": squadMatesResponse[i].id
                    });

                }

                //console.log("Mates.getAllMates mate.find allSquaMates: ", JSON.stringify(allSquaMates));
                responseBuilder.setSuccessStatus(response);
                response.payload.squad = allSquaMates;
                //console.log("Da.startConversation. Response", response);
                cb(null,response);
              }
            });


       } // else userIdFromToken is defined
    });  // VaGrit.getUserFromToken
  };  // Mates.getAllMates



  Mates.addMate = function(header, body, cb) {
     console.log("****Start of Mates.addMate.  header: ", JSON.stringify(header));
     var response = responseBuilder.createResponseObject();
     var errorMessage;
     var mateCount=0;

        VaGrit.getUserFromToken(app, header, function (err, userIdFromToken) {
           if (userIdFromToken === undefined || userIdFromToken === null){
              responseBuilder.setInvalidTokenStatus(response,header);
              cb(null,response);
           }
           else { // userIdFromToken is defined
             console.log('Mates.addMate VaGrit.getUserFromToken userIdFromToken: ', userIdFromToken);

             
             //Required fields check 
             if (body === undefined || body === null 
                   || body.name === undefined || body.name === null || body.name === "" || typeof body.name !== "string"
                   || body.cellPhone === undefined || body.cellPhone === null || body.cellPhone === "" || typeof body.cellPhone !== "string") {
               errorMessage = "Missing Required fields or values";
               responseBuilder.setInvalidRequestStatus(response, errorMessage);
               cb(null,response);
               return;
             }

             
             //console.log("****Start of Mates.addMate.  body: ", JSON.stringify(body));


             Mates.getAllMates(header, {}, function (err, getAllMatesResponse) {
                if (getAllMatesResponse === undefined || getAllMatesResponse === null){
                   errorMessage = "Error invoking mate.getAllMates. Error [" + err.message + "]";
                   console.log("Error invoking mate.getAllMates for userId " + userIdFromToken +" err: ", err);
                   responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                   cb(null,response);
                }
                else { // userIdFromToken is defined
                 
                 // If already has 5 mates, don't allow to add anymore and let app know
                 mateCount = getAllMatesResponse.payload.squad.length;
                 if (getAllMatesResponse.payload.squad.length >= 5) {
                   errorMessage = "Maximum Number of Squad Mates Reached. Squad Mate not added.";
                   responseBuilder.setSquadMatesMaxReachedStatus(response, errorMessage);
                   cb(null,response);
                 }
                 else {

                   // Check if phone number already exists for this user

                  app.models.mate.find({where: {and: [{vetId: userIdFromToken}, {cellPhone: body.cellPhone}]}}, function(err, duplicateSquadMateCheckResponse) {
                    if (err) {
                       errorMessage = "Error invoking mate.find if phone number already exists. Error [" + err.message + "]";
                       console.log("Error invoking mate.find if phone number already exists for userId " + userIdFromToken +" err: ", err);
                       responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                       cb(null,response);
                    }
                    else {
                      
                      if (duplicateSquadMateCheckResponse.length === 0) {
                      
                         //SEND ACCEPT SMS TO SQUAD MATE
                         var twilioBody = {
                            "vetId": userIdFromToken,
                            "name": body.name,
                            "cellPhone": body.cellPhone
                         };



                         //Get Vet Info (like Name) for SMS message
                         MongoClient.connect(mongoUrl, function(err, db) {
                           if (err) throw err;
                           var dbo = db.db(mongoDbName);
                           dbo.collection("vet").findOne({_id:userIdFromToken}, function(err, vetInstance) {
                             if (err) {
                                console.log('TwilioClient mongodbConnection: err: ', err);
                                errorMessage = "TwilioClient Error invoking vetServerClient.getUser - Error [" + err.message + "]";
                                //console.log("TwilioClient Error invoking vetServerClient.getUser for userFromToken " + userFromToken +" err: ", err);
                                //responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                cb(errorMessage, null);
                             } else {
                                // Send initial conversation to Carla

                                if (vetInstance === undefined || vetInstance === null) {

                                   errorMessage = "TwilioClient vet.find - No vet with that vetId found";
                                   //console.log("TwilioClient Error invoking vetServerClient.getUser for userFromToken " + userFromToken +" err: ", err);
                                 //  responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                   cb(errorMessage, null);

                                }
                                else {
                                  
                                  var client = new twilio(accountSid, authToken);
                                  var toPhone = body.cellPhone;

                                  var smsBody1 = "Hello " + body.name + ". This is a message on behalf of " + vetInstance.name + " who listed you as someone trusted who could provide talk or support via the GRIT app.";
                                  //"Hello " + body.name + ". This is a message from the VA GRIT App on behalf of " + vetInstance.name + " who may need your support.  Can we notify you if you we feel " + vetInstance.name + " needs your help? Reach out to " + vetInstance.name + " for more information. If you want to get more messages, respond with 1.";
                                  client.messages.create({
                                    body: smsBody1,
                                    from: fromPhoneNumber,
                                    to: toPhone
                                  })
                                  .then(function(message) {

                                    var smsBody2 = 'Reach out to ' + vetInstance.name;
                                    if (appStoreLink !== "") {
                                      smsBody2 += ' or ' + appStoreLink;
                                    }
                                    smsBody2 += ' for more info on the GRIT app for service members and veterans.';
                                    smsBody2 += ' If you agree to be notified, respond with "1".';

                                    client.messages.create({
                                      body: smsBody2,
                                      from: fromPhoneNumber,
                                      to: toPhone
                                    })
                                    .then(function(message) {
                                      
                                      //Add to auditEvents - Do not wait
                                      app.models.auditEvent.addEvent({
                                        vetId: userIdFromToken.toString(),
                                        eventType: app.models.auditEvent.eventTypes.ADD_SQUAD_MATE,
                                        detail: {}
                                      });

                                      Mates.create({
                                          "vetId": userIdFromToken,
                                           "name": body.name,
                                           "cellPhone": body.cellPhone,
                                           "state": "PENDING",
                                           "timestamp": new Date().toISOString(),
                                           "twilioAcceptSid": (message.sid != null) ? message.sid : ""
                                        }, mateCount,cb);
                                      }//then 2 - END
                                    );

                                  }//then 1 - END
                                  );




                                }// ELSE vetInstance === undefined - END
                             }
                             db.close();
                           });
                         });
                      }
                      else {
                      //  console.log(">>>>>>>>>>>>>>>>>>>>> Check if phone number already exists for this user 4: DO NOT ADD - DUPLICATE");
                        errorMessage = "Squad Mate with the same cell phone number already exists for this user.";
                        responseBuilder.setSquadMatesAlreadyExistsStatus(response, errorMessage);
                        cb(null,response);
                      }
                    }// Check if phone number already exists for this user - else no error - END
                  });// Check if phone number already exists for this user - END
                 } // Check if has 5 squad mates - end else
               }
            });// Mates.getAllMates - END
         } // else userIdFromToken is defined
      });  // VaGrit.getUserFromToken
  };  // Mates.addMate


  Mates.create = function(squadMate, mateCount,cb) {

    //console.log("Mates.create _ squadMate: ", squadMate);
    //console.log("Mates.create. mateCount:", mateCount);
    var response = responseBuilder.createResponseObject();
    var errorMessage;
    app.models.mate.create(squadMate, function(err, addedSquadMateResponse) {
      if (err) {
         errorMessage = "Error invoking mate.add. Error [" + err.message + "]";
         console.log("Error invoking mate.add for userId " + squadMate.vetId +" err: ", err);
         responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
         cb(null,response);
      }
      else {
        //console.log("Mates.getAllMates mate.create addedSquadMateResponse: ", JSON.stringify(addedSquadMateResponse));
        responseBuilder.setSuccessStatus(response);
        response.payload.id = addedSquadMateResponse.id;

        Mates.handleEventJournalAndBadge(addedSquadMateResponse.id,squadMate.vetId, mateCount, function(err,badge) {
           if (err) {
             errorMessage = "Error with handleEventJournalAndBadge Error [" + err.message + "]";
             console.log("Error with handleEventJournalAndBadge for userId " + squadMate.vetId +" err: ", err);
             responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
             cb(null,response);
           }
           else {
              if (badge !== null){
                 response.badgeAward.push(badge);
              }
           }

           //response.payload.sentSms = twilioResponse;
           //console.log("Mates.addMate response: ", response);
           cb(null,response);
        }); // checkForBadge
          }
    });// Add Mate - END



  };

  Mates.removeMate = function(header, body, cb) {
   console.log("****Start of Mates.removeMate.  header: ", JSON.stringify(header));
   var response = responseBuilder.createResponseObject();
   var errorMessage;

      VaGrit.getUserFromToken(app, header, function (err, userIdFromToken) {
         if (userIdFromToken === undefined || userIdFromToken === null){
            responseBuilder.setInvalidTokenStatus(response,header);
            cb(null,response);
         }
         else { // userIdFromToken is defined
            console.log(' VaGrit.getUserFromToken userIdFromToken: ', userIdFromToken);
            
            //Required fields check 
            if (body === undefined || body === null 
                  || body.id === undefined || body.id === null || body.id === "" || typeof body.id !== "string") {
              errorMessage = "Missing Required fields or values";
              responseBuilder.setInvalidRequestStatus(response, errorMessage);
              cb(null,response);
              return;
            }
            
            //console.log("****Start of Mates.removeMate.  body: ", JSON.stringify(body));

            //Check if this vet owns the squadmate
            app.models.mate.find({where: {and: [{vetId: userIdFromToken}, {_id: body.id}]}}, function(err, findVetSquadMateResponse) {
              if (err) {
                 errorMessage = "Error invoking mate.find. Error [" + err.message + "]";
                 console.log("Error invoking mate.find for userId " + userIdFromToken +" err: ", err);
                 responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                 cb(null,response);
              }
              else {
                //console.log("Mates.getAllMates mate.destroyAll findVetSquadMateResponse: ", JSON.stringify(findVetSquadMateResponse));
                //console.log("Mates.removeMate mate.find findVetSquadMateResponse: ", JSON.stringify(findVetSquadMateResponse));
                if (findVetSquadMateResponse.length === 0) {
                  responseBuilder.setSuccessStatus(response);
                  //response.payload.removedSquadMateCount = 0;
                  //console.log("Mates.removeMate response: ", response);
                  cb(null,response);
                }
                else {

                  // Delete mate by id
                  app.models.mate.destroyById(body.id, function(err, destroyedSquadMateResponse) {
                    if (err) {
                       errorMessage = "Error invoking mate.destroyAll. Error [" + err.message + "]";
                       console.log("Error invoking mate.destroyAll for userId " + userIdFromToken +" for squad mate id:" + body.id + " err: ", err);
                       responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                       cb(null,response);
                    }
                    else {
                      
                      
                      //Add to auditEvents - Do not wait
                      app.models.auditEvent.addEvent({
                        vetId: userIdFromToken.toString(),
                        eventType: app.models.auditEvent.eventTypes.REMOVE_SQUAD_MATE,
                        detail: {}
                      });
                      
                      //console.log("Mates.getAllMates mate.destroyAll destroyedSquadMateResponse: ", JSON.stringify(destroyedSquadMateResponse));
                      responseBuilder.setSuccessStatus(response);
                      //response.payload.removedSquadMateCount = destroyedSquadMateResponse.count;
                      //console.log("Mates.removeMate response: ", response);
                      cb(null,response);
                    }
                  });// mate.destroyAll - END

                }



              }
            });
       } // else userIdFromToken is defined
    });  // VaGrit.getUserFromToken
  };  // Mates.removeMate

Mates.handleEventJournalAndBadge = function(mateId,vetId,mateCount,cb){
  // Add the Journal Event for adding a squad mate
  app.models.journalEvent.upsert({vetId: vetId, eventType: journalEventConstants.JournalEventTypes.SQUAD_MEMBER_ADDED ,
                                  date:new Date(),detail:mateId}, function(err, eventInstance) {
    if (err){
      console.log("An error with journalEvent.upsert", err);
      cb(err,null);

    }
    else {   // else not an error with journalEvent.upsert
      //console.log("Successfully added a Mate: " + mateId + " Journal Event for vet id : " + vetId);
      var badge = null;
      if (mateCount === 0){
        badge = badgeConstants.BadgeTypes.SQUAD_1;
      }
      else if (mateCount === 2) {
        badge = badgeConstants.BadgeTypes.SQUAD_3;
      }
      else if (mateCount === 4) {
        badge = badgeConstants.BadgeTypes.SQUAD_5;
      }

      if (badge == null){
        // A badge is not awarded
        cb(null,null);
      }
      else {   // else a badge to add
        app.models.userBadge.addBadge({vetId: vetId, badge: badge},function(err, badge) {
          if (err){
            console.log("Error with userBadge.addBadge for badge [" + badge + "] for vetId[" + vetId + "]", err);
            cb(err,null);
          }
          else {
            //console.log("Successful with userBadge.addBadge for badge [" + badge + "] for vetId[" + vetId + "]", badge);
            cb(null,badge);
          }
        });   // userBadge.addBadge

      }  // end. else a badge to add

  } // end. / else not an error with journalEvent.upsert
 });  // end journalEvent.upsert
};


  Mates.remoteMethod('getAllMates', {
    accepts: [
      {arg: 'Authorization', type: 'string', http: { source: 'header' }},
      { arg: 'data', type: 'object', http: { source: 'body' } }],
    returns: {arg: 'response', type: 'string'},
    http: {path: '/getAllMates', verb: 'post'}
  });



  Mates.remoteMethod('addMate', {
    accepts: [
      {arg: 'Authorization', type: 'string', http: { source: 'header' }},
      { arg: 'data', type: 'object', http: { source: 'body' } }],
    returns: {arg: 'response', type: 'string'},
    http: {path: '/addMate', verb: 'post'}
  });


  Mates.remoteMethod('removeMate', {
    accepts: [
      {arg: 'Authorization', type: 'string', http: { source: 'header' }},
      { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/removeMate', verb: 'post'}
    });




};
