'use strict';
var app = require('../../server/server');
const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');
var badgeConstants = require('va-grit-microservices/constants/badgeConstants');
var journalEventConstants = require('va-grit-microservices/constants/journalEventConstants');
var surveyUtil = require('./surveyUtil');

var request = require('request-json');
var constants = require('../constants').constants(app);
const riskServer = constants.cognitive.riskServer;
const  toneAnalysisServer = constants.cognitive.blueTasServer + "tone/?="

module.exports = function(answer) {

answer.submitSurvey = function(header, request, cb) {
  console.log("****Start of answer.submitSurvey.  header: ", JSON.stringify(header));
  console.log("****Start of answer.submitSurvey.  request body: ", JSON.stringify(request));
  var response = responseBuilder.createResponseObject();
  var errorMessage;
  var vetId;

  var surveyType;
  var currentSurvey;
  var answers;

  var userSurveySession;    //  A DB Document. A vetId can have only one userSurveySession
  var currentSession;       // Current Session in the list of Baseline sessions in the userSurveySession
  var answeredQuestionSets; // In the session is the answeredQuestionSets

  var questionSetId;
  var surveyType;
  var abortSurveySession;

  var reassessment = false;
  var additionalQuestions = false;

  var version = 1;  // For now we are dealing with version 1 of the question set.  A later release can handle different versions.

  var metrics;

  // Step 1.  Determine the vetId from the token
  VaGrit.getUserFromToken(app, header, function (err, userId) {
    if (userId === undefined || userId === null){
      responseBuilder.setInvalidTokenStatus(response,header);
      cb(null,response);
    }
    else { // else was able to determine a userid from the token
      console.log(' VaGrit.getUserFromToken userId: ', userId);
      vetId = userId;

      // Step 2. Determine and validate the request
      if (request.questionSetType === undefined || request.questionSetType === null) {
        surveyType = surveyUtil.SurveyTypes.BASELINE;
        console.log("questionSetType was not defined. Defaulting to BASELINE for vetId:" + vetId)
      }
      else {
        surveyType = request.questionSetType;
      }

      abortSurveySession = request.abortSurveySession !== undefined && request.abortSurveySession !== null &&
                            (typeof(request.abortSurveySession) === "boolean") && request.abortSurveySession;
      
      if (abortSurveySession) {
        //Add to auditEvents - Do not wait
        app.models.auditEvent.addEvent({
          vetId: userId.toString(),
          eventType: app.models.auditEvent.eventTypes.SURVEY_ABORTED,
          detail:  (surveyType != null && surveyType !== undefined && surveyType !== surveyType) ? {surveyType: surveyType} : {}
        });
      }

      if (request.questionSetId === undefined || request.questionSetId === null) {
        errorMessage = "questionSetId is a required field";
        console.log(errorMessage,request);
        responseBuilder.setInvalidRequestStatus(response,errorMessage);
        cb(null,response);
      }
      else if (!abortSurveySession && (request.answers === undefined || request.answers === null || request.answers.length < 1)) {
        errorMessage = "answers is a required field when the abortSurveySession is false";
        console.log(errorMessage,request);
        responseBuilder.setInvalidRequestStatus(response,errorMessage);
        cb(null,response);
      }
      else if (surveyType !== surveyUtil.SurveyTypes.BASELINE &&
               surveyType !== surveyUtil.SurveyTypes.MOOD_FACES &&
               surveyType !== surveyUtil.SurveyTypes.MOOD_TEXT &&
               surveyType !== surveyUtil.SurveyTypes.LIFE_CHANGES) {
        errorMessage = "surveyType [" + surveyType + "] is invalid.";
        console.log(errorMessage);
        responseBuilder.setInvalidRequestStatus(response,errorMessage);
        cb(null,response);
      }
      else {  // request contains good data so far
        // Step 3. Find the Question Set for the survey
        findTheQuestionSets(surveyType, function(err,questionSets){
          if (err){
            errorMessage = "Error with findTheQuestionSets. Message: " + err.message;
            console.log(errorMessage,err);
            responseBuilder.setInvalidRequestStatus(response, errorMessage);
            cb(null,response);
          }
          else { // else not an error with findTheQuestionSets
            answers = request.answers;
            questionSetId = request.questionSetId;


            // Step 4. Validate the questionSetId. Make sure it is a valid questionSetId
            if (!validQuestionSet(surveyType,questionSetId,questionSets)){
              errorMessage = surveyType + " survey. questionSetId [" + questionSetId + "] is invalid";
              console.log(errorMessage);
              responseBuilder.setInvalidRequestStatus(response, errorMessage);
              cb(null,response);
            }
            else {  // else questionSetId is valid
              // Step 5. Find the user survey session for the vet id
              app.models.userSurveySession.find({where: {_id: vetId}}, function(err, userSurveySessions) {
                if (err){
                  errorMessage = "userSurveySession.find for vetId [" + vetId + "] resulted in an err" ;
                  console.log(errorMessage, err);
                  responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                  cb(null,response);
                }
                else { // else successful in the userSurveySession.find
                  // Step 6.  Validate an open survey session exists and the question set has not been answered.
                  errorMessage = validateOpenSurveySessionAndQSHasNotBeenAnswered(surveyType,userSurveySessions, questionSetId, vetId);
                  if (errorMessage !== null){
                    responseBuilder.setInvalidRequestStatus(response,errorMessage);
                    console.log(errorMessage);
                    cb(null,response);
                  }
                  else {  // else not an error with validateOpenSurveySessionAndQSHasNotBeenAnswered
                    userSurveySession = userSurveySessions[0];   // Doina a find with vet id, will result with only one hit
                    var sessions = userSurveySession.sessions;
                    currentSession = sessions[sessions.length-1];

                    if (surveyType === surveyUtil.SurveyTypes.BASELINE && sessions.length > 1){
                      reassessment = true;
                    }

                    // Step 7. Get the questions for the survey
                    app.models.surveyQuestions.find(function(err, surveys) {
                      if (err){
                        errorMessage = "surveyQuestions.find for surveyType resulted in an err"
                        console.log(errorMessage, err);
                        responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                        cb(null,response);
                      }
                      else {  // else not an error with surveyQuestions.find
                        if (surveys.length < 1) {
                          errorMessage = "There is not any survey questions. Something is wrong with the surveyQuestions database table";
                          console.log(errorMessage);
                          responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                          cb(null,response);
                        }
                        else {  // else found surveys
                          currentSurvey = surveys[0];
                          var metrics = new Array();
                          var textEntryMetrics = new Array();
                          // Step 8. Determine the metrics and textEntryMetrics
                          var errorMessage = determineMetrics(surveyType, currentSurvey.questions, answers, questionSetId, abortSurveySession,questionSets, metrics, textEntryMetrics);

                          if (errorMessage !== null){
                            console.log("error in determineMetrics:",errorMessage);
                            responseBuilder.setInvalidRequestStatus(response,errorMessage);
                            cb(null, response);
                          }
                          else {  // else not an error with determineMetrics
                            console.log("metrics", metrics);
                            console.log("textEntryMetrics", textEntryMetrics);

                            // Step 9. Determine the twitter handle, and tone analysis
                            handleTwitterHandleAndToneAnalysis(vetId,surveyType, request.answers,textEntryMetrics, function(err, toneAndTwitterResults){
                            if (err){
                              errorMessage = "handleTwitterHandleAndToneAnalysis resulted in an error" ;
                              console.log(errorMessage, err);
                              responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                              cb(null,response);
                            }
                            else {  // else not an error with handleTwitterHandleAndToneAnalysis
                              console.log("toneAndTwitterResults", toneAndTwitterResults);

                              // Step 10. Handle calling the scoring api and updating the flag for the vetId
                              handleScoring(vetId, surveyType, questionSetId,version,metrics,toneAndTwitterResults,reassessment,abortSurveySession, function(err,handleScoringResponse){
                                if (err){
                                  errorMessage = "There is an unexpected error with handleScoring:" + err.message;
                                  console.log(errorMessage,err);
                                  responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                  cb(null,response);
                                }
                                else { // not an error with handleScoring
                                  // Step 11. Update the UserSurveySession Record (not database rec) and determine if there are additional questions left for the survey
                                  additionalQuestions = updateUserSurveySession(surveyType, userSurveySession, currentSession, questionSets,questionSetId,answers,metrics,toneAndTwitterResults,handleScoringResponse, abortSurveySession);

                                  // Step 12. Determine if a survey is complete and an event and badge should be returned
                                  determineEventAndBadgeUser(vetId,surveyType,questionSetId, userSurveySession, reassessment, additionalQuestions,function(err,badgeAdded){
                                    if (err){
                                      errorMessage = "determineBadgeForUser resulted in an unexpected error";
                                      console.log(errorMessage, err);
                                      responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                      cb(null,response);
                                    }
                                    else {  // else not error with determineBadgeForuser
                                      // Step 13. Update the userSurveySession in the database
                                      app.models.userSurveySession.upsert(userSurveySession, function(err, sessions) {
                                        if (err){
                                          errorMessage = "userSurveySession.upsertSert for vetId [" + vetId + "] resulted in an err" ;
                                          console.log(errorMessage, err);
                                          responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                          cb(null,response);
                                        }
                                        else {
                                          // Step 14. Send the successful response back.
                                          responseBuilder.setSuccessStatus(response);
                                          response.payload.additionalQuestions = additionalQuestions;

                                          if (badgeAdded != null){
                                            response.badgeAward.push(badgeAdded);
                                          }

                                          console.log("**** answer.submitSurvey for vet id " + vetId + " Response", response);
                                          cb(null,response);
                                        }
                                      });   // upsert

                                    }  // else not error with determineBadgeForUser
                                  });   determineBadgeForUser

                                }  // else not an error with handleScoring  Step 11
                              }); // handleScoring

                            }  // else not an error with handleTwitterHandleAndToneAnalysis
                          });   // handleTwitterHandleAndToneAnalysis


                        } // else not an error with determineMetrics

                      }  // else found survey
                    }  // else not an error with the surveyQuestions.find
                  });  // survey.find

                }  // else not an error with validateOpenSurveySessionAndQSHasNotBeenAnswered
              }  // else not an error with the userSurveySession.find
            });  //  userSurveySession.find


          } // else questionSetId is valid
        }// else not an error with findTheQuestionSets
      }); // findtheQuestionSets


    } // request contains good data so far
  } // else (able to find a userid from the token)
});  // VaGrit.getUserFromToken
}  // Survey.answerQuestionSe


// ***********************************************************************
// Internal Functions
// **********************************************************************

function findTheQuestionSets(surveyType, cb){
  if (surveyType !== surveyUtil.SurveyTypes.BASELINE){
    cb(null,null);
  }
  else {
    app.models.surveyQuestionSets.find(function(err, questionSetsRecs) {
      if (err){
        errorMessage = "surveyQuestionSet.find resulted in an err" ;
        console.log(errorMessage, err);
        cb(err,null);
      }
      else {  // else not an error with questionSet.find
        if (questionSetsRecs.length < 0 || questionSetsRecs[0].questionSets === undefined ){
          errorMessage = "surveyQuestionSet.find resulted in a record without questionSets.  There is something wrong with the surveyQuestionSets database table" ;
          var err = new Object();
          err.message = errorMessage;
          console.log(errorMessage, err);
          cb(err,null);
        }
        else {
          cb(null,questionSetsRecs[0].questionSets);
        }
      } // else not an error with questionSet.find
    });  // questionSet.find

  }
}

function validQuestionSet(surveyType,questionSetId, questionSets){
  if (surveyType !== surveyUtil.SurveyTypes.BASELINE){
     // The other survey types that are not baseline do not use questionSets
     return true;
  }

  var found = false;

  for (var i=0; i < questionSets.length; i++){
     var questionSet = questionSets[i];
     if (questionSet.questionSetId === questionSetId){
        found = true;
        break;
      }
  }

  return found;
}

// *************************************************************************
// validateOpenSurveySessionAndQSHasNotBeenAnswered
// If a Baseline Survey, then it validates there is a survey session, the
// survey session is opened, and the question set has not been answeredQuestionSets.
// *************************************************************************
function validateOpenSurveySessionAndQSHasNotBeenAnswered(userSurveyType,userSurveySessions, questionSetId, vetId){
   if (userSurveyType !== surveyUtil.SurveyTypes.BASELINE){
      if (userSurveySessions === undefined || userSurveySessions === null || userSurveySessions.length < 1){
        return "There is not a userSurveySessions. The getQuestionSet for the first onboarding survey needs to be performed first";

      }
      else {
         return null;
      }
   }

   var message = null;
   console.log("userSurveySessions", userSurveySessions)
   if (userSurveySessions.length < 1){
      return "There is not a userSurveySession for " + vetId;
    }

   var userSurveySession = userSurveySessions[0];
   var sessions = userSurveySession.sessions;
   var currentSession = sessions[sessions.length-1];
   console.log("currentSession", currentSession);

   if (!currentSession.open) {
      return "Current survey session is closed.  There is not an active Baseline survey session for the submitSurvey ";
   }

   var answeredQuestionSets = currentSession.answeredQuestionSets;

   if (answeredQuestionSets.length < 0){
      // Nothing in the answeredQuestionSets, nothing to validate
      console.log("answeredQuestionSets is empty");
      return null;
   }

   for (var i=0; i < answeredQuestionSets.length; i++){
      var answeredQuestionSetId = answeredQuestionSets[i];
      if (answeredQuestionSetId === questionSetId){
         return "Question Set " + questionSetId + " has already been answered";
      }
   }

  return message;

}

function updateUserSurveySession(surveyType, userSurveySession, currentSession, questionSets,questionSetId,answers,metrics,toneAndTwitterResults,
                                 handleScoringResponse, abortSurveySession){

   var additionalQuestions = false;
   var date = new Date();

   var historyRec = new Object();
   historyRec.submitSurveyDate = date;
   historyRec.questionSetId = questionSetId;
   historyRec.answers = answers;
   historyRec.metrics = metrics;
   historyRec.abortSurveySession = abortSurveySession;

   if (toneAndTwitterResults.socialMedia !== undefined && toneAndTwitterResults.socialMedia !== null){
      historyRec.twitterHandles = toneAndTwitterResults.socialMedia.twitter;
   }

   if (surveyType === surveyUtil.SurveyTypes.BASELINE){

      currentSession.answeredQuestionSets.push(questionSetId);

      // Determine the next question set using the updated flags that is set by the scoring service
      var questionSet;

      if (abortSurveySession){
        currentSession.closeDate = date;
        currentSession.open = false;

      }
      else {  // else not an abortSurveySession
        questionSet = surveyUtil.determineNextQuestionSet(questionSets, handleScoringResponse.currentSessionFlagsFromRiskService, currentSession.answeredQuestionSets);

        if (questionSet !== null) {
           // more questions left for the Baselilne
           additionalQuestions = true;
        }  // if questionSet != null
        else {
           currentSession.closeDate = date;
           currentSession.open = false;
        }
      }  // end. else not an abortSurveySession

      historyRec.currentSessionFlagsToRiskService = handleScoringResponse.inputCurrentSessionFlags;
      historyRec.currentSessionFlagsFromRiskService = handleScoringResponse.currentSessionFlagsFromRiskService;
      currentSession.history.push(historyRec);
   }  // Survey is Baseline
   else if (surveyType === surveyUtil.SurveyTypes.LIFE_CHANGES) {
      userSurveySession.surveyLifeChangesCount = userSurveySession.surveyLifeChangesCount +1
      historyRec.toneAnalysis = toneAndTwitterResults.textEntryMetrics;
      userSurveySession.historyLifeChanges.push(historyRec);

   }
   else  if (surveyType === surveyUtil.SurveyTypes.MOOD_FACES) {
      userSurveySession.historyMoodFaces.push(historyRec);
   }
   else if (surveyType === surveyUtil.SurveyTypes.MOOD_TEXT) {
     historyRec.toneAnalysis = toneAndTwitterResults.textEntryMetrics;
     userSurveySession.historyMoodText.push(historyRec);
   }

  return additionalQuestions;
}


function determineMetrics(surveyType,questions, answers, questionSetId, abortSurveySession, questionSets, metrics, textEntryMetrics) {
  var tone = new Object;
  var errorMessage = null;
  var questionSet = null;
  var answeredQuestionIds = new Array();

  if (surveyType !== surveyUtil.SurveyTypes.BASELINE && abortSurveySession) {
    return null;
  }

  if (answers !== undefined && answers !== null && answers.length > 0) {
    for (var i=0; i<answers.length; i++){
      var answerRec = answers[i];
      //console.log("working with answer", answerRec);
      var questionType;
      var questionForAnswer = null;

      for (var j=0; j < questions.length; j++){
        var question = questions[j];
        if (question.questionId === answerRec.questionId){
          questionType = question.questionType;
          console.log("found the question", question);
          questionForAnswer = question;
          answeredQuestionIds.push(question.questionId);
          break;
        }
      } // for j

      if (questionForAnswer === null){
        errorMessage = "Unable to find the question for answer questionId [" + answerRec.questionId + "]. Data may be corrupt";
        console.log(errorMessage , answerRec);
        return errorMessage;
      }

      // Now get the metrics for the answers.. The metricRecs will be set
      errorMessage = handleMetricForDifferentQuestionTypes(questionForAnswer,answerRec, metrics, textEntryMetrics);

      if (errorMessage != null){
        return errorMessage;
      }

    }  // for i
  } // end. if (answers !== undefined && answers !== null && answers.length > 0)

  if (abortSurveySession){
    errorMessage = handleMetricForAbortSurveySession(questionSetId, questionSets, questions, answeredQuestionIds, metrics);
    if (errorMessage !== null){
      return errorMessage;
    }
  }

  return null;
}

function handleMetricForDifferentQuestionTypes(questionForAnswer,answerRec, metricRecs, textEntryMetrics){
   //console.log("Entering handleMetricForDifferentQuestionType. questionForAnswer", questionForAnswer);
   var metricRec = new Object();
   var questionType = questionForAnswer.questionType;
   console.log("questionType", questionType);

   if (questionType === surveyUtil.QuestionTypes.SINGLE_SELECT || questionType === surveyUtil.QuestionTypes.LIKERT ||
       questionType === surveyUtil.QuestionTypes.WONG_BAKER) {
      // Find the metric for the single answer
      var errorMessage  = findMetricInQuestionSingleOptions(questionForAnswer, answerRec, metricRec);
      if (errorMessage  !== null){
         return errorMessage;
      }
      else {
         metricRecs.push(metricRec);
      }
   }
   else if (questionType === surveyUtil.QuestionTypes.MULTI_SELECT ||
            questionType === surveyUtil.QuestionTypes.MULTI_SELECT_OTHER){
      // Find the metric for multiple answers
      var multiMetricRecs = new Array();
      var errorMessage  = findMetricInQuestionMultiOptions(questionForAnswer, answerRec, multiMetricRecs );
      if (errorMessage  !== null){
         return errorMessage;
      }

      for (var i=0; i < multiMetricRecs.length; i++){
         metricRec = multiMetricRecs[i];
         metricRecs.push(metricRec);
      }

      if (questionType === surveyUtil.QuestionTypes.MULTI_SELECT_OTHER){
         // find the text for multiSelectOther if there is one. If so, textEntryMetrics will be set
         // with the text metric and raw text.
         if (answerRec.refused === true){
            return null;
         }

         errorMessage = findTextForMultiSelectOther(answerRec, questionForAnswer, textEntryMetrics);
         return errorMessage;
      }

      return null;

   }    // else if questionType == "MULTISELECT" || MULTI_SELECT_OTHER
   else if (questionType === surveyUtil.QuestionTypes.NUMERICAL){
     var errorMessage  = findMetricInQuestionNumerical(questionForAnswer, answerRec, metricRec);
     if (errorMessage  !== null){
        return errorMessage;
     }
     else {
        metricRecs.push(metricRec);
     }
   }
   else if (questionType === surveyUtil.QuestionTypes.TEXT){

     if (answerRec.refused === true){
       metricRec.metricName = questionForAnswer.content.defaultMetricName;
       metricRec.value = questionForAnswer.content.defaultValue;
       metricRecs.push(metricRec);
     }
     else {

        if (answerRec.text === undefined || answerRec.text === null || answerRec.text.length < 1){
           return "text is required for questionId [" + questionForAnswer.questionId + "], which has a questionType of TEXT";
        }

        var textEntryMetricRec = new Object();
        textEntryMetricRec.metricName = questionForAnswer.content.options[0].answerMetricName;
        textEntryMetricRec.rawText = answerRec.text;
        textEntryMetrics.push(textEntryMetricRec);
    }

   }

   return null;
}

function findMetricInQuestionSingleOptions(questionForAnswer, answerRec, metricRec){
   //console.log("findMetricsInQuestionSingleOptions for question:", questionForAnswer);
   //console.log("findMetricsInQuestionSingleOptions for answerRec:", answerRec);
   var errorMessage;

   if (questionForAnswer.content === undefined || questionForAnswer.content === null){
      errorMessage = "questionId [" + questionForAnswer.questionId + "] question data appears to be corrupt. Please check the database";
      console.log(errorMessage,questionForAnswer);
      return errorMessage;
   }

   if (answerRec.refused === true){
     metricRec.metricName = questionForAnswer.content.defaultMetricName;
     metricRec.value = questionForAnswer.content.defaultValue;
     return null;
   }

   var found = false;

   if (answerRec.answerIds === undefined || answerRec.answerIds === null || answerRec.answerIds.length < 1){
      errorMessage = "refused is false, thus answerIds is a required field for questionId " + questionForAnswer.questionId;
      console.log(errorMessage,answerRec);
      return errorMessage;
   }

   // Take the first answer
   var answerId = answerRec.answerIds[0];

   if (questionForAnswer.content.options === undefined || questionForAnswer.content.options === null ||
       questionForAnswer.content.options.length < 1){
      errorMessage = "questionId [" + questionForAnswer.questionId + "] question option data appears to be corrupt. Please check the database";
      console.log(errorMessage,questionForAnswer);
      return errorMessage;
   }

   metricRec.metricName = questionForAnswer.content.options[0].answerMetricName;
   var options = questionForAnswer.content.options;

   for (var i=0; i < options.length; i++){
      var option = options[i];

      if (option.answerId === answerId){
         var strVal = option.answerId;
         metricRec.value = Number(strVal);
         //console.log("Found answer. metricRec:", metricRec);
         found = true;
         break;
       }
    }

    if (!found){
       errorMessage = "Could not find the answerId [" + answerId + "] in the question.options. The request or survey question data might be corrupt";
       console.log(errorMessage + ". answerRec:",answerRec);
       console.log(errorMessage + ". questionForAnswer:", questionForAnswer);
       return errorMessage;
    }

    // All is good. return a null errorMessage.  The metricRec is filled out
    return null;
}


function findMetricInQuestionMultiOptions(questionForAnswer, answerRec, metricRecs){
   //console.log("findMetricsInQuestionMultiOptions for questionOptions:", questionOptions);
   //console.log("!!!!!findMetricInQuestionMultiOptions for answerRec", answerRec)
   var errorMessage;
   var metricRec = null;

   if (questionForAnswer.content === undefined || questionForAnswer.content === null){
      errorMessage = "questionId [" + questionForAnswer.questionId + "] question data appears to be corrupt. Please check the database";
      console.log(errorMessage,questionForAnswer);
      return errorMessage;
   }

   var questionContent = questionForAnswer.content;

   if (answerRec.refused === true){
      metricRec = new Object();
      metricRec.metricName = questionContent.defaultMetricName;
      metricRec.value = questionContent.defaultValue;
      metricRecs.push(metricRec);
      return null;
   }

   if (answerRec.noneOfTheAbove === true){
     metricRec = new Object();
     if (questionContent.noneOfTheAbove === undefined || questionContent.noneOfTheAbove === null){
        errorMessage = "questionType " + questionForAnswer.questionType + " for questionId [" + questionForAnswer.questionId +
           "] does not have none of the above.  The question data may be corrupt";
        console.log(errorMessage, questionforAnswer);
        return errorMessage;
     }

     metricRec.metricName = questionContent.noneOfTheAbove.answerMetricName;
     metricRec.value = questionContent.noneOfTheAbove.value;
     metricRecs.push(metricRec);

     // All is good, return a null error message.  The noneOfTheAbove will be placed in the metricRecs
     return null;
   }

   if (answerRec.answerIds === undefined || answerRec.answerIds === null || answerRec.answerIds.length < 1){
      errorMessage = "noneOfTheAbove and refused are false. answerIds is a required field for questionId:" + questionForAnswer.questionId;
      console.log(errorMessage,answerRec);
      return errorMessage;
   }

   if (questionForAnswer.content.options === undefined || questionForAnswer.content.options === null ||
       questionForAnswer.content.options.length < 1){
      errorMessage = "questionId [" + questionForAnswer.questionId + "] question.content.option data appears to be corrupt. Please check the question in the database";
      console.log(errorMessage,questionForAnswer);
      return errorMessage;
   }

   var answerIds = answerRec.answerIds;
   for (var i=0; i < questionContent.options.length; i++){
      var option = questionContent.options[i];

      metricRec = new Object();
      metricRec.metricName = option.answerMetricName;
      metricRec.value = 0;

      for (var k = 0; k < answerIds.length; k++){
         if (answerIds[k] === option.answerId) {
            metricRec.value = option.metricValue;
            break;
         }
      } // for k

      console.log("metricRec", metricRec);
      metricRecs.push(metricRec);

    }  // for i

    // All is good. return a null error message.  All the answers will have a metric value of 0 for not selected, or the metric value.
    return null;
}


function findMetricInQuestionNumerical(questionForAnswer, answerRec, metricRec){
   //console.log("findMetricsInQuestionSingleOptions for question:", questionForAnswer);
   //console.log("findMetricsInQuestionSingleOptions for answerRec:", answerRec);
   var errorMessage;

   if (questionForAnswer.content === undefined || questionForAnswer.content === null){
      errorMessage = "questionId [" + questionForAnswer.questionId + "] question.content appears to be corrupt. Please check the database";
      console.log(errorMessage,questionForAnswer);
      return errorMessage;
   }

   if (answerRec.refused === true){
     metricRec.metricName = questionForAnswer.content.defaultMetricName;
     metricRec.value = questionForAnswer.content.defaultValue;
     return null;
   }

   if (questionForAnswer.content.range === undefined || questionForAnswer.content.range === null){
      errorMessage = "questionId [" + questionForAnswer.questionId + "] question.content.range appears to be corrupt. Please check the database";
      console.log(errorMessage,questionForAnswer);
      return errorMessage;
   }

   metricRec.metricName =  questionForAnswer.content.range.answerMetricName;
   metricRec.value =  answerRec.numericalAnswer;

   // All is good. return a null errorMessage.  The metricRec is filled out
   return null;
}

function findTextForMultiSelectOther(answerRec, questionForAnswer, textEntryMetrics){
   var checkText = false;

   if (answerRec.noneOfTheAbove !== undefined && answerRec.noneOfTheAbove === true){
      checkText = false;
   }
   else {
      // see if there is an answer that requires a text
      for (var i=0; i < answerRec.answerIds.length; i++){
         var answerId = answerRec.answerIds[i];
         for (var j=0; j < questionForAnswer.content.options.length; j++){
            if ( (questionForAnswer.content.options[j].answerId === answerId) &&
                 (questionForAnswer.content.options[j].askOther === true)){
               checkText = true;
               break;
            }
         }  // for j
         if (checkText){
           break;
         }
      }    // for i
   }  //else

   if (checkText){
      if (answerRec.other === undefined || answerRec.other === null){
         return "other field is required for questionId [" + questionForAnswer.questionId + "] because an option was selected that required text";
      }

      var textEntryMetricRec = new Object();
      textEntryMetricRec.metricName = questionForAnswer.content.followup.answerMetricName;
      textEntryMetricRec.rawText = answerRec.other;
      textEntryMetrics.push(textEntryMetricRec);
   }  // if checkText

   return null;

}

function handleMetricForAbortSurveySession(questionSetId, questionSets, questions, answeredQuestionIds, metrics){
  var useQuestionSet = null;

  for (var i=0; i < questionSets.length; i++){
    if (questionSets[i].questionSetId === questionSetId){
      useQuestionSet = questionSets[i];
      break;
    }
  }

  if (useQuestionSet === null) {
    return "Unable to find questionSet " + questionSetId + " in determineMetrics";
  }

  // Default any unanswered questions
  console.log("!!!!!useQuestionSet",useQuestionSet);
  for (var i=0; i < useQuestionSet.questionSet.length; i++) {
    var checkQuestionSetId = useQuestionSet.questionSet[i];


    if (!answeredQuestionIds.includes(checkQuestionSetId)){
      answeredQuestionIds.push(checkQuestionSetId);
      for (var j=0; j < questions.length; j++) {
        if (questions[j].questionId === checkQuestionSetId){
          var metricRec = new Object();
          metricRec.metricName = questions[j].content.defaultMetricName;
          metricRec.value = questions[j].content.defaultValue;
          metrics.push(metricRec);
          break;
        }
      }  // end. for (var j ...)

    } //  end. if (!answeredQuestionIds.contains(checkQuestionSetId)

  }  // end. for (var i... )

  console.log("metrics", metrics);
  return null;

}

function handleScoring(vetId, surveyType, questionSetId,version,metrics, toneAndTwitterResults,reassessment,abortSurveySession,cb){
  /*
  if (abortSurveySession && surveyType !== surveyUtil.SurveyTypes.BASELINE){
    console.log("abortSurveySession is true and survey is not BASELINE. Do not invoke the risk service");
    cb(null,null);
  }
  else {  // else continue with score api
    */
    
    var currentSessionFlagsToRiskService = new Array();
    console.log("metricsRec to send to score api", metrics);

    determineCurrentSessionFlags(vetId,surveyType, function(err,currentSessionFlags){
      if (err){
        console.log("An error with determineCurrentSessionFlags",err);
        cb(err,null);
      }
      else { // else not an error with determineCurrentSessionFlags
        var scoreRequest = new Object();
        scoreRequest.vetId = vetId;

        scoreRequest.surveyType = surveyType;
        scoreRequest.abortSurveySession = abortSurveySession;

        if (currentSessionFlags !== null){
          // Clean the data that is from the database with just the flagName and value.
          for (var i=0; i < currentSessionFlags.length; i++ ){
            var flagRec = new Object();
            flagRec.flagName = currentSessionFlags[i].flagName;
            flagRec.value = currentSessionFlags[i].value;
            currentSessionFlagsToRiskService.push(flagRec);
          }

        }
        scoreRequest.currentSessionFlags = currentSessionFlagsToRiskService;

        if (surveyType === surveyUtil.SurveyTypes.BASELINE){
          scoreRequest.questionSetId = questionSetId;
          scoreRequest.version = version;
        }
        scoreRequest.metrics = metrics;

        if (toneAndTwitterResults !== null) {
          if (toneAndTwitterResults.socialMedia !== null){
            scoreRequest.socialMedia = toneAndTwitterResults.socialMedia;
          }

          if (toneAndTwitterResults.textEntryMetrics !== undefined && toneAndTwitterResults.textEntryMetrics !== null){
            scoreRequest.textEntryMetrics = toneAndTwitterResults.textEntryMetrics;
          }
        }

        // Invoke the risk  services..
        console.log("Risk Service request:", scoreRequest);

        var errorMessage;
        var flag = new Object();
        flag.vetId = vetId;
        flag.currentSessionFlags = new Array();
        flag.nextSessionFlags = new Array();

       var riskServerClient = request.createClient(riskServer);

        riskServerClient.post('score/', scoreRequest, function(err,res,riskServerResponse) {
          // With the risk-service, you don't check for err, but rather that the riskServerResponse exist
          // and the currentSessionflags exist.  If you check err, and there really is something not working with the
          // the risk service you get a parse error, because the err object exists, but it is like empty
          // and cannot be parsed.

          console.log("--------riskServerClient.post. err:",err);
          //console.log("--------riskServerClient.post. res", res);   // Gives too much data.
          console.log("--------riskServerClient.post. riskServerResponse", riskServerResponse);
          if (riskServerResponse === undefined || riskServerResponse === null){
            var err = new Object();
            err.message = "Risk Service, /score api did not return a response in the third field. The risk-service may not be up."
            console.log(err.message);
            cb(err,null);
          }
          else if (riskServerResponse.currentSessionFlags === undefined || riskServerResponse.currentSessionFlags === null){
            var err = new Object();
            err.message = "Risk Service, /score api did not return a currentSessionFlags field in the riskServerResponse. Check the logs for detail for vetId" + vetId;
            console.log(err.message, riskServerResponse);
            cb(err,null);
          }
          else {  // else a riskServerResponse.currentSessionFlag
            console.log("currentSessionFlags returned from riskService:", riskServerResponse.currentSessionFlags);
            flag.currentSessionFlags = riskServerResponse.currentSessionFlags;

            app.models.flag.upsert(flag, function(err, sessions) {
              if (err){
                var errorMessage = "flag.upsertSert for vetId [" + vetId + "] resulted in an err" ;
                console.log(errorMessage, err);
                cb(err,null);
              }
              else {  // else not an error with flag.upsert
                //console.log("successful upsert of flag", flag);
                var response = new Object();
                response.inputCurrentSessionFlags = currentSessionFlagsToRiskService;
                response.currentSessionFlagsFromRiskService = flag.currentSessionFlags;
                console.log("response from handleScoring", response);
                cb(null,response);
              }   // end. else not an error with flag.upsert
            });  // end.   app.models.flag.upsert
          }     // end. else a riskServerResponse.currentSessionFlag

        });       // end. riskServerClient.post

      } // end. else not an error with determineCurrentSessionFlags
    });   // end. determineCurrentSessionFlags
  //} // end. else continue with score api

}


function fixCurrentSessionFlags(questionSetId, currentSessionFlags,flag){
  if (questionSetId === "BL_CUSTOM_1" || questionSetId === "BL_CUSTOM_2" ||
      questionSetId === "BL_CUSTOM_3" || questionSetId === "BL_CUSTOM_4") {
    var flagRec = new Object();
    flagRec.flagName = "afterBL";
    flagRec.value = true;
    flag.currentSessionFlags.push(flagRec);
  }
  else if (questionSetId === "RA_BN") {
    flag.currentSessionFlags = currentSessionFlags;
  }
  else if (questionSetId === "RA_EMP") {
    flag.currentSessionFlags = currentSessionFlags;

    var flagRec = new Object();

    flagRec.flagName = "askRAEmpMoUnemp";
    flagRec.value = true;
    flag.currentSessionFlags.push(flagRec);
  }
  else if (questionSetId === "RA_EMP_MO_UNEMP"){
    flag.currentSessionFlags = currentSessionFlags;
  }
  else if (questionSetId === "RA_HEA") {
    flag.currentSessionFlags = currentSessionFlags;
  }
  else if (questionSetId === "RA_MENT") {
    flag.currentSessionFlags = currentSessionFlags;
    var flagRec = new Object();
    flagRec.flagName = "askRAMentCond";
    flagRec.value = true;
    flag.currentSessionFlags.push(flagRec);
  }
  else if (questionSetId === "RA_MENT_COND"){
    flag.currentSessionFlags = currentSessionFlags;
  }
 else if (questionSetId === "RA_SOC"){
   flag.currentSessionFlags = currentSessionFlags;

   var flagRec = new Object();
   flagRec.flagName = "askRASocMoMaritalChg";
   flagRec.value = true;
   flag.currentSessionFlags.push(flagRec);
 }
 else if (questionSetId === "RA_SOC_MO_MARITAL_CHG"){
   flag.currentSessionFlags = currentSessionFlags;
 }

}


function handleTwitterHandleAndToneAnalysis(vetId, surveyType, answers, textEntryMetrics,cb){
    console.log("answers", answers);
    var text;
    var toneResults = new Object();
    toneResults.socialMedia = null;
    toneResults.textEntryMetric = null;

    //Get the different social handles
    app.models.handle.find({where:{vetId:vetId}}, function(err, handles){

      if (err !== undefined && err !== null) {
         errorMessage = "Error invoking handle.find for vetId " + vetId + ". Error [" + err.message + "]";
         console.log("Error invoking handle.find for vetId " + vetId + ". Error [" + err.message + "]", err);
         cb(err,null);
      }
      else {  // else not an error with handle.find
         console.log("handles for vetIdxxxxxx " + vetId, handles);
         if (handles !== undefined && handles.length > 0) {
           // For now swagger is only documented using "twitter", but it can handle other social medias later
            var socialMedia = new Object();
            socialMedia.twitter = new Array();
            for (var i=0; i < handles.length; i++){
               if (handles[i].socialMediaType === "TWITTER"){
                  socialMedia.twitter.push(handles[i].handle);
               }
            }
            toneResults.socialMedia = socialMedia;
         }   // if handles !== undefined ...

         if (surveyType !== surveyUtil.SurveyTypes.MOOD_TEXT &&
             surveyType !== surveyUtil.SurveyTypes.LIFE_CHANGES){
           cb(null,toneResults);
         }
         else if (textEntryMetrics.length < 1){
             cb(null,toneResults);
         }
         else {  // else there is text for tone analysis
            // Currently we only support one tone text per questionSet.
            // If we support more in the future, add a loop here.
            text = textEntryMetrics[0].rawText;

            console.log("text",text);
            var textEncoded = encodeURIComponent(text);
            console.log("textEncoded", textEncoded);

            var url = "BluetasAPI/api/v1/tone/?text=" + textEncoded;
            var toneAnalysisClient = request.createClient("http://169.62.10.213:9080");
            toneAnalysisClient.get(url, function(err,res,body) {
               if (err){
                  console.log("Error with blue tas tone analysis",err);
                  cb(err,null);
               }
               else {  // else successful toneAnalysisClient.get
                  console.log("!!!!!!BlueTas Tone Analysis. body ", body);
                  //console.log("****document_tone: ", body.document_tone);
                  //  console.log("****sentence_tone: ", body.sentences_tone);

                  textEntryMetrics[0].toneAnalysis = body;
                  toneResults.textEntryMetrics = textEntryMetrics;

                  cb(null,toneResults);
               }  // else successful toneAnalysisClient.get
            });   // toneAnalysisClient.get

         }  //else there is text for tone analysis

     }   // else not an error with handle.find
   });  // handle.find -
}

function determineCurrentSessionFlags(vetId,surveyType,cb){
  if (surveyType !== surveyUtil.SurveyTypes.BASELINE){
    cb(null,null);
  }
  else {
    app.models.flag.find({where : {vetId: vetId}},function(err, flags){
      if (err) {
        errorMessage = "flag.find for vetId [" + vetId + "] resulted in an error" ;
        console.log(errorMessage,err)
        cb(err,null);
      }
      else {  // else found the sessionFlagRecs
        var currentSessionFlags = new Array();

        if (flags !== null){
          console.log("!!!!!!!!!found the flags",flags);
          currentSessionFlags = flags[0].currentSessionFlags;
          console.log("currentSessionFlags", currentSessionFlags);
        }
        cb(null,currentSessionFlags);
      }
    });
  }

}

function determineEventAndBadgeUser(vetId,surveyType,questionSetId, userSurveySession,reassessment,additionalQuestions,cb){
  if (additionalQuestions){
     // The survey is not complete
     cb(null,null);
  }
  else {

   app.models.journalEvent.upsert({vetId: vetId, eventType: journalEventConstants.JournalEventTypes.SURVEY_COMPLETED,date:new Date(),detail:surveyType}, function(err, eventInstance) {
       if (err){
         console.log("An error with journalEvent.upsert", err);
         cb(err,null);

       }
       else {
         
         //Add to auditEvents - Do not wait
         app.models.auditEvent.addEvent({
           vetId: vetId.toString(),
           eventType: app.models.auditEvent.eventTypes.SURVEY_COMPLETE,
           detail: { surveyType: surveyType }
         });
         
         
         determineBadgeForUser(vetId,surveyType,questionSetId, userSurveySession,reassessment,additionalQuestions, function(err,badge){
           if (err){
             cb(err,null);
           }
           else {
             console.log("badge from determineBadgeForUser", badge);
             cb(null,badge);
           }
         });
       }
    });

  }

}

function determineBadgeForUser(vetId,surveyType,questionSetId, userSurveySession,reassessment,additionalQuestions,cb){

   var badge = null;
   var errorMessage;
   var checkForCountBadge =  false;
   var countBadge = null;

   if (additionalQuestions){
      // Only add a badge if there is not additonal questions left
      cb(null,null);
   }
   else { // else additionalQuestions is false


      if (surveyType !== surveyUtil.SurveyTypes.BASELINE){
         countBadge = updateCountAndDetermineBadgesForCount(surveyType,userSurveySession);
         if (countBadge === null){
            cb(null,null);
         }
         else {   // else a countBadge to add
            console.log("addBadge vetId:" + vetId + ", count badge is " + countBadge);
            app.models.userBadge.addBadge({vetId: vetId, badge: countBadge},function(err, responseFromAdd) {
               if (err){
                  console.log("Error with userBadge.addBadge for badge [" + countBadge + "] for vetId[" + vetId + "]", err);
                  cb(err,null);
               }
               else {
                  console.log("Successful with userBadge.addBadge for countBadge [" + countBadge + "] for vetId[" + vetId + "]");
                  cb(null,countBadge);
               }
            });   // userBadge.addBadge
         } // else a countBadge to add

      }   // if not baseline

      else {  // else baseline.

         app.models.questionSetBadge.find({where: {questionSetId: questionSetId}}, function(err, questionSetBadgeRecs) {
            if (err){
               var errorMessage = "questionSetBadge.find for questionSetId [" + questionSetId + "] resulted in an err" ;
               console.log(errorMessage, err);
               cb(err,null);
           }
           else {  // else not an error with questionSetBadge.find
              if (questionSetBadgeRecs === undefined || questionSetBadgeRecs === null || questionSetBadgeRecs.length < 1){
                 // No badge for this questions set
                 cb(null,null);
              }
              else { // else there is a badge for the questionSetId
                 var baselineBadge = questionSetBadgeRecs[0].badge;
                  console.log("found badge " + badge + " for questionSetBadge.find for questionSetId[" + questionSetId + "]. Record", questionSetBadgeRecs[0]);
                  if (baselineBadge === "COUNT"){
                     baselineBadge = updateCountAndDetermineBadgesForCount(surveyType,userSurveySession);
                  }

                  if (baselineBadge == null){
                     cb(null,null);
                  }
                  else {   // else there is a baseline badge to add
                     app.models.userBadge.addBadge({vetId: vetId, badge: baselineBadge},function(err, responseFromAdd) {
                        if (err){
                           console.log("Error with userBadge.addBadge for baselineBadge [" + baselineBadge + "] for vetId[" + vetId + "]", err);
                           cb(err,null);
                        }
                        else {
                           console.log("Successful with userBadge.addBadge for baselineBadge [" + baselineBadge + "] for vetId[" + vetId + "]");
                           cb(null,baselineBadge);
                        }
                     });   // userBadge.addBadge
                  }  // else there is a baselineBadge to add
              }  // else there is a badge for the questionSetId
           } // else not an error with questionSetBadge.find
         });

      }   // else baseline
  } // else additionalQuestions is false

}


function updateCountAndDetermineBadgesForCount(surveyType,userSurveySession){
   if (surveyType === surveyUtil.SurveyTypes.BASELINE ||
       surveyType === surveyUtil.SurveyTypes.LIFE_CHANGES){
      userSurveySession.surveyReassessmentCount = userSurveySession.surveyReassessmentCount +1;
      if (userSurveySession.surveyReassessmentCount === 1){
         return  badgeConstants.BadgeTypes.SURVEY_REASSESSMENT_1;
      }
      else if (userSurveySession.surveyReassessmentCount === 5){
         return badgeConstants.BadgeTypes.SURVEY_REASSESSMENT_5;
      }
      else if (userSurveySession.surveyReassessmentCount === 10){
         return badgeConstants.BadgeTypes.SURVEY_REASSESSMENT_10;
      }
      else if (userSurveySession.surveyReassessmentCount === 20){
         return badgeConstants.BadgeTypes.SURVEY_REASSESSMENT_20;
      }
      else if (userSurveySession.surveyReassessmentCount === 30){
         return badgeConstants.BadgeTypes.SURVEY_REASSESSMENT_30;
      }
      else if (userSurveySession.surveyReassessmentCount === 50){
         return badgeConstants.BadgeTypes.SURVEY_REASSESSMENT_50;
      }
   }
   else if (surveyType === surveyUtil.SurveyTypes.MOOD_FACES  ||
            surveyType === surveyUtil.SurveyTypes.MOOD_TEXT){
       userSurveySession.surveyMoodCount = userSurveySession.surveyMoodCount +1;

       if (userSurveySession.surveyMoodCount === 1){
          return badgeConstants.BadgeTypes.SURVEY_MOOD_1;
       }
       else if (userSurveySession.surveyMoodCount === 5){
          return badgeConstants.BadgeTypes.SURVEY_MOOD_5;
       }
       else if (userSurveySession.surveyMoodCount === 10){
          return badgeConstants.BadgeTypes.SURVEY_MOOD_10;
       }
       else if (userSurveySession.surveyMoodCount === 20){
          return badgeConstants.BadgeTypes.SURVEY_MOOD_20;
       }
       else if (userSurveySession.surveyMoodCount === 50){
          return badgeConstants.BadgeTypes.SURVEY_MOOD_50;
       }
       else if (userSurveySession.surveyMoodCount === 100){
          return badgeConstants.BadgeTypes.SURVEY_MOOD_100;
       }
   }


   return null;
}



answer.remoteMethod('submitSurvey', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/submitSurvey', verb: 'post'}
});


};
