'use strict';
var app = require('../../server/server');
const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');
var surveyUtil = require('./surveyUtil');

module.exports = function(question) {

var remotes = app.remotes();
    remotes.after('**', function (ctx, next) {
        if (ctx) {
          //console.log("ctx",ctx);
          if (ctx.result.response !== null) {
            ctx.result = ctx.result.response;
          }
        } else {
            console.log("ERROR. question.js. remotes.after. ctx is not set");
            next({payload: { },
                  status: {
                   statusType: responseBuilder.StatusTypes.UNEXPECTED_EXCEPTION,
                   errorMessage: "There was an error on the server. ctx is not set"
                 }
            });
        }
        next();
});

//  ****************************************************************
//   API getQuestionSet
// *****************************************************************
question.getQuestionSet = function(header, request, cb) {
  //console.log("****Start of question.getQuestionSet.  header: ", JSON.stringify(header));
  //console.log("****Start of question.getQuestionSet.  request  /body: ", JSON.stringify(request));

  var response = responseBuilder.createResponseObject();

  var surveyType;

  if (request.questionSetType !== undefined){
    surveyType = request.questionSetType;
  }
  else {
    surveyType = surveyUtil.SurveyTypes.BASELINE;
  }

  // **** Determine the vetId from the token
  VaGrit.getUserFromToken(app, header, function (err, userId) {
    if (userId === undefined || userId === null){
      responseBuilder.setInvalidTokenStatus(response,header);
      cb(null,response);
    }
    else { // else was able to determine a userid from the token

      if (surveyType === surveyUtil.SurveyTypes.BASELINE){
        handleBaselineSurvey(userId,request,response,function(err,response){
          cb(null,response);
        });
      }
      else { // else not a BASELINE survey
        handleNonBaselineSurvey(surveyType, response,function(err,response){
          cb(null,response);
        });
      } // end. else not a BASELINE survey

    } // else (able to find a userid from the token)
  });  // VaGrit.getUserFromToken

}  //  Function question.retrieveQuestionSet


// -------------------------------------------------------------------------
// Internal Functions
// -------------------------------------------------------------------------


// *************************************************************************
// handleBaselineSurvey.
// Handle the Baseline Survey.  The result will be the list of
// questions for the next question set for the baseline survey
// *************************************************************************
function handleBaselineSurvey(vetId,request,response,cb){
  var userSurveySession;  //  A DB Document. A vetId can have only one userSurveySession
  var sessions;           // List of sessions in the userSurveySession
  var currentSession;     // Current Session in the list of sessions

  var currentSessionFlags = new Array();
  var currentSurvey;

  var questionSet;

  var shouldContinue = true;
  var newSessionForBaselineFlag = false;
  var onboardingFlag = false;
  var updateFlag = true;  // Need to update flag
  var errorMessage;

  var newSessionRequest =    (request.newSession !== undefined) && (request.newSession !== null) &&
                             (typeof(request.newSession) === "boolean") && request.newSession;

  // Step 1. Find the user survey session for the vet id.  The userSurveySession will contain the
  //         the current survey session.
  app.models.userSurveySession.find({where: {_id: vetId}}, function(err, userSurveySessions) {
    if (err){
      errorMessage = "userSurveySession.find for vetId [" + vetId + "] resulted in an error." ;
      console.log(errorMessage, err);
      responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
      cb(null,response);
    }
    else { // else not an error with userSurveySession.find
      if (userSurveySessions.length < 1){
        // There is not a userSurveySession.  This indicates this is the first baseline survey of
        // the onboarding. The userSurveySession will need to be created.
        console.log("Did not find a userSurveySession for vetId " + vetId + ". This is the initial onboarding baseline.");
        userSurveySession = createUserSurveySessionObject(vetId);
        sessions = userSurveySession.sessions;
        currentSession = sessions[0];
        onboardingFlag = true;
      }
      else { // else found the user survey session.
        // There is a userSurveySession.  The survey session can be either open or closed
        userSurveySession = userSurveySessions[0];
        sessions = userSurveySession.sessions;
        currentSession = sessions[sessions.length-1];

        var sessionOpened = currentSession.open;

        if (sessionOpened){
          // The last survey is currently opened. The midtier will continue with the survey and ignore the newSession flag in the request.
          console.log("Survey session is still opened for vetId:" + vetId + ". There are still questions left.  The newSession Flag will be ignored");
        }
        else {  // else survey session is not opened
          if (newSessionRequest) {
            // The last survey session is closed and the request is to have a new session. Add a new survey session to the
            // userSurveySession
            console.log("session is closed and the newSession request is true. A new session will be created for the new baseline survey");
            userSurveySession = addNewSessionToUserSurveySession(userSurveySession, surveyUtil.SurveyTypes.BASELINE);
            sessions = userSurveySession.sessions;
            currentSession = sessions[sessions.length-1];
            newSessionForBaselineFlag = true;
          }
          else {   // else not a newSessionRequest
            // The last survey session is closed, but the newSession request is false.  Therefore, no questions
            // will be returned.
            errorMessage = "newSession flag is false and session is not opened. No questions will be returned";
            console.log(errorMessage);
            responseBuilder.setNoQuestionsAvailableStatus(response,errorMessage);
            cb(null,response);
            shouldContinue = false;
          }   // end. else not a newSessionRequest
        } // end. else survey session is not opened
      }  // end. else found the user survey session

      // Step 2. Should we continue?
      if (shouldContinue) {
        // Step 3. Find the currentSessionFlags that will be used for the predicate logic
        determineCurrentSessionFlags(vetId, onboardingFlag, newSessionForBaselineFlag, request, function(err,currentSessionFlags) {
          if (err) {
            // Most likely an error with the metadata field.
            errorMessage = err.message ;
            console.log("determineCurrentSessionFlags resulted in an error",err)
            responseBuilder.setInvalidRequestStatus(response,errorMessage);
            cb(null,response);
          }
          else {  // else no error with determineCurrentSessionFlags

            // Step 4. Retrieve the questions sets from the database
            app.models.surveyQuestionSets.find(function(err, questionSetsRecs) {
              if (err) {
                errorMessage = "question.getQuestionSet. questionSet.find for surveyType [" + surveyType + "] resulted in an err" ;
                console.log(errorMessage, err);
                responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                cb(null,response);
              }
              else {  // else not an err with questionSet.find
                if (questionSetsRecs.length < 1){
                  errorMessage = "Did not find any surveyQuestionSets. Something is wrong with the database surveyQuestionSets table";
                  console.log(errorMessage);
                  responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                  cb(null,response);
                }
                else {  // else found questionSets from the surveyQuestionSets.find
                  // Step 5. Determine the next question set for the survey
                  questionSet = surveyUtil.determineNextQuestionSet(questionSetsRecs[0].questionSets, currentSessionFlags, currentSession.answeredQuestionSets);
                  console.log("questionSetTo Use", questionSet)

                  if (questionSet === null){
                    // This is considered an error. Because the mobile app should received additionalQuestions= true and
                    // not invoked this getQuestionSet api
                    errorMessage = "Did not find any questionSets that passed the predicate test.";
                    console.log(errorMessage + ". currentSessionFlags:", currentSession.flags);
                    responseBuilder.setNoQuestionsAvailableStatus(response,errorMessage);
                    cb(null,response);
                  }
                  else { // else questionSet is null
                    // Step 6. Retrieve the survey questions from the database
                    app.models.surveyQuestions.find(function(err, surveys) {
                      if (err) {
                        errorMessage = "survey.find resulted in an err" ;
                        console.log(errorMessage, err);
                        responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                        cb(null,response);
                      }
                    else {  // else not an error with survey.find

                      if (surveys.length < 1){
                        errorMessage = "There is not any surveyQuestions. Something is wrong with the database surveyQuestions";
                        console.log(errorMessage);
                        responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                        cb(null,response);
                      }
                      else {  // else found survey
                        currentSurvey = surveys[0];

                        // Step 7. Generate the questions to be returned that is in the questionSet
                        var questions = generateQuestionList(currentSurvey.questions,questionSet.questionSet);

                        // Step 8. Update the currentSessionFlags to the database
                        updateFlagToDB(vetId, currentSessionFlags, function(err,updatedflag) {
                          if (err){
                            errorMessage("Unexpected error with updateFlagToDB");
                            console.log(errorMessage, err);
                            responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                            cb(null,response);
                          }
                          else { // else not an error with updateFlagToDB
                            // Step 9. Update the userSurveySession
                            app.models.userSurveySession.upsert(userSurveySession, function(err, sessions) {
                              if (err){
                                errorMessage = "question.getQuestionSet. userSurveySession.upsertSert for vetId [" + vetId + "] resulted in an err" ;
                                console.log(errorMessage, err);
                                responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                cb(null,response);
                              }
                              else {  // else not an error with userSurveySession.upsert
                                console.log("Successful upsert of userSurveySession");
                                responseBuilder.setSuccessStatus(response);
                                response.payload.questionSetId = questionSet.questionSetId;
                                response.payload.questionSetType = surveyUtil.SurveyTypes.BASELINE;
                                response.payload.version = questionSet.version;
                                response.payload.questions = questions;

                                // Step 10.  Return the response with the questions
                                console.log("**** Leaving question.getQuestionSet. For vet id " + vetId + " Response", response);
                                cb(null,response);
                              } // else not an error with userSurveySession.upsert
                            });  // userSurveySession.upsert

                          } // else not an error with updateFlagToDB
                        }); // updateFlagToDB

                      }  // else found survey
                    }  // else not an error with the survey.find
                  });  // survey.find

                } // surveyUtil.determineNextQuestion found a questionSet to use
                } // else found questionSets from the surveyQuestionSet.find
              }  // else not an error with questionSet.find
            });  // questionSet.find

          }  // end. else no error with determineCurrentSessionFlags
        });  // end. determineCurrentSessionFlags


      } // else shouldContinue
    }  // else not an error with the userSurveySession.find
  });  //  userSurveySession.find

}

// *************************************************************
// handleNonBaselineSurvey
// Handle a NonBaseline Survey : MOOD_TEXT, MOOD_FACES, LIFE_CHANGES
// It will determine the question for the survey.
// ***************************************************************
function handleNonBaselineSurvey(surveyType, response,cb) {
  var errorMessage;

  // Step 1. Retrieve the survey questions
  app.models.surveyQuestions.find(function(err, surveys) {
    if (err) {
      errorMessage = "surveyQuestions.find resulted in an err" ;
      console.log(errorMessage, err);
      responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
      cb(null,response);
    }
    else {  // else not an error with survey.find

      responseBuilder.setSuccessStatus(response);

      if (surveys.length < 1){
        errorMessage = "surveyQuestions.find resulted in zero results. Sometime is wrong with the surveyQuestions database table";
        console.log(errorMessage);
        responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
        cb(null,response);
      }
      else {  // else found surveys
        // Step 2. Retrieve the question meta data that tells the question for the non baseline survey
        app.models.questionMetaData.find(function(err, questionMetaDataRecs) {
          if (err) {
            errorMessage = "questionMetaData.find resulted in an err" ;
            console.log(errorMessage, err);
            responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
            cb(null,response);
          }
          else { // else not an error with questionMetaData.find
            // found the meta deta
            if (questionMetaDataRecs.length < 1){
              errorMessage = "There is not any questionMetaData. Something is wrong with the questionMetaData database table";
              console.log(errorMessage);
              responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
              cb(null,response);
            }
            else {
              // Step 3. Format and return the question
              var questionMetaData = questionMetaDataRecs[0];
              console.log("questionMetaData", questionMetaData);

              var questionSet = new Array();

              if (surveyType === surveyUtil.SurveyTypes.LIFE_CHANGES){
                questionSet.push(questionMetaData.lifeChangesQuestion);
              }
              else if (surveyType === surveyUtil.SurveyTypes.MOOD_FACES){
                questionSet.push(questionMetaData.moodScaleQuestion);
              }
              else {
                questionSet.push(questionMetaData.moodDescriptionQuestion)
              }
              response.payload.questionSetId = surveyType;
              response.payload.questionSetType = surveyType;
              response.payload.version = 1;  // Only one version for the non baseline survey
              response.payload.questions = generateQuestionList(surveys[0].questions,questionSet);
              cb(null,response);
            }
          }   // end. else not error with questionMetaData.find
        });  // end. questionMetaData.find

      }  // end. else found surveys
    }  // end. else not an error with the survey.find
  });  // end. survey.find

}


function determineCurrentSessionFlags(vetId, onboardingFlag, newSessionForBaselineFlag, request,cb) {
  var currentSessionFlags;
  if (onboardingFlag){
    currentSessionFlags = new Array(); // No flags are set
    cb(null,currentSessionFlags);
  }
  else if (newSessionForBaselineFlag){
    // New survey.  Get the nextSessionFlag from the metadata. It needs to be an array
    if (request.metadata === undefined || request.metadata === null) {
      var err = new Object();
      err.message = "A new baseline survey session. The required meta in the request is not defined.";
      console.log(err.message);
      cb(err,null);
    }
    else if (typeof request.metadata === "string"){
      if (request.metadata.trim() === "") {
        var err = new Object();
        err.message = "The required metadata cannot be an empty string";
        console.log(err.message,request.metadata);
        cb(err,null);
      }
      else { // else not an empty string
        currentSessionFlags = JSON.parse(request.metadata);

        if (currentSessionFlags === null){
          var err = new Object();
          err.message = "A new baseline survey session. The required meta in the request is a string. But parsing it out resulting in null";
          console.log(err.message,request.metadata);
          cb(err,null);
        }
        else if (Array.isArray(currentSessionFlags) === false) {
          var err = new Object();
          err.message = "A new baseline survey session. The required meta in the request is a string. But parsing the string out did not result in an array.";
          console.log(err.message,request.metadata);
          cb(err,null);
        }
        else {
          console.log("A valid meta data for new baseline survey session for vetId:" + vetId +". metadata:", currentSessionFlags);
          cb(null,currentSessionFlags);
        }
      }  // end. else not an empty string
    }  // end. else if (typeof request.metadata === "string")
    else {
      var err = new Object();
      err.message = "A new baseline survey session. The required meta in the request has a typeof " + (typeof request.metadata) + ". It needs to be a string.";
      console.log(err.message,request.metadata);
      cb(err,null);

    }

  }
  else {   // else continuing with the survey
    // retrieve it from the database
    app.models.flag.find({where : {vetId: vetId}},function(err, flags){
      if (err) {
        var err = new Object();
        err.message =  "flag.find for vetId [" + vetId + "] resulted in an error" ;
        console.log(err.message);
        cb(err,null);
      }
      else {  // else not an error with flag.find
        if (flags === null || flags.length < 1){
          console.log ("Continuing the survey.  And Could not find flag in the database for vetId " + vetId + ", setting currentSessionFlags to be an empty array");
          cb(null, new Array());
        }
        else {
          currentSessionFlags = flags[0].currentSessionFlags;
          console.log ("Continuing the survey and found the flag in the database for vetId " + vetId +
            ", currentSessionFlags use for the getQuestionSet:", currentSessionFlags);

          cb(null,currentSessionFlags);
        }

      }  // end. else not an error with flag.find
    });

  }  // end. else continuing with the survey


}



function generateQuestionList(questions, questionIds){
   //console.log("questions", questions);
   //console.log("questionIds", questionIds);
   var questionsToUse = new Array();
   var questionsToReturn = new Array();

   for (var i=0; i< questionIds.length; i++) {
      var questionSetQuestionId = questionIds[i];
      for (var j=0; j < questions.length; j++){
         var questionQuestionId= questions[j].questionId;
         if (questionSetQuestionId === questionQuestionId){
            questionsToUse.push(questions[j]);
            break;
         }
     }  // for j
   } // for i

   if (questionsToUse.length < 1){
      return null;
   }

   // Now create the question rec in the output format
   for (var i=0; i < questionsToUse.length; i++) {
      var questionToUse = questionsToUse[i];
      //console.log("questionToUse", questionToUse);
      var questionRec = new Object();
      var questionType = questionToUse.questionType;
      var content = new Object();

      questionRec.questionId = questionToUse.questionId;
      questionRec.questionType = questionType;
      questionRec.questionText = questionToUse.questionText;

      if (questionType === surveyUtil.QuestionTypes.SINGLE_SELECT ||
          questionType === surveyUtil.QuestionTypes.LIKERT ||
          questionType === surveyUtil.QuestionTypes.WONG_BAKER ||
          questionType === surveyUtil.QuestionTypes.MULTI_SELECT ||
          questionType === surveyUtil.QuestionTypes.MULTI_SELECT_OTHER){
         if (questionToUse.content !== undefined &&
             questionToUse.content !== null &&
             questionToUse.content.options !== undefined){
            content.options = questionToUse.content.options;
            var options = new Array();
            for (var j=0; j < questionToUse.content.options.length; j++){
               var option = new Object();
               var questionToUseOption = questionToUse.content.options[j];
               option.answerId = questionToUseOption.answerId;
               option.label = questionToUseOption.label;
               if (questionType === surveyUtil.QuestionTypes.MULTI_SELECT_OTHER) {
                  option.askOther = false;
                  if (questionToUseOption.askOther !== undefined &&
                       questionToUseOption.askOther === true)
                  option.askOther = true;
               }
               options.push(option);
            }

            if (questionType === surveyUtil.QuestionTypes.MULTI_SELECT_OTHER) {
                var other = new Object();
                if (questionToUse.content.followup !== undefined &&
                    questionToUse.content.followup !== null &&
                    questionToUse.content.followup.followupText !== undefined &&
                    questionToUse.content.followup.followupText !== null) {
                   other.label = questionToUse.content.followup.followupText;
                   content.other = other;
                }
                else {
                   console.log("questionToUseRecord has missing followup fields", questionToUse);
                   questionRec = null;
                }
            }

            if (questionType === surveyUtil.QuestionTypes.MULTI_SELECT ||
                questionType === surveyUtil.QuestionTypes.MULTI_SELECT_OTHER ){
                content.noneOfTheAbove = false;
                if (questionToUse.content.noneOfTheAbove !== undefined &&
                    questionToUse.content.noneOfTheAbove !== null &&
                    questionToUse.content.noneOfTheAbove.value !== undefined &&
                    questionToUse.content.noneOfTheAbove.value !== null) {
                   if (questionToUse.content.noneOfTheAbove.value === 1){
                      content.noneOfTheAbove = true;
                   }
                }

            }

            if (questionRec !== null){
               content.options = options;
               questionRec.content = content;
            }
          }
          else {
             console.log("questionToUseRecord is " + questionType + ",but have missing key fields for content", questionToUse);
             questionRec = null;
          }
      }
      else if (questionType === surveyUtil.QuestionTypes.NUMERICAL){
         if (questionToUse.content !== undefined &&
             questionToUse.content !== null &&
             questionToUse.content.range !== undefined) {
            var range = new Object();
            range.minimum =  questionToUse.content.range.minimum;
            range.maximum =  questionToUse.content.range.maximum;
            range.interval = questionToUse.content.range.stepInterval;
            content.range = range;
            questionRec.content =  content;
          }
          else {
             console.log("questionToUseRecord is NUMERICAL,but have missing key fields for contentNumeric", questionToUse);
             questionRec = null;
          }
      }
      else if (questionType === surveyUtil.QuestionTypes.TEXT){
         // Text does not have content
      }
      else {
         console.log("questionToUse contains an unsupported question Type " + questionType, questionToUse);
         questionRec = null;
      }

      if (questionRec !== null) {
         questionsToReturn.push(questionRec);
      }
      else {
         console.log("unable to use question ", questionToUse);
      }
   }  // for

   console.log("questions that will be returned", questionsToReturn);
   return questionsToReturn;

}



function updateFlagToDB(vetId, currentSessionFlags,cb){
  var flag = new Object();
  flag.vetId = vetId;
  flag.currentSessionFlags = currentSessionFlags;

  app.models.flag.upsert(flag,function(err, sessions) {
    if (err){
      var errorMessage = "flag upsertSert for vetId [" + vetId + "] resulted in an err" ;
      console.log(errorMessage, err);
      cb(err,null);
    }
    else {
      console.log("Successful upsert of flag", flag);
      cb(null,flag);
    }
  });

}


function createUserSurveySessionObject(vetId){
   // Starting a new user survey session. This means it is a Baseline Survey
   var surveySession = new Object();
   var sessionRec = createSessionObject(1, surveyUtil.SurveyTypes.BASELINE);
   surveySession.vetId = vetId;
   surveySession.surveyReassessmentCount = 0;
   surveySession.surveyMoodCount = 0;
   surveySession.surveyLifeChangesCount = 0;
   surveySession.sessions = new Array();
   surveySession.sessions.push(sessionRec);
   surveySession.historyMoodFaces = new Array();
   surveySession.historyMoodText = new Array();
   surveySession.historyLifeChanges = new Array();
   return surveySession;
 }

 function addNewSessionToUserSurveySession(userSurveySession, surveyType){
    var sessionId =  userSurveySession.sessions.length + 1;
    var sessionRec = createSessionObject(sessionId, surveyType);
    console.log("Adding a new session rec to sessions with surveyType of " + surveyType, sessionRec);
    userSurveySession.sessions.push(sessionRec);
    return userSurveySession;
  }

function createSessionObject(sessionId,surveyType){
   var session = new Object();
   session.sessionId = sessionId;
   session.open = true;
   session.answeredQuestionSets = new Array();
   session.surveyType = surveyType;
   session.history = new Array();

   var date = new Date();
   session.startDate = date;
   return session;
 }


// ******************************************************************
// Remote Method Definitions
// ******************************************************************
question.remoteMethod('getQuestionSet', {
   accepts: [
      {arg: 'Authorization', type: 'string', http: { source: 'header' }},
      { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/getQuestionSet', verb: 'post'}
});



};
