// Survey Utility used by question.js and answer.js
var SurveyTypes = {
  BASELINE : "BASELINE",
  MOOD_FACES: "MOOD_FACES",
  MOOD_TEXT: "MOOD_TEXT",
  LIFE_CHANGES: "LIFE_CHANGES"
};

var QuestionTypes = {
  MULTI_SELECT : "MULTI_SELECT",
  SINGLE_SELECT : "SINGLE_SELECT",
  LIKERT        : "LIKERT",
  NUMERICAL     : "NUMERICAL",
  MULTI_SELECT_OTHER : "MULTI_SELECT_OTHER",
  WONG_BAKER : "WONG_BAKER",
  TEXT : "TEXT"
};

// *************************************************************************
// determineNextQuestionSet
// Determines which question set is next using the predicate logic of
// the questionSet and the sessson flags
// *************************************************************************
function determineNextQuestionSet(inQuestionSets,userSurveySessionFlags,answeredQuestionSets){
  //console.log("function determineNextQuestionSet. questionSets ", questionSets);
  console.log("function determineNextQuestionSet. userSurveySessionFlags:", userSurveySessionFlags);
  console.log("answeredQuestionSets:", answeredQuestionSets);

  if (inQuestionSets === null){
    console.log("surveyUtils.determineNextQuestionSet.  inQuestionSets is null");
    return null;
  }

  var nofSets = inQuestionSets.length;

  if (nofSets === 0) {
    console.log("surveyUtils.determineNextQuestionSet. questionSets is empty");
    return null;
  }

  var questionSets = new Array();

  // Loop through the questions set and only considered the ones that have not been answered
  for (var i=0; i < inQuestionSets.length; i++){
    var questionSet = inQuestionSets[i];

    if (answeredQuestionSets === undefined || answeredQuestionSets === null ||
                                               answeredQuestionSets.length < 1){
      questionSets.push(questionSet);
    }
    else {
      if (!answeredQuestionSets.includes(questionSet.questionSetId)){
        questionSets.push(questionSet);
      }
    }
  }

  var invalidQuestionSetIds = new Array();
  var questionSetToReturn = null;

  var done = false;

  while (!done){
    var tryQuestionSet = findNextValidQuestionSet(questionSets, invalidQuestionSetIds);

    if (tryQuestionSet === null){
      done = true;
      console.log("Exhausted the list of questionSets. All are invalid");
    }
    else {
      // Check if this question set is valid
      if (validateQuestionSet(tryQuestionSet,userSurveySessionFlags)){
        // The question set is the one to return
        done = true;
        questionSetToReturn = tryQuestionSet;
        console.log("Found a questionSet to return", questionSetToReturn);
      }
      else{
        // The question set is not valid. Put it in the invalid list
        invalidQuestionSetIds.push(tryQuestionSet.questionSetId);
      }
    }

  }  // while !done

  return questionSetToReturn;

}

// *************************************************************************
// findNextValidQuestionSet
// Find the next question with the lowest priority number that is not
// in the invalidQuestionSetIds list
// ************************************************************************
function findNextValidQuestionSet(questionSets,invalidQuestionSetIds){
  //console.log("findNextValidQuestionSet, questionSet", questionSets);
  //console.log("findNextValidQuestionSet, invalidQuestionSetIds", invalidQuestionSetIds);
  if (questionSets.length === invalidQuestionSetIds.length){
    //All the questionSets are invalid
    return null;
  }

  var orderNum = 10000;  // initialize to a big number that won't be an order num
  var version = 0;
  var useIndex = -1;

  for (var i=0; i < questionSets.length; i++){
    var questionSet = questionSets[i];

    if (!invalidQuestionSetIds.includes(questionSet.questionSetId)){
      if (questionSet.orderNum < orderNum) {
        useIndex = i;
        orderNum = questionSet.orderNum;
      }
      else if (questionSet.orderNum === orderNum){
        if (questionSet.version > version){
          version = questionSet.version;
          useIndex = i;
        }
      }
    }
  }  // for

  if (useIndex === -1){
     // Nothing is valid
    return null;
  }

  return questionSets[useIndex];
}

// *************************************************************************
// validateQuestionSet
// Validate that the question set is valid using the predicate logic
// *************************************************************************
function validateQuestionSet(questionSet,userSurveySessionFlags){
  //console.log("validateQuestionSet for questionSet", questionSet);
  if (questionSet.predicate === undefined || questionSet.predicate === null){
    //console.log("questionSet does not have a predicate", questionSet);
    return true;
  }

  var valid = evaluatePredicate(questionSet.predicate, userSurveySessionFlags);

   return valid;
}

// *************************************************************************
// evaluatePredicate
// Evaulate if the predicate using the flags is valid
// *************************************************************************
function evaluatePredicate(predicate, flags) {
  //console.log("****evaluatePredicate. inPredicate", predicate);
  //console.log("****evaluatePredicate. flags", flags);

  if (predicate.value !== undefined && predicate.value != null) {
    var flagValue = determineFlagsHasLabel(predicate.value,flags);
    if (flagValue == null){
      // flags does not have the predicate value.
      return false;
    }
    else {
      // flags has the value. return the flag value
      return flagValue;
    }
  } else if (predicate.and !== undefined && predicate.and != null) {
    for (var i=0; i< predicate.and.length; i++) {
      if (evaluatePredicate(predicate.and[i], flags) == false) {
        return false
      }
    }
    return true
  } else if (predicate.or !== undefined && predicate.or != null) {
    for (var i=0; i < predicate.or.length; i++) {
      if (evaluatePredicate(predicate.or[i], flags)) {
        return true
      }
    }
    return false
  } else if (predicate.not !== undefined && predicate.not != null) {
    return !(evaluatePredicate(predicate.not, flags));
  } else {
    console.log("???predicate does not have (value, and, or, not). going to return false", predicate);
    return false
  }
}


// *************************************************************************
// determineFlagsHasLabel
// Determines if the flags has the label
// *************************************************************************
function determineFlagsHasLabel(label,flags){
  //console.log("determineFlagsHasLabel:" + label + ". flags",flags);
  for (var i=0; i<flags.length;i++){
    var flag = flags[i];
    if (flag.flagName === label){
      //console.log("found it, returning flag.value", flag.value);
      return flag.value;
    }
  }

  return null;
}


function consoleOutPredicate(predicate) {
  console.log("!!!!!!!!!outputting predicate",predicate);
  if (predicate.or !== undefined && predicate.or !== null){
    console.log("predicate.or.length: ", predicate.or.length);
    console.log("predicate.or: ", predicate.or);

    for (var j=0; j< predicate.or.length; j++){
       //console.log("predicate.or[" + j + "]", predicate.or[j]);
       consoleOutPredicate(predicate.or[j]);
    }
  }
  else if (predicate.and !== undefined && predicate.and !== null){
    console.log("predicate.and: ", predicate.and);
    for (var i=0; i < predicate.and.length; i++){
       consoleOutPredicate(predicate.and[i]);
    }
  }
  else if (predicate.not !== undefined && predicate.not !== null){
    console.log("predicate.not: ", predicate.not);
  }
  else if (predicate.value !== undefined){
    console.log("predicate.value: ", predicate.value);
  }
  else {
    console.log("predicate does not have and,not,value", predicate);
  }


}



module.exports.SurveyTypes = SurveyTypes;
module.exports.QuestionTypes = QuestionTypes;
module.exports.determineNextQuestionSet = determineNextQuestionSet;
