# VA-GRIT Microservices

This readme file will describe the set up for additional microservice environments to work with docker, docker-compose, mongodb, and the gateway.

The app is the survey data microservice for the docker-compose app.

# Current Microservices

Here are the current microservices in the project that follow the pattern described here.

- Vet
- Survey

## Requirements
- Docker
- Docker Compose

# Uses Loopback
The project is generated by [LoopBack](http://loopback.io) with basic User auth.

# Change the port in server/config.json
Give each microservice (and other components) its own port number.



<!-- NOT NEEDED - WILL WORK WITH AUTHENTICATION MODEL WHICH IS SAVED TO MONGODB
# Add the vet model

For authentication purposes, create the vet model, which should extend the Loopback Built-in User model.  The vet model on the vet microservice do not need to be duplicated in any way other than:
 - Name it the same name (vet) as model on vet service
 - Have it extend the LoopBack Built-in User model
 - Have it's datasource as the mongodb
 - Any custom functions from vet-service does not need to be duplicated here
 --->

# Configure for mongodb
```
npm install loopback-connector-mongodb --save
```

Add the vaGritMidtierMongoDb to datasources.json - this will be the same database the other microservices are using
```
# server/datasources.json

"vaGritMidtierMongoDb": {
  "host": "db",
  "port": 27017,
  "url": "",
  "database": "vaGritMidtier",
  "password": "",
  "name": "vaGritMidtierMongoDb",
  "user": "",
  "connector": "mongodb"
}
```
Configure Loopback Built-in and Custom models to use <b>vaGritMidtierMongoDb</b> dataSource
```
# server/model-config.json

"User": {
  "dataSource": "vaGritMidtierMongoDb"
},
"AccessToken": {
  "dataSource": "vaGritMidtierMongoDb",
  "public": false
},
"ACL": {
  "dataSource": "vaGritMidtierMongoDb",
  "public": false
},
"RoleMapping": {
  "dataSource": "vaGritMidtierMongoDb",
  "public": false,
  "options": {
    "strictObjectIDCoercion": true
  }
},
"Role": {
  "dataSource": "vaGritMidtierMongoDb",
  "public": false
},
"aCustomModel": {
  "dataSource": "vaGritMidtierMongoDb",
  "public": true
},
etc...
```

# Configure for Docker

```
# Dockerfile in root folder of project

FROM node:8.11.3-onbuild
WORKDIR /app
## nodemon will restart server on changes, also need to mount volume in docker-compose.yml
RUN npm install -g nodemon
COPY . /app
RUN npm install
# Expose same port you indicated in server/config.json
EXPOSE 5001
```
```
Also add to Docker-compose.yml in folder above project root folder
```

# Add APIs to Gateway
To add this microservice's API's to the gateway see gateway.config.yml:

```
../va-grit-midtier-gateway/config/gateway.config.yml
```

# Running with docker and docker-compose

This app is a microservice within a larger app. This project includes a Dockerfile in the root directory f this project that can be built and run on it's own via Docker CLI Commands, and together with other apps as defined in Docker-compose.yml in the folder above this project

## To Run all services via docker-compose
- Go to project root directory in console

### To Build, Run, and Observe in logs
```
docker-compose -p va-grit-midtier up
or
docker-compose -p va-grit-midtier up --force-recreate  --build  
```

### To Start Docker Compose project
Will start all containers in project
```
docker-compose -p va-grit-midtier start
```

### To Stop Docker Compose project
Will stop all containers in project
```
docker-compose -p va-grit-midtier stop
```

# To Run microservice alone on localhost for developing

This app includes a Docker file as is meant to work with docker-compose project.  

However, the project can be run from the root directory with:

```
npm start
```
- NOTE DEPENDENCY ISSUES COULD EXIST IF RUNNING LOCALLY AND NOT RUNNING IN THE DOCKER-COMPOSE APP, PARTICULARLY IF TRYING TO COMMUNICATE WITH OTHER APPS IN OR OUT OTHE DOCKER NETWORK AS CONNECTION CONFIGURATION OPTIONS WILL NO LONGER BE VALID. <b>DATASOURCE CONFIGURATIONS CAN BE CHANGED IN SERVER/DATASOURCES.JSON</b>
