'use strict';
var app = require('../../server/server');
const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');


module.exports = function(Badges) {

Badges.getBadgeTypes = function(request, cb) {
   console.log("****Start of Badges.getBadgeTypes.  request: ", JSON.stringify(request));
   var response = responseBuilder.createResponseObject();

   var highestAddOrder = 0;
   var errorMessage;

   if (request.highestAddOrder !== undefined && request.highestAddOrder !== null){
      console.log("Defaulting highestAddOrder to 0");
      highestAddOrder = request.highestAddOrder;
   }

   app.models.badges.find({where: {addOrder : {gt : highestAddOrder}}}, function(err, badgeRecords) {
      if (err) {
         errorMessage = "Error invoking badges.find with addOrder " + highestAddOrder + " Error [" + err.message + "]";
         console.log(errorMessage, err);
         responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
         cb(null,response);
      }
      else { // else not an error with badges.find
         if (badgeRecords === undefined || badgeRecords.length < 1){
            console.log("badges.find for highestAddOrder" + highestAddOrder + " did not find any badges");
            responseBuilder.setSuccessStatus(response);
            response.payload.badges = new Array();
            cb(null,response);
         }
         else {   // else there are badgeRecords
            //console.log("Successful badges.find. highestAddOrder " + highestAddOrder + " badgeInstance: ", badgeRecords);
            app.models.badgeBase64.find(function(err, base64Records) {
            if (err) {
               errorMessage = "Error invoking badgeBase64.find.  Error [" + err.message + "]";
               console.log(errorMessage, err);
               responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
               cb(null,response);
            }
            else { // else not an error with badgeBase64.find
               if (base64Records === undefined || base64Records === null || base64Records.length < 1){
                  errorMessage = "The badgeBase64 database documents are missing.";
                  console.log(errorMessage, err);
                  responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                  cb(null,response);
               }
               else { // else there are base64 records
                  var badges = new Array();

                  errorMessage = "";
                  var errorFlag = false;
                  for (var i=0; i< badgeRecords.length; i++) {
                     var  badgeRecord = badgeRecords[i];
                     var outBadge = new Object();
                     outBadge.name = badgeRecord.badgeId;
                     outBadge.addOrder = badgeRecord.addOrder;
                     outBadge.category = badgeRecord.category
                     outBadge.title = badgeRecord.title;
                     outBadge.description = badgeRecord.description;
                     outBadge.congratulatoryMessage = badgeRecord.congratulatoryMessage;

                     outBadge.awardedImage = findBase64(base64Records, badgeRecord.awardedSmallBase64);
                     if (outBadge.awardedImage === null){
                        errorMessage = errorMessage + "Badge " + outBadge.name +
                              " has missing awarded small Base64 Image for id [" + badgeRecord.awardedSmallBase64 + "].  ";
                        errorFlag = true;
                      }

                      outBadge.unawardedImage = findBase64(base64Records, badgeRecord.unawardedSmallBase64);
                      if (outBadge.unawardedImage === null){
                         errorMessage = errorMessage + "Badge " + outBadge.name +
                                " has missing unawarded small Base64 Image for id [" + badgeRecord.unawardedSmallBase64 + "].  ";
                         errorFlag = true;
                      }

                      outBadge.enlargedAwardedImage = findBase64(base64Records, badgeRecord.awardedLargeBase64);
                      if (outBadge.enlargedAwardedImage === null){
                         errorMessage = errorMessage + "Badge " + outBadge.name +
                                " has missing awarded large Base64 Image for id [" + badgeRecord.awardedLargeBase64 + "].";
                         errorFlag = true;
                      }

                      outBadge.enlargedUnawardedImage = findBase64(base64Records, badgeRecord.unawardedLargeBase64);
                      if (outBadge.enlargedUnawardedImage === null){
                         errorMessage = errorMessage + "Badge " + outBadge.name +
                                  " has missing unawarded large Base64 Image for id [" + badgeRecord.unawardedLargeBase64 + "].";
                         errorFlag = true;
                      }

                      if (!errorFlag){
                         badges.push(outBadge);
                      }
                  }  // for

                  if (!errorFlag){
                     responseBuilder.setSuccessStatus(response);
                     response.payload.badges = badges;
                     console.log("Successfully retrieved the badgeTypes");
                     for (var i=0; i < badges.length; i++){
                         console.log("returning badge " + badges[i].name);
                     }
                     cb(null,response);
                  }
                  else {
                     console.log(errorMessage, err);
                     responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                     cb(null,response);
                  }
              } // else there are base64 rcords
            }  // else not an error with badgeBase64.find

         });   // end of badgeBase64.find
       } // else there are badgeRecords

      }  // else not an error with badges.find
   });   // end of badges.find
}

function findBase64(base64Records, base64Id){
    var base64String = null;
    for (var i=0; i < base64Records.length; i++){
       var base64Record = base64Records[i];
       if (base64Record.badgeBase64Id === base64Id){
          base64String = base64Record.base64;
          break;
       }
    }

    if (base64String === null){
      console.log("Unable to find base64Id [" + base64Id + "] in the base64Records");
    }
    return base64String;
}

Badges.remoteMethod('getBadgeTypes', {
         accepts: { arg: 'data', type: 'object', http: { source: 'body' } },
         returns: {arg: 'response', type: 'string'},
         http: {path: '/getBadgeTypes', verb: 'post'}
});


};
