'use strict';
var app = require('../../server/server');
const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');
var journalEventConstants = require('va-grit-microservices/constants/journalEventConstants');
var JournalEventTypes = journalEventConstants.JournalEventTypes;
var request = require('request-json');
var constants = require('../constants').constants(app);
const riskServer = constants.cognitive.riskServer;

module.exports = function(Journal) {

Journal.getJournalEvents = function(header, request, cb) {
  console.log("****Start of getJournalEvents.  header: ", JSON.stringify(header));
  console.log("****Start of getJournalEvents.  request body: ", JSON.stringify(request));

  var response = responseBuilder.createResponseObject();
  var errorMessage = null;
  var vetId;
  var startDate;
  var endDate;

  VaGrit.getUserFromToken(app, header, function (err, userId) {
    if (userId === undefined || userId === null){
      responseBuilder.setInvalidTokenStatus(response,header);
      cb(null,response);
    }
    else { // else was able to determine a userid from the token
      console.log(' VaGrit.getUserFromToken userId: ', userId);

      if  (request.startDate !== undefined) {
        startDate = new Date(request.startDate);
      }
      else {
        // Default startDate to an old date
        startDate = new Date("2018-01-01");
      }

      if  (request.endDate !== undefined) {
        endDate = new Date(request.endDate);
      }
      else {
        // Default the endDate to today
        endDate = new Date();
      }

      console.log("startDate", startDate);
      console.log("endDate", endDate);


      if (isNaN(startDate.getTime())) {
        errorMessage = "startDate is an invalid date.  ";
      }

      if (isNaN(endDate.getTime())) {
        errorMessage = errorMessage + "endDate is an invalid date.  ";
      }

      if (errorMessage !== null){
        console.log(errorMessage);
        responseBuilder.setInvalidRequestStatus(response, errorMessage);
        cb(null,response);
      }
      else {  // else not an invalid request
        vetId = userId;

        app.models.journalEvent.find({where:{and: [{vetId: vetId}, {date: {between:[startDate,endDate]}} ] } }, function(err,journalEvents){
          if (err){
            errorMessage = "error with journalEvent.find using vetId in filter. for vetId:" + vetId;
            console.log(errorMessage,err);
            responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
            cb(null,response);
          }
          else {  // else not an error with journalEvent.find
            console.log("vetId [" + vetId + "]. startDate:" + startDate + ", .endDate: " + endDate + " .Resulted in journalEvents:", journalEvents);

            responseBuilder.setSuccessStatus(response);
            if (journalEvents.length > 0){
              response.payload.events = determineEvents(journalEvents);
              console.log("!!!!!getJournalEvents with resulted in journalEvents. response", response);

            }
            else {  // else did not find journalEvents
              response.payload.events = new Array();
              console.log("getJournalEvents for vetId " + vetId + " resulted in zero hits ", response);
            }
            cb(null,response);

          } // end. else not an error with journalEvent.find
        });

      } // end. else not an invalid request
    } // else was able to determine a userid from the token
  });

}

Journal.getJournalMoodScores = function(header, request, cb) {
  console.log("****Start of getJournalMoodScores.  header: ", JSON.stringify(header));

  var response = responseBuilder.createResponseObject();
  var errorMessage = null;

  // Determine the Vetid
  VaGrit.getUserFromToken(app, header, function (err, userId) {
    if (userId === undefined || userId === null){
      responseBuilder.setInvalidTokenStatus(response,header);
      cb(null,response);
    }
    else { // else was able to determine a userid from the token
      console.log(' VaGrit.getUserFromToken userId: ', userId);
      handleRiskServerForMoodScores(userId, function(err,scores) {
        if (err){
          console.log("handleRiskServerForMoodScores resulted in an err",err);
          responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
          cb(null,response);
        }
        else {
          responseBuilder.setSuccessStatus(response);
          response.payload.scores = scores;
          console.log("getJournalMoodScore. response:", response);
          cb(null,response);

        }
      });

    }
 });
}

function determineEvents(journalEvents){

  var events = new Array();
  if (journalEvents === null || journalEvents.length < 1){
     return events;
  }

  var squadMemberAddedEvents = new Array();
  var digitalAssistantEvents = new Array();
  var surveyCompletedEvents = new Array();
  var badgesAwardedEvents = new Array();
  var savedJobEvents = new Array();

  for (var i=0; i < journalEvents.length; i++){
    var journalEvent = journalEvents[i];
    var instanceRecord = new Object();
    instanceRecord.date = journalEvent.date;
    instanceRecord.detail = journalEvent.detail;

    if (journalEvent.eventType === JournalEventTypes.SQUAD_MEMBER_ADDED){
      squadMemberAddedEvents.push(instanceRecord)
    }
    else if (journalEvent.eventType === JournalEventTypes.DIGITAL_ASSISTANT){
      digitalAssistantEvents.push(instanceRecord)
    }
    else if (journalEvent.eventType === JournalEventTypes.SURVEY_COMPLETED){
      surveyCompletedEvents.push(instanceRecord)
    }
    else if (journalEvent.eventType === JournalEventTypes.BADGES_AWARDED){
      badgesAwardedEvents.push(instanceRecord)
    }
    else if (journalEvent.eventType === JournalEventTypes.SAVED_JOB){
      savedJobEvents.push(instanceRecord)
    }
  }


  var eventRecord;

  if (squadMemberAddedEvents.length > 0){
      eventRecord = new Object();
      eventRecord.eventType =  JournalEventTypes.SQUAD_MEMBER_ADDED;
      eventRecord.instances = squadMemberAddedEvents;
      events.push(eventRecord);
  }

  if (digitalAssistantEvents.length > 0){
      eventRecord = new Object();
      eventRecord.eventType =  JournalEventTypes.DIGITAL_ASSISTANT;
      eventRecord.instances = digitalAssistantEvents;
      events.push(eventRecord);
  }

  if (surveyCompletedEvents.length > 0){
      eventRecord = new Object();
      eventRecord.eventType =  JournalEventTypes.SURVEY_COMPLETED;
      eventRecord.instances = surveyCompletedEvents;
      events.push(eventRecord);
  }

  if (badgesAwardedEvents.length > 0){
      eventRecord = new Object();
      eventRecord.eventType =  JournalEventTypes.BADGES_AWARDED;
      eventRecord.instances = badgesAwardedEvents;
      events.push(eventRecord);
  }

  if (savedJobEvents.length > 0){
      eventRecord = new Object();
      eventRecord.eventType =  JournalEventTypes.SAVED_JOB
      eventRecord.instances = savedJobEvents;
      events.push(eventRecord);
  }

  return events;

}

function handleRiskServerForMoodScores(vetId, cb){
  // Invoke the riskServer, getMoodScores api.. for now mock setInterval(function () {
  var errorMessage;

  var scoreRequest = new Object();
  scoreRequest.vetId = vetId;


  var riskServerClient = request.createClient(riskServer);

   riskServerClient.post('getMoodScores/', scoreRequest, function(err,res,riskServerResponse) {
     if (err){
       errorMessage = "Error with riskServerClient.post for " + riskServer + ". err:";
       console.log(errorMessage,err);
       cb(err,null);
     }
     else {  // else not an error with riskServerClient.post
       if (riskServerResponse === undefined || riskServerResponse === null){
         var err = new Object();
         err.message = "Risk Service, /getMoodScores api did not return a response in the third field"
         console.log(err.message);
         cb(err,null);
       }
       else if (riskServerResponse.scores === undefined || riskServerResponse.scores === null){
         var err = new Object();
         err.message = "Risk Service, /getMoodScores api did not return in the response the scores array";
         console.log(err.message);
         cb(err,null);
      }
      else {
        console.log("riskServerResponse.scores", riskServerResponse.scores);
        var scores = flipTheScores(riskServerResponse.scores);
        cb(err,scores);
      }
    } // end. else not an error with riskServerClient.post
  });


}


function flipTheScores(riskScores){
   if (riskScores == null || riskScores.length < 1){
      return new Array();
   }

   var scores = new Array();
   var rec;
   for (var i=0; i < riskScores.length; i++){
     rec = new Object();
     rec.date = riskScores[i].date;
     rec.score = 1 - riskScores[i].score;
     scores.push(rec);
   }

   return scores;
}



Journal.remoteMethod('getJournalEvents', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/getJournalEvents', verb: 'post'}
});

Journal.remoteMethod('getJournalMoodScores', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/getJournalMoodScores', verb: 'post'}
});


};
