'use strict';
var app = require('../../server/server');
var constants = require('../constants').constants(app);
var mongoUrl = constants.mongo.url;
var mongoDbName = constants.mongo.dbName;
const sendgridApiKey = constants.sendgrid.apiKey;
const sgMail = require('@sendgrid/mail');

const VaGrit = require('va-grit-microservices');
var responseBuilder = require('va-grit-microservices/response/responseBuilder');
var badgeConstants = require('va-grit-microservices/constants/badgeConstants');

// Mongo Client.  Needed for the vet.deleteAccount api.
var MongoClient = require('mongodb').MongoClient;

var request = require('request-json');
var constants = require('../constants').constants(app);
const riskServer = constants.cognitive.riskServer;
var blueTasServer = constants.cognitive.blueTasServer;
var blueTasCollectionId = constants.cognitive.blueTasCollectionId;

var wemServerBaseUrl = constants.cognitive.wemServerBaseUrl;


module.exports = function(vet) {

  //Loopback wraps the response in a "response" object and we don't want to
    // return that, this removes the response we created from the "response" object
    // and only returns the response we created.
    // TODO: May need to adjust this so only works on custom remote methods we created.
      var remotes = app.remotes();
        remotes.after('**', function (ctx, next) {

            if (ctx) {
              if (ctx.result.response !== null) {
                ctx.result = ctx.result.response;
              }
            } else {
                console.log("ERROR. vet.js. remotes.after. ctx is not set");
                next({payload: { },
                      status: {
                       statusType: responseBuilder.StatusTypes.UNEXPECTED_EXCEPTION,
                       errorMessage: "There was an error on the server. ctx is not set"
                     }
                });
            }
            next();
        });

        /* TODO: HANDLE SERVER ERRORS AND RETURN A RESPONSE IN CORRECT FORMAT
    vet.afterRemoteError('exist', function(ctx, next) {
      console.log("afterRemoteError !!!!!!!!");
      var errorResponse = {payload: { },
                          status: {
                           statusType: "UNEXPECTED_ERROR",
                           errorMessage: "There was an error on the server."
                         }
                    };
      if (ctx.error !== null) {
        errorResponse.status.statusType = ctx.error.statusCode;
        errorResponse.status.errorMessage = ctx.error.message;
      }

      ctx.res.send(errorResponse);
      //next();
    });
    */

  // ******************************************
  // API function vet.signup
  // *****************************************
  vet.signup = function(request, cb) {

     var errorMessage = "";
     var response = responseBuilder.createResponseObject();

     if (request === undefined || request === null ||
          request.email === undefined || request.email === null || request.email === "" || typeof request.email !== "string" ||
          request.password === undefined || request.password === null || request.password === "" || typeof request.password !== "string" ||
          request.name === undefined || request.name === null || request.name === "" || typeof request.name !== "string") {
            errorMessage = "Missing Required fields or values";
            responseBuilder.setInvalidRequestStatus(response, errorMessage);
            cb(null, response);
            return;
    }
    
    //console.log("*** Start vet signup. Request: ", JSON.stringify(request));
     var emailId = request.email.toLowerCase();
     var signupEmail = request.email;
     var password = request.password;
     var name = request.name;
     var vet = app.models.vet;

    //TODO: Add user roles if helpful
    //var Role = app.models.Role;
    //var RoleMapping = app.models.RoleMapping;

    //TODO: Finalize vet Model and add more fields here if necessary

    vet.create({email: emailId, password: password, name: name, createDate: new Date().toISOString(), signupEmail: signupEmail},
    function(err, userInstance) {
      if (err) {
         console.log("Error. vet.create for user " + emailId + ". err: ", err);
         if (err.statusCode === 422 && err.message.includes("exist")){
           responseBuilder.setInvalidRequestStatus(response,"Email Id [" + emailId + "] already exists");
         }
         else{
            errorMessage = "Unexpected exception creating user using vet.create. Error [" + err.message + "]";
            responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
         }
         cb(null,response);
      }
      else {  // Not an error with vet.create
        if (userInstance === undefined || userInstance === null){
             errorMessage = "No error with vet.create for email " + emailId + ", but userInstance is not set";
             responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
             cb(null,response);
        }
        else {
           //console.log("Successfully created vet using vet.create. userInstance: ", userInstance);
           responseBuilder.setSuccessStatus(response);
           //response.payload.gritUserId = userInstance.id;
           
           
           //Add to auditEvents - Do not wait
           app.models.auditEvent.addEvent({
             vetId: userInstance.id.toString(),
             eventType: app.models.auditEvent.eventTypes.SIGN_UP,
             detail: {}
           });

           // Now login and get the token
           vet.login({email: emailId, password: password}, 'user',
           function(err, userInstance) {
             if (err) {
                console.log("Error with vet.login err: ", err.message);
                errorMessage = "Login Failed. Error [" + err.message + "]";
                responseBuilder.setLoginFailedStatus(response,errorMessage);
                err = null;
             }

             else if (userInstance !== undefined) {
                responseBuilder.setSuccessStatus(response);
                response.payload.token = userInstance.id;
             }
             
             //Check if email ends in ".test", if yes, don't send email (don't enable on prod)
             var checkForDotTestSubstring = (constants.testEmailExtension === signupEmail.substring(signupEmail.length-constants.testEmailExtension.length, signupEmail.length));
             
             if (!checkForDotTestSubstring || (checkForDotTestSubstring && process.env.NODE_ENV === constants.prodServerEnv)) {
               var emailBodyHtml = "<p>Hello " + name + ", thanks for creating a GRIT account! We can't wait to help with your work to build you best self." +
                                   "  Search for jobs, add members to your squad or get feedback from assessments today!</p>";
               var emailBodyText = "Hello " + name + ", thanks for creating a GRIT account! We can't wait to help with your work to build you best self." +
                                   "  Search for jobs, add members to your squad or get feedback from assessments today!";
               sgMail.setApiKey(sendgridApiKey);
                const msg = {
                  to: signupEmail,
                  from: constants.sendgrid.fromEmail,
                  subject: 'GRIT Account Signup',
                  text: emailBodyText,
                  html: emailBodyHtml,
                };
                sgMail.send(msg);
                
             } 

             cb(null,response);
           });  // end vet.login


        } // end else a userinstance with vet.create

      }  // end else not an error with vet.create

    });  // end vet.create

  }

  // ******************************************
  // API Function vet.signin
  // *****************************************
  vet.signin = function(request, cb) {

     var errorMessage = "";
     var response = responseBuilder.createResponseObject();

     if (request === undefined || request === null ||
          request.email === undefined || request.email === null || request.email === "" || typeof request.email !== "string" ||
          request.password === undefined || request.password === null || request.password === "" || typeof request.password !== "string" ){
           errorMessage = "Missing Required fields or values";
           responseBuilder.setInvalidRequestStatus(response, errorMessage);
           cb(null, response);
           return;
    }
    
    var emailId = request.email.toLowerCase();
    var password = request.password;
    var vet = app.models.vet;

    vet.login({email: emailId, password: password}, 'user',
    function(err, userInstance) {
      if (err) {
         console.log("Error with vet.login err: ", err.message);
         errorMessage = "Login Failed. Error [" + err.message + "]";
         responseBuilder.setLoginFailedStatus(response,errorMessage);
         cb(null,response);
      }

      else if (userInstance !== undefined) {
         responseBuilder.setSuccessStatus(response);
         //response.payload.gritUserId = userInstance.userId;
         response.payload.token = userInstance.id;

         // TODO. Determine why we can't get the name from the login. The
         // userInstance.user object is this native function, and not a json.
         // For now get it from the find
         vet.find({where: {email: emailId}},
         function(err, userInstance) {
            if (err) {
               errorMessage = "Error invoking vet.find. Error [" + err.message + "]";
               console.log("Error invoking vet.find err: ", err);
               responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
            }
            else {
               if (userInstance !== undefined && userInstance.length > 0) {
                  response.payload.name = userInstance[0].name;
               }
               else {
                  response.payload.name = "unknownFirstName";
               }


             }  //else
             cb(null, response);

         });   // end of vet.find

      }

    });  // end vet.login

  }

  // ******************************************
  // API Function vet.exist
  // *****************************************
    vet.exist = function(request, cb) {
       var errorMessage = "";
       var response = responseBuilder.createResponseObject();


       if (request === undefined || request === null ||
            request.email === undefined || request.email === null || request.email === "" || typeof request.email !== "string") {
        errorMessage = "Missing Required fields or values";
        responseBuilder.setInvalidRequestStatus(response, errorMessage);
        cb(null, response);
        return;
      }
      
      //console.log("****Start of vet.exist.  request: ", JSON.stringify(request));

       var emailId = request.email.toLowerCase();
       var vet = app.models.vet;

       vet.find({where: {email: emailId}},
       function(err, userInstance) {
          response.payload.exist = false;

          if (err) {
             errorMessage = "Error invoking vet.find. Error [" + err.message + "]";
             console.log("Error invoking vet.find  err: ", err);
             responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
          }
          else {
             responseBuilder.setSuccessStatus(response);
             if (userInstance !== undefined && userInstance.length > 0) {
                console.log("Success vet.find. EmailId " + emailId + " exist - userInstance: ", userInstance);
                console.log("name " + userInstance[0].name);
                response.payload.exist = true;
             }
             else {
                console.log("Success vet.find. EmailId " + emailId + " does not exist.");
                response.payload.exist = false;
             }
           }  //else

         cb(null, response);
       });   // end of vet.find
    }

  // ******************************************
  // API Function vet.getAwardedBadges
  // *****************************************
  vet.getAwardedBadges = function(header, request, cb) {
      console.log("*********************************Start of vet.getAwardedBadges.  request: ", JSON.stringify(request));
      var response = responseBuilder.createResponseObject();

      VaGrit.getUserFromToken(app, header, function (err, vetId) {

         if (vetId === undefined || vetId === null){
            responseBuilder.setInvalidTokenStatus(response, header);
            cb(null,response);
         }
         else {  // else found vetId
            var userBadge = app.models.userBadge;
            console.log("Finding badges for vetId " + vetId);

            userBadge.find({where: {_id: vetId}},
                 function(err, results) {
                   if (err){
                     errorMessage = "userBadge.find for vetId [" + vetId + "] resulted in an err" ;
                     console.log(errorMessage, err);
                     responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                     cb(null,response);
                   }
                   else{
                     console.log("userBadge.find results:", results);
                     responseBuilder.setSuccessStatus(response);
                     if (results.length > 0){
                        response.payload.badges = results[0].badges;
                     }
                     else {
                       response.payload.badges = new Array();
                     }
                     console.log("userBadge.retrieveListOfBadges. Response", response);
                     cb(null,response);
                   }
              });  // userBadge.find

           }   // else found vetId
     });  // VaGrit.getUserFromToken
  }

  vet.remoteMethod('signup', {
        accepts: { arg: 'data', type: 'object', http: { source: 'body' } },
        returns: {arg: 'response', type: 'string'}
  });

  vet.remoteMethod('signin', {
        accepts: { arg: 'data', type: 'object', http: { source: 'body' } },
        returns: {arg: 'response', type: 'string'}
  });

  vet.remoteMethod('exist', {
        accepts: { arg: 'data', type: 'object', http: { source: 'body' } },
        returns: {arg: 'response', type: 'string'},
        http: {path: '/exist', verb: 'post'}
  });

  vet.remoteMethod('getAwardedBadges', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/getAwardedBadges', verb: 'post'}
  });




  // ******************************************
  // API Function vet.getUser
  // *****************************************
    vet.getUser = function(header, body, cb) {
       //console.log("****Start of vet.getUser.  header: ", JSON.stringify(header));
       //console.log("**** vet.getUser.  body: ", JSON.stringify(body));
       var response = responseBuilder.createResponseObject();
       var errorMessage;

       VaGrit.getUserFromToken(app, header, function (err, userId) {

          if (userId === undefined || userId === null){
              responseBuilder.setInvalidToken(response, header);
              console.log("vet.getUser. Invalid Token", header);
              cb(null,response);
          }
          else {
              console.log(' VaGrit.getUserFromToken userId: ', userId);
              vet.findById(userId, function(err, userById) {
                  console.log(' vet.findById userById: ', userById);
                  console.log(' vet.findById err: ', err);

                  if (userById != null) {

                      responseBuilder.setSuccessStatus(response);
                      response.payload.user = {
                        name: (userById.name != null) ? userById.name : "",
                        email: (userById.signupEmail != null) ? userById.signupEmail : "",
                        location: (userById.location != null) ? userById.location : "",
                        birthdate: (userById.birthdate != null) ? userById.birthdate : ""
                      };
                      console.log("vet.getUser callback. response", response);
                      cb(null, response);
                  } else {
                   errorMessage = "Error invoking vet.findById. Error [" + err.message + "]";
                   responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                   console.log("Error invoking vet.findById for id " + userId +" err: ", err);
                   console.log("vet.getUser. setUnexpectedExceptionStatus: ", errorMessage);
                   cb(null,response);
                  }
              });
          }
      });
    }


    /*
    * Update User Profile
      // 1) Get User Info
      // 2) Check if email changed
        // 3) If email did change, check if new email exists
          // 3a) If does not exist - proceed to Update(Step 5)
          // 3b) If does exist - send error
        // 4) If email did not change - proceed to Update(Step 5)
        // 5) Update Profile
          // 5a) If update success - send back updated user info
          // 5b) If update fail - send error
    */

    // ******************************************
    // API Function vet.updateProfile
    // *****************************************
      vet.updateProfile = function(header, body, cb) {
         //console.log("****Start of vet.updateProfile.  header: ", JSON.stringify(header));
         var response = responseBuilder.createResponseObject();
         var errorMessage;

         VaGrit.getUserFromToken(app, header, function (err, userId) {

            if (userId === undefined || userId === null){
                responseBuilder.setInvalidToken(response, header);
                console.log("vet.updateProfile. Invalid Token", header);
                cb(null,response);
            }
            else {
                console.log(' VaGrit.getUserFromToken userId: ', userId);

                 if (body === undefined || body === null ||
                      body.name === undefined || body.name === null || body.name === "" || typeof body.name !== "string" ||
                      body.email === undefined || body.email === null || body.email === "" || typeof body.email !== "string") {
                  errorMessage = "Missing Required fields or values";
                  responseBuilder.setInvalidRequestStatus(response, errorMessage);
                  cb(null, response);
                  return;
                }

                //console.log("**** vet.updateProfile.  body: ", JSON.stringify(body));
                
                 var emailId = body.email.toLowerCase();
                 
                // 1) Get User Info
                vet.findById(userId, function(err, userById) {
                    console.log(' vet.findById userById: ', userById);
                    console.log(' vet.findById err: ', err);

                    if (userById !== null) {
                        console.log(' vet.findById userById: STEP 2');
                        console.log(' vet.findById userById: STEP 2 userById: ', userById);

                      // 2) Check if email changed
                      if (userById.email !== emailId) {
                        // 3) If email did change, check if new email exists
                            console.log(' vet.findById userById: STEP 3');
                         vet.find({where: {and: [{email: emailId}, {id: {neq: userId}}]}}, function(err, userInstance) {

                           console.log("vet.findById userById: STEP 3 userInstance: ", userInstance);
                            if (userInstance !== undefined && userInstance.length == 0) {
                               // 3a) If does not exist - proceed to Update(Step 5)
                               console.log(' vet.findById userById: STEP 3a');
                               vet.doUpdateUserProfile(header, userId, userById, body, cb);
                            } else {
                               // 3b) If does exist - send error
                              console.log(' vet.findById userById: STEP 3b');
                              errorMessage = "Email address in use, cannot update user profile.";
                              responseBuilder.setEmailInUseStatus(response, errorMessage);
                              console.log("Error invoking vet.findById for id " + userId +" err: ", err);
                              console.log("vet.getUser. setUnexpectedExceptionStatus: ", errorMessage);
                              cb(null,response);
                            }
                         });
                      } else {
                        // 4) If email did not change - proceed to Update(Step 5)
                        console.log(' vet.findById userById: STEP 4');
                        vet.doUpdateUserProfile(header, userId, userById, body, cb);
                      }
                    } else {
                      //No user found
                     errorMessage = "Error invoking vet.findById. Error [" + err.message + "]";
                     responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                     console.log("Error invoking vet.findById for id " + userId +" err: ", err);
                     console.log("vet.updateProfile. setUnexpectedExceptionStatus: ", errorMessage);
                     cb(null,response);
                    }
                });
            }
        });
      }

      // 5) Update Profile
      vet.doUpdateUserProfile = function(token, userId, currentUserProfile, newUserProfile, cb) {
        console.log(' vet.findById userById: STEP 5 - Update Profile');
        console.log(' vet.findById userById: STEP 5 - Update Profile currentUserProfile: ', currentUserProfile);
        console.log(' vet.findById userById: STEP 5 - Update Profile newUserProfile: ', newUserProfile);

        var response = responseBuilder.createResponseObject();
        var errorMessage;

        vet.findById(userId, function (err, currentVet){

            var emailHasChanged = false;
            if (currentVet.email !== newUserProfile.email.toLowerCase()) {
              emailHasChanged = true;
            }
            
           // Update Values for vet Profile
           currentVet.name = newUserProfile.name;
           currentVet.email = newUserProfile.email.toLowerCase();
           currentVet.signupEmail = newUserProfile.email;
           currentVet.location =  (newUserProfile.location !== undefined && newUserProfile.location !== null) ? newUserProfile.location : "";
           currentVet.birthdate = (newUserProfile.birthdate !== undefined && newUserProfile.birthdate !== null) ? newUserProfile.birthdate : "";

           // Save new values to vet profile
           currentVet.save(function (err, savedVet){
              if (err === null) {
                // 5a) If update success - send back success

                checkForBadge(savedVet, function(err,badge) {
                  console.log("badge is",badge);

                if (emailHasChanged) {
                  currentVet.createAccessToken(99999999, function (err, token) {
                    if (err === null){
                      responseBuilder.setSuccessStatus(response);
                      response.payload.token = token.id;
                      if (badge !== null){
                         response.badgeAward.push(badge);
                      }
                      console.log("vet.updateProfile createAccessToken callback. response", response);
                      cb(null, response);

                    } else {
                       console.log("Error with vet.doUpdateUserProfile for emailId " + currentVet.email + ". err: ", err.message);
                       errorMessage = "createAccessToken Failed. Error [" + err.message + "]";
                       responseBuilder.setLoginFailedStatus(response,errorMessage);
                       cb(null,response);
                    }
                  });
                } else {
                  responseBuilder.setSuccessStatus(response);
                  response.payload.token = token;

                  if (badge !== null){
                     response.badgeAward.push(badge);
                  }

                  console.log("vet.updateProfile createAccessToken callback. response", response);
                  cb(null, response);
                }

              });  // checkForBadge


              } else {
                 // 5b) If update fail - send error
                 errorMessage = "Error saving vet profile. Error [" + err.message + "]";
                 responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                 console.log("Error saving vet profile for id " + userId +" err: ", err);
                 console.log("vet.updateProfile. setUnexpectedExceptionStatus: ", errorMessage);
                 cb(null,response);
              }
           })
       })
      }


      function checkForBadge(savedVet,cb){
        var completeProfile = savedVet.name != null && savedVet.name.length > 1 &&
                              savedVet.location !== undefined && savedVet.location !== null && savedVet.location.length > 1 &&
                              savedVet.birthdate !== undefined && savedVet.birthdate !== null && savedVet.birthdate.length > 1;

        if (!completeProfile){
           // A badge is not awarded
           cb(null,null);
        }
        else {
          var badge = badgeConstants.BadgeTypes.PROFILE_COMPLETED;
          var vetId = savedVet.id;
          app.models.userBadge.addBadge({vetId: vetId, badge: badge},function(err, badge) {
             if (err){
                console.log("Error with userBadge.addBadge for badge [" + badge + "] for vetId[" + vetId + "]", err);
                cb(err,null);
             }
             else {
                console.log("Successful with userBadge.addBadge for badge [" + badge + "] for vetId[" + vetId + "]", badge);
                cb(null,badge);
             }
          });   // userBadge.addBadge
        }
      }

      // ******************************************
      // API Function vet.updatePassword
      // *****************************************
        vet.updatePassword = function(header, body, cb) {
           console.log("****Start of vet.updatePassword.  header: ", JSON.stringify(header));


           var response = responseBuilder.createResponseObject();
           var errorMessage;

           VaGrit.getUserFromToken(app, header, function (err, userId) {

              if (userId === undefined || userId === null){
                  responseBuilder.setInvalidToken(response, header);
                  console.log("vet.updatePassword. Invalid Token", header);
                  cb(null,response);
              }
              else {
                  console.log(' VaGrit.getUserFromToken userId: ', userId);

                   if (body === undefined || body === null ||
                        body.currentPassword === undefined || body.currentPassword === null || body.currentPassword === "" || typeof body.currentPassword !== "string" ||
                        body.newPassword === undefined || body.newPassword === null || body.newPassword === "" || typeof body.newPassword !== "string") {
                    errorMessage = "Missing Required fields or values";
                    responseBuilder.setInvalidRequestStatus(response, errorMessage);
                    cb(null, response);
                    return;
                  }


                  // 1) Check if old password matches what is in the database
                  // 2) If doesn't match - send error OLD_PASSWORD_MISMATCH
                  // 3) If does match - update password
                        // 3a) If update success - get new token - send back success
                        // 3b) If update fail - send error

                  vet.findById(userId, function(err, currentVet) {
                      console.log(' vet.findById currentVet: ', currentVet);
                      console.log(' vet.findById err: ', err);

                      if (currentVet != null) {

                        // 1) Check if old password matches what is in the database
                          currentVet.hasPassword(body.currentPassword, function(err, oldPasswordMatches) {
                              console.log(' vet.hasPassword oldPasswordMatches: ', oldPasswordMatches);
                              console.log(' vet.hasPassword err: ', err);


                            if (err === null) {

                              if (oldPasswordMatches) {
                                  // 3a) if old password match is success, do update password

                                  console.log(' CHANGE THE PASSWORD!!!!');
                                  // Save new values to vet profile
                                  currentVet.setPassword(body.newPassword, function (err){
                                     if (err === null) {
                                       // 3a) If update success - send back success

                                       // Login user again and get new token
                                        vet.login({email: currentVet.email, password: body.newPassword}, 'user',
                                        function(err, userInstance) {
                                          if (err) {
                                             console.log("Error with vet.login for emailId " + emailId + ". err: ", err.message);
                                             errorMessage = "Login Failed. Error [" + err.message + "]";
                                             responseBuilder.setLoginFailedStatus(response,errorMessage);
                                             err = null;
                                          }
                                          else if (userInstance !== undefined) {
                                             console.log("vet.login Success. userInstance :", userInstance);
                                             responseBuilder.setSuccessStatus(response);
                                             response.payload.token = userInstance.id;
                                             console.log("token... " + userInstance.id);
                                          }
                                          console.log("callback. response", response);
                                          cb(null,response);
                                        });  // end vet.login

                                     } else {
                                        // 3b) If update fail - send error
                                        errorMessage = "Error saving vet password. Error [" + err.message + "]";
                                        responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                                        console.log("Error saving vet password for id " + userId +" err: ", err);
                                        console.log("vet.updatePassword. setUnexpectedExceptionStatus: ", errorMessage);
                                        cb(null,response);
                                     }
                                  })
                              } else {
                                  // 2) Old password doesn't match - send error OLD_PASSWORD_MISMATCH
                                   errorMessage = "Old password does not match the password in the database.";
                                   responseBuilder.setOldPasswordDoesntMatchStatus(response, errorMessage);
                                   console.log("Error updating vet password for id " + userId +" err: ", err);
                                   console.log("vet.updatePassword. setOldPasswordDoesntMatchStatus: ", errorMessage);
                                   cb(null,response);
                              }

                            } else {
                               //UNEXPECTED_EXCEPTION
                              errorMessage = "Error invoking vet.updatePassword. Error [" + err.message + "]";
                              responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                              console.log("Error invoking vet.updatePassword for id " + userId +" err: ", err);
                              console.log("vet.updatePassword. setUnexpectedExceptionStatus: ", errorMessage);
                              cb(null,response);
                            }

                          });
                      } else {
                        //Cannot findUserById
                       errorMessage = "Error invoking vet.findById. Error [" + err.message + "]";
                       responseBuilder.setUnexpectedExceptionStatus(response, errorMessage);
                       console.log("Error invoking vet.findById for id " + userId +" err: ", err);
                       console.log("vet.updatePassword. setUnexpectedExceptionStatus: ", errorMessage);
                       cb(null,response);
                      }
                  });
              }
          });
        }

  vet.remoteMethod('getUser', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/getUser', verb: 'post'}
  });


  vet.remoteMethod('updateProfile', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/updateProfile', verb: 'post'}
  });

  vet.remoteMethod('updatePassword', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/updatePassword', verb: 'post'}
  });

  vet.getNotifications = function(header, body, cb) {
       console.log("****Start of vet.retrieveNotifications.  header: ", JSON.stringify(header));
       var response = responseBuilder.createResponseObject();
       var errorMessage;

       VaGrit.getUserFromToken(app, header, function (err, userId) {
          if (userId === undefined || userId === null){
             responseBuilder.setInvalidTokenStatus(response,header);
             cb(null,response);
          }
          else { // userId is defined
             console.log(' VaGrit.getUserFromToken userId: ', userId);
             //var notification = app.models.notification;

             // Retrieve the notifications for the userId
             app.models.notification.find({where: {vetId: userId}, fields: {vetId: false}},
                  function(err, results) {
                    if (err){
                      errorMessage = "notification.find for vetId [" + userId + "] resulted in an err" ;
                      console.log(errorMessage, err);
                      responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                      cb(null,response);
                    }
                    else{
                      console.log("notification.find results:", results);
                      
                      //Add to auditEvents - Do not wait
                      var auditEventNotifications = [];
                      for (var i=0; i < results.length; i++) {
                        auditEventNotifications.push(results[i].id.toString());
                      }
                      app.models.auditEvent.addEvent({
                        vetId: userId.toString(),
                        eventType: app.models.auditEvent.eventTypes.GET_NOTIFICATIONS,
                        detail: {notificationIds: auditEventNotifications}
                      });
                      
                      responseBuilder.setSuccessStatus(response);
                      response.payload.notifications = results;
                      console.log("vet.retrieveNotifications. Response", response);
                      cb(null,response);
                    }
               });  // notification.find

      } // else userId is defined

      });  // VaGrit.getUserFromToken

    };  // vet.retrieveNotifications

    vet.remoteMethod('getNotifications', {
        accepts: [
          {arg: 'Authorization', type: 'string', http: { source: 'header' }},
          { arg: 'data', type: 'object', http: { source: 'body' } }],
        returns: {arg: 'response', type: 'string'},
        http: {path: '/getNotifications', verb: 'post'}
    });


    vet.resolveNotification = function(header, body, cb) {
         //console.log("****Start of vet.resolveNotification.  header: ", JSON.stringify(header));
         var response = responseBuilder.createResponseObject();
         var errorMessage;

         VaGrit.getUserFromToken(app, header, function (err, userId) {
            if (userId === undefined || userId === null){
               responseBuilder.setInvalidTokenStatus(response,header);
               cb(null,response);
            }
            else { // userId is defined
      
               if (body === undefined || body === null ||
                    body.notificationId === undefined || body.notificationId === null || body.notificationId === "" || typeof body.notificationId !== "string") {
                  errorMessage = "Required Fields are missing";
                  responseBuilder.setInvalidRequestStatus(response, errorMessage);
                  cb(null,response);
                  return;
               }
               
               console.log("****Start of vet.resolveNotification.  body: ", body);
               //console.log("****Start of vet.resolveNotification.  body.notificationId: ", body.notificationId);

              
              
               console.log(' VaGrit.getUserFromToken userId: ', userId);
               //var notification = app.models.notification;

               // Retrieve the notifications for the userId


               //Check if notificationId belongs to authenticated user/vetId
               app.models.notification.find({where: {and: [{vetId: userId}, {notificationId: body.notificationId}]}},
                  function(err, results) {
                    if (err){
                      errorMessage = "notification.find for vetId [" + userId + "] resulted in an err" ;
                      console.log(errorMessage, err);
                      responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                      cb(null,response);
                    }
                    else{
                      console.log("notification.find results:", results);

                      if (results !== null && results.length > 0) {
                      //Delete the notification
                       app.models.notification.destroyAll({and: [{vetId: userId}, {notificationId: body.notificationId}]},
                            function(err, destroyAllResults) {
                              if (err){
                                errorMessage = "notification.destroyAll for body.notificationId [" + body.notificationId + "] resulted in an err" ;
                                console.log(errorMessage, err);
                                responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                cb(null,response);
                              }
                              else{
                                console.log("notification.destroyAll destroyAllResults:", destroyAllResults);
                                if (destroyAllResults.count === 1) {
                                  
                                  
                                  app.models.auditEvent.addEvent({
                                    vetId: userId.toString(),
                                    eventType: app.models.auditEvent.eventTypes.RESOLVE_NOTIFICATION,
                                    detail: {notificationID: body.notificationId}
                                  });
                                  
                                  responseBuilder.setSuccessStatus(response);
                                } else {
                                  errorMessage = "notification.destroyAll for body.notificationId [" + body.notificationId + "] resulted in an error. This many records were deleted: " + destroyAllResults.count;
                                  console.log(errorMessage, err);
                                  responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                }
                                console.log("vet.resolveNotification. Response", response);
                                cb(null,response);
                              }
                         });  // notification.destroyAll

                       }
                       else {
                         //No notification record found to delete, ok to just send success response to app
                         responseBuilder.setSuccessStatus(response);
                         console.log("vet.resolveNotification. Response", response);
                         cb(null,response);

                       }
                      }
                 });  // notification.find

               } // else userId is defined

        });  // VaGrit.getUserFromToken

      };  // vet.resolveNotification

      vet.remoteMethod('resolveNotification', {
          accepts: [
            {arg: 'Authorization', type: 'string', http: { source: 'header' }},
            { arg: 'data', type: 'object', http: { source: 'body' } }],
          returns: {arg: 'response', type: 'string'},
          http: {path: '/resolveNotification', verb: 'post'}
      });


//**********************************************************************
//  vet. deleteAccount
//  Delete the user from GRIT by removing all the data for the userId associated
//  with the token.
//    Step 1. Determine the userId associated with the Token
//    Step 2. Determine the emailId and name of the user that is being deleted
//    Step 3. Delete userSurveySession data
//    Step 4. Delete mate data
//    Step 5. Delete userBadge data
//    Step 6. Remove the handle data and the handle from Blue Tas
//    Step 7. Delete flag data
//    Step 8. Delete notification updatePassword
//    Step 9. Delete journalEvents
//    Step 10. Remve the job data of job and resume document.
//    Step 10b. Check for and remove possible old temp resume files.
//    Step 11. Remove the cognitive data of risk service and WEM
//    Step 12. Remove the createAccessToken
//    Step 13. Remove the user from the vet table
//**********************************************************************
vet.deleteAccount = function(header, body, cb) {
  console.log("****Start of vet.deleteAccount.  header: ", JSON.stringify(header));
  var response = responseBuilder.createResponseObject();
  var errorMessage;

  // Step 1. Determine the User Id from the token
  VaGrit.getUserFromToken(app, header, function (err, userId) {
    if (userId === undefined || userId === null){
      responseBuilder.setInvalidTokenStatus(response,header);
      cb(null,response);
    }
    else { // else was able to determine a userid from the token
      // Step 2.  For Logging Reason, find the userId info, so we can
      // show the emailId and name that was deleted
      app.models.vet.findById(userId, function(err,userInstance){
        if (err){
          errorMessage = "Error with vet.findById for vet " + userId;
          console.log(errorMessage,err)
          responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
          cb(null,response);
        }
        else { // else not an error with vet.findById
          console.log("Removing data for user: ", userInstance);

           var vetId = userId;;

          var dbo;
          MongoClient.connect(mongoUrl, function(err, db) {
            if (err) {
              errorMessage = "Unable to connect to mongoUrl " + mongoUrl;
              console.log(errorMessage,err)
              responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
              cb(null,response);
            }
            else {  // else not an error with MongoClient.connect
              dbo = db.db(mongoDbName);

              // Step 3. Delete userSurveySession data
              dbo.collection("userSurveySession").deleteOne({_id:userId}, function(err, deleteUserSurveySessionResponse) {
                if (err) {
                  errorMessage = "Error with dbo.collect(userSurveySession).deleteOne";
                  console.log(errorMessage,err)
                  responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                  cb(null,response);
                }
                else {  // else not an error with userSurveySession delete

                  // Step 4. Delete squad mate data
                  //dbo.collection("mate").deleteMany({vetId:vetId}, function(err, deleteMateResponse) {
                  app.models.mate.destroyAll({"vetId": userId}, function(err, deleteMateResponse){
                    if (err) {
                      errorMessage = "Error with dbo.collect(mate).deleteMany";
                      console.log(errorMessage,err)
                      responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                      cb(null,response);
                    }
                    else { // else not an error with delete mate
                      console.log("mate delete result",deleteMateResponse);

                      // Step 5. Delete userBadge data
                      dbo.collection("userBadge").deleteOne({_id:userId}, function(err, deleteUserBadgeResponse) {
                        if (err) {
                          errorMessage = "Error with dbo.collect(userBadge).deleteOne";
                          console.log(errorMessage,err)
                          responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                          cb(null,response);
                        }
                        else { // else not an error with delete userBadge
                          // Successfully deleted user badge
                          console.log("userBadge delete result", deleteUserBadgeResponse.result);

                          // Step 6. Delete handle data for social media data
                          handleDeleteSocialHandle(userId,dbo, function(err, errMessage,deleteSocialHandleResponse) {
                            if (err) {
                              errorMessage = "Error with handleDeleteSocialHandle. msg[" + errMessage + "]";
                              console.log(errorMessage,err)
                              responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                              cb(null,response);
                            }
                            else { // else  not an error with handleDeleteSocialHandle

                              // Step 7. Delete flag data
                              dbo.collection("flag").deleteOne({_id:userId}, function(err, deleteFlagResponse) {
                                if (err) {
                                  errorMessage = "Error with dbo.collect(flag).deleteOne";
                                  console.log(errorMessage,err)
                                  responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                  cb(null,response);
                                }
                                else { // else not an error with delete flag
                                  console.log("delete flag resultz'", deleteFlagResponse.result);

                                  // Step 8. Delete notification
                                  app.models.notification.destroyAll({"vetId": userId}, function(err, deleteNotificationResponse){
                                  // dbo.collection("notification").deleteMany({"vetId":userId}, function(err, deleteNotificationResponse) {
                                    if (err) {
                                      errorMessage = "Error with dbo.collect(notification).deleteMany";
                                      console.log(errorMessage,err)
                                      responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                      cb(null,response);
                                    }
                                    else { // else  not an error with delete nofication
                                      console.log("delete notification result:", deleteNotificationResponse);

                                      // Step 9.  Delete the JournalEvents
                                      //dbo.collection("journalEvent").deleteMany({"vetId": vetId}, function(err, deleteJournalEventResponse) {
                                      app.models.journalEvent.destroyAll({"vetId": userId}, function(err, deleteJournalEventResponse){
                                        if (err) {
                                          errorMessage = "Error with dbo.collect(journalEvent).deleteMany";
                                          console.log(errorMessage,err)
                                          responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                          cb(null,response);
                                        }
                                        else {  // else not an error journalEvent.destroyAll
                                          console.log("delete journalEventResponse:", deleteJournalEventResponse);

                                          // Step 10. Remove the job data
                                          handleDeleteJobData(userId, function(err, errMessage,deleteJobResponse)  {
                                            if (err){
                                              errorMessage = "Error with handleDeleteJobData, err [" + errMessage + "]";
                                              console.log(errorMessage,err)
                                              responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                              cb(null,response);
                                            }
                                            else { //  else not an error with handleDeleteJobData

                                              // Step 11.  Remove the Cognitive
                                              handleDeleteCognitiveData(userId, function(err,errMessage,deleteCognitiveResponse){
                                                if (err){
                                                  errorMessage = "Error with handleDeleteCognitiveData. msg [" + errMessge + "]";
                                                  console.log(errorMessage,err)
                                                  responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                                  cb(null,response);
                                                }
                                                else {  // else not an error with handleDeleteCognitiveData

                                                  // Step 12. Remove the AccessToken
                                                  dbo.collection("AccessToken").deleteMany({"userId":userId}, function(err, deleteAccessTokenResponse) {
                                                  if (err) {
                                                    errorMessage = "Error with dbo.collect(AccessToken).deleteMany";
                                                    console.log(errorMessage,err)
                                                    responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                                    cb(null,response);
                                                  }
                                                  else {  // else not an error with accessToken.deleteMany
                                                    console.log("delete AccessToken result", deleteAccessTokenResponse.result);

                                                    // Step 13. Finally remove from vet
                                                    app.models.vet.destroyById(userId, function(err,vetRemoveResponse) {
                                                    if (err) {
                                                      errorMessage = "Error with vet.destroyById for vet " + userId;
                                                      console.log(errorMessage,err)
                                                      responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                                                      cb(null,response);
                                                    }
                                                    else {  // else not an error with vet.destroyById
                                                      console.log("Successfully removed vetId. response",vetRemoveResponse);
                                                      console.log("User with vetid [" + userId + "] and email [" + userInstance.email + "] no longer exists in GRIT");
                                                      
                                                      
                                                      //Add to auditEvents - Do not wait
                                                      app.models.auditEvent.addEvent({
                                                        vetId: userId.toString(),
                                                        eventType: app.models.auditEvent.eventTypes.DELETED_ACCOUNT,
                                                        detail: {}
                                                      });
                                                      
                                                      
                                                      responseBuilder.setSuccessStatus(response);
                                                      cb(null,response);
                                                    }
                                                  });  // vet.destroyById

                                                  }   // end. else not an error with accessToken.deleteMany
                                                });   // end. dbo.collection("AccessToken").deleteMany


                                              } //  else not an error with handleDeleteCognitiveData
                                            }); //  handleDeleteCognitiveData

                                          }   // else. not an error with handleDeleteJobData
                                        });   // end. handleDeleteJobData

                                      }  // end. else not an error with dbo.collection("journalEvent").deleteMany
                                    });  // end. journalEvent.destroyAll

                                  }  // else not an error with delete notification
                                }); // nofication.destroyAll

                              }  // else not an error with delete flag
                            });  // dbo.collection("flag").deleteOne

                           } // end. else  not an error with handleDeleteSocialHandle
                          }); // end. handleDeleteSocialHandle

                        } // end. else not an error with delete userBadge
                      }); // end. dbo.collection(userBadge).deleteOne

                    }  // end. else not an error with delete mate
                  });   // end. mate.destroyAll

                } // end. else not an error with delete userSurveySession
              }); // end. dbo.collection("userSurveySession").deleteOne

            } // end. else not an error with MongoClient.connect
          });  // end. MongoClient.connect

        } // else not an error with vet.findById
      });  // vet.findUserById

    }  // else able to get a userId from token
  });   // VaGrit.getUserFromToken

  }

  // -------------------------------------------------------------------
  // handleDeleteSocialHandle
  // Determine if the vetId has a handle in the database. If so delete
  // it from the database and invoke blueTas to delete the handle.
  // -------------------------------------------------------------------
  function handleDeleteSocialHandle(vetId,dbo, cb){
    var errObject;
    var errorMessage;

    // Step 1. Get the social handles
    console.log("Looking for the handles for vetId:"+ vetId);

    app.models.handle.find({where:{vetId:vetId}}, function(err, findHandleInstance){  // This works and return an object that contains an array
      if (err) {
        errorMessage = "Error with app.models.handle.find for vetId " + vetId;
        console.log(errorMessage,err);
        cb(err,errorMessage,null);
      }
      else { // else not an error with handle.find
        // Step 2. Determine if there are any social handles
        if (findHandleInstance === undefined || findHandleInstance === null || findHandleInstance.length < 1 ){
          console.log("vet " + vetId + " does not have any handles. Nothing to delete");
          cb(null,null,false);
        }
        else if (findHandleInstance[0] === undefined){
          console.log("findHandleInstance is not an array");
          console.log("typeof findHandleInstance ", typeof findHandleInstance);
          console.log("findHandleInstance", findHandleInstance);
          errObject = new Object();
          errorMessage = "findHandleInstance is not an array. Unable to determine the handle";
          errObject.message = errorMessage;
          console.log(errorMessage);
          cb(errObject,errorMessage,null);

        }
        else { // else found social handles to delete

          console.log("findHandleInstance", findHandleInstance);

          var blueTasHandle = findHandleInstance[0].handle;
          console.log("blueTasHandle [" + blueTasHandle + "]");

          // Step 3. Delete all the handles for the vetId from the database
          console.log("Deleting the handles");
          app.models.handle.destroyAll({"vetId": vetId}, function(err, deleteHandleResponse){
          //dbo.collection("handle").deleteMany({"vetId":vetId}, function(err, deleteHandleResponse) {
            if (err) {
              errorMessage = "Error with handle.destroyAll"
              console.log(errorMessage,err);
              cb(err,errorMessage,null);
            }
            else { // else not an error with handle.destroyAll
              console.log("handle delete result", deleteHandleResponse);

              // Step 4. Delete from blueTas
              var blueTasServerClient = request.createClient(blueTasServer);
              var removeHandleUrl = "collections/search_terms/delete/?terms="+blueTasHandle+"&collection_id="+blueTasCollectionId;
              blueTasServerClient.post(removeHandleUrl, {}, function(err, res, body) {
                console.log("SOCIAL ++++++ callblueTasServerClient!!! blueTasServerClient removeHandle START!!!!!");
                console.log("SOCIAL ++++++ callblueTasServerClient!!! blueTasServerClient removeHandle removeHandleUrl: ", removeHandleUrl);
                console.log("SOCIAL ++++++ callblueTasServerClient!!! blueTasServerClient removeHandle error: ", err);
                console.log("SOCIAL ++++++ >>>>>>>>>>>>>>>>>> callblueTasServerClient!!! blueTasServerClient removeHandle body: ", body);

                if (err) {
                  errorMessage = "Error with blueTasServerClient.post for delete";
                  console.log(errorMessage,err);
                  cb(err,errorMessage,null);
                }
                else {
                  console.log("Successful blueTasServierClient.post for delete.");
                  cb(null,null,true);
                }
              });  // end. blueTasServerClient.post

            }  // end. else not an error with handle.destroyAll
          }); //  end.  handle.destroyAll
        } // end.  else found social handles to delete

     } // end. else not an error with handle.find
   });  // end.  handle.find
}


// -------------------------------------------------------------------
// handleDeleteCognitiveData
// Delete the cognitve data of risk service and WEM.
// The Bluetas handle data was deleted with the handleDeleteSocialHandle function
// -------------------------------------------------------------------
function handleDeleteCognitiveData(vetId, cb){
  // Invoke the riskServer, getMoodScores api.. for now mock setInterval(function () {
  var errorMessage;

  var scoreRequest = new Object();
  scoreRequest.vetId = vetId;


  var riskServerClient = request.createClient(riskServer);

  // Purge the data from the Risk Server
  riskServerClient.post('purgeData/', scoreRequest, function(err,res,riskServerResponse) {
    if (err){
      errorMessage = "Error with riskServerClient.post for " + riskServer + " for purgeData api. err:";
      console.log(errorMessage,err);
      cb(err,errorMessage,null);
    }
    else {  // else not an error with riskServerClient.post
      console.log("Not an error with riskServerClient purgeData, response:", riskServerResponse);

      // Purge the data from WEM
      var wemUrl = wemServerBaseUrl + "WEMUsers/";
      var wemClient = request.createClient(wemUrl);

      var apiData = "deleteUserProfile/" + vetId;
      wemClient.get(apiData, function(err,res,wemResponse) {
        if (err){
          errorMessage = "Error with wemClient.get for purgeData api. err:";
          console.log(errorMessage,err);
          cb(err,errorMessage,null);
        }
        else {
          console.log("Not an error with WEM deleteUserProfile. response:", wemResponse);
          cb(null, null, true);
        }
      });


    } // end. else not an error with riskServerClient.post
  });
}

function handleDeleteJobData(vetId,cb){
  var errorMessage;
  app.models.resume.destroyAll({"vetId": vetId}, function(err, deleteResumeResponse){
    if (err){
       errorMessage = "Error with app.models.resume.destroyAll for vetId:" + vetId;
       console.log(errorMessage,err);
       cb(err,errorMessage,null);
    }
    else {
      console.log("deleteResumeResponse", deleteResumeResponse);
      app.models.job.destroyAll({"vetId": vetId}, function(err, deleteJobResponse){
        if (err){
          errorMessage = "Error with app.models.job.destroyAll for vetId:" + vetId;
          console.log(errorMessage,err);
          cb(err,errorMessage,null);
        }
        else {  // else not an error with job.destroyAll
          console.log("deleteJobResponse", deleteJobResponse);
          app.models.resumeText.destroyAll({"vetId": vetId}, function(err, deleteResumeTextResponse){
            if (err){
              errorMessage = "Error with app.models.resumeText.destroyAll for vetId:" + vetId;
              console.log(errorMessage,err);
              cb(err,errorMessage,null);
            }
            else {
              console.log("deleteResumeTextResponse", deleteResumeTextResponse);
              cb(null,null,true);
            }
          });  // end. resumeText.destroyAll

        }     //  end.else not an error with job.destroyAll
      });    // end. job.destroyAll

    }       // end. else not an error with resume.destroyAll
  });      // end. resume.destroyAll


}


  vet.remoteMethod('deleteAccount', {
      accepts: [
        {arg: 'Authorization', type: 'string', http: { source: 'header' }},
        { arg: 'data', type: 'object', http: { source: 'body' } }],
      returns: {arg: 'response', type: 'string'},
      http: {path: '/deleteAccount', verb: 'post'}
  });






























    // ******************************************
    // API function vet.forgotPassword
    // *****************************************
    vet.forgotPassword = function(body, cb) {

       var errorMessage = "";
       var response = responseBuilder.createResponseObject();

        if (body === undefined || body === null ||
             body.email === undefined || body.email === null || body.email === "" || typeof body.email !== "string") {
           errorMessage = "Required Fields are missing";
           responseBuilder.setInvalidRequestStatus(response, errorMessage);
           cb(null,response);
           return;
        }
        console.log("*** Start vet forgotPassword. body: ", JSON.stringify(body));


      // 1) Find vet with that email
      // 2) Update vet where email - tempPassword and tempPasswordTimestamp
      // 3) Send email with tempPassword to vet
      // 4) Return response to app


      var emailId = body.email.toLowerCase();
      var vet = app.models.vet;

      console.log("Invoking vet.findOne for emailId: ", emailId);
      // 1) Find vet with that email
      vet.findOne({where:{email: emailId}},
      function(err, userForgotPasswordInstance) {
            console.log("Invoking vet.findOne for userForgotPasswordInstance: ", userForgotPasswordInstance);
        if (err) {
          console.log("Error. vet.findOne for user " + emailId + ". err: ", err);
          errorMessage = "Unexpected exception Finding user using vet.findOne. Error [" + err.message + "]";
          responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
          cb(null,response);
        }
        else {  // Not an error with vet.findOne
          if (userForgotPasswordInstance === undefined || userForgotPasswordInstance === null){
               errorMessage = "Cannot find user with email " + emailId;
               responseBuilder.setEmailNotFoundStatus(response,errorMessage);
               cb(null,response);
          }
          else {

             var passChars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz!@#$*!@#$*";
             var tempPassword = "";
             var tempPasswordRandomLength = Math.floor(Math.random() * 10);
             if (tempPasswordRandomLength < 7) {
               tempPasswordRandomLength = 7;
             }
             for (var i=0; i < tempPasswordRandomLength; i++) {
               var randomNum = Math.floor(Math.random() * passChars.length);
               tempPassword += passChars.substring(randomNum,randomNum+1);
             }
             userForgotPasswordInstance.tempPassword = tempPassword;
             userForgotPasswordInstance.tempPasswordTimestamp = new Date().toISOString();
             
             
             var emailIdToSendTo = userForgotPasswordInstance.signupEmail;

             // 2) Update vet where email - tempPassword and tempPasswordTimestamp
             userForgotPasswordInstance.save(function(err, userForgotPasswordSaveResult){

               if (err) {
                 errorMessage = "Unexpected exception Saving user temp password [" + err.message + "]";
                 responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                 cb(null,response);
                 return;
               }

               // 3) Send email with tempPassword to vet
               // 4) Return response to app
               
               //Check if email ends in ".test", if yes, don't send email (don't enable on prod)
               if (emailIdToSendTo !== undefined && emailIdToSendTo !== null && emailIdToSendTo !== "") {
                 var checkForDotTestSubstring = (constants.testEmailExtension === emailIdToSendTo.substring(emailIdToSendTo.length-constants.testEmailExtension.length, emailIdToSendTo.length));

                 if (!checkForDotTestSubstring || (checkForDotTestSubstring && process.env.NODE_ENV === constants.prodServerEnv)) {
                   var emailBodyHtml = "<p>We received a request to reset your password for your GRIT account. We are here to help!</p><p>Copy and paste the characters below into the password field on the app, enter your email and press submit.</p><p>" + userForgotPasswordInstance.tempPassword + "</p><p>Ps. If you didn’t request this email, don’t worry about it!</p>";
                   var emailBodyText = "We received a request to reset your password for your GRIT account. We are here to help!\n\nCopy and paste the characters below into the password field on the app, enter your email and press submit.\n\n" + userForgotPasswordInstance.tempPassword + "\n\nPs. If you didn’t request this email, don’t worry about it!";
                   sgMail.setApiKey(sendgridApiKey);
                    const msg = {
                      to: emailIdToSendTo,
                      from: constants.sendgrid.fromEmail,
                      subject: 'Reset Password',
                      text: emailBodyText,
                      html: emailBodyHtml,
                    };
                    sgMail.send(msg);
                 } 
               }

               console.log("Successfully found vet using vet.findOne. userForgotPasswordInstance: ", userForgotPasswordInstance);
               responseBuilder.setSuccessStatus(response);
               response.payload.expireTime = userForgotPasswordInstance.tempPasswordTimestamp;
               cb(null,response);
            });// end save vet with temp password info

          } // end else a userForgotPasswordInstance with vet.create

        }  // end else not an error with vet.create

      });  // end vet.create

    }

    vet.remoteMethod('forgotPassword', {
        accepts: [
          { arg: 'data', type: 'object', http: { source: 'body' } }],
        returns: {arg: 'response', type: 'string'},
        http: {path: '/forgotPassword', verb: 'post'}
    });




    // ******************************************
    // API function vet.resetPassword
    // *****************************************
    vet.resetPassword = function(body, cb) {
       
       var errorMessage = "";
       var response = responseBuilder.createResponseObject();

       if (body === undefined || body === null
            || body.email === undefined || body.email === null || body.email === ""
            || body.tempPassword === undefined || body.tempPassword === null || body.tempPassword === "" || typeof body.tempPassword !== "string"
            || body.newPassword === undefined || body.newPassword === null || body.newPassword === "" || typeof body.newPassword !== "string"){
          errorMessage = "Required fields are missing";
          responseBuilder.setInvalidRequestStatus(response,errorMessage);
          console.log("vet.resetPassword. error[" + errorMessage + "]");
          cb(null, response);
          return;
      }

      //console.log("*** Start vet resetPassword. body: ", JSON.stringify(body));


             var emailId = body.email.toLowerCase();
             var tempPassword = body.tempPassword;
             var newPassword = body.newPassword;

      // 1) Find vet with that email and tempPassword
      // 2) If password is not expired
      // 3a) Update pass w newPassword
      // 3b) Auto-Login - send token
      // 3c) Else don't update - send error



      var vet = app.models.vet;

      console.log("vet.resetPassword - Invoking vet.findOne for emailId: ", emailId);
      // 1) Find vet with that email
      vet.findOne({where:{and: [{email: emailId}, {tempPassword: tempPassword}]}},
      function(err, userResetPasswordInstance) {
            console.log("vet.resetPassword - Invoking vet.findOne for userResetPasswordInstance: ", userResetPasswordInstance);
        if (err) {
          console.log("Error. vet.findOne for user " + emailId + ". err: ", err);
          errorMessage = "Unexpected exception Finding user using vet.findOne. Error [" + err.message + "]";
          responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
          cb(null,response);
        }
        else {  // Not an error with vet.findOne
          if (userResetPasswordInstance === undefined || userResetPasswordInstance === null){
             errorMessage = "Cannot find user with email and temp password for: " + emailId;
             responseBuilder.setTempPasswordMismatchStatus(response,errorMessage);
             cb(null,response);
          }
          else {

            console.log("vet.resetPassword - Invoking vet.findOne for userResetPasswordInstance: ", userResetPasswordInstance);


            // 2) If password is not expired
            var isPasswordValid = false;

            var timeDiff = Math.abs(new Date().getTime() - new Date(userResetPasswordInstance.tempPasswordTimestamp).getTime());

            var days, hours, minutes, seconds;
            seconds = Math.floor(timeDiff / 1000);
            minutes = Math.floor(seconds / 60);
            seconds = seconds % 60;
            hours = Math.floor(minutes / 60);
            minutes = minutes % 60;
            days = Math.floor(hours / 24);
            hours = hours % 24;

            if (days < 1) {
              isPasswordValid = true;
            }

            console.log("vet.resetPassword - Invoking vet.findOne for timeDiff: ", timeDiff);
            console.log("vet.resetPassword - Invoking vet.findOne for isPasswordValid: ", isPasswordValid);
            console.log("vet.resetPassword - Invoking vet.findOne for diffDays days: ", days);
            console.log("vet.resetPassword - Invoking vet.findOne for diffDays hours: ", hours);
            console.log("vet.resetPassword - Invoking vet.findOne for diffDays minutes: ", minutes);
            console.log("vet.resetPassword - Invoking vet.findOne for diffDays seconds: ", seconds);

            if (!isPasswordValid) {
              errorMessage = "Temp password is expired.";
              responseBuilder.setTempPasswordMismatchStatus(response,errorMessage);
              cb(null,response);
              return;
            }

            // 3a) Update pass w newPassword
            userResetPasswordInstance.tempPasswordTimestamp = "";
            userResetPasswordInstance.tempPassword = "";
            userResetPasswordInstance.password = newPassword;
            userResetPasswordInstance.save(function(err, userSavePasswordInstance){
              if (err !== null) {
                errorMessage = "Error saving new password. Error: ", err;
                responseBuilder.setUnexpectedExceptionStatus(response,errorMessage);
                cb(null,response);
                return;
              }

              // 3b) Auto-Login - send token
              vet.login({email: emailId, password: newPassword}, 'user',
              function(err, userInstance) {
                if (err) {
                   console.log("vet.resetPassword - Error with vet.login for emailId " + emailId + ". err: ", err.message);
                   errorMessage = "Login Failed. Error [" + err.message + "]";
                   responseBuilder.setLoginFailedStatus(response,errorMessage);
                   err = null;
                }
                else if (userInstance !== undefined) {
                   //console.log("+=+=+=+=+=+=+=+=+= vet.login Success. userInstance.toJSON() :", userInstance.toJSON());
                   responseBuilder.setSuccessStatus(response);
                   response.payload.token = userInstance.toJSON().id;
                   response.payload.name = userInstance.toJSON().user.name;
                }
                //console.log("callback. response", response);
                cb(null,response);
              });  // end vet.login

            });

          } // end else a userResetPasswordInstance with vet.create

        }  // end else not an error with vet.create

      });  // end vet.create

    }

    vet.remoteMethod('resetPassword', {
        accepts: [
          { arg: 'data', type: 'object', http: { source: 'body' } }],
        returns: {arg: 'response', type: 'string'},
        http: {path: '/resetPassword', verb: 'post'}
    });




};
